/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.caching;

import java.io.Serializable;

/**
 * A data class for holding cache key information
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public final class CacheKeyImpl implements ICacheKey {
    //Immutable cache key
    public static final ICacheKey EMPTY_KEY = new CacheKeyImpl("");
    
	private final String key;
    private final Serializable validityObject;
	
	public CacheKeyImpl (String key) {
	    this(key, null);
	}
	
	public CacheKeyImpl(String key, Serializable validityObject) {
	    this.key = key;
	    this.validityObject = validityObject;
	}

    public String getKey() {
        return key;
    }
	
	public Serializable getValidityObject() {
		return validityObject;
	}

    public String toString() {
        return this.key;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof ICacheKey)) {
            return false;
        }
        
        final ICacheKey otherKey = (ICacheKey)obj;
        final String otherKeyString = otherKey.getKey();
        
        return this.key == otherKeyString || (this.key != null && this.key.equals(otherKeyString));
    }

    @Override
    public int hashCode() {
        if (this.key == null) {
            return 0;
        }
        else {
            return key.hashCode();
        }
    }
}
