/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.caching;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class CacheKeyListImpl implements ICacheKeyList {
    private final String key;
    private final List<String> keys;
    private final List<Serializable> validityObjects;
    
    public CacheKeyListImpl(ICacheKey[] cacheKeys) {
        this(Arrays.asList(cacheKeys));
    }
    
    public CacheKeyListImpl(String[] keys, Serializable[] validityObjects) {
        if (keys == null) {
            throw new IllegalArgumentException("keys can not be null");
        }
        if (validityObjects == null) {
            throw new IllegalArgumentException("validityObjects can not be null");
        }
        
        this.keys = Arrays.asList(keys);
        this.validityObjects = Arrays.asList(validityObjects);
        
        final StringBuilder keyBuilder = new StringBuilder();
        for (final String keyString : keys) {
            keyBuilder.append(keyString);
        }
        this.key = keyBuilder.toString();
    }
    
    public CacheKeyListImpl(List<ICacheKey> cacheKeys) {
        if (cacheKeys == null) {
            throw new IllegalArgumentException("cacheKeys can not be null");
        }

        final StringBuilder keyBuilder = new StringBuilder();
        this.keys = new ArrayList<String>(cacheKeys.size());
        this.validityObjects = new ArrayList<Serializable>(cacheKeys.size());
        
        for (final ICacheKey cacheKey : cacheKeys) {
            final String keyString = cacheKey.getKey();
            keyBuilder.append(keyString);
            this.keys.add(keyString);
            this.validityObjects.add(cacheKey.getValidityObject());
        }
        
        this.key = keyBuilder.toString();
    }

    public String[] getKeyList() {
        return this.keys.toArray(new String[this.keys.size()]);
    }

    public Serializable[] getValidityObjectList() {
        return this.validityObjects.toArray(new Serializable[this.validityObjects.size()]);
    }

    public String getKey() {
        return this.key;
    }

    public Serializable getValidityObject() {
        return this.getValidityObjectList();
    }
    
    public String toString() {
        return this.key;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof ICacheKey)) {
            return false;
        }
        
        final ICacheKey otherKey = (ICacheKey)obj;
        final String otherKeyString = otherKey.getKey();
        
        return this.key.equals(otherKeyString);
    }

    @Override
    public int hashCode() {
        return key.hashCode();
    }
}
