/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.caching;

import java.io.Serializable;

import org.springmodules.cache.CacheException;
import org.springmodules.cache.provider.CacheProviderFacade;

/**
 * Provides a version of the {@link CacheProviderFacade} from Spring-Modules that
 * doesn't require client code to know the models being used for caching and flushing.
 * 
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public interface ISimpleCacheProviderFacade {
    /**
     * @see org.springmodules.cache.provider.CacheProviderFacade#cancelCacheUpdate(java.io.Serializable)
     */
    public void cancelCacheUpdate(Serializable key) throws CacheException;

    /**
     * @see org.springmodules.cache.provider.CacheProviderFacade#flushCache(org.springmodules.cache.FlushingModel)
     */
    public void flushCache();

    /**
     * @see org.springmodules.cache.provider.CacheProviderFacade#getFromCache(java.io.Serializable, org.springmodules.cache.CachingModel)
     */
    public Object getFromCache(Serializable key) throws CacheException;

    /**
     * @see org.springmodules.cache.provider.CacheProviderFacade#isFailQuietlyEnabled()
     */
    public boolean isFailQuietlyEnabled();

    /**
     * @see org.springmodules.cache.provider.CacheProviderFacade#putInCache(java.io.Serializable, org.springmodules.cache.CachingModel, java.lang.Object)
     */
    public void putInCache(Serializable key, Object obj) throws CacheException;

    /**
     * @see org.springmodules.cache.provider.CacheProviderFacade#removeFromCache(java.io.Serializable, org.springmodules.cache.CachingModel)
     */
    public void removeFromCache(Serializable key) throws CacheException;
}
