/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.caching;

import java.io.Serializable;

import org.jasig.portal.collections.LruExpirationMap;
import org.springmodules.cache.CacheException;

/**
 * A simple Least Recently Used (LRU) cache implementation.
 * @version $Revision: 11955 $
 */
public class LruCacheProviderFacade implements ISimpleCacheProviderFacade, ICacheStatistics {
    private static final int MAX_DEFAULT = 1024;
    
    private final LruExpirationMap cacheMap;
    
    /**
     * Passes {@link #MAX_DEFAULT} to {@link #LruCacheProviderFacade(int)}
     * 
     * @see #LruCacheProviderFacade(int) 
     */
    public LruCacheProviderFacade() {
        this(MAX_DEFAULT);
    }
    
    /**
     * @see LruExpirationMap#LruExpirationMap(int)
     */
    public LruCacheProviderFacade(int maxSize) {
        this.cacheMap = new LruExpirationMap(maxSize);
    }
    
    /**
     * @see LruExpirationMap#LruExpirationMap(int, boolean)
     */
    public LruCacheProviderFacade(int maxSize, boolean scanUntilRemovable) {
        this.cacheMap = new LruExpirationMap(maxSize, scanUntilRemovable);
    }
    
    
    //********** properties **********//
    
    public int getTimeToIdle() {
        return this.cacheMap.getTimeToIdle();
    }

    public int getTimeToLive() {
        return this.cacheMap.getTimeToLive();
    }

    public void setTimeToIdle(int timeToIdle) {
        this.cacheMap.setTimeToIdle(timeToIdle);
    }

    public void setTimeToLive(int timeToLive) {
        this.cacheMap.setTimeToLive(timeToLive);
    }

    
    //********** ISimpleCacheProviderFacade methods **********//
    
    public void cancelCacheUpdate(Serializable key) throws CacheException {
        //NO-OP
    }

    public void flushCache() {
        this.cacheMap.clear();
    }

    public Object getFromCache(Serializable key) throws CacheException {
        return this.cacheMap.get(key);
    }

    public boolean isFailQuietlyEnabled() {
        return true;
    }

    public void putInCache(Serializable key, Object obj) throws CacheException {
        this.cacheMap.put(key, obj);        
    }

    public void removeFromCache(Serializable key) throws CacheException {
        this.cacheMap.remove(key);        
    }

    
    //********** ICacheStatistics methods **********//
    
    public long getCacheExpirations() {
        return this.cacheMap.getExpirationCount();
    }

    public long getCacheHits() {
        return this.cacheMap.getHitCount();
    }

    public long getCacheMisses() {
        return this.cacheMap.getMissCount();
    }

    public int getSize() {
        return this.cacheMap.size();
    }

    public long getCachePuts() {
        return -1;
    }
}
