/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.caching;

import java.io.Serializable;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Required;
import org.springmodules.cache.CacheException;
import org.springmodules.cache.CachingModel;
import org.springmodules.cache.FlushingModel;
import org.springmodules.cache.provider.CacheProviderFacade;

/**
 * A {@link ISimpleCacheProviderFacade} which is configured with a {@link CacheProviderFacade}
 * and appropriate {@link CachingModel} and {@link FlushingModel}. 
 * 
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public class SimpleCacheProviderFacadeImpl implements ISimpleCacheProviderFacade, InitializingBean {
    private CacheProviderFacade cacheProviderFacade;
    private FlushingModel flushingModel;
    private CachingModel cachingModel;
    

    //********** Getters / Setters **********//
    
    /**
     * @return the cacheProviderFacade
     */
    public CacheProviderFacade getCacheProviderFacade() {
        return this.cacheProviderFacade;
    }
    /**
     * @param cacheProviderFacade the cacheProviderFacade to set
     */
    @Required
    public void setCacheProviderFacade(CacheProviderFacade cacheProviderFacade) {
        this.cacheProviderFacade = cacheProviderFacade;
    }
    /**
     * @return the cachingModel
     */
    public CachingModel getCachingModel() {
        return this.cachingModel;
    }
    /**
     * @param cachingModel the cachingModel to set
     */
    @Required
    public void setCachingModel(CachingModel cachingModel) {
        this.cachingModel = cachingModel;
    }
    /**
     * @return the flushingModel
     */
    public FlushingModel getFlushingModel() {
        return this.flushingModel;
    }
    /**
     * @param flushingModel the flushingModel to set
     */
    @Required
    public void setFlushingModel(FlushingModel flushingModel) {
        this.flushingModel = flushingModel;
    }

    
    //********** ISimpleCacheProviderFacade **********//
    
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#cancelCacheUpdate(java.io.Serializable)
     */
    public void cancelCacheUpdate(Serializable key) throws CacheException {
        this.cacheProviderFacade.cancelCacheUpdate(key);
    }

    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#flushCache()
     */
    public void flushCache() throws CacheException {
        this.cacheProviderFacade.flushCache(this.flushingModel);
    }

    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#getFromCache(java.io.Serializable)
     */
    public Object getFromCache(Serializable key) throws CacheException {
        return this.cacheProviderFacade.getFromCache(key, this.cachingModel);
    }

    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#isFailQuietlyEnabled()
     */
    public boolean isFailQuietlyEnabled() {
        return this.cacheProviderFacade.isFailQuietlyEnabled();
    }

    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#putInCache(java.io.Serializable, java.lang.Object)
     */
    public void putInCache(Serializable key, Object obj) throws CacheException {
        this.cacheProviderFacade.putInCache(key, this.cachingModel, obj);
    }

    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#removeFromCache(java.io.Serializable)
     */
    public void removeFromCache(Serializable key) throws CacheException {
        this.cacheProviderFacade.removeFromCache(key, this.cachingModel);
    }
    
    
    //********** InitializingBean **********//

    /**
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    public void afterPropertiesSet() throws Exception {
        //TODO verify members are not null?
        this.cacheProviderFacade.afterPropertiesSet();
    }
    
    
    //********** Object **********//
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this, ToStringStyle.SHORT_PREFIX_STYLE)
            .append("cacheProviderFacade", this.cacheProviderFacade)
            .append("cachingModel", this.cachingModel)
            .append("flushingModel", this.flushingModel)
            .toString();
    }
}
