/* Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.caching;

import java.io.Serializable;
import java.util.Collection;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Required;
import org.springframework.util.Assert;

/**
 * @author Eric Dalquist
 * @version $Revision$
 */
public class SimpleCacheProviderMapFacade<K extends Serializable, V> implements Map<K, V> {
    private ISimpleCacheProviderFacade cacheProviderFacade;

    /**
     * @return the cacheProviderFacade
     */
    public ISimpleCacheProviderFacade getCacheProviderFacade() {
        return this.cacheProviderFacade;
    }

    /**
     * @param cacheProviderFacade the cacheProviderFacade to set
     */
    @Required
    public void setCacheProviderFacade(ISimpleCacheProviderFacade cacheProviderFacade) {
        this.cacheProviderFacade = cacheProviderFacade;
    }
    
    
    //***** Methods that map to the cache facade *****//

    public void clear() {
        this.cacheProviderFacade.flushCache();
    }

    public boolean containsKey(Object key) {
        Assert.isInstanceOf(Serializable.class, key, "key must be Serializable");
        final Object value = this.cacheProviderFacade.getFromCache((Serializable)key);
        return value != null;
    }

    @SuppressWarnings("unchecked")
    public V get(Object key) {
        Assert.isInstanceOf(Serializable.class, key, "key must be Serializable");
        return (V)this.cacheProviderFacade.getFromCache((Serializable)key);
    }

    @SuppressWarnings("unchecked")
    public V put(K key, V value) {
        final V oldValue = (V)this.cacheProviderFacade.getFromCache(key);
        this.cacheProviderFacade.putInCache(key,  value);
        return oldValue;
    }

    public void putAll(Map<? extends K, ? extends V> t) {
        for (final Map.Entry<? extends K, ? extends V> entry : t.entrySet()) {
            final K key = entry.getKey();
            final V value = entry.getValue();
            this.put(key, value);
        }
    }

    @SuppressWarnings("unchecked")
    public V remove(Object key) {
        Assert.isInstanceOf(Serializable.class, key, "key must be Serializable");
        final Serializable sKey = (Serializable)key;
        final V oldValue = (V)this.cacheProviderFacade.getFromCache(sKey);
        this.cacheProviderFacade.removeFromCache(sKey);
        return oldValue;
    }
    
    
    //***** Methods that can't be translated to the cache facade *****//

    public boolean containsValue(Object value) {
        throw new UnsupportedOperationException("containsValue(Object) is not supported");
    }

    public Set<java.util.Map.Entry<K, V>> entrySet() {
        throw new UnsupportedOperationException("entrySet() is not supported");
    }

    public boolean isEmpty() {
        throw new UnsupportedOperationException("isEmpty() is not supported");
    }

    public Set<K> keySet() {
        throw new UnsupportedOperationException("keySet() is not supported");
    }

    public int size() {
        throw new UnsupportedOperationException("size() is not supported");
    }

    public Collection<V> values() {
        throw new UnsupportedOperationException("values() is not supported");
    }

    @Override
    public boolean equals(Object obj) {
        return this.cacheProviderFacade.equals(obj);
    }

    @Override
    public int hashCode() {
        return this.cacheProviderFacade.hashCode();
    }

    @Override
    public String toString() {
        return this.cacheProviderFacade.toString();
    }
}
