/* Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.caching;

import java.io.Serializable;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Required;
import org.springmodules.cache.CacheException;
import org.springmodules.cache.provider.CacheProviderFacade;
import org.springmodules.cache.provider.ehcache.EhCacheCachingModel;
import org.springmodules.cache.provider.ehcache.EhCacheFlushingModel;

/**
 * Wraps a SimpleCacheProviderFacadeImpl and requires only that a cacheName and cacheProviderFacade
 * are specified. Simplifying use of the EhCacheProvider
 * 
 * @author Eric Dalquist
 * @version $Revision$
 */
public class SimpleEhCacheProviderFacadeImpl implements ISimpleCacheProviderFacade, InitializingBean {
    private final SimpleCacheProviderFacadeImpl delegateCacheProvider = new SimpleCacheProviderFacadeImpl();
    private String cacheName;
    
    /**
     * @return the cacheName
     */
    public String getCacheName() {
        return this.cacheName;
    }
    /**
     * @param cacheName the cacheName to set
     */
    @Required
    public void setCacheName(String cacheName) {
        this.cacheName = cacheName;
    }
    
    /**
     * @see org.jasig.portal.caching.SimpleCacheProviderFacadeImpl#getCacheProviderFacade()
     */
    public CacheProviderFacade getCacheProviderFacade() {
        return this.delegateCacheProvider.getCacheProviderFacade();
    }
    /**
     * @see org.jasig.portal.caching.SimpleCacheProviderFacadeImpl#setCacheProviderFacade(org.springmodules.cache.provider.CacheProviderFacade)
     */
    @Required
    public void setCacheProviderFacade(CacheProviderFacade cacheProviderFacade) {
        this.delegateCacheProvider.setCacheProviderFacade(cacheProviderFacade);
    }
    
    
    /**
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    public void afterPropertiesSet() throws Exception {
        this.delegateCacheProvider.setCachingModel(new EhCacheCachingModel(this.cacheName));
        this.delegateCacheProvider.setFlushingModel(new EhCacheFlushingModel(new String[] { this.cacheName }));
        
        this.delegateCacheProvider.afterPropertiesSet();
    }
    
    
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#cancelCacheUpdate(java.io.Serializable)
     */
    public void cancelCacheUpdate(Serializable key) throws CacheException {
        this.delegateCacheProvider.cancelCacheUpdate(key);
    }
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#flushCache()
     */
    public void flushCache() {
        this.delegateCacheProvider.flushCache();        
    }
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#getFromCache(java.io.Serializable)
     */
    public Object getFromCache(Serializable key) throws CacheException {
        return this.delegateCacheProvider.getFromCache(key);
    }
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#isFailQuietlyEnabled()
     */
    public boolean isFailQuietlyEnabled() {
        return this.delegateCacheProvider.isFailQuietlyEnabled();
    }
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#putInCache(java.io.Serializable, java.lang.Object)
     */
    public void putInCache(Serializable key, Object obj) throws CacheException {
        this.delegateCacheProvider.putInCache(key, obj);        
    }
    /**
     * @see org.jasig.portal.caching.ISimpleCacheProviderFacade#removeFromCache(java.io.Serializable)
     */
    public void removeFromCache(Serializable key) throws CacheException {
        this.delegateCacheProvider.removeFromCache(key);        
    }
}
