/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/


package org.jasig.portal.context;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jasig.portal.context.common.IHttpRequestResponseAccessor;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.rendering.IPortalRenderer;
import org.jasig.portal.url.IRequestParameterController;
import org.jasig.portal.url.IWritableHttpServletRequest;
import org.jasig.portal.url.WritableHttpServletRequestImpl;

/**
 * A context that separates URL processing and rendering responsibilities
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class CompoundContext implements IContext {
    private IRequestParameterController urlController;
    private IPortalRenderer renderer;
    private IHttpRequestResponseAccessor httpRequestResponseAccessor;
    
    
    /**
     * @return Returns the urlController.
     */
    public IRequestParameterController getUrlController() {
        return urlController;
    }
    /**
     * @param urlController The urlController to set.
     */
    public void setUrlController(IRequestParameterController urlController) {
        this.urlController = urlController;
    }
    /**
     * @return Returns the renderer.
     */
    public IPortalRenderer getRenderer() {
        return renderer;
    }
    /**
     * @param renderer The renderer to set.
     */
    public void setRenderer(IPortalRenderer renderer) {
        this.renderer = renderer;
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.context.IContext#service(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public void service(HttpServletRequest req, HttpServletResponse res) throws PortalException {
        IWritableHttpServletRequest wreq=new WritableHttpServletRequestImpl(req);
        if(httpRequestResponseAccessor!=null) {
            httpRequestResponseAccessor.setCurrentHttpServletRequest(req);
            httpRequestResponseAccessor.setCurrentHttpServletResponse(res);
        }
        try {
            urlController.processParameters(wreq,res);
            if(!res.isCommitted()) {
                renderer.render(wreq,res);
            }
        } finally {
            if(httpRequestResponseAccessor!=null) {
                httpRequestResponseAccessor.setCurrentHttpServletRequest(null);
                httpRequestResponseAccessor.setCurrentHttpServletResponse(null);
            }
        }
    }
    
    /**
     * Specify http servlet request&response accessor. If provided, context will
     * set request&response objects prior to processing, and set them to null at the end of processing.
     * @param httpRequestResponseAccessor
     */
    public void setHttpRequestResponseAccessor(IHttpRequestResponseAccessor httpRequestResponseAccessor) {
        this.httpRequestResponseAccessor = httpRequestResponseAccessor;
    }
}
