/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/


package org.jasig.portal.context;

import java.util.Collection;
import java.util.Map;

import org.jasig.portal.core.exception.PortalException;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.beans.factory.BeanFactoryAware;

/**
 * ContextFactoryImpl implements a simple, Map-configuration based context
 * factory.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class ContextFactoryImpl implements IContextFactory, BeanFactoryAware {
	protected Map<String,String> contexts;
	protected BeanFactory factory;
	
	/* (non-Javadoc)
	 * @see org.jasig.portal.context.IContextFactory#getKnownContextNames()
	 */
	public Collection getKnownContextNames() {
		return getContexts().keySet();
	}
	/* (non-Javadoc)
	 * @see org.jasig.portal.context.IContextFactory#isContextNameKnown(java.lang.String)
	 */
	public boolean isContextNameKnown(String contextName) {
		return getContexts().containsKey(contextName);
	}
	/* (non-Javadoc)
	 * @see org.jasig.portal.context.IContextFactory#getContextInstance(java.lang.String)
	 */
	public IContext getContext(String contextName) throws PortalException {
		try {
            String contextBeanName=contexts.get(contextName);
            if(contextBeanName!=null) {
                return (IContext)getBeanFactory().getBean(contextBeanName, IContext.class);
            } else {
                return null;
            }
		} catch (Exception e) {
			throw new PortalException(e);
		}
	}
	
	/**
	 * @return Returns the contexts.
	 */
	public Map<String,String> getContexts() {
		return contexts;
	}
	/**
	 * @param contexts The contexts to set.
	 */
	public void setContexts(Map<String,String> contexts) {
		this.contexts = contexts;
	}
	/**
	 * @return Returns the bean factory in which the given class has been configured.
	 */
	public BeanFactory getBeanFactory() {
		return factory;
	}
	/**
	 * @param factory Sets the factory in which the given class has been configured.
	 */
	public void setBeanFactory(BeanFactory factory) {
		this.factory = factory;
	}
}
