/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.context;

import java.util.Collection;

import org.jasig.portal.core.exception.PortalException;

/**
 * IContextFactory is an interface describing a portal context factory.
 * The interface is designed exclusively for the use by the context resolvers.
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 * @see IContextResolver
 */
public interface IContextFactory {
	/**
	 * Obtain a list of known context names
	 * @return - a collection of <class>String</code> names
	 */
	public Collection getKnownContextNames();
	
	/**
	 * Check if the factory is configured to produce a context
	 * of a given name.
	 * @param contextName - the name of a context
	 * @return - a boolean answer
	 */
	public boolean isContextNameKnown(String contextName);
	
	/**
	 * Obtain a new instance of a context. Please note that
	 * depending on the context configuration, the instance
	 * returned may actually be a singleton.
	 * 
	 * @param contextName - the name of a context to instantiate
	 * @return - a new instance of a specified context
	 */
	public IContext getContext(String contextName) throws PortalException;
}
