/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/


package org.jasig.portal.context;

import java.util.Collection;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.jasig.portal.core.exception.PortalException;

/**
 * IContextResolver interface describes an object that selects an appropriate {@link IContext}
 * to be used with the current <class>HttpServletRequest</class>
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public interface IContextResolver {
	/**
	 * Resovles the context associated with the current request.
	 * @param req - current <class>HttpServletRequest</class>
	 * @return {@link IContext} to be used with the specified request 
	 */
	public IContext resolveContext(HttpServletRequest req) throws PortalException;
	
	/**
	 * Returns the list of the available contexts.
	 * @return a <class>Collection</class> of available context names
	 */
	public Collection getAvailableContexts();
	
	/**
	 * Returns the context instance for a given name, for a given sesion.
	 * @param contextName - a name of the desired context
	 * @param session - HTTP servlet session for which the context should be instantiated
	 * @return a {@link IContext} for the specified name
	 */
	public IContext getContext(String contextName,HttpSession session) throws PortalException;
}
