/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.context;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.ArrayList;
import java.util.Collection;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.jasig.portal.core.exception.PortalException;
import org.springframework.beans.factory.annotation.Required;

/**
 * Resolves context based on a value of a pre-defined request parameter.
 * Note: must provide value of <code>defaultContextName</code> 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 1.0$
 */
public class ParameterNameContextResolver implements IContextResolver {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(ParameterNameContextResolver.class);

    String parameterName;
    String defaultContextName;
    IContextFactory contextFactory;
    
    /**
     * Specify name of the default context bean. This context will be used
     * if the specified parameter is not passed with the request, or its value
     * does not match to a valid context bean.
     * @param defaultContextName
     */
    @Required
    public void setDefaultContextName(String defaultContextName) {
        this.defaultContextName = defaultContextName;
    }
    
    /**
     * Specify name of the request parameter that will be used to specify context name
     * @param parameterName
     */
    @Required
    public void setParameterName(String parameterName) {
        this.parameterName = parameterName;
    }
    @Required
    public void setContextFactory(IContextFactory contextFactory) {
        this.contextFactory = contextFactory;
    }

    public Collection getAvailableContexts() {
        ArrayList<String> s=new ArrayList<String>(1);
        s.add(defaultContextName);
        return s;
    }

    public IContext getContext(String contextName, HttpSession session) throws PortalException {
        return contextFactory.getContext(contextName);
    }

    public IContext resolveContext(HttpServletRequest req) throws PortalException {
        String contextValue=req.getParameter(parameterName);
        if(contextValue!=null) {
            // attempt to retrieve specified context
            try {
                IContext context= contextFactory.getContext(contextValue);
                if(context!=null) {
                    return context;
                } else {
                    logger.warn("resolveContext(HttpServletRequest) -  unable to obtain request-specified context, will try default: contextValue=" + contextValue);
                }
            } catch (Exception e) {
                logger.warn("resolveContext(HttpServletRequest) -  unable to obtain request-specified context, will try default: contextValue=" + contextValue, e);
            }
        }
        
        // obtain default context
        IContext defaultContext= contextFactory.getContext(this.defaultContextName);
        if(defaultContext==null) {
            logger.error("resolveContext(HttpServletRequest) -  unable to resolve default context: defaultContextName=" + defaultContextName, null);
        }
        return defaultContext;
    }
    
    
    
}
