/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.context.common.profile;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * BasicProfileConfig containing basic profile information
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class BasicProfileConfig implements IProfileConfig {
    ProfileId id;
    Map<String, Object> properties;
    IProfileConfig parentProfile;
    List<IProfileConfig> childProfiles = Collections.emptyList();
    Map<ProfileId, IProfileConfig> allProfiles = null;

    /**
     * @return Returns the id.
     */
    public ProfileId getId() {
        return id;
    }

    public void setProfileId(String profileId) {
        this.id = new ProfileId(profileId);
    }

    public void setProperties(Map<String, Object> properties) {
        this.properties = properties;
    }

    public Object getProperty(String key) {
        Object value = null;
        if (properties != null) {
            value = properties.get(key);
        }
        if (value == null && parentProfile != null) {
            return parentProfile.getProperty(key);
        }
        else {
            return value;
        }
    }

    public Map<String, Object> getProperties() {
        if (parentProfile != null) {
            Map<String, Object> parentProperties = parentProfile.getProperties();
            if (parentProperties == null) {
                if (this.properties != null) {
                    return Collections.unmodifiableMap(this.properties);
                }
                else {
                    return null;
                }
            }
            else {
                if (this.properties != null) {
                    Map<String, Object> mergedProperties = new HashMap<String, Object>(parentProperties);
                    mergedProperties.putAll(this.properties);
                    return mergedProperties;
                }
                else {
                    return Collections.unmodifiableMap(parentProperties);
                }
            }
        }
        else {
            if (this.properties != null) {
                return Collections.unmodifiableMap(this.properties);
            }
            else {
                return null;
            }
        }
    }

    public Map<ProfileId, IProfileConfig> getAllProfiles() {
        return allProfiles;
    }

    public List<IProfileConfig> getChildProfiles() {
        return this.childProfiles;
    }

    public IProfileConfig getProfile(ProfileId profileId) {
        if (this.id != null && this.id.equals(profileId)) {
            return this;
        }
        if (allProfiles != null) {
            return allProfiles.get(profileId);
        }
        return null;
    }

    public void setChildProfiles(List<IProfileConfig> childProfiles) {
        this.childProfiles = childProfiles;
        // assemble a map of all profiles and set parent profile renferences
        allProfiles = new HashMap<ProfileId, IProfileConfig>();
        for (IProfileConfig profile : childProfiles) {
            // set parent profile references
            profile.setParentProfile(this);
            // assemble all descendant profiles in a map
            ProfileId profileId = profile.getId();
            if (profileId != null) {
                allProfiles.put(profile.getId(), profile);
            }
            Map<ProfileId, IProfileConfig> profiles = profile.getAllProfiles();
            if (profiles != null) {
                allProfiles.putAll(profiles);
            }
        }
    }

    public void setParentProfile(IProfileConfig parentProfile) {
        this.parentProfile = parentProfile;
    }
}
