/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.context.common.profile;

import java.util.List;
import java.util.Map;

/**
 * An interface describing part of, or an entire profile configuration 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IProfileConfig extends IProfile {
    /**
     * Obtain immediate children of the current profile
     * @return
     */
    List<IProfileConfig> getChildProfiles();

    /**
     * Obtain map of all descendants of the current profile
     * @return
     */
    Map<ProfileId, IProfileConfig> getAllProfiles();

    /**
     * Specify a list of child profiles
     * @param childProfiles
     */
    public void setChildProfiles(List<IProfileConfig> childProfiles);

    /**
     * Specify parent profile
     * @param parentProfile
     */
    public void setParentProfile(IProfileConfig parentProfile);

    /**
     * Obtain a profile by id
     * @param profileId
     * @return profile corresponding to the specified id, or <code>null</code> if no such profile exists
     */
    IProfileConfig getProfile(ProfileId profileId);
}
