/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.context.common.profile.dao;

import org.jasig.portal.context.common.profile.ProfileId;
import org.jasig.portal.security.UserId;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

/**
 * Default Hibernate implementaion of {@link org.jasig.portal.context.common.profile.dao.IProfilePreferenceDao}  
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 1.0$
 */
public class ProfilePreferenceDaoImpl extends HibernateDaoSupport implements IProfilePreferenceDao {
    public ProfileId getPreferredProfile(UserId userId) {
        final ProfileChoice profileChoice = getProfileChoice(userId);
        if (profileChoice != null) {
            if (logger.isDebugEnabled()) {
                logger.debug("retrieved profile preference for the user : userId=" + userId + ", profileId=" + profileChoice.getProfileId());
            }
            return profileChoice.getProfileId();
        }
        else {
            if (logger.isDebugEnabled()) {
                logger.debug("profile preference missing for the user : userId=" + userId);
            }
            return null;
        }
    }

    public void setPreferredProfile(UserId userId, ProfileId profileId) {
        ProfileChoice profileChoice = getProfileChoice(userId);
        if (profileChoice == null) {
            profileChoice = new ProfileChoice();
            profileChoice.setUserId(userId);
        }
        profileChoice.setProfileId(profileId);
        
        final HibernateTemplate hibernateTemplate = getHibernateTemplate();
        hibernateTemplate.saveOrUpdate(profileChoice);

        if (logger.isDebugEnabled()) {
            logger.debug("persisted profile preference : userId=" + userId + ", profileId=" + profileId);
        }
    }

    protected ProfileChoice getProfileChoice(UserId userId) {
        final HibernateTemplate hibernateTemplate = getHibernateTemplate();
        final ProfileChoice profileChoice = (ProfileChoice)hibernateTemplate.get(ProfileChoice.class, userId);
        return profileChoice;
    }
}
