/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.context.common.user;

import java.util.Collections;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.security.IUser;
import org.jasig.portal.utils.WeakHashSet;

/**
 * Serves as a base for building user controllers. Manages event listeners and notifications.  
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class AbstractUserController {
    private static final Log logger = LogFactory.getLog(AbstractUserController.class);

    Set<IUserControllerEventListener> listeners=Collections.<IUserControllerEventListener>synchronizedSet(new WeakHashSet());
    
    public void addUserControllerEventListener(IUserControllerEventListener listener) {
        listeners.add(listener);
    }
    public void removeUserControllerEventListener(IUserControllerEventListener listener) {
        listeners.remove(listener);
    }
    
    protected void distributeUserChangedEvent(IUser oldUser, Boolean oldAuthenticationState, HttpServletRequest request) {
        UserControllerEvent event=new UserControllerEvent(this,UserControllerEvent.userChanged,request,oldUser, oldAuthenticationState);
        distributeEvent(event);
    }
    
    protected void distributeAuthenticationChangedEvent(HttpServletRequest request) {
        distributeEvent(new UserControllerEvent(this,UserControllerEvent.authenticationChanged,request));
    }
    
    /**
     * Distribute user event
     * @param event - event object
     */
    protected void distributeEvent(UserControllerEvent event) {
        if (logger.isDebugEnabled()) {
            logger.debug("distributeEvent(UserControllerEvent event = " + event.getEventType().toString() + ") - distributing user controller event");
        }

        for(IUserControllerEventListener listener: listeners) {
            listener.receiveUserControllerEvent(event);
        }
    }
}
