/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.context.common.user;

import org.jasig.portal.security.IUser;
import org.jasig.portal.security.UserId;

/**
 * IUserController provides user information to the context components.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IUserController {
    /**
     * Obtain user id for the current session
     * @return the user ID
     */
    public UserId getUserId();
    
    /**
     * Obtain user description
     * @return complete user object
     */
    public IUser getUser();

    /**
     * Check if the current user is authenticated
     * @return <code>true</code> if the user is authenticated, otherwise <code>false</code>
     */
    public boolean isUserAuthenticated();
    
    /**
     * Add user controller event listener
     * @param listener
     */
    public void addUserControllerEventListener(IUserControllerEventListener listener);
    
    /**
     * Remove user controller event listener
     * @param listener
     */
    public void removeUserControllerEventListener(IUserControllerEventListener listener);
    
    /**
     * Determine if the user is anonymous
     * @return <code>true</code> if the user is anonymous, <code>false</code> otherwise.
     */
    public boolean isAnonymous();
}
