/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.context.common.user;

import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.context.common.persistence.IPersistenceController;
import org.jasig.portal.security.IUser;
import org.jasig.portal.security.UserId;
import org.jasig.portal.utils.ListenerSet;

/**
 * A user controller adapter that fires off persistence reset events
 * after the user change. 
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PersistencePolicyUserController implements IUserController, IUserControllerEventListener {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PersistencePolicyUserController.class);

    IUserController userController;
    IPersistenceController persistenceController;
    boolean resetAfterUserChange=true;
    
    protected Set<IUserControllerEventListener> listeners=new ListenerSet<IUserControllerEventListener>();
    
    
    public void receiveUserControllerEvent(UserControllerEvent event) {
        if(UserControllerEvent.userChanged.equals(event.getEventType())) {
            distributeEvent(new UserControllerEvent(this,UserControllerEvent.userChanged,event.getHttpServletRequest()));
            if(resetAfterUserChange) {
                persistenceController.reset();
            }
        } else {
            distributeEvent(new UserControllerEvent(this,event.getEventType(),event.getHttpServletRequest()));
        }
    }
    
    public void setResetAfterUserChange(boolean resetAfterUserChange) {
        this.resetAfterUserChange = resetAfterUserChange;
    }
    public void setUserController(IUserController userController) {
        this.userController = userController;
        userController.addUserControllerEventListener(this);
    }
    
    public void setPersistenceController(IPersistenceController persistenceController) {
        this.persistenceController = persistenceController;
    }
    
    public void addUserControllerEventListener(IUserControllerEventListener listener) {
        listeners.add(listener);
    }
    public void removeUserControllerEventListener(IUserControllerEventListener listener) {
        listeners.remove(listener);
    }
    
    /**
     * Distribute user event
     * @param event - event object
     */
    void distributeEvent(UserControllerEvent event) {
        if (logger.isDebugEnabled()) {
            logger.debug("distributeEvent(UserControllerEvent event = " + event.getEventType().toString() + ") - distributing user controller event");
        }

        for(IUserControllerEventListener l: listeners) {
            if(l!=null) {
                l.receiveUserControllerEvent(event);
            }
        }
    }
    
    
    // delegate methods
    public IUser getUser() {
        return userController.getUser();
    }
    public UserId getUserId() {
        return userController.getUserId();
    }
    public boolean isAnonymous() {
        return userController.isAnonymous();
    }
    public boolean isUserAuthenticated() {
        return userController.isUserAuthenticated();
    }

}
