/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.context.common.user;

import java.util.EventObject;

import javax.servlet.http.HttpServletRequest;

import org.jasig.portal.security.IUser;

/**
 *  Describes user controller event
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class UserControllerEvent extends EventObject {
    final EventType eventType;
    
    protected HttpServletRequest httpServletRequest;
    protected IUser oldUser=null;
    protected Boolean oldAuthenticationState=null;
    /**
     * An event signifying that a new user is now associated with the session.
     * This also implies that the authentication state may have changed.
     */
    public static final EventType userChanged=new EventType("user changed");
    
    /**
     * An event signifying that the authentication state has changed, while
     * the user remained the same.
     */
    public static final EventType authenticationChanged=new EventType("authentication changed");
    
    /**
     * Create a persistance controller event
     * @param source - event source
     * @param eventType - event type
     */
    public UserControllerEvent(Object source, EventType eventType, HttpServletRequest httpServletRequest) {
        super(source);
        this.eventType=eventType;
        this.httpServletRequest=httpServletRequest;
    }
    
    /**
     * Create a persistance controller event, specifying old user
     * @param source - event source
     * @param eventType - event type
     * @param oldUser - old user for the user-changed event
     * @param oldAuthenticationState - old users' authentication state
     */
    public UserControllerEvent(Object source, EventType eventType, HttpServletRequest httpServletRequest, IUser oldUser, Boolean oldAuthenticationState) {
        this(source,eventType,httpServletRequest);
        this.oldUser=oldUser;
        this.oldAuthenticationState=oldAuthenticationState;
    }
    
    public static class EventType {
        final String name;
        
        public EventType(String name) {
            this.name=name;
        }
        
        /**
         * @return Returns the name.
         */
        public String getName() {
            return name;
        }
        
        public String toString() {
            return getName();
        }
    }
    
    /**
     * @return Returns the eventType.
     */
    public EventType getEventType() {
        return eventType;
    }

    public HttpServletRequest getHttpServletRequest() {
        return httpServletRequest;
    }

    public void setHttpServletRequest(HttpServletRequest httpServletRequest) {
        this.httpServletRequest = httpServletRequest;
    }

    public void setOldUser(IUser oldUser) {
        this.oldUser = oldUser;
    }

    public IUser getOldUser() {
        return oldUser;
    }

    public Boolean getOldAuthenticationState() {
        return oldAuthenticationState;
    }

    public void setOldAuthenticationState(Boolean oldAuthenticationState) {
        this.oldAuthenticationState = oldAuthenticationState;
    }
}
