/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.context.dummy;

import java.io.Serializable;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.parsers.SAXParserFactory;

import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.rendering.IPortalRenderer;
import org.jasig.portal.rendering.sax.ISaxConsumer;
import org.jasig.portal.rendering.sax.ISaxRenderer;
import org.xml.sax.ContentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.ext.LexicalHandler;



/**
 * A simple layout source that reads layout from the specified file. Useful for testing.
 * note: lexical handler setting may be ignored, depending on the implementation
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class XmlFileLayoutSource implements IPortalRenderer, ISaxRenderer, ICacheable {

    protected String fileName;

    protected ContentHandler contentHandler;
    protected DTDHandler dtdHandler;
    protected ErrorHandler errorHandler;
    protected LexicalHandler lexicalHandler;
    
    protected ISaxConsumer consumer;
    
    boolean renderRequested=false;
    
    /**
     * Get the value of fileName.
     * @return value of fileName.
     */
    public String getFileName() {
        return fileName;
    }
    
    /**
     * Set the value of fileName.
     * @param fileName  Value to assign to fileName.
     */
    public void setFileName(String  fileName) {
        this.fileName = fileName;
    }
    
    public void setAllHandlers(ContentHandler ch) {
        setContentHandler(ch);
        if(ch instanceof DTDHandler) {
            setDTDHandler((DTDHandler)ch);
        }
        if(ch instanceof ErrorHandler) {
            setErrorHandler((ErrorHandler)ch);
        }
        if(ch instanceof LexicalHandler) {
            setLexicalHandler((LexicalHandler)ch);
        }
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISaxProducer#setSaxConsumer(org.jasig.portal.rendering.sax.ISaxConsumer)
     */
    public void setSaxConsumer(ISaxConsumer consumer) {
        this.consumer=consumer;
        this.setAllHandlers(consumer);
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISaxPipelineComponent#canRenderSax()
     */
    public boolean canRenderSax() {
        return (consumer!=null && consumer.canRenderSax());
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.ISaxProducer#isContenteHandlerSet()
     */
    public boolean isContenteHandlerSet() {
        return this.contentHandler!=null;
    }

    public void setDTDHandler(DTDHandler handler) {
        this.dtdHandler=handler;
    }
    public void setContentHandler(ContentHandler handler) {
        this.contentHandler=handler;
    }
    public void setErrorHandler(ErrorHandler handler) {
        this.errorHandler=handler;
    }
    public void setLexicalHandler(LexicalHandler handler) {
        this.lexicalHandler=handler;
    }

    public void render(HttpServletRequest req, HttpServletResponse res) throws PortalException {
        try { 
            renderRequested=true;
            XMLReader reader=SAXParserFactory.newInstance().newSAXParser().getXMLReader();
            reader.setFeature("http://xml.org/sax/features/validation",false);
            reader.setFeature("http://xml.org/sax/features/namespaces",true);
            reader.setFeature("http://xml.org/sax/features/namespace-prefixes",false);
            reader.setContentHandler(contentHandler);
            reader.setErrorHandler(errorHandler);
            reader.setDTDHandler(dtdHandler);
            reader.parse(new InputSource(XmlFileLayoutSource.class.getResourceAsStream(fileName)));
        } catch (Exception e) {
            throw new PortalException(e);
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#generateKey()
     */
    public ICacheKey generateKey() {
        return new CacheKeyImpl(this.fileName);
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#isCacheValid(java.io.Serializable)
     */
    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }
    
    /**
     * @return Returns the renderRequested.
     */
    public boolean isRenderRequested() {
        return renderRequested;
    }
    /**
     * @param renderRequested The renderRequested to set.
     */
    public void setRenderRequested(boolean renderRequested) {
        this.renderRequested = renderRequested;
    }
}
