/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.context.uportal2;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.portlet.WindowState;

import org.jasig.portal.context.common.PortletStateRenderingAttributeProvider;
import org.jasig.portal.portlet.rendering.PortletWindowManagerEvent;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.rendering.attributes.ElementResolverKey;
import org.xml.sax.Attributes;

/**
 * Provides rendering attributes describing portlet state information necessary for the uP2 theme transform.
 * Specifically, it provides the values of <code>minimized</code> attributes for the portlet elements
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision: 11955 $
 */
public class ThemeTransformationPortletStateRenderingAttributeProvider extends PortletStateRenderingAttributeProvider {
    static final String MINIMIZED_ATTRIBUTE = "minimized";
    
    Map defaultAttributeValuesMap;
    
    public ThemeTransformationPortletStateRenderingAttributeProvider() {
        super();
        Map defaults=new HashMap();
        defaults.put(MINIMIZED_ATTRIBUTE,"false");
        this.defaultAttributeValuesMap=Collections.unmodifiableMap(defaults);
    }
    
    @Override
    protected void updateValues(ElementResolverKey elementKey) {
        PortletWindowId portletWindowId=this.getPortletWindowId(elementKey);
        if(WindowState.MINIMIZED.equals(this.getPortletWindowManager().getWindowState(portletWindowId))) {
            renderingAttributeValues.setElementAttributeValue(elementKey,MINIMIZED_ATTRIBUTE,"true");
        } else {
            renderingAttributeValues.setElementAttributeValue(elementKey,MINIMIZED_ATTRIBUTE,"false");
        }
    }

    public void receivePortletWindowManagerEvent(PortletWindowManagerEvent event) {
        if(PortletWindowManagerEvent.windowStateChangedEventType.equals(event.getEventType())) {
            PortletWindowId windowId=event.getWindowId();
            if(windowId!=null) {
                ElementResolverKey elementKey=this.getElementResolverKey(windowId);
                if(WindowState.MINIMIZED.equals(event.getNewWindowState())) {
                    renderingAttributeValues.setElementAttributeValue(elementKey,MINIMIZED_ATTRIBUTE,"true");
                } else {
                    renderingAttributeValues.setElementAttributeValue(elementKey,MINIMIZED_ATTRIBUTE,"false");
                }
            }
        }
    }

    /**
     * Allow set attribute method to change minimized state
     */
    @Override
    public void setAttributeValue(String attributeName, String attributeValue, String namespaceUri, String localName, String qName, Attributes atts) {
        ElementResolverKey elementKey=elementResolver.resolveElementKey(namespaceUri,localName,qName,atts);
        if(elementKey!=null) {
            String minimizedValue=atts.getValue(MINIMIZED_ATTRIBUTE);
            if(minimizedValue!=null) {
                PortletWindowId windowId=this.getPortletWindowId(elementKey);
                if(minimizedValue.equals("true")) {
                    // switch to minimized state
                    this.portletWindowManager.setWindowState(windowId,WindowState.MINIMIZED);
                } else {
                    // switch to normal state currently minimized
                    if(WindowState.MINIMIZED.equals(this.getPortletWindowManager().getWindowState(windowId))) {
                        this.portletWindowManager.setWindowState(windowId,WindowState.NORMAL);
                    }
                }
            }
        }
    }

    @Override
    protected Map getDefaultAttributeValueMap() {
        return this.defaultAttributeValuesMap;
    }
}
