/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.context.uportal2;

import javax.servlet.http.HttpServletResponse;

import org.jasig.portal.context.common.persistence.IPersistenceController;
import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.layout.uportal2.LegacyLayoutSourceRequestParameterProcessor;
import org.jasig.portal.rendering.attributes.IRenderingAttributeProvider;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IWritableHttpServletRequest;

/**
 *  A helper parameter processor that providers information
 *  required by the standard uPortal2 stylesheets.
 *  
 *  This includes the following structure stylesheet params:
 *  authenticated
 *  
 *  This includes the following theme stylesheet params:
 *  userName
 *  uP_productAndVersion
 *  
 *  It also takes care of communicating uPortal2 "save settings" command
 *  to the persistance controller.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class TransformationInformationProvider implements IRequestParameterProcessor {
    protected IUserController userController;
    protected IRenderingAttributeProvider structureRenderingAttributeProvider;
    protected IRenderingAttributeProvider themeRenderingAttributeProvider;
    protected String productAndVersion="uPortal 3";
    protected IPersistenceController persistenceController;
    
    
    protected final static String UP_SAVE_COMMAND="uP_save";

    public boolean canEvaluateVariables() {
        return false;
    }

    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        structureRenderingAttributeProvider.setParameterValue("authenticated",Boolean.toString(!userController.isAnonymous()));
        // TODO: move to standardized user attribute for display name (using UserDaoImpl local constant for now)
        themeRenderingAttributeProvider.setParameterValue("userName",userController.getUser().getUserName());
        themeRenderingAttributeProvider.setParameterValue("uP_productAndVersion",productAndVersion);
        
        // intercept add targets command, and pass additional params to the stylesheet
        // note: this is an example of unwanted coupling between layout and styleshet responsibilities,
        // that exists in the integrated modes implementation.
        // A better solution would be to have layout manager supply detailed description
        // of the node being added along with the markings, so that stylesheets can analyze it and
        // make appropriate decisions.
        String addTargetsCommand=req.getParameter(LegacyLayoutSourceRequestParameterProcessor.REQUEST_ADD_TARGETS_COMMAND);
        if(addTargetsCommand!=null) {
            // we can safely pass the values to the transformer. if they exist in the current
            // request, transformer parameter processor elements will pick them up later
            structureRenderingAttributeProvider.setParameterValue("mode","preferences");
            structureRenderingAttributeProvider.setParameterValue("targetRestriction",addTargetsCommand);
            structureRenderingAttributeProvider.setParameterValue("targetAction",addTargetsCommand=="channel"?"New Channel":"New Folder");
        }
        // process uPortal2 persistance command
        if(req.getParameter(UP_SAVE_COMMAND)!=null) {
            persistenceController.persist();
        }
        return true;
    }

    public void setUserController(IUserController userController) {
        this.userController = userController;
    }

    public void setProductAndVersion(String portalVersion) {
        productAndVersion = portalVersion;
    }

    public void setPersistenceController(IPersistenceController persistanceController) {
        this.persistenceController = persistanceController;
    }

    public void setStructureRenderingAttributeProvider(IRenderingAttributeProvider structureRenderingAttributeProvider) {
        this.structureRenderingAttributeProvider = structureRenderingAttributeProvider;
    }

    public void setThemeRenderingAttributeProvider(IRenderingAttributeProvider themeRenderingAttributeProvider) {
        this.themeRenderingAttributeProvider = themeRenderingAttributeProvider;
    }

}
