/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.core;

import java.io.IOException;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.pluto.PortletContainer;
import org.jasig.portal.context.IContext;
import org.jasig.portal.context.IContextResolver;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.spring.ServletConfigAwareProcessorServlet;
import org.jasig.portal.utils.ResourceNotFoundException;
import org.springframework.context.ApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

/**
 * Initializes pluto container, routes requests to the appropriate portal context
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class PortalServlet extends ServletConfigAwareProcessorServlet {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PortalServlet.class);
    
    /**
     * Name of the context resolver bean
     */
    public final static String CONTEXT_RESOLVER_BEAN_NAME="contextResolver";
    
    /**
     * Name of the container accessor bean
     */
    private static final String CONTAINER_NAME="portal.portletcontainer.container";
	
	protected IContextResolver contextResolver=null;
	
	public void init() throws ServletException {
		super.init();
		ApplicationContext appContext = WebApplicationContextUtils.getWebApplicationContext(getServletContext());
        contextResolver=(IContextResolver) appContext.getBean(CONTEXT_RESOLVER_BEAN_NAME, IContextResolver.class);
        PortletContainer portletContainer=(PortletContainer) appContext.getBean(CONTAINER_NAME, PortletContainer.class);
        if(portletContainer==null) {
            throw new ResourceNotFoundException(CONTAINER_NAME+" bean");
        }
        if (logger.isDebugEnabled()) {
            logger.debug("init() - portal servlet initialized");
        }
	}
	
	protected void service(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
        if (logger.isDebugEnabled()) {
            // write out parameter values
            Map<String,String[]> parameterMap=req.getParameterMap();
            StringBuffer pbuf=new StringBuffer();
            for( Map.Entry<String,String[]> parameter: parameterMap.entrySet()) {
                pbuf.append(" '").append(parameter.getKey()).append("'=[");
                for(int i=0;i<parameter.getValue().length;i++) {
                    pbuf.append("'").append(parameter.getValue()[i]).append("' ");
                }
                pbuf.append("] ");
            }
            logger.debug("service(HttpServletRequest, HttpServletResponse) - processing portal request. parameters: "+pbuf.toString());
        }

		if(contextResolver!=null) {
			try {
				IContext context=contextResolver.resolveContext(req);
                context.service(req,res);
			} catch (PortalException pe) {
				throw new ServletException("Unable to resolve portal context",pe.getCause());
			}	
		} else {
			throw new ServletException("Context resolver bean (\""+CONTEXT_RESOLVER_BEAN_NAME+"\") could not be obtained from application context.");
		}
	}

}
