/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.core.exception;

import org.jasig.portal.problem.IProblemsManager;
import org.jasig.portal.problem.RecentProblemsManager;

/**
 * Base portal exception class.
 * This class is intended to be extended by more specific exceptions.
 * @author andrew.petro@yale.edu
 * @version $Revision: 11936 $ $Date: 2007-04-05 15:21:56 -0500 (Thu, 05 Apr 2007) $
 */
public class PortalException extends Exception {
    private static IProblemsManager problemsManager = new RecentProblemsManager();
    
    /**
     * Set the problems manager to which PortalExceptions
     * should report themselves.
     * @param problemsManager - a manager to which to report insantiated PortalExceptions.
     */
    public static final void setProblemsManager(IProblemsManager problemsManager) {
        PortalException.problemsManager = problemsManager;
    }
    
    public static final IProblemsManager getProblemsManager() {
        return PortalException.problemsManager;
    }

    public PortalException(String message, Throwable cause) {
        super(message, cause);
        this.report();
    }

    public PortalException(String message) {
        super(message);
        this.report();
    }

    public PortalException(Throwable cause) {
        super(cause);
        this.report();
    }
    
    private void report() {
        if (PortalException.problemsManager != null) {
            PortalException.problemsManager.reportThrowable(this);
        }
    }
}