/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.core.exception;

import org.jasig.portal.problem.IProblemsManager;
import org.jasig.portal.problem.RecentProblemsManager;

/**
 * Base portal exception class.
 * This class is intended to be extended by more specific exceptions.
 * @author andrew.petro@yale.edu
 * @version $Revision: 11911 $ $Date: 2007-03-23 10:16:18 -0500 (Fri, 23 Mar 2007) $
 */
public class PortalRuntimeException extends RuntimeException {
    private static IProblemsManager problemsManager = new RecentProblemsManager();
    
    /**
     * Set the problems manager to which PortalExceptions
     * should report themselves.
     * @param problemsManager - a manager to which to report insantiated PortalExceptions.
     */
    public static final void setProblemsManager(IProblemsManager problemsManager) {
        PortalRuntimeException.problemsManager = problemsManager;
    }
    
    public static final IProblemsManager getProblemsManager() {
        return PortalRuntimeException.problemsManager;
    }


    public PortalRuntimeException(String message, Throwable cause) {
        super(message, cause);
        this.report();
    }

    public PortalRuntimeException(String message) {
        super(message);
        this.report();
    }

    public PortalRuntimeException(Throwable cause) {
        super(cause);
        this.report();
    }
    
    private void report() {
        if (PortalRuntimeException.problemsManager != null) {
            PortalRuntimeException.problemsManager.reportThrowable(this);
        }
    }
}