/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.core.hibernate;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import org.hibernate.HibernateException;
import org.jasig.portal.core.BaseStringObjectId;

/**
 * Base Hibernate UserType for {@link BaseStringObjectId} based classes.
 * 
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public abstract class BaseStringObjectIdUserType<GT_ObjectId extends BaseStringObjectId> extends BaseObjectIdUserType<GT_ObjectId> {
    private static final int[] TYPES = { Types.VARCHAR };

    /**
     * Creates a new id that implements {@link BaseStringObjectId} for the String id.
     */
    protected abstract GT_ObjectId createObjectId(String strId);

    /**
     * @see org.hibernate.usertype.UserType#nullSafeGet(java.sql.ResultSet, java.lang.String[], java.lang.Object)
     */
    public Object nullSafeGet(ResultSet rs, String[] names, Object owner) throws HibernateException, SQLException {
        final String strId = rs.getString(names[0]);
        if (strId != null) {
            return this.createObjectId(strId);
        }
        else {
            return null;
        }
    }

    
    /**
     * @see org.jasig.portal.core.hibernate.BaseObjectIdUserType#notNullSet(java.sql.PreparedStatement, org.jasig.portal.core.ObjectId, int)
     */
    @Override
    protected void notNullSet(PreparedStatement st, GT_ObjectId value, int index) throws HibernateException, SQLException {
        final String strId = value.toString();
        st.setString(index, strId);
    }

    /**
     * @see org.hibernate.usertype.UserType#sqlTypes()
     */
    public int[] sqlTypes() {
        return TYPES;
    }
}
