/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.event;

/**
 * Represents the source of the PortalEvent.
 * 
 * @author andrew.petro@yale.edu
 * @author Michael Ivanov: mivanov@unicon.net
 * @version $Revision: 11911 $
 */
public class PortalEventSource {
    

    /** String representation of the type of event source. */
    private final String name;

    /**
     * The user layout as source of events, for example control actuation events 
     * (button presses) and channel window manipulation events 
     * (minimize and maximize).
     */
    public static final PortalEventSource LAYOUT = 
        new PortalEventSource("layout");

    /**
     * Framework-generated events, such as sessions ending and unsubscription
     * from channels.
     */
    public static final PortalEventSource FRAMEWORK = 
        new PortalEventSource("framework");


    private PortalEventSource(String name) {
        this.name = name;
    }

    public String toString() {
        return this.name;
    }
    
    /**
     * Two PortalEventSources are equal if their typeNames are equal.
     * @param other an object for comparison
     * @return true if other is a PortalEventSource with the same typeName.
     */
    public boolean equals(Object other) {
        if ( other == null || !(other instanceof PortalEventSource))
          return false;
        PortalEventSource otherSource = (PortalEventSource) other;
        return name.equals(otherSource.name);
    }
    
    public int hashCode() {
        return name.hashCode();
    }
}