/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.groups.security;

import java.io.Serializable;

/**
 *
 * This interface represents the set of questions any channel or service 
 * ("Owner") must answer if it wants to delegate the responsibility of assigning
 * and viewing permissions to the Permissions Manager portlet.
 *
 * Owners will be registered by the IPermissible classname that represents them.
 *
 * @author  Alex vigdor av317@columbia.edu
 */
public interface IPermissible {

    /** 
     * Return a list of tokens representing all the activities a target
     * controls with permissions. These tokens can be used by the channel to 
     * ascertain permissions at runtime after they have been entered with the 
     * Permissions Manager.
     */    
    public String[] getActivityTokens(String targetToken);
    
    /** 
     * For a given activity token, return a human-readable name.
     * Used in rendering the Permissions Manager GUI.
     * 
     * @token activity identifier
     */    
    public String getActivityName(String token);
    
    /** 
     * For a given activity token, return a more detailed string that describes
     * the activity.
     * Used in rendering the Permissions Manager GUI.
     * 
     * @token activity identifier
     */    
    public String getActivityDescription(String token);
    
    /** 
     * Return an array of tokens representing all targets this channel controls
     * with permissions.
     */    
    public String[] getTargetTokens();
    
    /** 
     * For a given target token, return a human-readable name.
     * Used in rendering the Permissions Manager GUI.
     * 
     * @token target identifier
     */    
    public String getTargetName(String token);
    
    /** 
     * Return more detailed description of a target.
     * Used in rendering the Permissions Manager GUI.
     * 
     * @token target identifier
     */    
    public String getTargetDescription(String token);
    
    /** 
     * Return the token used by this channel to represent itself as the owner of
     * generated permissions. Can be arbitrary, but must be unique - I've been 
     * using classnames.  This is also used by the channel/portlet
     * to request a PermissionManager from the AuthorizationService at runtime.
     */    
    public String getOwnerToken();

    /** 
     * Human-readable name of the owner - normally the Channel/Portlet name.
     */    
    public String getOwnerName();
    
    /** 
     * Return more detailed description of the owner.
     * Used in rendering the Permissions Manager GUI.
     */    
    public String getOwnerDescription();
}

