/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.groups.services;

import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.gap.groups.IEntityNameFinder;
import org.jasig.gap.groups.IEntityNameFinderFactory;

/**
 * @author Michael Ivanov
 * @version $Revision: 11911 $
 */
public class EntityNameFinderService {
    
  private static final Log log = LogFactory.getLog(EntityNameFinderService.class);
    
  private Map entityNameFinders;
  
/**
 * @return org.jasig.portal.groups.IEntityNameFinder
 */
public IEntityNameFinder getEntityNameFinder(Class type) {
        IEntityNameFinder finder = (IEntityNameFinder) getEntityNameFinders().get(type);
        if ( finder == null ) {
          log.error("Entity name finder for " + type.getName() + " could not be located.");
          throw new RuntimeException("Entity name finder for " + type.getName() + " could not be located.");
        }  
        return finder;
}

public void setEntityNameFinders ( Set entityNameFinders ) {
   for ( Iterator i = entityNameFinders.iterator(); i.hasNext(); ) {
      IEntityNameFinder finder = (IEntityNameFinder) i.next();
      getEntityNameFinders().put(finder.getType(),finder);
   }
}


/**
  * Gets all the entity types and tries to instantiate and cache a finder for each
  * one.  There needn't be a finder for every entity type, so if there's no entry
  * in the portal.properties, we just log the fact and continue.
 */
public void setEntityNameFinderFactoryNames( final Set entityNameFinderFactoryNames) {
    
    Iterator factoryClassNames = entityNameFinderFactoryNames.iterator();

    while ( factoryClassNames.hasNext() ) {
        
        String factoryClassName = (String) factoryClassNames.next();
        
        IEntityNameFinderFactory factory = null;
        try { 
            factory = (IEntityNameFinderFactory) Class.forName(factoryClassName).newInstance();
        } catch ( Exception e ) {
            throw new RuntimeException ( "Cannot load class "+factoryClassName,e);
        }  
        
        if ( factory != null ) {
          IEntityNameFinder entityNameFinder = factory.newFinder();
          getEntityNameFinders().put(entityNameFinder.getType(),entityNameFinder);
        }
    }
}
  
protected Map getEntityNameFinders() {
  if ( entityNameFinders == null )
      entityNameFinders = new HashMap();
  return entityNameFinders;
}

}