/* Copyright 2002 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.groups.services;

import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.definition.ILocalizedPortletDefinition;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.LocalizedPortletDefinitionImpl;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.registry.general.IPortletDefinitionRegistry;

/**
 * Reference implementation of <code>IEntityNameFinder</code> for <code>Portlet definitions</code>.
 * @author Michael Ivanov
 * @version $Revision: 11955 $
 */
public class ReferencePortletDefinitionNameFinder extends GenericNameFinder {
    
    private IPortletDefinitionRegistry portletDefinitionRegistry;
    private ILocaleManager localeManager;

    /**
     * Given the key, returns the entity's name.
     * @param key java.lang.String
     */
    public String getName (String key) {
      IPortletDefinition portletDef = portletDefinitionRegistry.getPortletDefinition(new PortletDefinitionId(key));
      if ( portletDef != null ) {
         ILocalizedPortletDefinition localPortDef = new LocalizedPortletDefinitionImpl(portletDef, localeManager);
         return localPortDef.getPortletInfo().getTitle();
      } else {
         throw new IllegalArgumentException("The portlet definition with ID="+key+" could not be found!");
      }
    }

    /**
     * Returns the entity type for this <code>IEntityFinder</code>.
     * @return java.lang.Class
     */
    public Class getType () {
       return PortletDefinitionId.class;
    }

    /**
     * @param portletDefinitionRegistry The portletDefinitionRegistry to set.
     */
    public void setPortletDefinitionRegistry(IPortletDefinitionRegistry portletDefinitionRegistry) {
        this.portletDefinitionRegistry = portletDefinitionRegistry;
    }

    public ILocaleManager getLocaleManager() {
        return localeManager;
    }

    public void setLocaleManager(ILocaleManager localeManager) {
        this.localeManager = localeManager;
    }

}