/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.i18n;

import java.util.Locale;
import java.util.Set;

import org.jasig.portal.portlet.window.IPortletWindow;

/**
 * Interface describing a stateful provider of locale information for a portal session.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public interface ILocaleManager {
    
    /**
     * This is a general method that can be used to resolve locale for various
     * components.
     * 
     * @param availableLocales unordered set of supported locales, may not be null or zero length.
     * @return best matching locale given various locale preferences, may not be null
     **/
    public Locale resolveLocale(Set availableLocales);
    
    /**
     * This is a general method that can be used to resolve locale for various
     * components.
     * 
     * @return Best locale given various locale preferences, may not be null.
     */
    public Locale resolveLocale();

    /**
     * This is a specific framework method for resolving
     * locale for a specific portlet window
     * 
     * @param portletWindow The Portlet Window to resolve the locale for, may not be null.
     * @return best matching locale given various locale preferences, may not be null
     **/
    public Locale resolveLocale(IPortletWindow portletWindow);
     
}
