/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.i18n;

import java.util.List;
import java.util.Locale;

import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.security.UserId;

/**
 * Portlet user locale preferences data access object.
 * 
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11955 $
 */
public interface IUserLocalePreferencesDao {

	/**
	 * Get the portal-wide user preferences.
	 * @param userId user id
	 * @return IUserLocalePreferencesTransfer
	 */
	public List<IUserLocalePreferenceTransfer> getUserLocalePreferences(UserId userId);
	
	/**
	 * Persist the user's locale preferences.
	 * @param userLocalePreference
	 */
	public void storeUserLocalePreference(IUserLocalePreferenceTransfer userLocalePreference);
	
	/**
	 * Removes a locale preference from a user.
	 * @param userLocalePreference
	 */
	public void removeUserLocalePreference(IUserLocalePreferenceTransfer userLocalePreference);
	
	/**
	 * Get the locale preferences for a specific piece of content.
	 * @param userId user id
	 * @param portletWindowId content id
	 * @return IUserLocalePreferencesTransfer
	 */
	public List<IUserLocalePreferenceTransfer> getContentUserLocalePreferences(UserId userId, PortletWindowId portletWindowId);
	
	/**
	 * Persist the user's locale prefernces for a content.
	 * @param userLocalePreference
	 */
	public void setContentUserLocalePreference(IUserLocalePreferenceTransfer userLocalePreference);
	
	/**
	 * Removes a locale preference from the user for a content.
	 * @param userLocalePreference
	 */
	public void removeContentUserLocalePreference(IUserLocalePreferenceTransfer userLocalePreference);
	
	/**
	 * Creates a new UserLocalePreferenceTransfer object.
	 * @param userId
	 * @param locale
	 * @return IUserLocalePreferenceTransfer
	 */
	public IUserLocalePreferenceTransfer createUserLocalePreference(UserId userId, Locale locale);
	
	/**
	 * Creates a new UserLocalePreferenceTransfer object
	 * @param userId
	 * @param portletWindowId
	 * @param locale
	 * @return IUserLocalePreferenceTransfer
	 */
	public IUserLocalePreferenceTransfer createContentUserLocalePreference(UserId userId, PortletWindowId portletWindowId, Locale locale);
}
