/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.i18n;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;

import org.jasig.portal.utils.LocaleUtils;

/**
 * System locale preference provider.  The preference is a comman separated list of locales.
 * 
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11911 $
 */
public class LocaleListPreferenceProvider implements ILocalePreferenceProvider {
	private List localePreferences;
    
	/**
     * @return Returns the localePreferences.
     */
    public List getLocalePreferences() {
        return this.localePreferences;
    }
    /**
     * @param localePreferences The localePreferences to set.
     */
    public void setLocalePreferences(List localePreferences) {
        final List parsedLocalePrefs = new ArrayList(localePreferences.size());
        for (final Iterator localePrefItr = localePreferences.iterator(); localePrefItr.hasNext();) {
            final Object locale = localePrefItr.next();
            if (locale instanceof Locale) {
                parsedLocalePrefs.add(locale);
            }
            else {
                final String localeString = (String)locale;
                Locale newLocale = LocaleUtils.parseToLocale(localeString);
                if (newLocale != null) {
                	parsedLocalePrefs.add(newLocale);
                }
            }
        }
        
        this.localePreferences = Collections.unmodifiableList(parsedLocalePrefs);
    }


    public List getLocalePreferenceOrder() {
        return this.localePreferences;
	}
}
