/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.i18n;

import java.util.Locale;

import javax.servlet.http.HttpServletResponse;

import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IWritableHttpServletRequest;

/**
 * Processor to get the locale out of the request for the locale manager.
 * 
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11911 $
 */
public class LocaleParameterProcessor implements IRequestParameterProcessor {

	private Locale browserLocale;
	private ILocalePreferenceProcessor browserLocalePreferenceProvider;
	
	public void setBrowserLocalePreferenceProvider(ILocalePreferenceProcessor browserLocalePreferenceProvider) {
		this.browserLocalePreferenceProvider = browserLocalePreferenceProvider;
	}

	/**
	 * Get the browser locale from the http request.
	 */
	public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
		browserLocale = req.getLocale();
		browserLocalePreferenceProvider.setLocale(browserLocale);
		return true;
	}

	public boolean canEvaluateVariables() {
		return false;
	}

	/**
	 * Get the locale that was set by processParameters.
	 * @return Locale of the browser
	 */
	public Locale getBrowserLocale() {
		return browserLocale;
	}

}
