/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.i18n;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;

import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.security.UserId;

/**
 * User locale preference provider.  The preferences are stored in the database, so it 
 * uses the DAO to get them.
 * 
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11955 $
 */
public class UserLocalePreferenceProvider implements ILocalePreferenceProvider {
	private IUserLocalePreferencesDao userLocalePreferencesDao;
	private IUserController userController;
	private List<Locale> userLocales=null;

	/**
     * @return Returns the userController.
     */
    public IUserController getUserController() {
        return this.userController;
    }
    /**
     * @param userController The userController to set.
     */
    public void setUserController(IUserController userController) {
        this.userController = userController;
    }

    /**
     * @return Returns the userLocalePreferencesDao.
     */
    public IUserLocalePreferencesDao getUserLocalePreferencesDao() {
        return this.userLocalePreferencesDao;
    }
    /**
     * @param userLocalePreferencesDao The userLocalePreferencesDao to set.
     */
    public void setUserLocalePreferencesDao(IUserLocalePreferencesDao userLocalePreferencesDao) {
        this.userLocalePreferencesDao = userLocalePreferencesDao;
    }


    public List getLocalePreferenceOrder() {
        if(userLocales==null) {
            final UserId userId = userController.getUserId();
            
            if (userId != null) {
                final List<IUserLocalePreferenceTransfer> userLocalePreferences = userLocalePreferencesDao.getUserLocalePreferences(userId);
                
                this.userLocales = new ArrayList<Locale>(userLocalePreferences.size());
                for (final IUserLocalePreferenceTransfer userLocalPref : userLocalePreferences) {
                    userLocales.add(userLocalPref.getLocale());
                }
            }
        }
        
        if(userLocales!=null) {
            return userLocales;
        } else {
            return Collections.EMPTY_LIST;
        }
	}
}
