/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.i18n;

import java.util.Locale;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.security.UserId;

/**
 * User locale preferences.
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11955 $
 */
public class UserLocalePreferenceTransferImpl implements IUserLocalePreferenceTransfer {

	private long userLocalePreferenceId;
	private UserId userId;
	private PortletWindowId portletWindowId;
	private Locale locale;
	
	public UserLocalePreferenceTransferImpl() {
		super();
	}

	/**
	 * Get the user locale preference id.
	 */
	public long getUserLocalePreferenceId() {
		return userLocalePreferenceId;
	}

	/**
	 * Set the user locale preference id.
	 * @param userLocalePreferenceId locale preference id
	 */
	public void setUserLocalePreferenceId(long userLocalePreferenceId) {
		this.userLocalePreferenceId = userLocalePreferenceId;
	}

	/**
	 * Get the user id.
	 * @return UserId
	 */
	public UserId getUserId() {
		return userId;
	}

	/**
	 * Set the user id.
	 * @param userId 
	 */
	public void setUserId(UserId userId) {
		this.userId = userId;
	}

	/**
	 * Get the portlet window id
	 * @return PortletWindowId
	 */
	public PortletWindowId getPortletWindowId() {
		return portletWindowId;
	}

	/**
	 * Set the portlet window id.
	 * @param portletWindowId
	 */
	public void setPortletWindowId(PortletWindowId portletWindowId) {
		this.portletWindowId = portletWindowId;
	}

	/**
	 * Get the locale.
	 * @return Locale
	 */
	public Locale getLocale() {
		return locale;
	}

	/**
	 * Set the locae.
	 * @param locale
	 */
	public void setLocale(Locale locale) {
		this.locale = locale;
	}

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof IUserLocalePreferenceTransfer)) {
            return false;
        }
        IUserLocalePreferenceTransfer rhs = (IUserLocalePreferenceTransfer)object;
        return new EqualsBuilder()
            .append(this.locale, rhs.getLocale())
            .append(this.userId, rhs.getUserId())
            .append(this.portletWindowId, rhs.getPortletWindowId())
                .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(190351811, 632466401)
            .append(this.locale)
            .append(this.userId)
            .append(this.portletWindowId)
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
            .append("userLocalePreferenceId", this.userLocalePreferenceId)
            .append("userId", this.userId)
            .append("locale", this.locale)
            .append("portletWindowId", this.portletWindowId)
            .toString();
    }
}
