/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout;

import java.util.Set;

import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * An entity providing mapping from one set of windowIds (template Ids) to another (target Ids). 
 * An example would be mapping from window Ids in the template layout (which belong to
 * a template user) to window Ids in the actual users' layout.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletWindowMapping {
    /**
     * Obtain a mapped portlet windowId
     * @param portletWindowId
     * @return mapped portlet window id or <code>null</code> if no mapping has been
     * established
     */
    PortletWindowId getPortletWindowId(PortletWindowId portletWindowId);
    
    /**
     * Obtain a mapped portlet windowId, creating a new portlet window and 
     * establishing a mapping if necessary.
     * @param portletWindowId
     * @return
     */
    PortletWindowId getCreatePortletWindowId(PortletWindowId portletWindowId);
    
    /**
     * Synchronize portlet window Id mapping with a complete list of template Ids.
     * The procedure will create new windows and mappings if necessary, and
     * (depending on the implementation) removing mappings for template windows
     * that are not present in the list.
     * @param portletWindowIds a Set of template {@link PortletWindowId}s
     */
    void updateMapping(Set portletWindowIds);
}
