/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout.transitory;

import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * An interface describing transient layout manager that can
 * operate on single portlets. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface ITransientLayoutManager extends ITransientLayoutManagerCommands {
    /**
     * Obtain active portlet window
     * @return active portlet window, <code>null</code> if none has been requested,
     * or if a window could not be produced for the requested functionalName.
     */
    IPortletWindow getPortletWindow();
    
    /**
     * Obtain id of active portlet window
     * @return active portlet window id or <code>null</code> if none has been requested,
     * or if a window could not be produced for the requested functionalName.
     */
    PortletWindowId getPortletWindowId();

    /**
     * Obtain portlet window Id for a given functional name.
     * The portlet window may be either transient (i.e. created
     * by the current class), or from underlying persistent
     * user layout.
     * 
     * @param functionalName
     * @return portlet window id 
     */
    PortletWindowId getPortletWindowId(String functionalName);
}
