/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout.transitory;

import javax.portlet.WindowState;
import javax.servlet.http.HttpServletResponse;

import org.jasig.portal.portlet.rendering.IPortletWindowManager;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IWritableHttpServletRequest;
import org.springframework.beans.factory.annotation.Required;

/**
 * Processes uPortal2 "channel by fname" commands. 
 *
 * When a channel is requested, a rendering attribute can be set (to focus on the injected channel).
 * The name of the attribute is given by focusModeRenderingAttributeName property (defaults to userLayoutRoot).
 * Appropriate rendering attribute provider (i.e. structure) is also specified as a property.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class LegacyTransientUserLayoutParameterProcessor implements IRequestParameterProcessor {
    protected ITransientLayoutManager transientLayoutManager;
    protected IPortletWindowManager portletWindowManager;
    
    /**
     * The query parameter name that can be appended to a baseActionUrl along
     * with its value, the fname of a channel, to cause an instance of that
     * channel to appear in focused mode without the user having to subcribe
     * to that channel. Examples are infrastructure channels like
     * CChannelManager and CUserPreferences. They don't reside in a user's
     * layout but are merged in via the fname functionality as needed.
     */
    public static final String FNAME_PARAM = "uP_fname";
    
    protected String transientPortletLayoutId="pt1";

    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        String fName=req.getParameter(FNAME_PARAM);
        if(fName!=null) {
            PortletWindowId windowId=transientLayoutManager.showPortlet(fName);
            if(windowId!=null) {
                // focus on the portlet
                portletWindowManager.setWindowState(windowId,WindowState.MAXIMIZED);
            }
        }
        return true;
    }
    
    public boolean canEvaluateVariables() {
        return false;
    }

    public void setTransientLayoutManager(ITransientLayoutManager transientLayoutManager) {
        this.transientLayoutManager = transientLayoutManager;
    }


    /**
     * Layout node id under which transient portlet is injected (defaults to "pt1")
     * @param transientPortletLayoutId
     */
    public void setTransientPortletLayoutId(String transientPortletLayoutId) {
        this.transientPortletLayoutId = transientPortletLayoutId;
    }

    /**
     * Provide reference to portlet window manager to focus on the transient portlets
     * @param portletWindowManager
     */
    @Required
    public void setPortletWindowManager(IPortletWindowManager portletWindowManager) {
        this.portletWindowManager = portletWindowManager;
    }
    
    
}
