/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout.transitory;

import java.util.Stack;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xalan.extensions.XSLProcessorContext;
import org.apache.xalan.templates.ElemExtensionCall;
import org.apache.xalan.transformer.TransformerImpl;
import org.jasig.portal.url.IPortalUrl;
import org.jasig.portal.url.PortalUrlXalanElements;
import org.w3c.dom.Node;

/**
 * Xalan extension elements for {@link org.jasig.portal.layout.transitory.ITransientLayoutManagerCommands} 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11917 $
 */
public class TransientLayoutManagerXalanElements {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(TransientLayoutManagerXalanElements.class);

    /**
     * usage <tlayout:portlet fname="uP3.functionalName""/>
     * 
     * @param context
     * @param elem
     * @throws TransformerException
     */
    public void parameter(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl=context.getTransformer();
        Node contextNode=context.getContextNode();
        ITransientLayoutManagerCommands rapc=getTransientLayoutManagerCommands(transformerImpl);
        String fname=elem.getAttribute("fname",contextNode,transformerImpl);
        rapc.showPortlet(fname);
    }
    
    /**
     * Obtain {@link ITransientLayoutManagerCommands} URL constructor
     * @param transformer
     * @return
     */
    protected ITransientLayoutManagerCommands getTransientLayoutManagerCommands(Transformer transformer) {
        Stack urlStack=PortalUrlXalanElements.getUrlStack(transformer);
        if(urlStack!=null) {
            IPortalUrl portalUrl=(IPortalUrl) urlStack.peek();
            if(portalUrl!=null) {
                ITransientLayoutManagerCommands rapc=(ITransientLayoutManagerCommands) portalUrl.getUrlConstructor(ITransientLayoutManagerCommands.class.getName());
                if(rapc==null) {
                    final IllegalStateException ise = new IllegalStateException("transient layout manager commands url constructor must be registered");
                    logger.error("unable to obtain transient layout manager commands constructor", ise);
                    throw ise;
                } else {
                    return rapc;
                }
            } else {
                final IllegalStateException ise = new IllegalStateException("url stack on the transformer is empty");
                logger.error("unable to obtain portal url from the url stack", ise);
                throw ise;
            }
        } else {
            final IllegalStateException ise = new IllegalStateException("url stack is null");
            logger.error("unable to obtain url stack from the transformer", ise);
            throw ise;
        }
    }
}
