/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout.transitory;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.layout.uportal2.IUserLayoutManager;
import org.jasig.portal.portlet.registry.transitory.ITransientPortletWindowRegistry;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlet.window.IPortletWindowControl;
import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * Default implementation of the transient layout manager,
 * designed to work with uPortal2 layouts.  
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class TransientUserLayoutManagerImpl implements ITransientLayoutManager {
    //  injected properties
    IUserLayoutManager underlyingLayoutManager;
    ITransientPortletWindowRegistry transientPortletWindowRegistry;
    
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(TransientUserLayoutManagerImpl.class);
    protected Map windowIdByFunctionalName=new HashMap();
    protected PortletWindowId currentWindowId=null;
    
    
    public IPortletWindow getPortletWindow() {
        if(currentWindowId!=null) {
            return transientPortletWindowRegistry.getPortletWindow(currentWindowId);
        } else {
            return null;
        }
    }
    public PortletWindowId getPortletWindowId() {
        return currentWindowId;
    }
    public PortletWindowId getPortletWindowId(String functionalName) {
        // check local fname map
        PortletWindowId windowId=(PortletWindowId) windowIdByFunctionalName.get(functionalName);
        if(windowId==null) {
            // see if the specified portlet is present in the underlying user layout
            try {
                String str_windowId=underlyingLayoutManager.getSubscribeId(functionalName);
                if(str_windowId!=null) {
                    windowId=new PortletWindowId(str_windowId);
                }
            } catch (PortalException e) {
                if (logger.isWarnEnabled()) {
                    logger.warn("getPortletWindowId(String) - exception raised when trying to locate portlet by functional name in the user layout : functionalName=" + functionalName, e);
                }
            }
            if(windowId==null) {
                // create new transient window
                IPortletWindowControl window=transientPortletWindowRegistry.createTransientPortletWindowControl(functionalName);
                if(window!=null) {
                    windowId=window.getPortletWindowId();
                }
            }
            if(windowId!=null) {
                windowIdByFunctionalName.put(functionalName,windowId);
            }
        }
        
        return windowId;
    }

    public PortletWindowId showPortlet(String functionalName) {
        PortletWindowId windowId=getPortletWindowId(functionalName);
        // replace current window only if a valid alternative was requested
        if(windowId!=null) {
            currentWindowId=windowId;
        }
        return windowId;
    }
    public void setUnderlyingLayoutManager(IUserLayoutManager underlyingLayoutManager) {
        this.underlyingLayoutManager = underlyingLayoutManager;
    }
    public void setTransientPortletWindowRegistry(ITransientPortletWindowRegistry transientPortletWindowRegistry) {
        this.transientPortletWindowRegistry = transientPortletWindowRegistry;
    }
    
    
    
}
