/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.layout.uportal2;


/**
 * ILegacyLayoutSourceCommands is a command interface for legacy user layout managers
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public interface ILegacyLayoutSourceCommands {
    public final static String CONSTRUCTOR_NAME = ILegacyLayoutSourceCommands.class.getName();
    
    /**
     * Reload layout from store
     */
    public void reloadLayout();
    /**
     * Clear all markings (move and add)
     */
    public void clearMarkings();
    /**
     * Delete a given node
     * @param nodeId id of a node to be deleted
     */
    public void removeNode(String nodeId);
    /**
     * Perform a move operation
     * @param nodeId node to be moved
     * @param parentId id of a future parent node
     * @param nextId id of a future next sibling node
     */
    public void moveNode(String nodeId, String parentId, String nextId);
    /**
     * Rename a given node
     * @param nodeId id of a node to be renamed
     * @param newName new name
     */
    public void renameNode(String nodeId, String newName);
    /**
     * Show "add" operation markings
     */
    public void addMarkings();
    /**
     * Perform add operation (add subscribe-selected channel or folder)
     * @param parentId id of a future parent node
     * @param nextId id of a future next sibling node
     * @return newly assigned nodeId
     */
    public String addNode(String parentId, String nextId);
    /**
     * Show "move" operation markings
     * @param nodeId id of a node to be moved
     */
    public void moveMarkings(String nodeId);
    
    /**
     * Parse out and execute a structure of add commands
     * @param command
     */
    public void processAddCommand(String command);
}
