/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.layout.uportal2;

import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * INodeAdditionManager defines an interface for a user session entity that contains information about
 * the portlets user is trying to subscribe to. The manager entity serves as a middle man between subscription channel
 * and the user layout manager.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public interface INodeAdditionManager {
    
    /**
     * Explicitly supply a node
     * @param node
     */
    public void addNode(IUserLayoutNodeDescription node);
    
    /**
     * Obtain node that's being added or subscribed to.
     * @return layout node that's bein added.
     */
    public IUserLayoutNodeDescription getNode();
    
    
    /**
     * Notifies node addition manager that the node has been added
     * @param node
     */
    public void nodeAdded(IUserLayoutNodeDescription node);
    
    
    /**
     * Removes any outstanding nodes
     */
    public void clear();
    
    /**
     * Add portlet node
     * @param portletWindowId
     * @param portletName
     * @param functionalName functional name of the portlet
     */
    public void addPortlet(PortletWindowId portletWindowId, String portletName, String functionalName);
    
    /**
     * Add folder node
     * @param folderName
     */
    public void addFolder(String folderName);
}
