/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.layout.uportal2;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.url.IUrlConstructor;
import org.jasig.portal.url.UrlInfo;
import org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider;

/**
 * LegacyLayoutSourceCommandsUrlConstructor is a url constructor implementing uP2 user layout manager syntax
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class LegacyLayoutSourceCommandsUrlConstructor implements ILegacyLayoutSourceCommands, IUrlConstructor {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(LegacyLayoutSourceCommandsUrlConstructor.class);

    private boolean addMarkings = false;
    
    private boolean addNode = false;
    private String addNodeParentId = null;
    private String addNodeNexId = null;
    
    private boolean clearMarkings = false;
    
    private boolean moveMarkings = false;
    private String moveMarkingsNodeId = null;
    
    private boolean moveNode = false;
    private String moveNodeNodeId = null;
    private String moveNodeParentId = null;
    private String moveNodeNextId = null;
    
    private boolean reloadLayout = false;
    
    private boolean removeNode = false;
    private String removeNodeNodeId = null;
    
    private boolean renameNode = false;
    private String renameNodeNodeId = null;
    private String renameNodeNewName = null;
    
    private String addCommands=null;
    
    
    //Methods from ILegacyLayoutSourceCommands
    public void addMarkings() {
        this.addMarkings = true;        
    }
    
    /**
     * @return <code>NULL</code>
     */
    public String addNode(String parentId, String nextId) {
        if (parentId == null) {
            throw new IllegalArgumentException("parentId may not be null");
        }

        this.addNode = true;
        this.addNodeParentId = parentId;
        this.addNodeNexId = nextId;
        return null;
    }

    public void clearMarkings() {
        this.clearMarkings = true;
    }

    public void moveMarkings(String nodeId) {
        if (nodeId == null) {
            throw new IllegalArgumentException("nodeId may not be null");
        }

        this.moveMarkings = true;
        this.moveMarkingsNodeId = nodeId;
    }

    public void moveNode(String nodeId, String parentId, String nextId) {
        if (nodeId == null) {
            throw new IllegalArgumentException("nodeId may not be null");
        }
        if (parentId == null) {
            throw new IllegalArgumentException("parentId may not be null");
        }
        
        this.moveNode = true;
        this.moveNodeNodeId = nodeId;
        this.moveNodeParentId = parentId;
        this.moveNodeNextId = nextId;
    }

    public void reloadLayout() {
        this.reloadLayout = true;
    }

    public void removeNode(String nodeId) {
        if (nodeId == null) {
            throw new IllegalArgumentException("nodeId may not be null");
        }
        
        this.removeNode = true;
        this.removeNodeNodeId = nodeId;
    }

    public void renameNode(String nodeId, String newName) {
        if (nodeId == null) {
            throw new IllegalArgumentException("nodeId may not be null");
        }
        if (newName == null) {
            throw new IllegalArgumentException("newName may not be null");
        }
        
        this.renameNode = true;
        this.renameNodeNodeId = nodeId;
        this.renameNodeNewName = newName;
    }

    
    //Methods from IUrlConstructor
    public void writeToUrl(UrlInfo urlInfo) {
        if (this.addMarkings) {
            urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.REQUEST_ADD_TARGETS_COMMAND,
                                      LegacyLayoutSourceRequestParameterProcessor.CHANNEL_NODE_TYPE);
        }
        
        if (this.addNode) {
            urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.ADD_TARGET_COMMAND,
                                      LegacyLayoutSourceRequestParameterProcessor.CHANNEL_NODE_TYPE);
            
            urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.PARENT_ID_PARAM,
                                      this.addNodeParentId);

            if (this.addNodeNexId != null) {
                urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.NEXT_ID_PARAM,
                                          this.addNodeNexId);
            }
        }
        
        if (this.clearMarkings) {
            urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.CLEAR_MARAKINGS_COMMAND,
                                      Boolean.TRUE.toString());
        }
        
        if (this.moveMarkings) {
            urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.REQUEST_MOVE_TARGETS_COMMAND,
                                      this.moveMarkingsNodeId);
        }
        
        if (this.moveNode) {
            urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.MOVE_TARGET_COMMAND,
                                      this.moveNodeNodeId);
            
            urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.PARENT_ID_PARAM,
                                      this.moveNodeParentId);
            
            if (this.moveNodeNextId != null) {
                urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.NEXT_ID_PARAM,
                                          this.moveNodeNextId);
            }
        }
        
        if (this.reloadLayout) {
            urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.RELOAD_LAYOUT_COMMAND,
                                      Boolean.TRUE.toString());
        }
        
        if (this.removeNode) {
            urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.REMOVE_TARGET_COMMAND,
                                      this.removeNodeNodeId);
        }
        
        if (this.renameNode) {
            urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.RENAME_TARGET_COMMAND,
                                      this.renameNodeNodeId);
            
            urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.RENAME_NAME_PARAM,
                                      this.renameNodeNewName);
        }
        
        if(this.addCommands!=null) {
            try {
                urlInfo.addParameterValue(LegacyLayoutSourceRequestParameterProcessor.ADD_COMMANDS_COMMAND,URLEncoder.encode(this.addCommands,IUrlBasicSyntaxProvider.DEFAULT_ENCODING));
            } catch (UnsupportedEncodingException e) {
                logger.error("writeToUrl(UrlInfo) - can not url-encode add commands", e);
            }
        }
    }

    public void processAddCommand(String command) {
        if(this.addCommands!=null) {
            logger.warn("processAddCommand(String) - layout commands can be specified only once in this implementation: current commands="+this.addCommands+" new command=" + command, null);
        }
        this.addCommands=command;
    }
}
