/* Copyright 2002 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.layout.uportal2;

import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


/**
 * A class managing information contained in a user layout channel node.
 *
 * @author <a href="mailto:pkharchenko@interactivebusiness.com">Peter Kharchenko</a>
 * @version 1.0
 */
public class UserLayoutChannelDescription extends UserLayoutNodeDescription implements IUserLayoutChannelDescription  {

    PortletWindowId portletWindowId;
    String functionalName;

    public UserLayoutChannelDescription() {
        super();
    }

    public UserLayoutChannelDescription(IUserLayoutChannelDescription d) {
        super(d);
        this.portletWindowId=d.getPortletWindowId();
    }

    /**
     * Reconstruct channel information from an xml <code>Element</code>
     *
     * @param xmlNode a user layout channel <code>Element</code> value
     * @exception PortalException if xml is malformed
     */
    public UserLayoutChannelDescription(Element xmlNode) throws PortalException {
        this();
        if(!xmlNode.getNodeName().equals("channel")) {
            throw new PortalException("Given XML Element is not a channel!");
        }

        // could do some validation here, but this code will probably go away anyhow

        // standard Node attributes
        this.setId(xmlNode.getAttribute("ID"));
        this.setName(xmlNode.getAttribute("name"));
        this.setFunctionalName(xmlNode.getAttribute("fname"));
        this.setUnremovable((new Boolean(xmlNode.getAttribute("unremovable"))).booleanValue());
        this.setImmutable((new Boolean(xmlNode.getAttribute("immutable"))).booleanValue());

        // channel-specific attributes
        this.setPortletWindowId(new PortletWindowId(xmlNode.getAttribute(("portletWindowId"))));
    }

    

  
    /**
     * Creates a <code>org.w3c.dom.Element</code> representation of the current channel.
     *
     * @param root a <code>Document</code> for which the <code>Element</code> should be created.
     * @return a <code>Node</code> value
     */
    public Element getXML(Document root) {
        Element node=root.createElement("channel");
        this.addNodeAttributes(node);
        return node;
    }
    public void addNodeAttributes(Element node) {
        super.addNodeAttributes(node);
        node.setAttribute("portletWindowId",this.getPortletWindowId().toString());       
        node.setAttribute("fname",this.getFunctionalName());
    }

    /**
     * Returns a type of the node, could be FOLDER or CHANNEL integer constant.
     *
     * @return a type
     */
    public int getType() {
      return CHANNEL;
    }


    /**
     * @return Returns the portletWindowId.
     */
    public PortletWindowId getPortletWindowId() {
        return portletWindowId;
    }
    /**
     * @param portletWindowId The portletWindowId to set.
     */
    public void setPortletWindowId(PortletWindowId portletWindowId) {
        this.portletWindowId = portletWindowId;
    }

    public String getFunctionalName() {
        return functionalName;
    }

    public void setFunctionalName(String functionalName) {
        this.functionalName = functionalName;
    }
}
