/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.layout.uportal2.simple;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.core.exception.PortalRuntimeException;
import org.jasig.portal.layout.uportal2.ILegacyLayoutManagerFactory;
import org.jasig.portal.layout.uportal2.IUserLayoutManager;
import org.jasig.portal.layout.uportal2.simple.dao.ITemplateUserWindowMappingDao;
import org.jasig.portal.layout.uportal2.simple.dao.IUserLayoutDao;
import org.jasig.portal.layout.uportal2.simple.dao.TemplateUserWindowMappingDaoWrapper;
import org.jasig.portal.portlet.factory.IPortletApplicationEntityRegistryFactory;
import org.jasig.portal.portlet.factory.IPortletEntityRegistryFactory;
import org.jasig.portal.portlet.factory.IPortletWindowRegistryFactory;
import org.jasig.portal.portlet.registry.general.IPortletApplicationEntityRegistry;
import org.jasig.portal.portlet.registry.general.IPortletEntityRegistry;
import org.jasig.portal.portlet.registry.general.IPortletWindowRegistry;
import org.jasig.portal.rendering.sax.xslt.ITransformerHelper;
import org.jasig.portal.security.UserId;

/**
 * SimpleUserLayoutManagerFactory instantiates simple user layout based on a static value
 * of a template user id.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class SimpleUserLayoutManagerFactory implements ILegacyLayoutManagerFactory {
    // injected properties
    IUserLayoutDao store;
    ITransformerHelper transformerHelper;
    IPortletWindowRegistry windowRegistry;
    IPortletEntityRegistry entityRegistry;
    IPortletApplicationEntityRegistry applicationEntityRegistry;
    IPortletEntityRegistryFactory entityRegistryFactory;
    IPortletWindowRegistryFactory windowRegistryFactory;
    IPortletApplicationEntityRegistryFactory applicationEntityRegistryFactory;
    ITemplateUserWindowMappingDao userWindowMappingDao;
    
    String templateUserId;
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(SimpleUserLayoutManagerFactory.class);


    
    /**
     * @param store The store to set.
     */
    public void setStore(IUserLayoutDao store) {
        this.store = store;
    }

    public IUserLayoutManager createUserLayoutManager(UserId userId) {
        try {
            UserId templateUser = new UserId(templateUserId); 
            // set up template user window mapping DAO
            IPortletWindowRegistry templatePortletWindowRegistry=windowRegistryFactory.getPortletWindowRegistry(templateUser);
            IPortletEntityRegistry templatePortletEntityRegistry=entityRegistryFactory.getPortletEntityRegistry(templateUser);
            IPortletApplicationEntityRegistry templateApplicationEntityRegistry=applicationEntityRegistryFactory.getPortletApplicationEntityRegistry(templateUser);
            TemplateUserWindowMappingDaoWrapper dao=new TemplateUserWindowMappingDaoWrapper();
            dao.setDao(userWindowMappingDao);
            dao.setEntityRegistry(entityRegistry);
            dao.setTemplateUserEntityRegistry(templatePortletEntityRegistry);
            dao.setWindowRegistry(windowRegistry);
            dao.setTemplateUserWindowRegistry(templatePortletWindowRegistry);
            dao.setApplicationEntityRegistry(applicationEntityRegistry);
            dao.setTemplateUserApplicationEntityRegistry(templateApplicationEntityRegistry);
            SimpleUserLayoutManager sulm=new SimpleUserLayoutManager(userId,getStore(),templateUser,dao);
            sulm.setTransformerHelper(getTransformerHelper());
            return sulm;
        } catch (PortalException e) {
            logger.error("exception raised while instantiating simple user layout manager for userId = " + userId + "", e);
            throw new PortalRuntimeException("exception encountered while instantiating SLM",e);
        }
    }
    /**
     * @return Returns the transformerHelper.
     */
    public ITransformerHelper getTransformerHelper() {
        if(transformerHelper==null) {
            logger.error("getTransformerHelper() - transformer helper has not been provided", null);
        }
        return transformerHelper;
    }
    /**
     * @param transformerHelper The transformerHelper to set.
     */
    public void setTransformerHelper(ITransformerHelper transformerHelper) {
        this.transformerHelper = transformerHelper;
    }
    /**
     * @return Returns the store.
     */
    public IUserLayoutDao getStore() {
        if(store==null) {
            logger.error("getStore() - store implementation has not been provided!", null);
        }
        return store;
    }
    public void setTemplateUserId(String templateUserId) {
        this.templateUserId = templateUserId;
    }
    public void setEntityRegistry(IPortletEntityRegistry entityRegistry) {
        this.entityRegistry = entityRegistry;
    }
    public void setEntityRegistryFactory(IPortletEntityRegistryFactory entityRegistryFactory) {
        this.entityRegistryFactory = entityRegistryFactory;
    }
    public void setWindowRegistry(IPortletWindowRegistry windowRegistry) {
        this.windowRegistry = windowRegistry;
    }
    public void setWindowRegistryFactory(IPortletWindowRegistryFactory windowRegistryFactory) {
        this.windowRegistryFactory = windowRegistryFactory;
    }
    public void setUserWindowMappingDao(ITemplateUserWindowMappingDao userWindowMappingDao) {
        this.userWindowMappingDao = userWindowMappingDao;
    }
    public void setApplicationEntityRegistry(IPortletApplicationEntityRegistry applicationEntityRegistry) {
        this.applicationEntityRegistry = applicationEntityRegistry;
    }
    public void setApplicationEntityRegistryFactory(IPortletApplicationEntityRegistryFactory applicationEntityRegistryFactory) {
        this.applicationEntityRegistryFactory = applicationEntityRegistryFactory;
    }
}
