/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.layout.uportal2.simple.dao;

import org.jasig.portal.security.UserId;
import org.w3c.dom.Document;

/**
 * Data Access Object for uPortal2-style user layouts.
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IUserLayoutDao {

    /**
     * Retreive a user layout document.
     *
     * @param user an <code>UserId</code> object representing the user
     * @return a <code>Document</code> containing user layout (conforms to userLayout.dtd)
     * @exception Exception if an error occurs
     */
    public Document getUserLayout(UserId user) throws Exception;

    /**
     * Persist user layout document.
     *
     * @param user an <code>UserId</code> object representing the user
     * @param layoutXML a <code>Document</code> containing user layout (conforming to userLayout.dtd)
     * @param channelsAdded a boolean flag specifying if new channels have been added to the current user layout (for performance optimization purposes)
     * @exception Exception if an error occurs
     */
    public void setUserLayout(UserId user, Document layoutXML, boolean channelsAdded) throws Exception;
    
    /**
     * Generate an instance id for a channel being added to the user layout
     *
     * @param user an <code>UserId</code> object representing the user
     * @return a <code>String</code> value
     * @exception Exception if an error occurs
     */
    public String generateNewChannelSubscribeId(UserId user) throws Exception;
      
    /**
     * Generate a folder id for a folder being added to the user layout
     *
     * @param user an <code>UserId</code> object representing the user
     * @return a <code>String</code> value
     * @exception Exception if an error occurs
     */
    public String generateNewFolderId(UserId user) throws Exception;

}

