/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout.uportal2.simple.dao;

import java.util.HashMap;
import java.util.List;

import org.jasig.portal.core.exception.PortalRuntimeException;
import org.jasig.portal.layout.IPortletWindowMapping;
import org.jasig.portal.layout.PortletWindowMappingImpl;
import org.jasig.portal.security.UserId;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

/**
 * Default hibernate implementation of {@link org.jasig.portal.layout.uportal2.simple.dao.ITemplateUserWindowMappingDao},
 * that's designed to work only with {@link org.jasig.portal.layout.PortletWindowMappingImpl}
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class TemplateUserWindowMappingDao extends HibernateDaoSupport implements ITemplateUserWindowMappingDao {
    private static final String QUERY_FIND_SLM_WIN_MAP_BY_USER_ID_TMPL_USER_ID = "FIND_SLM_WIN_MAP_BY_USER_ID_TMPL_USER_ID";
    private static final String QUERY_PARAM_USER_ID = "userId";
    private static final String QUERY_PARAM_TEMPLATE_USER_ID = "templateUserId";

    public IPortletWindowMapping getTemplateUserWindowMapping(UserId templateUserId, UserId userId) {
        // retrieve
        HibernateTemplate template = this.getHibernateTemplate();
        List l= template.findByNamedQueryAndNamedParam(
                QUERY_FIND_SLM_WIN_MAP_BY_USER_ID_TMPL_USER_ID,
                new String[] {QUERY_PARAM_USER_ID,QUERY_PARAM_TEMPLATE_USER_ID},
                new UserId[] {userId, templateUserId});
        PortletWindowMappingImpl wm = (PortletWindowMappingImpl) DataAccessUtils.uniqueResult(l);
        if(wm==null) {
            // create new mapping

            if (logger.isInfoEnabled()) {
                logger.info("creating new window mapping : templateUserId=" + templateUserId + ", userId=" + userId);
            }

            wm=new PortletWindowMappingImpl();
            wm.setUserId(userId);
            wm.setTemplateUserId(templateUserId);
            wm.setWindowMapping(new HashMap());
            wm.setEntityMapping(new HashMap());
            wm.setApplicationEntityMapping(new HashMap());
        } else {
            if (logger.isDebugEnabled()) {
                logger.debug("successfully read in window mapping : templateUserId=" + templateUserId + ", userId=" + userId);
            }
        }

        return wm;
    }

    public void setTemplateUserWindowMapping(UserId templateUserId, UserId userId, IPortletWindowMapping portletWindowMapping) {
        if(portletWindowMapping instanceof PortletWindowMappingImpl) {
            PortletWindowMappingImpl wm=(PortletWindowMappingImpl)portletWindowMapping;

            if (logger.isDebugEnabled()) {
                logger.debug("persisting template layout mappings : templateUserId=" + templateUserId + ", userId=" + userId);
            }
            this.getHibernateTemplate().saveOrUpdate(wm);
        } else {
            logger.error("unable to persist window mapping, since it is not an instance of PortletWindowMappingImpl : templateUserId=" + templateUserId + ", userId=" + userId, null);
            throw new PortalRuntimeException("unable to persist window mapping, since it is not an instance of PortletWindowMappingImpl");
        }
    }
}
