/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout.uportal2.simple.dao.filesystem;

import java.net.URI;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;

import org.jasig.portal.rendering.sax.xslt.ITransformerHelper;
import org.jasig.portal.utils.DtdResolver;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * An extension of {@link org.jasig.portal.layout.uportal2.simple.dao.filesystem.FileLayoutDao}
 * that performs an XSLT transform upon loading layout from file. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class TransformingLayoutDao extends FileLayoutDao {
    ITransformerHelper transformerHelper;
    String stylesheetLocation;

    protected Document loadLayoutFromFile(String layoutFile) throws Exception {
        URI stylesheetUri=new URI(this.getClass().getResource(stylesheetLocation).toString());
        Transformer transformer=transformerHelper.getTransformer(stylesheetUri);
        DOMResult dr=new DOMResult();
        
        DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
        docBuilderFactory.setValidating(true);
        DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
        docBuilder.setEntityResolver(new DtdResolver("layout-SNAPSHOT.dtd"));
        
        Document normalDom= docBuilder.parse(this.getClass().getResourceAsStream(layoutFile));
        String id = normalDom.getDocumentElement().getAttribute("ID");
        
        if (id != null && !id.equals("")) {
          normalDom.getDocumentElement().setIdAttribute("ID", true);
        }
        
        transformer.transform(new DOMSource(normalDom),dr);
        Document doc=(Document) dr.getNode();
        
        // this is horrible, but so far I can't get IDs to work after transforms
        NodeList folders=doc.getElementsByTagName("folder");
        for(int i=0;i<folders.getLength();i++) {
            Element e=(Element) folders.item(i);
            String idattr=e.getAttribute("ID");
            if(idattr!=null && !idattr.equals("")) {
                e.setIdAttribute("ID",true);
            }
        }
        
        NodeList channels=doc.getElementsByTagName("channel");
        for(int i=0;i<channels.getLength();i++) {
            Element e=(Element) channels.item(i);
            String idattr=e.getAttribute("ID");
            if(idattr!=null && !idattr.equals("")) {
                e.setIdAttribute("ID",true);
            }
        }
        return doc;
    }

    public void setTransformerHelper(ITransformerHelper transformerHelper) {
        this.transformerHelper = transformerHelper;
    }

    public void setStylesheetLocation(String stylesheetLocation) {
        this.stylesheetLocation = stylesheetLocation;
    }

}
