/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.layout.uportal2.simple.dao.hibernate;

import java.io.ByteArrayInputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.xml.serialize.OutputFormat;
import org.jasig.portal.layout.uportal2.simple.dao.IUserLayoutDao;
import org.jasig.portal.security.UserId;
import org.jasig.portal.utils.DtdResolver;
import org.jasig.portal.utils.IIncrementer;
import org.jasig.portal.utils.XmlUtils;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.w3c.dom.Document;

/**
 * A basic hibernate implementation of {@link org.jasig.portal.layout.uportal2.simple.dao.IUserLayoutDao} 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class SerializingUserLayoutDao extends HibernateDaoSupport implements IUserLayoutDao {
    IIncrementer idIncrementer;
    
    final String publicDoctype="-//JA-SIG.//DTD user layout 3.0//EN";
    final String systemDoctype="http://developer.ja-sig.org/projects/dtds/uportal/3/userlayout-SLM-SNAPSHOT.dtd";
    
    OutputFormat layoutOutputFormat;
    
    public SerializingUserLayoutDao() {
        layoutOutputFormat=new OutputFormat();
        layoutOutputFormat.setIndenting(true);
        layoutOutputFormat.setLineWidth(0);
        layoutOutputFormat.setOmitDocumentType(false);
        layoutOutputFormat.setPreserveSpace(true);
        layoutOutputFormat.setEncoding("UTF-8");
        layoutOutputFormat.setOmitComments(false);
        layoutOutputFormat.setOmitXMLDeclaration(false);
        layoutOutputFormat.setDoctype(publicDoctype, systemDoctype);
    }
    
    public Document getUserLayout(UserId user) throws Exception {
        UserLayout userLayout = this.getUserLayoutBean(user);
        if(userLayout!=null && userLayout.getSerializedLayout()!=null) {
            // parse out DOM
            DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
            docBuilderFactory.setValidating(true);
            DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
            docBuilder.setEntityResolver(new DtdResolver("userlayout-SLM-SNAPSHOT.dtd"));
            
            Document normalDom= docBuilder.parse(new ByteArrayInputStream(userLayout.getSerializedLayout().getBytes()));
            String id = normalDom.getDocumentElement().getAttribute("ID");
            
            if (id != null && !id.equals(""))
              normalDom.getDocumentElement().setIdAttribute("ID", true);

            if (logger.isInfoEnabled()) {
                logger.info("retrieved user layout : user=" + user);
            }

            return normalDom;
        } else {
            if (logger.isInfoEnabled()) {
                logger.info("retrieved user layout is null : user=" + user);
            }
            return null;
        }
        
    }

    public void setUserLayout(UserId user, Document layoutXML, boolean channelsAdded) throws Exception {
        UserLayout userLayout = this.getUserLayoutBean(user);
        if (userLayout == null) {
            userLayout = new UserLayout(user);
        }
        
        String serializedLayout = XmlUtils.serializeNode(layoutXML, layoutOutputFormat);
        userLayout.setSerializedLayout(serializedLayout);
        final HibernateTemplate hibernateTemplate = getHibernateTemplate();
        hibernateTemplate.saveOrUpdate(userLayout);
        
        if (logger.isInfoEnabled()) {
            logger.info("persisted user layout : user=" + user);
        }
    }

    public String generateNewChannelSubscribeId(UserId user) throws Exception {
        return "a" + Long.toString(idIncrementer.getNextKey());
    }

    public String generateNewFolderId(UserId user) throws Exception {
        return "a" + Long.toString(idIncrementer.getNextKey());
    }

    protected UserLayout getUserLayoutBean(UserId user) {
        UserLayout userLayout = null;
        try {
            final HibernateTemplate hibernateTemplate = getHibernateTemplate();
            userLayout = (UserLayout)hibernateTemplate.load(UserLayout.class, user);
        }
        catch (Exception e) { // I don't know why, but Spring messes up usual ObjectNotFoundException, so I can't catch it specifically
            if (logger.isInfoEnabled()) {
                logger.info("unable to find layout : user=" + user, e);
            }
            return null;
        }
        return userLayout;
    }

    /**
     * Set the incrementer to be used in determining next channel and folder ids
     * @param idIncrementer The idIncrementer to set.
     */
    //TODO: @Required
    public void setIdIncrementer(IIncrementer idIncrementer) {
        this.idIncrementer = idIncrementer;
    }
    //TODO: @Required
    public void setLayoutOutputFormat(OutputFormat layoutOutputFormat) {
        this.layoutOutputFormat = layoutOutputFormat;
    }

}
