/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.pluto.PortletWindow;
import org.apache.pluto.PortletWindowID;
import org.apache.pluto.internal.InternalPortletWindow;
import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.portlet.factory.IPortletWindowRegistryFactory;
import org.jasig.portal.portlet.registry.general.IPortletWindowRegistry;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.security.UserId;
import org.springframework.beans.factory.annotation.Required;

/**
 * A helper implementation that assumes that domain object implementations implement
 * both Portal and Pluto's interfaces.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class CastingPlutoDomainObjectHelper implements IPlutoDomainObjectHelper {
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private IPortletWindowRegistryFactory portletWindowRegistryFactory;
    private IUserController userController;
    
    public IPortletWindowRegistryFactory getPortletWindowRegistryFactory() {
        return this.portletWindowRegistryFactory;
    }
    @Required
    public void setPortletWindowRegistryFactory(IPortletWindowRegistryFactory portletWindowRegistryFactory) {
        this.portletWindowRegistryFactory = portletWindowRegistryFactory;
    }
    
    public IUserController getUserController() {
        return this.userController;
    }
    @Required
    public void setUserController(IUserController userController) {
        this.userController = userController;
    }
    
    
    public PortletWindow getPortletWindow(IPortletWindow window) {
        return (PortletWindow)window;
    }
    
    public IPortletWindow getPortletWindow(InternalPortletWindow internalPlutoWindow) {
        return this.getPortletWindow(internalPlutoWindow.getOriginalPortletWindow());
    }

    public IPortletWindow getPortletWindow(PortletWindow plutoWindow) {
        if (plutoWindow instanceof IPortletWindow) {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug("Passed Pluto PortletWindow also implements uPortal IPortletWindow, returning it directly");
            }

            return ((IPortletWindow)plutoWindow);
        }
        else if (plutoWindow instanceof InternalPortletWindow) {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug("Passed Pluto PortletWindow also implements Pluto InternalPortletWindow, calling getPortletWindow(InternalPortletWindow) on it.");
            }
            
            return this.getPortletWindow(((InternalPortletWindow)plutoWindow));
        }
        else {
            if (this.logger.isDebugEnabled()) {
                this.logger.debug("Passed Pluto PortletWindow does not implement any supported interfaces, using ID to retrieve uPortal IPortletWindow from a registry.");
            }
            
            //TODO this is hackish, the user should be resolved for the window id not just for the current user
            final UserId userId = this.userController.getUserId();
            final IPortletWindowRegistry portletWindowRegistry = this.portletWindowRegistryFactory.getPortletWindowRegistry(userId);
            
            final PortletWindowID plutoPortWinId = plutoWindow.getId();
            final PortletWindowId portWinId = this.convertPortletWindowId(plutoPortWinId);
            
            final IPortletWindow portWin = portletWindowRegistry.getPortletWindow(portWinId);
            return portWin;
        }
    }
    
    protected PortletWindowId convertPortletWindowId(final PortletWindowID plutoPortWinId) {
        final PortletWindowId portWinId;
        if (plutoPortWinId instanceof PortletWindowId) {
            portWinId = (PortletWindowId)plutoPortWinId;
        }
        else {
            portWinId = new PortletWindowId(plutoPortWinId.toString());
        }
        return portWinId;
    }
}
