/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet;

import java.util.Collection;

import org.jasig.portal.url.IUrlConstructor;

/**
 * Provides access to additional URL constructors containing
 * information for constructing the redirect URL. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 1.0$
 */
public interface IRedirectUrlConstructorsAccessor {
    /**
     * Obtain specific Url constructor if it has already
     * been added.
     * @param name
     * @return
     */
    IUrlConstructor getUrlConstructor(String name);
    
    /**
     * Add a redirect Url constructor. The constructor will be invoked
     * in constructing portlet action redirect url
     * @param name constructor name
     * @param constructor 
     */
    void addUrlConstructor(String name, IUrlConstructor constructor);
    
    /**
     * Obtain all url constructors that should be incorporated
     * into the redirect URL 
     * @return
     */
    Collection<IUrlConstructor> getUrlConstructors();
}
