/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.jasig.portal.context.common.IHttpRequestResponseAccessor;
import org.jasig.portal.url.IUrlConstructor;
import org.springframework.beans.factory.annotation.Required;

/**
 * An implementation of {@link org.jasig.portal.portlet.IRedirectUrlConstructorsAccessor}
 * that uses a request attribute to keep track of redirect url constructor map 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 1.0$
 */
public class RedirectUrlConstructorsAccessorImpl implements IRedirectUrlConstructorsAccessor {
    IHttpRequestResponseAccessor requestAccessor;
    private static final String CONSTRUCTOR_MAP_ATTRIBUTE_NAME=RedirectUrlConstructorsAccessorImpl.class.toString()+".constructorMap";

    @Required
    public void setRequestAccessor(IHttpRequestResponseAccessor requestAccessor) {
        this.requestAccessor = requestAccessor;
    }

    public void addUrlConstructor(String name, IUrlConstructor constructor) {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        Map<String,IUrlConstructor> constructorMap=(Map<String, IUrlConstructor>) req.getAttribute(CONSTRUCTOR_MAP_ATTRIBUTE_NAME);
        if(constructorMap==null) {
            constructorMap=new HashMap<String,IUrlConstructor>();
            req.setAttribute(CONSTRUCTOR_MAP_ATTRIBUTE_NAME,constructorMap);
        }
        constructorMap.put(name,constructor);
    }

    public IUrlConstructor getUrlConstructor(String name) {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        Map<String,IUrlConstructor> constructorMap=(Map<String, IUrlConstructor>) req.getAttribute(CONSTRUCTOR_MAP_ATTRIBUTE_NAME);
        if(constructorMap!=null) {
            return constructorMap.get(name);
        }
        return null;
    }

    public Collection<IUrlConstructor> getUrlConstructors() {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        Map<String,IUrlConstructor> constructorMap=(Map<String, IUrlConstructor>) req.getAttribute(CONSTRUCTOR_MAP_ATTRIBUTE_NAME);
        if(constructorMap!=null) {
            return constructorMap.values();
        }
        return Collections.EMPTY_LIST;
    }
    
    
}
