/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.window.IPortletWindow;

/**
 * Provides utility methods for resolving a locale based on a LocaleManager and
 * PortletWindow.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public abstract class BaseLocalizedBean {
    private final ILocaleManager localeManager;
    private final IPortletWindow portletWindow;
    
    public BaseLocalizedBean(ILocaleManager localeManager) {
        this(localeManager, null);
    }
    
    public BaseLocalizedBean(ILocaleManager localeManager, IPortletWindow portletWindow) {
        if (localeManager == null) {
            throw new IllegalArgumentException("localeManager may not be null");
        }
        
        this.localeManager = localeManager;
        this.portletWindow = portletWindow;
    }
    
    
    
    protected IDisplayName resolveLocalDisplayName(Map<Locale, IDisplayName> displayNames) {
        if (displayNames.size() == 0) {
            return null;
        }
        else {
            final Locale locale = this.resolveLocal(displayNames.keySet());
            final IDisplayName displayName = displayNames.get(locale);
            return displayName;
        }
    }
    
    protected IDisplayName createLocalDisplayName(String displayNameText) {
        final Locale locale = this.resolveLocal();
        final IDisplayName displayName = new DisplayNameImpl(displayNameText, locale);
        return displayName;
    }
    
    protected IDescription resolveLocalDescription(Map<Locale, IDescription> descriptions) {
        if (descriptions.size() == 0) {
            return null;
        }
        else {
            final Locale locale = this.resolveLocal(descriptions.keySet());
            IDescription description = descriptions.get(locale);
            return description;
        }
    }
    
    protected IDescription createLocalDescription(String descriptionText) {
        final Locale locale = this.resolveLocal();
        final IDescription description = new DescriptionImpl(descriptionText, locale);
        return description;
    }
    
    
    public Locale resolveLocal(Set options) {
        Locale locale = this.getPreferedLocale();
        
        if (locale == null) {
            locale = this.localeManager.resolveLocale(options);
        }
        
        if (locale == null) {
            locale = this.localeManager.resolveLocale();
        }
        
        return locale;
    }
    
    public Locale resolveLocal() {
        Locale locale = this.getPreferedLocale();
        
        if (locale == null) {
            locale = this.localeManager.resolveLocale();
        }
        
        return locale;
    }
    
    public Locale getPreferedLocale() {
        if (this.portletWindow != null) {
            return this.localeManager.resolveLocale(this.portletWindow);
        }
        
        return null;
    }
}
