/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.portlet.PortletMode;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.pluto.descriptors.common.DescriptionDD;
import org.apache.pluto.descriptors.portlet.CustomPortletModeDD;
import org.jasig.portal.portlet.deployment.PlutoDescriptionImpl;

/**
 * Implementation of ICustomPortletMode.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11911 $
 */
public class CustomPortletModeImpl implements ICustomPortletMode {
    private final CustomPortletModeDD customPortletModeDD;

    public CustomPortletModeImpl(final CustomPortletModeDD customPortletModeDD) {
        if (customPortletModeDD == null) {
            throw new IllegalArgumentException("CustomPortletModeDD can not be null");
        }

        this.customPortletModeDD = customPortletModeDD;
    }

    /**
     * @see org.jasig.portal.portlet.common.ICustomPortletMode#getPortletMode()
     */
    public PortletMode getPortletMode() {
        final String portletModeStr = this.customPortletModeDD.getPortletMode();
        return new PortletMode(portletModeStr);
    }

    /**
     * @see org.jasig.portal.portlet.common.IDescribable#getDescriptions()
     */
    @SuppressWarnings("unchecked")
    public Map<Locale, IDescription> getDescriptions() {
        final List<DescriptionDD> plutoDescriptions = this.customPortletModeDD.getDescriptions();
        final Map<Locale, IDescription> upDescriptions = new HashMap<Locale, IDescription>(plutoDescriptions.size());
        
        for (final DescriptionDD plutoDescription : plutoDescriptions) {
            final IDescription upDescription = new PlutoDescriptionImpl(plutoDescription);
            upDescriptions.put(upDescription.getLocale(), upDescription);
        }
        
        return Collections.unmodifiableMap(upDescriptions);
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof ICustomPortletMode)) {
            return false;
        }
        ICustomPortletMode rhs = (ICustomPortletMode)object;
        return new EqualsBuilder()
            .append(this.getPortletMode(), rhs.getPortletMode())
            .append(this.getDescriptions(), rhs.getDescriptions())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(-1808315941, 754042845)
            .append(this.getPortletMode())
            .append(this.getDescriptions())
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
            .append("portletMode", this.getPortletMode())
            .append("descriptions", this.getDescriptions())
            .toString();
    }
}
