/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.portlet.WindowState;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.pluto.descriptors.common.DescriptionDD;
import org.apache.pluto.descriptors.portlet.CustomWindowStateDD;
import org.jasig.portal.portlet.deployment.PlutoDescriptionImpl;

/**
 * Implementation of ICustomWindowState.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11911 $
 */
public class CustomWindowStateImpl implements ICustomWindowState {
    private final CustomWindowStateDD customWindowStateDD;

    public CustomWindowStateImpl(final CustomWindowStateDD customWindowStateDD) {
        if (customWindowStateDD == null) {
            throw new IllegalArgumentException("CustomWindowStateDD can not be null");
        }

        this.customWindowStateDD = customWindowStateDD;
    }

    /**
     * @see org.jasig.portal.portlet.common.ICustomWindowState#getWindowState()
     */
    public WindowState getWindowState() {
        final String windowStateStr = this.customWindowStateDD.getWindowState();
        return new WindowState(windowStateStr);
    }

    /**
     * @see org.jasig.portal.portlet.common.IDescribable#getDescriptions()
     */
    @SuppressWarnings("unchecked")
    public Map<Locale, IDescription> getDescriptions() {
        final List<DescriptionDD> plutoDescriptions = this.customWindowStateDD.getDescriptions();
        final Map<Locale, IDescription> upDescriptions = new HashMap<Locale, IDescription>(plutoDescriptions.size());
        
        for (final DescriptionDD plutoDescription : plutoDescriptions) {
            final IDescription upDescription = new PlutoDescriptionImpl(plutoDescription);
            upDescriptions.put(upDescription.getLocale(), upDescription);
        }
        
        return Collections.unmodifiableMap(upDescriptions);
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof ICustomWindowState)) {
            return false;
        }
        ICustomWindowState rhs = (ICustomWindowState)object;
        return new EqualsBuilder()
            .append(this.getWindowState(), rhs.getWindowState())
            .append(this.getDescriptions(), rhs.getDescriptions())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(-1808315941, 754042845)
            .append(this.getWindowState())
            .append(this.getDescriptions())
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
            .append("windowState", this.getWindowState())
            .append("descriptions", this.getDescriptions())
            .toString();
    }

}
