/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.Locale;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * Implementation of IDescription.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11911 $
 */
public class DescriptionImpl implements IDescription {

    private String descriptionText;
    private Locale locale;
    
    public DescriptionImpl() {
    }
    
    public DescriptionImpl(final String descriptionText) {
        this(descriptionText, Locale.ENGLISH); //Portlet spec says "en" is the default locale
    }
    
    public DescriptionImpl(IDescription description) {
        this(description.getDescriptionText(), description.getLocale());
    }
    
    public DescriptionImpl(final String descriptionText, Locale locale) {
        this.descriptionText = descriptionText;
        this.locale = locale;
    }
    
    public String getDescriptionText() {
        return descriptionText;
    }

    public Locale getLocale() {
        return locale;
    }
    
    public void setDescriptionText(final String descriptionText) {
        this.descriptionText = descriptionText;
    }
    
    public void setLocale(final Locale locale) {
        this.locale = locale;
    }

    
    // Additional method to satisfy Pluto's Description
    
    public String getDescription() {
        return descriptionText;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
            .append("locale", this.locale)
            .append("descriptionText", this.descriptionText)
            .toString();
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (!(obj instanceof IDescription)) {
            return false;
        }
        if (this == obj) {
            return true;
        }
        
        IDescription other = (IDescription)obj;
        return new EqualsBuilder()
            .append(this.locale, other.getLocale())
            .append(this.descriptionText, other.getDescriptionText())
            .isEquals();
    }
    
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        //Binary or on each to ensure we get an odd number
        final int classCode = this.getClass().hashCode() | 1;
        final int classNameCode = this.getClass().getName().hashCode() | 1;
        
        return new HashCodeBuilder(classCode, classNameCode)
            .append(this.locale)
            .append(this.descriptionText)
            .toHashCode();
    }
}
