/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.Locale;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * Implementation of IDisplayName.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11911 $
 */
public class DisplayNameImpl implements IDisplayName {

    private String displayNameText;
    private Locale locale;
    
    public DisplayNameImpl() {
    }
    
    public DisplayNameImpl(final String displayNameText) {
        this(displayNameText, Locale.ENGLISH); //Default to "en" per portlet spec.
    }
    
    public DisplayNameImpl(final String displayNameText, Locale locale) {
        this.displayNameText = displayNameText;
        this.locale = locale;
    }
    
    public DisplayNameImpl(IDisplayName displayName) {
        this(displayName.getDisplayNameText(), displayName.getLocale());
    }
    
    public String getDisplayNameText() {
        return displayNameText;
    }

    public Locale getLocale() {
        return locale;
    }
    
    public void setDisplayNameText(final String displayNameText) {
        this.displayNameText = displayNameText;
    }
    
    public void setLocale(final Locale locale) {
        this.locale = locale;
    }
    
    
    // Additional method to satisfy Pluto's DisplayName
    
    public String getDisplayName() {
        return displayNameText;
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
            .append("locale", this.locale)
            .append("displayNameText", this.displayNameText)
            .toString();
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (!(obj instanceof IDisplayName)) {
            return false;
        }
        if (this == obj) {
            return true;
        }
        
        IDisplayName other = (IDisplayName)obj;
        return new EqualsBuilder()
            .append(this.locale, other.getLocale())
            .append(this.displayNameText, other.getDisplayNameText())
            .isEquals();
    }
    
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        //Binary or on each to ensure we get an odd number
        final int classCode = this.getClass().hashCode() | 1;
        final int classNameCode = this.getClass().getName().hashCode() | 1;
        
        return new HashCodeBuilder(classCode, classNameCode)
            .append(this.locale)
            .append(this.displayNameText)
            .toHashCode();
    }
}
