/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.List;

/**
 * Holds portlet information such as title, short title,
 * and keywords.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11911 $
 */
public interface IPortletInfo {
    
    /**
     * Returns the title
     * @return the title
     */
    public String getTitle();

    /**
     * Returns the short title. The return value may be NULL.
     * @return the short title
     */
    public String getShortTitle();

    /**
     * Returns the keywords as a list of <code>String</code>
     * objects.
     * @return the keywords as a list of <code>String</code> objects
     */
    public List<String> getKeywords();
    
    /**
     * Returns the keywords as a comma seperated String.
     * @return the keywords as a comma seperated String.
     */
    public String getKeywordsString();

    /**
     * Sets the title
     * @param title the title
     */
    public void setTitle(String title);

    /**
     * Sets the short title.
     * @param shortTitle the short title
     */
    public void setShortTitle(String shortTitle);

    /**
     * Sets the keywords as a <code>String</code> of
     * keywords separated by commas.
     * @param keywords the keywords as a <code>String</code> value
     */
    public void setKeywordsString(String keywords);
    
    /**
     * Sets the keywords as a List of String objects.
     * @param keywords the keywords as a List of String objects, may not be null.
     */
    public void setKeywords(List<String> keywords);
}
