/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.List;

/**
 * Preference consiting of a name, list of values, and a read-only flag.
 * <br>
 * <br>
 * Properties to use when persisting: <code>name</code>, <code>valueList</code>, <code>readOnly</code>
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11911 $
 */
public interface IPreference {
    /**
     * @return The preference ID.
     */
    public long getId();
    
    /**
     * @param id The preference ID.
     */
    public void setId(long id);
    
    /**
     * Returns the name associated with this preference.
     * @return the name associated with this preference, may not be null.
     */
    public String getName();
    
    /**
     * @param name The name of the preference, may not be null.
     */
    public void setName(String name);

    /**
     * Returns a mutable List of String values. Objects added to
     * this list must be of type String.
     * @return A mutable List of String values, may be null, may be empty.
     */
    public List<String> getValues();
    
    /**
     * Sets the preference values as a list of
     * <code>java.lang.String</code> values.
     * @param values the preference values, may be null.
     */
    public void setValues(List<String> values);
    
    /**
     * Returns whether or not this preference is read only.
     * @return <code>true</code> if this preference is read-only, otherwise <code>false</code>
     */
    public boolean isReadOnly();
    
    /**
     * Sets the read only property of this preference.
     * @param readOnly <code>true</code> if preference is read only, otherwise <code>false</code>
     */
    public void setReadOnly(boolean readOnly);
}
