/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/
package org.jasig.portal.portlet.common;

import java.util.List;
import java.util.Map;

/**
 * A set of preferences and optional related preferences validator.
 * <br>
 * <br>
 * Properties to use when persisting: <code>preferences</code>, <code>preferencesValidatorClassName</code>
 * 
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11911 $
 */
public interface IPreferences {
    public long getId();
    
    public void setId(long id);
    
    /**
     * Returns the preference for the given preference name.
     * @param preferenceName The name of the preference to find. May not be null.
     * @return the preference for the given preference name, may be null if no preference found for the name.
     */
    public IPreference getPreference(String preferenceName);
    
    /**
     * Adds a new preference to the set.
     * @param preference a preference, may not be null.
     */
    public void addPreference(IPreference preference);
    
    /**
     * Adds new preferences to the set.
     * @param preferences The List of preferences, may not be null, may be empty.
     */
    public void addPreferences(List<IPreference> preferences);

    /**
     * Removes a preference from the set
     * @param preferenceName The name of the preference to remove, may not be null.
     * @return The removed preference, null if no preference existed for the name.
     */
    public IPreference removePreference(String preferenceName);
    
    /**
     * A set of {@link String} to {@link IPreference} mappings. The key must be the name of
     * the mapped preference. The returned map may not be synchronized.
     * 
     * @return A set of mutible {@link String} to {@link IPreference} mappings, may not be null, may be empty.
     * @deprecated Use {@link #getPreferencesMap()} isntead.
     */
    public List<IPreference> getPreferences();
    
    /**
     * A set {@link String} to {@link IPreference} mappings. The key must be the name of
     * the mapped preference.
     * 
     * @param preferences A set {@link String} to {@link IPreference} mappings, may not be null.
     * @throws IllegalArgumentException If the constraints on the contents of the Map are violated.
     */
    public void setPreferences(List<IPreference> preferences);

    /**
     * Sets the preferences validator class name.
     * @param preferencesValidatorClassName the preferences validator class name
     */
    public void setPreferencesValidatorClassName(String preferencesValidatorClassName);
    
    /**
     * Returns the optional preferences validator class name
     * @return the preferences validator class name, may be null.
     */
    public String getPreferencesValidatorClassName();
    
    
    public Map<String, IPreference> getPreferencesMap();
    
    public void setPreferencesMap(Map<String, IPreference> preferences);
}
