/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.pluto.descriptors.common.DescriptionDD;
import org.apache.pluto.descriptors.common.InitParamDD;
import org.jasig.portal.portlet.deployment.PlutoDescriptionImpl;

/**
 * Implementation of IParameter.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11911 $
 */
public class InitParameterImpl implements IInitParameter {
    private final InitParamDD initParamDD;
    
    public InitParameterImpl(InitParamDD initParamDD) {
        if (initParamDD == null) {
            throw new IllegalArgumentException("InitParamDD can not be null");
        }
        
        this.initParamDD = initParamDD;
    }

    /**
     * @see org.jasig.portal.portlet.common.IInitParameter#getName()
     */
    public String getName() {
        return this.initParamDD.getParamName();
    }

    /**
     * @see org.jasig.portal.portlet.common.IInitParameter#getValue()
     */
    public String getValue() {
        return this.initParamDD.getParamValue();
    }

    /**
     * @see org.jasig.portal.portlet.common.IDescribable#getDescriptions()
     */
    @SuppressWarnings("unchecked")
    public Map<Locale, IDescription> getDescriptions() {
        final List<DescriptionDD> plutoDescriptions = this.initParamDD.getDescriptions();
        final Map<Locale, IDescription> upDescriptions = new HashMap<Locale, IDescription>(plutoDescriptions.size());
        
        for (final DescriptionDD plutoDescription : plutoDescriptions) {
            final IDescription upDescription = new PlutoDescriptionImpl(plutoDescription);
            upDescriptions.put(upDescription.getLocale(), upDescription);
        }
        
        return Collections.unmodifiableMap(upDescriptions);
    }
}
