/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.Enumeration;
import java.util.HashSet;
import java.util.List;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.Set;

import org.apache.commons.collections.EnumerationUtils;
import org.apache.commons.collections.iterators.IteratorEnumeration;

/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class LayeredResourceBundle extends ResourceBundle {
    private final ResourceBundle targetBundle;
    private final ResourceBundle baseBundle;
    
    public LayeredResourceBundle(ResourceBundle targetBundle, ResourceBundle baseBundle) {
        if (targetBundle == null) {
            throw new IllegalArgumentException("targetBundle may not be null");
        }
        if (baseBundle == null) {
            throw new IllegalArgumentException("baseBundle may not be null");
        }

        this.targetBundle = targetBundle;
        this.baseBundle = baseBundle;
    }
    
    /**
     * @see java.util.ResourceBundle#getKeys()
     */
    public Enumeration getKeys() {
        final Set keySet = new HashSet();
        
        final List targetKeys = EnumerationUtils.toList(this.targetBundle.getKeys());
        keySet.addAll(targetKeys);
        
        final List baseKeys = EnumerationUtils.toList(this.baseBundle.getKeys());
        keySet.addAll(baseKeys);
        
        return new IteratorEnumeration(keySet.iterator());
    }

    /**
     * @see java.util.ResourceBundle#handleGetObject(java.lang.String)
     */
    protected Object handleGetObject(String key) {
        Object value = null;
        
        try {
            value = this.targetBundle.getObject(key);
        }
        catch (MissingResourceException mse) {
            value = null;
        }
        
        try {
            value = this.baseBundle.getObject(key);
        }
        catch (MissingResourceException mse) {
            value = null;
        }
        
        return value;
    }
}