/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * Implementation of IPreference.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11911 $
 */
public class PreferenceImpl implements IPreference, Serializable {
	/** Transient objects are marked with this value */
	public static final long UNPERSISTED_ID = -1L;
    private static final long serialVersionUID = 1L;

    private long id = UNPERSISTED_ID;
    private String name;
    private List<String> values = new ArrayList<String>();
    private boolean readOnly = false;
    
    public PreferenceImpl() {
        this((String)null);
    }
    
    public PreferenceImpl(IPreference preference) {
        if (preference == null) {
            throw new IllegalArgumentException("preference may not be null");
        }

        this.setId(preference.getId());
        this.setName(preference.getName());
        this.setValues(preference.getValues());
        this.setReadOnly(preference.isReadOnly());
    }
    
    public PreferenceImpl(String name) {
        this.name = name;
    }
    
    //********** IPreference Methods **********//
    
    public long getId() {
        return this.id;
    }
    public void setId(long id) {
        this.id = id;
    }

    
    public String getName() {
        return this.name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public List<String> getValues() {
        return this.values;
    }

    public void setValues(List<String> values) {
        if (values == null) {
            values = new ArrayList<String>();
        }

        this.values = values;
    }

    public boolean isReadOnly() {
        return this.readOnly;
    }

    public void setReadOnly(boolean readOnly) {
        this.readOnly = readOnly;        
    }
    
    
    //********** Object Methods **********//
    
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        else if (!(obj instanceof IPreference)) {
            return false;
        }
        else {
            final IPreference other = (IPreference)obj;
            return new EqualsBuilder()
                .append(this.getName(), other.getName())            
                .append(this.getValues(), other.getValues())
                .append(this.isReadOnly(), other.isReadOnly())
                .isEquals();
        }
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        //Binary or on each to ensure we get an odd number
        final int classCode = this.getClass().hashCode() | 1;
        final int classNameCode = this.getClass().getName().hashCode() | 1;
        
        return new HashCodeBuilder(classCode, classNameCode)
            .append(this.name)
            .append(this.values)
            .append(this.readOnly)
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
            .append("id", this.id)
            .append("name", this.name)
            .append("values", this.values)
            .append("readOnly", this.readOnly)
            .toString();
    }

    
}
