/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.pluto.descriptors.common.DescriptionDD;
import org.apache.pluto.descriptors.common.SecurityRoleRefDD;
import org.jasig.portal.portlet.deployment.PlutoDescriptionImpl;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * Security role reference consisting of a role name, role link,
 * and optional set of descriptions.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11911 $
 */
public class SecurityRoleReferenceImpl implements ISecurityRoleReference {
    private final SecurityRoleRefDD securityRoleRefDD;
    
    public SecurityRoleReferenceImpl(SecurityRoleRefDD securityRoleRefDD) {
        if (securityRoleRefDD == null) {
            throw new IllegalArgumentException("SecurityRoleRefDD can not be null");
        }
        
        this.securityRoleRefDD = securityRoleRefDD;
    }

    /**
     * @see org.jasig.portal.portlet.common.ISecurityRoleReference#getRoleLink()
     */
    public String getRoleLink() {
        return this.securityRoleRefDD.getRoleLink();
    }

    /**
     * @see org.jasig.portal.portlet.common.ISecurityRoleReference#getRoleName()
     */
    public String getRoleName() {
        return this.securityRoleRefDD.getRoleName();
    }

    /**
     * @see org.jasig.portal.portlet.common.IDescribable#getDescriptions()
     */
    @SuppressWarnings("unchecked")
    public Map<Locale, IDescription> getDescriptions() {
        final List<DescriptionDD> plutoDescriptions = this.securityRoleRefDD.getDescriptions();
        final Map<Locale, IDescription> upDescriptions = new HashMap<Locale, IDescription>(plutoDescriptions.size());

        for (final DescriptionDD plutoDescription : plutoDescriptions) {
            final IDescription upDescription = new PlutoDescriptionImpl(plutoDescription);
            upDescriptions.put(upDescription.getLocale(), upDescription);
        }

        return Collections.unmodifiableMap(upDescriptions);
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof ISecurityRoleReference)) {
            return false;
        }
        ISecurityRoleReference rhs = (ISecurityRoleReference)object;
        return new EqualsBuilder()
            .append(this.getDescriptions(), rhs.getDescriptions())
            .append(this.getRoleLink(), rhs.getRoleLink())
            .append(this.getRoleName(), rhs.getRoleName())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(-186011791, 1288612897)
            .append(this.getDescriptions())
            .append(this.getRoleLink())
            .append(this.getRoleName())
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
            .append("descriptions", this.getDescriptions())
            .append("roleLink", this.getRoleLink())
            .append("roleName", this.getRoleName())
            .toString();
    }
}
