/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.pluto.descriptors.common.DescriptionDD;
import org.apache.pluto.descriptors.portlet.UserAttributeDD;
import org.jasig.portal.portlet.deployment.PlutoDescriptionImpl;

/**
 * Implementation of IUserAttribute.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11911 $
 */
public class UserAttributeImpl implements IUserAttribute {
    private final UserAttributeDD userAttributeDD;

    public UserAttributeImpl(final UserAttributeDD userAttributeDD) {
        if (userAttributeDD == null) {
            throw new IllegalArgumentException("UserAttributeDD can not be null");
        }

        this.userAttributeDD = userAttributeDD;
    }


    /**
     * @see org.jasig.portal.portlet.common.IUserAttribute#getName()
     */
    public String getName() {
        return this.userAttributeDD.getName();
    }

    /**
     * @see org.jasig.portal.portlet.common.IDescribable#getDescriptions()
     */
    @SuppressWarnings("unchecked")
    public Map<Locale, IDescription> getDescriptions() {
        final List<DescriptionDD> plutoDescriptions = this.userAttributeDD.getDescriptions();
        final Map<Locale, IDescription> upDescriptions = new HashMap<Locale, IDescription>(plutoDescriptions.size());
        
        for (final DescriptionDD plutoDescription : plutoDescriptions) {
            final IDescription upDescription = new PlutoDescriptionImpl(plutoDescription);
            upDescriptions.put(upDescription.getLocale(), upDescription);
        }
        
        return Collections.unmodifiableMap(upDescriptions);
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof IUserAttribute)) {
            return false;
        }
        IUserAttribute rhs = (IUserAttribute)object;
        return new EqualsBuilder()
            .append(this.getName(), rhs.getName())
            .append(this.getDescriptions(), rhs.getDescriptions())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(-1808315941, 754042845)
            .append(this.getName())
            .append(this.getDescriptions())
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
            .append("userAttribute", this.getName())
            .append("descriptions", this.getDescriptions())
            .toString();
    }
}
