/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.common;

import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * Implementation of IUserDataConstraint.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11911 $
 */
public class UserDataConstraintImpl implements IUserDataConstraint {
    
    private final Map<Locale, IDescription> descriptions;
    private final TransportGuarantee transportGuarantee;
    
    public UserDataConstraintImpl(final TransportGuarantee transportGuarantee, final Map<Locale, IDescription> descriptions) {
        if (transportGuarantee == null) {
            throw new IllegalArgumentException("TransportGuarantee can not be null");
        }
        
        this.descriptions = descriptions;
        this.transportGuarantee = transportGuarantee;
    }

    public Map<Locale, IDescription> getDescriptions() {
        return descriptions;
    }

    public TransportGuarantee getTransportGuarantee() {
        return transportGuarantee;
    }


    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
            .append("transportGuarantee", this.transportGuarantee)
            .append("descriptions", this.descriptions)
            .toString();
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (!(obj instanceof IUserDataConstraint)) {
            return false;
        }
        if (this == obj) {
            return true;
        }
        
        IUserDataConstraint other = (IUserDataConstraint)obj;
        return new EqualsBuilder()
            .append(this.transportGuarantee, other.getTransportGuarantee())
            .append(this.descriptions, other.getDescriptions())
            .isEquals();
    }
    
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        //Binary or on each to ensure we get an odd number
        final int classCode = this.getClass().hashCode() | 1;
        final int classNameCode = this.getClass().getName().hashCode() | 1;
        
        return new HashCodeBuilder(classCode, classNameCode)
            .append(this.transportGuarantee)
            .append(this.descriptions)
            .toHashCode();
    }
}
