/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.container.services;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.pluto.PortletContainer;
import org.apache.pluto.internal.InternalActionRequest;
import org.apache.pluto.internal.InternalActionResponse;
import org.apache.pluto.internal.InternalPortletWindow;
import org.apache.pluto.internal.InternalRenderRequest;
import org.apache.pluto.internal.InternalRenderResponse;
import org.apache.pluto.spi.optional.PortletEnvironmentService;
import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.IPlutoDomainObjectHelper;
import org.jasig.portal.portlet.IRedirectUrlConstructorsAccessor;
import org.jasig.portal.portlet.rendering.PortletActionRequestImpl;
import org.jasig.portal.portlet.rendering.PortletActionResponseImpl;
import org.jasig.portal.portlet.rendering.PortletRenderRequestImpl;
import org.jasig.portal.portlet.rendering.PortletRenderResponseImpl;
import org.jasig.portal.portlet.url.IPortletUrlFactory;
import org.jasig.portal.portlet.userattributes.IMappedUserAttributeProvider;
import org.jasig.portal.security.IUser;
import org.jasig.portal.url.IUrlConstructorProviderFactoryServiceAccessor;
import org.jasig.portal.url.UrlConstructorProviderFactoryServiceAccessorImpl;
import org.springframework.beans.factory.annotation.Required;

/**
 * Default implementation of <code>PortletPreferencesService</code>
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 1.0$
 */
public class PortletEnvironmentServiceImpl implements PortletEnvironmentService {
    private IUrlConstructorProviderFactoryServiceAccessor constructorProviderFactoryServiceAccessor = new UrlConstructorProviderFactoryServiceAccessorImpl();
    private IRedirectUrlConstructorsAccessor redirectUrlConstructorAccessor;
    private IPlutoDomainObjectHelper plutoDomainObjectHelper;
    private ILocaleManager localeManager;
    private IPortletUrlFactory protletUrlFactory;
    private IMappedUserAttributeProvider mappedUserAttributeProvider;
    private IUserController userController;
    
    
    public InternalActionRequest createActionRequest(PortletContainer container, HttpServletRequest req, HttpServletResponse res, InternalPortletWindow internalWindow) {
        final IUser user = this.userController.getUser();
        return new PortletActionRequestImpl(container, internalWindow, req, this.mappedUserAttributeProvider, this.plutoDomainObjectHelper, user);
    }

    public InternalActionResponse createActionResponse(PortletContainer container, HttpServletRequest req, HttpServletResponse res, InternalPortletWindow internalWindow) {
        return new PortletActionResponseImpl(container,internalWindow,req,res,constructorProviderFactoryServiceAccessor.getUrlConstructorProviderFactoryService(req),redirectUrlConstructorAccessor);
    }

    public InternalRenderRequest createRenderRequest(PortletContainer container, HttpServletRequest req, HttpServletResponse res, InternalPortletWindow internalWindow) {
        final IUser user = this.userController.getUser();
        return new PortletRenderRequestImpl(container, internalWindow, req, this.mappedUserAttributeProvider, this.plutoDomainObjectHelper, user);
    }

    public InternalRenderResponse createRenderResponse(PortletContainer container, HttpServletRequest req, HttpServletResponse res, InternalPortletWindow internalWindow) {
        return new PortletRenderResponseImpl(container,  internalWindow, req,  res,  localeManager, protletUrlFactory, plutoDomainObjectHelper);
    }

    public IUrlConstructorProviderFactoryServiceAccessor getConstructorProviderFactoryServiceAccessor() {
        return this.constructorProviderFactoryServiceAccessor;
    }

    @Required
    public void setConstructorProviderFactoryServiceAccessor(IUrlConstructorProviderFactoryServiceAccessor constructorProviderFactoryServiceAccessor) {
        this.constructorProviderFactoryServiceAccessor = constructorProviderFactoryServiceAccessor;
    }

    public ILocaleManager getLocaleManager() {
        return this.localeManager;
    }

    @Required
    public void setLocaleManager(ILocaleManager localeManager) {
        this.localeManager = localeManager;
    }

    public IMappedUserAttributeProvider getMappedUserAttributeProvider() {
        return this.mappedUserAttributeProvider;
    }

    @Required
    public void setMappedUserAttributeProvider(IMappedUserAttributeProvider mappedUserAttributeProvider) {
        this.mappedUserAttributeProvider = mappedUserAttributeProvider;
    }

    public IPlutoDomainObjectHelper getPlutoDomainObjectHelper() {
        return this.plutoDomainObjectHelper;
    }

    @Required
    public void setPlutoDomainObjectHelper(IPlutoDomainObjectHelper plutoDomainObjectHelper) {
        this.plutoDomainObjectHelper = plutoDomainObjectHelper;
    }

    public IPortletUrlFactory getProtletUrlFactory() {
        return this.protletUrlFactory;
    }

    @Required
    public void setProtletUrlFactory(IPortletUrlFactory protletUrlFactory) {
        this.protletUrlFactory = protletUrlFactory;
    }

    public IRedirectUrlConstructorsAccessor getRedirectUrlConstructorAccessor() {
        return this.redirectUrlConstructorAccessor;
    }

    @Required
    public void setRedirectUrlConstructorAccessor(IRedirectUrlConstructorsAccessor redirectUrlConstructorAccessor) {
        this.redirectUrlConstructorAccessor = redirectUrlConstructorAccessor;
    }

    public IUserController getUserController() {
        return this.userController;
    }

    @Required
    public void setUserController(IUserController userController) {
        this.userController = userController;
    }
}
