/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.container.services;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import javax.portlet.PortletRequest;

import org.apache.pluto.PortletContainerException;
import org.apache.pluto.PortletWindow;
import org.apache.pluto.internal.InternalPortletPreference;
import org.apache.pluto.internal.impl.PortletPreferenceImpl;
import org.apache.pluto.spi.optional.PortletPreferencesService;
import org.jasig.portal.portlet.IPlutoDomainObjectHelper;
import org.jasig.portal.portlet.common.IPreference;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.common.PreferenceImpl;
import org.jasig.portal.portlet.entity.IPortletEntity;
import org.jasig.portal.portlet.factory.IPortletEntityRegistryFactory;
import org.jasig.portal.portlet.registry.general.IPortletEntityRegistry;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.springframework.beans.factory.annotation.Required;

/**
 * TODO portlet preference support needs some serious revisiting
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 1.0$
 */
public class PortletPreferencesServiceImpl implements PortletPreferencesService {
    private IPlutoDomainObjectHelper plutoDomainObjectHelper;
    private IPortletEntityRegistryFactory portletEntityRegistryFactory;
    

    public IPlutoDomainObjectHelper getPlutoDomainObjectHelper() {
        return this.plutoDomainObjectHelper;
    }
    @Required
    public void setPlutoDomainObjectHelper(IPlutoDomainObjectHelper plutoDomainObjectHelper) {
        this.plutoDomainObjectHelper = plutoDomainObjectHelper;
    }
    
    public IPortletEntityRegistryFactory getPortletEntityRegistryFactory() {
        return this.portletEntityRegistryFactory;
    }
    @Required
    public void setPortletEntityRegistryFactory(IPortletEntityRegistryFactory portletEntityRegistryFactory) {
        this.portletEntityRegistryFactory = portletEntityRegistryFactory;
    }
    
    
    public InternalPortletPreference[] getStoredPreferences(PortletWindow plutoPortletWindow, PortletRequest preq) throws PortletContainerException {
        final IPortletWindow portletWindow = this.plutoDomainObjectHelper.getPortletWindow(plutoPortletWindow);
        final IPreferences preferences = portletWindow.getPreferences();
        
        final Map<String, IPreference> preferencesMap = preferences.getPreferencesMap();
        final List<InternalPortletPreference> plutoPreferences = new ArrayList<InternalPortletPreference>(preferencesMap.size());
        
        for (final IPreference preference : preferencesMap.values()) {
            final List<String> valuesList = preference.getValues();
            final String[] values = valuesList.toArray(new String[valuesList.size()]);
            
            final InternalPortletPreference plutoPreference = new PortletPreferenceImpl(preference.getName(), values, preference.isReadOnly());
            
            plutoPreferences.add(plutoPreference);
        }
        
        return plutoPreferences.toArray(new InternalPortletPreference[plutoPreferences.size()]);
    }

    public void store(PortletWindow plutoPortletWindow, PortletRequest preq, InternalPortletPreference[] plutoPreferences) throws PortletContainerException {
        final IPortletWindow portletWindow = this.plutoDomainObjectHelper.getPortletWindow(plutoPortletWindow);
        final IPreferences preferences = portletWindow.getPreferences();
        
        boolean prefsChanged = false;
        for (final InternalPortletPreference plutoPreference : plutoPreferences) {
            final String prefName = plutoPreference.getName();
            
            final IPreference newPreference = new PreferenceImpl(prefName);
            newPreference.setReadOnly(plutoPreference.isReadOnly());
            newPreference.setValues(Arrays.asList(plutoPreference.getValues().clone()));
            
            //TODO This doesn't deal with deleting preferences!!
            final IPreference preference = preferences.getPreference(prefName);
            if (!newPreference.equals(preference)) {
                prefsChanged = true;
                preferences.addPreference(newPreference);
            }
        }
        
        if (prefsChanged) {
            final IPortletEntity portletEntity = portletWindow.getPortletEntity();
            final IPortletEntityRegistry portEntReg = this.portletEntityRegistryFactory.getPortletEntityRegistry(portletEntity.getUserId());
            portEntReg.updatePortletEntity(portletEntity);
        }
    }
}
