/* Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.container.services;

import java.util.Iterator;

import javax.portlet.PortletConfig;
import javax.portlet.PortletContext;

import org.apache.pluto.OptionalContainerServices;
import org.apache.pluto.PortletContainerException;
import org.apache.pluto.descriptors.portlet.PortletAppDD;
import org.apache.pluto.descriptors.portlet.PortletDD;
import org.apache.pluto.spi.optional.PortletRegistryListener;
import org.apache.pluto.spi.optional.PortletRegistryService;
import org.springframework.beans.factory.annotation.Required;

/**
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public class PortletRegistryServiceProvider implements PortletRegistryService {
    private OptionalContainerServices optionalContainerServices;
    
    public OptionalContainerServices getOptionalContainerServices() {
        return this.optionalContainerServices;
    }

    @Required
    public void setOptionalContainerServices(OptionalContainerServices optionalContainerServices) {
        this.optionalContainerServices = optionalContainerServices;
    }

    /**
     * @see org.apache.pluto.spi.optional.PortletRegistryService#addPortletRegistryListener(org.apache.pluto.spi.optional.PortletRegistryListener)
     */
    public void addPortletRegistryListener(PortletRegistryListener listener) {
        final PortletRegistryService portletRegistryService = this.getPortletRegistryService();
        portletRegistryService.addPortletRegistryListener(listener);
    }

    /**
     * @see org.apache.pluto.spi.optional.PortletRegistryService#getPortletApplicationDescriptor(java.lang.String)
     */
    public PortletAppDD getPortletApplicationDescriptor(String name) throws PortletContainerException {
        final PortletRegistryService portletRegistryService = this.getPortletRegistryService();
        return portletRegistryService.getPortletApplicationDescriptor(name);
    }

    /**
     * @see org.apache.pluto.spi.optional.PortletRegistryService#getPortletConfig(java.lang.String, java.lang.String)
     */
    public PortletConfig getPortletConfig(String applicationId, String portletName) throws PortletContainerException {
        final PortletRegistryService portletRegistryService = this.getPortletRegistryService();
        return portletRegistryService.getPortletConfig(applicationId, portletName);
    }

    /**
     * @see org.apache.pluto.spi.optional.PortletRegistryService#getPortletContext(java.lang.String)
     */
    public PortletContext getPortletContext(String applicationId) throws PortletContainerException {
        final PortletRegistryService portletRegistryService = this.getPortletRegistryService();
        return portletRegistryService.getPortletContext(applicationId);
    }

    /**
     * @see org.apache.pluto.spi.optional.PortletRegistryService#getPortletDescriptor(java.lang.String, java.lang.String)
     */
    public PortletDD getPortletDescriptor(String applicationId, String portletName) throws PortletContainerException {
        final PortletRegistryService portletRegistryService = this.getPortletRegistryService();
        return portletRegistryService.getPortletDescriptor(applicationId, portletName);
    }

    /**
     * @see org.apache.pluto.spi.optional.PortletRegistryService#getRegisteredPortletApplicationIds()
     */
    public Iterator getRegisteredPortletApplicationIds() {
        final PortletRegistryService portletRegistryService = this.getPortletRegistryService();
        return portletRegistryService.getRegisteredPortletApplicationIds();
    }

    /**
     * @see org.apache.pluto.spi.optional.PortletRegistryService#getRegisteredPortletApplications()
     */
    public Iterator getRegisteredPortletApplications() {
        final PortletRegistryService portletRegistryService = this.getPortletRegistryService();
        return portletRegistryService.getRegisteredPortletApplications();
    }

    /**
     * @see org.apache.pluto.spi.optional.PortletRegistryService#removePortletRegistryListener(org.apache.pluto.spi.optional.PortletRegistryListener)
     */
    public void removePortletRegistryListener(PortletRegistryListener listener) {
        final PortletRegistryService portletRegistryService = this.getPortletRegistryService();
        portletRegistryService.removePortletRegistryListener(listener);
    }

    protected PortletRegistryService getPortletRegistryService() {
        return this.optionalContainerServices.getPortletRegistryService();
    }
}
