/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.container.services;

import java.util.Collection;
import java.util.Map;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.pluto.spi.PortletURLProvider;
import org.jasig.portal.portlet.IRedirectUrlConstructorsAccessor;
import org.jasig.portal.portlet.url.IPortletUrl;
import org.jasig.portal.portlet.url.IPortletUrlConstructor;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.url.IUrlConstructor;
import org.jasig.portal.url.IUrlConstructorCommands;

/**
 * Default portlet URL provider 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 1.0$
 */
public class PortletUrlProviderImpl implements PortletURLProvider {

    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PortletUrlProviderImpl.class);

    private PortletWindowId portletWindowId;
    private IPortletUrlConstructor portletUrlConstructor;
    private IPortletUrl portletUrl;
    private IRedirectUrlConstructorsAccessor redirectUrlConstructorAccessor;

    /**
     * Default constructor
     * @param portletUrl portlet url instance
     * @param portletWindowId action portlet window id
     * @param portletWindowManager window manager associated with the request (can be <code>null</code>).
     * @param req current http request
     * used to retrieve remaining render parameters for other portlets
     */
    public PortletUrlProviderImpl(IPortletUrl portletUrl,PortletWindowId portletWindowId,IRedirectUrlConstructorsAccessor redirectUrlConstructorAccessor) {
        this.portletUrl = portletUrl;
        this.portletWindowId = portletWindowId;
        this.redirectUrlConstructorAccessor=redirectUrlConstructorAccessor;
        portletUrlConstructor = (IPortletUrlConstructor) portletUrl.getUrlConstructor(IPortletUrlConstructor.CONSTRUCTOR_NAME);
        if(portletUrlConstructor==null) {
            logger.error("unable to obtain portletUrlConstructor. constructorName="+IPortletUrlConstructor.CONSTRUCTOR_NAME);
            throw new IllegalStateException(IPortletUrlConstructor.CONSTRUCTOR_NAME+" url constructor must be configured");
        }
        
    }
    
    public void clearParameters() {
        portletUrlConstructor.clearParameters(portletWindowId);
    }

    public void setAction(boolean action) {
        portletUrlConstructor.setActionUrl(action);
        
    }

    public void setParameters(Map parameters) {
        // note: this method is only called on post-action processing redirect
        
        // run any outstanding redirect url construcors        
        Collection<IUrlConstructor> constructors=redirectUrlConstructorAccessor.getUrlConstructors();
        if(constructors!=null && !constructors.isEmpty()) {
            IUrlConstructorCommands urlConstructorCommands = (IUrlConstructorCommands) portletUrl.getUrlConstructor(IUrlConstructorCommands.CONSTRUCTOR_NAME);
            if(urlConstructorCommands==null) {
                logger.error("unable to obtain urlConstructorCommands url constructor. constructorName="+IUrlConstructorCommands.CONSTRUCTOR_NAME);
                throw new IllegalStateException(IUrlConstructorCommands.CONSTRUCTOR_NAME+" url constructor must be configured");
            }
            urlConstructorCommands.setUrlConstructors(constructors);
        }
        
        // add portlet parameters
        portletUrlConstructor.setParameters(portletWindowId,parameters);
        
        // add any outstanding render request parameters for other portlets
        /*
         * TODO We need to modify portlet window manager to keep track of "new"
         * render parameters that are coming in with a particular request, and have
         * an additional method that would allow obtaining all these params from
         * the portlet window manager before redirect.
         *
        if(portletWindowManager!=null) {
            Map renderParameterMap=portletWindowManager.getPortletRenderParameters();
            if(renderParameterMap!=null) {
                for(Iterator i=renderParameterMap.entrySet().iterator();i.hasNext();) {
                    Map.Entry pmapEntry=(Map.Entry)i.next();
                    portletUrlConstructor.setParameters((PortletWindowId)pmapEntry.getKey(),(Map)pmapEntry.getValue());
                }
            }
        }
        */
        
    }

    public void setPortletMode(PortletMode mode) {
        if ( mode != null ) {   
            portletUrlConstructor.setPortletMode(portletWindowId,mode);
        } 
    }

    public void setSecure() {
        portletUrlConstructor.setSecure(true);
    }
    
    public boolean isSecureSupported() {
        return true; //TODO should this be configurable?
    }

    public void setWindowState(WindowState state) {
        if ( state != null ) {  
            portletUrlConstructor.setWindowState(portletWindowId,state);
        } 
    }

    @Override
    public String toString() {
        return this.portletUrl.toString();
    }
}
