/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao;

import java.util.Set;

import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletApplicationDefinitionTransfer;


/**
 * Portlet application definition data access object.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletApplicationDefinitionDao {
    
    /**
     * Returns all available portlet application definitions.  This is a set of 
     * IPortletApplicationDefinition objects.
     * @return the portlet application definitions or empty collection.  
     */
    public Set<IPortletApplicationDefinitionTransfer> getPortletApplicationDefinitions();
    
    /**
     * Returns all available portlet application definitions.  This is a set of 
     * IPortletApplicationDefinition objects.
     * @param portAppDepId The PortletApplicationDeploymentId the returned definitions are based on.
     * @return the portlet application definitions or empty collection.  
     */
    public Set<IPortletApplicationDefinitionTransfer> getPortletApplicationDefinitions(PortletApplicationDeploymentId portAppDepId);
    
    /**
     * Returns a portlet application definition.
     * @param portletApplicationDefinitionId the portlet application definition object identifier, may not be null.
     * @return the portlet application definition, may be null if no application definition exists for id.
     */
    public IPortletApplicationDefinitionTransfer getPortletApplicationDefinition(PortletApplicationDefinitionId portletApplicationDefinitionId);
    
    /**
     * Persists a portlet application definition.
     * @param portletApplicationDefinition the portlet application definition, may not be null.
     */
    public void storePortletApplicationDefinition(IPortletApplicationDefinitionTransfer portletApplicationDefinition);

    /**
     * Creates a new portlet application definition and persists it.
     * @param portletApplicationDeploymentId the portlet application deployment object identifier, may not be null.
     * @return the new portlet application definition, may not be null.
     */
    public IPortletApplicationDefinitionTransfer createPortletApplicationDefinition(PortletApplicationDeploymentId portletApplicationDeploymentId);
    
    /**
     * Removes a portlet application definition.
     * @param portletApplicationDefinitionId the portlet application definition identifier, may not be null.
     */
    public void removePortletApplicationDefinition(PortletApplicationDefinitionId portletApplicationDefinitionId);

}
