/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao;

import java.util.Set;

import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.transfer.IPortletApplicationEntityTransfer;
import org.jasig.portal.security.UserId;

/**
 * Portlet application entity data access object.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletApplicationEntityDao {
    
    /**
     * Returns all available portlet application entities.  The set contains IPortletApplicationEntityTransfer
     * objects.
     * @param userId the user ID, may not be null.
     * @param portAppDefId the ID of the portlet application defintion the entities should be based on, may not be null.
     * @return the portlet application entities or empty collection
     */
    public Set<IPortletApplicationEntityTransfer> getPortletApplicationEntities(UserId userId, PortletApplicationDefinitionId portAppDefId);
    
    /**
     * Returns a portlet application entity.
     * @param userId the user ID, may not be null.
     * @param portletApplicationEntityId the portlet application entity object identifier, may not be null.
     * @return the portlet application entity may be null if no entity exists for the ids.
     */
    public IPortletApplicationEntityTransfer getPortletApplicationEntity(UserId userId, PortletApplicationEntityId portletApplicationEntityId);
    
    /**
     * Persists a portlet application entity.
     * @param userId the user ID, may not be null.
     * @param portletApplicationEntity the portlet application entity, may not be null.
     */
    public void storePortletApplicationEntity(UserId userId, IPortletApplicationEntityTransfer portletApplicationEntity);

    /**
     * Creates a new minimal portlet application entity and persists it.
     * @param userId the user ID, may not be null.
     * @param portletApplicationDefinitionId the portlet application definition identifier, may not be null.
     * @return the new portlet application entity, may not be null.
     */
    public IPortletApplicationEntityTransfer createPortletApplicationEntity(UserId userId, PortletApplicationDefinitionId portletApplicationDefinitionId);

    /**
     * Removes a portlet application entity.
     * @param userId the user ID, may not be null.
     * @param portletApplicationEntityId the portlet application entity identifier, may not be null.
     */
    public void removePortletApplicationEntity(UserId userId, PortletApplicationEntityId portletApplicationEntityId);

    /**
     * Removes from the persistent store all portlet application entities related to the 
     * specified portlet application definition
     * 
     * @param applDefId identifier for a parent portlet application, child 
     *   portlet application entities of which should be deleted. May not be null
     */
    public void removePortletApplicationEntities(PortletApplicationDefinitionId applDefId);

}
