/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao;

import java.util.Set;

import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletDefinitionTransfer;

/**
 * Portlet definition data access object.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public interface IPortletDefinitionDao {

    /**
     * Returns a portlet definition.
     * @param portletDefinitionId the portlet definition object identifier, may not be null.
     * @return the portlet definition, may be null if id doesn't exist.
     */
    public IPortletDefinitionTransfer getPortletDefinition(PortletDefinitionId portletDefinitionId);
    
    /**
     * Returns a portlet definition.
     * @param functionalName the portlet functional name, may not be null.
     * @return the portlet definition, may be null if id doesn't exist.
     */
    public IPortletDefinitionTransfer getPortletDefinitionByFunctionalName(String functionalName);

    /**
     * Persists the portlet definition.
     * @param portletDefinition the portlet definition, may not be null.
     */
    public void storePortletDefinition(IPortletDefinitionTransfer portletDefinition);
    
    /**
     * Returns the portlet definitions for a given portlet application definition.  The set contains
     * IPortletDefinitionTransfer objects.
     * @param portletApplicationDefinitionId the portlet application definition object identifier, may not be null.
     * @return the portlet definitions or empty collection
     */
    public Set<IPortletDefinitionTransfer> getPortletDefinitions(PortletApplicationDefinitionId portletApplicationDefinitionId);

    /**
     * Removes the portlet definition.
     * @param portletDefinitionId the portlet definition object identifier, may not be null.
     */
    public void removePortletDefinition(PortletDefinitionId portletDefinitionId);

    /**
     * Creates a new minimal IPortletDefinitionTransfer and persists it.
     * 
     * @param portletApplicationDefintionId The portlet app def that is the parent, may not be null.
     * @param portletDeploymentId The portlet dep that is the parent, may not be null.
     * @param functionalName The functional name (key) for the portlet definition, may not be null. 
     * @return The new portlet def transfer, may not be null.
     */
    public IPortletDefinitionTransfer createPortletDefinition(PortletApplicationDefinitionId portletApplicationDefintionId, PortletDeploymentId portletDeploymentId, String functionalName);

    /**
     * Removes from the persistent store all portlet definitions related to the 
     * specified portlet application definition
     * 
     * @param applDefId identifier for a parent portlet application, child 
     *   portlet definitions of which should be deleted. May not be null
     */
    public void removePortletDefinitions(PortletApplicationDefinitionId applDefId);
}
