/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao;

import java.util.Set;

import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.transfer.IPortletWindowTransfer;
import org.jasig.portal.portlet.window.PortletWindowId;


/**
 * Portlet window data access object.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public interface IPortletWindowDao {

    /**
     * Returns a portlet window for the specified id.
     * @param portletWindowId the portlet window object identifier, may not be null.
     * @return the portlet window, null if no window exists for the specified ID.
     */
    public IPortletWindowTransfer getPortletWindow(PortletWindowId portletWindowId);

    /**
     * Persists the portlet window. The window does not have to already exist.
     * @param portletWindow the portlet window to persist, may not be null.
     */
    public void storePortletWindow(IPortletWindowTransfer portletWindow);
    
    /**
     * Returns the portlet windows for a given portlet entity.  The set contains 
     * IPortletWindowTransfer objects.
     * @param portletEntityId the portlet entity object identifier, may not be null.
     * @return the portlet windows or empty collection
     */
    public Set<IPortletWindowTransfer> getPortletWindows(PortletEntityId portletEntityId);
    
    /**
     * Creates a new portlet window. The new window will have been persisted.
     * @param portletEntityId the portlet entity identifer to base the new window on. May not be null.
     * @return the new portlet window, may not be null.
     */
    public IPortletWindowTransfer createPortletWindow(PortletEntityId portletEntityId);
    
    /**
     * Removes the portlet window from the persistent store.
     * @param portletWindowId the portlet window object identifier, may not be null.
     */
    public void removePortletWindow(PortletWindowId portletWindowId);

    /**
     * Removes from the persistent store all portlet windows related to the 
     * specified portlet application definition
     * 
     * @param applDefId identifier for a parent portlet application definition, 
     *   child portlet windows of which should be deleted. May not be null
     */
    public void removePortletWindows(PortletApplicationDefinitionId applDefId);

    /**
     * Removes from the persistent store all portlet windows related to the 
     * specified portlet application entity
     * 
     * @param applEntityId identifier for a parent portlet application entity, child 
     *   portlet windows of which should be deleted. May not be null
     */
    public void removePortletWindows(PortletApplicationEntityId applEntityId);

    /**
     * Removes from the persistent store all portlet windows related to the 
     * specified portlet definition
     * 
     * @param portletDefId identifier for a parent portlet definition, child 
     *   portlet windows of which should be deleted. May not be null
     */
    public void removePortletWindows(PortletDefinitionId portletDefId);

    /**
     * Removes from the persistent store all portlet windows related to the 
     * specified portlet entity
     * 
     * @param portletEntityId identifier for a parent portlet entity, child 
     *   portlet windows of which should be deleted. May not be null
     */
    public void removePortletWindows(PortletEntityId portletEntityId);

}
