/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao.filesystem;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.pluto.PortletContainerException;
import org.apache.pluto.descriptors.portlet.PortletAppDD;
import org.apache.pluto.spi.optional.PortletRegistryService;
import org.jasig.portal.portlet.dao.IPortletApplicationDeploymentDao;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletApplicationDeploymentTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletApplicationDeploymentTransferImpl;
import org.springframework.beans.factory.annotation.Required;

/**
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public class PlutoPortletApplicationDeploymentDao implements IPortletApplicationDeploymentDao {
    protected final Log logger = LogFactory.getLog(this.getClass());

    private PortletRegistryService portletRegistryService;
    

    /**
     * @return the portletRegistryService
     */
    public PortletRegistryService getPortletRegistryService() {
        return this.portletRegistryService;
    }
    /**
     * @param portletRegistryService the portletRegistryService to set
     */
    @Required
    public void setPortletRegistryService(PortletRegistryService portletRegistryService) {
        this.portletRegistryService = portletRegistryService;
    }

    /**
     * @see org.jasig.portal.portlet.dao.IPortletApplicationDeploymentDao#getPortletApplicationDeployment(org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId)
     */
    public IPortletApplicationDeploymentTransfer getPortletApplicationDeployment(PortletApplicationDeploymentId portletApplicationDeploymentId) {
        final String portletAppName = portletApplicationDeploymentId.toString();

        return getPortletApplicationDeployment(portletAppName);
    }

    /**
     * @see org.jasig.portal.portlet.dao.IPortletApplicationDeploymentDao#getPortletApplicationDeployments()
     */
    @SuppressWarnings("unchecked")
    public Set<IPortletApplicationDeploymentTransfer> getPortletApplicationDeployments() {
        final Iterator<String> registeredPortletApplicationIds = this.portletRegistryService.getRegisteredPortletApplicationIds();
        
        final Set<IPortletApplicationDeploymentTransfer> deployments = new HashSet<IPortletApplicationDeploymentTransfer>();
        while (registeredPortletApplicationIds.hasNext()) {
            final String portAppId = registeredPortletApplicationIds.next();
            final IPortletApplicationDeploymentTransfer portAppDep = this.getPortletApplicationDeployment(portAppId);
            deployments.add(portAppDep);
        }
        
        return deployments;
    }

    
    /**
     * Similar to {@link #getPortletApplicationDeployment(PortletApplicationDeploymentId)} but takes a string like the underlying
     * Pluto API does.
     */
    protected IPortletApplicationDeploymentTransfer getPortletApplicationDeployment(final String portletAppName) {
        final PortletAppDD portAppDescriptor;
        try {
            portAppDescriptor = this.portletRegistryService.getPortletApplicationDescriptor(portletAppName);
        }
        catch (PortletContainerException pce) {
            this.logger.warn("Could not retrieve IPortletApplicationDeploymentTransfer for id='" + portletAppName + "'", pce);
            return null;
        }
        
        final IPortletApplicationDeploymentTransfer portAppDepTrans = new PortletApplicationDeploymentTransferImpl(portletAppName, portAppDescriptor);
        
        return portAppDepTrans;
    }
}
