/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao.filesystem;

import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.pluto.PortletContainerException;
import org.apache.pluto.descriptors.portlet.PortletAppDD;
import org.apache.pluto.descriptors.portlet.PortletDD;
import org.apache.pluto.spi.optional.PortletRegistryService;
import org.jasig.portal.portlet.dao.IPortletDeploymentDao;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletDeploymentTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletDeploymentTransferImpl;
import org.springframework.beans.factory.annotation.Required;

/**
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public class PlutoPortletDeploymentDao implements IPortletDeploymentDao {
    protected final Log logger = LogFactory.getLog(this.getClass());
    
    private PortletRegistryService portletRegistryService;
    

    /**
     * @return the portletRegistryService
     */
    public PortletRegistryService getPortletRegistryService() {
        return this.portletRegistryService;
    }
    /**
     * @param portletRegistryService the portletRegistryService to set
     */
    @Required
    public void setPortletRegistryService(PortletRegistryService portletRegistryService) {
        this.portletRegistryService = portletRegistryService;
    }
    
    
    /**
     * @see org.jasig.portal.portlet.dao.IPortletDeploymentDao#getPortletDeployment(org.jasig.portal.portlet.deployment.PortletDeploymentId)
     */
    @SuppressWarnings("unchecked")
    public IPortletDeploymentTransfer getPortletDeployment(PortletDeploymentId portDepId) {
        final PortletApplicationDeploymentId portAppDepId = portDepId.getPortletApplicationDeploymentId();
        
        final PortletDD portletDescriptor;
        try {
            portletDescriptor = this.portletRegistryService.getPortletDescriptor(portAppDepId.toString(), portDepId.toString());
        }
        catch (PortletContainerException pce) {
            this.logger.warn("Could not retrieve IPortletDeploymentTransfer for id='" + portDepId + "'", pce);
            return null;
        }
        if (portletDescriptor == null) {
            return null;
        }

        final IPortletDeploymentTransfer portDepTrans = new PortletDeploymentTransferImpl(portDepId, portletDescriptor);
        return portDepTrans;

    }

    /**
     * @see org.jasig.portal.portlet.dao.IPortletDeploymentDao#getPortletDeployments(org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId)
     */
    @SuppressWarnings("unchecked")
    public Set<IPortletDeploymentTransfer> getPortletDeployments(PortletApplicationDeploymentId portletApplicationDeploymentId) {
        final String portletAppName = portletApplicationDeploymentId.toString();
        
        final PortletAppDD portAppDescriptor;
        try {
            portAppDescriptor = this.portletRegistryService.getPortletApplicationDescriptor(portletAppName);
        }
        catch (PortletContainerException pce) {
            this.logger.warn("Could not retrieve IPortletApplicationDeploymentTransfer for id='" + portletAppName + "'", pce);
            return null;
        }
        
        if (portAppDescriptor == null) {
            return null;
        }
        
        final List<PortletDD> portletDescriptors = portAppDescriptor.getPortlets();
        final Set<IPortletDeploymentTransfer> upPortletDeployments = new HashSet<IPortletDeploymentTransfer>(portletDescriptors.size());
        for (final PortletDD portletDescriptor : portletDescriptors) {
            final String portletName = portletDescriptor.getPortletName();
            final PortletDeploymentId portletDeploymentId = new PortletDeploymentId(portletApplicationDeploymentId, portletName);
            final IPortletDeploymentTransfer portDepTrans = new PortletDeploymentTransferImpl(portletDeploymentId, portletDescriptor);
            upPortletDeployments.add(portDepTrans);
        }
        
        return Collections.unmodifiableSet(upPortletDeployments);
    }
}
