/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao.hibernate;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.jasig.portal.portlet.dao.IPortletApplicationDefinitionDao;
import org.jasig.portal.portlet.dao.IPortletApplicationEntityDao;
import org.jasig.portal.portlet.dao.IPortletDefinitionDao;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletApplicationDefinitionTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletApplicationDefinitionTransferImpl;
import org.jasig.portal.utils.IIncrementer;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

/**
 * DAO implmentation of the application definition using hibernate.
 * 
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11955 $
 */
public class HibernatePortletApplicationDefinitionDao extends HibernateDaoSupport implements IPortletApplicationDefinitionDao {
    private static final String QUERY_FIND_APPL_DEF_BY_DEF_ID = "FIND_APPL_DEF_BY_DEF_ID";
    private static final String QUERY_PARAM_PORT_APPL_DEF_ID = "portAppDefId";
    
    private static final String QUERY_FIND_APPL_DEF_BY_DEP_ID = "FIND_APPL_DEF_BY_DEP_ID";
    private static final String QUERY_PARAM_PORT_APPL_DEP_ID = "portAppDepId";
    
    private static final String QUERY_ALL_PORT_APPL_DEF = "ALL_PORT_APPL_DEF";
    

    private IIncrementer portletApplicationDefinitionIdIncrementer;
    private IPortletDefinitionDao portletDefinitionDao;
    private IPortletApplicationEntityDao portletApplicationEntityDao;
    
    public IIncrementer getPortletApplicationDefinitionIdIncrementer() {
        return portletApplicationDefinitionIdIncrementer;
    }

    public void setPortletApplicationDefinitionIdIncrementer(IIncrementer incrementer) {
        this.portletApplicationDefinitionIdIncrementer = incrementer;
    }
    
	public IPortletApplicationEntityDao getPortletApplicationEntityDao() {
        return portletApplicationEntityDao;
    }

    public void setPortletApplicationEntityDao(
            IPortletApplicationEntityDao portletApplicationEntityDao) {
        this.portletApplicationEntityDao = portletApplicationEntityDao;
    }

    public IPortletDefinitionDao getPortletDefinitionDao() {
        return portletDefinitionDao;
    }

    public void setPortletDefinitionDao(IPortletDefinitionDao portletDefinitionDao) {
        this.portletDefinitionDao = portletDefinitionDao;
    }

    public IPortletApplicationDefinitionTransfer getPortletApplicationDefinition(PortletApplicationDefinitionId portletApplicationDefinitionId) {
		if (portletApplicationDefinitionId == null) {
			throw new IllegalArgumentException("portletApplicationDefinitionId may not be null");
		}
        
        final HibernateTemplate template = this.getHibernateTemplate();
        final List portAppDefList = template.findByNamedQueryAndNamedParam(
            QUERY_FIND_APPL_DEF_BY_DEF_ID,
            QUERY_PARAM_PORT_APPL_DEF_ID,
            portletApplicationDefinitionId);
        
        final IPortletApplicationDefinitionTransfer portAppDefTrans = (IPortletApplicationDefinitionTransfer)DataAccessUtils.uniqueResult(portAppDefList);
        if (logger.isDebugEnabled()) {
            logger.debug("Retrieved: " + portAppDefTrans);
        }
        return portAppDefTrans;
	}

	@SuppressWarnings("unchecked")
    public Set<IPortletApplicationDefinitionTransfer> getPortletApplicationDefinitions() {
        final HibernateTemplate template = this.getHibernateTemplate();
        final List<IPortletApplicationDefinitionTransfer> portAppDefs = template.findByNamedQuery(QUERY_ALL_PORT_APPL_DEF);
		
		final Set<IPortletApplicationDefinitionTransfer> portAppDefsTrans = new HashSet<IPortletApplicationDefinitionTransfer>(portAppDefs);
		
        if (logger.isDebugEnabled()) {
            for (final Iterator itr = portAppDefsTrans.iterator(); itr.hasNext();) {
                logger.debug("Retrieved: " + itr.next());
            }
		}

		return portAppDefsTrans;
	}
    
    @SuppressWarnings("unchecked")
    public Set<IPortletApplicationDefinitionTransfer> getPortletApplicationDefinitions(PortletApplicationDeploymentId portAppDepId) {
        final HibernateTemplate template = this.getHibernateTemplate();
        final List<IPortletApplicationDefinitionTransfer> portAppDefs = template.findByNamedQueryAndNamedParam(
            QUERY_FIND_APPL_DEF_BY_DEP_ID,
            QUERY_PARAM_PORT_APPL_DEP_ID,
            portAppDepId);
        
        final Set<IPortletApplicationDefinitionTransfer> portAppDefsTrans = new HashSet<IPortletApplicationDefinitionTransfer>(portAppDefs);
        
        if (logger.isDebugEnabled()) {
            for (final Iterator itr = portAppDefsTrans.iterator(); itr.hasNext();) {
                logger.debug("Retrieved: " + itr.next());
            }
        }

        return portAppDefsTrans;
    }

    public IPortletApplicationDefinitionTransfer createPortletApplicationDefinition(PortletApplicationDeploymentId portletApplicationDeploymentId) {
        if (portletApplicationDeploymentId == null) {
            throw new IllegalArgumentException("portletApplicationDeploymentId may not be null");
        }
        
        final IPortletApplicationDefinitionTransfer portAppDefTrans = new PortletApplicationDefinitionTransferImpl();
        final long nextKey = this.portletApplicationDefinitionIdIncrementer.getNextKey();
        final PortletApplicationDefinitionId portAppDefId = new PortletApplicationDefinitionId(Long.toString(nextKey));
        
        portAppDefTrans.setPortletApplicationDefinitionId(portAppDefId);
        portAppDefTrans.setPortletApplicationDeploymentId(portletApplicationDeploymentId);
        
        this.storePortletApplicationDefinition(portAppDefTrans);
        
        return portAppDefTrans;
	}

	public void removePortletApplicationDefinition(PortletApplicationDefinitionId portletApplicationDefinitionId) {
        if (portletApplicationDefinitionId == null) {
            throw new IllegalArgumentException("portletApplicationDefinitionId may not be null");
        }
        
        final IPortletApplicationDefinitionTransfer portAppDefTrans = getPortletApplicationDefinition(portletApplicationDefinitionId);

        if (portAppDefTrans != null) {
            
            portletApplicationEntityDao.removePortletApplicationEntities(portletApplicationDefinitionId);
            portletDefinitionDao.removePortletDefinitions(portletApplicationDefinitionId);
            
            final HibernateTemplate template = this.getHibernateTemplate();
            template.delete(portAppDefTrans);
            
            if (logger.isDebugEnabled()) {
                logger.debug("Removed: " + portletApplicationDefinitionId);
            }
        }
	}

	public void storePortletApplicationDefinition(IPortletApplicationDefinitionTransfer portletApplicationDefinition) {
        if (portletApplicationDefinition == null) {
            throw new IllegalArgumentException("portletApplicationDefinition may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        template.saveOrUpdate(portletApplicationDefinition);
        
        if (logger.isDebugEnabled()) {
            logger.debug("Stored: " + portletApplicationDefinition);
        }
	}
}
