/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao.hibernate;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.hibernate.Query;
import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.dao.IPortletApplicationEntityDao;
import org.jasig.portal.portlet.dao.IPortletEntityDao;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.transfer.IPortletApplicationEntityTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletApplicationEntityTransferImpl;
import org.jasig.portal.security.UserId;
import org.jasig.portal.utils.IIncrementer;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

/**
 * Dao implementation of application entity dao using hibernate.
 * 
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11955 $
 */
public class HibernatePortletApplicationEntityDao extends HibernateDaoSupport implements IPortletApplicationEntityDao {
    private static final String QUERY_ALL_PORT_APP_ENT_BY_APP_DEF_ID_USER_ID = "ALL_PORT_APP_ENT_BY_APP_DEF_ID_USER_ID";
    private static final String QUERY_PARAM_USER_ID = "userId";
    private static final String QUERY_PARAM_APP_DEF_ID = "portAppDefId";

    private static final String QUERY_FIND_PORT_APP_ENT_BY_APP_ENT_ID_USER_ID = "FIND_PORT_APP_ENT_BY_APP_ENT_ID_USER_ID";
    private static final String QUERY_PARAM_APP_ENT_ID = "portAppEntId";
    
    private static final String QUERY_DELETE_PORT_APP_ENTS_BY_APP_DEF_ID = "DELETE_PORT_APP_ENTS_BY_APP_DEF_ID";

    
    private IIncrementer portletApplicationEntityIdIncrementer;
    private IPortletEntityDao portletEntityDao;

    public IIncrementer getPortletApplicationEntityIdIncrementer() {
        return portletApplicationEntityIdIncrementer;
    }

    public void setPortletApplicationEntityIdIncrementer(IIncrementer incrementer) {
        this.portletApplicationEntityIdIncrementer = incrementer;
    }

	public IPortletEntityDao getPortletEntityDao() {
        return portletEntityDao;
    }

    public void setPortletEntityDao(IPortletEntityDao portletEntityDao) {
        this.portletEntityDao = portletEntityDao;
    }

    @SuppressWarnings("unchecked")
    public Set<IPortletApplicationEntityTransfer> getPortletApplicationEntities(UserId userId, PortletApplicationDefinitionId portAppDefId) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        final List<IPortletApplicationEntityTransfer> portAppEnts = template.findByNamedQueryAndNamedParam(
            QUERY_ALL_PORT_APP_ENT_BY_APP_DEF_ID_USER_ID,
            new String[] {QUERY_PARAM_APP_DEF_ID, QUERY_PARAM_USER_ID},
            new ObjectId[] {portAppDefId, userId});
    
        final Set<IPortletApplicationEntityTransfer> portAppEntsTrans = new HashSet<IPortletApplicationEntityTransfer>(portAppEnts);
        
        if (logger.isDebugEnabled()) {
            for (final Iterator itr = portAppEntsTrans.iterator(); itr.hasNext();) {
                logger.debug("Retrieved: " + itr.next());
            }
        }

        return portAppEntsTrans;
	}

	public IPortletApplicationEntityTransfer getPortletApplicationEntity(UserId userId, PortletApplicationEntityId portletApplicationEntityId) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        if (portletApplicationEntityId == null) {
            throw new IllegalArgumentException("portletApplicationEntityId may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        final List portAppEnts = template.findByNamedQueryAndNamedParam(
            QUERY_FIND_PORT_APP_ENT_BY_APP_ENT_ID_USER_ID,
            new String[] {QUERY_PARAM_APP_ENT_ID, QUERY_PARAM_USER_ID},
            new ObjectId[] {portletApplicationEntityId, userId});
    
        final IPortletApplicationEntityTransfer portAppEntTrans = (IPortletApplicationEntityTransfer)DataAccessUtils.uniqueResult(portAppEnts);
        if (logger.isDebugEnabled()) {
            logger.debug("Retrieved: " + portAppEntTrans);
        }
        return portAppEntTrans;
	}

	public void storePortletApplicationEntity(UserId userId, IPortletApplicationEntityTransfer portletApplicationEntity) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        if (portletApplicationEntity == null) {
            throw new IllegalArgumentException("portletApplicationEntity may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        portletApplicationEntity.setUserId(userId);
        template.saveOrUpdate(portletApplicationEntity);
        
        if (logger.isDebugEnabled()) {
            logger.debug("Stored: " + portletApplicationEntity);
        }
	}

	public IPortletApplicationEntityTransfer createPortletApplicationEntity(UserId userId, PortletApplicationDefinitionId portletApplicationDefinitionId) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        if (portletApplicationDefinitionId == null) {
            throw new IllegalArgumentException("portletApplicationDefinitionId may not be null");
        }
        
        final IPortletApplicationEntityTransfer portAppEntTrans = new PortletApplicationEntityTransferImpl();
        final long nextKey = this.portletApplicationEntityIdIncrementer.getNextKey();
        final PortletApplicationEntityId portAppEntId = new PortletApplicationEntityId(Long.toString(nextKey));

        portAppEntTrans.setPortletApplicationEntityId(portAppEntId);
        portAppEntTrans.setPortletApplicationDefinitionId(portletApplicationDefinitionId);
        
        this.storePortletApplicationEntity(userId, portAppEntTrans);
        
        return portAppEntTrans;
	}

	public void removePortletApplicationEntity(UserId userId, PortletApplicationEntityId portletApplicationEntityId) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        if (portletApplicationEntityId == null) {
            throw new IllegalArgumentException("portletApplicationEntityId may not be null");
        }
        
        final IPortletApplicationEntityTransfer portAppEntTrans = getPortletApplicationEntity(userId, portletApplicationEntityId);
        
        if (portAppEntTrans != null) {
            
            portletEntityDao.removePortletEntities(portAppEntTrans.getPortletApplicationEntityId());
            
            final HibernateTemplate template = this.getHibernateTemplate();
            template.delete(portAppEntTrans);
            
            if (logger.isDebugEnabled()) {
                logger.debug("Removed: " + portAppEntTrans);
            }
        }
	}

    /**
     * Implements deletion using a HQL DML-style query. IPortletApplicationEntities
     * are per-user objects, so loading them up one-by-one for deletion is not desirable
     * in a large installation.
     * 
     */
    public void removePortletApplicationEntities(PortletApplicationDefinitionId applDefId) {
        
        if ( applDefId == null ) {
            throw new IllegalArgumentException("applDefId may not be null");
        }
        
        portletEntityDao.removePortletEntities(applDefId); // handles portlet windows too
        
        final HibernateTemplate template = this.getHibernateTemplate();
        
        Query deleteQuery = template.getSessionFactory()
            .getCurrentSession()
            .getNamedQuery(QUERY_DELETE_PORT_APP_ENTS_BY_APP_DEF_ID);
        String queryString = deleteQuery.getQueryString();
        int result = template.bulkUpdate(queryString, applDefId);
        
        if ( logger.isDebugEnabled() ) {
            logger.debug("Removed portlet application entities for portlet application definition [appl def id: " 
                    + applDefId + "][instances deleted: " + result + "]");
        }
        
    }
}
