/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao.hibernate;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.hibernate.Query;
import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.dao.IPortletEntityDao;
import org.jasig.portal.portlet.dao.IPortletWindowDao;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.transfer.IPortletEntityTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletEntityTransferImpl;
import org.jasig.portal.security.UserId;
import org.jasig.portal.utils.IIncrementer;
import org.springframework.dao.support.DataAccessUtils;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

/**
 * Dao implementation of portlet entity dao using hibernate.
 * 
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11955 $
 */
public class HibernatePortletEntityDao extends HibernateDaoSupport implements IPortletEntityDao {
    private static final String QUERY_FIND_PORT_ENT_BY_ENT_ID_USER_ID = "FIND_PORT_ENT_BY_ENT_ID_USER_ID";
    private static final String QUERY_PARAM_ENT_ID = "portEntId";
    private static final String QUERY_PARAM_USER_ID = "userId";
    
    private static final String QUERY_FIND_PORT_ENT_BY_APP_ENT_ID_USER_ID = "FIND_PORT_ENT_BY_APP_ENT_ID_USER_ID";
    private static final String QUERY_PARAM_APP_ENT_ID = "portAppEntId";
    
    private static final String QUERY_FIND_PORT_ENT_BY_APP_ENT_ID_DEF_ID_USER_ID = "FIND_PORT_ENT_BY_APP_ENT_ID_DEF_ID_USER_ID";
    private static final String QUERY_PARAM_DEF_ID = "portDefId";
    
    private static final String QUERY_FIND_PORT_ENT_BY_USER_ID = "FIND_PORT_ENT_BY_USER_ID";
    
    private static final String QUERY_DELETE_PORT_ENTS_BY_APP_DEF_ID = "DELETE_PORT_ENTS_BY_APP_DEF_ID";
    
    private static final String QUERY_DELETE_PORT_ENTS_BY_APP_ENT_ID = "DELETE_PORT_ENTS_BY_APP_ENT_ID";
    
    private static final String QUERY_DELETE_PORT_ENTS_BY_DEF_ID = "DELETE_PORT_ENTS_BY_DEF_ID";
    
    private static final String QUERY_DELETE_PORT_ENT_PREFS_BY_APP_DEF_ID = "DELETE_PORT_ENT_PREFS_BY_APP_DEF_ID";
    
    private static final String QUERY_DELETE_PORT_ENT_PREF_VALS_BY_APP_DEF_ID = "DELETE_PORT_ENT_PREF_VALS_BY_APP_DEF_ID";
    
    private static final String QUERY_DELETE_PORT_ENT_PREFS_BY_APP_ENT_ID = "DELETE_PORT_ENT_PREFS_BY_APP_ENT_ID";
    
    private static final String QUERY_DELETE_PORT_ENT_PREF_VALS_BY_APP_ENT_ID = "DELETE_PORT_ENT_PREF_VALS_BY_APP_ENT_ID";
    
    private static final String QUERY_DELETE_PORT_ENT_PREFS_BY_DEF_ID = "DELETE_PORT_ENT_PREFS_BY_DEF_ID";
    
    private static final String QUERY_DELETE_PORT_ENT_PREF_VALS_BY_DEF_ID = "DELETE_PORT_ENT_PREF_VALS_BY_DEF_ID";
    
    private static final long UNPERSISTED_PREF_ID = -1;
    private static final long UNPERSISTED_PREF_VAL_ID = -1;
    
    private IIncrementer portletEntityIdIncrementer;
    private IIncrementer portletPreferenceIdIncrementer;
    private IIncrementer portletPreferenceValueIdIncrementer;
    private long unpersistedPrefId = UNPERSISTED_PREF_ID;
    private long unpersistedPrefValueId = UNPERSISTED_PREF_VAL_ID;
    private IPortletWindowDao portletWindowDao;
    
    public IIncrementer getPortletEntityIdIncrementer() {
        return portletEntityIdIncrementer;
    }

    public void setPortletEntityIdIncrementer(IIncrementer incrementer) {
        this.portletEntityIdIncrementer = incrementer;
    }
    
    public IIncrementer getPortletPreferenceIdIncrementer() {
        return portletPreferenceIdIncrementer;
    }

    public void setPortletPreferenceIdIncrementer(
            IIncrementer portletPreferenceIdIncrementer) {
        this.portletPreferenceIdIncrementer = portletPreferenceIdIncrementer;
    }
    
    /**
     * Unpersisted IPreferences will have this value assigned
     * as their current Id.
     * 
     * @return the value used to indicate unpersisted IPreferences
     */
    public long getUnpersistedPrefId() {
        return unpersistedPrefId;
    }

    /**
     * Assign the Id value used to indicate an unpersisted IPreference
     * 
     * @param unpersistedPrefId
     */
    public void setUnpersistedPrefId(long unpersistedPrefId) {
        this.unpersistedPrefId = unpersistedPrefId;
    }

    public IIncrementer getPortletPreferenceValueIdIncrementer() {
        return portletPreferenceValueIdIncrementer;
    }

    public void setPortletPreferenceValueIdIncrementer(
            IIncrementer portletPreferenceValueIdIncrementer) {
        this.portletPreferenceValueIdIncrementer = portletPreferenceValueIdIncrementer;
    }

    public long getUnpersistedPrefValueId() {
        return unpersistedPrefValueId;
    }

    public void setUnpersistedPrefValueId(long unpersistedPrefValueId) {
        this.unpersistedPrefValueId = unpersistedPrefValueId;
    }

	public IPortletWindowDao getPortletWindowDao() {
        return portletWindowDao;
    }

    public void setPortletWindowDao(IPortletWindowDao portletWindowDao) {
        this.portletWindowDao = portletWindowDao;
    }

    public IPortletEntityTransfer getPortletEntity(UserId userId, PortletEntityId portletEntityId) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        if (portletEntityId == null) {
            throw new IllegalArgumentException("portletEntityId may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        final List portEnts = template.findByNamedQueryAndNamedParam(
            QUERY_FIND_PORT_ENT_BY_ENT_ID_USER_ID,
            new String[] {QUERY_PARAM_ENT_ID, QUERY_PARAM_USER_ID},
            new ObjectId[] {portletEntityId, userId});
    
        final IPortletEntityTransfer portEntTrans = (IPortletEntityTransfer)DataAccessUtils.uniqueResult(portEnts);
        if (logger.isDebugEnabled()) {
            logger.debug("Retrieved (userId=" + userId + ", portEntId=" + portletEntityId + "): " + portEntTrans);
        }
        return portEntTrans;
	}

	public void removePortletEntity(UserId userId, PortletEntityId portletEntityId) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        if (portletEntityId == null) {
            throw new IllegalArgumentException("portletEntityId may not be null");
        }
        
        final IPortletEntityTransfer portEntTrans = getPortletEntity(userId, portletEntityId);
        
        if (portEntTrans != null) {
            
            portletWindowDao.removePortletWindows(portletEntityId);
            
            final HibernateTemplate template = this.getHibernateTemplate();
            template.delete(portEntTrans); // should take care of deleting entity prefs
            
            if (logger.isDebugEnabled()) {
                logger.debug("Removed (userId=" + userId + ", portEntId=" + portletEntityId + "): " + portEntTrans);
            }
        }
	}
    
    /**
     * Implementation includes deletion of entity-scoped preferences.
     * Portlet definition-scoped preferences are not affected.
     */
    public void removePortletEntities(PortletApplicationDefinitionId applDefId) {
        
        removePortletEntities(QUERY_DELETE_PORT_ENTS_BY_APP_DEF_ID, 
                QUERY_DELETE_PORT_ENT_PREFS_BY_APP_DEF_ID,
                QUERY_DELETE_PORT_ENT_PREF_VALS_BY_APP_DEF_ID,
                applDefId);
        
    }

    /**
     * Implementation includes deletion of entity-scoped preferences.
     * Portlet definition-scoped preferences are not affected.
     */
    public void removePortletEntities(PortletApplicationEntityId applEntityId) {
        
        removePortletEntities(QUERY_DELETE_PORT_ENTS_BY_APP_ENT_ID, 
                QUERY_DELETE_PORT_ENT_PREFS_BY_APP_ENT_ID,
                QUERY_DELETE_PORT_ENT_PREF_VALS_BY_APP_ENT_ID,
                applEntityId);
        
    }

    /**
     * Implementation includes deletion of entity-scoped preferences.
     * Portlet definition-scoped preferences are not affected.
     */
    public void removePortletEntities(PortletDefinitionId portletDefId) {
        
        removePortletEntities(QUERY_DELETE_PORT_ENTS_BY_DEF_ID, 
                QUERY_DELETE_PORT_ENT_PREFS_BY_DEF_ID,
                QUERY_DELETE_PORT_ENT_PREF_VALS_BY_DEF_ID,
                portletDefId);
        
    }
    
    /**
     * Implements cascaded deletion of portlet entity records in bulk.
     * 
     * @param queryName
     * @param fk
     */
    private void removePortletEntities(String portEntQueryName, 
            String prefNamesQueryName, String prefValsQueryName, ObjectId fk) {
        
        if ( fk instanceof PortletApplicationDefinitionId ) {
            portletWindowDao.removePortletWindows((PortletApplicationDefinitionId)fk);
        } else if ( fk instanceof PortletApplicationEntityId ) {
            portletWindowDao.removePortletWindows((PortletApplicationEntityId)fk);
        } else if ( fk instanceof PortletDefinitionId ) {
            portletWindowDao.removePortletWindows((PortletDefinitionId)fk);
        } else {
            throw new IllegalArgumentException("foerign key [" + fk + "] not of recognized type");
        }
        
//        removePreferences(prefNamesQueryName, prefValsQueryName, fk);
        bulkUpdateByObjectId(portEntQueryName, fk);
             
    }
    
    /**
     * Implements 'algorithm' for deleting preference name and value records associated with a 
     * DO represented by its ObjectId. Specifically, it deletes preference values, then name
     * records using the two query references provided.
     * 
     * @param prefValsQueryName name of a query which will be used to delete IPreference values
     * @param prefsQueryName name of a query which will be used to delete IPreference value holders
     * @param fk Id of a DO, child preferences of which are to be deleted.
     */
    private void removePreferences( final String prefsQueryName, final String prefValsQueryName, final ObjectId fk) {
        
        bulkUpdateByObjectId(prefValsQueryName, fk);
        bulkUpdateByObjectId(prefsQueryName, fk);
          
    }
    
    /**
     * Performs a bulk update operation using a named query and an
     * ObjectId parameter.
     * 
     * @param queryName
     * @param fk
     */
    private void bulkUpdateByObjectId(String queryName, ObjectId id) {
        
        String queryString = getNamedQueryString(queryName);
        final HibernateTemplate template = this.getHibernateTemplate();
        int result = template.bulkUpdate(queryString, id);
        
        if ( logger.isDebugEnabled() ) {
            logger.debug("Performed bulk update by object key [object key type: " 
                    + id.getClass() + "][foreign key value: " 
                    + id + "][instances deleted: " + result + 
                    "][query: " + queryString + "]");
        }
    }
    
    private Query getNamedQuery(String queryName) {
        
        final HibernateTemplate template = this.getHibernateTemplate();
        Query query = template.getSessionFactory().getCurrentSession().getNamedQuery(queryName);
        return query;
    }
    
    private String getNamedQueryString(String queryName) {
        return getNamedQuery(queryName).getQueryString();
    }
    
	@SuppressWarnings("unchecked")
    public Set<IPortletEntityTransfer> getPortletEntities(UserId userId, PortletApplicationEntityId portAppEntId, PortletDefinitionId portDefId) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        if (portAppEntId == null) {
            throw new IllegalArgumentException("portAppEntId may not be null");
        }
        if (portDefId == null) {
            throw new IllegalArgumentException("portDefId may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        final List<IPortletEntityTransfer> portEnts = template.findByNamedQueryAndNamedParam(
            QUERY_FIND_PORT_ENT_BY_APP_ENT_ID_DEF_ID_USER_ID,
            new String[] {QUERY_PARAM_DEF_ID, QUERY_PARAM_APP_ENT_ID, QUERY_PARAM_USER_ID},
            new ObjectId[] {portDefId, portAppEntId, userId});
        final Set<IPortletEntityTransfer> portEntsTrans = new HashSet<IPortletEntityTransfer>(portEnts);
        if (logger.isDebugEnabled()) {
            for (final Iterator itr = portEntsTrans.iterator(); itr.hasNext();) {
                logger.debug("Retrieved: " + itr.next());
            }
        }
        return portEntsTrans;
    }

    @SuppressWarnings("unchecked")
    public Set<IPortletEntityTransfer> getPortletEntities(UserId userId, PortletApplicationEntityId portletApplicationEntityId) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        if (portletApplicationEntityId == null) {
            throw new IllegalArgumentException("portletApplicationEntityId may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        final List<IPortletEntityTransfer> portEnts = template.findByNamedQueryAndNamedParam(
            QUERY_FIND_PORT_ENT_BY_APP_ENT_ID_USER_ID,
            new String[] {QUERY_PARAM_APP_ENT_ID, QUERY_PARAM_USER_ID},
            new ObjectId[] {portletApplicationEntityId, userId});
        final Set<IPortletEntityTransfer> portEntsTrans = new HashSet<IPortletEntityTransfer>(portEnts);
        if (logger.isDebugEnabled()) {
            for (final Iterator itr = portEntsTrans.iterator(); itr.hasNext();) {
                logger.debug("Retrieved: " + itr.next());
            }
        }
        return portEntsTrans;
	}

    @SuppressWarnings("unchecked")
    public Set<IPortletEntityTransfer> getPortletEntities(UserId userId) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        final List<IPortletEntityTransfer> portEnts = template.findByNamedQueryAndNamedParam(
            QUERY_FIND_PORT_ENT_BY_USER_ID,
            new String[] {QUERY_PARAM_USER_ID},
            new ObjectId[] {userId});
        final Set<IPortletEntityTransfer> portEntsTrans = new HashSet<IPortletEntityTransfer>(portEnts);
        if (logger.isDebugEnabled()) {
            for (final Iterator itr = portEntsTrans.iterator(); itr.hasNext();) {
                logger.debug("Retrieved: " + itr.next());
            }
        }
        return portEntsTrans;
    }

    public IPortletEntityTransfer createPortletEntity(
                                                   UserId userId,
                                                   PortletApplicationEntityId portletApplicationEntityId,
                                                   PortletDefinitionId portletDefinitionId) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        if (portletApplicationEntityId == null) {
            throw new IllegalArgumentException("portletApplicationEntityId may not be null");
        }
        if (portletDefinitionId == null) {
            throw new IllegalArgumentException("portletDefinitionId may not be null");
        }
        
        final IPortletEntityTransfer portEntTrans = new PortletEntityTransferImpl();
        final long nextKey = this.portletEntityIdIncrementer.getNextKey();
        final PortletEntityId portEntId = new PortletEntityId(Long.toString(nextKey));
        
        portEntTrans.setPortletEntityId(portEntId);
        portEntTrans.setPortletApplicationEntityId(portletApplicationEntityId);
        portEntTrans.setPortletDefinitionId(portletDefinitionId);
        
        this.storePortletEntity(userId, portEntTrans);

        return portEntTrans;
	}

	public void storePortletEntity(UserId userId, IPortletEntityTransfer portletEntity) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        if (portletEntity == null) {
            throw new IllegalArgumentException("portletEntity may not be null");
        }
        
        final HibernateTemplate template = this.getHibernateTemplate();
        portletEntity.setUserId(userId);

        final IPreferences preferences = portletEntity.getPreferences();
        if (preferences != null) {
            template.saveOrUpdate(preferences);
        }
        
        template.saveOrUpdate(portletEntity);
        
        // explicitly delete prefs, otherwise will end up with orphans in the db
//        template.deleteAll(template.findByNamedQuery(QUERY_FIND_ORPHANED_PORT_PREFS));
        
        if (logger.isDebugEnabled()) {
            logger.debug("Stored: " + portletEntity);
        }
	}
}
