/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao.hibernate.types;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import org.hibernate.HibernateException;
import org.jasig.portal.core.hibernate.BaseObjectIdUserType;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;


/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class PortletDeploymentIdUserType extends BaseObjectIdUserType<PortletDeploymentId> {
    private static final int[] TYPES = new int [] { Types.VARCHAR, Types.VARCHAR };
    
    /**
     * @see org.hibernate.usertype.UserType#returnedClass()
     */
    public Class returnedClass() {
        return PortletDeploymentId.class;
    }

    /**
     * @see org.hibernate.usertype.UserType#sqlTypes()
     */
    public int[] sqlTypes() {
        return TYPES;
    }


    /**
     * @see org.jasig.portal.core.hibernate.BaseObjectIdUserType#notNullSet(java.sql.PreparedStatement, org.jasig.portal.core.ObjectId, int)
     */
    @Override
    protected void notNullSet(PreparedStatement st, PortletDeploymentId value, int index) throws HibernateException, SQLException {
        final PortletApplicationDeploymentId portAppDepId = value.getPortletApplicationDeploymentId();
        st.setString(index, portAppDepId.toString());
        st.setString(index + 1, value.toString());
    }

    /**
     * @see org.hibernate.usertype.UserType#nullSafeGet(java.sql.ResultSet, java.lang.String[], java.lang.Object)
     */
    public Object nullSafeGet(ResultSet rs, String[] names, Object owner) throws HibernateException, SQLException {
        final String portAppDepIdStr = rs.getString(names[0]);
        final String portDepIdStr = rs.getString(names[1]);

        if (portAppDepIdStr != null && portDepIdStr != null) {
            final PortletApplicationDeploymentId portAppDepId = new PortletApplicationDeploymentId(portAppDepIdStr);
            return new PortletDeploymentId(portAppDepId, portDepIdStr);
        }
        else {
            return null;
        }
    }
}
