/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.dao.hibernate.types;

import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import javax.portlet.WindowState;

import org.hibernate.HibernateException;
import org.hibernate.usertype.UserType;

/**
 * WindowStateUserType
 * 
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11911 $
 */
public class WindowStateUserType implements UserType {
    private static final int[] TYPES = new int [] { Types.VARCHAR };

    /**
     * @see org.hibernate.usertype.UserType#returnedClass()
     */
    public Class returnedClass() {
    	return WindowState.class;
    }
    
    /**
     * @see org.hibernate.usertype.UserType#assemble(java.io.Serializable, java.lang.Object)
     */
    public Object assemble(Serializable cached, Object owner) throws HibernateException {
        if (cached != null) {
            final String state = (String)cached;
            return this.createWindowState(state);
        }
        else {
            return null;
        }
    }

    /**
     * @see org.hibernate.usertype.UserType#deepCopy(java.lang.Object)
     */
    public Object deepCopy(Object value) throws HibernateException {
        return value;
    }

    /**
     * @see org.hibernate.usertype.UserType#disassemble(java.lang.Object)
     */
    public Serializable disassemble(Object value) throws HibernateException {
        final WindowState state = (WindowState)value;
        return state.toString();
    }

    /**
     * @see org.hibernate.usertype.UserType#equals(java.lang.Object, java.lang.Object)
     */
    public boolean equals(Object x, Object y) throws HibernateException {
        return (x == y || (x != null && x.equals(y)));
    }

    /**
     * @see org.hibernate.usertype.UserType#hashCode(java.lang.Object)
     */
    public int hashCode(Object x) throws HibernateException {
        return x.hashCode();
    }

    /**
     * @see org.hibernate.usertype.UserType#isMutable()
     */
    public boolean isMutable() {
        return false;
    }

    /**
     * @see org.hibernate.usertype.UserType#nullSafeGet(java.sql.ResultSet, java.lang.String[], java.lang.Object)
     */
    public Object nullSafeGet(ResultSet rs, String[] names, Object owner) throws HibernateException, SQLException {
        final String state = rs.getString(names[0]);
        if (state != null) {
            return this.createWindowState(state);
        }
        else {
            return null;
        }
    }

    /**
     * @see org.hibernate.usertype.UserType#nullSafeSet(java.sql.PreparedStatement, java.lang.Object, int)
     */
    public void nullSafeSet(PreparedStatement st, Object value, int index) throws HibernateException, SQLException {
        if (value == null) {
            st.setNull(index, Types.VARCHAR);
        } else {
            final WindowState state = (WindowState)value;
            st.setString(index, state.toString());
        }
    }

    /**
     * @see org.hibernate.usertype.UserType#replace(java.lang.Object, java.lang.Object, java.lang.Object)
     */
    public Object replace(Object original, Object target, Object owner) throws HibernateException {
        return original;
    }
    
    /**
     * @see org.hibernate.usertype.UserType#sqlTypes()
     */
    public int[] sqlTypes() {
        return TYPES;
    }
    
    
    /**
     * Construct a WindowState object given a state string.
     */
    private WindowState createWindowState(String state) {
        if (WindowState.MINIMIZED.toString().equalsIgnoreCase(state)) {
            return WindowState.MINIMIZED;
        }
        else if (WindowState.NORMAL.toString().equalsIgnoreCase(state)) {
            return WindowState.NORMAL;
        }
        else if (WindowState.MAXIMIZED.toString().equalsIgnoreCase(state)) {
            return WindowState.MAXIMIZED;
        }
        else {
            return new WindowState(state);
        }
    }
}
