/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.definition;

import java.util.Locale;
import java.util.Map;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.deployment.IPortletApplicationDeployment;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletApplicationDefinitionTransfer;

/**
 * Immutable portlet application definition whose preferences
 * may have been customized during portlet application publishing.
 * There is no difference right now between an IPortletApplicationDefinition
 * and a IPortletApplicationDeployment, but this serves as a placeholder for 
 * any future differences.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletApplicationDefinition extends IPortletApplicationDefinitionInformationProvider {

    /**
     * Gets the ID of this portlet application definition.
     * @return id the object identifier
     */
    public PortletApplicationDefinitionId getPortletApplicationDefinitionId();

    /**
     * Returns the ID of the portlet application deployment
     * that this portlet application definition is based on.
     * @return the portlet application deployment ID
     */
    public PortletApplicationDeploymentId getPortletApplicationDeploymentId();
    
    /**
     * @return The localized names of the published portlet application. Will not be null, may be empty.
     */
    public Map<Locale, IDisplayName> getDisplayNames();
    
    /**
     * @return The localized descriptions of the of the published portlet application. Will not be null, may be empty.
     */
    public Map<Locale, IDescription> getDescriptions();
    
    /**
     * Mapping of uPortal PortletModes (keys) to custom PortletModes defined
     * by the portlet application deployment (values). <br>
     * <br>
     * -The Map returned must be immutable. <br>
     * -It can only contain PortletMode or null for its keys and values. <br>
     * -Each uPortal PortletMode may be mapped to exactly one custom
     *  PortletMode. Each custom PortletMode may have multiple uPortal
     *  PortletModes mapped to it. <br>
     * 
     * @return A mapping of uPortal PortletModes to custom PortletModes (immutable).
     */
    public Map<PortletMode, PortletMode> getPortletModeMappings();
    
    /**
     * Mapping of uPortal WindowStates (keys) to custom WindowStates defined
     * by the portlet application deployment (values). <br>
     * <br>
     * -The Map returned must be immutable. <br>
     * -It can only contain WindowState or null for its keys and values. <br>
     * -Each uPortal WindowState may be mapped to exactly one custom
     *  WindowState. Each custom WindowState may have multiple uPortal
     *  WindowStates mapped to it. <br>
     * 
     * @return A mapping of uPortal WindowStates to custom WindowStates (immutable).
     */
    public Map<WindowState, WindowState> getWindowStateMappings();
    
    /**
     * Mapping of portlet user attributes names (keys) to uPortal user
     * attribute names (values). <br>
     * <br>
     * -The Map returned must be immutable. <br>
     * -It can only contain String or null for its keys and values. <br>
     * -Each portlet user attribute name may be mapped to exactly one
     *  uPortal user attribute name. Each uPortal user attribute name may
     *  have multiple portlet user attribute names mapped to it. <br>
     * 
     * @return A mapping of portlet user attribute names to uPortal user attribute names (immutable).
     */
    public Map<String, String> getUserAttributeMappings();
    
    /**
     * Gets the underlying the transfer object for the defintion. Provides
     * unlayered view of the properties of this object.
     * 
     * @return The underlying transfer object for the defintion, May not be null.
     */
    public IPortletApplicationDefinitionTransfer getPortletApplicationDefinitionTransfer();
    
    /**
     * @return The parent IPortletApplicationDeployment of this definition
     */
    public IPortletApplicationDeployment getPortletApplicationDeployment();
}
