/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.definition;

import java.util.Locale;
import java.util.Map;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;

/**
 * Mutable portlet application definition.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletApplicationDefinitionControl extends IPortletApplicationDefinition {

    /**
     * Sets the ID of this portlet application definition.
     * @param id the object identifier
     */
    public void setPortletApplicationDefinitionId(PortletApplicationDefinitionId id);
           
    /**
     * @param displayNames The localized names of the published portlet application. Must not be null, may be empty.
     * @throws IllegalArgumentException If displayNames is null.
     */
    public void setDisplayNames(Map<Locale, IDisplayName> displayNames);
    
    /**
     * @param descriptions The localized descriptions of the published portlet application. May be null.
     * @throws IllegalArgumentException If descriptions is null.
     */
    public void setDescriptions(Map<Locale, IDescription> descriptions);
    
    /**
     * Mapping of uPortal PortletModes (keys) to custom PortletModes defined
     * by the portlet application deployment (values). <br>
     * <br>
     * -It can only contain PortletMode or null for its keys and values. <br>
     * -Each uPortal PortletMode may be mapped to exactly one custom
     *  PortletMode. Each custom PortletMode may have multiple uPortal
     *  PortletModes mapped to it. <br>
     *  
     * @param mappings A mapping of uPortal PortletModes to custom PortletModes. May be null.
     * @throws IllegalArgumentException If the Map contains data other than {@link javax.portlet.PortletMode} or null.
     */
    public void setPortletModeMappings(Map<PortletMode, PortletMode> mappings);
    
    /**
     * Mapping of uPortal WindowStates (keys) to custom WindowStates defined
     * by the portlet application deployment (values). <br>
     * <br>
     * -It can only contain WindowState or null for its keys and values. <br>
     * -Each uPortal WindowState may be mapped to exactly one custom
     *  WindowState. Each custom WindowState may have multiple uPortal
     *  WindowStates mapped to it. <br>
     *  
     * @param mappings A mapping of uPortal WindowStates to custom WindowStates. May be null.
     * @throws IllegalArgumentException If the Map contains data other than {@link javax.portlet.WindowState} or null.
     */
    public void setWindowStateMappings(Map<WindowState, WindowState> mappings);
    
    /**
     * Mapping of portlet user attributes names (keys) to uPortal user
     * attribute names (values). <br>
     * <br>
     * -It can only contain String or null for its keys and values. <br>
     * -Each portlet user attribute name may be mapped to exactly one
     *  uPortal user attribute name. Each uPortal user attribute name may
     *  have multiple portlet user attribute names mapped to it. <br>
     *  
     * @param mappings A mapping of portlet user attribute names to uPortal user attribute names. May be null.
     */
    public void setUserAttributeMappings(Map<String, String> mappings);
}
