/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.definition;


import org.jasig.portal.portlet.deployment.IPortletDeployment;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletDefinitionTransfer;

/**
 * Immutable portlet definition whose portlet preferences
 * may have been customized during portlet publishing.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public interface IPortletDefinition extends IPortletDefinitionInformationProvider {
    
    /**
     * Returns the ID of the portlet definition
     * that this portlet definition is based on.
     * @return the portlet definition ID
     */
    public PortletDefinitionId getPortletDefinitionId();

    /**
     * Returns the ID of the portlet deployment
     * that this portlet definition is based on.
     * @return the portlet deployment ID, May not be null.
     */
    public PortletDeploymentId getPortletDeploymentId();
    
    /**
     * Returns parent deployment instance
     * @return parent deployment instance
     */
    public IPortletDeployment getPortletDeployment();

    /**
     * Returns the ID of the portlet application definition that this portlet resides in.
     * @return the portlet application definition ID of this portlet, May not be null.
     */
    public PortletApplicationDefinitionId getPortletApplicationDefinitionId();
    
    /**
     * Gets the underlying the transfer object for the defintion. Provides
     * unlayered view of the properties of this object.
     * 
     * @return The underlying transfer object for the defintion, May not be null.
     */
    public IPortletDefinitionTransfer getPortletDefinitionTransfer();
}
