/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.definition;

import java.util.Locale;
import java.util.Map;

import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.common.IPreferences;

/**
 * Mutable portlet definition.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public interface IPortletDefinitionControl extends IPortletDefinition {

    /**
     * Sets the ID of this portlet application definition.
     * @param id the object identifier
     */
    public void setPortletDefinitionId(PortletDefinitionId id);
    
    /**
     * May not be null.
     * @param functionalName The unique functional name for the portlet definition, may not be null.
     */
    public void setFunctionalName(String functionalName);
    
    /**
     * Sets the ID of the portlet application definition that this portlet resides in.
     * May not be null.
     * @param portletApplicationDefinitionId the portlet application definition ID of this portlet
     */
    public void setPortletApplicationDefinitionId(PortletApplicationDefinitionId portletApplicationDefinitionId);

    /**
     * Sets the publish time portlet preferences which overlay the portlet
     * preferences from the portlet deployment.
     * May not be null.
     * @param preferences the publish-time portlet preferences
     */
    public void setPreferences(IPreferences preferences);
    
    /**
     * Sets the display names of this portlet.
     * May not be null.
     * @param displayNames display names for this portlet
     */
    public void setDisplayNames(Map<Locale, IDisplayName> displayNames);
    
    /**
     * Sets the descriptions for this portlet.
     * May not be null.
     * @param descriptions the descriptions for this portlet
     */
    public void setDescriptions(Map<Locale, IDescription> descriptions);
    
    /**
     * Sets the time out value defined during publishing the portlet.
     * @param timeOut the time out value
     */
    public void setRenderTimeOut(long timeOut);
    
    /**
     * Sets the PortletInfo defined during publising the portlet.
     * @param portletInfo the portlet info.
     */
    public void setPortletInfo(IPortletInfo portletInfo);
}
