/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.definition;

import java.util.Locale;
import java.util.Map;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.window.IPortletWindow;


/**
 * Provides the setter methods for the localized methods.
 * 
 * @see LocalizedPortletApplicationDefinitionImpl
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class LocalizedPortletApplicationDefinitionControlImpl extends LocalizedPortletApplicationDefinitionImpl implements ILocalizedPortletApplicationDefinitionControl {
    private final IPortletApplicationDefinitionControl portletDefinitionControl;
    
    public LocalizedPortletApplicationDefinitionControlImpl(IPortletApplicationDefinitionControl portletDefinitionControl, ILocaleManager localeManager) {
        this(portletDefinitionControl, localeManager, null);
    }

    public LocalizedPortletApplicationDefinitionControlImpl(IPortletApplicationDefinitionControl portletDefinitionControl, ILocaleManager localeManager, IPortletWindow portWin) {
        super(portletDefinitionControl, localeManager, portWin);

        if (portletDefinitionControl == null) {
            throw new IllegalArgumentException("portletDefinitionControl may not be null");
        }
            
        this.portletDefinitionControl = portletDefinitionControl;
    }

    
    //********** Impls fopr ILocalizedPortletDefintionControl **********\\
    
    public void setLocalDisplayName(String displayNameText) {
        final IDisplayName displayName = this.createLocalDisplayName(displayNameText);
        final Map<Locale, IDisplayName> displayNames = this.portletDefinitionControl.getDisplayNames();
        displayNames.put(displayName.getLocale(), displayName);
    }
    
    public void setLocalDescription(String descriptionText) {
        final IDescription description = this.createLocalDescription(descriptionText);
        final Map<Locale, IDescription> descriptions = this.portletDefinitionControl.getDescriptions();
        descriptions.put(description.getLocale(), description);
    }

    
    //********** Impls fopr IPortletDefintionControl **********\\
    public void setPortletApplicationDefinitionId(PortletApplicationDefinitionId id) {
        this.portletDefinitionControl.setPortletApplicationDefinitionId(id);
    }

    public void setDisplayNames(Map<Locale, IDisplayName> displayNames) {
        this.portletDefinitionControl.setDisplayNames(displayNames);
    }

    public void setDescriptions(Map<Locale, IDescription> descriptions) {
        this.portletDefinitionControl.setDescriptions(descriptions);
    }

    public void setPortletModeMappings(Map<PortletMode, PortletMode> mappings) {
        this.portletDefinitionControl.setPortletModeMappings(mappings);
    }

    public void setWindowStateMappings(Map<WindowState, WindowState> mappings) {
        this.portletDefinitionControl.setWindowStateMappings(mappings);
    }

    public void setUserAttributeMappings(Map<String, String> mappings) {
        this.portletDefinitionControl.setUserAttributeMappings(mappings);
    }
}