/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.definition;

import java.util.Locale;
import java.util.Map;
import java.util.Set;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.common.BaseLocalizedBean;
import org.jasig.portal.portlet.common.ICustomPortletMode;
import org.jasig.portal.portlet.common.ICustomWindowState;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.ISecurityConstraint;
import org.jasig.portal.portlet.common.IUserAttribute;
import org.jasig.portal.portlet.deployment.IPortletApplicationDeployment;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletApplicationDefinitionTransfer;
import org.jasig.portal.portlet.window.IPortletWindow;


/**
 * Localized portlet definition. Wraps a {@link IPortletApplicationDefinition}
 * with a specified locale. The specified locale is used by the
 * {@link ILocalizedPortletApplicationDefinition} methods when they are called.
 * <br>
 * <br>
 * There is a default locale defined of Locale.ENGLISH. If specified 
 * in the constructor the localized methods will fall back to the default
 * locale if no value is found for the configured locale.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class LocalizedPortletApplicationDefinitionImpl extends BaseLocalizedBean implements ILocalizedPortletApplicationDefinition {
    private final IPortletApplicationDefinition portletDefinition;
    
    public LocalizedPortletApplicationDefinitionImpl(IPortletApplicationDefinition portletDefinition, ILocaleManager localeManager) {
        this(portletDefinition, localeManager, null);
    }
    
    public LocalizedPortletApplicationDefinitionImpl(IPortletApplicationDefinition portletDefinition, ILocaleManager localeManager, IPortletWindow portWin) {
        super(localeManager, portWin);

        if (portletDefinition == null) {
            throw new IllegalArgumentException("portletDefinition may not be null");
        }

        this.portletDefinition = portletDefinition;
    }

    //********** Impls fopr ILocalizedPortletDefintion **********\\
    
    public String getLocalDisplayName() {
        final Map<Locale, IDisplayName> displayNames = this.portletDefinition.getDisplayNames();
        final IDisplayName displayName = this.resolveLocalDisplayName(displayNames);
        
        if (displayName != null) {
            return displayName.getDisplayNameText();
        }
        else {
            return null;
        }
    }
    
    public String getLocalDescription() {
        final Map<Locale, IDescription> descriptions = this.portletDefinition.getDescriptions();
        final IDescription description = this.resolveLocalDescription(descriptions);
        
        if (description != null) {
            return description.getDescriptionText();
        }
        else {
            return null;
        }
    }


    //********** Delegates for IPortletApplicationDefintion **********\\
    public PortletApplicationDefinitionId getPortletApplicationDefinitionId() {
        return this.portletDefinition.getPortletApplicationDefinitionId();
    }

    public PortletApplicationDeploymentId getPortletApplicationDeploymentId() {
        return this.portletDefinition.getPortletApplicationDeploymentId();
    }

    public Map<Locale, IDisplayName> getDisplayNames() {
        return this.portletDefinition.getDisplayNames();
    }

    public Map<Locale, IDescription> getDescriptions() {
        return this.portletDefinition.getDescriptions();
    }

    public Map<PortletMode, PortletMode> getPortletModeMappings() {
        return this.portletDefinition.getPortletModeMappings();
    }

    public Map<WindowState, WindowState> getWindowStateMappings() {
        return this.portletDefinition.getWindowStateMappings();
    }

    public Map<String, String> getUserAttributeMappings() {
        return this.portletDefinition.getUserAttributeMappings();
    }

    public String getVersion() {
        return this.portletDefinition.getVersion();
    }

    public Set<ICustomPortletMode> getCustomPortletModes() {
        return this.portletDefinition.getCustomPortletModes();
    }

    public Set<ICustomWindowState> getCustomWindowStates() {
        return this.portletDefinition.getCustomWindowStates();
    }

    public Set<IUserAttribute> getUserAttributes() {
        return this.portletDefinition.getUserAttributes();
    }

    public Set<ISecurityConstraint> getSecurityConstraints() {
        return this.portletDefinition.getSecurityConstraints();
    }

    public IPortletApplicationDefinitionTransfer getPortletApplicationDefinitionTransfer() {
        return this.portletDefinition.getPortletApplicationDefinitionTransfer();
    }

    public IPortletApplicationDeployment getPortletApplicationDeployment() {
        return this.portletDefinition.getPortletApplicationDeployment();
    }
}
