/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.definition;

import java.util.Locale;
import java.util.Map;

import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.window.IPortletWindow;


/**
 * Provides the setter methods for the localized methods.
 * 
 * @see LocalizedPortletDefinitionImpl
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class LocalizedPortletDefinitionControlImpl extends LocalizedPortletDefinitionImpl implements ILocalizedPortletDefinitionControl {
    private final IPortletDefinitionControl portletDefinitionControl;
    
    public LocalizedPortletDefinitionControlImpl(IPortletDefinitionControl portletDefinitionControl, ILocaleManager localeManager) {
        this(portletDefinitionControl, localeManager, null);
    }

    public LocalizedPortletDefinitionControlImpl(IPortletDefinitionControl portletDefinitionControl, ILocaleManager localeManager, IPortletWindow portWin) {
        super(portletDefinitionControl, localeManager, portWin);

        if (portletDefinitionControl == null) {
            throw new IllegalArgumentException("portletDefinitionControl may not be null");
        }
            
        this.portletDefinitionControl = portletDefinitionControl;
    }

    
    //********** Impls fopr ILocalizedPortletDefintionControl **********\\
    
    public void setLocalDisplayName(String displayNameText) {
        final IDisplayName displayName = this.createLocalDisplayName(displayNameText);
        final Map<Locale, IDisplayName> displayNames = this.portletDefinitionControl.getDisplayNames();
        displayNames.put(displayName.getLocale(), displayName);
    }

    public void setLocalDescription(String descriptionText) {
        final IDescription description = this.createLocalDescription(descriptionText);
        final Map<Locale, IDescription> descriptions = this.portletDefinitionControl.getDescriptions();
        descriptions.put(description.getLocale(), description);
    }

    
    //********** Impls fopr IPortletDefintionControl **********\\
    
    public void setDescriptions(Map<Locale, IDescription> descriptions) {
        this.portletDefinitionControl.setDescriptions(descriptions);
    }

    public void setDisplayNames(Map<Locale, IDisplayName> displayNames) {
        this.portletDefinitionControl.setDisplayNames(displayNames);        
    }

    public void setFunctionalName(String functionalName) {
        this.portletDefinitionControl.setFunctionalName(functionalName);        
    }

    public void setPortletDefinitionId(PortletDefinitionId id) {
        this.portletDefinitionControl.setPortletDefinitionId(id);        
    }

    public void setPortletApplicationDefinitionId(PortletApplicationDefinitionId portletApplicationDefinitionId) {
        this.portletDefinitionControl.setPortletApplicationDefinitionId(portletApplicationDefinitionId);        
    }

    public void setPreferences(IPreferences preferences) {
        this.portletDefinitionControl.setPreferences(preferences);        
    }

    public void setRenderTimeOut(long timeOut) {
        this.portletDefinitionControl.setRenderTimeOut(timeOut);
    }
    
    public void setPortletInfo(IPortletInfo portletInfo) {
        this.portletDefinitionControl.setPortletInfo(portletInfo);
    }
}
