/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.definition;

import java.util.Locale;
import java.util.Map;
import java.util.Set;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.jasig.portal.portlet.common.ICustomPortletMode;
import org.jasig.portal.portlet.common.ICustomWindowState;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.ISecurityConstraint;
import org.jasig.portal.portlet.common.IUserAttribute;
import org.jasig.portal.portlet.deployment.IPortletApplicationDeployment;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletApplicationDefinitionTransfer;

/**
 * Implementation of a portlet application definition.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public class PortletApplicationDefinitionImpl implements IPortletApplicationDefinitionControl {
    
    private final IPortletApplicationDefinitionTransfer portletApplicationDefinitionTransfer;
    private final IPortletApplicationDeployment portletApplicationDeployment;

    
    public PortletApplicationDefinitionImpl(IPortletApplicationDefinitionTransfer portletApplicationDefinitionTransfer,
                                            IPortletApplicationDeployment portletApplicationDeployment) {

        if (portletApplicationDefinitionTransfer == null) {
            throw new IllegalArgumentException("portletApplicationDefinitionTransfer may not be null");
        }
        if (portletApplicationDeployment == null) {
            throw new IllegalArgumentException("portletApplicationDeployment may not be null");
        }
        if (!portletApplicationDefinitionTransfer.getPortletApplicationDeploymentId().equals(portletApplicationDeployment.getPortletApplicationDeploymentId())) {
            throw new IllegalArgumentException("portletApplicationDefinitionTransfer.getPortletApplicationDeploymentId() must equal portletApplicationDeployment.getPortletApplicationDeploymentId(). '" + portletApplicationDefinitionTransfer.getPortletApplicationDeploymentId() + "' != '" + portletApplicationDeployment.getPortletApplicationDeploymentId() + "'");
        }
        
        this.portletApplicationDefinitionTransfer = portletApplicationDefinitionTransfer;
        this.portletApplicationDeployment = portletApplicationDeployment;
    }
    
    
    public PortletApplicationDefinitionId getPortletApplicationDefinitionId() {
        return this.portletApplicationDefinitionTransfer.getPortletApplicationDefinitionId();
    }
    
    public void setPortletApplicationDefinitionId(PortletApplicationDefinitionId id) {
    	this.portletApplicationDefinitionTransfer.setPortletApplicationDefinitionId(id);
    }
    
    public PortletApplicationDeploymentId getPortletApplicationDeploymentId() {
        return this.portletApplicationDefinitionTransfer.getPortletApplicationDeploymentId();
    }
  
    public Map<Locale, IDescription> getDescriptions() {
        return this.portletApplicationDefinitionTransfer.getDescriptions();
    }

    public Map<Locale, IDisplayName> getDisplayNames() {
        return this.portletApplicationDefinitionTransfer.getDisplayNames();
    }

    public Map<PortletMode, PortletMode> getPortletModeMappings() {
        return this.portletApplicationDefinitionTransfer.getPortletModeMappings();
    }

    public Map<String, String> getUserAttributeMappings() {
        return this.portletApplicationDefinitionTransfer.getUserAttributeMappings();
    }

    public Map<WindowState, WindowState> getWindowStateMappings() {
        return this.portletApplicationDefinitionTransfer.getWindowStateMappings();
    }

    public void setDescriptions(Map<Locale, IDescription> descriptions) {
        this.portletApplicationDefinitionTransfer.setDescriptions(descriptions);
    }

    public void setDisplayNames(Map<Locale, IDisplayName> name) {
        this.portletApplicationDefinitionTransfer.setDisplayNames(name);
    }

    public void setPortletModeMappings(Map<PortletMode, PortletMode> mappings) {
        this.portletApplicationDefinitionTransfer.setPortletModeMappings(mappings);
    }

    public void setUserAttributeMappings(Map<String, String> mappings) {
        this.portletApplicationDefinitionTransfer.setUserAttributeMappings(mappings);
    }

    public void setWindowStateMappings(Map<WindowState, WindowState> mappings) {
        this.portletApplicationDefinitionTransfer.setWindowStateMappings(mappings);
    }
    
    public IPortletApplicationDefinitionTransfer getPortletApplicationDefinitionTransfer() {
        return this.portletApplicationDefinitionTransfer;
    }

    
    // IPortletApplicationDeploymentInformationProvider methods

    public String getVersion() {
        return this.portletApplicationDeployment.getVersion();
    }
    
    public Set<ICustomPortletMode> getCustomPortletModes() {
        return this.portletApplicationDeployment.getCustomPortletModes();
    }

    public Set<ICustomWindowState> getCustomWindowStates() {
        return this.portletApplicationDeployment.getCustomWindowStates();
    }
    
    public Set<IUserAttribute> getUserAttributes() {
        return this.portletApplicationDeployment.getUserAttributes();
    }
    
    public Set<ISecurityConstraint> getSecurityConstraints() {
        return this.portletApplicationDeployment.getSecurityConstraints();
    }
    
    public IPortletApplicationDeployment getPortletApplicationDeployment() {
        return this.portletApplicationDeployment;
    }
    
    
    // Helper methods

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (!(obj instanceof IPortletApplicationDefinition)) {
            return false;
        }
        if (this == obj) {
            return true;
        }
        
        IPortletApplicationDefinition other = (IPortletApplicationDefinition)obj;
        return new EqualsBuilder()
            .append(this.portletApplicationDefinitionTransfer, other.getPortletApplicationDefinitionTransfer())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return this.portletApplicationDefinitionTransfer.hashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
        .append("portletApplicationDefinitionTransfer", this.portletApplicationDefinitionTransfer)
        .toString();
    }
}
