/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.deployment;

import java.io.Serializable;
import java.util.Set;

import org.jasig.portal.portlet.common.ICustomPortletMode;
import org.jasig.portal.portlet.common.ICustomWindowState;
import org.jasig.portal.portlet.common.ISecurityConstraint;
import org.jasig.portal.portlet.common.IUserAttribute;

/**
 * Knows how to supply basic portlet application information
 * as specified in the portlet application deployment
 * descriptor (portlet.xml) file.
 * @author Michael Ivanov, mivanov@unicon.net
 * @version $Revision: 11911 $
 */
public interface IPortletApplicationDeploymentInformationProvider extends Serializable {

    /**
     * Returns the version of this portlet application.
     * The return value may be NULL.
     * @return the version
     */
    public String getVersion();

    /**
     * Returns the custom portlet modes defined in this portlet application.
     * The return value may be NULL.
     * @return the custom portlet modes
     */
    public Set<ICustomPortletMode> getCustomPortletModes();
    
    /**
     * Returns the custom window states defined in this portlet application.
     * The return value may be NULL.
     * @return the custom window states
     */
    public Set<ICustomWindowState> getCustomWindowStates();
    
    /**
     * Returns the user attributes defined in this portlet application.
     * The return value may be NULL.
     * @return the user attributes
     */
    public Set<IUserAttribute> getUserAttributes();
    
    /**
     * Returns the security contraints defined in this portlet application.
     * The return value may be NULL.
     * @return the security constraints
     */
    public Set<ISecurityConstraint> getSecurityConstraints();

}
