/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.deployment;

import java.util.Locale;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.pluto.descriptors.common.DescriptionDD;
import org.jasig.portal.portlet.common.IDescription;

/**
 * An IDescription that wraps a Pluto DescriptionDD object
 * 
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public class PlutoDescriptionImpl implements IDescription {
    private final DescriptionDD descriptionDD;
    
    public PlutoDescriptionImpl(DescriptionDD descriptionDD) {
        if (descriptionDD == null) {
            throw new IllegalArgumentException("DescriptionDD can not be null");
        }
        
        this.descriptionDD = descriptionDD;
    }

    /**
     * @see org.jasig.portal.portlet.common.IDescription#getDescriptionText()
     */
    public String getDescriptionText() {
        return this.descriptionDD.getDescription();
    }

    /**
     * @see org.jasig.portal.portlet.common.IDescription#setDescriptionText(java.lang.String)
     */
    public void setDescriptionText(String descriptionText) {
        throw new UnsupportedOperationException("PlutoDescriptionImpl is immutable");
    }

    /**
     * @see org.jasig.portal.portlet.common.ILocalizable#getLocale()
     */
    public Locale getLocale() {
        final String lang = this.descriptionDD.getLang();
        return new Locale(lang);
    }

    /**
     * @see org.jasig.portal.portlet.common.ILocalizable#setLocale(java.util.Locale)
     */
    public void setLocale(Locale locale) {
        throw new UnsupportedOperationException("PlutoDescriptionImpl is immutable");
    }


    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
            .append("locale", this.getLocale())
            .append("descriptionText", this.getDescriptionText())
            .toString();
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (!(obj instanceof IDescription)) {
            return false;
        }
        if (this == obj) {
            return true;
        }
        
        IDescription other = (IDescription)obj;
        return new EqualsBuilder()
            .append(this.getLocale(), other.getLocale())
            .append(this.getDescriptionText(), other.getDescriptionText())
            .isEquals();
    }
    
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        //Binary or on each to ensure we get an odd number
        final int classCode = this.getClass().hashCode() | 1;
        final int classNameCode = this.getClass().getName().hashCode() | 1;
        
        return new HashCodeBuilder(classCode, classNameCode)
            .append(this.getLocale())
            .append(this.getDescriptionText())
            .toHashCode();
    }
}
