/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.deployment;

import java.util.Locale;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.pluto.descriptors.common.DisplayNameDD;
import org.jasig.portal.portlet.common.IDisplayName;

/**
 * An IDescription that wraps a Pluto DisplayNameDD object
 * 
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public class PlutoDisplayNameImpl implements IDisplayName {
    private final DisplayNameDD displayNameDD;
    
    public PlutoDisplayNameImpl(DisplayNameDD displayNameDD) {
        if (displayNameDD == null) {
            throw new IllegalArgumentException("DisplayNameDD can not be null");
        }
        
        this.displayNameDD = displayNameDD;
    }

    
    /**
     * @see org.jasig.portal.portlet.common.IDisplayName#getDisplayNameText()
     */
    public String getDisplayNameText() {
        return this.displayNameDD.getDisplayName();
    }

    /**
     * @see org.jasig.portal.portlet.common.IDisplayName#setDisplayNameText(java.lang.String)
     */
    public void setDisplayNameText(String displayNameText) {
        // TODO Auto-generated method stub
        throw new UnsupportedOperationException("PlutoDisplayNameImpl is immutable");
    }

    /**
     * @see org.jasig.portal.portlet.common.ILocalizable#getLocale()
     */
    public Locale getLocale() {
        final String lang = this.displayNameDD.getLang();
        return new Locale(lang);
    }

    /**
     * @see org.jasig.portal.portlet.common.ILocalizable#setLocale(java.util.Locale)
     */
    public void setLocale(Locale locale) {
        throw new UnsupportedOperationException("PlutoDisplayNameImpl is immutable");
    }


    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
            .append("locale", this.getLocale())
            .append("descriptionText", this.getDisplayNameText())
            .toString();
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (!(obj instanceof IDisplayName)) {
            return false;
        }
        if (this == obj) {
            return true;
        }
        
        IDisplayName other = (IDisplayName)obj;
        return new EqualsBuilder()
            .append(this.getLocale(), other.getLocale())
            .append(this.getDisplayNameText(), other.getDisplayNameText())
            .isEquals();
    }
    
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        //Binary or on each to ensure we get an odd number
        final int classCode = this.getClass().hashCode() | 1;
        final int classNameCode = this.getClass().getName().hashCode() | 1;
        
        return new HashCodeBuilder(classCode, classNameCode)
            .append(this.getLocale())
            .append(this.getDisplayNameText())
            .toHashCode();
    }
}
