/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.deployment;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.pluto.descriptors.portlet.PortletInfoDD;
import org.jasig.portal.portlet.common.IPortletInfo;

/**
 * Portlet info impl for wrapping a Pluto PortletInfoDD object.
 * 
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public class PlutoPortletInfo implements IPortletInfo {
    private final PortletInfoDD portletInfoDD;
    
    public PlutoPortletInfo(PortletInfoDD portletInfoDD) {
        if (portletInfoDD == null) {
            throw new IllegalArgumentException("PortletInfoDD can not be null");
        }
        
        this.portletInfoDD = portletInfoDD;
    }

    /**
     * @see org.jasig.portal.portlet.common.IPortletInfo#getKeywords()
     */
    public List<String> getKeywords() {
        final String keywordsStr = this.portletInfoDD.getKeywords();
        if (keywordsStr == null) {
            return null;
        }
        
        final String[] keywordsArray = keywordsStr.split(",");
        final List<String> keywords = new ArrayList<String>(keywordsArray.length);
        for (final String keyword : keywordsArray) {
            keywords.add(keyword.trim());
        }
        return Collections.unmodifiableList(keywords);
    }

    /**
     * @see org.jasig.portal.portlet.common.IPortletInfo#getKeywordsString()
     */
    public String getKeywordsString() {
        return this.portletInfoDD.getKeywords();
    }

    /**
     * @see org.jasig.portal.portlet.common.IPortletInfo#getShortTitle()
     */
    public String getShortTitle() {
        return this.portletInfoDD.getShortTitle();
    }

    /**
     * @see org.jasig.portal.portlet.common.IPortletInfo#getTitle()
     */
    public String getTitle() {
        return this.portletInfoDD.getTitle();
    }
    

    /**
     * @see org.jasig.portal.portlet.common.IPortletInfo#setKeywords(java.util.List)
     */
    public void setKeywords(List<String> keywords) {
        throw new UnsupportedOperationException("PlutoPortletInfo is read only");
    }

    /**
     * @see org.jasig.portal.portlet.common.IPortletInfo#setKeywordsString(java.lang.String)
     */
    public void setKeywordsString(String keywords) {
        throw new UnsupportedOperationException("PlutoPortletInfo is read only");
    }

    /**
     * @see org.jasig.portal.portlet.common.IPortletInfo#setShortTitle(java.lang.String)
     */
    public void setShortTitle(String shortTitle) {
        throw new UnsupportedOperationException("PlutoPortletInfo is read only");
    }

    /**
     * @see org.jasig.portal.portlet.common.IPortletInfo#setTitle(java.lang.String)
     */
    public void setTitle(String title) {
        throw new UnsupportedOperationException("PlutoPortletInfo is read only");
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof IPortletInfo)) {
            return false;
        }
        IPortletInfo rhs = (IPortletInfo)object;
        return new EqualsBuilder()
        .append(this.getShortTitle(), rhs.getShortTitle())
        .append(this.getTitle(), rhs.getTitle())
        .append(this.getKeywords(), rhs.getKeywords())
        .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(1237991005, -640905427)
        .append(this.getShortTitle())
        .append(this.getTitle())
        .append(this.getKeywords())
        .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
            .append("shortTitle", this.getShortTitle())
            .append("title", this.getTitle())
            .append("keywordsString", this.getKeywordsString())
            .toString();
    }
    
    
}
