/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.deployment;

import java.util.List;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.pluto.descriptors.portlet.PortletPreferenceDD;
import org.jasig.portal.portlet.common.IPreference;

/**
 * IPreference wrapper of the pluto PortletPreferenceDD object.
 * 
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public class PlutoPortletPreference implements IPreference {
    private final PortletPreferenceDD portletPreferenceDD;
    
    public PlutoPortletPreference(PortletPreferenceDD portletPreferenceDD) {
        if (portletPreferenceDD == null) {
            throw new IllegalArgumentException("PortletPreferenceDD can not be null"); 
        }
        
        this.portletPreferenceDD = portletPreferenceDD;
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreference#getId()
     */
    public long getId() {
        return this.portletPreferenceDD.getName().hashCode();
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreference#getName()
     */
    public String getName() {
        return this.portletPreferenceDD.getName();
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreference#getValues()
     */
    @SuppressWarnings("unchecked")
    public List<String> getValues() {
        return this.portletPreferenceDD.getValues();
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreference#isReadOnly()
     */
    public boolean isReadOnly() {
        return this.portletPreferenceDD.isReadOnly();
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreference#setId(long)
     */
    public void setId(long id) {
        throw new UnsupportedOperationException("PlutoPortletPreference is read only");
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreference#setName(java.lang.String)
     */
    public void setName(String name) {
        throw new UnsupportedOperationException("PlutoPortletPreference is read only");
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreference#setReadOnly(boolean)
     */
    public void setReadOnly(boolean readOnly) {
        throw new UnsupportedOperationException("PlutoPortletPreference is read only");
    }

    /**
     * @see org.jasig.portal.portlet.common.IPreference#setValues(java.util.List)
     */
    public void setValues(List<String> values) {
        throw new UnsupportedOperationException("PlutoPortletPreference is read only");
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof IPreference)) {
            return false;
        }
        IPreference rhs = (IPreference)object;
        return new EqualsBuilder()
            .append(this.getId(), rhs.getId())
            .append(this.getName(), rhs.getName())
            .append(this.isReadOnly(), rhs.isReadOnly())
            .append(this.getValues(), rhs.getValues())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(-1913602905, 1392612231)
            .append(this.getId())
            .append(this.getName())
            .append(this.isReadOnly())
            .append(this.getValues())
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
            .append("id", this.getId())
            .append("name", this.getName())
            .append("readOnly", this.isReadOnly())
            .append("valueList", this.getValues())
            .toString();
    }
}
