/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.deployment;

import java.util.Set;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.jasig.portal.portlet.common.ICustomPortletMode;
import org.jasig.portal.portlet.common.ICustomWindowState;
import org.jasig.portal.portlet.common.ISecurityConstraint;
import org.jasig.portal.portlet.common.IUserAttribute;
import org.jasig.portal.portlet.transfer.IPortletApplicationDeploymentTransfer;

/**
 * Implementation of a portlet application deployment.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public class PortletApplicationDeploymentImpl implements IPortletApplicationDeployment {
    private final IPortletApplicationDeploymentTransfer portletApplicationDeploymentTransfer;
    
    public PortletApplicationDeploymentImpl(IPortletApplicationDeploymentTransfer portletApplicationDeploymentTransfer ) {
        if (portletApplicationDeploymentTransfer == null) {
            throw new IllegalArgumentException("portletApplicationDeploymentTransfer may not be null");
        }

        this.portletApplicationDeploymentTransfer = portletApplicationDeploymentTransfer;
    }
    
    public PortletApplicationDeploymentId getPortletApplicationDeploymentId() {
        return this.portletApplicationDeploymentTransfer.getPortletApplicationDeploymentId();
    }
      
    public String getVersion() {
        return this.portletApplicationDeploymentTransfer.getVersion();
    }
    
    public Set<ICustomPortletMode> getCustomPortletModes() {
        return this.portletApplicationDeploymentTransfer.getCustomPortletModes();
    }
    
    public Set<ICustomWindowState> getCustomWindowStates() {
        return this.portletApplicationDeploymentTransfer.getCustomWindowStates();
    }
    
    public Set<IUserAttribute> getUserAttributes() {
        return this.portletApplicationDeploymentTransfer.getUserAttributes();
    }
    
    public Set<ISecurityConstraint> getSecurityConstraints() {
        return this.portletApplicationDeploymentTransfer.getSecurityConstraints();
    }
    
    public IPortletApplicationDeploymentTransfer getPortletApplicationDeploymentTransfer() {
        return this.portletApplicationDeploymentTransfer;
    }
    
    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (!(obj instanceof IPortletApplicationDeployment)) {
            return false;
        }
        if (this == obj) {
            return true;
        }
        
        IPortletApplicationDeployment other = (IPortletApplicationDeployment)obj;
        return new EqualsBuilder()
            .append(this.portletApplicationDeploymentTransfer, other.getPortletApplicationDeploymentTransfer())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return this.portletApplicationDeploymentTransfer.hashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
        .append("portletApplicationDeploymentTransfer", this.portletApplicationDeploymentTransfer)
        .toString();
    }
}
