/**
 * 
 */
package org.jasig.portal.portlet.deployment;

import org.apache.commons.lang.builder.HashCodeBuilder;
import org.jasig.portal.core.BaseStringObjectId;
import org.springframework.util.Assert;

/**
 * Identifies a {@link IPortletDeployment}.
 * 
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision: 11942 $
 */
public class PortletDeploymentId extends BaseStringObjectId<PortletDeploymentId> {
    private final PortletApplicationDeploymentId portletApplicationDeploymentId;
    
    /**
     * @param portAppDepId The ID of the parent {@link IPortletApplicationDeployment}
     * @param id The ID string of the {@link IPortletDeployment}
     * @see BaseStringObjectId#BaseStringObjectId(String)
     */
    public PortletDeploymentId(PortletApplicationDeploymentId portAppDepId, String id) {
        super(id);
        
        Assert.notNull(portAppDepId, "PortletApplicationDeploymentId can not be null");
        this.portletApplicationDeploymentId = portAppDepId;        
    }
    
    /**
     * @see org.jasig.portal.portlet.deployment.PortletDeploymentId#getPortletApplicationDeploymentId()
     */
    public PortletApplicationDeploymentId getPortletApplicationDeploymentId() {
        return this.portletApplicationDeploymentId;
    }
    
    
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(-1195226191, -1679281191)
            .append(this.portletApplicationDeploymentId)
            .append(this.toString())
            .toHashCode();
    }
    
    /**
     * Compares the {@link PortletApplicationDeploymentId}s using {@link PortletApplicationDeploymentId#compareTo(PortletApplicationDeploymentId)} and
     * adds the result to a call to super.compareTo(PortletDeploymentId).
     * 
     * @see org.jasig.portal.core.ObjectId#compareTo(org.jasig.portal.core.ObjectId)
     */
    @Override
    public int compareTo(PortletDeploymentId o) {
        if (o == null) {
            return Integer.MIN_VALUE;
        }
        
        return this.portletApplicationDeploymentId.compareTo(o.portletApplicationDeploymentId) + super.compareTo(o);
    }
}