/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.deployment;

import java.util.Locale;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.jasig.portal.portlet.common.IContentType;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.IInitParameter;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.common.ISecurityRoleReference;
import org.jasig.portal.portlet.transfer.IPortletDeploymentTransfer;


/**
 * Portlet definition implementation.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public class PortletDeploymentImpl implements IPortletDeployment {

    private IPortletDeploymentTransfer portletDeploymentTransfer;
    
    public PortletDeploymentImpl() { }
    
    public PortletDeploymentImpl (IPortletDeploymentTransfer portletDeploymentTransfer ) { 
        this.portletDeploymentTransfer = portletDeploymentTransfer;
    }
    
    public void setPortletDeploymentTransfer( IPortletDeploymentTransfer portletDeploymentTransfer ) { 
        this.portletDeploymentTransfer = portletDeploymentTransfer;
    }
    
    public PortletDeploymentId getPortletDeploymentId() {
        return this.portletDeploymentTransfer.getPortletDeploymentId();
    }
    
    public Map<Locale, IDescription> getDescriptions() {
        return this.portletDeploymentTransfer.getDescriptions();
    }

    public String getPortletName() {
        return this.portletDeploymentTransfer.getPortletName();
    }

    public Map<Locale, IDisplayName> getDisplayNames() {
        return this.portletDeploymentTransfer.getDisplayNames();
    }

    public String getPortletClassName() {
        return this.portletDeploymentTransfer.getPortletClassName();
    }

    public Map<String, IInitParameter> getInitParameters() {
        return this.portletDeploymentTransfer.getInitParameters();
    }

    public Integer getPortletExpirationCache() {
        return this.portletDeploymentTransfer.getPortletExpirationCache();
    }

    public Map<String, IContentType> getSupportedContentTypes() {
        return this.portletDeploymentTransfer.getSupportedContentTypes();
    }

    public Set<Locale> getSupportedLocales() {
        return this.portletDeploymentTransfer.getSupportedLocales();
    }

    public IPortletInfo getPortletInfo() {
        return this.portletDeploymentTransfer.getPortletInfo();
    }

    public IPreferences getPreferences() {
        return this.portletDeploymentTransfer.getPreferences();
    }

    public Map<String, ISecurityRoleReference> getSecurityRoleReferences() {
        return this.portletDeploymentTransfer.getSecurityRoleReferences();
    }

    public PortletApplicationDeploymentId getPortletApplicationDeploymentId() {
        return this.portletDeploymentTransfer.getPortletApplicationDeploymentId();
    }

    public IPortletDeploymentTransfer getPortletDeploymentTransfer() {
        return this.portletDeploymentTransfer;
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof IPortletDeployment)) {
            return false;
        }
        IPortletDeployment rhs = (IPortletDeployment)object;
        return new EqualsBuilder()
            .append(this.portletDeploymentTransfer, rhs.getPortletDeploymentTransfer())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(226417175, -455483123)
            .append(this.portletDeploymentTransfer)
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
            .appendSuper(super.toString())
            .append("portletDeploymentTransfer", this.portletDeploymentTransfer)
            .toString();
    }
}
