/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.entity;

import java.util.Set;

import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.transfer.IPortletEntityTransfer;
import org.jasig.portal.security.UserId;


/**
 * The portlet entity interface.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */

public interface IPortletEntity extends IPortletEntityInformationProvider {
	
	/**
     * Returns the ID of the portlet entity
     * that this portlet definition is based on.
     * @return the portlet definition ID
     */
    public PortletEntityId getPortletEntityId();
   
    
    /**
     * @return The IUserID of the user this entity is for.
     */
    public UserId getUserId();
	
     /**
     * Returns the ID of the corresponding portlet definition.
     * @return the portlet definition ID
     */
    public PortletDefinitionId getPortletDefinitionId();
    
    /**
     * @return The parent portlet definition
     */
    public IPortletDefinition getPortletDefinition();
    
    /**
     * Returns the ID of the portlet application entity that this portlet entity resides in.
     * @return the portlet application entity ID of this portlet
     */
    public PortletApplicationEntityId getPortletApplicationEntityId();
    
    /**
     * Returns the list of the portlet windows of this portlet entity.
     * @return the portlet windows collection
     */
    public Set getPortletWindows();
    
    /**
     * @return The underlying transfer object, may not be null.
     */
    public IPortletEntityTransfer getPortletEntityTransfer();
}
