/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.entity;

import java.util.Set;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.jasig.portal.portlet.common.ICustomPortletMode;
import org.jasig.portal.portlet.common.ICustomWindowState;
import org.jasig.portal.portlet.common.ISecurityConstraint;
import org.jasig.portal.portlet.common.IUserAttribute;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.transfer.IPortletApplicationEntityTransfer;

/**
 * Implementation of a portlet application entity.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public class PortletApplicationEntityImpl implements IPortletApplicationEntityControl {
    private final IPortletApplicationEntityTransfer portletApplicationEntityTransfer;
    private final IPortletApplicationDefinition portletApplicationDefinition;

    public PortletApplicationEntityImpl(IPortletApplicationEntityTransfer portletApplicationEntityTransfer,
                                        IPortletApplicationDefinition portletApplicationDefinition) {
        
        if (portletApplicationEntityTransfer == null) {
            throw new IllegalArgumentException("portletApplicationEntityTransfer may not be null");
        }
        if (portletApplicationDefinition == null) {
            throw new IllegalArgumentException("portletApplicationDefinition may not be null");
        }
        if (!portletApplicationEntityTransfer.getPortletApplicationDefinitionId().equals(portletApplicationDefinition.getPortletApplicationDefinitionId())) {
            throw new IllegalArgumentException("portletApplicationEntityTransfer.getPortletApplicationDefinitionId() must equal portletApplicationDefinition.getPortletApplicationDefinitionId(). '" + portletApplicationEntityTransfer.getPortletApplicationDefinitionId() + "' != '" + portletApplicationDefinition.getPortletApplicationDefinitionId() + "'");
        }
        
        this.portletApplicationEntityTransfer = portletApplicationEntityTransfer;
        this.portletApplicationDefinition = portletApplicationDefinition;
    }

    public PortletApplicationEntityId getPortletApplicationEntityId() {
        return this.portletApplicationEntityTransfer.getPortletApplicationEntityId();
    }

    public void setPortletApplicationEntityId(final PortletApplicationEntityId id) {
        this.portletApplicationEntityTransfer.setPortletApplicationEntityId(id);
    }

    public PortletApplicationDefinitionId getPortletApplicationDefinitionId() {
        return this.portletApplicationEntityTransfer.getPortletApplicationDefinitionId();
    }

    public String getVersion() {
        return this.portletApplicationDefinition.getVersion();
    }

    public Set< ICustomPortletMode> getCustomPortletModes() {
        return this.portletApplicationDefinition.getCustomPortletModes();
    }

    public Set<ICustomWindowState> getCustomWindowStates() {
        return this.portletApplicationDefinition.getCustomWindowStates();
    }

    public Set<IUserAttribute> getUserAttributes() {
        return this.portletApplicationDefinition.getUserAttributes();
    }

    public Set<ISecurityConstraint> getSecurityConstraints() {
        return this.portletApplicationDefinition.getSecurityConstraints();
    }
    
    public IPortletApplicationEntityTransfer getPortletApplicationEntityTransfer() {
        return this.portletApplicationEntityTransfer;
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (!(obj instanceof IPortletApplicationEntity)) {
            return false;
        }
        if (this == obj) {
            return true;
        }
        
        IPortletApplicationEntity other = (IPortletApplicationEntity)obj;
        return new EqualsBuilder()
            .append(this.portletApplicationEntityTransfer, other.getPortletApplicationEntityTransfer())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return this.portletApplicationEntityTransfer.hashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
        .append("portletApplicationEntityTransfer", this.portletApplicationEntityTransfer)
        .toString();
    }
}
