/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.factory;

import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinitionControl;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.IPortletDefinitionControl;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.deployment.IPortletApplicationDeployment;
import org.jasig.portal.portlet.deployment.IPortletDeployment;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.transfer.IPortletApplicationDefinitionTransfer;
import org.jasig.portal.portlet.transfer.IPortletDefinitionTransfer;

/**
 *  An extension of <code>IPortletDefinitionFactory</code> that returns control objects.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletDefinitionControlFactory extends IPortletDefinitionFactory {
    /**
     * Creates the portlet definition instance.
     * @param portletDefinitionTransfer the transfer object
     * @return the portlet definiton
     * @throws DomainObjectCreationException 
     */
    public IPortletDefinitionControl createPortletDefinitionControl(IPortletDefinitionTransfer portletDefinitionTransfer) throws DomainObjectCreationException;
    
    /**
     * Create a portlet definition control with empty preferences.
     * @param portletDeployment
     * @param portletApplicationDefinition
     * @param definitionId
     * @return portlet definition control
     * @throws DomainObjectCreationException 
     */
    public IPortletDefinitionControl createPortletDefinitionControl(IPortletDeployment portletDeployment, IPortletApplicationDefinition portletApplicationDefinition, PortletDefinitionId definitionId) throws DomainObjectCreationException;
 
    /**
     * Creates the portlet application definition instance.
     * @param portletApplicationDefinitionTransfer the transfer object
     * @return the portlet application definition
     * @throws DomainObjectCreationException 
     */
    public IPortletApplicationDefinitionControl createPortletApplicationDefinitionControl(IPortletApplicationDefinitionTransfer portletApplicationDefinitionTransfer) throws DomainObjectCreationException;
    
    /**
     * Creates the portlet application definition instance.
     * @param portletApplicationDeployment The deployment to base the definition on.
     * @param portletApplicationDefinitionId The id for the definition
     * @return the portlet application definition
     * @throws DomainObjectCreationException 
     */
    public IPortletApplicationDefinitionControl createPortletApplicationDefinitionControl(IPortletApplicationDeployment portletApplicationDeployment, PortletApplicationDefinitionId portletApplicationDefinitionId) throws DomainObjectCreationException;
}