/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.factory;

import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.entity.IPortletApplicationEntity;
import org.jasig.portal.portlet.entity.IPortletApplicationEntityControl;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.IPortletEntityControl;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.transfer.IPortletApplicationEntityTransfer;
import org.jasig.portal.portlet.transfer.IPortletEntityTransfer;
import org.jasig.portal.security.UserId;

/**
 *  An extension of <code>IPortletEntityFactory</code> returning control objects.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletEntityControlFactory extends IPortletEntityFactory {
    /**
     * Creates the portlet entity instance.
     * @param userId the user identifier
     * @param portletEntityTransfer the portlet entity transfer object
     * @return the portlet entity
     * @throws DomainObjectCreationException 
     */
    public IPortletEntityControl createPortletEntityControl(UserId userId, IPortletEntityTransfer portletEntityTransfer) throws DomainObjectCreationException;
    
    /**
     * Creates new portlet entity with empty preferences
     * @param portletDefinition
     * @param portletApplicationEntity
     * @param portletEntityId
     * @param userId
     * @return the portlet entity control
     * @throws DomainObjectCreationException 
     */
    public IPortletEntityControl createPortletEntityControl(UserId userId, IPortletDefinition portletDefinition, IPortletApplicationEntity portletApplicationEntity, PortletEntityId portletEntityId) throws DomainObjectCreationException;
        
    
    /**
     * Creates the portlet application entity instance.
     * @param portletApplicationEntityTransfer the transfer object
     * @return the portlet application entity
     * @throws DomainObjectCreationException 
     */
    public IPortletApplicationEntityControl createPortletApplicationEntityControl(IPortletApplicationEntityTransfer portletApplicationEntityTransfer) throws DomainObjectCreationException;

    /**
     * Create a portlet application entity
     * @param portletApplicationDefinition
     * @param portletApplicationEntityId
     * @return the portlet application entity control
     * @throws DomainObjectCreationException 
     */
    public IPortletApplicationEntityControl createPortletApplicationEntityControl(IPortletApplicationDefinition portletApplicationDefinition, PortletApplicationEntityId portletApplicationEntityId) throws DomainObjectCreationException;
}
