/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.factory;

import org.jasig.gap.services.caching.ICache;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.caching.NamespacingCacheWrapper;
import org.jasig.portal.portlet.dao.IPortletApplicationEntityDao;
import org.jasig.portal.portlet.registry.general.IPortletApplicationEntityRegistry;
import org.jasig.portal.portlet.registry.general.PortletApplicationEntityRegistryImpl;
import org.jasig.portal.security.UserId;

/**
 * The portlet entity registry factory implementation. The {@link ICache} implementation
 * used for the <code>cache</code> property of this object must allow non-serializable 
 * values. The cache wired to the <code>portletApplicationEntityCache</code> property can
 * require serializable values.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class PortletApplicationEntityRegistryFactoryImpl implements IPortletApplicationEntityRegistryFactory {
    private IPortletApplicationEntityDao portletApplicationEntityDao;
    private IPortletEntityFactory portletEntityFactory;
    private ISimpleCacheProviderFacade portletApplicationEntityCache;
    private ISimpleCacheProviderFacade portletApplicationEntityRegistryCache;

    
    /**
     * @return Returns the portletApplicationEntityDao.
     */
    public IPortletApplicationEntityDao getPortletApplicationEntityDao() {
        return this.portletApplicationEntityDao;
    }
    /**
     * @param portletApplicationEntityDao The portletApplicationEntityDao to set.
     */
    public void setPortletApplicationEntityDao(IPortletApplicationEntityDao portletApplicationEntityDao) {
        this.portletApplicationEntityDao = portletApplicationEntityDao;
    }

    /**
     * @return Returns the portletEntityFactory.
     */
    public IPortletEntityFactory getPortletEntityFactory() {
        return this.portletEntityFactory;
    }
    /**
     * @param portletEntityFactory The portletEntityFactory to set.
     */
    public void setPortletEntityFactory(IPortletEntityFactory portletEntityFactory) {
        this.portletEntityFactory = portletEntityFactory;
    }
    
    /**
     * @return Returns the portletApplicationEntityCache.
     */
    public ISimpleCacheProviderFacade getPortletApplicationEntityCache() {
        return this.portletApplicationEntityCache;
    }
    /**
     * @param portletApplicationEntityCache The portletApplicationEntityCache to set.
     */
    public void setPortletApplicationEntityCache(ISimpleCacheProviderFacade portletApplicationEntityCache) {
        this.portletApplicationEntityCache = portletApplicationEntityCache;
    }
    
    /**
     * @return the portletApplicationEntityRegistryCache
     */
    public ISimpleCacheProviderFacade getPortletApplicationEntityRegistryCache() {
        return this.portletApplicationEntityRegistryCache;
    }
    /**
     * @param portletApplicationEntityRegistryCache the portletApplicationEntityRegistryCache to set
     */
    public void setPortletApplicationEntityRegistryCache(ISimpleCacheProviderFacade portletApplicationEntityRegistryCache) {
        this.portletApplicationEntityRegistryCache = portletApplicationEntityRegistryCache;
    }

    public synchronized IPortletApplicationEntityRegistry getPortletApplicationEntityRegistry(UserId userId) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        
        IPortletApplicationEntityRegistry portAppEntReg = null;
        
        if (this.portletApplicationEntityRegistryCache != null) {
            portAppEntReg = (IPortletApplicationEntityRegistry)this.portletApplicationEntityRegistryCache.getFromCache(userId);
            
            if (portAppEntReg == null) {
                portAppEntReg = this.createPortletApplicationEntityRegistry(userId);
                this.portletApplicationEntityRegistryCache.putInCache(userId, portAppEntReg);
            }
        }
        else {
            portAppEntReg = this.createPortletApplicationEntityRegistry(userId);
        }
        
        return portAppEntReg;
    }

    private IPortletApplicationEntityRegistry createPortletApplicationEntityRegistry(UserId userId) {
        
        final NamespacingCacheWrapper cacheWrapper = new NamespacingCacheWrapper();
        cacheWrapper.setNamespace(userId.toString() + "_");
        cacheWrapper.setWrappedCache(this.portletApplicationEntityCache);
        
        final PortletApplicationEntityRegistryImpl newRegistry = new PortletApplicationEntityRegistryImpl(userId, this.portletApplicationEntityDao, this.portletEntityFactory, cacheWrapper);
        
        return newRegistry;
    }
}