/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.factory;

import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinitionControl;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.IPortletDefinitionControl;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionImpl;
import org.jasig.portal.portlet.definition.PortletDefinitionImpl;
import org.jasig.portal.portlet.deployment.IPortletApplicationDeployment;
import org.jasig.portal.portlet.deployment.IPortletDeployment;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.registry.general.IPortletApplicationDeploymentRegistry;
import org.jasig.portal.portlet.registry.general.IPortletDeploymentRegistry;
import org.jasig.portal.portlet.transfer.IPortletApplicationDefinitionTransfer;
import org.jasig.portal.portlet.transfer.IPortletDefinitionTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletApplicationDefinitionTransferImpl;
import org.jasig.portal.portlet.transfer.impl.PortletDefinitionTransferImpl;
import org.springframework.beans.factory.annotation.Required;

/**
 * The portlet definition factory implementation.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public class PortletDefinitionFactoryImpl implements IPortletDefinitionControlFactory {
    private IPortletApplicationDeploymentRegistry portletApplicationDeploymentRegistry;
    private IPortletDeploymentRegistry portletDeploymentRegistry;
    

    /**
     * @return the portletApplicationDeploymentRegistry
     */
    public IPortletApplicationDeploymentRegistry getPortletApplicationDeploymentRegistry() {
        return this.portletApplicationDeploymentRegistry;
    }

    /**
     * @param portletApplicationDeploymentRegistry the portletApplicationDeploymentRegistry to set
     */
    @Required
    public void setPortletApplicationDeploymentRegistry(IPortletApplicationDeploymentRegistry portletApplicationDeploymentRegistry) {
        this.portletApplicationDeploymentRegistry = portletApplicationDeploymentRegistry;
    }

    /**
     * @return the portletDeploymentRegistry
     */
    public IPortletDeploymentRegistry getPortletDeploymentRegistry() {
        return this.portletDeploymentRegistry;
    }

    /**
     * @param portletDeploymentRegistry the portletDeploymentRegistry to set
     */
    @Required
    public void setPortletDeploymentRegistry(IPortletDeploymentRegistry portletDeploymentRegistry) {
        this.portletDeploymentRegistry = portletDeploymentRegistry;
    }

    public IPortletDefinition createPortletDefinition(IPortletDefinitionTransfer portletDefinitionTransfer) throws DomainObjectCreationException {
       return this.createPortletDefinitionControl(portletDefinitionTransfer);
    }
 
    public IPortletDefinitionControl createPortletDefinitionControl(IPortletDefinitionTransfer portletDefinitionTransfer) throws DomainObjectCreationException {
        if (portletDefinitionTransfer == null) {
            throw new IllegalArgumentException("portletDefinitionTransfer may not be null");
        }
        
        final IPortletDeployment portDep = this.portletDeploymentRegistry.getPortletDeployment(portletDefinitionTransfer.getPortletDeploymentId());
        if (portDep == null) {
            throw new DomainObjectCreationException(IPortletDefinitionControl.class, IPortletDeployment.class, portletDefinitionTransfer.getPortletDeploymentId());
        }
        
        return new PortletDefinitionImpl(portletDefinitionTransfer, portDep);
    }
    
    public IPortletDefinitionControl createPortletDefinitionControl(IPortletDeployment portletDeployment, IPortletApplicationDefinition portletApplicationDefinition, PortletDefinitionId portletDefinitionId) throws DomainObjectCreationException {
        if (portletDeployment == null) {
            throw new IllegalArgumentException("portletDeployment may not be null");
        }
        if (portletApplicationDefinition == null) {
            throw new IllegalArgumentException("portletApplicationDefinition may not be null");
        }
        if (portletDefinitionId == null) {
            throw new IllegalArgumentException("portletDefinitionId may not be null");
        }
        
        final IPortletDefinitionTransfer portDefTrans = new PortletDefinitionTransferImpl();
        portDefTrans.setPortletDefinitionId(portletDefinitionId);
        portDefTrans.setPortletDeploymentId(portletDeployment.getPortletDeploymentId());
        portDefTrans.setPortletApplicationDefinitionId(portletApplicationDefinition.getPortletApplicationDefinitionId());

        return new PortletDefinitionImpl(portDefTrans, portletDeployment);
    }
    
    
    
    
    
    public IPortletApplicationDefinition createPortletApplicationDefinition(IPortletApplicationDefinitionTransfer portletApplicationDefinitionTransfer) throws DomainObjectCreationException {
        return this.createPortletApplicationDefinitionControl(portletApplicationDefinitionTransfer);
    }
 
    public IPortletApplicationDefinitionControl createPortletApplicationDefinitionControl(IPortletApplicationDefinitionTransfer portletApplicationDefinitionTransfer) throws DomainObjectCreationException {
        if (portletApplicationDefinitionTransfer == null) {
            throw new IllegalArgumentException("portletApplicationDefinitionTransfer may not be null");
        }
        
        final IPortletApplicationDeployment portAppDep = this.portletApplicationDeploymentRegistry.getPortletApplicationDeployment(portletApplicationDefinitionTransfer.getPortletApplicationDeploymentId());
        if (portAppDep == null) {
            throw new DomainObjectCreationException(IPortletApplicationDeployment.class, IPortletApplicationDeployment.class, portletApplicationDefinitionTransfer.getPortletApplicationDeploymentId());
        }

        return new PortletApplicationDefinitionImpl(portletApplicationDefinitionTransfer, portAppDep);
    }
    
    public IPortletApplicationDefinitionControl createPortletApplicationDefinitionControl(IPortletApplicationDeployment portletApplicationDeployment, PortletApplicationDefinitionId portletApplicationDefinitionId) throws DomainObjectCreationException {
        if (portletApplicationDeployment == null) {
            throw new IllegalArgumentException("portletApplicationDeployment may not be null");
        }
        if (portletApplicationDefinitionId == null) {
            throw new IllegalArgumentException("portletApplicationDefinitionId may not be null");
        }
        
        final IPortletApplicationDefinitionTransfer portAppDefTrans = new PortletApplicationDefinitionTransferImpl();
        portAppDefTrans.setPortletApplicationDefinitionId(portletApplicationDefinitionId);
        portAppDefTrans.setPortletApplicationDeploymentId(portletApplicationDeployment.getPortletApplicationDeploymentId());
        
        return new PortletApplicationDefinitionImpl(portAppDefTrans, portletApplicationDeployment);
    }
}
