/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.factory;

import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.entity.IPortletApplicationEntity;
import org.jasig.portal.portlet.entity.IPortletApplicationEntityControl;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.IPortletEntity;
import org.jasig.portal.portlet.entity.IPortletEntityControl;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityImpl;
import org.jasig.portal.portlet.entity.PortletEntityImpl;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.registry.general.IPortletApplicationDefinitionRegistry;
import org.jasig.portal.portlet.registry.general.IPortletDefinitionRegistry;
import org.jasig.portal.portlet.registry.general.IPortletWindowRegistry;
import org.jasig.portal.portlet.transfer.IPortletApplicationEntityTransfer;
import org.jasig.portal.portlet.transfer.IPortletEntityTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletApplicationEntityTransferImpl;
import org.jasig.portal.portlet.transfer.impl.PortletEntityTransferImpl;
import org.jasig.portal.security.UserId;
import org.springframework.beans.factory.annotation.Required;


/**
 * The portlet entity factory implementation.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public class PortletEntityFactoryImpl implements IPortletEntityControlFactory {
	
	private IPortletWindowRegistryFactory portletWindowRegistryFactory;
    private IPortletDefinitionRegistry portletDefinitionRegistry;
    private IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry;
    
    /**
     * @return the portletApplicationDefinitionRegistry
     */
    public IPortletApplicationDefinitionRegistry getPortletApplicationDefinitionRegistry() {
        return this.portletApplicationDefinitionRegistry;
    }

    /**
     * @param portletApplicationDefinitionRegistry the portletApplicationDefinitionRegistry to set
     */
    @Required
    public void setPortletApplicationDefinitionRegistry(IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry) {
        this.portletApplicationDefinitionRegistry = portletApplicationDefinitionRegistry;
    }

    /**
     * @return the portletDefinitionRegistry
     */
    public IPortletDefinitionRegistry getPortletDefinitionRegistry() {
        return this.portletDefinitionRegistry;
    }

    /**
     * @param portletDefinitionRegistry the portletDefinitionRegistry to set
     */
    @Required
    public void setPortletDefinitionRegistry(IPortletDefinitionRegistry portletDefinitionRegistry) {
        this.portletDefinitionRegistry = portletDefinitionRegistry;
    }

    /**
     * @return the portletWindowRegistryFactory
     */
    public IPortletWindowRegistryFactory getPortletWindowRegistryFactory() {
        return this.portletWindowRegistryFactory;
    }

    /**
     * @param portletWindowRegistryFactory the portletWindowRegistryFactory to set
     */
    @Required
    public void setPortletWindowRegistryFactory(IPortletWindowRegistryFactory portletWindowRegistryFactory) {
        this.portletWindowRegistryFactory = portletWindowRegistryFactory;
    }
    
    

    public IPortletEntity createPortletEntity(UserId userId,IPortletEntityTransfer portletEntityTransfer) throws DomainObjectCreationException {
        return this.createPortletEntityControl(userId,portletEntityTransfer);
    }
    
    public IPortletEntityControl createPortletEntityControl(UserId userId, IPortletEntityTransfer portEntTrans) throws DomainObjectCreationException {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        if (portEntTrans == null) {
            throw new IllegalArgumentException("portEntTrans may not be null");
        }
        
        
        final IPortletDefinition portDef = this.portletDefinitionRegistry.getPortletDefinition(portEntTrans.getPortletDefinitionId());
        if (portDef == null) {
            throw new DomainObjectCreationException(IPortletEntityControl.class, IPortletDefinition.class, portEntTrans.getPortletDefinitionId());
        }
        
        return this.createPortletEntityControl(userId, portEntTrans, portDef);
    }
    
    public IPortletEntityControl createPortletEntityControl(UserId userId, IPortletDefinition portDef, IPortletApplicationEntity portAppEnt, PortletEntityId portEntId) throws DomainObjectCreationException {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        if (portDef == null) {
            throw new IllegalArgumentException("portDef may not be null");
        }
        if (portAppEnt == null) {
            throw new IllegalArgumentException("portAppEnt may not be null");
        }

        
        final IPortletEntityTransfer portEntTrans = new PortletEntityTransferImpl();
        portEntTrans.setPortletEntityId(portEntId);
        portEntTrans.setPortletDefinitionId(portDef.getPortletDefinitionId());
        portEntTrans.setPortletApplicationEntityId(portAppEnt.getPortletApplicationEntityId());

        return this.createPortletEntityControl(userId, portEntTrans, portDef);
    }
    
    private IPortletEntityControl createPortletEntityControl(UserId userId, IPortletEntityTransfer portEntTrans, IPortletDefinition portDef) {
        final IPortletWindowRegistry portWinReg = this.portletWindowRegistryFactory.getPortletWindowRegistry(userId);
        
        return new PortletEntityImpl(portEntTrans, portDef, portWinReg);
    }
    
    
    
    public IPortletApplicationEntity createPortletApplicationEntity(IPortletApplicationEntityTransfer portAppEntTrans) throws DomainObjectCreationException {
        return this.createPortletApplicationEntityControl(portAppEntTrans);
    }
    
    public IPortletApplicationEntityControl createPortletApplicationEntityControl(IPortletApplicationEntityTransfer portAppEntTrans) throws DomainObjectCreationException {
        if (portAppEntTrans == null) {
            throw new IllegalArgumentException("portAppEntTrans may not be null");
        }
        
        
        final IPortletApplicationDefinition portAppDef = this.portletApplicationDefinitionRegistry.getPortletApplicationDefinition(portAppEntTrans.getPortletApplicationDefinitionId());
        if (portAppDef == null) {
            throw new DomainObjectCreationException(IPortletApplicationEntityControl.class, IPortletApplicationDefinition.class, portAppEntTrans.getPortletApplicationDefinitionId());
        }
        
        return new PortletApplicationEntityImpl(portAppEntTrans, portAppDef);
    }

    public IPortletApplicationEntityControl createPortletApplicationEntityControl(IPortletApplicationDefinition portAppDef, PortletApplicationEntityId portAppEntId) throws DomainObjectCreationException {
        if (portAppDef == null) {
            throw new IllegalArgumentException("portAppDef may not be null");
        }
        if (portAppEntId == null) {
            throw new IllegalArgumentException("portAppEntId may not be null");
        }
        
        
        final IPortletApplicationEntityTransfer portAppEntTrans = new PortletApplicationEntityTransferImpl();
        portAppEntTrans.setPortletApplicationEntityId(portAppEntId);
        portAppEntTrans.setPortletApplicationDefinitionId(portAppDef.getPortletApplicationDefinitionId());
        //TODO does portAppEntTrans.setUserId need to be called here?
        
        return new PortletApplicationEntityImpl(portAppEntTrans, portAppDef);
    }
}
