/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.factory;

import org.jasig.gap.services.caching.ICache;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.caching.NamespacingCacheWrapper;
import org.jasig.portal.portlet.dao.IPortletEntityDao;
import org.jasig.portal.portlet.registry.general.IPortletEntityRegistry;
import org.jasig.portal.portlet.registry.general.PortletEntityRegistryImpl;
import org.jasig.portal.security.UserId;

/**
 * The portlet entity registry factory implementation. The {@link ICache} implementation
 * used for the <code>cache</code> property of this object must allow non-serializable 
 * values. The cache wired to the <code>portletEntityCache</code> property can require
 * serializable values.
 * 
 * @author Michael Ivanov, mvi@immagic.com
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class PortletEntityRegistryFactoryImpl implements IPortletEntityRegistryFactory {
    private IPortletEntityDao portletEntityDao;
    private IPortletEntityFactory portletEntityFactory;
    private ISimpleCacheProviderFacade portletEntityCache;
    private ISimpleCacheProviderFacade portletEntityRegistryCache;

    /**
     * @return Returns the portletEntityDao.
     */
    public IPortletEntityDao getPortletEntityDao() {
        return this.portletEntityDao;
    }
    /**
     * @param portletEntityDao The portletEntityDao to set.
     */
    public void setPortletEntityDao(IPortletEntityDao portletEntityDao) {
        this.portletEntityDao = portletEntityDao;
    }

    /**
     * @return Returns the portletEntityFactory.
     */
    public IPortletEntityFactory getPortletEntityFactory() {
        return this.portletEntityFactory;
    }
    /**
     * @param portletEntityFactory The portletEntityFactory to set.
     */
    public void setPortletEntityFactory(IPortletEntityFactory portletEntityFactory) {
        this.portletEntityFactory = portletEntityFactory;
    }
    
    /**
     * @return Returns the portletEntityCache.
     */
    public ISimpleCacheProviderFacade getPortletEntityCache() {
        return this.portletEntityCache;
    }
    
    /**
     * @param portletEntityCache The portletEntityCache to set.
     */
    public void setPortletEntityCache(ISimpleCacheProviderFacade portletEntityCache) {
        this.portletEntityCache = portletEntityCache;
    }
    
    /**
     * @return the portletEntityRegistryCache
     */
    public ISimpleCacheProviderFacade getPortletEntityRegistryCache() {
        return this.portletEntityRegistryCache;
    }
    /**
     * @param portletEntityRegistryCache the portletEntityRegistryCache to set
     */
    public void setPortletEntityRegistryCache(ISimpleCacheProviderFacade portletEntityRegistryCache) {
        this.portletEntityRegistryCache = portletEntityRegistryCache;
    }
    
    public synchronized IPortletEntityRegistry getPortletEntityRegistry(UserId userId) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        
        IPortletEntityRegistry portEntReg = null;
        
        if (this.portletEntityRegistryCache != null) {
            portEntReg = (IPortletEntityRegistry)this.portletEntityRegistryCache.getFromCache(userId);
            
            if (portEntReg == null) {
                portEntReg = this.createPortletEntityRegistry(userId);
                this.portletEntityRegistryCache.putInCache(userId, portEntReg);
            }
        }
        else {
            portEntReg = this.createPortletEntityRegistry(userId);
        }
        
        return portEntReg;
    }

    private IPortletEntityRegistry createPortletEntityRegistry(UserId userId) {
        final NamespacingCacheWrapper cacheWrapper = new NamespacingCacheWrapper();
        cacheWrapper.setNamespace(userId.toString() + "_");
        cacheWrapper.setWrappedCache(this.portletEntityCache);
        
        PortletEntityRegistryImpl newRegistry = new PortletEntityRegistryImpl(userId, this.portletEntityDao, this.portletEntityFactory, cacheWrapper);
        
        return newRegistry;
    }
}