/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.factory;

import org.jasig.portal.portlet.entity.IPortletEntity;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.registry.general.IPortletEntityRegistry;
import org.jasig.portal.portlet.transfer.IPortletWindowTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletWindowTransferImpl;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlet.window.IPortletWindowControl;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.portlet.window.PortletWindowImpl;
import org.jasig.portal.security.UserId;

/**
 * The portlet window factory implementation.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public class PortletWindowFactoryImpl implements IPortletWindowControlFactory {
    private IPortletEntityRegistryFactory portletEntityRegistryFactory;
    
    /**
     * @return Returns the portletEntityRegistryFactory.
     */
    public IPortletEntityRegistryFactory getPortletEntityRegistryFactory() {
        return this.portletEntityRegistryFactory;
    }
    /**
     * @param portletEntityRegistryFactory The portletEntityRegistryFactory to set.
     */
    public void setPortletEntityRegistryFactory(IPortletEntityRegistryFactory portletEntityRegistryFactory) {
        this.portletEntityRegistryFactory = portletEntityRegistryFactory;
    }
    
    public IPortletWindow createPortletWindow(UserId userId, IPortletWindowTransfer portWinTrans) throws DomainObjectCreationException {
        return this.createPortletWindowControl(userId, portWinTrans);
    }

    public IPortletWindowControl createPortletWindowControl(UserId userId, IPortletWindowTransfer portWinTrans) throws DomainObjectCreationException {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        if (portWinTrans == null) {
            throw new IllegalArgumentException("portWinTrans may not be null");
        }
        
        
        final IPortletEntityRegistry portEntReg = this.portletEntityRegistryFactory.getPortletEntityRegistry(userId);
        final IPortletEntity portEnt = portEntReg.getPortletEntity(portWinTrans.getPortletEntityId());
        if (portEnt == null) {
            throw new DomainObjectCreationException(IPortletWindowControl.class, IPortletEntity.class, portWinTrans.getPortletEntityId());
        }

        return new PortletWindowImpl(portWinTrans, portEnt);
    }

    public IPortletWindow createPortletWindow(IPortletEntity portEnt, PortletWindowId portWinId) throws DomainObjectCreationException {
        return this.createPortletWindowControl(portEnt, portWinId);
    }

    public IPortletWindowControl createPortletWindowControl(IPortletEntity portEnt, PortletWindowId portWinId) throws DomainObjectCreationException {
        if (portEnt == null) {
            throw new IllegalArgumentException("portEnt may not be null");
        }
        if (portWinId == null) {
            throw new IllegalArgumentException("portWinId may not be null");
        }
        
        
        final IPortletWindowTransfer portWinTrans = new PortletWindowTransferImpl();
        portWinTrans.setPortletWindowId(portWinId);
        portWinTrans.setPortletEntityId(portEnt.getPortletEntityId());
        
        return new PortletWindowImpl(portWinTrans, portEnt);
    }
}
