/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.factory;

import org.jasig.gap.services.caching.ICache;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.caching.NamespacingCacheWrapper;
import org.jasig.portal.portlet.dao.IPortletWindowDao;
import org.jasig.portal.portlet.registry.general.IPortletWindowRegistry;
import org.jasig.portal.portlet.registry.general.PortletWindowRegistryImpl;
import org.jasig.portal.security.UserId;

/**
 * The portlet window registry factory implementation. The {@link ICache} implementation
 * used for the <code>cache</code> property of this object must allow non-serializable 
 * values. The cache wired to the <code>portletWindowCache</code> property can require
 * serializable values.
 * 
 * @author Michael Ivanov, mvi@immagic.com
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class PortletWindowRegistryFactoryImpl implements IPortletWindowRegistryFactory {
    private IPortletWindowDao portletWindowDao;
    private IPortletWindowFactory portletWindowFactory;
    private ISimpleCacheProviderFacade portletWindowCache;
    private ISimpleCacheProviderFacade portletWindowRegistryCache;

    /**
     * @return Returns the portletWindowDao.
     */
    public IPortletWindowDao getPortletWindowDao() {
        return this.portletWindowDao;
    }
    /**
     * @param portletWindowDao The portletWindowDao to set.
     */
    public void setPortletWindowDao(IPortletWindowDao portletWindowDao) {
        this.portletWindowDao = portletWindowDao;
    }

    /**
     * @return Returns the portletWindowFactory.
     */
    public IPortletWindowFactory getPortletWindowFactory() {
        return this.portletWindowFactory;
    }
    /**
     * @param portletWindowFactory The portletWindowFactory to set.
     */
    public void setPortletWindowFactory(IPortletWindowFactory portletWindowFactory) {
        this.portletWindowFactory = portletWindowFactory;
    }
    
    /**
     * @return Returns the portletWindowCache.
     */
    public ISimpleCacheProviderFacade getPortletWindowCache() {
        return this.portletWindowCache;
    }
    
    /**
     * @param portletWindowCache The portletWindowCache to set.
     */
    public void setPortletWindowCache(ISimpleCacheProviderFacade portletWindowCache) {
        this.portletWindowCache = portletWindowCache;
    }
    
    /**
     * @return the portletWindowRegistryCache
     */
    public ISimpleCacheProviderFacade getPortletWindowRegistryCache() {
        return this.portletWindowRegistryCache;
    }
    /**
     * @param portletWindowRegistryCache the portletWindowRegistryCache to set
     */
    public void setPortletWindowRegistryCache(ISimpleCacheProviderFacade portletWindowRegistryCache) {
        this.portletWindowRegistryCache = portletWindowRegistryCache;
    }
    
    
    public synchronized IPortletWindowRegistry getPortletWindowRegistry(UserId userId) {
        if (userId == null) {
            throw new IllegalArgumentException("userId may not be null");
        }
        
        IPortletWindowRegistry portWinReg = null;
        
        if (this.portletWindowRegistryCache != null) {
            portWinReg = (IPortletWindowRegistry)this.portletWindowRegistryCache.getFromCache(userId);
            
            if (portWinReg == null) {
                portWinReg = this.createPortletWindowRegistry(userId);
                this.portletWindowRegistryCache.putInCache(userId, portWinReg);
            }
        }
        else {
            portWinReg = this.createPortletWindowRegistry(userId);
        }
        
        return portWinReg;
    }

    private IPortletWindowRegistry createPortletWindowRegistry(UserId userId) {
        
        final NamespacingCacheWrapper cacheWrapper = new NamespacingCacheWrapper();
        cacheWrapper.setNamespace(userId.toString() + "_");
        cacheWrapper.setWrappedCache(this.portletWindowCache);
        
        PortletWindowRegistryImpl newRegistry = new PortletWindowRegistryImpl(userId, this.portletWindowDao, this.portletWindowFactory, cacheWrapper);
        
        return newRegistry;
    }
}