/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry;

import org.jasig.portal.core.ObjectId;
import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.core.exception.PortalException;


/**
 * This exception is used by domain object implementations to signal
 * that during initialization objects they depend on are not available.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class DomainObjectCreationException extends PortalException {
    private final Class domainClass;
    private final Class dependencyClass;
    private final ObjectId dependencyId;
    private final String dependencyKey;
    private final String msg;
    
    public DomainObjectCreationException(Class domainClass, Class dependencyClass, ObjectId dependencyId) {
        this(domainClass, dependencyClass, dependencyId, null);
    }
    
    public DomainObjectCreationException(Class domainClass, Class dependencyClass, ObjectId dependencyId, Throwable cause) {
        this(domainClass, dependencyClass, dependencyId, null, cause);
    }
    
    public DomainObjectCreationException(Class domainClass, Class dependencyClass, String dependencyKey) {
        this(domainClass, dependencyClass, dependencyKey, null);
    }
    
    public DomainObjectCreationException(Class domainClass, Class dependencyClass, String dependencyKey, Throwable cause) {
        this(domainClass, dependencyClass, null, dependencyKey, cause);
    }
    
    private DomainObjectCreationException(Class domainClass, Class dependencyClass, ObjectId dependencyId, String dependencyKey, Throwable cause) {
        super(cause);
        
        this.domainClass = domainClass;
        this.dependencyClass = dependencyClass;
        this.dependencyId = dependencyId;
        this.dependencyKey = dependencyKey;
        
        StringBuffer msgBuilder = new StringBuffer();
        msgBuilder.append("While creating a domain object of type '");
        msgBuilder.append(this.domainClass.toString());
        msgBuilder.append("' a dependency of type '");
        msgBuilder.append(this.dependencyClass.toString());
        msgBuilder.append("' with id '");

        if (this.dependencyId != null) {
            msgBuilder.append(this.dependencyId.toString());
        }
        else {
            msgBuilder.append(this.dependencyKey);
        }

        msgBuilder.append("' could not be found.");
        
        this.msg = msgBuilder.toString();
    }

    /**
     * @see java.lang.Throwable#getLocalizedMessage()
     */
    public String getLocalizedMessage() {
        return this.getMessage();
    }

    /**
     * @see java.lang.Throwable#getMessage()
     */
    public String getMessage() {
        return this.msg;
    }

    
    /**
     * @return Returns the dependencyClass.
     */
    public Class getDependencyClass() {
        return this.dependencyClass;
    }

    /**
     * @return Returns the dependencyId.
     */
    public ObjectId getDependencyId() {
        return this.dependencyId;
    }
    
    /**
     * @return Returns the dependencyKey.
     */
    public String getDependencyKey() {
        return this.dependencyKey;
    }
    
    /**
     * @return Returns the domainClass.
     */
    public Class getDomainClass() {
        return this.domainClass;
    }
    
    /**
     * @return Returns the msg.
     */
    public String getMsg() {
        return this.msg;
    }
}
