/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.registry.event;

import org.jasig.portal.core.ObjectId;

/**
 * A generic registry event listener  
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IRegistryEventListener<T extends ObjectId, U> {
    /**
     * Notifies that a new domain object has been created
     * 
     * Note: the event is fired after the creation
     * Note: registry blocks on event methods, so execution time of this method should be reduced to a minimum
     * @param event event object containing domain object id and the event source registry reference
     */
    void objectCreated(RegistryEvent<T,U> event);
    
    /**
     * Notifies that a new domain object has been loaded into the registry
     * 
     * Note: the event is fired after the load procedure completed
     * Note: registry blocks on event methods, so execution time of this method should be reduced to a minimum
     * @param event event object containing domain object id and the event source registry reference
     */
    void objectLoaded(RegistryEvent<T,U> event);
    
    /**
     * Notifies that a new domain object has been removed
     * 
     * Note: the event is fired after the removal procedure completed
     * Note: registry blocks on event methods, so execution time of this method should be reduced to a minimum
     * @param event event object containing domain object id and the event source registry reference
     */
    void objectRemoved(RegistryEvent<T,U> event);
    
    /**
     * Notifies that a new domain object has been updatd
     * 
     * Note: the event is fired after the update procedure completed
     * Note: registry blocks on event methods, so execution time of this method should be reduced to a minimum
     * @param event event object containing domain object id and the event source registry reference
     */
    void objectUpdated(RegistryEvent<T,U> event);
}
