/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.general;

import java.util.Set;

import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletApplicationDefinitionTransfer;

/**
 * The portlet definition registry interface.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public interface IPortletApplicationDefinitionRegistry {
	
	/**
     * Returns all portlet application definitions contained in the registry.
     * The return value cannot be NULL.
     * @return the portlet application definitions, Set is immutable.
     */
    public Set<IPortletApplicationDefinition> getPortletApplicationDefinitions();
    
    /**
     * Returns all portlet application definitions based on the specified portlet
     * application deployment.
     * 
     * @param portAppDepId The ID of the app deployment to get definitions for.
     * @return the portlet application definitions, Set is immutable.
     */
    public Set<IPortletApplicationDefinition> getPortletApplicationDefinitions(PortletApplicationDeploymentId portAppDepId);
    
    /**
     * Obtains a copy of the portlet application definition and registers it in the registry with a new ID.
     * @param portletApplicationDefinitionTransfer The portlet application transfer definition to copy the properties of
     * @return the new portlet application definition
     */
    public IPortletApplicationDefinition createPortletApplicationDefinition(IPortletApplicationDefinitionTransfer portletApplicationDefinition);
   
    /**
     * Returns the portlet application definition.
     * @param portletApplicationDefinitionId the object identifier
     * @return the portlet application definition
     */
    public IPortletApplicationDefinition getPortletApplicationDefinition(PortletApplicationDefinitionId portletApplicationDefinitionId);

    /**
     * Stores a portlet application definition in a datastore. EX: updating existing definitions.
     * @param portletApplicationDefinition the portlet application definition
     */
    public void updatePortletApplicationDefinition(IPortletApplicationDefinition portletApplicationDefinition);
 
    /**
     * Removes a portlet application definition from the registry. All IPortletApplicationEntity and
     * IPortletDefinition objects based on this application definition will also be removed. 
     * @param portletApplicationDefinitionId the portlet application definition identifier
     */
    public void removePortletApplicationDefinition(PortletApplicationDefinitionId portletApplicationDefinitionId);
}
