/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.general;

import java.util.Set;

import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.entity.IPortletApplicationEntity;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.registry.event.IPortletApplicationEntityEventListener;
import org.jasig.portal.portlet.registry.event.IRegistryEventSource;
import org.jasig.portal.portlet.transfer.IPortletApplicationEntityTransfer;

/**
 * The portlet entity registry interface.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */

public interface IPortletApplicationEntityRegistry extends IUserScopedRegistry, IRegistryEventSource<IPortletApplicationEntityEventListener> {
    /**
     * Returns all portlet application entities based on the specified portlet
     * application definition.
     * @param portletApplicationDefinitionId The application definition to get entities for, may not be null.
     * @return The porlet application entities, may not be null, may be empty.
     */
    public Set<IPortletApplicationEntity> getPortletApplicationEntities(PortletApplicationDefinitionId portletApplicationDefinitionId);
    
    /**
     * Returns the portlet application entities.
     * @param portletApplicationEntityId the object identifier, may not be null.
     * @return the portlet application entity may be null if no application entity exists for the id.
     */
    public IPortletApplicationEntity getPortletApplicationEntity(PortletApplicationEntityId portletApplicationEntityId);
	
    /**
     * Obtains a new portlet application entity and registers it in the registry.
     * @param portletApplicationEntity The portlet application entity to create the new IPortletApplicationEntity from.
     * @return the new portlet application entity, may not be null.
     */
    public IPortletApplicationEntity createPortletApplicationEntity(IPortletApplicationEntityTransfer portletApplicationEntityTransfer);
    
    /**
     * Removes a portlet entity from the registry.
     * @param portletApplicationEntityId the portlet application entity identifier, may not be null.
     */
    public void removePortletApplicationEntity(PortletApplicationEntityId portletApplicationEntityId);
}
