/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.general;

import java.util.Set;

import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.registry.event.IPortletDefinitionEventListener;
import org.jasig.portal.portlet.registry.event.IRegistryEventSource;

/**
 * The portlet definition registry interface.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public interface IPortletDefinitionRegistry extends IRegistryEventSource<IPortletDefinitionEventListener> {
	
    /**
     * Returns the portlet definitions contained in a portlet application definition.
     * @param portletApplicationDefinitionId the portlet application definition identifier
     * @return the portlet definitions for a particular portlet application definition
     */
    public Set<IPortletDefinition> getPortletDefinitions(PortletApplicationDefinitionId portletApplicationDefinitionId);
    
    /**
     * Obtains a copy of the portlet definition and registers it in the registry with a new ID.
     * @param portletDefinition The portlet definition to copy the properties of
     * @return the new portlet definition
     */
    public IPortletDefinition createPortletDefinition(IPortletDefinition portletDefinition);

    /**
     * Returns the portlet contained in the registry by id.
     * @param portletDefinitionId the portlet identifier
     * @return a <code>IPortletDefinition</code> object
     */
    public IPortletDefinition getPortletDefinition(PortletDefinitionId portletDefinitionId);
    
    /**
     * Returns the portlet contained in the registry by the functional name.
     * @param funcName the functional name
     * @return a <code>IPortletDefinition</code> object
     */
    public IPortletDefinition getPortletDefinitionByFunctionalName(String funcName);
    
    /**
     * Stores a portlet definition in a datastore.
     * @param portletDefinition the portlet definition
     */
    public void updatePortletDefinition(IPortletDefinition portletDefinition);
    
    /**
     * Removes a portlet definition from the registry. All IPortletEntity objects
     * based on this definition will also be removed.
     * @param portletDefinitionId the portlet definition identifier
     */
    public void removePortletDefinition(PortletDefinitionId portletDefinitionId);
}
