/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.general;

import java.util.Set;

import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.IPortletEntity;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.registry.event.IPortletEntityEventListener;
import org.jasig.portal.portlet.registry.event.IRegistryEventSource;
import org.jasig.portal.portlet.transfer.IPortletEntityTransfer;

/**
 * The portlet entity registry interface.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */

public interface IPortletEntityRegistry extends IUserScopedRegistry, IRegistryEventSource<IPortletEntityEventListener>{

    /**
     * Returns all portlet entities existing in the portal at the moment.
     * @return the portlet entities, may not be null, may empty.
     */
    public Set<IPortletEntity> getPortletEntities();
    
    /**
     * Returns all portlet entities based on the specified portlet application entity
     * @param portletApplicationEntityId The application entity to get entities for, may not be null.
     * @return The porlet entities, may not be null, may be empty.
     */
    public Set<IPortletEntity> getPortletEntities(PortletApplicationEntityId portletApplicationEntityId);
	
    /**
     * Returns all portlet entities based on the specified portlet application entity and
     * portlet definition.
     * @param portAppEntId The application entity to get entities for, may not be null.
     * @param portDefId The definition to get entities for, may not be null.
     * @return The porlet entities, may not be null, may be empty.
     */
    public Set<IPortletEntity> getPortletEntities(PortletApplicationEntityId portAppEntId, PortletDefinitionId portDefId);
    
    /**
     * Returns the portlet entity contained in the registry by id.
     * @param portletEntityId the portlet entity identifier, may not be null.
     * @return a <code>IPortletEntity</code> object, may be null if no entity exists for the id.
     */
    public IPortletEntity getPortletEntity(PortletEntityId portletEntityId);

    /**
     * Creates a new portlet entity and registers it in the registry.
     * @param portEntityTransfer The portlet entity transfer to base the new entity on.
     * @return the new porltet entity, may not be <code>null</code>.
     */
    public IPortletEntity createPortletEntity(IPortletEntityTransfer portEntityTransfer);
    
    /**
     * Removes a portlet entity from the registry.
     * @param portletEntityId the portlet entity, may not be null.
     */
    public void removePortletEntity(PortletEntityId portletEntityId);
    
    /**
     * Stores a portlet entity in a datastore
     * @param portletEntity the portlet entity, may not be null.
     */
    public void updatePortletEntity(IPortletEntity portletEntity);
    
    /**
     * Creates a clone from IPortletEntity instance. The new instance is persisted. Needed by WSRP producer.
     * @param portletEntityId ID of the portlet entity to be cloned, may not be null.
     * @return portletEntity the new portlet entity, may not be null.
     */
     public IPortletEntity clonePortletEntity(PortletEntityId portletEntityId);
}
