/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.general;

import java.util.Set;

import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.registry.event.IPortletWindowEventListener;
import org.jasig.portal.portlet.registry.event.IRegistryEventSource;
import org.jasig.portal.portlet.transfer.IPortletWindowTransfer;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * The portlet window registry interface.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */

public interface IPortletWindowRegistry extends IUserScopedRegistry, IRegistryEventSource<IPortletWindowEventListener> {
	
    /**
     * Returns all portlet windows derived from a particular
     * portlet entity.
     * Returns <code>null</code> if no portlet windows are found.
     * @param portletEntityId the portlet entity identifier
     * @return the portlet windows
     */
    public Set<IPortletWindow> getPortletWindows(PortletEntityId portletEntityId);

    /**
     * Obtains a new portlet window and registers it in the registry.
     * @param portWindowTransfer the IPortletWindowTransfer to clone for the new window to store in the registry
     * @return the new porltet window
     */
    public IPortletWindow createPortletWindow(IPortletWindowTransfer portWindowTransfer);
    
    /**
     * Returns the portlet window contained in the registry by id.
     * Returns <code>null</code> if no portlet window is found.
     * @param portletWindowId the portlet window identifier
     * @return a <code>IPortletWindow</code> object
     */
    public IPortletWindow getPortletWindow(PortletWindowId portletWindowId);
        
    /**
     * Removes a portlet window from the registry.
     * @param portletWindowId the portlet window
     */
    public void removePortletWindow(PortletWindowId portletWindowId);
    
    /**
     * Stores a portlet in a datastore.
     * @param portletWindow the portlet window
     */
    public void updatePortletWindow(IPortletWindow portletWindow);
    
}