/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.general;

import java.util.Set;

import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.dao.IPortletApplicationDeploymentDao;
import org.jasig.portal.portlet.deployment.IPortletApplicationDeployment;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.factory.IPortletDeploymentFactory;
import org.jasig.portal.portlet.registry.event.IPortletApplicationDeploymentEventListener;
import org.jasig.portal.portlet.transfer.IPortletApplicationDeploymentTransfer;
import org.springframework.beans.factory.annotation.Required;


/**
 * 
 * The portlet application definition registry implementation.
 * 
 * Template types to actual types
 * <ol>
 *   <li>GT_Stage - {@link IPortletApplicationDeployment}</li>
 *   <li>GT_StageId - {@link PortletApplicationDeploymentId}</li>
 *   <li>GT_StageTransfer - {@link IPortletApplicationDeploymentTransfer}</li>
 *   <li>GT_ParentStageId - {@link ObjectId}</li>
 *   <li>GT_ParentApplicationStageId - {@link ObjectId}</li>
 *   <li>GT_Registry - {@link IPortletApplicationDeploymentRegistry}</li> 
 *   <li>GT_RegistryListener - {@link IPortletApplicationDeploymentEventListener}</li>
 * </ol>
 * 
 * @author Eric Dalquist
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletApplicationDeploymentRegistryImpl
        extends GenericPortletRegistry<IPortletApplicationDeployment, PortletApplicationDeploymentId, IPortletApplicationDeploymentTransfer, ObjectId, ObjectId, IPortletApplicationDeploymentRegistry, IPortletApplicationDeploymentEventListener>
        implements IPortletApplicationDeploymentRegistry {

    private IPortletApplicationDeploymentDao portletApplicationDeploymentDao;
    private IPortletDeploymentFactory portletDeploymentFactory;

    /**
     * @param portletApplicationDeploymentDao The portletApplicationDeploymentDao to set.
     */
    @Required
    public void setPortletApplicationDeploymentDao(IPortletApplicationDeploymentDao portletApplicationDeploymentDao) {
        this.portletApplicationDeploymentDao = portletApplicationDeploymentDao;
    }

    /**
     * @param portletDeploymentFactory The portletDeploymentFactory to set.
     */
    @Required
    public void setPortletDeploymentFactory(IPortletDeploymentFactory portletDeploymentFactory) {
        this.portletDeploymentFactory = portletDeploymentFactory;
    }
    
    //********** IPortletApplicationDeploymentRegistry Methods **********//
    
    public Set<IPortletApplicationDeployment> getPortletApplicationDeployments() {
        return this.getAll();
    }
    
    public IPortletApplicationDeployment getPortletApplicationDeployment(PortletApplicationDeploymentId portletApplicationDeploymentId) {
        return this.get(portletApplicationDeploymentId);
    }

    
    /* =========== GenericPortletRegistry Methods ==================== */

    @Override
    IPortletApplicationDeploymentTransfer addTransferObjectToDAO(IPortletApplicationDeploymentTransfer transfer) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    IPortletApplicationDeployment createDomainObjectFromTransfer(IPortletApplicationDeploymentTransfer transfer) {
        return this.portletDeploymentFactory.createPortletApplicationDeployment(transfer);
    }

    @Override
    ObjectId getApplicationId(IPortletApplicationDeploymentTransfer transfer) {
        return null;
    }

    @Override
    PortletApplicationDeploymentId getId(IPortletApplicationDeploymentTransfer transfer) {
        return transfer.getPortletApplicationDeploymentId();
    }

    @Override
    ObjectId getParentId(IPortletApplicationDeploymentTransfer transfer) {
        return null;
    }

    @Override
    Set<String> getSecondaryKeys(IPortletApplicationDeploymentTransfer transfer) {
        return null;
    }

    @Override
    IPortletApplicationDeploymentTransfer getTransferObjectBySecondaryKeyFromDAO(String key) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    IPortletApplicationDeploymentTransfer getTransferObjectFromDAO(PortletApplicationDeploymentId id) {
        return this.portletApplicationDeploymentDao.getPortletApplicationDeployment(id);
    }

    @Override
    Set<IPortletApplicationDeploymentTransfer> getTransferObjectsForApplicationIdAndParentIdFromDAO(ObjectId applicationId, ObjectId parentId) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    Set<IPortletApplicationDeploymentTransfer> getTransferObjectsForApplicationIdFromDAO(ObjectId applicationId) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    Set<IPortletApplicationDeploymentTransfer> getTransferObjectsForParentIdFromDAO(ObjectId parentId) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    Set<IPortletApplicationDeploymentTransfer> getTransferObjectsFromDAO() {
        return this.portletApplicationDeploymentDao.getPortletApplicationDeployments();
    }

    @Override
    void removeObjectFromDAO(PortletApplicationDeploymentId id) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }

    @Override
    void updateObjectInDAO(IPortletApplicationDeploymentTransfer transfer) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }
}
