/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.portlet.registry.general;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.jasig.gap.groups.GroupsException;
import org.jasig.gap.groups.IEntitySearcher;
import org.jasig.gap.groups.services.local.JdbcGroupDao;
import org.jasig.gap.services.entitytypes.IEntityIdentifier;
import org.jasig.gap.services.entitytypes.impl.EntityIdentifier;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.PortletDefinitionId;

public class PortletDefinitionSearcher implements IEntitySearcher {

    private IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry;

    private IPortletDefinitionRegistry portletDefinitionRegistry;

    public PortletDefinitionSearcher(
        IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry,
        IPortletDefinitionRegistry portletDefinitionRegistry) {
        this.portletApplicationDefinitionRegistry = portletApplicationDefinitionRegistry;
        this.portletDefinitionRegistry = portletDefinitionRegistry;
    }

    public IEntityIdentifier[] searchForEntities(String query, int method,
        Class type) throws GroupsException {

        Set<IEntityIdentifier> result = new HashSet<IEntityIdentifier>();
        String upperQuery = query.toUpperCase();

        if (PortletDefinitionId.class.equals(type)) {
            IPortletDefinition pd;
            Iterator itr = getPortletDefinitions().iterator();
            String upperPortletName;
            while (itr.hasNext()) {
                pd = (IPortletDefinition) itr.next();
                upperPortletName = pd.getPortletName().toUpperCase();
                
                switch (method) {
                case JdbcGroupDao.IS:
                    if (upperPortletName.equals(upperQuery)) {
                        result.add(createPortletEntityIdentifier(pd));
                    }
                    break;
                case JdbcGroupDao.STARTS_WITH:
                    if (upperPortletName.startsWith(upperQuery)) {
                        result.add(createPortletEntityIdentifier(pd));
                    }
                    break;
                case JdbcGroupDao.ENDS_WITH:
                    if (upperPortletName.endsWith(upperQuery)) {
                        result.add(createPortletEntityIdentifier(pd));
                    }
                    break;
                case JdbcGroupDao.CONTAINS:
                    if (upperPortletName.contains(upperQuery)) {
                        result.add(createPortletEntityIdentifier(pd));
                    }
                    break;
                default:
                    throw new GroupsException("Unknown search type");
                }
            }
        }
        return result.toArray(new IEntityIdentifier[result.size()]);
    }
    
    private IEntityIdentifier createPortletEntityIdentifier(IPortletDefinition pd) {
        return new EntityIdentifier(pd.getPortletDefinitionId().toString(), PortletDefinitionId.class);
    }

    private Set getPortletDefinitions() {
        Set s = new HashSet();
        IPortletApplicationDefinition pad;
        Iterator itr = portletApplicationDefinitionRegistry
            .getPortletApplicationDefinitions().iterator();
        while (itr.hasNext()) {
            pad = (IPortletApplicationDefinition) itr.next();
            s.addAll(portletDefinitionRegistry.getPortletDefinitions(pad
                .getPortletApplicationDefinitionId()));
        }
        return s;
    }

}
