/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.general;

import java.util.Set;

import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.dao.IPortletDeploymentDao;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.deployment.IPortletDeployment;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;
import org.jasig.portal.portlet.factory.IPortletDeploymentFactory;
import org.jasig.portal.portlet.registry.event.IPortletDeploymentEventListener;
import org.jasig.portal.portlet.transfer.IPortletDeploymentTransfer;
import org.springframework.beans.factory.annotation.Required;

/**
 * The portlet deployment registry implementation.
 * 
 * Template types to actual types
 * <ol>
 *   <li>GT_Stage - {@link IPortletDeployment}</li>
 *   <li>GT_StageId - {@link PortletDeploymentId}</li>
 *   <li>GT_StageTransfer - {@link IPortletDeploymentTransfer}</li>
 *   <li>GT_ParentStageId - {@link ObjectId}</li>
 *   <li>GT_ParentApplicationStageId - {@link PortletApplicationDeploymentId}</li>
 *   <li>GT_Registry - {@link IPortletDeploymentRegistry}</li> 
 *   <li>GT_RegistryListener - {@link IPortletDeploymentEventListener}</li>
 * </ol>
 * 
 * The portlet deployment registry implementation.
 * @author Michael Ivanov
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletDeploymentRegistryImpl
        extends GenericPortletRegistry<IPortletDeployment, PortletDeploymentId, IPortletDeploymentTransfer, ObjectId, PortletApplicationDeploymentId, IPortletDeploymentRegistry, IPortletDeploymentEventListener>
        implements IPortletDeploymentRegistry {

    private IPortletDeploymentDao portletDeploymentDao;
    private IPortletDeploymentFactory portletDeploymentFactory;
    

    /**
     * @param portletDeploymentDao The portletDeploymentDao to set.
     */
    @Required
    public void setPortletDeploymentDao(IPortletDeploymentDao portletDeploymentDao) {
        this.portletDeploymentDao = portletDeploymentDao;
    }

    /**
     * @param portletDeploymentFactory The portletDeploymentFactory to set.
     */
    @Required
    public void setPortletDeploymentFactory(IPortletDeploymentFactory portletDeploymentFactory) {
        this.portletDeploymentFactory = portletDeploymentFactory;
    }
    
    //********** IPortletDeploymentRegistry Methods **********//
    
    public Set<IPortletDeployment> getPortletDeployments(PortletApplicationDeploymentId portAppDepId) {
        return this.getForApplication(portAppDepId);
    }
    
    public IPortletDeployment getPortletDeployment(PortletDeploymentId portDepId) {
        return this.get(portDepId);
    }
    

    /* =========== GenericPortletRegistry Methods ==================== */

    @Override
    IPortletDeploymentTransfer addTransferObjectToDAO(IPortletDeploymentTransfer transfer) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }
    
    @Override
    IPortletDeployment createDomainObjectFromTransfer(IPortletDeploymentTransfer transfer) {
        return this.portletDeploymentFactory.createPortletDeployment(transfer);
    }
    
    @Override
    PortletApplicationDeploymentId getApplicationId(IPortletDeploymentTransfer transfer) {
        return transfer.getPortletApplicationDeploymentId();
    }
    
    @Override
    PortletDeploymentId getId(IPortletDeploymentTransfer transfer) {
        return transfer.getPortletDeploymentId();
    }
    
    @Override
    ObjectId getParentId(IPortletDeploymentTransfer transfer) {
        return null;
    }
    
    @Override
    Set<String> getSecondaryKeys(IPortletDeploymentTransfer transfer) {
        return null;
    }
    
    @Override
    IPortletDeploymentTransfer getTransferObjectBySecondaryKeyFromDAO(String key) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }
    
    @Override
    IPortletDeploymentTransfer getTransferObjectFromDAO(PortletDeploymentId id) {
        return this.portletDeploymentDao.getPortletDeployment(id);
    }
    
    @Override
    Set<IPortletDeploymentTransfer> getTransferObjectsForApplicationIdAndParentIdFromDAO(PortletApplicationDeploymentId applicationId, ObjectId parentId) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }
    
    @Override
    Set<IPortletDeploymentTransfer> getTransferObjectsForApplicationIdFromDAO(PortletApplicationDeploymentId applicationId) {
        return this.portletDeploymentDao.getPortletDeployments(applicationId);
    }
    
    @Override
    Set<IPortletDeploymentTransfer> getTransferObjectsForParentIdFromDAO(ObjectId parentId) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }
    
    @Override
    Set<IPortletDeploymentTransfer> getTransferObjectsFromDAO() {
        throw new UnsupportedOperationException("not implemented in DAO");
    }
    
    @Override
    void removeObjectFromDAO(PortletDeploymentId id) {
        throw new UnsupportedOperationException("not implemented in DAO");
        
    }
    
    @Override
    void updateObjectInDAO(IPortletDeploymentTransfer transfer) {
        throw new UnsupportedOperationException("not implemented in DAO");
    }
}