/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.transitory;

import org.jasig.portal.portlet.definition.IPortletApplicationDefinitionControl;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.registry.general.IPortletApplicationDefinitionRegistry;


/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public interface ITransientPortletApplicationDefinitionRegistry extends IPortletApplicationDefinitionRegistry {

    /**
     * Create a transient {@link IPortletApplicationDefinitionControl}. The object will have a
     * {@link org.jasig.portal.portlet.definition.PortletApplicationDefinitionId} associated
     * with this transient registry. The object is only stored in memory. To persist the object
     * use {@link IPortletApplicationDefinitionRegistry#createPortletApplicationDefinition(org.jasig.portal.portlet.definition.IPortletApplicationDefinition)}.
     * <br>
     * <br>
     * The new object will be based on the specified {@link org.jasig.portal.portlet.transfer.PortletApplicationDeploymentId}. The
     * definition level properties will have their default values. 
     * 
     * @param portletApplicationDeployment The application deployment to base the definition on.
     * @return Temporary IPortletApplicationDefinitionControl with default values.
     */
    IPortletApplicationDefinitionControl createTransientPortletApplicationDefinitionControl(PortletApplicationDeploymentId portletApplicationDeployment);
    
    /**
     * Removes a transient portlet application definition from the registry. All IPortletApplicationEntity and
     * IPortletDefinition objects based on this definition will also be removed. If the id is not from the
     * transient registry or it does not exist nothing happens. 
     * 
     * @param portletApplicationDefinitionId the portlet definition identifier
     */
    void removeTransientPortletApplicationDefinition(PortletApplicationDefinitionId portletApplicationDefinitionId);
}
