/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.transitory;

import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.entity.IPortletApplicationEntityControl;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.registry.general.IPortletApplicationEntityRegistry;
import org.jasig.portal.portlet.transfer.IPortletApplicationEntityTransfer;


/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public interface ITransientPortletApplicationEntityRegistry extends IPortletApplicationEntityRegistry {

    /**
     * Create a transient {@link IPortletApplicationEntityControl}. The object will have a
     * {@link org.jasig.portal.portlet.entity.PortletApplicationEntityId} associated
     * with this transient registry. The object is only stored in memory. To persist the object
     * use {@link IPortletApplicationEntityRegistry#createPortletApplicationEntity(IPortletApplicationEntityTransfer)}.
     * <br>
     * <br>
     * The new object will be based on the specified {@link PortletApplicationDeploymentId}. The
     * definition level properties will have their default values. 
     * 
     * @param portAppDefId The application deployment to base the definition on.
     * @return Temporary IPortletApplicationDefinitionControl with default values.
     */
    IPortletApplicationEntityControl createTransientPortletApplicationEntityControl(PortletApplicationDefinitionId portAppDefId);
    
    /**
     * Removes a transient portlet application entity from the registry. All IPortletEntity
     * objects based on this entity will also be removed. If the id is not from the
     * transient registry or it does not exist nothing happens. 
     * 
     * @param portletApplicationEntityId the portlet entity identifier
     */
    void removeTransientPortletApplicationEntity(PortletApplicationEntityId portletApplicationEntityId);
}
