/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.transitory;

import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.IPortletDefinitionControl;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.deployment.IPortletApplicationDeployment;
import org.jasig.portal.portlet.deployment.IPortletDeployment;
import org.jasig.portal.portlet.registry.general.IPortletDefinitionRegistry;


/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public interface ITransientPortletDefinitionRegistry extends IPortletDefinitionRegistry {
    /**
     * Create a transient {@link IPortletDefinitionControl}. The object will have a
     * {@link org.jasig.portal.portlet.definition.PortletDefinitionId} associated
     * with this transient registry. The object is only stored in memory. To persist the object
     * use {@link IPortletDefinitionRegistry#createPortletDefinition(org.jasig.portal.portlet.definition.IPortletDefinition)}.
     * <br>
     * <br>
     * The new object will be based on the specified {@link IPortletDeployment} and
     * {@link IPortletApplicationDefinition}. The definition level properties will
     * have their default values. 
     * 
     * @param portletDeployment The deployment to base the definition on.
     * @param portletApplicationDefinition The application definition to base the definition on.
     * @return Temporary IPortletDefinitionControl with default values.
     */
    IPortletDefinitionControl createTransientPortletDefinitionControl(IPortletDeployment portletDeployment, IPortletApplicationDefinition portletApplicationDefinition);
    
    /**
     * Create a transient {@link IPortletDefinitionControl}. The object will have a
     * {@link org.jasig.portal.portlet.definition.PortletDefinitionId} associated
     * with this transient registry. The object is only stored in memory. To persist the object
     * use {@link IPortletDefinitionRegistry#createPortletDefinition(org.jasig.portal.portlet.definition.IPortletDefinition)}.
     * <br>
     * <br>
     * The new object will be based on the specified {@link IPortletDeployment} and
     * {@link IPortletApplicationDeployment}. The definition level properties will
     * have their default values. 
     * 
     * @param portletDeployment The deployment to base the definition on.
     * @param portletApplicationDeployment The application deployment to base the definition on.
     * @return Temporary IPortletDefinitionControl with default values.
     */
    IPortletDefinitionControl createTransientPortletDefinitionControl(IPortletDeployment portletDeployment, IPortletApplicationDeployment portletApplicationDeployment);
    
    /**
     * Removes a transient portlet definition from the registry. All IPortletEntity objects
     * based on this definition will also be removed. If the id is not from the transient
     * registry or it does not exist nothing happens. 
     * 
     * @param portletDefinitionId the portlet definition identifier
     */
    void removeTransientPortletDefinition(PortletDefinitionId portletDefinitionId);
}
