/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.transitory;

import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.IPortletEntityControl;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.registry.general.IPortletEntityRegistry;
import org.jasig.portal.portlet.transfer.IPortletEntityTransfer;


/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public interface ITransientPortletEntityRegistry extends IPortletEntityRegistry {
    /**
     * Create a transient {@link IPortletEntityControl}. The object will have a
     * {@link org.jasig.portal.portlet.entity.PortletEntityId} associated
     * with this transient registry. The object is only stored in memory. To persist the object
     * use {@link IPortletEntityRegistry#createPortletEntity(IPortletEntityTransfer)}.
     * <br>
     * <br>
     * The new object will be based on the specified {@link PortletDeploymentId} and
     * {@link PortletApplicationEntityId}. The entity level properties will
     * have their default values. 
     * 
     * @param portAppEntId The application entity to base the entity on.
     * @param portDefId The definition to base the entity on.
     * @return Temporary IPortletEntityControl with default values.
     */
    IPortletEntityControl createTransientPortletEntityControl(PortletApplicationEntityId portAppEnt, PortletDefinitionId portDef);
    
    /**
     * Removes a transient portlet entity from the registry. All IPortletWindow
     * objects based on this entity will also be removed. If the id is not from the
     * transient registry or it does not exist nothing happens. 
     * 
     * @param portletEntityId the portlet entity identifier
     */
    void removeTransientPortletEntity(PortletEntityId portletEntityId);
}
