/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.transitory;

import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.registry.general.IPortletWindowRegistry;
import org.jasig.portal.portlet.transfer.IPortletWindowTransfer;
import org.jasig.portal.portlet.window.IPortletWindowControl;
import org.jasig.portal.portlet.window.PortletWindowId;


/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public interface ITransientPortletWindowRegistry extends IPortletWindowRegistry {
    /**
     * Create a transient {@link IPortletWindowControl}. The object will have a
     * {@link org.jasig.portal.portlet.window.PortletWindowId} associated
     * with this transient registry. The object is only stored in memory. To persist the object
     * use {@link IPortletWindowRegistry#createPortletWindow(IPortletWindowTransfer)}.
     * <br>
     * <br>
     * The new object will be based on the specified {@link PortletEntityId}.
     * The window level properties will have their default values. 
     * 
     * @param portletEntity The entity to base the window on.
     * @return Temporary IPortletWindowControl with default values.
     */
    IPortletWindowControl createTransientPortletWindowControl(PortletEntityId portletEntity);
    
    /**
     * Create a transient {@link IPortletWindowControl}. The object will have a
     * {@link org.jasig.portal.portlet.window.PortletWindowId} associated
     * with this transient registry. The object is only stored in memory. To persist the object
     * use {@link IPortletWindowRegistry#createPortletWindow(IPortletWindowTransfer)}.
     * <br>
     * <br>
     * The new object will be based on the specified {@link PortletDefinitionId}.
     * The window level properties will have their default values. 
     * 
     * @param portletDefinitionId The definition to base the window on.
     * @return Temporary IPortletWindowControl with default values.
     */
    IPortletWindowControl createTransientPortletWindowControl(PortletDefinitionId portletDefinition);
    
    /**
     * Create a transient {@link IPortletWindowControl} for a specified functional name.
     * @param functionalName functional name
     * @return transient portlet window controll, or <code>null</code> if a portlet definition can not be found for a specified functional name.
     */
    IPortletWindowControl createTransientPortletWindowControl(String functionalName);
    
    
    /**
     * Removes a transient portlet window from the registry. If the id is not from the transient
     * registry or it does not exist nothing happens. 
     * 
     * @param portletWindowId the portlet definition identifier
     */
    void removeTransientPortletWindow(PortletWindowId portletWindowId);
}
