/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.transitory;

import java.util.Set;

import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinitionControl;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.factory.IPortletDefinitionControlFactory;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.registry.event.IPortletApplicationDefinitionEventListener;
import org.jasig.portal.portlet.registry.general.GenericPortletRegistry;
import org.jasig.portal.portlet.registry.general.GenericTransientRegistry;
import org.jasig.portal.portlet.registry.general.IPortletApplicationDefinitionRegistry;
import org.jasig.portal.portlet.transfer.IPortletApplicationDefinitionTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletApplicationDefinitionTransferImpl;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.util.Assert;


/**
 * Transient registry for PortletApplicationDefinition objects
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class TransientPortletApplicationDefinitionRegistryImpl
    extends GenericTransientRegistry<IPortletApplicationDefinition, PortletApplicationDefinitionId, IPortletApplicationDefinitionTransfer, PortletApplicationDeploymentId, ObjectId, IPortletApplicationDefinitionRegistry, IPortletApplicationDefinitionEventListener>
    implements ITransientPortletApplicationDefinitionRegistry {

    //private static final String DOMAIN_OBJECT_ID_PRFX = TransientPortletApplicationDefinitionRegistryImpl.class.getName();
    //  above is too long :)
    private static final String DOMAIN_OBJECT_ID_PRFX = "t";

    private IPortletDefinitionControlFactory portletDefinitionControlFactory;
    private GenericPortletRegistry<IPortletApplicationDefinition, PortletApplicationDefinitionId, IPortletApplicationDefinitionTransfer, PortletApplicationDeploymentId, ObjectId, IPortletApplicationDefinitionRegistry, IPortletApplicationDefinitionEventListener> portletApplicationDefinitionRegistry;

    /**
     * @param portletApplicationDefinitionRegistry The portletApplicationDefinitionRegistry to set.
     */
    @Required
    public void setPortletApplicationDefinitionRegistry(IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry) {
        this.portletApplicationDefinitionRegistry = (GenericPortletRegistry<IPortletApplicationDefinition, PortletApplicationDefinitionId, IPortletApplicationDefinitionTransfer, PortletApplicationDeploymentId, ObjectId, IPortletApplicationDefinitionRegistry, IPortletApplicationDefinitionEventListener>)portletApplicationDefinitionRegistry;
        this.setGeneralRegistry(this.portletApplicationDefinitionRegistry);
    }

    /**
     * @param portletDefinitionControlFactory The portletDefinitionControlFactory to set.
     */
    @Required
    public void setPortletDefinitionControlFactory(IPortletDefinitionControlFactory portletDefinitionControlFactory) {
        this.portletDefinitionControlFactory = portletDefinitionControlFactory;
    }

    
    //********** ITransientPortletApplicationDefinitionRegistry Methods **********//

    public IPortletApplicationDefinitionControl createTransientPortletApplicationDefinitionControl(PortletApplicationDeploymentId portAppDepId) {
        // make up a new transfer object
        PortletApplicationDefinitionTransferImpl transfer=new PortletApplicationDefinitionTransferImpl();
        transfer.setPortletApplicationDeploymentId(portAppDepId);
        return (IPortletApplicationDefinitionControl) this.createTransient(transfer);
    }
    
    public void removeTransientPortletApplicationDefinition(PortletApplicationDefinitionId portAppDefId) {
        this.invalidateObject(portAppDefId);        
    }
    
    public Set<IPortletApplicationDefinition> getPortletApplicationDefinitions() {
        return this.getAll();
    }
    
    public Set<IPortletApplicationDefinition> getPortletApplicationDefinitions(PortletApplicationDeploymentId portAppDepId) {
        return this.getForParent(portAppDepId);
    }
    
    public IPortletApplicationDefinition createPortletApplicationDefinition(IPortletApplicationDefinitionTransfer transfer) {
        return this.create(transfer);
    }

    public IPortletApplicationDefinition getPortletApplicationDefinition(PortletApplicationDefinitionId portAppDefId) {
        return this.get(portAppDefId);
    }

    public void updatePortletApplicationDefinition(IPortletApplicationDefinition portAppDef) {
        Assert.notNull(portAppDef, "portAppDef can not be null");
        this.update(portAppDef.getPortletApplicationDefinitionTransfer());
    }

    public void removePortletApplicationDefinition(PortletApplicationDefinitionId portAppDefId) {
        this.remove(portAppDefId);
    }
    

    /* =========== GenericTransientRegistry Methods ==================== */

    @Override
    protected GenericPortletRegistry<IPortletApplicationDefinition, PortletApplicationDefinitionId, IPortletApplicationDefinitionTransfer, PortletApplicationDeploymentId, ObjectId, IPortletApplicationDefinitionRegistry, IPortletApplicationDefinitionEventListener> getGeneralRegistry() {
        return this.portletApplicationDefinitionRegistry;
    }
    
    @Override
    protected IPortletApplicationDefinition createDomainObjectFromTransfer(IPortletApplicationDefinitionTransfer transfer) {
        try {
            return this.portletDefinitionControlFactory.createPortletApplicationDefinitionControl(transfer);
        }
        catch (DomainObjectCreationException doce) {
            logger.error("can not create portlet application definition domain object from an existing transfer object", doce);
            return null;
        }
    }
    
    @Override
    protected void setId(IPortletApplicationDefinitionTransfer transfer, long id) {
        final PortletApplicationDefinitionId portAppDefId = new PortletApplicationDefinitionId(DOMAIN_OBJECT_ID_PRFX + Long.toString(id));
        transfer.setPortletApplicationDefinitionId(portAppDefId);
    }
}
