/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.transitory;

import java.util.Set;

import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.context.common.user.IUserControllerEventListener;
import org.jasig.portal.context.common.user.UserControllerEvent;
import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.entity.IPortletApplicationEntity;
import org.jasig.portal.portlet.entity.IPortletApplicationEntityControl;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.factory.IPortletApplicationEntityRegistryFactory;
import org.jasig.portal.portlet.factory.IPortletEntityControlFactory;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.registry.event.IPortletApplicationEntityEventListener;
import org.jasig.portal.portlet.registry.general.GenericPortletRegistry;
import org.jasig.portal.portlet.registry.general.GenericTransientRegistry;
import org.jasig.portal.portlet.registry.general.IPortletApplicationEntityRegistry;
import org.jasig.portal.portlet.transfer.IPortletApplicationEntityTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletApplicationEntityTransferImpl;
import org.jasig.portal.security.UserId;
import org.springframework.beans.factory.annotation.Required;

/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class TransientPortletApplicationEntityRegistryImpl
        extends GenericTransientRegistry<IPortletApplicationEntity, PortletApplicationEntityId, IPortletApplicationEntityTransfer, PortletApplicationDefinitionId, ObjectId, IPortletApplicationEntityRegistry, IPortletApplicationEntityEventListener>
        implements ITransientPortletApplicationEntityRegistry, IUserControllerEventListener {

    //private static final String DOMAIN_OBJECT_ID_PRFX = TransientPortletApplicationEntityRegistryImpl.class.getName();
    //  above is too long :)
    private static final String DOMAIN_OBJECT_ID_PRFX = "t";
    
    private IPortletEntityControlFactory portletEntityControlFactory;
    private IPortletApplicationEntityRegistryFactory portletApplicationEntityRegistryFactory;
    private IUserController userController;

    /**
     * @param portletApplicationEntityRegistryFactory The portletApplicationEntityRegistryFactory to set.
     */
    @Required
    public void setPortletApplicationEntityRegistryFactory(IPortletApplicationEntityRegistryFactory portletApplicationEntityRegistryFactory) {
        this.portletApplicationEntityRegistryFactory = portletApplicationEntityRegistryFactory;
        if (userController != null) {
            this.initGeneralRegistry();
        }
    }
    
    /**
     * @param portletEntityControlFactory The portletEntityControlFactory to set.
     */
    @Required
    public void setPortletEntityControlFactory(IPortletEntityControlFactory portletEntityControlFactory) {
        this.portletEntityControlFactory = portletEntityControlFactory;
    }

    /**
     * @param userController The userController to set.
     */
    public void setUserController(IUserController userController) {
        if (this.userController != null) {
            this.userController.removeUserControllerEventListener(this);
        }
        
        this.userController = userController;
        if (this.userController != null) {
            this.userController.addUserControllerEventListener(this);
        }
        
        if (this.portletApplicationEntityRegistryFactory != null) {
            this.initGeneralRegistry();
        }
    }

    private void initGeneralRegistry() {
        this.setGeneralRegistry(getGeneralRegistry());
    }

    
    //********** ITransientPortletApplicationEntityRegistry Methods **********//

    public IPortletApplicationEntityControl createTransientPortletApplicationEntityControl(PortletApplicationDefinitionId portAppDefId) {
        if (portAppDefId == null) {
            throw new IllegalArgumentException("portAppDefId may not be null");
        }

        // create transfer object
        PortletApplicationEntityTransferImpl transfer = new PortletApplicationEntityTransferImpl();
        transfer.setUserId(this.getUserId());
        transfer.setPortletApplicationDefinitionId(portAppDefId);
        return (IPortletApplicationEntityControl)this.createTransient(transfer);
    }
    
    public void removeTransientPortletApplicationEntity(PortletApplicationEntityId portAppEntId) {
        this.invalidateObject(portAppEntId);      
    }

    public Set<IPortletApplicationEntity> getPortletApplicationEntities(PortletApplicationDefinitionId portAppDefId) {
        return this.getForParent(portAppDefId);
    }

    public IPortletApplicationEntity getPortletApplicationEntity(PortletApplicationEntityId portAppEntId) {
        return this.get(portAppEntId);
    }

    public IPortletApplicationEntity createPortletApplicationEntity(IPortletApplicationEntityTransfer portAppEntTransfer) {
        return this.create(portAppEntTransfer);    
    }

    public void removePortletApplicationEntity(PortletApplicationEntityId portAppEntId) {
        this.remove(portAppEntId);
    }

    public UserId getUserId() {
        return this.userController.getUserId();
    }

    
    //********** GenericTransientRegistry Methods **********//

    @Override
    protected GenericPortletRegistry<IPortletApplicationEntity, PortletApplicationEntityId, IPortletApplicationEntityTransfer, PortletApplicationDefinitionId, ObjectId, IPortletApplicationEntityRegistry, IPortletApplicationEntityEventListener> getGeneralRegistry() {
        return (GenericPortletRegistry<IPortletApplicationEntity, PortletApplicationEntityId, IPortletApplicationEntityTransfer, PortletApplicationDefinitionId, ObjectId, IPortletApplicationEntityRegistry, IPortletApplicationEntityEventListener>) portletApplicationEntityRegistryFactory.getPortletApplicationEntityRegistry(this.getUserId());
    }
    
    @Override
    protected IPortletApplicationEntity createDomainObjectFromTransfer(IPortletApplicationEntityTransfer transfer) {
        try {
            return this.portletEntityControlFactory.createPortletApplicationEntityControl(transfer);
        }
        catch (DomainObjectCreationException doce) {
            logger.error("can not create portlet application entity domain object from an existing transfer object.", doce);
            return null;
        }
    }

    @Override
    protected void setId(IPortletApplicationEntityTransfer transfer, long id) {
        final PortletApplicationEntityId portAppEntId = new PortletApplicationEntityId(DOMAIN_OBJECT_ID_PRFX + Long.toString(id));
        transfer.setPortletApplicationEntityId(portAppEntId);
    }

    
    //********** IUserControllerEventListener Methods **********//

    public void receiveUserControllerEvent(UserControllerEvent event) {
        // reinitialize registry upon user change
        this.initGeneralRegistry();
    }
}
