/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.transitory;

import java.util.Set;

import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.IPortletDefinitionControl;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.deployment.IPortletApplicationDeployment;
import org.jasig.portal.portlet.deployment.IPortletDeployment;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;
import org.jasig.portal.portlet.factory.IPortletDefinitionControlFactory;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.registry.event.IPortletDefinitionEventListener;
import org.jasig.portal.portlet.registry.general.GenericPortletRegistry;
import org.jasig.portal.portlet.registry.general.GenericTransientRegistry;
import org.jasig.portal.portlet.registry.general.IPortletDefinitionRegistry;
import org.jasig.portal.portlet.transfer.IPortletDefinitionTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletDefinitionTransferImpl;
import org.springframework.beans.factory.annotation.Required;

/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class TransientPortletDefinitionRegistryImpl
        extends GenericTransientRegistry<IPortletDefinition, PortletDefinitionId, IPortletDefinitionTransfer, PortletDeploymentId, PortletApplicationDefinitionId, IPortletDefinitionRegistry, IPortletDefinitionEventListener>
        implements ITransientPortletDefinitionRegistry {

    //private static final String DOMAIN_OBJECT_ID_PRFX = TransientPortletDefinitionRegistryImpl.class.getName();
    //  above is too long :)
    private static final String DOMAIN_OBJECT_ID_PRFX = "t";
    
    private IPortletDefinitionControlFactory portletDefinitionControlFactory;
    private ITransientPortletApplicationDefinitionRegistry transientPortletApplicationDefinitionRegistry;
    private GenericPortletRegistry<IPortletDefinition, PortletDefinitionId, IPortletDefinitionTransfer, PortletDeploymentId, PortletApplicationDefinitionId, IPortletDefinitionRegistry, IPortletDefinitionEventListener> portletDefinitionRegistry;

    /**
     * @param portletDefinitionControlFactory The portletDefinitionControlFactory to set.
     */
    @Required
    public void setPortletDefinitionControlFactory(IPortletDefinitionControlFactory portletDefinitionControlFactory) {
        this.portletDefinitionControlFactory = portletDefinitionControlFactory;
    }

    /**
     * @param portletDefinitionRegistry The portletDefinitionRegistry to set.
     */
    @Required
    public void setPortletDefinitionRegistry(IPortletDefinitionRegistry portletDefinitionRegistry) {
        this.portletDefinitionRegistry=(GenericPortletRegistry<IPortletDefinition, PortletDefinitionId, IPortletDefinitionTransfer, PortletDeploymentId, PortletApplicationDefinitionId, IPortletDefinitionRegistry, IPortletDefinitionEventListener>) portletDefinitionRegistry;
        this.setGeneralRegistry(this.portletDefinitionRegistry);
    }

    /**
     * @param transientPortletApplicationDefinitionRegistry The transientPortletApplicationDefinitionRegistry to set.
     */
    @Required
    public void setTransientPortletApplicationDefinitionRegistry(ITransientPortletApplicationDefinitionRegistry transientPortletApplicationDefinitionRegistry) {
        this.transientPortletApplicationDefinitionRegistry = transientPortletApplicationDefinitionRegistry;
    }

    
    //********** ITransientPortletDefinitionRegistry Methods **********//

    public IPortletDefinitionControl createTransientPortletDefinitionControl(IPortletDeployment portDep, IPortletApplicationDefinition portAppDef) {
        if (portDep == null) {
            throw new IllegalArgumentException("portDep may not be null");
        }
        if (portAppDef == null) {
            throw new IllegalArgumentException("portAppDef may not be null");
        }

        // create transfer object
        PortletDefinitionTransferImpl transfer = new PortletDefinitionTransferImpl();
        transfer.setPortletDeploymentId(portDep.getPortletDeploymentId());
        transfer.setPortletApplicationDefinitionId(portAppDef.getPortletApplicationDefinitionId());
        return (IPortletDefinitionControl)this.createTransient(transfer);
    }
    
    public IPortletDefinitionControl createTransientPortletDefinitionControl(IPortletDeployment portDep, IPortletApplicationDeployment portAppDep) {
        if (portDep == null) {
            throw new IllegalArgumentException("portDep may not be null");
        }
        if (portAppDep == null) {
            throw new IllegalArgumentException("portAppDep may not be null");
        }

        final Set portAppDefs = this.transientPortletApplicationDefinitionRegistry.getPortletApplicationDefinitions(portAppDep.getPortletApplicationDeploymentId());
        final IPortletApplicationDefinition portAppDef;
        if (portAppDefs.size() > 0) {
            portAppDef = (IPortletApplicationDefinition)portAppDefs.iterator().next();

            if (logger.isDebugEnabled()) {
                logger.debug("Using existing IPortletApplicationDefinition='" + portAppDef + "'");
            }
        }
        else {
            portAppDef = this.transientPortletApplicationDefinitionRegistry.createTransientPortletApplicationDefinitionControl(portAppDep.getPortletApplicationDeploymentId());

            if (logger.isDebugEnabled()) {
                logger.debug("Created new transient IPortletApplicationDefinition='" + portAppDef + "'");
            }
        }

        return this.createTransientPortletDefinitionControl(portDep, portAppDef); 
    }
    
    public void removeTransientPortletDefinition(PortletDefinitionId portDefId) {
        this.invalidateObject(portDefId);    
    }

    public IPortletDefinition createPortletDefinition(IPortletDefinition portDef) {
        return this.create(portDef.getPortletDefinitionTransfer());
    }

    public IPortletDefinition getPortletDefinition(PortletDefinitionId portDefId) {
        return this.get(portDefId);
    }

    public IPortletDefinition getPortletDefinitionByFunctionalName(String funcName) {
        return this.getBySecondaryKey(funcName);
    }

    public Set<IPortletDefinition> getPortletDefinitions(PortletApplicationDefinitionId portAppDefId) {
        return this.getForApplication(portAppDefId);
    }

    public void removePortletDefinition(PortletDefinitionId portDefId) {
        this.remove(portDefId);    
    }

    public void updatePortletDefinition(IPortletDefinition portDef) {
        if (portDef == null) {
            throw new IllegalArgumentException("portDef may not be null");
        }
        this.update(portDef.getPortletDefinitionTransfer());
    }
    

    /* =========== GenericTransientRegistry Methods ==================== */

    @Override
    protected GenericPortletRegistry<IPortletDefinition, PortletDefinitionId, IPortletDefinitionTransfer, PortletDeploymentId, PortletApplicationDefinitionId, IPortletDefinitionRegistry, IPortletDefinitionEventListener> getGeneralRegistry() {
        return this.portletDefinitionRegistry;
    }
    
    @Override
    protected IPortletDefinition createDomainObjectFromTransfer(IPortletDefinitionTransfer transfer) {
        try {
            return this.portletDefinitionControlFactory.createPortletDefinition(transfer);
        }
        catch (DomainObjectCreationException doce) {
            logger.error("can not create portlet definition domain object from an existing transfer object.", doce);
            return null;
        }
    }

    @Override
    protected void setId(IPortletDefinitionTransfer transfer, long id) {
        final PortletDefinitionId portDefId = new PortletDefinitionId(DOMAIN_OBJECT_ID_PRFX + Long.toString(id));
        transfer.setPortletDefinitionId(portDefId);
    }
}
