/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.transitory;

import java.util.Set;

import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.context.common.user.IUserControllerEventListener;
import org.jasig.portal.context.common.user.UserControllerEvent;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.entity.IPortletEntity;
import org.jasig.portal.portlet.entity.IPortletEntityControl;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.factory.IPortletEntityControlFactory;
import org.jasig.portal.portlet.factory.IPortletEntityRegistryFactory;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.registry.event.IPortletEntityEventListener;
import org.jasig.portal.portlet.registry.general.GenericPortletRegistry;
import org.jasig.portal.portlet.registry.general.GenericTransientRegistry;
import org.jasig.portal.portlet.registry.general.IPortletEntityRegistry;
import org.jasig.portal.portlet.transfer.IPortletEntityTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletEntityTransferImpl;
import org.jasig.portal.security.UserId;
import org.springframework.beans.factory.annotation.Required;

/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class TransientPortletEntityRegistryImpl
        extends GenericTransientRegistry<IPortletEntity, PortletEntityId, IPortletEntityTransfer, PortletDefinitionId, PortletApplicationEntityId, IPortletEntityRegistry, IPortletEntityEventListener>
        implements ITransientPortletEntityRegistry, IUserControllerEventListener {

    //private static final String DOMAIN_OBJECT_ID_PRFX = TransientPortletEntityRegistryImpl.class.getName();
    // above is too long :)
    private static final String DOMAIN_OBJECT_ID_PRFX = "t";
    
    private IPortletEntityControlFactory portletEntityControlFactory;
    private IPortletEntityRegistryFactory portletEntityRegistryFactory;
    private IUserController userController;
    
    /**
     * @param portletEntityControlFactory The portletEntityControlFactory to set.
     */
    @Required
    public void setPortletEntityControlFactory(IPortletEntityControlFactory portletEntityControlFactory) {
        this.portletEntityControlFactory = portletEntityControlFactory;
    }

    /**
     * @param portletEntityRegistryFactory The portletEntityRegistryFactory to set.
     */
    @Required
    public void setPortletEntityRegistryFactory(IPortletEntityRegistryFactory portletEntityRegistryFactory) {
        this.portletEntityRegistryFactory = portletEntityRegistryFactory;
        if (this.userController != null) {
            this.initGeneralRegistry();
        }
    }

    /**
     * @param userController The userController to set.
     */
    @Required
    public void setUserController(IUserController userController) {
        if (this.userController != null) {
            this.userController.removeUserControllerEventListener(this);
        }

        this.userController = userController;
        if (this.userController != null) {
            this.userController.addUserControllerEventListener(this);
        }

        if (this.portletEntityRegistryFactory != null) {
            this.initGeneralRegistry();
        }
    }

    private void initGeneralRegistry() {
        this.setGeneralRegistry(getGeneralRegistry());
    } 

    
    //********** ITransientPortletEntityRegistry Methods **********//

    public IPortletEntityControl createTransientPortletEntityControl(PortletApplicationEntityId portAppEntId, PortletDefinitionId portDefId) {
        if (portAppEntId == null) {
            throw new IllegalArgumentException("portAppEntId may not be null");
        }
        if (portDefId == null) {
            throw new IllegalArgumentException("portDefId may not be null");
        }

        // create a transfer object
        PortletEntityTransferImpl transfer = new PortletEntityTransferImpl();
        transfer.setPortletApplicationEntityId(portAppEntId);
        transfer.setPortletDefinitionId(portDefId);
        transfer.setUserId(this.getUserId());
        return (IPortletEntityControl)this.createTransient(transfer);
    }
    
    public void removeTransientPortletEntity(PortletEntityId portEntId) {
        this.invalidateObject(portEntId);
    }
    
    public Set<IPortletEntity> getPortletEntities() {
        return this.getAll();
    }

    public Set<IPortletEntity> getPortletEntities(PortletApplicationEntityId portAppEntId) {
        return this.getForApplication(portAppEntId);
    }
    
    public Set<IPortletEntity> getPortletEntities(PortletApplicationEntityId portAppEntId, PortletDefinitionId portDefId) {
        return this.getForApplicationAndParent(portAppEntId,portDefId);
    }
    
    public IPortletEntity getPortletEntity(PortletEntityId portEntId) {
        return this.get(portEntId);
    }

    public IPortletEntity createPortletEntity(IPortletEntityTransfer portEntityTransfer) {
        return this.create(portEntityTransfer);
    }

    public void removePortletEntity(PortletEntityId portEntId) {
        this.remove(portEntId);
    }

    public void updatePortletEntity(IPortletEntity portEnt) {
        if (portEnt == null) {
            throw new IllegalArgumentException("portEnt may not be null");
        }
        
        this.update(portEnt.getPortletEntityTransfer());
    }

    public IPortletEntity clonePortletEntity(PortletEntityId portEntId) {
        if (portEntId == null) {
            throw new IllegalArgumentException("portEntId may not be null");
        }
        
        final IPortletEntityTransfer portEntTrans = this.getTransientTransfer(portEntId);
        if (portEntTrans != null) {
            // id mapped to a transient object, make a transient clone
            return this.createTransient(portEntTrans);
        }
        else {
            IPortletEntity entity = this.get(portEntId);
            if (entity != null) {
                return this.create(entity.getPortletEntityTransfer());
            }
            else {
                return null;
            }
        }
    }

    public UserId getUserId() {
        return this.userController.getUserId();
    }

    
    //********** GenericTransientRegistry Methods **********//

    @Override
    protected GenericPortletRegistry<IPortletEntity, PortletEntityId, IPortletEntityTransfer, PortletDefinitionId, PortletApplicationEntityId, IPortletEntityRegistry, IPortletEntityEventListener> getGeneralRegistry() {
        return (GenericPortletRegistry<IPortletEntity, PortletEntityId, IPortletEntityTransfer, PortletDefinitionId, PortletApplicationEntityId, IPortletEntityRegistry, IPortletEntityEventListener>) portletEntityRegistryFactory.getPortletEntityRegistry(this.getUserId());
    }
    
    @Override
    protected IPortletEntity createDomainObjectFromTransfer(IPortletEntityTransfer transfer) {
        try {
            return this.portletEntityControlFactory.createPortletEntity(this.getUserId(), transfer);
        }
        catch (DomainObjectCreationException doce) {
            logger.error("can not create portlet entity domain object from an existing transfer object.", doce);
            return null;
        }
    }

    @Override
    protected void setId(IPortletEntityTransfer transfer, long id) {
        final PortletEntityId portEntId = new PortletEntityId(DOMAIN_OBJECT_ID_PRFX + Long.toString(id));
        transfer.setPortletEntityId(portEntId);
    }

    
    //********** IUserControllerEventListener Methods **********//

    public void receiveUserControllerEvent(UserControllerEvent event) {
        // reinitialize registry upon user change
        this.initGeneralRegistry();
    }
}
