/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.registry.transitory;

import java.util.Set;

import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.context.common.user.IUserControllerEventListener;
import org.jasig.portal.context.common.user.UserControllerEvent;
import org.jasig.portal.core.ObjectId;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.entity.IPortletApplicationEntity;
import org.jasig.portal.portlet.entity.IPortletEntity;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.factory.IPortletWindowControlFactory;
import org.jasig.portal.portlet.factory.IPortletWindowRegistryFactory;
import org.jasig.portal.portlet.registry.DomainObjectCreationException;
import org.jasig.portal.portlet.registry.event.IPortletWindowEventListener;
import org.jasig.portal.portlet.registry.general.GenericPortletRegistry;
import org.jasig.portal.portlet.registry.general.GenericTransientRegistry;
import org.jasig.portal.portlet.registry.general.IPortletDefinitionRegistry;
import org.jasig.portal.portlet.registry.general.IPortletWindowRegistry;
import org.jasig.portal.portlet.transfer.IPortletWindowTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletWindowTransferImpl;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlet.window.IPortletWindowControl;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.security.UserId;
import org.springframework.beans.factory.annotation.Required;

/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class TransientPortletWindowRegistryImpl
        extends GenericTransientRegistry<IPortletWindow, PortletWindowId, IPortletWindowTransfer, PortletEntityId, ObjectId, IPortletWindowRegistry, IPortletWindowEventListener>
        implements ITransientPortletWindowRegistry, IUserControllerEventListener {

    //private static final String DOMAIN_OBJECT_ID_PRFX = TransientPortletWindowRegistryImpl.class.getName();
    // above is too long :)
    private static final String DOMAIN_OBJECT_ID_PRFX = "t";
    
    private IPortletWindowControlFactory portletWindowControlFactory;
    private IPortletWindowRegistryFactory portletWindowRegistryFactory;
    private ITransientPortletEntityRegistry transientPortletEntityRegistry;
    private ITransientPortletApplicationEntityRegistry transientPortletApplicationEntityRegistry;
    private IPortletDefinitionRegistry portletDefinitionRegistry;
    private IUserController userController;
    
    /**
     * @param portletWindowControlFactory The portletWindowControlFactory to set.
     */
    @Required
    public void setPortletWindowControlFactory(IPortletWindowControlFactory portletWindowControlFactory) {
        this.portletWindowControlFactory = portletWindowControlFactory;
    }

    /**
     * @param portletWindowRegistryFactory The portletWindowRegistryFactory to set.
     */
    @Required
    public void setPortletWindowRegistryFactory(IPortletWindowRegistryFactory portletWindowRegistryFactory) {
        this.portletWindowRegistryFactory = portletWindowRegistryFactory;
        if (this.userController != null) {
            this.initGeneralRegistry();
        }
    }

    /**
     * @param transientPortletApplicationEntityRegistry The transientPortletApplicationEntityRegistry to set.
     */
    @Required
    public void setTransientPortletApplicationEntityRegistry(ITransientPortletApplicationEntityRegistry transientPortletApplicationEntityRegistry) {
        this.transientPortletApplicationEntityRegistry = transientPortletApplicationEntityRegistry;
    }

    /**
     * @param transientPortletEntityRegistry The transientPortletEntityRegistry to set.
     */
    @Required
    public void setTransientPortletEntityRegistry(ITransientPortletEntityRegistry transientPortletEntityRegistry) {
        this.transientPortletEntityRegistry = transientPortletEntityRegistry;
    }

    /**
     * @param userController The userController to set.
     */
    @Required
    public void setUserController(IUserController userController) {
        if(this.userController!=null) {
            this.userController.removeUserControllerEventListener(this);
        }
        
        this.userController = userController;
        if(this.userController!=null) {
            this.userController.addUserControllerEventListener(this);
        }
        
        if(this.portletWindowRegistryFactory!=null) {
            this.initGeneralRegistry();
        }
    }
    
    /**
     * @param portletDefinitionRegistry The portletDefinitionRegistry to set.
     */
    public void setPortletDefinitionRegistry(IPortletDefinitionRegistry portletDefinitionRegistry) {
        this.portletDefinitionRegistry = portletDefinitionRegistry;
    }

    private void initGeneralRegistry() {
        this.setGeneralRegistry(getGeneralRegistry());
    }

    protected IPortletApplicationEntity getOrCreatePortletApplicationEntity(IPortletDefinition definition) {
        final PortletApplicationDefinitionId portAppDefId = definition.getPortletApplicationDefinitionId();

        final Set<IPortletApplicationEntity> portAppEnts = this.transientPortletApplicationEntityRegistry.getPortletApplicationEntities(portAppDefId);
        final IPortletApplicationEntity portAppEnt;
        if (portAppEnts.size() > 0) {
            portAppEnt = portAppEnts.iterator().next();

            if (logger.isDebugEnabled()) {
                logger.debug("Using existing IPortletApplicationEntity='" + portAppEnt + "'");
            }
        }
        else {
            portAppEnt = this.transientPortletApplicationEntityRegistry.createTransientPortletApplicationEntityControl(portAppDefId);

            if (logger.isDebugEnabled()) {
                logger.debug("Created new transient IPortletApplicationEntity='" + portAppEnt + "'");
            }
        }
        return portAppEnt;
    }

    protected IPortletEntity getOrCreatePortletEntity(PortletDefinitionId portDefId, final IPortletApplicationEntity portAppEnt) {
        final PortletApplicationEntityId portAppEntId = portAppEnt.getPortletApplicationEntityId();
        final Set<IPortletEntity> portEnts = this.transientPortletEntityRegistry.getPortletEntities(portAppEntId, portDefId);
        final IPortletEntity portEnt;
        if (portEnts.size() > 0) {
            portEnt = portEnts.iterator().next();

            if (logger.isDebugEnabled()) {
                logger.debug("Using existing IPortletEntity='" + portEnt + "'");
            }
        }
        else {
            portEnt = this.transientPortletEntityRegistry.createTransientPortletEntityControl(portAppEntId, portDefId);

            if (logger.isDebugEnabled()) {
                logger.debug("Created new transient IPortletEntity='" + portEnt + "'");
            }
        }
        return portEnt;
    }

    
    //********** ITransientPortletWindowRegistry Methods **********//

    public IPortletWindowControl createTransientPortletWindowControl(PortletEntityId portEntId) {
        if (portEntId == null) {
            throw new IllegalArgumentException("portEntId may not be null");
        }
        // create transfer
        PortletWindowTransferImpl transfer = new PortletWindowTransferImpl();
        transfer.setPortletEntityId(portEntId);
        return (IPortletWindowControl)this.createTransient(transfer);
    }

    public IPortletWindowControl createTransientPortletWindowControl(PortletDefinitionId portDefId) {
        if (portDefId == null) {
            throw new IllegalArgumentException("portDefId may not be null");
        }
        IPortletDefinition definition = portletDefinitionRegistry.getPortletDefinition(portDefId);
        if (definition == null) {
            return null;
        }

        final IPortletApplicationEntity portAppEnt = this.getOrCreatePortletApplicationEntity(definition);

        final IPortletEntity portEnt = this.getOrCreatePortletEntity(portDefId, portAppEnt);

        return this.createTransientPortletWindowControl(portEnt.getPortletEntityId());
    }

    public IPortletWindowControl createTransientPortletWindowControl(String functionalName) {
        if (functionalName == null) {
            throw new IllegalArgumentException("functionalName may not be null");
        }

        // look up portlet definition
        IPortletDefinition definition = portletDefinitionRegistry.getPortletDefinitionByFunctionalName(functionalName);
        if (definition != null) {
            return createTransientPortletWindowControl(definition.getPortletDefinitionId());
        }
        else {
            return null;
        }
    }
    
    public void removeTransientPortletWindow(PortletWindowId portWinId) {
        this.invalidateObject(portWinId);
    }
    
    public Set<IPortletWindow> getPortletWindows(PortletEntityId portEntId) {
        return this.getForParent(portEntId);
    }

    public IPortletWindow createPortletWindow(IPortletWindowTransfer portWindowTransfer) {
        return this.create(portWindowTransfer);
    }

    public IPortletWindow getPortletWindow(PortletWindowId portWinId) {
        return this.get(portWinId);
    }

    public void removePortletWindow(PortletWindowId portWinId) {
        this.remove(portWinId);
    }

    public void updatePortletWindow(IPortletWindow portWin) {
        if (portWin == null) {
            throw new IllegalArgumentException("portWin may not be null");
        }
        
        this.update(portWin.getPortletWindowTransfer());
    }
    

    public UserId getUserId() {
        return this.userController.getUserId();
    }

    
    //********** GenericTransientRegistry Methods **********//

    @Override
    protected GenericPortletRegistry<IPortletWindow, PortletWindowId, IPortletWindowTransfer, PortletEntityId, ObjectId, IPortletWindowRegistry, IPortletWindowEventListener> getGeneralRegistry() {
        return (GenericPortletRegistry<IPortletWindow, PortletWindowId, IPortletWindowTransfer, PortletEntityId, ObjectId, IPortletWindowRegistry, IPortletWindowEventListener>) portletWindowRegistryFactory.getPortletWindowRegistry(this.getUserId());
    }
    
    @Override
    protected IPortletWindow createDomainObjectFromTransfer(IPortletWindowTransfer transfer) {
        try {
            return this.portletWindowControlFactory.createPortletWindow(this.getUserId(), transfer);
        }
        catch (DomainObjectCreationException doce) {
            logger.error("can not create portlet window domain object from an existing transfer object.", doce);
            return null;
        }
    }

    @Override
    protected void setId(IPortletWindowTransfer transfer, long id) {
        final PortletWindowId portWinId = new PortletWindowId(DOMAIN_OBJECT_ID_PRFX + Long.toString(id));
        transfer.setPortletWindowId(portWinId);
    }

    
    //********** IUserControllerEventListener Methods **********//

    public void receiveUserControllerEvent(UserControllerEvent event) {
        // reinitialize registry upon user change
        this.initGeneralRegistry();
    }
}
