/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.rendering;

import org.jasig.portal.portlet.window.PortletWindowId;

/**
 *  Defines an interface for performing restart/refresh operations
 *  on the porlets currently replaced by the error portlet
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IErrorPortletCommands {
    /**
     * Replace ErrorPortlet with the original window.
     * Portlet window to be returned to its default state (with session and params cleared).
     * 
     * @param windowId window id
     */
    void restartPortletWindow(PortletWindowId windowId);
    
    /**
     * Replace ErrorPortlet with the original window.
     * Session is kept the same as it was during the last request to that portlet window,
     * action/request parameters that were passed to the portlet during the failed request
     * are passed again.
     * 
     * @param windowId window id
     */
    void retryPortletWindow(PortletWindowId windowId);
    
    
    /**
     * Replace ErrorPortlet with the original window.
     * Session is kept the same as it was during the last request to that portlet window.
     * No action/render parameters are passed to the portlet.
     * 
     * @param windowId window id
     */
    void refreshPortletWindow(PortletWindowId windowId);
}
