/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.rendering;

import javax.servlet.http.HttpServletRequest;

import org.jasig.portal.portlet.window.PortletWindowId;

/**
 *  Used by portlet window manager to manage and retrieve
 *  cached portlet content.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletCacheManager {
    
    /**
     * Check if the cached content is currently available for this portlet.
     * If <code>true</code> is returned, call to <code>getPortletRenderContent</code> within the same request will return a valid content.
     * Note: this assumes that the caller will check for protlet being targeted with request params.
     * 
     * @param portletWindowId portlet windowid
     * @param req current request
     */
    boolean isCachedContentAvailable(PortletWindowId portletWindowId, HttpServletRequest req);
    
    
    /**
     * Retrieve cached portlet content.
     * Call to isCachedContentAvailable has to preceed call to this method.
     * 
     * @param windowId
     * @param req
     * @return cached portlet content
     */
    IPortletRenderContent getPortletRenderContent(PortletWindowId windowId, HttpServletRequest req);
    
    /**
     * Record a portlet content cache
     * 
     * @param windowId
     * @param req
     */
    void setPortletRenderContent(IPortletRenderContent renderContent, PortletWindowId windowId, HttpServletRequest req);
    
    /**
     * Invalidate cache for a given portlet window
     * @param windowId
     */
    void invalidateCache(PortletWindowId windowId);
}
