/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.rendering;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.portlets.error.bean.ErrorData;

/**
 *  An entity responsible for handling portlet errors.
 *  This includes intial replacement of a portlet by an error display, and
 *  portlet retry/restart controls
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletErrorHandler {

    /**
     * Handle portlet rendering problem. 
     * 
     * @param windowId window id of the portlet that caused the problem
     * @param req current request
     * @param res current response
     * @param ed error data
     * @return portlet content for content retrieval phase errors, <code>null</code> otherwise.
     */
    public IPortletRenderContent handleProblem(PortletWindowId windowId, HttpServletRequest req, HttpServletResponse res, ErrorData ed);

    /**
     * Returns portlet window id of an error portlet replacing given window id
     * 
     * @param windowId id of a window that was remapped to an error portlet
     * @param req current request
     * @return an error portlet window, or <code>null</code> if the window has not been remapped to the error portlet
     */
    public PortletWindowId getReplacingErrorPortletWindowId(PortletWindowId windowId, HttpServletRequest req);
    
    /**
     * Replaces ErrorPortlet with the original window.
     * Session is kept the same as it was during the last request to that portlet window.
     * 
     * @param windowId id of a window that's currently being replaced with an error portlet and should be refreshed
     * @param req http request
     * @param res http response
     * @param passLastRequestParameters <code>true</code> if the action or render parameters should be passed again, <code>false</code> if the parameter maps should be cleared
     */
    public void refreshPortletWindow(PortletWindowId windowId, HttpServletRequest req, HttpServletResponse res, boolean passLastRequestParameters);

    /**
     * Replaces ErrorPortlet with the original window.
     * Portlet window to be returned to its default state (with session and params cleared).
     * 
     * @param windowId id of a window that's currently being replaced by an error portlet and should be restarted
     * @param req http request
     * @param res http response
     */
    public void restartPortletWindow(PortletWindowId windowId, HttpServletRequest req, HttpServletResponse res);

}