/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.portlet.rendering;

import java.io.UnsupportedEncodingException;
import java.util.Locale;

/**
 * IPortletRenderContent is a datastructure class holding all content
 * information for a single portlet rendering
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public interface IPortletRenderContent {
    
    /**
     * Obtain recorded character portlet output.
     * @return characters sent by the portlet to the <code>Writer</code>
     */
    public String getCharacterOutput() throws UnsupportedEncodingException;
    
    /**
     * Obtain recorded binary portlet output.
     * @return byte array sent by the portlet to the <code>OutputStream</code>
     */
    public byte[] getBinaryOutput();
    
    /**
     * Obtain title specified by the portlet
     * @return title specified by the portlet
     */
    public String getTitle();
    
    /**
     * Obtain content type specified by the portlet
     * @return MIME content type
     */
    public String getContentType();
    
    /**
     * Determine the locale specified by the portlet
     */
    public Locale getLocale();
    
    /**
     * Returns portlet cache expiration directive specified in the response.
     * @return string representation of the content cache expiration time.
     */
    public String getPortletExpirationCache();
}
