/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.portlet.rendering;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.security.IUser;

/**
 * IPortletRenderingModel defines basic portlet rendering/action directives called by the portlet window manager.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public interface IPortletRenderingModel {   
    /**
     * Indicate that a portlet rendering can be initiated on a given portlet window
     * @param window portlet window
     * @param renderParams render request params
     * @param user user for which the request is being made
     * @param req http request
     * @param res http response
     */
    void startPortletRendering(IPortletWindow window, Map<String, String[]> renderParams, IUser user, HttpServletRequest req, HttpServletResponse res) throws PortletRenderingException;
        
    /**
     * Acquire portlet window content
     * @param window portlet window
     * @param req http request
     * @param res http response
     */
    IPortletRenderContent getPortletRenderContent(IPortletWindow window, HttpServletRequest req, HttpServletResponse res) throws PortletRenderingException;
    
    /**
     * Process portlet action
     * @param window portlet window
     * @param actionParams action params
     * @param user user for which the request is being made
     * @param req http request
     * @param res http response
     */
    void processPortletAction(IPortletWindow window, Map<String, String[]> actionParams, IUser user, HttpServletRequest req, HttpServletResponse res) throws PortletRenderingException;
}