/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.portlet.rendering;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.pluto.PortletContainer;
import org.apache.pluto.PortletWindow;
import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.IPlutoDomainObjectHelper;
import org.jasig.portal.portlet.registry.general.IPortletApplicationEntityRegistry;
import org.jasig.portal.portlet.registry.general.IPortletEntityRegistry;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.security.IUser;
import org.springframework.beans.factory.annotation.Required;

/**
 * LinearPortletRenderingModel is a base portlet rendering model implementation that renders portlets
 * within <code>startPortletRendering</code> method in the same thread.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class LinearPortletRenderingModel implements IPortletRenderingModel {
    protected IPlutoDomainObjectHelper plutoDomainObjectHelper;
    protected PortletContainer portletContainer;
    protected IRenderContentAccessor renderContentAccessor;
    protected ILocaleManager localeManager;
    protected IPortletEntityRegistry portletEntityRegistry;
    protected IPortletApplicationEntityRegistry portletApplicationEntityRegistry;

    
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletRenderingModel#getPortletRenderContent(org.jasig.portal.portlet.window.PortletWindowId, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public IPortletRenderContent getPortletRenderContent(IPortletWindow window, HttpServletRequest req, HttpServletResponse res) throws PortletRenderingException {
        PortletWindow plutoWindow=plutoDomainObjectHelper.getPortletWindow(window);
        IPortletRenderContent content=renderContentAccessor.getRenderedContent(plutoWindow.getId().toString(),req);
        return content;
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletRenderingModel#startPortletRendering(org.jasig.portal.portlet.window.IPortletWindow, java.util.Map, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public void startPortletRendering(IPortletWindow window, Map<String, String[]> renderParams, IUser user, HttpServletRequest req, HttpServletResponse res) throws PortletRenderingException {
        try {
            PortletWindow plutoWindow=plutoDomainObjectHelper.getPortletWindow(window);
            
            // construct wrapped request and response
            PortletHttpServletRequestWrapper wrappedRequest=new PortletHttpServletRequestWrapper(req, renderParams, user);
            PortletHttpServletResponseWrapper wrappedResponse=new PortletHttpServletResponseWrapper(plutoWindow.getId(),req,res,renderContentAccessor);
            
            
            // render
            portletContainer.doRender(plutoWindow,wrappedRequest,wrappedResponse);
            // record content
            renderContentAccessor.setRenderedContent(plutoWindow.getId().toString(),wrappedResponse.getPortletRenderContent(),req);
        } catch (Exception e) {
            throw new PortletRenderingException(window.getPortletWindowId(),e);
        }
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.portlet.rendering.IPortletRenderingModel#processPortletAction(org.jasig.portal.portlet.window.IPortletWindow, java.util.Map, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    public void processPortletAction(IPortletWindow window, Map<String, String[]> actionParams, IUser user, HttpServletRequest req, HttpServletResponse res) throws PortletRenderingException {
        PortletWindow plutoWindow=plutoDomainObjectHelper.getPortletWindow(window);
        PortletHttpServletRequestWrapper wrappedRequest=new PortletHttpServletRequestWrapper(req, actionParams, user);
        PortletHttpServletResponseWrapper wrappedResponse=new PortletHttpServletResponseWrapper(plutoWindow.getId(),req,res,renderContentAccessor);
        try {
            portletContainer.doAction(plutoWindow,wrappedRequest,wrappedResponse);
        } catch (Exception e) {
            throw new PortletRenderingException(window.getPortletWindowId(),e);
        }
    }

    /**
     * @param plutoDomainObjectHelper The plutoDomainObjectHelper to set.
     */
    @Required
    public void setPlutoDomainObjectHelper(IPlutoDomainObjectHelper plutoDomainObjectHelper) {
        this.plutoDomainObjectHelper = plutoDomainObjectHelper;
    }
    /**
     * @param portletContainerAccessor The portletContainerAccessor to set.
     */

    @Required
    public void setLocaleManager(ILocaleManager localeManager) {
        this.localeManager = localeManager;
    }

    public IPortletApplicationEntityRegistry getPortletApplicationEntityRegistry() {
        return this.portletApplicationEntityRegistry;
    }

    @Required
    public void setPortletApplicationEntityRegistry(IPortletApplicationEntityRegistry portletApplicationEntityRegistry) {
        this.portletApplicationEntityRegistry = portletApplicationEntityRegistry;
    }

    public IPortletEntityRegistry getPortletEntityRegistry() {
        return this.portletEntityRegistry;
    }

    @Required
    public void setPortletEntityRegistry(IPortletEntityRegistry portletEntityRegistry) {
        this.portletEntityRegistry = portletEntityRegistry;
    }

    @Required
    public void setPortletContainer(PortletContainer portletContainer) {
        this.portletContainer = portletContainer;
    }

    @Required
    public void setRenderContentAccessor(IRenderContentAccessor renderContentAccessor) {
        this.renderContentAccessor = renderContentAccessor;
    }
}
