/* Copyright 2007 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.rendering;

import java.util.Map;

import javax.portlet.PortletPreferences;
import javax.servlet.http.HttpServletRequest;

import org.apache.pluto.PortletContainer;
import org.apache.pluto.internal.InternalPortletWindow;
import org.apache.pluto.internal.impl.ActionRequestImpl;
import org.jasig.portal.portlet.IPlutoDomainObjectHelper;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.IPortletEntity;
import org.jasig.portal.portlet.userattributes.IMappedUserAttributeProvider;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.security.IUser;

/**
 * Provide custom logic for some RenderRequest methods
 * 
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public class PortletActionRequestImpl extends ActionRequestImpl {
    private final IMappedUserAttributeProvider mappedUserAttributeProvider;
    private final IPlutoDomainObjectHelper plutoDomainObjectHelper;
    private final IUser user;

    public PortletActionRequestImpl(PortletContainer container, InternalPortletWindow internalPortletWindow, HttpServletRequest servletRequest, IMappedUserAttributeProvider mappedUserAttributeProvider, IPlutoDomainObjectHelper plutoDomainObjectHelper, IUser user) {
        super(container, internalPortletWindow, servletRequest);
        this.mappedUserAttributeProvider = mappedUserAttributeProvider;
        this.plutoDomainObjectHelper = plutoDomainObjectHelper;
        this.user = user;
    }

    @Override
    public Map createUserInfoMap() {
        if (this.getUserPrincipal() == null) {
            return null;
        }
        
        final InternalPortletWindow plutoWindow = this.getInternalPortletWindow();
        final IPortletWindow portletWindow = this.plutoDomainObjectHelper.getPortletWindow(plutoWindow);
        final IPortletEntity portletEntity = portletWindow.getPortletEntity();
        final PortletApplicationEntityId portAppEntId = portletEntity.getPortletApplicationEntityId();
        
        final Map<String, String> mappedUserAttributes = this.mappedUserAttributeProvider.getMappedUserAttributes(this.user, portAppEntId);
        return mappedUserAttributes;
    }

    @Override
    public PortletPreferences getPreferences() {
        // TODO return custom uP3 impl
        return super.getPreferences();
    }
}
