/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.rendering;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.pluto.PortletContainer;
import org.apache.pluto.internal.InternalPortletWindow;
import org.apache.pluto.internal.impl.ActionResponseImpl;
import org.jasig.portal.portlet.IActionResponse;
import org.jasig.portal.portlet.IRedirectUrlConstructorsAccessor;
import org.jasig.portal.url.IUrlConstructor;
import org.jasig.portal.url.IUrlConstructorProvider;
import org.jasig.portal.url.IUrlConstructorProviderFactoryService;

/**
 * An extension of action response that provides access to redirect url constructors 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 1.0$
 */
public class PortletActionResponseImpl extends ActionResponseImpl implements IActionResponse {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PortletActionResponseImpl.class);

    private IUrlConstructorProviderFactoryService urlConstructorConfiguration;
    IRedirectUrlConstructorsAccessor constructorAccessor;

    public PortletActionResponseImpl(PortletContainer container,InternalPortletWindow internalPortletWindow, HttpServletRequest servletRequest, HttpServletResponse servletResponse,IUrlConstructorProviderFactoryService urlConstructorConfiguration, IRedirectUrlConstructorsAccessor constructorAccessor) {
        super(container,internalPortletWindow,servletRequest,servletResponse);
        this.urlConstructorConfiguration=urlConstructorConfiguration;
        this.constructorAccessor=constructorAccessor;
    }

    public Object getRedirectUrlConstructor(String name) {
        IUrlConstructor urlConstructor=constructorAccessor.getUrlConstructor(name);
        if(urlConstructor==null) {
            IUrlConstructorProvider urlProvider = (IUrlConstructorProvider) urlConstructorConfiguration.getUrlConstructorProvider(name);
            if(urlProvider!=null) {
                urlConstructor=urlProvider.getUrlConstructor();
                if(urlConstructor!=null) { // record instance in a map
                    constructorAccessor.addUrlConstructor(name,urlConstructor);
                    if (logger.isDebugEnabled()) {
                        logger.debug("getRedirectUrlConstructor(String) - added new redirect url constructor : name=" + name);
                    }
                } else {
                    logger.error("getRedirectUrlConstructor(String) - null constructor returned by a provider : name=" + name, null);
                }
            } else {
                logger.warn("getRedirectUrlConstructor(String) - unable to obtain url constructor provider for redirect url : name=" + name, null);
            }
        }
        return urlConstructor;
    }

}
