/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.rendering;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

import org.apache.pluto.PortletWindowID;

/**
 * An http response wrapper that is used in portlet action and render processing.
 * Allows to override methods from <code>HttpServletResponse</code>.
 * 
 * The only feature it currently adds is ability to set {@link org.jasig.portal.portlet.rendering.IPortletRenderContent}
 * in the case of render processing.
 *
 * @author Michael Ivanov, mivanov at unicon.net
 * @version $Revision: 11911 $
 *
 */

public class PortletHttpServletResponseWrapper extends HttpServletResponseWrapper {
    IRenderContentAccessor renderContentAccessor;
    HttpServletRequest req;
    PortletWindowID portletWindowId;
    
    public PortletHttpServletResponseWrapper(PortletWindowID windowId,HttpServletRequest request, HttpServletResponse response, IRenderContentAccessor renderContentAccessor) {
        super(response);
        this.req=request; this.portletWindowId=windowId; this.renderContentAccessor=renderContentAccessor; 
    }

    /**
     * Obtain render content instance associated with this response. 
     * @return render content associate with this response or <code>null</code>
     */
    public IPortletRenderContent getPortletRenderContent() {
        return renderContentAccessor.getRenderedContent(portletWindowId.toString(),req);
    }

    /**
     * Associate portlet render content instance with this response
     * @param portletRenderContent
     */
    public void setPortletRenderContent(IPortletRenderContent portletRenderContent) {
        renderContentAccessor.setRenderedContent(portletWindowId.toString(),portletRenderContent,this.req);
    }
    
}
