/*
 * Copyright 2004 The JA-SIG Collaborative. All rights reserved. See license distributed with this
 * file and available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.portlet.rendering;

import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.rendering.RenderingException;

/**
 * PortletRenderingException is a superclass of an exception reporting on portlet rendering errors.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletRenderingException extends RenderingException {
    private final PortletWindowId windowId;

    public PortletRenderingException(PortletWindowId windowId) {
        this(windowId, (Throwable)null);
    }

    public PortletRenderingException(PortletWindowId windowId, Throwable cause) {
        this(windowId, "Problem occurred in rendering portlet window id=" + windowId, cause);
    }
    
    public PortletRenderingException(PortletWindowId windowId, String message) {
        this(windowId, message, null);
    }
    
    public PortletRenderingException(PortletWindowId windowId, String message, Throwable cause) {
        super(message, cause);
        this.windowId = windowId;
    }

    /**
     * @return Returns the windowId.
     */
    public PortletWindowId getWindowId() {
        return windowId;
    }
}
