/*
 * Copyright 2004 The JA-SIG Collaborative. All rights reserved. See license distributed with this
 * file and available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.portlet.rendering;

import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * PortletRenderingTimeoutException is used when portlet exceeded its allowed rendering time
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletRenderingTimeoutException extends PortletRenderingException {
    private final long allowedTime;
    private final long usedTime;

    public PortletRenderingTimeoutException(PortletWindowId windowId, long allowedTime, long usedTime) {
        super(windowId, "Portlet window (id=" + windowId + ") exceeded allowed rendering time (allowed " + allowedTime + " msec, used " + usedTime + " msec).");
        this.allowedTime = allowedTime;
        this.usedTime = usedTime;
    }

    /**
     * @return Returns the allowedTime.
     */
    public long getAllowedTime() {
        return allowedTime;
    }

    /**
     * @return Returns the usedTime.
     */
    public long getUsedTime() {
        return usedTime;
    }
}
