/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/
package org.jasig.portal.portlet.servant;

import javax.portlet.PortletRequest;


/** An interface that declares methods for finding and returning a new instance of a portlet servant.
 *
 * @author Michael Ivanov - mivanov at unicon.net
 * @version $Revision: 11911 $
 */

public interface IPortletServantLocator {
    
    /**
     * Obtain new or existing portlet servant instance
     * @param functionalPortletName servant functional name
     * @param request current request
     * @param createNew whether a new servant instance should be created if one currently doesn't exist 
     * @return portlet servant or <code>null</code> if no current instance is found and createNew is <code>false</code>
     */
    public IPortletServant getServant(String functionalPortletName, PortletRequest request, boolean createNew);
    
    /**
     * Obtain new or existing portlet servant instance
     * @param functionalPortletName servant functional name
     * @param instanceKey distinguishes between multiple servants with the same functional name
     * @param request current request
     * @param createNew whether a new servant instance should be created if one currently doesn't exist 
     * @return portlet servant or <code>null</code> if no current instance is found and createNew is <code>false</code>
     */
    public IPortletServant getServant(String functionalPortletName, String instanceKey, PortletRequest request, boolean createNew);
}
