/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.portlet.servant;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.servlet.http.HttpServletRequest;

import org.jasig.portal.context.common.IHttpRequestResponseAccessor;
import org.jasig.portal.portlet.rendering.IPortletRenderContent;
import org.jasig.portal.portlet.rendering.IPortletWindowManager;
import org.jasig.portal.portlet.window.IPortletWindow;

/** An implementation of <code>IPortletServant</code> interface.
 * 
 * @author Michael Ivanov - mivanov at unicon.net
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class PortletServantImpl implements IPortletServant {
    private final IPortletWindow portletWindow;
    private final IPortletWindowManager portletWindowManager;
    private final IHttpRequestResponseAccessor requestAccessor;

    public PortletServantImpl(IPortletWindow portletWindow, IPortletWindowManager portletWindowManager, IHttpRequestResponseAccessor requestAccessor) {
        this.portletWindow = portletWindow;
        this.portletWindowManager = portletWindowManager;
        this.requestAccessor = requestAccessor;
    }

    public IPortletRenderContent getRenderContent(RenderRequest request, RenderResponse response) {
        //set isServantMode flag to true
        //TODO is this thread safe?
        final HttpServletRequest currentHttpServletRequest = this.requestAccessor.getCurrentHttpServletRequest();
        currentHttpServletRequest.setAttribute(ServantUtil.IN_SERVANT_MODE, Boolean.TRUE);
        
        //use portletWindowManager to render the servant
        final IPortletRenderContent portletRenderContent = this.portletWindowManager.getPortletRenderContent(this.portletWindow.getPortletWindowId());
        
        //clear isServantMode flag
        currentHttpServletRequest.removeAttribute(ServantUtil.IN_SERVANT_MODE);
        
        return portletRenderContent;
    }

    public String getId() {
        return this.portletWindow.getPortletWindowId().toString();
    }
    
    public boolean isComplete(PortletRequest request) {
        final Boolean isDone = (Boolean)request.getAttribute(ServantUtil.SERVANT_DONE);
        return isDone != null && isDone;
    }

    public Object getAttribute(PortletRequest request, String key) {
        return request.getAttribute(key);
    }

    public void setAttribute(PortletRequest request, String key, Object value) {
        //TODO is this thread safe?
        final HttpServletRequest currentHttpServletRequest = this.requestAccessor.getCurrentHttpServletRequest();
        currentHttpServletRequest.setAttribute(key, value);
    }

    public void render(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        final IPortletRenderContent portletContent = this.getRenderContent(request, response);
        response.setContentType(portletContent.getContentType());
        final PrintWriter writer = response.getWriter();
        writer.print(portletContent.getCharacterOutput());
        writer.flush();
    }

    public void processAction(Map<String, String[]> actionRequestParameterMap, ActionRequest request, ActionResponse response) throws PortletException, IOException {
        //set isServantMode flag to true
        //TODO is this thread safe?
        final HttpServletRequest currentHttpServletRequest = this.requestAccessor.getCurrentHttpServletRequest();
        currentHttpServletRequest.setAttribute(ServantUtil.IN_SERVANT_MODE, Boolean.TRUE);
        
        //use portletWindowManager to render the servant
        this.portletWindowManager.setPortletActionRequestParameters(this.portletWindow.getPortletWindowId(), actionRequestParameterMap);
        
        //clear isServantMode flag
        currentHttpServletRequest.removeAttribute(ServantUtil.IN_SERVANT_MODE);
    }

    public void reset(PortletRequest request) {
        //Use the portlet window manager to reset the servant
        this.portletWindowManager.restartPortletWindow(this.portletWindow.getPortletWindowId());
    }

    public Object getResult(PortletRequest request) {
        //Gets the result object from the servant
        return request.getAttribute(ServantUtil.RETURN_VALUE);
    }

    public boolean isCancelled(PortletRequest request) {
        final Boolean isCancelled = (Boolean)request.getAttribute(ServantUtil.SERVANT_CANCLED);
        return isCancelled != null && isCancelled;
    }
}
