/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.portlet.servant;

import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;

import org.jasig.portal.context.common.IHttpRequestResponseAccessor;
import org.jasig.portal.core.exception.PortalRuntimeException;
import org.jasig.portal.portlet.registry.transitory.ITransientPortletWindowRegistry;
import org.jasig.portal.portlet.rendering.IPortletWindowManager;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.springframework.beans.factory.annotation.Required;

/** An implementation of  <code>IPortletServantLocator</code>.
 *
 * @author Michael Ivanov - mivanov at unicon.net
 * @version $Revision: 11918 $
 */

public class PortletServantLocatorImpl implements IPortletServantLocator {
    private ITransientPortletWindowRegistry transientPortletWindowRegistry;
    private IPortletWindowManager portletWindowManager;
    private IHttpRequestResponseAccessor requestAccessor;

    private static String PORTLET_SERVANT_PREFIX = "org.jasig.portal.portlet.servant.";

    private String getAttributeKey(String fname, String instanceKey) {
        final StringBuilder key = new StringBuilder();
        key.append(PORTLET_SERVANT_PREFIX);
        key.append(fname);
        if (instanceKey != null) {
            key.append(".");
            key.append(instanceKey);
        }
        
        return key.toString();
    }

    /**
     * Gets a new IPortletServant instance for running the specified portlet as a servant.
     **/
    public IPortletServant getServant(String functionalName, PortletRequest request, boolean createNew) {
        return this.getServant(functionalName, null, request, createNew);
    }

    public IPortletServant getServant(String functionalName, String instanceKey, PortletRequest request, boolean createNew) {
        final PortletSession session = request.getPortletSession();
        final String servantKey = this.getAttributeKey(functionalName, instanceKey);
        IPortletServant servant = (IPortletServant)session.getAttribute(servantKey);

        if (servant == null && createNew) {
            final IPortletWindow window = this.transientPortletWindowRegistry.createTransientPortletWindowControl(functionalName);

            if (window == null) {
                throw new PortalRuntimeException("The servant locator could not find the portlet definition with the functional name=" + functionalName);
            }
            
            servant = new PortletServantImpl(window, this.portletWindowManager, this.requestAccessor);
            session.setAttribute(servantKey, servant);
        }

        return servant;
    }

    /**
     * @return the requestAccessor
     */
    public IHttpRequestResponseAccessor getRequestAccessor() {
        return this.requestAccessor;
    }
    /**
     * @param requestAccessor the requestAccessor to set
     */
    @Required
    public void setRequestAccessor(IHttpRequestResponseAccessor requestAccessor) {
        this.requestAccessor = requestAccessor;
    }
    /**
     * @return the portletWindowManager
     */
    public IPortletWindowManager getPortletWindowManager() {
        return this.portletWindowManager;
    }
    /**
     * @param portletWindowManager the portletWindowManager to set
     */
    @Required
    public void setPortletWindowManager(IPortletWindowManager portletWindowManager) {
        this.portletWindowManager = portletWindowManager;
    }
    /**
     * @return the transientPortletWindowRegistry
     */
    public ITransientPortletWindowRegistry getTransientPortletWindowRegistry() {
        return this.transientPortletWindowRegistry;
    }
    /**
     * @param transientPortletWindowRegistry the transientPortletWindowRegistry to set
     */
    @Required
    public void setTransientPortletWindowRegistry(ITransientPortletWindowRegistry transientPortletWindowRegistry) {
        this.transientPortletWindowRegistry = transientPortletWindowRegistry;
    }
}
