/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.transfer;

import java.io.Serializable;
import java.util.Locale;
import java.util.Map;

import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;

/**
 * Representation of a portlet definition
 * required for persistence.
 * @author Michael Ivanov, mivanov@unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletDefinitionTransfer extends Serializable {

    /**
     * Returns the identifier of this object.
     * @return the object identifier
     */
    public PortletDefinitionId getPortletDefinitionId();
    
    /**
     * Sets the identifier of this object.
     * @param id the object identifier
     */
    public void setPortletDefinitionId(PortletDefinitionId id);
    
    /**
     * @return The unique functional name for this portlet definition, may not be null.
     */
    public String getFunctionalName();
    
    /**
     * @param functionalName The unique functional name for the portlet definition, may not be null.
     */
    public void setFunctionalName(String functionalName);

    /**
     * Returns the ID of the portlet deployment
     * that this portlet definition is based on.
     * @return the portlet deployment ID
     */
    public PortletDeploymentId getPortletDeploymentId();

    /**
     * Sets the ID of the portlet deployment that this portlet is based on.
     * @param portletDeploymentId the portlet deployment ID of this portlet
     */
    public void setPortletDeploymentId(PortletDeploymentId portletDeploymentId);

    /**
     * Returns the ID of the portlet application definition that this portlet resides in.
     * @return the portlet application definition ID of this portlet
     */
    public PortletApplicationDefinitionId getPortletApplicationDefinitionId();

    /**
     * Sets the ID of the portlet application definition that this portlet resides in.
     * @param porletApplicationDefinitionId the portlet application definition ID of this portlet
     */
    public void setPortletApplicationDefinitionId(PortletApplicationDefinitionId porletApplicationDefinitionId);

    /**
     * Returns the display names of this portlet.
     * The return value may be NULL.
     * @return display names for this portlet
     */
    public Map<Locale, IDisplayName> getDisplayNames();

    /**
     * Sets the display names of this portlet.
     * May be NULL.
     * @param displayNames display names for this portlet
     */
    public void setDisplayNames(Map<Locale, IDisplayName> displayNames);
    
    /**
     * Returns the descriptions for this portlet.
     * The return value may be NULL.
     * @return the descriptions for this portlet
     */
    public Map<Locale, IDescription> getDescriptions();
    
    /**
     * Sets the descriptions for this portlet.
     * May be NULL.
     * @param descriptions the descriptions for this portlet
     */
    public void setDescriptions(Map<Locale, IDescription> descriptions);
    
    /**
     * @return Gets the portlet info for the defintion transfer, may be null.
     */
    public IPortletInfo getPortletInfo();
    
    /**
     * @param portletInfo Sets the portlet info for the defintion transfer, may be null.
     */
    public void setPortletInfo(IPortletInfo portletInfo);
    
    /**
     * Returns the preferences associated with this portlet definition.
     * @return the preferences for this portlet definition
     */
    public IPreferences getPreferences();
    
    /**
     * Sets the preferences assiciated with this portlet definition.
     * @param portletDefinitionPreferences the preferences for this portlet definition
     */
    public void setPreferences(IPreferences portletDefinitionPreferences);
    
    /**
     * Returns the time out value defined during publishing the portlet.
     * @return the time out value
     */
    public long getRenderTimeOut();
    
   /**
    * Sets the time out value defined during publishing the portlet.
    * @param timeOut the time out value
    */
   public void setRenderTimeOut(long timeOut);
}
