/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.transfer;

import java.io.Serializable;

import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.security.UserId;

/**
 * Representation of a portlet entity
 * required for persistence.
 * @author Michael Ivanov, mivanov@unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletEntityTransfer extends Serializable {

    /**
     * Returns the identifier of this object.
     * @return the object identifier
     */
    public PortletEntityId getPortletEntityId();
    
    /**
     * Sets the identifier of this object.
     * @param id the object identifier
     */
    public void setPortletEntityId(PortletEntityId id);

    /**
     * Returns the ID of the portlet definition
     * that this portlet entity is based on.
     * @return the portlet definition ID
     */
    public PortletDefinitionId getPortletDefinitionId();

    /**
     * Sets the ID of the portlet definition that this portlet entity is based on.
     * @param portletDefinitionId the portlet definition ID of this portlet entity
     */
    public void setPortletDefinitionId(PortletDefinitionId portletDefinitionId);

    /**
     * Returns the ID of the portlet application entity that this portlet entity resides in.
     * @return the portlet application entity ID of this portlet entity
     */
    public PortletApplicationEntityId getPortletApplicationEntityId();

    /**
     * Sets the ID of the portlet application entity that this portlet entity resides in.
     * @param porletApplicationEntityId the portlet application entity ID of this portlet entity
     */
    public void setPortletApplicationEntityId(PortletApplicationEntityId porletApplicationEntityId);
    
    /**
     * @return The UserId associated with this object.
     */
    public UserId getUserId();

    /**
     * @param userId The UserId associted with this object.
     */
    public void setUserId(UserId userId);
    
    /**
     * Returns the preferences associated with this portlet entity.
     * @return the preferences for this portlet entity
     */
    public IPreferences getPreferences();
    
    /**
     * Sets the preferences assiciated with this portlet entity.
     * @param portletEntityPreferences the preferences for this portlet entity
     */
    public void setPreferences(IPreferences portletEntityPreferences);
    
    /**
     * Returns the time out value defined during publishing the portlet.
     * @return the time out value
     */
    public long getRenderTimeOut();
    
    /**
     * Sets the time out value defined during publishing the portlet.
     * @param timeOut the time out value
     */
    public void setRenderTimeOut(long timeOut);
}
