/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.transfer;

import java.io.Serializable;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * Representation of a portlet window
 * required for persistence.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletWindowTransfer extends Serializable {

    /**
     * Returns the identifier of this object.
     * @return the object identifier
     */
    public PortletWindowId getPortletWindowId();
    
    /**
     * Sets the identifier of this object.
     * @param id the object identifier
     */
    public void setPortletWindowId(PortletWindowId id);

    /**
     * Returns the ID of the portlet entity
     * that this portlet window is based on.
     * @return the portlet entity ID
     */
    public PortletEntityId getPortletEntityId();

    /**
     * Sets the ID of the portlet entity that this portlet window is based on.
     * @param portletEntityId the portlet entity ID of this portlet window
     */
    public void setPortletEntityId(PortletEntityId portletEntityId);
    
    /**
     * Returns the window state of the corresponding portlet window.
     * @return the window state
     */
    public WindowState getWindowState();
    
    /**
     * Returns the portlet mode of the corresponding portlet window.
     * @return the portlet mode
     */
    public PortletMode getPortletMode();
    
    /**
     * Sets the window state of the corresponding portlet window.
     * @param windowState window state
     */
    public void setWindowState( WindowState windowState );
    
    /**
     * Sets the portlet mode of the corresponding portlet window.
     * @param portletMode portlet mode
     */
    public void setPortletMode( PortletMode portletMode );
    
}
