/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.transfer.impl;

import java.util.Collections;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.transfer.IPortletApplicationDefinitionTransfer;

/**
 * Implementation of a portlet application definition transfer object.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11955 $
 */
public class PortletApplicationDefinitionTransferImpl implements IPortletApplicationDefinitionTransfer {
    
    private PortletApplicationDefinitionId id;
    private PortletApplicationDeploymentId portletApplicationDeploymentId;
    
    private Map<Locale, IDisplayName> displayNames = new HashMap<Locale, IDisplayName>();
    private Map<Locale, IDescription> descriptions = new HashMap<Locale, IDescription>();
    private Map<PortletMode, PortletMode> portletModeMappings = Collections.emptyMap();
    private Map<WindowState, WindowState> windowStateMappings = Collections.emptyMap();
    private Map<String, String> userAttributeMappings = Collections.emptyMap();

    public PortletApplicationDefinitionId getPortletApplicationDefinitionId() {
        return id;
    }
    
    public void setPortletApplicationDefinitionId(PortletApplicationDefinitionId id) {
        this.id = id;
    }
    
    public PortletApplicationDeploymentId getPortletApplicationDeploymentId() {
        return portletApplicationDeploymentId;
    }
  
    public void setPortletApplicationDeploymentId(PortletApplicationDeploymentId portletApplicationDeploymentId) {
        this.portletApplicationDeploymentId = portletApplicationDeploymentId;
    }
    
    public void setDisplayNames(Map<Locale, IDisplayName> displayNames) {
        if (displayNames == null) {
            throw new IllegalArgumentException("displayNames cannot be null");
        }
        
        this.displayNames = displayNames;
    }

    public Map<Locale, IDisplayName> getDisplayNames() {
        return this.displayNames;
    }

    public void setDescriptions(Map<Locale, IDescription> descriptions) {
        if (descriptions == null) {
            throw new IllegalArgumentException("descriptions cannot be null");
        }
        
        this.descriptions = descriptions;
    }

    public Map<Locale, IDescription> getDescriptions() {
        return this.descriptions;
    }

    public void setPortletModeMappings(Map<PortletMode, PortletMode> mappings) {
        if (mappings == null) {
            this.portletModeMappings = Collections.emptyMap();
        }
        else {
            //Create appropriately sized typesafe Map
            final Map<PortletMode, PortletMode> checkedMappings = new HashMap<PortletMode, PortletMode>(mappings.size());
            //Clone the mappings (does type checking as well)
            checkedMappings.putAll(mappings);
            //Ensure the cloned map is now immutable
            this.portletModeMappings = Collections.unmodifiableMap(checkedMappings);
        }
    }

    public Map<PortletMode, PortletMode> getPortletModeMappings() {
        return this.portletModeMappings;
    }

    public void setWindowStateMappings(Map<WindowState, WindowState> mappings) {
        if (mappings == null) {
            this.windowStateMappings = Collections.emptyMap();
        }
        else {
            //Create appropriately sized typesafe Map
            final Map<WindowState, WindowState> checkedMappings = new HashMap<WindowState, WindowState>(mappings.size());
            //Clone the mappings (does type checking as well)
            checkedMappings.putAll(mappings);
            //Ensure the cloned map is now immutable
            this.windowStateMappings = Collections.unmodifiableMap(checkedMappings);
        }
    }

    public Map<WindowState, WindowState> getWindowStateMappings() {
        return this.windowStateMappings;
    }

    public void setUserAttributeMappings(Map<String, String> mappings) {
        if (mappings == null) {
            this.userAttributeMappings = Collections.emptyMap();
        }
        else {
            //Create appropriately sized typesafe Map
            final Map<String, String> checkedMappings = new HashMap<String, String>(mappings.size());
            //Clone the mappings (does type checking as well)
            checkedMappings.putAll(mappings);
            //Ensure the cloned map is now immutable
            this.userAttributeMappings = Collections.unmodifiableMap(checkedMappings);
        }
    }

    public Map<String, String> getUserAttributeMappings() {
        return userAttributeMappings;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
                .append("portletApplicationDefinitionId", this.id)
                .append("portletApplicationDeploymentId", this.portletApplicationDeploymentId)
                .append("displayNames", this.displayNames)
                .append("descriptions", this.descriptions)
                .append("windowStateMappings", this.windowStateMappings)
                .append("portletModeMappings", this.portletModeMappings)
                .append("userAttributeMappings", this.userAttributeMappings)
                .toString();
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof IPortletApplicationDefinitionTransfer)) {
            return false;
        }
        IPortletApplicationDefinitionTransfer rhs = (IPortletApplicationDefinitionTransfer)object;
        return new EqualsBuilder()
            .append(this.descriptions, rhs.getDescriptions())
            .append(this.displayNames, rhs.getDisplayNames())
            .append(this.portletModeMappings, rhs.getPortletModeMappings())
            .append(this.windowStateMappings, rhs.getWindowStateMappings())
            .append(this.portletApplicationDeploymentId, rhs.getPortletApplicationDeploymentId())
            .append(this.userAttributeMappings, rhs.getUserAttributeMappings())
            .append(this.id, rhs.getPortletApplicationDefinitionId())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(-764623465, -633146475)
            .appendSuper(super.hashCode())
            .append(this.descriptions)
            .append(this.displayNames)
            .append(this.portletModeMappings)
            .append(this.windowStateMappings)
            .append(this.portletApplicationDeploymentId)
            .append(this.userAttributeMappings)
            .append(this.id)
            .toHashCode();
    }
}
