/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.transfer.impl;

import java.io.Serializable;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.transfer.IPortletApplicationEntityTransfer;
import org.jasig.portal.security.UserId;

/**
 * Implementation of a portlet application entity transfer object.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11955 $
 */
public class PortletApplicationEntityTransferImpl implements IPortletApplicationEntityTransfer, Serializable {

    private PortletApplicationEntityId id;
    private PortletApplicationDefinitionId portletApplicationDefinitionId;
    private UserId userId;
    
    
    public PortletApplicationEntityTransferImpl() {};
    
    /**
     * Copy constructor
     * @param transfer
     */
    public PortletApplicationEntityTransferImpl(IPortletApplicationEntityTransfer transfer) {
        this.id=transfer.getPortletApplicationEntityId();
        this.portletApplicationDefinitionId=transfer.getPortletApplicationDefinitionId();
        this.userId=transfer.getUserId();
        TransferUtils.deepCopyPortletApplicationEntityTransferProperties(transfer,this);
    }

    public PortletApplicationEntityId getPortletApplicationEntityId() {
        return id;
    }

    public void setPortletApplicationEntityId(final PortletApplicationEntityId id) {
        this.id = id;
    }

    public UserId getUserId() {
		return userId;
	}

	public void setUserId(UserId userId) {
		this.userId = userId;
	}

	public PortletApplicationDefinitionId getPortletApplicationDefinitionId() {
        return portletApplicationDefinitionId;
    }

    public void setPortletApplicationDefinitionId(final PortletApplicationDefinitionId portletApplicationDefinitionId) {
        this.portletApplicationDefinitionId = portletApplicationDefinitionId;
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof PortletApplicationEntityTransferImpl)) {
            return false;
        }
        PortletApplicationEntityTransferImpl rhs = (PortletApplicationEntityTransferImpl)object;
        return new EqualsBuilder()
            .append(this.portletApplicationDefinitionId, rhs.portletApplicationDefinitionId)
            .append(this.id, rhs.id)
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(-806870903, 1638293155)
            .appendSuper(super.hashCode())
            .append(this.portletApplicationDefinitionId)
            .append(this.id)
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
            .appendSuper(super.toString())
            .append("portletApplicationDefinitionId", this.portletApplicationDefinitionId)
            .append("portletApplicationEntityId", this.id)
            .toString();
    }
}
