/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.transfer.impl;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.common.PortletInfoImpl;
import org.jasig.portal.portlet.common.PreferencesImpl;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;
import org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider;
import org.jasig.portal.portlet.transfer.IPortletDefinitionTransfer;

/**
 * Implementation of portlet definition transfer object.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public class PortletDefinitionTransferImpl implements IPortletDefinitionTransfer {
    private PortletDefinitionId id;
    private String functionalName;
    private IPreferences preferences = new PreferencesImpl();
    private PortletDeploymentId portletDeploymentId;
    private PortletApplicationDefinitionId portletApplicationDefinitionId;
    private long renderTimeOut = IPortletDeploymentInformationProvider.UNDEFINED_TIMEOUT;
    private Map<Locale, IDescription> descriptions = new HashMap<Locale, IDescription>();
    private Map<Locale, IDisplayName> displayNames = new HashMap<Locale, IDisplayName>();
    private IPortletInfo portletInfo = new PortletInfoImpl();
    

    // IPortletDefinitionTransfer methods
    
    public PortletDefinitionId getPortletDefinitionId() {
        return id;
    }
    
    public void setPortletDefinitionId(PortletDefinitionId id) {
        this.id = id;
    }
    
    public String getFunctionalName() {
        return functionalName;
    }

    public void setFunctionalName(String functionalName) {
        this.functionalName = functionalName;
    }

    public IPreferences getPreferences() {
        return this.preferences;
    }

    public void setPreferences(IPreferences preferences) {
        if (preferences == null) {
            throw new IllegalArgumentException("preferences may not be null");
        }
        
        this.preferences = preferences;
    }
    
    public PortletDeploymentId getPortletDeploymentId() {
        return portletDeploymentId;
    }
    
    public void setPortletDeploymentId(PortletDeploymentId portletDeploymentId) {
        this.portletDeploymentId = portletDeploymentId;   
    }
    
    public PortletApplicationDefinitionId getPortletApplicationDefinitionId() {
        return portletApplicationDefinitionId;
    }
    
    public void setPortletApplicationDefinitionId(PortletApplicationDefinitionId portletApplicationDefinitionId) {
        this.portletApplicationDefinitionId = portletApplicationDefinitionId;
    }
    
    public long getRenderTimeOut() {
      return renderTimeOut;
    }
    
    public void setRenderTimeOut(long renderTimeOut) {
      this.renderTimeOut = renderTimeOut;
    }

    public Map<Locale, IDescription> getDescriptions() {
        return this.descriptions;
    }

    public Map<Locale, IDisplayName> getDisplayNames() {
        return this.displayNames;
    }

    public void setDescriptions(Map<Locale, IDescription> descriptions) {
        this.descriptions = descriptions;
    }

    public void setDisplayNames(Map<Locale, IDisplayName> displayNames) {
        this.displayNames = displayNames;
    }

    public IPortletInfo getPortletInfo() {
        return this.portletInfo;
    }

    public void setPortletInfo(IPortletInfo portletInfo) {
        this.portletInfo = portletInfo;
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof IPortletDefinitionTransfer)) {
            return false;
        }
        IPortletDefinitionTransfer rhs = (IPortletDefinitionTransfer)object;
        return new EqualsBuilder()
            .append(this.functionalName, rhs.getFunctionalName())
            .append(this.descriptions, rhs.getDescriptions())
            .append(this.displayNames, rhs.getDisplayNames())
            .append(this.portletApplicationDefinitionId, rhs.getPortletApplicationDefinitionId())
            .append(this.preferences, rhs.getPreferences())
            .append(this.renderTimeOut, rhs.getRenderTimeOut())
            .append(this.portletDeploymentId, rhs.getPortletDeploymentId())
            .append(this.id, rhs.getPortletDefinitionId())
            .append(this.portletInfo, rhs.getPortletInfo())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(-1119486397, -1121863227)
            .appendSuper(super.hashCode())
            .append(this.functionalName)
            .append(this.descriptions)
            .append(this.displayNames)
            .append(this.portletApplicationDefinitionId)
            .append(this.preferences)
            .append(this.renderTimeOut)
            .append(this.portletDeploymentId)
            .append(this.id)
            .append(this.portletInfo)
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
            .appendSuper(super.toString())
            .append("portletDefinitionId", this.id)
            .append("portletApplicationDefinitionId", this.portletApplicationDefinitionId)
            .append("portletDeploymentId", this.portletDeploymentId)
            .append("functionalName", this.functionalName)
            .append("displayNames", this.displayNames)
            .append("descriptions", this.descriptions)
            .append("portletInfo", this.portletInfo)
            .append("preferences", this.preferences)
            .append("renderTimeOut", this.renderTimeOut)
            .toString();
    }
}
