/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.transfer.impl;

import java.io.Serializable;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.common.PreferencesImpl;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.deployment.IPortletDeploymentInformationProvider;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.transfer.IPortletEntityTransfer;
import org.jasig.portal.security.UserId;

/**
 * Implementation of portlet entity transfer object.
 * @author Michael Ivanov, mivanov@unicon.net
 * @version $Revision: 11955 $
 */
public class PortletEntityTransferImpl implements IPortletEntityTransfer, Serializable {
    
    private PortletEntityId id;
    private PortletDefinitionId portletDefinitionId;
    private PortletApplicationEntityId portletApplicationEntityId;
    private IPreferences preferences = new PreferencesImpl();
    private long renderTimeOut = IPortletDeploymentInformationProvider.UNDEFINED_TIMEOUT;
    private UserId userId;
    private static final long serialVersionUID = 1l;
    
    public PortletEntityTransferImpl() {};
    /**
     * Copy constructor
     * @param transfer template
     */
    public PortletEntityTransferImpl(IPortletEntityTransfer transfer) {
        this.id=transfer.getPortletEntityId();
        this.portletDefinitionId=transfer.getPortletDefinitionId();
        this.portletApplicationEntityId=transfer.getPortletApplicationEntityId();
        this.userId=transfer.getUserId();
        TransferUtils.deepCopyPortletEntityTransferProperties(transfer,this);
    }

    public PortletEntityId getPortletEntityId() {
        return id;
    }

    public void setPortletEntityId(final PortletEntityId id) {
        this.id = id;
    }

    public UserId getUserId() {
		return userId;
	}

	public void setUserId(UserId userId) {
		this.userId = userId;
	}

	public PortletDefinitionId getPortletDefinitionId() {
        return portletDefinitionId;
    }

    public void setPortletDefinitionId(final PortletDefinitionId portletDefinitionId) {
        this.portletDefinitionId = portletDefinitionId;
    }

    public PortletApplicationEntityId getPortletApplicationEntityId() {
        return portletApplicationEntityId;
    }

    public void setPortletApplicationEntityId(final PortletApplicationEntityId porletApplicationEntityId) {
        this.portletApplicationEntityId = porletApplicationEntityId;
    }

    public IPreferences getPreferences() {
        return preferences;
    }

    public void setPreferences(final IPreferences portletEntityPreferences) {
        if (portletEntityPreferences == null) {
            throw new IllegalArgumentException("portletEntityPreferences may not be null");
        }
            
        this.preferences = portletEntityPreferences;
    }

    /**
     * @return Returns the renderTimeOut.
     */
    public long getRenderTimeOut() {
        return renderTimeOut;
    }
    /**
     * @param renderTimeOut The renderTimeOut to set.
     */
    public void setRenderTimeOut(long renderTimeOut) {
        this.renderTimeOut = renderTimeOut;
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof PortletEntityTransferImpl)) {
            return false;
        }
        PortletEntityTransferImpl rhs = (PortletEntityTransferImpl)object;
        return new EqualsBuilder()
            .append(this.portletDefinitionId, rhs.portletDefinitionId)
            .append(this.preferences, rhs.preferences)
            .append(this.portletApplicationEntityId, rhs.portletApplicationEntityId)
            .append(this.renderTimeOut, rhs.renderTimeOut)
            .append(this.id, rhs.id)
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(403664007, 112279195)
            .appendSuper(super.hashCode())
            .append(this.portletDefinitionId)
            .append(this.preferences)
            .append(this.portletApplicationEntityId)
            .append(this.renderTimeOut)
            .append(this.id)
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
            .appendSuper(super.toString())
            .append("portletApplicationEntityId", this.portletApplicationEntityId)
            .append("portletEntityId", this.id)
            .append("userId", this.userId)
            .append("preferences", this.preferences)
            .append("renderTimeOut", this.renderTimeOut)
            .append("portletDefinitionId", this.portletDefinitionId)
            .toString();
    }
}
