/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.transfer.impl;


import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.transfer.IPortletWindowTransfer;
import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * Implementation of a portlet window
 * required for persistence.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public class PortletWindowTransferImpl implements IPortletWindowTransfer {
	
	private PortletWindowId portletWindowId;
	private PortletEntityId portletEntityId;
	
    // default mode
	private transient PortletMode portletMode = PortletMode.VIEW;
	// default window state
	private transient WindowState windowState = WindowState.NORMAL;
    
    
    public PortletWindowTransferImpl() {};
    /**
     * Copy constructor
     * @param transfer a transfer object to copy information from
     */
    public PortletWindowTransferImpl(IPortletWindowTransfer transfer) {
        this.portletEntityId=transfer.getPortletEntityId();
        this.portletWindowId=transfer.getPortletWindowId();
        TransferUtils.deepCopyPortletWindowTransferProperties(transfer,this);
    }
	
	 /**
     * Returns the identifier of this object.
     * @return the object identifier
     */
    public PortletWindowId getPortletWindowId() {
    	return portletWindowId;
    }
    
    /**
     * Sets the identifier of this object.
     * @param id the object identifier
     */
    public void setPortletWindowId(PortletWindowId id) {
    	this.portletWindowId = id;
    }

    /**
     * Returns the ID of the portlet entity
     * that this portlet window is based on.
     * @return the portlet entity ID
     */
    public PortletEntityId getPortletEntityId() {
    	return portletEntityId;
    }

    /**
     * Sets the ID of the portlet entity that this portlet window is based on.
     * @param portletEntityId the portlet entity ID of this portlet window
     */
    public void setPortletEntityId(PortletEntityId portletEntityId) {
    	this.portletEntityId = portletEntityId;
    }
    
    /**
     * Returns the window state of the corresponding portlet window.
     * @return the window state
     */
    public WindowState getWindowState() {
    	return windowState;
    }
    
    /**
     * Returns the portlet mode of the corresponding portlet window.
     * @return the portlet mode
     */
    public PortletMode getPortletMode() {
    	return portletMode;
    }
    
    /**
     * Sets the window state of the corresponding portlet window.
     * @param windowState window state
     */
    public void setWindowState( WindowState windowState ) {
    	this.windowState = windowState;
    }
    
    /**
     * Sets the portlet mode of the corresponding portlet window.
     * @param portletMode portlet mode
     */
    public void setPortletMode( PortletMode portletMode ) {
    	this.portletMode = portletMode;
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof PortletWindowTransferImpl)) {
            return false;
        }
        PortletWindowTransferImpl rhs = (PortletWindowTransferImpl)object;
        return new EqualsBuilder()
            .append(this.windowState, rhs.windowState)
            .append(this.portletEntityId, rhs.portletEntityId)
            .append(this.portletWindowId, rhs.portletWindowId)
            .append(this.portletMode, rhs.portletMode)
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(-1294579115, 812697433)
            .appendSuper(super.hashCode())
            .append(this.windowState)
            .append(this.portletEntityId)
            .append(this.portletWindowId)
            .append(this.portletMode)
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
            .appendSuper(super.toString())
            .append("portletEntityId", this.portletEntityId)
            .append("portletWindowId", this.portletWindowId)
            .append("windowState", this.windowState)
            .append("portletMode", this.portletMode)
            .toString();
    }
}
