/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.transfer.impl;

import java.util.HashMap;
import java.util.Map;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.common.PortletInfoImpl;
import org.jasig.portal.portlet.common.PreferencesImpl;
import org.jasig.portal.portlet.transfer.IPortletApplicationDefinitionTransfer;
import org.jasig.portal.portlet.transfer.IPortletApplicationEntityTransfer;
import org.jasig.portal.portlet.transfer.IPortletDefinitionTransfer;
import org.jasig.portal.portlet.transfer.IPortletEntityTransfer;
import org.jasig.portal.portlet.transfer.IPortletWindowTransfer;

/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public final class TransferUtils {
    private TransferUtils() {
    };
    
    private static final Log LOG = LogFactory.getLog(TransferUtils.class);
    
    public static void deepCopyPortletWindowTransferProperties(IPortletWindowTransfer source, IPortletWindowTransfer dest) {
        if (source == null) {
            throw new IllegalArgumentException("source IPortletWindowTransfer may not be null");
        }
        if (source == null) {
            throw new IllegalArgumentException("dest IPortletWindowTransfer may not be null");
        }
        
        if (LOG.isDebugEnabled()) {
            LOG.debug("Copying properties from '" + source + "' to '" + dest + "'");
        }
        
        dest.setPortletMode(source.getPortletMode());
        dest.setWindowState(source.getWindowState());
    }
    
    public static void deepCopyPortletEntityTransferProperties(IPortletEntityTransfer source, IPortletEntityTransfer dest) {
        if (source == null) {
            throw new IllegalArgumentException("source IPortletEntityTransfer may not be null");
        }
        if (dest == null) {
            throw new IllegalArgumentException("dest IPortletEntityTransfer may not be null");
        }
        
        if (LOG.isDebugEnabled()) {
            LOG.debug("Copying properties from '" + source + "' to '" + dest + "'");
        }
        
        dest.setPreferences(clonePreferences(source.getPreferences()));
        dest.setRenderTimeOut(source.getRenderTimeOut());
    }
    
    public static void deepCopyPortletApplicationEntityTransferProperties(IPortletApplicationEntityTransfer source, IPortletApplicationEntityTransfer dest) {
        if (source == null) {
            throw new IllegalArgumentException("source IPortletApplicationEntityTransfer may not be null");
        }
        if (dest == null) {
            throw new IllegalArgumentException("dest IPortletApplicationEntityTransfer may not be null");
        }
        
        if (LOG.isDebugEnabled()) {
            LOG.debug("Copying properties from '" + source + "' to '" + dest + "'");
        }
    }
    
    public static void deepCopyPortletDefinitionTransferProperties(IPortletDefinitionTransfer source, IPortletDefinitionTransfer dest) {
        if (source == null) {
            throw new IllegalArgumentException("source IPortletDefinitionTransfer may not be null");
        }
        if (dest == null) {
            throw new IllegalArgumentException("dest IPortletDefinitionTransfer may not be null");
        }
    
        if (LOG.isDebugEnabled()) {
            LOG.debug("Copying properties from '" + source + "' to '" + dest + "'");
        }

        dest.setDisplayNames(cloneMap(source.getDisplayNames()));
        dest.setDescriptions(cloneMap(source.getDescriptions()));
        dest.setPortletInfo(clonePortletInfo(source.getPortletInfo()));
        dest.setPreferences(clonePreferences(source.getPreferences()));
        dest.setRenderTimeOut(source.getRenderTimeOut());
        dest.setFunctionalName(source.getFunctionalName());
    }
    
    public static void deepCopyPortletApplicationDefinitionTransferProperties(IPortletApplicationDefinitionTransfer source, IPortletApplicationDefinitionTransfer dest) {
        if (source == null) {
            throw new IllegalArgumentException("source IPortletApplicationDefinitionTransfer may not be null");
        }
        if (dest == null) {
            throw new IllegalArgumentException("dest IPortletApplicationDefinitionTransfer may not be null");
        }
    
        if (LOG.isDebugEnabled()) {
            LOG.debug("Copying properties from '" + source + "' to '" + dest + "'");
        }

        dest.setDisplayNames(cloneMap(source.getDisplayNames()));
        dest.setDescriptions(cloneMap(source.getDescriptions()));
        dest.setPortletModeMappings(cloneMap(source.getPortletModeMappings()));
        dest.setWindowStateMappings(cloneMap(source.getWindowStateMappings()));
        dest.setUserAttributeMappings(cloneMap(source.getUserAttributeMappings()));
    }
    
    public static IPortletInfo clonePortletInfo(IPortletInfo source) {
        if (source != null) {
            return new PortletInfoImpl(source);
        }
        else {
            return null;
        }
    }
    
    public static IPreferences clonePreferences(IPreferences source) {
        if (source != null) {
            return new PreferencesImpl(source);
        }
        else {
            return null;
        }
    }
    
    public static <K,V> Map<K,V>  cloneMap(Map<? extends K, ? extends V> source) {
        if (source != null) {
            final Map<K, V> dest = new HashMap<K, V>(source.size());
            
            for (final Map.Entry<? extends K, ? extends V> entry : source.entrySet()) {
                final K key = entry.getKey();
                final V value = entry.getValue();
                
                dest.put(clone(key), clone(value));
            }
            
            return dest;
        }
        else {
            return null;
        }
    }
    
    public static PortletMode clonePortletMode(PortletMode source) {
        return source;
    }
    
    public static WindowState cloneWindowState(WindowState source) {
        return source;
    }
    
    public static String cloneString(String source) {
        return source;
    }
    
    @SuppressWarnings("unchecked")
    private static <O> O clone(O source) {
        if (source == null) {
            return null;
        }
        else if (source instanceof Map) {
            return (O)cloneMap((Map)source);
        }
        else if (source instanceof IPreferences) {
            return (O)clonePreferences((IPreferences)source);
        }
        else if (source instanceof PortletMode) {
            return (O)clonePortletMode((PortletMode)source);
        }
        else if (source instanceof WindowState) {
            return (O)cloneWindowState((WindowState)source);
        }
        else if (source instanceof String) {
            return (O)cloneString((String)source);
        }
        else {
            LOG.warn("No clone method found for Class='" + source.getClass() + "'");
            return source;
        }
    }
}
