/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.url;


import javax.servlet.http.HttpServletRequest;
import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * The generic portlet parameter syntax interface.
 * It is used by PortletRequestParameterProcessor.
 *
 * @author Michael Ivanov: mivanov at unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletParameterSyntaxProvider {
	
	   /**
	    * Sets the parameter name space for the current parameter syntax.
	    * @param parameterNameSpace The parameterNameSpace to set.
	    */
	    public void setParameterNameSpace(String parameterNameSpace);    
	
	    /**
	     * Returns the portlet mode command name.
	     * @param windowId a portlet window ID
	     * @return a command name
	     */
	    public String getPortletModeCommandName( PortletWindowId windowId );
		
	    /**
		 * Returns the window state command name.
		 * @param windowId a portlet window ID
		 * @return a command name
		 */
		public String getWindowStateCommandName( PortletWindowId windowId );
		
		/**
		 * Returns the action target command name.
		 * @return a command name
		 */
		public String getActionTargetCommandName();
        
        /**
         * Returns the target window command name
         * @return
         */
        public String getTargetWindowCommandName();
		
		/**
		 * Checks if the given parameter is the action target command.
		 * @param requestParamName a HTTP request parameter name 
		 * @return a boolean value
		 */
		public boolean isActionTargetCommand( String requestParamName );
        
        /**
         * Checks if parameter gives target window info
         * @param requestParamName
         * @return
         */
        public boolean isTargetWindowCommand( String requestParamName );
		
		/**
		 * Checks if the given parameter is a portlet mode command.
		 * @param requestParamName a HTTP request parameter name 
		 * @return a boolean value
		 */
		public boolean isPortletModeCommand( String requestParamName );
		
		/**
		 * Checks if the given parameter is a window state command.
		 * @param requestParamName a HTTP request parameter name 
		 * @return a boolean value
		 */
		public boolean isWindowStateCommand( String requestParamName );
		
		/**
		 * Checks if the given parameter is the portlet parameter.
		 * @param requestParamName a HTTP request parameter name 
		 * @return a boolean value
		 */
		public boolean isPortletParameter( String requestParamName );
		
		/**
		 * Returns the identifier of the portlet window by a given parameter name.
		 * @param requestParamName  a HTTP request parameter name 
		 * @return the portlet window ID
		 */
		public PortletWindowId getPortletWindowId ( String requestParamName );
		
		/**
		 * Returns the original portlet parameter name.
		 * @param requestParamName a HTTP request parameter name 
		 * @return a real portlet parameter name to be used with <code>IPortletWindowManager</code>
		 */
		public String getRealParameterName ( String requestParamName );
		
		/**
		 * Returns the request portlet parameter name that is constructor/processor-specific.
		 * @param realParamName a portlet window parameter name 
		 * @param windowId a portlet window ID
		 * @return a request portlet parameter name
		 */
		public String getRequestParameterName ( String realParamName, PortletWindowId windowId );
		
		/**
		 * Returns the identifier of the targeted portlet window.
		 * @param request a HTTP reques
		 * @return the portlet window ID
		 */
		public PortletWindowId getTargetWindow(HttpServletRequest request);
        
        /**
         * Determines if the current request is an action request for the target window
         * @param request a HTTP reques
         * @return the portlet window ID
         */
        public boolean isAction(HttpServletRequest request);
		
}
