/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.url;

import java.util.Map;


import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.url.IUrlConstructor;

/**
 * The PortletURL constructor interface.
 * It is used by PortletURLProvider.
 *
 * @author Michael Ivanov: mivanov at unicon.net
 * @version $Revision: 11955 $
 */
public interface IPortletUrlConstructor extends IUrlConstructor {
	
	/**
	 * Holds the constructor name.
	 */
	public static final String CONSTRUCTOR_NAME = IPortletUrlConstructor.class.getName();
	
	/**
     * Change portlet window mode in the portlet URL for the next request.
     * @param windowId window id
     * @param mode new portlet mode
     */
    void setPortletMode(PortletWindowId windowId, PortletMode mode);
    
    /**
     * Change portlet window state in the portlet URL for the next request.
     * @param windowId window id
     * @param state new portlet window state
     */
    void setWindowState(PortletWindowId windowId, WindowState state);
    
    /**
     * Set target window for this URL. Only one target window can be set per either
     * action or render URL.
     * @param windowId
     */
    void setTargetWindowId(PortletWindowId windowId);
    
    /**
     * Obtain current target window id
     * @return window id of a target window
     */
    PortletWindowId getTargetWindowId();
    
    /**
     * Specify if the current URL is an action URL (default is render)
     * @param value <code>true</code> if this is an action URL, <code>false</code> if this is a render URL.
     */
    void setActionUrl(boolean value);
    
    /**
     * Sets the parameters in the portlet URL for the next request. 
     * Overwrites previously added parameters for the given window.
     * @param windowId window id
     * @param parameters a map of portlet URL parameters
     */
    void setParameters(PortletWindowId windowId, Map parameters);
    
    /**
     * Adds the parameters to existing parameters in the portlet URL for the next request.
     * @param windowId window id
     * @param parameters a map of portlet URL parameters
     */
    void addParameters(PortletWindowId windowId, Map parameters);
    
    /**
     * Sets the parameter in the portlet URL for the next request.
     * @param windowId window id
     * @param value a parameter value
     */
    void setParameter(PortletWindowId windowId, String paramName, String value);
    
    /**
     * Sets the parameter in the portlet URL for the next request.
     * @param windowId window id
     * @param values a parameter values
     */
    void setParameter(PortletWindowId windowId, String paramName, String[] values);
    
    /**
     * Removes portlet URL parameters for the given window.
     * @param windowId window id
     */
    void clearParameters(PortletWindowId windowId);
    
    /**
     * Clears the entire portlet URL
     */
    void clearUrl();
    
    /**
     * Clears the window portlet URL
     * @param windowId window id
     */
    public void clearUrl(PortletWindowId windowId);
    
    
    /**
     * Specify secure/non-secure URL
     * @param secure
     */
    public void setSecure(boolean secure);
    
    /**
     * Specifies URL of the current request (without query params)
     * @param url
     */
    public void setRequestUrl(String url);
}
