/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.url.impl;


import javax.servlet.http.HttpServletRequest;

import org.jasig.portal.portlet.url.IPortletParameterSyntaxProvider;
import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * The portlet parameter syntax implementation.
 *
 * @author Michael Ivanov: mivanov at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletParameterSyntaxProviderImpl implements IPortletParameterSyntaxProvider {
    // TODO: switch to using IUrlBasicSyntaxProvider
	
	    public static final String WINDOW_STATE_COMMAND = "window_state";
	    public static final String PORTLET_MODE_COMMAND = "portlet_mode";
	    public static final String TARGET_WINDOW_COMMAND = "target_window";
        public static final String ACTION_WINDOW_COMMAND = "action_window";
	    public static final String SEPARATOR = "_";
	    
	    private String parameterNameSpace = "portlet"; // default prefix value

		/**
		 * @return Returns the parameterNameSpace.
		 */
		public String getParameterNameSpace() {
			return parameterNameSpace;
		}
		
		/**
		 * @param parameterNameSpace The parameterNameSpace to set.
		 */
		public void setParameterNameSpace(String parameterNameSpace) {
			this.parameterNameSpace = parameterNameSpace;
		}
		
	    public String getPortletModeCommandName( PortletWindowId windowId ) {
		  return getParameterNameSpace() + SEPARATOR + PORTLET_MODE_COMMAND + SEPARATOR + windowId.toString();
		}
		
		public String getWindowStateCommandName( PortletWindowId windowId ) {
		  return getParameterNameSpace() + SEPARATOR + WINDOW_STATE_COMMAND + SEPARATOR + windowId.toString();
		}
		
		public String getActionTargetCommandName() {
		  return getParameterNameSpace() + SEPARATOR + ACTION_WINDOW_COMMAND;
	    }
		
		public boolean isActionTargetCommand( String requestParamName ) {
			return ( requestParamName.indexOf(ACTION_WINDOW_COMMAND) > -1 );
		}
		
		public String getTargetWindowCommandName() {
            return getParameterNameSpace() + SEPARATOR + TARGET_WINDOW_COMMAND;
        }

        public boolean isTargetWindowCommand(String requestParamName) {
            return ( requestParamName.indexOf(TARGET_WINDOW_COMMAND) > -1 );
        }

        public boolean isPortletModeCommand( String requestParamName ) {
			return ( requestParamName.indexOf(PORTLET_MODE_COMMAND) > -1 );
		}
		
		public boolean isWindowStateCommand( String requestParamName ) {
			return ( requestParamName.indexOf(WINDOW_STATE_COMMAND) > -1 );
		}
		
		public boolean isPortletParameter( String requestParamName ) {
			return (requestParamName.indexOf(getParameterNameSpace()) > -1 );	
		}
		
		public PortletWindowId getPortletWindowId ( String requestParamName ) {
			String winId = requestParamName.substring(requestParamName.lastIndexOf(SEPARATOR)+1);
			return new PortletWindowId(winId);
		}
		
		public String getRequestParameterName( String realParamName, PortletWindowId windowId ) {
			return getParameterNameSpace() + SEPARATOR + realParamName + SEPARATOR + windowId.toString();
		}
		
		public String getRealParameterName ( String requestParamName ) {
			String winId = requestParamName.substring(requestParamName.lastIndexOf(SEPARATOR)+1);
			return requestParamName.substring(getParameterNameSpace().length()+1,requestParamName.lastIndexOf(winId)-1);
		}
		
		public PortletWindowId getTargetWindow(HttpServletRequest request) {
		  String targetValue = request.getParameter(getTargetWindowCommandName());
	  	  return ( targetValue != null ) ? new PortletWindowId(targetValue) : null;
		}
        
        public boolean isAction(HttpServletRequest request) {
              String actionValue = request.getParameter(getActionTargetCommandName());
              return actionValue!=null;
        }
}
