/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.url.impl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;
import javax.servlet.http.HttpServletResponse;

import org.jasig.portal.portlet.rendering.IPortletWindowManager;
import org.jasig.portal.portlet.url.IPortletParameterSyntaxProvider;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IWritableHttpServletRequest;

/**
 * Http request portlet parameter processor.
 *
 * @author Michael Ivanov: mivanov at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletRequestParameterProcessor implements IRequestParameterProcessor {
	
	private IPortletWindowManager windowManager;
	private IPortletParameterSyntaxProvider parameterSyntax;
	
	/**
	 * @param parameterSyntax The parameterSyntax to set.
	 */
	public void setParameterSyntaxProvider(IPortletParameterSyntaxProvider parameterSyntax) {
		this.parameterSyntax = parameterSyntax;
	}
	
	/**
	 * @param windowManager The windowManager to set.
	 */
	public void setWindowManager(IPortletWindowManager windowManager) {
		this.windowManager = windowManager;
	}
	
    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        Map parameters = req.getParameterMap();

        if (parameters != null && !parameters.isEmpty()) {
            //Stores map of windowId to Map of parameters
            Map windowMap = new HashMap();

            // Getting targeted window ID 
            PortletWindowId targetWindow = parameterSyntax.getTargetWindow(req);
            if (targetWindow != null) {

                boolean isAction = parameterSyntax.isAction(req);
                // create a parameter map for the target window
                Map targetWindowParamMap = new HashMap();
                windowMap.put(targetWindow, targetWindowParamMap);
                
                for (Iterator i = parameters.keySet().iterator(); i.hasNext();) {
                    String paramName = (String)i.next();

                    // if the param name is the action target command - skip it
                    if (parameterSyntax.isTargetWindowCommand(paramName) || parameterSyntax.isActionTargetCommand(paramName))
                        continue;

                    String[] paramValues = (String[])parameters.get(paramName);

                    // Select ONLY portlet parameters for URL processing, but put the rest of parameters as they are because
                    // they could be portlet action parameters coming from forms
                    if (!parameterSyntax.isPortletParameter(paramName)) {
                        // action request parameters that needed to be decoded
                        if (isAction) {
                            targetWindowParamMap.put(paramName, paramValues);
                        }
                        continue;
                    }

                    PortletWindowId windowId = parameterSyntax.getPortletWindowId(paramName);

                    // Initialize/get request window params
                    Map requestParams = (Map)windowMap.get(windowId);
                    if (requestParams == null) {
                        requestParams = new HashMap();
                        windowMap.put(windowId, requestParams);
                    }

                    // Window state
                    if (parameterSyntax.isWindowStateCommand(paramName)) {
                        windowManager.setWindowState(windowId, new WindowState(paramValues[0]));
                        continue;
                    }

                    // Portlet mode
                    if (parameterSyntax.isPortletModeCommand(paramName)) {
                        windowManager.setPortletMode(windowId, new PortletMode(paramValues[0]));
                        continue;
                    }

                    // Render/Action request parameters that needed to be decoded
                    requestParams.put(parameterSyntax.getRealParameterName(paramName), paramValues);

                }

                // Setting action and render request parameters
                for (Iterator i = windowMap.keySet().iterator(); i.hasNext();) {
                    PortletWindowId windowId = (PortletWindowId)i.next();
                    Map params = (Map)windowMap.get(windowId);
                    if (windowId.equals(targetWindow) && isAction) {
                        windowManager.setPortletActionRequestParameters(windowId, params);
                    }
                    else {
                        windowManager.setPortletRenderRequestParameters(windowId, params);
                    }
                }
            }
        }

        return true;
    }
    
    /**
     * Specifies whether given processor has an ability to evaluate request parameter variables.
     * Note that a processor returning <code>false</code> value can make use of parameters containing variables, iff
     * these variables are defined and evaluated by some other processor.
     *
     * @return <code>true</code> if the processor can potentially replace a variable with a value, <code>false</code> if the processor
     * does not, by itself, define and evaluate variables.
     */
    public boolean canEvaluateVariables() {
    	return false;
    }
}
