/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.url.impl;

import org.jasig.portal.portlet.url.IPortletParameterSyntaxProvider;
import org.jasig.portal.portlet.url.IPortletUrlConstructor;
import org.jasig.portal.url.IUrlConstructor;
import org.jasig.portal.url.IUrlConstructorProvider;
import org.jasig.portal.url.UrlInfo;

/**
 * A portlet URL constructor provider. 
 * It provides uPortal URL with portlet URL constructors depending on the current context. 
 *
 * @author Michael Ivanov, mivanov at unicon.net
 * @version $Revision: 11911 $
 */
public class PortletUrlConstructorProviderImpl implements IUrlConstructorProvider {
	
	private IPortletParameterSyntaxProvider syntaxProvider;
    private String secureProtocolName=UrlInfo.HTTPS_PROTOCOL;
    private String unsecureProtocolName=UrlInfo.HTTP_PROTOCOL;
    private String securePortNumber=null;
    private String unsecurePortNumber=null;
	

	public void setSyntaxProvider(IPortletParameterSyntaxProvider syntaxProvider) {
		this.syntaxProvider = syntaxProvider;
	}
	
	/**
	 * Obtain a constructor that uses given url info object
	 * @return URL constructor implementation
	 */
	public IUrlConstructor getUrlConstructor() {
		PortletUrlConstructorImpl constructor =  new PortletUrlConstructorImpl();
		constructor.setSyntaxProvider(syntaxProvider);
        constructor.setSecurePortNumber(securePortNumber);
        constructor.setUnsecurePortNumber(unsecurePortNumber);
        constructor.setSecureProtocolName(secureProtocolName);
        constructor.setUnsecureProtocolName(unsecureProtocolName);
		return constructor;
	}
    
    /**
     * Gets the constructor name unique across all URL constructors in the portal.
     * Note: by convention, the name should be a full name of the relevant command interface.
     * @return a constructor name
     */
    public String getName() {
    	return IPortletUrlConstructor.CONSTRUCTOR_NAME;
    }

    /**
     * Set the secure protocol (defaults to "https")
     * @param secureProtocolName
     */
    public void setSecureProtocolName(String secureProtocolName) {
        this.secureProtocolName = secureProtocolName;
    }

    /**
     * Set unsecure protocol (defaults to "http")
     * @param unsecureProtocolName
     */
    public void setUnsecureProtocolName(String unsecureProtocolName) {
        this.unsecureProtocolName = unsecureProtocolName;
    }

    /**
     * Set secure port number (defaults to <code>null</code>)
     * @param securePortNumber port number, or <code>null</code> if no port should be secified
     */
    public void setSecurePortNumber(String securePortNumber) {
        this.securePortNumber = securePortNumber;
    }
    /**
     * Set unsecure port number (defaults to <code>null</code>)
     * @param unsecurePortNumber port number, or <code>null</code> if no port should be secified
     */
    public void setUnsecurePortNumber(String unsecurePortNumber) {
        this.unsecurePortNumber = unsecurePortNumber;
    }
}
