/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.portlet.url.impl;

import java.util.List;

import javax.portlet.PortalContext;
import javax.servlet.http.HttpServletRequest;

import org.jasig.portal.portlet.url.IPortletUrl;
import org.jasig.portal.portlet.url.IPortletUrlFactory;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.url.IUrlConstructorProviderFactoryService;
import org.jasig.portal.url.IUrlConstructorProviderFactoryServiceAccessor;
import org.jasig.portal.url.IUrlDecoratorListAccessor;
import org.jasig.portal.url.UrlConstructorProviderFactoryServiceAccessorImpl;
import org.jasig.portal.url.UrlDecoratorListAccessorImpl;
import org.springframework.beans.factory.annotation.Required;


/**
 * A portlet url factory implementation
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11911 $
 */
public class PortletUrlFactoryImpl implements IPortletUrlFactory {
    protected IUrlConstructorProviderFactoryServiceAccessor constructorProviderFactoryServiceAccessor=new UrlConstructorProviderFactoryServiceAccessorImpl();
    protected IUrlDecoratorListAccessor urlDecoratorIteratorAccessor=new UrlDecoratorListAccessorImpl();
    protected PortalContext portalContext;
    

    public IPortletUrl createPortletUrl(IPortletWindow portletWindow,
                                    HttpServletRequest servletRequest,
                                    boolean isAction) {
        // obtain url constructor factory service for this request
        IUrlConstructorProviderFactoryService urlConstructorConfiguration=constructorProviderFactoryServiceAccessor.getUrlConstructorProviderFactoryService(servletRequest);
        List urlDecorators=urlDecoratorIteratorAccessor.getUrlDecorators(servletRequest);
        return new PortletUrlImpl(portletWindow,servletRequest,isAction,portalContext,urlConstructorConfiguration,urlDecorators);
    }

    /**
     * @param constructorProviderFactoryServiceAccessor The constructorProviderFactoryServiceAccessor to set.
     */
    @Required
    public void setConstructorProviderFactoryServiceAccessor(IUrlConstructorProviderFactoryServiceAccessor constructorProviderFactoryServiceAccessor) {
        this.constructorProviderFactoryServiceAccessor = constructorProviderFactoryServiceAccessor;
    }

    @Required
    public void setUrlDecoratorIteratorAccessor(IUrlDecoratorListAccessor urlDecoratorIteratorAccessor) {
        this.urlDecoratorIteratorAccessor = urlDecoratorIteratorAccessor;
    }

    @Required
    public void setPortalContext(PortalContext portalContext) {
        this.portalContext = portalContext;
    }
}