/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.url.impl;


import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.portlet.PortalContext;
import javax.portlet.PortletMode;
import javax.portlet.PortletModeException;
import javax.portlet.PortletSecurityException;
import javax.portlet.PortletURL;
import javax.portlet.WindowState;
import javax.portlet.WindowStateException;
import javax.servlet.http.HttpServletRequest;

import org.jasig.portal.portlet.common.IContentType;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.url.IPortletUrl;
import org.jasig.portal.portlet.url.IPortletUrlConstructor;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.url.IUrlConstructorProviderFactoryService;
import org.jasig.portal.url.PortalUrlImpl;


/**
 * Implementation of JSR 168 javax.portlet.PortletURL.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11911 $
 */

public class PortletUrlImpl extends PortalUrlImpl implements PortletURL, IPortletUrl {
	
    private final IPortletUrlConstructor portletUrlConstructor;
    private final IPortletWindow portletWindow;
    private final PortalContext portalContext;
    
    public PortletUrlImpl(IPortletWindow portletWindow, HttpServletRequest servletRequest,
            boolean isAction, PortalContext portalContext,
            IUrlConstructorProviderFactoryService urlConstructorProviderService, List urlDecorators) {
        
        super(urlConstructorProviderService, urlDecorators);
        
        if (portletWindow == null) {
            throw new IllegalArgumentException("portletWindow can not be null");
        }
        if (portalContext == null) {
            throw new IllegalArgumentException("portalContext can not be null");
        }
        if (portletWindow == null) {
            throw new IllegalArgumentException("portletWindow can not be null");
        }
        
        this.portletWindow = portletWindow;
        
        this.portalContext = portalContext;

        portletUrlConstructor = (IPortletUrlConstructor)getUrlConstructor(IPortletUrlConstructor.CONSTRUCTOR_NAME);
        portletUrlConstructor.setTargetWindowId(portletWindow.getPortletWindowId());
        portletUrlConstructor.setActionUrl(isAction);
        portletUrlConstructor.setRequestUrl(servletRequest.getRequestURL().toString());
    }
    
    // javax.portlet.PortletURL -------------------------------------------------------------------
    public void setWindowState(WindowState windowState) throws WindowStateException {
        
        if (windowState != null && isWindowStateAllowed(windowState))
        {
        	portletUrlConstructor.setWindowState(portletWindow.getPortletWindowId(),windowState);                
            return;             
        }
 
        throw new WindowStateException("unsupported Window State used: " + windowState,windowState);
    }

    public void setPortletMode (PortletMode portletMode) throws PortletModeException {
  
        if (portletMode != null && isPortletModeAllowed(portletMode))
        {
            if (isPortletModeSupported(portletMode, portletWindow))
            {
            	portletUrlConstructor.setPortletMode(portletWindow.getPortletWindowId(),portletMode);         
                return;             
            }
        }
        
        throw new PortletModeException("unsupported Portlet Mode used: " + portletMode,portletMode);
    }

    public void setParameter(String name, String value) {
    	
        if (name == null || value == null) {
            throw new IllegalArgumentException("name and value must not be null");
        }

        portletUrlConstructor.setParameter(portletWindow.getPortletWindowId(),name,value);
    }

    public void setParameter (String name, String[] values)
    {
        if (name == null || values == null || values.length == 0) {
            throw new IllegalArgumentException("name and values must not be null or values be an empty array");
        }

        portletUrlConstructor.setParameter(portletWindow.getPortletWindowId(),name,values);         
    }
    
    /* (non-Javadoc)
     * @see javax.portlet.PortletURL#setParameters(Map)
     */
    public void setParameters(Map parameters) {
    	
        if (parameters == null) {
            throw new IllegalArgumentException("Parameters must not be null.");
        }
        for (Iterator iter = parameters.entrySet().iterator(); iter.hasNext();) {
            Map.Entry entry = (Map.Entry)iter.next();
            if (!(entry.getKey() instanceof String)) {
                throw new IllegalArgumentException("Key must not be null and of type java.lang.String.");
            }
            if (!(entry.getValue() instanceof String[])) {
                throw new IllegalArgumentException("Value must not be null and of type java.lang.String[].");
            }
        }
        portletUrlConstructor.setParameters(portletWindow.getPortletWindowId(),parameters);        
    }

    public void setSecure (boolean secure) throws PortletSecurityException { 
        portletUrlConstructor.setSecure(secure);
    }

    // --------------------------------------------------------------------------------------------

    // internal methods ---------------------------------------------------------------------------
    private boolean isPortletModeSupported(PortletMode requestedPortletMode,IPortletWindow referencedPortletWindow) 
    {
        // PLT 8.1: View Portlet Mode should always be supported by a portlet, even if not defined in the descriptor
        if (requestedPortletMode.equals(PortletMode.VIEW) || requestedPortletMode.toString().equalsIgnoreCase("view"))
        {
            return true;
        }
        IPortletDefinition portletDefinition = referencedPortletWindow.getPortletEntity().getPortletDefinition();
        Map<String,IContentType> contentTypes = portletDefinition.getSupportedContentTypes();
        for(IContentType type: contentTypes.values()) {
            Set<PortletMode> modeSet=type.getPortletModeSet();
            if(modeSet.contains(requestedPortletMode)) {
                return true;
            }
        }
        return false;
    }
    

    private boolean isPortletModeAllowed(PortletMode mode) throws PortletModeException {
        Enumeration supportedModes = portalContext.getSupportedPortletModes();
        while (supportedModes.hasMoreElements()) {
            if (supportedModes.nextElement().toString().toUpperCase().equals(
                    (mode.toString().toUpperCase()))) {
                return true;
            }
        }
        return false;
    }

    private boolean isWindowStateAllowed(WindowState state) {
        Enumeration supportedStates = portalContext.getSupportedWindowStates();
        while (supportedStates.hasMoreElements()) {
            if (supportedStates.nextElement().toString().toUpperCase().equals(
                    (state.toString().toUpperCase()))) {
                return true;
            }
        }
        return false;
    }
    // --------------------------------------------------------------------------------------------
	
 
}