/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlet.url.impl;

import java.util.Stack;

import javax.portlet.PortletMode;
import javax.portlet.PortletModeException;
import javax.portlet.PortletSecurityException;
import javax.portlet.RenderResponse;
import javax.portlet.WindowState;
import javax.portlet.WindowStateException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;

import org.apache.xalan.extensions.XSLProcessorContext;
import org.apache.xalan.templates.ElemExtensionCall;
import org.apache.xalan.transformer.TransformerImpl;
import org.jasig.portal.layout.uportal2.ILegacyLayoutSourceCommands;
import org.jasig.portal.portlet.url.IPortletUrl;
import org.jasig.portal.portlet.url.IPortletUrlConstructor;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.url.PortalUrlXalanElements;
import org.w3c.dom.Node;

/**
 * Xalan extension elements for constructing portlet url.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletUrlXalanElements extends PortalUrlXalanElements {
    /**
     * TransformerImpl parameter name for the render reponse (for PortletUrlImpl construction)
     */
    public static final String RENDER_RESPONSE_PARAMETER_NAME="renderResponse";
    
    /**
     * Implementation of the outer &lt;actionUrl/&gt; element.
     * 
     * @param context
     * @param elem
     * @return serialized version of action url
     * @throws TransformerException
     */
    public String actionUrl(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl = context.getTransformer();

        // retrieve configuration
        RenderResponse renderResponse=(RenderResponse) transformerImpl.getParameter(RENDER_RESPONSE_PARAMETER_NAME);
        
        // create new portlet url instance
        IPortletUrl portletUrl=(IPortletUrl) renderResponse.createActionURL();
        // put it on the stack
        Stack urlStack=getCreateUrlStack(transformerImpl);
        urlStack.push(portletUrl);
        transformerImpl.executeChildTemplates(elem,true);
        // remove url fromt the stack
        urlStack.pop();
        return portletUrl.toString();
    }
    
    public String legacyLayoutActionUrl(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl = context.getTransformer();

        // retrieve configuration
        RenderResponse renderResponse=(RenderResponse) transformerImpl.getParameter(RENDER_RESPONSE_PARAMETER_NAME);
        
        // create new portlet url instance
        IPortletUrl portletUrl=(IPortletUrl) renderResponse.createActionURL();
        // add the legacy layout markings
        ILegacyLayoutSourceCommands llsc = (ILegacyLayoutSourceCommands)portletUrl.getUrlConstructor(ILegacyLayoutSourceCommands.class.getName());
        llsc.addMarkings();
        
        // put it on the stack
        Stack urlStack=getCreateUrlStack(transformerImpl);
        urlStack.push(portletUrl);
        transformerImpl.executeChildTemplates(elem,true);
        // remove url fromt the stack
        urlStack.pop();
        return portletUrl.toString();
    }
    
    /**
     * Implementation of the outer <renderUrl/> element.
     * 
     * @param context
     * @param elem
     * @return serialized version of render url
     * @throws TransformerException
     */
    public String renderUrl(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl = context.getTransformer();

        // retrieve configuration
        RenderResponse renderResponse=(RenderResponse) transformerImpl.getParameter(RENDER_RESPONSE_PARAMETER_NAME);
        
        // create new portlet url instance
        IPortletUrl portletUrl=(IPortletUrl) renderResponse.createRenderURL();
        // put it on the stack
        Stack urlStack=getCreateUrlStack(transformerImpl);
        urlStack.push(portletUrl);
        transformerImpl.executeChildTemplates(elem,true);
        // remove url fromt the stack
        urlStack.pop();
        return portletUrl.toString();
    }
    
    
    
    /**
     * A command element to set window state.
     * <windowState state="normal"/> - set window state of the current window to normal
     * <windowState windowId="anotherWindow" state="minimized"/> - set window state of another window to minimized
     * 
     * @param context
     * @param elem
     * @throws TransformerException
     * @throws WindowStateException
     */
    public void windowState(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException, WindowStateException {
        TransformerImpl transformerImpl=context.getTransformer();
        Node contextNode=context.getContextNode();
        String str_state=elem.getAttribute("state",contextNode,transformerImpl);
        WindowState state=new WindowState(str_state);
        IPortletUrl portletUrl = getPortletUrl(transformerImpl);
        String str_windowId=elem.getAttribute("windowId",contextNode,transformerImpl);
        if(str_windowId!=null && str_windowId!="") {
            // use portlet url constructor directly to specify a different window id
            PortletWindowId windowId=new PortletWindowId(str_windowId);
            IPortletUrlConstructor portletUrlConstructor = (IPortletUrlConstructor) portletUrl.getUrlConstructor(IPortletUrlConstructor.CONSTRUCTOR_NAME);
            portletUrlConstructor.setWindowState(windowId,state);
        } else {
            // set state on the current window
            portletUrl.setWindowState(state);
        }
    }
    
    
    /**
     * A command element to set portlet mode
     * <portletMode mode="view"/> - set mode of the current window to view
     * <portletMode windowId="anotherWindow" state="edit"/> - set mode of another window to edit
     * 
     * @param context
     * @param elem
     * @throws TransformerException
     * @throws WindowStateException
     * @throws PortletModeException 
     */
    public void portletMode(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException, PortletModeException {
        TransformerImpl transformerImpl=context.getTransformer();
        Node contextNode=context.getContextNode();
        String str_mode=elem.getAttribute("mode",contextNode,transformerImpl);
        PortletMode mode=new PortletMode(str_mode);
        IPortletUrl portletUrl = getPortletUrl(transformerImpl);
        String str_windowId=elem.getAttribute("windowId",contextNode,transformerImpl);
        if(str_windowId!=null && str_windowId!="") {
            // use portlet url constructor directly to specify a different window id
            PortletWindowId windowId=new PortletWindowId(str_windowId);
            IPortletUrlConstructor portletUrlConstructor = (IPortletUrlConstructor) portletUrl.getUrlConstructor(IPortletUrlConstructor.CONSTRUCTOR_NAME);
            portletUrlConstructor.setPortletMode(windowId,mode);
        } else {
            // set state on the current window
            portletUrl.setPortletMode(mode);
        }
    }
    
    /**
     * A command element set parameter value
     * <parameter name="paramName" value="parameter value"/> - add request parameter for the current window
     * <parameter windowId="anotherWindow" name="paramName" value="parameter value"/> - add request parameter for another window
     * 
     * @param context
     * @param elem
     * @throws TransformerException
     * @throws WindowStateException
     * @throws PortletModeException 
     */
    public void parameter(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl=context.getTransformer();
        Node contextNode=context.getContextNode();
        String name=elem.getAttribute("name",contextNode,transformerImpl);
        String value=elem.getAttribute("value",contextNode,transformerImpl);
        IPortletUrl portletUrl = getPortletUrl(transformerImpl);
        String str_windowId=elem.getAttribute("windowId",contextNode,transformerImpl);
        if(str_windowId!=null && str_windowId!="") {
            // use portlet url constructor directly to specify a different window id
            PortletWindowId windowId=new PortletWindowId(str_windowId);
            IPortletUrlConstructor portletUrlConstructor = (IPortletUrlConstructor) portletUrl.getUrlConstructor(IPortletUrlConstructor.CONSTRUCTOR_NAME);
            portletUrlConstructor.setParameter(windowId,name,value);
        } else {
            // set state on the current window
            portletUrl.setParameter(name,value);
        }
    }
    
    
    /**
     * A command element to set secure url mode
     * <secure/> or <secure value="true"/> - specify secure mode
     * <secure value="false"/> - do not require secure url
     * 
     * @param context
     * @param elem
     * @throws TransformerException
     * @throws PortletSecurityException  
     * @throws WindowStateException
     * @throws PortletModeException 
     */
    public void secure(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException, PortletSecurityException {
        TransformerImpl transformerImpl=context.getTransformer();
        Node contextNode=context.getContextNode();
        boolean secure=true;
        String value=elem.getAttribute("value",contextNode,transformerImpl);
        if("false".equals(value)) {
            secure=false;
        }
        IPortletUrl portletUrl = getPortletUrl(transformerImpl);
        portletUrl.setSecure(secure);
    }
    

    /**
     * Obtain portlet url from the stack
     * @param transformer
     * @return portlet url
     */
    protected IPortletUrl getPortletUrl(Transformer transformer) {
        // obtain IPortletUrl
        Stack portletUrlStack=getUrlStack(transformer);
        if(portletUrlStack==null) {
            throw new IllegalStateException("portlet url stack is missing from the TransfomerImpl attributes");
        }
        IPortletUrl portletUrl=(IPortletUrl) portletUrlStack.peek();
        if(portletUrl==null) {
            throw new IllegalStateException("portlet url stack is empty");
        }
        return portletUrl;
    }

}
