/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.userattributes;

import java.io.Serializable;
import java.util.Map;

import org.jasig.portal.caching.CacheKeyListImpl;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.security.IUser;
import org.springframework.beans.factory.annotation.Required;

/**
 * TODO replace this with a declarative cache
 * 
 * Wraps another IMappedUserAttributeProvider and caches its results. The cache keys are generated
 * by a combination of the userId and portlet domain object ID. The <code>cacheNullResults</code>
 * property specifies if null results should be cached. It defaults to false, if setting to true
 * be sure the configured ICache implementation can handle null values.
 * 
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public class CachingMappedUserAttributeProvider extends MappedUserAttributeProviderWrapper {
    private ISimpleCacheProviderFacade mappedUserAttributeCache;
    private boolean cacheNullResults = false;
    
    public ISimpleCacheProviderFacade getMappedUserAttributeCache() {
        return this.mappedUserAttributeCache;
    }

    @Required
    public void setMappedUserAttributeCache(ISimpleCacheProviderFacade mappedUserAttributeCache) {
        this.mappedUserAttributeCache = mappedUserAttributeCache;
    }
    
    /**
     * @return the cacheNullResults
     */
    public boolean isCacheNullResults() {
        return this.cacheNullResults;
    }

    /**
     * @param cacheNullResults the cacheNullResults to set
     */
    public void setCacheNullResults(boolean cacheNullResults) {
        this.cacheNullResults = cacheNullResults;
    }

    /**
     * @see org.jasig.portal.portlet.userattributes.IMappedUserAttributeProvider#getMappedUserAttributes(org.jasig.portal.security.IUser, org.jasig.portal.portlet.entity.PortletApplicationEntityId)
     */
    @SuppressWarnings("unchecked")
    public Map<String, String> getMappedUserAttributes(IUser user, PortletApplicationEntityId portAppEntId) {
        final Serializable cacheKey = this.getCacheKey(user, portAppEntId);
        Map<String, String> mappedUserAttributes = (Map<String, String>)this.mappedUserAttributeCache.getFromCache(cacheKey);
        
        if (mappedUserAttributes == null) {
            mappedUserAttributes = super.getMappedUserAttributes(user, portAppEntId);
            
            if (this.cacheNullResults || mappedUserAttributes != null) {
                this.mappedUserAttributeCache.putInCache(cacheKey, mappedUserAttributes);
            }
        }

        return mappedUserAttributes;
    }

    /**
     * @see org.jasig.portal.portlet.userattributes.IMappedUserAttributeProvider#getMappedUserAttributes(org.jasig.portal.security.IUser, org.jasig.portal.portlet.definition.PortletApplicationDefinitionId)
     */
    @SuppressWarnings("unchecked")
    public Map<String, String> getMappedUserAttributes(IUser user, PortletApplicationDefinitionId portAppDefId) {
        final Serializable cacheKey = this.getCacheKey(user, portAppDefId);
        Map<String, String> mappedUserAttributes = (Map<String, String>)this.mappedUserAttributeCache.getFromCache(cacheKey);
        
        if (mappedUserAttributes == null) {
            mappedUserAttributes = super.getMappedUserAttributes(user, portAppDefId);
            this.mappedUserAttributeCache.putInCache(cacheKey, mappedUserAttributes);
        }

        return mappedUserAttributes;
    }

    protected Serializable getCacheKey(IUser user, PortletApplicationEntityId portAppEntId) {
        return new CacheKeyListImpl(new String[] { user.getId().toString(), portAppEntId.toString() },
                                    new Serializable[] { null, null });
    }

    protected Serializable getCacheKey(IUser user, PortletApplicationDefinitionId portAppDefId) {
        return new CacheKeyListImpl(new String[] { user.getId().toString(), portAppDefId.toString() },
                                    new Serializable[] { null, null });
    }
}
