/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.userattributes;

import java.util.Map;

import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.entity.PortletApplicationEntityId;
import org.jasig.portal.security.IUser;

/**
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public interface IMappedUserAttributeProvider {
    /**
     * Gets the user attributes for a portlet application defintion mapped from the portals
     * concept of user attributes.
     * 
     * @param user The user to get the attributes for
     * @param portAppEntId The portlet application entity to get the attributes for 
     * @return A Map of user attributes (String to String) for the specified portlet application definition and user. Will never be null.
     */
    public Map<String, String> getMappedUserAttributes(IUser user, PortletApplicationEntityId portAppEntId);
    
    /**
     * Gets the user attributes for a portlet application defintion mapped from the portals
     * concept of user attributes.
     * 
     * @param user The user to get the attributes for
     * @param portAppDefId The portlet application definition to get the attributes for 
     * @return A Map of user attributes (String to String) for the specified portlet application definition and user. Will never be null.
     */
    public Map<String, String> getMappedUserAttributes(IUser user, PortletApplicationDefinitionId portAppDefId);
}