/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/


package org.jasig.portal.portlet.window;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.jasig.portal.portlet.entity.IPortletEntity;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.transfer.IPortletWindowTransfer;



/**
 * The portlet window interface.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */

public interface IPortletWindow extends IPortletWindowInformationProvider {
     
	/**
     * Returns the ID of the corresponding portlet window.
     * @return the portlet window ID
     */
    public PortletWindowId getPortletWindowId();
    
    /**
     * Returns portlet entity corresponding to this window
     * @return
     */
    public IPortletEntity getPortletEntity();
    
    /**
     * Returns the portlet entity ID of the corresponding portlet window.
     * @return the portlet entity ID
     */
    public PortletEntityId getPortletEntityId();
    
    /**
     * Returns the window state of the corresponding portlet window.
     * @return the window state
     */
    public WindowState getWindowState();
    
    /**
     * Returns the portlet mode of the corresponding portlet window.
     * @return the portlet mode
     */
    public PortletMode getPortletMode();
    
    /**
     * Sets the window state of the corresponding portlet window.
     * @param windowState the window state
     */
    public void setWindowState( WindowState windowState );
    
    /**
     * Sets the portlet mode of the corresponding portlet window.
     * @param portletMode the portlet mode
     */
    public void setPortletMode( PortletMode portletMode );

    
    /**
     * @return Get the underlying IPortletWindowTransfer, may not be null.
     */
    public IPortletWindowTransfer getPortletWindowTransfer();
}
