/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlet.window;

import java.util.Locale;
import java.util.Map;
import java.util.Set;

import javax.portlet.PortletMode;
import javax.portlet.WindowState;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.pluto.PortletWindow;
import org.apache.pluto.PortletWindowID;
import org.jasig.portal.portlet.common.IContentType;
import org.jasig.portal.portlet.common.IDescription;
import org.jasig.portal.portlet.common.IDisplayName;
import org.jasig.portal.portlet.common.IInitParameter;
import org.jasig.portal.portlet.common.IPortletInfo;
import org.jasig.portal.portlet.common.IPreferences;
import org.jasig.portal.portlet.common.ISecurityRoleReference;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.deployment.IPortletDeployment;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.entity.IPortletEntity;
import org.jasig.portal.portlet.entity.PortletEntityId;
import org.jasig.portal.portlet.transfer.IPortletWindowTransfer;


/**
 * Portlet window implementation.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public class PortletWindowImpl implements IPortletWindowControl, PortletWindow {
    private final IPortletWindowTransfer portletWindowTransfer;
    private final IPortletEntity portletEntity;

    public PortletWindowImpl(IPortletWindowTransfer portletWindowTransfer,
                             IPortletEntity portletEntity) {

        if (portletWindowTransfer == null) {
            throw new IllegalArgumentException("portletWindowTransfer may not be null");
        }
        if (portletEntity == null) {
            throw new IllegalArgumentException("portletEntity may not be null");
        }
        if (!portletWindowTransfer.getPortletEntityId().equals(portletEntity.getPortletEntityId())) {
            throw new IllegalArgumentException("portletWindowTransfer.getPortletEntityId() must equal portletEntity.getPortletEntityId(). '" + portletWindowTransfer.getPortletEntityId() + "' != '" + portletEntity.getPortletEntityId() + "'");
        }
        
        this.portletWindowTransfer = portletWindowTransfer;
        this.portletEntity = portletEntity;
    }
    
    public PortletEntityId getPortletEntityId() {
        return this.portletWindowTransfer.getPortletEntityId();
    }
    
    public PortletWindowId getPortletWindowId() {
        return this.portletWindowTransfer.getPortletWindowId();
    }
    
    public void setPortletWindowId(PortletWindowId id) {
        this.portletWindowTransfer.setPortletWindowId(id);
    }
    
    public IPortletWindowTransfer getPortletWindowTransfer() {
        return this.portletWindowTransfer;
    }

    public void setWindowState( WindowState windowState ) {
        this.portletWindowTransfer.setWindowState(windowState);
    }
    
    public void setPortletMode( PortletMode portletMode ) {
        this.portletWindowTransfer.setPortletMode(portletMode);
    }
    
    public Map<Locale, IDescription> getDescriptions() {
        return this.portletEntity.getDescriptions();
    }

    public String getPortletName() {
        return this.portletEntity.getPortletName();
    }

    public Map<Locale, IDisplayName> getDisplayNames() {
        return this.portletEntity.getDisplayNames();
    }

    public String getPortletClassName() {
        return this.portletEntity.getPortletClassName();
    }

    public String getFunctionalName() {
        return this.portletEntity.getFunctionalName();
    }

    public Map<String, IInitParameter> getInitParameters() {
        return this.portletEntity.getInitParameters();
    }

    public Integer getPortletExpirationCache() {
        return this.portletEntity.getPortletExpirationCache();
    }

    public Map<String, IContentType> getSupportedContentTypes() {
        return this.portletEntity.getSupportedContentTypes();
    }

    public Set<Locale> getSupportedLocales() {
        return this.portletEntity.getSupportedLocales();
    }

    public IPortletInfo getPortletInfo() {
        return this.portletEntity.getPortletInfo();
    }

    public IPreferences getPreferences() {	
        return this.portletEntity.getPreferences();
    }

    public Map<String, ISecurityRoleReference> getSecurityRoleReferences() {
        return this.portletEntity.getSecurityRoleReferences();
    }
    
    public long getRenderTimeOut() {
        return this.portletEntity.getRenderTimeOut();
    }
    
    public IPortletEntity getPortletEntity() {
        return this.portletEntity;
    }
    
    /***** Pluto's PortletWindow methods *****/
    
    public WindowState getWindowState() {
        return this.portletWindowTransfer.getWindowState();
    }
    
    public PortletMode getPortletMode() {
        return this.portletWindowTransfer.getPortletMode();
    }
    
    /***** Pluto's PortletWindow methods *****/
    
    /**
     * @see org.apache.pluto.PortletWindow#getContextPath()
     */
    public String getContextPath() {
        //This assumes the PortletApplicationDeploymentId is the path to the ServletContext for the portlet
        final IPortletDefinition portletDefinition = this.portletEntity.getPortletDefinition();
        final IPortletDeployment portletDeployment = portletDefinition.getPortletDeployment();
        final PortletApplicationDeploymentId portletApplicationDeploymentId = portletDeployment.getPortletApplicationDeploymentId();
        return portletApplicationDeploymentId.toString();
    }

    /**
     * @see org.apache.pluto.PortletWindow#getId()
     */
    public PortletWindowID getId() {
        return this.getPortletWindowId();
    }
    

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (!(obj instanceof IPortletWindow)) {
            return false;
        }
        if (this == obj) {
            return true;
        }
        
        IPortletWindow other = (IPortletWindow)obj;
        return new EqualsBuilder()
            .append(this.portletWindowTransfer, other.getPortletWindowTransfer())
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return this.portletWindowTransfer.hashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this)
        .append("portletWindowTransfer", this.portletWindowTransfer)
        .toString();
    }
}
