/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlets;

import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.Portlet;
import javax.portlet.PortletConfig;
import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.portal.spring.PortletWebApplicationContextUtils;
import org.springframework.web.context.WebApplicationContext;

/**
 *  A generic base for implementing simple spring-configured portlets.
 *  Overwrite <code>getBeanLocation()</code> method to return location of the portlet bean.
 *  The bean must implement {@link javax.portlet.Portlet} interface.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public abstract class BaseBeanPortlet extends GenericPortlet {
    /**
     * Subclasses must overload this method to return location of the portlet bean
     * in the application context.
     * 
     * @return location of the portlet bean in the web app application context.
     */
    abstract protected String getBeanLocation();
    
    protected Portlet getBean() {
        WebApplicationContext context = PortletWebApplicationContextUtils.getRequiredWebApplicationContext(getPortletContext());
        return (Portlet) context.getBean(getBeanLocation(), Portlet.class);
    }
    
    // delegate methods

    public void init(PortletConfig portletConfig) throws PortletException {
        super.init(portletConfig);
        getBean().init(portletConfig);
    }

    public void processAction(ActionRequest req, ActionResponse res) throws PortletException, IOException {
        getBean().processAction(req, res);
    }

    public void render(RenderRequest req, RenderResponse res) throws PortletException, IOException {
        getBean().render(req, res);
    }

    public void destroy() {
        getBean().destroy();
    }
    
    


}
