/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.cachemanager.web;

import java.util.Map;

import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.lang.StringUtils;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.portlets.cachemanager.beans.CacheInfoBean;
import org.jasig.portal.spring.TypeAwareBeanRegistry;
import org.springframework.web.portlet.ModelAndView;
import org.springframework.web.portlet.mvc.AbstractController;

/**
 * Controller that handles only render requests. The render handler takes a 
 * {@link #CACHE_NAME_REQUEST_PARAM} parameter and returns the {@link org.jasig.portal.caching.ICache}
 * specified by the name as the model and {@link #VIEW_NAME} as the view.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class CacheDetailsController extends AbstractController {
	
	public static final String CACHE_NAME_REQUEST_PARAM = "cacheName";
	public static final String MODEL_NAME = "cacheInfo";
	public static final String VIEW_NAME = "cacheDetails";
    
	private TypeAwareBeanRegistry<ISimpleCacheProviderFacade> typeAwareBeanRegistry;
    
    /**
     * @return the typeAwareBeanRegistry
     */
    public TypeAwareBeanRegistry<ISimpleCacheProviderFacade> getTypeAwareBeanRegistry() {
        return this.typeAwareBeanRegistry;
    }
    /**
     * @param typeAwareBeanRegistry the typeAwareBeanRegistry to set
     */
    public void setTypeAwareBeanRegistry(TypeAwareBeanRegistry<ISimpleCacheProviderFacade> typeAwareBeanRegistry) {
        this.typeAwareBeanRegistry = typeAwareBeanRegistry;
    }
    
    
    /**
     * @see org.springframework.web.portlet.mvc.AbstractController#handleRenderRequestInternal(javax.portlet.RenderRequest, javax.portlet.RenderResponse)
     */
    protected ModelAndView handleRenderRequestInternal(RenderRequest request, RenderResponse response) throws Exception {
        final String cacheName = StringUtils.trimToNull(request.getParameter(CACHE_NAME_REQUEST_PARAM));
        if (cacheName == null) {
            throw new PortletException("The '" + CACHE_NAME_REQUEST_PARAM + "' parameter must be specified.");
        }

        final Map<String, ISimpleCacheProviderFacade> caches = this.typeAwareBeanRegistry.getBeansOfType(ISimpleCacheProviderFacade.class);
        final ISimpleCacheProviderFacade cache = caches.get(cacheName);
        if (cache == null) {
            throw new PortletException("No '" + ISimpleCacheProviderFacade.class.getName() + "' available for bean name '" + cacheName + "'.");
        }

        final CacheInfoBean cacheInfo = new CacheInfoBean(cacheName, cache);
        return new ModelAndView(VIEW_NAME, MODEL_NAME, cacheInfo);
    }
}
