/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.cachemanager.web;

import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;

import org.apache.commons.lang.StringUtils;
import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.spring.TypeAwareBeanRegistry;
import org.springframework.web.portlet.mvc.AbstractController;

/**
 * Controller that handles only action requests. The action handler takes
 * a {@link #CACHE_NAME_REQUEST_PARAM} parameter, loads that {@link org.jasig.portal.caching.ISimpleCacheProviderFacade} and
 * calls {@link org.jasig.portal.caching.ISimpleCacheProviderFacade#clear()}.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class ClearCacheController extends AbstractController {
	public static final String CACHE_NAME_REQUEST_PARAM = "cacheName";
	
	private TypeAwareBeanRegistry<ISimpleCacheProviderFacade> typeAwareBeanRegistry;
    
    /**
     * @return the typeAwareBeanRegistry
     */
    public TypeAwareBeanRegistry<ISimpleCacheProviderFacade> getTypeAwareBeanRegistry() {
        return this.typeAwareBeanRegistry;
    }
    /**
     * @param typeAwareBeanRegistry the typeAwareBeanRegistry to set
     */
    public void setTypeAwareBeanRegistry(TypeAwareBeanRegistry<ISimpleCacheProviderFacade> typeAwareBeanRegistry) {
        this.typeAwareBeanRegistry = typeAwareBeanRegistry;
    }

    
    /**
     * @see org.springframework.web.portlet.mvc.AbstractController#handleActionRequestInternal(javax.portlet.ActionRequest, javax.portlet.ActionResponse)
     */
    protected void handleActionRequestInternal(ActionRequest request, ActionResponse response) throws Exception {
        final String cacheName = StringUtils.trimToNull(request.getParameter(CACHE_NAME_REQUEST_PARAM));
        if (cacheName == null) {
            throw new PortletException("The '" + CACHE_NAME_REQUEST_PARAM + "' parameter must be specified.");
        }

        final Map<String, ISimpleCacheProviderFacade> caches = this.typeAwareBeanRegistry.getBeansOfType(ISimpleCacheProviderFacade.class);
        final ISimpleCacheProviderFacade cache = caches.get(cacheName);
        if (cache == null) {
            throw new PortletException("No '" + ISimpleCacheProviderFacade.class.getName() + "' available for bean name '" + cacheName + "'.");
        }

        cache.flushCache();
    }
}
