/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlets.channeladapter;

import java.util.Map;

import org.jasig.portal.ChannelRuntimeData;
import org.jasig.portal.PortalException;
import org.jasig.portal.UPFileSpec;

/**
 * Portlet adapter extension of {@link org.jasig.portal.ChannelRuntimeData} 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class PortletAdapterChannelRuntimeData extends ChannelRuntimeData {
    static final long serialVersionUID=-1;
    protected String baseActionUrl;
    
    public PortletAdapterChannelRuntimeData(Map parameters) {
        // set parameters
        super.putAll(parameters);
    }
    
    /**
     * Get the UPFileSpec
     * @return channelUPFile the UPFileSpec
     */
    public UPFileSpec getUPFile() {
        return null;
    }
    
    /**
     * Returns a baseActionURL - parameters of a request coming in on the baseActionURL
     * will be placed into the ChannelRuntimeData object for channel's use.
     *
     * @return a value of URL to which parameter sequences should be appended.
     */
    public String getBaseActionURL() {
        return baseActionUrl;
    }
    
    public void setBaseActionURL(String url) {
        this.baseActionUrl=url;
    }

    /**
     * Returns a baseActionURL - parameters of a request coming in on the baseActionURL
     * will be placed into the ChannelRuntimeData object for channel's use.
     *
     * @param idempotent a <code>boolean</code> value specifying if a given URL should be idepotent.
     * @return a value of URL to which parameter sequences should be appended.
     */
    public String getBaseActionURL(boolean idempotent) {
        return this.baseActionUrl;
    }
    
    
    /**
     * Returns an idempotent URL that includes a single query parameter that 
     * targets a channel for focus mode by functional name. Additional 
     * query parameters appended will be passed to the focused channel via 
     * the channel's ChannelRuntimeData object.
     *
     * @return a value of URL including a single query parameter.
     * @since  2.5.1
     */
    public String getFnameActionURL(String fname) {
        return null;
    }
    
    /**
     * Returns the URL to invoke one of the workers specified in PortalSessionManager.
     * Typically the channel that is invoked with the worker will have to implement an
     * interface specific for that worker.
     * @param worker - Worker string must be a UPFileSpec.xxx value.
     * @return URL to invoke the worker.
     */
    public String getBaseWorkerURL(String worker) {
        return null;
    }

    /**
       Returns a media base appropriate for web-visible resources used by and
       deployed with the passed in object. If the class of the passed in
       object was loaded from a CAR then a URL appropriate for accessing
       images in CARs is returned. Otherwise, a URL to the base media
       in the web application's document root is returned.
     */
    public String getBaseMediaURL(Object aChannelObject) throws PortalException {
        return getBaseMediaURL( aChannelObject.getClass() );
    }

    /**
       Returns a media base appropriate for web-visible resources used by and
       deployed with the passed in class. If the class of the passed in
       object was loaded from a CAR then a URL appropriate for accessing
       images in CARs is returned. Otherwise, a URL to the base media
       in the web application's document root is returned.
     */
    public String getBaseMediaURL(Class aChannelClass) throws PortalException {
        return null;
    }

    /**
       Returns a media base appropriate for the resource path passed in. The
       resource path is the path to the resource within its channel archive.
       (See org.jasig.portal.car.CarResources class for more information.)
       If the passed in resourcePath matches that of a resource loaded from
       CARs then this method returns a URL appropriate to obtain CAR
       deployed, web-visible resources. Otherwise it returns a URL to the
       traditional media path under the uPortal web application's document
       root.
     */
    public String getBaseMediaURL(String resourcePath) throws PortalException {
        return null;
    }
    
    
    /**
     * Returns the URL to invoke one of the workers specified in PortalSessionManager.
     * Typically the channel that is invoked with the worker will have to implement an
     * interface specific for that worker.
     * @param worker - Worker string must be a UPFileSpec.xxx value.
     * @param idempotent a <code>boolean</code> value sepcifying if a URL should be idempotent
     * @return URL to invoke the worker.
     * @exception PortalException if an error occurs
     */
    public String getBaseWorkerURL(String worker, boolean idempotent) throws PortalException {
        return null;
    }
}
