/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.error.web;

import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletConfig;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.portal.portlet.rendering.IErrorPortletCommands;
import org.jasig.portal.portlet.url.IPortletUrl;
import org.jasig.portal.portlets.error.action.ErrorAction;
import org.jasig.portal.portlets.error.bean.ErrorData;

/**
 * This is the primary controller for the Error Portlet.  Most
 * 'work' is done in the ErrorAction class.  This portlet is just
 * to handle the portlet request, fill the ErrorData object and
 * pass off to the action class.
 *
 * @author Cris J Holdorph, holdorph@unicon.net
 * @version $Revision: 11911 $
 */
public class ErrorPortlet extends GenericPortlet
{
	private static final String DEFAULT_VIEW = "error_html.jsp";
    
    private static final String SHOW_STACK_ACTION = "SHOW_STACK_TRACE";
    private static final String HIDE_STACK_ACTION = "HIDE_STACK_TRACE";

	private static final String ACTION_ATTRIBUTE = "ACTION";
    private static final String VIEW_ATTRIBUTE = "VIEW";
    private static final String REFRESH_URL_ATTRIBUTE = "refreshUrl";
	private static final String RETRY_URL_ATTRIBUTE = "retryUrl";
    private static final String RESTART_URL_ATTRIBUTE = "restartUrl";
    
    private static final String ERROR_DATA_REQUEST_ATTRIBUTE="data";

	private ErrorAction actions = new ErrorAction();


	/**
	 * Called when the portlet is initialized.
	 *
	 * @param config The standard portlet configuration data
	 * structure for any Portlet
	 */
	public void init(PortletConfig config) throws PortletException
	{
		super.init(config);
	}

	/**
	 * Called when the user is interacting with the portlet,
	 * especially with an actionURL.
	 *
	 * @param request The standard portlet request data structure
	 * @param response The standard portlet request data structure
	 */
	public void processAction(ActionRequest request, ActionResponse response)
		throws PortletException, IOException
	{
        PortletSession session = request.getPortletSession();      
        
        // TODO: ErrorPortlet is the last resort, and should handle things like this.
        if (session == null) {
            throw new PortletException("Fatal: PortletSession should not be null.");
        }
       
        ErrorData data=(ErrorData) session.getAttribute(ErrorData.ERROR_DATA_SESSION_ATTRIBUTE);
        if(data!=null) {
            String action = request.getParameter(ACTION_ATTRIBUTE);
            if(HIDE_STACK_ACTION.equals(action)) {
                data.setShowStackTrace(false);
            } else if(SHOW_STACK_ACTION.equals(action)) {
                data.setShowStackTrace(true);
            }
        }
	}

	/**
	 * Called when the portal asks this portlet to render itself.
	 *
	 * @param request The standard portlet request data structure
	 * @param response The standard portlet request data structure
	 */
	public void doView(RenderRequest request, RenderResponse response)
		throws PortletException, IOException
	{
		PortletSession session = request.getPortletSession();
        
		// TODO: ErrorPortlet is the last resort, and should handle things like this.
		if (session == null) {
			throw new PortletException("Fatal: PortletSession should not be null.");
		}

        
		String view = (String)session.getAttribute(VIEW_ATTRIBUTE);

		if (view == null || view.length() == 0) {
			view = DEFAULT_VIEW;
			session.setAttribute(VIEW_ATTRIBUTE,DEFAULT_VIEW);
        }
        
        ErrorData data=(ErrorData) session.getAttribute(ErrorData.ERROR_DATA_SESSION_ATTRIBUTE);
        
        if(data==null) {
            // create no-data error message            
			data = actions.initialize(data);
			session.setAttribute(ErrorData.ERROR_DATA_SESSION_ATTRIBUTE,data);
		}
	
        // make error data available for jsp
        request.setAttribute(ERROR_DATA_REQUEST_ATTRIBUTE,data);
        
        if(data!=null && data.getWindowId() !=null) {
            // generate retry/refresh URLs
            IPortletUrl refreshUrl=(IPortletUrl) response.createRenderURL();
            IErrorPortletCommands refreshCommands=(IErrorPortletCommands) refreshUrl.getUrlConstructor(IErrorPortletCommands.class.getName());
            refreshCommands.refreshPortletWindow(data.getWindowId());
            request.setAttribute(REFRESH_URL_ATTRIBUTE,refreshUrl.toString());
            
            IPortletUrl retryUrl=(IPortletUrl) response.createRenderURL();
            IErrorPortletCommands retryCommands=(IErrorPortletCommands) retryUrl.getUrlConstructor(IErrorPortletCommands.class.getName());
            retryCommands.retryPortletWindow(data.getWindowId());
            request.setAttribute(RETRY_URL_ATTRIBUTE,retryUrl.toString());
            
            IPortletUrl restartUrl=(IPortletUrl) response.createRenderURL();
            IErrorPortletCommands restartCommands=(IErrorPortletCommands) restartUrl.getUrlConstructor(IErrorPortletCommands.class.getName());
            restartCommands.restartPortletWindow(data.getWindowId());
            request.setAttribute(RESTART_URL_ATTRIBUTE,restartUrl.toString());
            String title=data.getWindowTitle();
            if(title!=null) {
                response.setTitle(title);
            }
        }
        
		StringBuffer jspPath = new StringBuffer("/WEB-INF/jsp/error/");
		jspPath.append(view);

		PortletContext context = getPortletContext();
		PortletRequestDispatcher prd = context.getRequestDispatcher(jspPath.toString());
		response.setContentType("text/html");
		prd.include(request,response);
	}
}
