/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.locale;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.i18n.ILocalePreferenceProvider;
import org.jasig.portal.i18n.IUserLocalePreferenceTransfer;
import org.jasig.portal.i18n.IUserLocalePreferencesDao;
import org.jasig.portal.i18n.UserLocalePreferenceTransferImpl;
import org.jasig.portal.portlets.utils.BaseActionPage;
import org.jasig.portal.portlets.utils.RenderUtils;
import org.jasig.portal.security.UserId;
import org.jasig.portal.utils.LocaleUtils;


/**
 * @author David Young, dyoung@unicon.net
 * @version $Revision: 11955 $
 */
public class LocaleSelectorPortlet extends BaseActionPage {
    private static final String JSP = "/WEB-INF/jsp/locale/localeSelector.jsp";

    private static final String COMMA = ",";
    private static final String SAVE ="Save";
    
    private IUserController userController;
    private IUserLocalePreferencesDao userLocalePreferencesDao;
    private List localePreferenceProviders;
    
    /**
     * @param userController The userController to set.
     */
    public void setUserController(IUserController userController) {
        this.userController = userController;
    }

    public IUserController getUserController() {
		return userController;
	}

	/**
     * @param userLocalePreferencesDao Dao to get user locale preferences.
     */
    public void setUserLocalePreferencesDao(
			IUserLocalePreferencesDao userLocalePreferencesDao) {
		this.userLocalePreferencesDao = userLocalePreferencesDao;
	}

    public IUserLocalePreferencesDao getUserLocalePreferencesDao() {
		return userLocalePreferencesDao;
	}

	/*
     * @param localePreferenceProviders providers that gives us the list of available locales
     */
	public void setLocalePreferenceProviders(List localePreferenceProviders) {
		this.localePreferenceProviders = localePreferenceProviders;
	}

	public List getLocalePreferenceProviders() {
		return localePreferenceProviders;
	}

	/**
	 * Render the page for selecting the locale.
	 * @param request http request
	 * @param response http response
	 */
	public void renderPage(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        Locale currentLocale = request.getLocale();
        request.setAttribute("LOCALE", currentLocale);
        request.setAttribute("LOCALE_NAME", currentLocale.getDisplayName(currentLocale));
        UserId userId = userController.getUserId();
        final List<IUserLocalePreferenceTransfer> userPrefs = userLocalePreferencesDao.getUserLocalePreferences(userId);
        
        PortletSession session = request.getPortletSession();
        Boolean moveAction = (Boolean)session.getAttribute("moveAction");
        List userLocales = (List)session.getAttribute("userLocales");
        List selectedLocales = new ArrayList();
        if (moveAction != null && moveAction.booleanValue()) {
        	selectedLocales.addAll(userLocales);
        } else {
	        selectedLocales.addAll(userPrefs);
        }

        request.setAttribute("SELECTED_MAP", getLocaleDisplayNameMap(selectedLocales));
        
        final List locales = new ArrayList(this.localePreferenceProviders.size() * 5);
        for (final Iterator providers = this.localePreferenceProviders.iterator(); providers.hasNext();) {
            final ILocalePreferenceProvider provider = (ILocalePreferenceProvider)providers.next();
            final List prefOrder = provider.getLocalePreferenceOrder();
            
            if (prefOrder != null && prefOrder.size() > 0) {
                locales.addAll(prefOrder);
            }
        }
        
        // remove the locales that are in the user preference list
        locales.removeAll(selectedLocales);
        selectedLocales.addAll(locales);
        request.setAttribute("LOCALES_MAP", getLocaleDisplayNameMap(selectedLocales));
        String userSelectedLocalesString = generateUserSelectedLocalesString(selectedLocales);
        request.setAttribute("USER_SELECTED_LOCALES", userSelectedLocalesString);
        
        RenderUtils.dispatch(request, response, JSP);
    }

    public void processResults(ActionRequest request, ActionResponse response) throws IOException, PortletException {
    	
    }
    
    /**
     * User actions are processed here.  This supports browsers that does not have javascript on.
     * @param request http request
     * @param response http response
     * @throws IOException
     * @throws PortletException
     */
    public void select(ActionRequest request, ActionResponse response) throws IOException, PortletException {
    	List userLocales = new ArrayList();
        PortletSession session = request.getPortletSession();
        session.setAttribute("moveAction", new Boolean(true));
        String userSelectedLocalesString = request.getParameter("USER_SELECTED_LOCALES");
        if (userSelectedLocalesString != null) {
            List selectedLocales = generateLocales(userSelectedLocalesString);
            userLocales.addAll(selectedLocales);
        }
        
        // browser javascript disabled support
        String up = request.getParameter("up");
        String down = request.getParameter("down");
        if (up != null || down != null) {
        	String[] localeStrings = request.getParameterValues("availableLocales");
        	if (localeStrings != null) {
        		for (int localeStringIndex=0; localeStringIndex<localeStrings.length; localeStringIndex++) {
        			String localeString = localeStrings[localeStringIndex];
	        		Locale locale = LocaleUtils.parseToLocale(localeString);
	        		int index = userLocales.indexOf(locale);
	        		if (up != null) {
		        		if (index > 0) {
			        		Locale tempLocale = (Locale)userLocales.get(index - 1);
			        		userLocales.set(index, tempLocale);
			        		userLocales.set(index - 1, locale);
		        		}
	        		} else if (down != null) {
		        		if (index < (userLocales.size() - 1) && index >= 0) {
			        		Locale tempLocale = (Locale)userLocales.get(index + 1);
			        		userLocales.set(index, tempLocale);
			        		userLocales.set(index + 1, locale);
		        		}
	        		}
        		}
        	}
        }
        
        session.setAttribute("userLocales", userLocales);
        
        // save the preferences to store
        String save = request.getParameter("save");
        if (save != null && save.equals(SAVE)) {
        	UserId userId = userController.getUserId();
            final List<IUserLocalePreferenceTransfer> userPrefs = userLocalePreferencesDao.getUserLocalePreferences(userId);
        	// delete old preferences
        	for (int localePreferenceIndex=0; localePreferenceIndex<userPrefs.size(); localePreferenceIndex++) {
        		IUserLocalePreferenceTransfer userLocalePref = (IUserLocalePreferenceTransfer)userPrefs.get(localePreferenceIndex);
        		userLocalePreferencesDao.removeUserLocalePreference(userLocalePref);
        	}
        	// save the new preferences
        	for (int userLocalesIndex=0; userLocalesIndex<userLocales.size(); userLocalesIndex++) {
	        	IUserLocalePreferenceTransfer userLocalePreference = new UserLocalePreferenceTransferImpl();
	        	userLocalePreference.setUserId(userId);
	        	Locale l = (Locale)userLocales.get(userLocalesIndex);
	        	userLocalePreference.setLocale(l);
	        	userLocalePreferencesDao.storeUserLocalePreference(userLocalePreference);
        	}
        }
    }

    /**
     * Generate the list of locales from a list that is comma delimited.
     * @param localesString string of locale code separated by comma
     * @return List of Locales
     */
    private List generateLocales(String localesString) {
    	final String[] localeString = localesString.split(COMMA);
    	List locales = new ArrayList(localeString.length);
    	for (int localeStringIndex=0; localeStringIndex<localeString.length; localeStringIndex++) {
    		String localeCode = localeString[localeStringIndex];
    		if (localeCode != null && localeCode.trim().length() > 0) {
    			locales.add(LocaleUtils.parseToLocale(localeCode.trim()));
    		}
    	}
    	return locales;
    }
    
    /**
     * Given a list of locales, this creates a string of locale codes separated by commas.
     * @param locales List of locale objects
     * @return String
     */
    private String generateUserSelectedLocalesString(List locales) {
    	StringBuffer localesString = new StringBuffer();
    	for (final Iterator localeIt = locales.iterator(); localeIt.hasNext(); ) {
    		Locale l = (Locale)localeIt.next();
    		localesString.append(l);
   			localesString.append(",");
    	}
    	
    	return localesString.toString();
    }
    
    /**
     * Create a map with the locale code as key and the display name in the locale's language.
     * @param locales list of locales
     * @return Map 
     */
    private Map getLocaleDisplayNameMap(List locales) {
    	Map localesMap = new LinkedHashMap(locales.size());
    	for (Iterator localesIt = locales.iterator(); localesIt.hasNext(); ) {
    		Locale locale = (Locale)localesIt.next();
    		localesMap.put(locale.toString(), locale.getDisplayName(locale));
    	}
    	
    	return localesMap;
    }
}
