package org.jasig.portal.portlets.login.web;


import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletMode;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.PortletSession;
import javax.portlet.PortletURL;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.acegisecurity.providers.ProviderManager;
import org.acegisecurity.providers.UsernamePasswordAuthenticationToken;
import org.jasig.portal.context.common.user.AcegiSessionUserController;
import org.jasig.portal.url.WritableHttpServletRequestImpl;

/**
 * This is the primary controller for the Login Portlet. At login a
 * UserControllerImpl is placed in the session.
 * 
 * @author Dennis Skudney, dskudney@unicon.net
 * @version $Revision: 11911 $
 */

public class LoginPortlet extends GenericPortlet {
	private static final String LOGIN_JSP = "/WEB-INF/jsp/login/login.jsp";

	private static final String LOGOUT_JSP = "/WEB-INF/jsp/login/logout.jsp";
	
	private ProviderManager providerManager;
	/**
	 * Called when the portal asks this portlet to render itself.
	 * 
	 * @param request
	 *            The standard portlet request data structure
	 * @param response
	 *            The standard portlet request data structure
	 */
	protected void doView(RenderRequest request, RenderResponse response)
			throws PortletException, IOException {
		
		PortletSession session = request.getPortletSession();
		String page = LOGIN_JSP;
		if (session.getAttribute("user") != null) {
			page = LOGOUT_JSP;
		} else {
			page = LOGIN_JSP;
		}
		response.setContentType("text/html");
		PortletURL actionURL = response.createActionURL();
		actionURL.setPortletMode(PortletMode.VIEW);
		request.setAttribute("actionURL", actionURL.toString());
		PortletContext context = getPortletContext();
		PortletRequestDispatcher prd = context.getRequestDispatcher(page);
		prd.include(request, response);
	}

	/**
	 * Called when the user is interacting with the portlet, especially with an
	 * actionURL.
	 * 
	 * @param request
	 *            The standard portlet request data structure
	 * @param response
	 *            The standard portlet request data structure
	 */
	public void processAction(ActionRequest request, ActionResponse response)
			throws PortletException, IOException {
		//Login
		//ToDo
		//Acegi verification will take place here
		//for now just fake it by placing an instance of UserControllerImpl
		//in the session
		
		/* removed 11/1/06, dmccallum. I don't see that we do anything with the providerManager reference */
		/*
		if(providerManager == null){
			WebApplicationContext contextw = 
				PortletWebApplicationContextUtils.getRequiredWebApplicationContext(this.getPortletContext());
			providerManager = 
				(ProviderManager)contextw.getBean("portal.security.authenticationManager", ProviderManager.class);
		}
		*/
		String userName = request.getParameter("j_username");
		String passWord = request.getParameter("j_password");
		if(userName != null && passWord != null){
			new UsernamePasswordAuthenticationToken(request.getParameter("j_username"), request.getParameter("j_password"));
		}
		PortletSession session = request.getPortletSession();
		if (session.getAttribute("user") == null) {
			WritableHttpServletRequestImpl writeableRequest = new WritableHttpServletRequestImpl(
					(HttpServletRequest) request);
			AcegiSessionUserController user = new AcegiSessionUserController();
			user.processParameters(writeableRequest,
					(HttpServletResponse) response);
			session.setAttribute("user", user);
		}
		//LogOut
		if(request.getParameter("mvcAction") != null && 
				request.getParameter("mvcAction").toString().startsWith("logout")){
			session.removeAttribute("user");
		}
	}

}

