/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.definition.ILocalizedPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.ILocalizedPortletDeployment;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.LocalizedPortletApplicationDefinitionImpl;
import org.jasig.portal.portlet.deployment.PortletApplicationDeploymentId;
import org.jasig.portal.portlet.deployment.IPortletDeployment;
import org.jasig.portal.portlet.deployment.LocalizedPortletDeploymentImpl;
import org.jasig.portal.portlet.registry.general.IPortletApplicationDefinitionRegistry;
import org.jasig.portal.portlet.registry.general.IPortletDeploymentRegistry;
import org.jasig.portal.portlets.utils.BaseActionPage;
import org.jasig.portal.portlets.utils.RenderUtils;
import org.jasig.portal.utils.Comparators;


/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class ViewPortletApplicationDefinitionsPage extends BaseActionPage {
    private static final String JSP = "/WEB-INF/jsp/manager/manageApps.jsp";
    
    private IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry;
    private IPortletDeploymentRegistry portletDeploymentRegistry;
    private ILocaleManager localeManager;
    
    /**
     * @return Returns the portletApplicationDefinitionRegistry.
     */
    public IPortletApplicationDefinitionRegistry getPortletApplicationDefinitionRegistry() {
        return this.portletApplicationDefinitionRegistry;
    }
    /**
     * @param portletApplicationDefinitionRegistry The portletApplicationDefinitionRegistry to set.
     */
    public void setPortletApplicationDefinitionRegistry(IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry) {
        this.portletApplicationDefinitionRegistry = portletApplicationDefinitionRegistry;
    }
    
    /**
     * @return Returns the portletDeploymentRegistry.
     */
    public IPortletDeploymentRegistry getPortletDeploymentRegistry() {
        return this.portletDeploymentRegistry;
    }
    /**
     * @param portletDeploymentRegistry The portletDeploymentRegistry to set.
     */
    public void setPortletDeploymentRegistry(IPortletDeploymentRegistry portletDeploymentRegistry) {
        this.portletDeploymentRegistry = portletDeploymentRegistry;
    }
    
    /**
     * @return Returns the localeManager.
     */
    public ILocaleManager getLocaleManager() {
        return this.localeManager;
    }
    /**
     * @param localeManager The localeManager to set.
     */
    public void setLocaleManager(ILocaleManager localeManager) {
        this.localeManager = localeManager;
    }
    
    
    public void renderPage(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        //Get a list of the portlet app defs
        final Set appDefs = this.portletApplicationDefinitionRegistry.getPortletApplicationDefinitions();
        
        
        //Sorted set of portlet app defs
        final SortedSet portletAppDefinitions = new TreeSet(Comparators.LOCALIZED_PORTLET_APPLICATION_DEFINITION_NAME_COMPARATOR);
        
        //Map of portlet app defs to sorted sets of portlet deployments
        final Map portletDeployments = new HashMap();
        
        //Number of total items to be displayed
        int itemCount = 0;
        
        
        //Iterate through the app defs, updating the sets and map
        for (final Iterator appItr = appDefs.iterator(); appItr.hasNext();) {
            final IPortletApplicationDefinition appDef = (IPortletApplicationDefinition)appItr.next();
            final ILocalizedPortletApplicationDefinition localAppDef = new LocalizedPortletApplicationDefinitionImpl(appDef, this.localeManager);
            
            portletAppDefinitions.add(localAppDef);
            
            final PortletApplicationDeploymentId appDepId = localAppDef.getPortletApplicationDeploymentId();
            
            final Set portlets = this.portletDeploymentRegistry.getPortletDeployments(appDepId);
            final SortedSet sortedPortlets = new TreeSet(Comparators.PORTLET_DEPLOYMENT_NAME_COMPARATOR);

            //Add all the portlet defs to the sorted set
            for (final Iterator portDepItr = portlets.iterator(); portDepItr.hasNext();) {
                final IPortletDeployment portDep = (IPortletDeployment)portDepItr.next();
                final ILocalizedPortletDeployment localPortDep = new LocalizedPortletDeploymentImpl(portDep, this.localeManager);
                sortedPortlets.add(localPortDep);
            }
            
            itemCount += sortedPortlets.size();
            
            portletDeployments.put(localAppDef, sortedPortlets);
        }
        
        request.setAttribute("PORT_APP_DEFS", portletAppDefinitions);
        request.setAttribute("PORT_DEPS", portletDeployments);
        
        RenderUtils.dispatch(request, response, JSP);
    }

    public void processResults(ActionRequest request, ActionResponse response) throws PortletException, IOException {
        //TODO list filtering / sorting / pagination code can go here
    }
}
