/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web;

import java.io.IOException;
import java.util.Iterator;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.definition.ILocalizedPortletDefinition;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.LocalizedPortletDefinitionImpl;
import org.jasig.portal.portlet.registry.general.IPortletApplicationDefinitionRegistry;
import org.jasig.portal.portlet.registry.general.IPortletDefinitionRegistry;
import org.jasig.portal.portlets.utils.BaseActionPage;
import org.jasig.portal.portlets.utils.RenderUtils;
import org.jasig.portal.utils.Comparators;


/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class ViewPortletDefinitionsPage extends BaseActionPage {
    private static final String JSP = "/WEB-INF/jsp/manager/managePortlets.jsp";
    
    private IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry;
    private IPortletDefinitionRegistry portletDefinitionRegistry;
    private ILocaleManager localeManager;
    
    
    /**
     * @return Returns the portletApplicationDefinitionRegistry.
     */
    public IPortletApplicationDefinitionRegistry getPortletApplicationDefinitionRegistry() {
        return this.portletApplicationDefinitionRegistry;
    }
    /**
     * @param portletApplicationDefinitionRegistry The portletApplicationDefinitionRegistry to set.
     */
    public void setPortletApplicationDefinitionRegistry(IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry) {
        this.portletApplicationDefinitionRegistry = portletApplicationDefinitionRegistry;
    }
    
    /**
     * @return Returns the portletDefinitionRegistry.
     */
    public IPortletDefinitionRegistry getPortletDefinitionRegistry() {
        return this.portletDefinitionRegistry;
    }
    /**
     * @param portletDefinitionRegistry The portletDefinitionRegistry to set.
     */
    public void setPortletDefinitionRegistry(IPortletDefinitionRegistry portletDefinitionRegistry) {
        this.portletDefinitionRegistry = portletDefinitionRegistry;
    }
    
    /**
     * @return Returns the localeManager.
     */
    public ILocaleManager getLocaleManager() {
        return this.localeManager;
    }
    /**
     * @param localeManager The localeManager to set.
     */
    public void setLocaleManager(ILocaleManager localeManager) {
        this.localeManager = localeManager;
    }
    
    
    public void renderPage(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        //Get all portlet app definitions
        final Set appDefs = this.portletApplicationDefinitionRegistry.getPortletApplicationDefinitions();
        
        final SortedSet portletDefinitions = new TreeSet(Comparators.LOCALIZED_PORTLET_DEFINITION_NAME_COMPARATOR);
                                                         
        //Iterate throw app defs
        for (final Iterator appItr = appDefs.iterator(); appItr.hasNext();) {
            final IPortletApplicationDefinition appDef = (IPortletApplicationDefinition)appItr.next();
            final PortletApplicationDefinitionId appDefId = appDef.getPortletApplicationDefinitionId();
            
            //Get all portlet defs for the app def
            final Set portletDefs = this.portletDefinitionRegistry.getPortletDefinitions(appDefId);
            
            //Add all the portlet defs to the sorted set
            for (final Iterator portDefItr = portletDefs.iterator(); portDefItr.hasNext();) {
                final IPortletDefinition portDef = (IPortletDefinition)portDefItr.next();
                final ILocalizedPortletDefinition localPortDef = new LocalizedPortletDefinitionImpl(portDef, this.localeManager);
                portletDefinitions.add(localPortDef);
            }
        }
        
        request.setAttribute("PORT_DEFS", portletDefinitions);
        
        RenderUtils.dispatch(request, response, JSP);
    }

    public void processResults(ActionRequest request, ActionResponse response) throws PortletException, IOException {
        //TODO list filtering / sorting / pagination code can go here
    }
}
