/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web.pages.application;

import java.io.IOException;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.lang.StringUtils;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinitionControl;
import org.jasig.portal.portlets.utils.ConfigurationException;
import org.jasig.services.persondir.IPersonAttributeDao;


/**
 * Displayes the User Attribute mapping wizard page for portlet application
 * definition mangement.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class UserAttributesWizardPage extends BasePortletApplicationWizardPage {
    private static final String MAPPING_PREFIX = "ATTR_MAPPING__";
    
    private IPersonAttributeDao personAttributeDao;
    
    /**
     * @return Returns the personAttributeDao.
     */
    public IPersonAttributeDao getPersonAttributeDao() {
        return this.personAttributeDao;
    }
    /**
     * @param personAttributeDao The personAttributeDao to set.
     */
    public void setPersonAttributeDao(IPersonAttributeDao personAttributeDao) {
        this.personAttributeDao = personAttributeDao;
    }

    
    public String getTitle() {
        return "User Attributes";
    }

    public void renderPage(RenderRequest request, RenderResponse response) throws IOException, PortletException {
        if (this.personAttributeDao == null) {
            throw new IllegalStateException("No IPersonAttributeDao configured");
        }

        request.setAttribute("PORT_USER_ATTRS", this.personAttributeDao.getPossibleUserAttributeNames());
        
        super.renderPage(request, response);
    }

    public void processResults(ActionRequest request, ActionResponse response) throws IOException, PortletException, ConfigurationException {
        final PortletSession session = request.getPortletSession();
        final IPortletApplicationDefinitionControl portAppDefControl = (IPortletApplicationDefinitionControl)session.getAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF);
        
        if (portAppDefControl == null) {
            throw new IllegalStateException("IPortletApplicationDefinitionControl cannot be found in session for key '" + PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF + "'");
        }
        
        final Map<String, String> attrMapping = new HashMap<String, String>();
        final Set upAttributes = this.personAttributeDao.getPossibleUserAttributeNames();
        final Map<String, String> errors = new HashMap<String, String>();
        
        for (final Enumeration mappingEnum = request.getParameterNames(); mappingEnum.hasMoreElements();) {
            final String paramName = (String)mappingEnum.nextElement();
            
            if (paramName.startsWith(MAPPING_PREFIX)) {
                final String appAttrName = paramName.substring(MAPPING_PREFIX.length());
                final String upAttrName = StringUtils.trimToNull(request.getParameter(paramName));
                
                if (upAttrName != null && upAttributes != null && !upAttributes.contains(upAttrName)) {
                    errors.put(appAttrName, "'" + upAttrName + "' is not a valid uPortal user attribute to map '" + appAttrName + "' to.");
                }
                else {
                    attrMapping.put(appAttrName, upAttrName);
                }
            }
        }

        portAppDefControl.setUserAttributeMappings(attrMapping);
        
        if (errors.size() > 0) {
            throw new ConfigurationException(errors);
        }
    }
    
    public Map<String, String> isCompleted(ActionRequest request, ActionResponse response) throws IOException, PortletException {
        final Map<String, String> errors = new HashMap<String, String>();

        final PortletSession session = request.getPortletSession();
        final IPortletApplicationDefinitionControl portAppDefControl = (IPortletApplicationDefinitionControl)session.getAttribute(PortletApplicationDefinitionWizard.PARAM_PORT_APP_DEF);
        
        if (portAppDefControl != null) {
            if (portAppDefControl.getUserAttributeMappings() == null) {
                errors.put("USER_ATTR_MAPPINGS", "No user attribute mappings specified for the PortletApplicationDefinition");
            }
        }
        else {
            errors.put("", "No PortletApplicationDefinition in the session.");
        }
        
        return Collections.unmodifiableMap(errors);
    }
}
