/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web.pages.portlet;

import java.io.IOException;

import javax.portlet.PortletException;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.portal.portlet.definition.ILocalizedPortletDefinitionControl;
import org.jasig.portal.portlets.utils.JspWizardPage;


/**
 * Simply copies the LocalizedPortletDefinitionControlImpl from the PortletSession
 * to the request attributes using the {@link PortletDefinitionWizard#PARAM_PORT_DEF}
 * key for both. This is done for the renderPage and renderSummary methods.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
abstract class BasePortletWizardPage extends JspWizardPage {
    public void renderPage(RenderRequest request, RenderResponse response) throws IOException, PortletException {
        final PortletSession session = request.getPortletSession();
        final ILocalizedPortletDefinitionControl localizedPortDef = (ILocalizedPortletDefinitionControl)session.getAttribute(PortletDefinitionWizard.PARAM_PORT_DEF);
        if (localizedPortDef != null) {
            request.setAttribute(PortletDefinitionWizard.PARAM_PORT_DEF, localizedPortDef);
        }
        
        super.renderPage(request, response);
    }

    public void renderSummary(RenderRequest request, RenderResponse response) throws IOException, PortletException {
        final PortletSession session = request.getPortletSession();
        final ILocalizedPortletDefinitionControl localizedPortDef = (ILocalizedPortletDefinitionControl)session.getAttribute(PortletDefinitionWizard.PARAM_PORT_DEF);
        if (localizedPortDef != null) {
            request.setAttribute(PortletDefinitionWizard.PARAM_PORT_DEF, localizedPortDef);
        }

        super.renderSummary(request, response);
    }
}
