/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web.pages.portlet;

import java.io.IOException;
import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.groups.IEntityGroup;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.portal.portlet.servant.IPortletServant;
import org.jasig.portal.portlets.utils.ConfigurationException;


/**
 * Displayes the Categories information wizard page for portlet
 * definition mangement.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class CategoriesWizardPage extends BasePortletWizardPage {
	
    private Set<Class> categoriesEntityTypeFilter = null;
    private ICompositeGroupService compositeGroupService;
    private IPortletServant groupsManagerServant;
    
    public String getTitle() {
        return "Categories";
    }
    
    public void renderPage(RenderRequest request, RenderResponse response) throws IOException, PortletException {
        final IPortletServant servant = this.getPortletServant(request);
        servant.render(request, response);
        final IGroupMember[] categoryGroups = (IGroupMember[]) servant.getResult(request);
        final PortletSession session = request.getPortletSession();
        session.setAttribute(PortletDefinitionWizard.PARAM_CATEGORY_GROUPS, categoryGroups);
            
        // TODO: signalling for the next page is too late at this point.
        // Portlet event model (a la jsr286) will allow for an easy solution, but
        // for now it's probably best to remove the "Done with selection" button in the
        // group portlet, and enable next button on the wizard as soon as any group is
        // selected. 
    }
    
    public void renderSummary(RenderRequest request, RenderResponse response) throws IOException, PortletException {
        final PortletSession session = request.getPortletSession();
        final IGroupMember[] categoryGroups = (IGroupMember[])session.getAttribute(PortletDefinitionWizard.PARAM_CATEGORY_GROUPS);

        if (categoryGroups != null) {
            final IEntityGroup[] entityGroups = new IEntityGroup[categoryGroups.length];
            
            for (int index = 0; index < categoryGroups.length; index++) {
                entityGroups[index] = compositeGroupService.findGroup(categoryGroups[index].getEntityIdentifier().getKey());
            }
            
            request.setAttribute(PortletDefinitionWizard.PARAM_CATEGORY_GROUPS, entityGroups);
        }
        
        super.renderSummary(request, response);
    }
    
    public void processResults(ActionRequest request, ActionResponse response) throws IOException, PortletException, ConfigurationException {
        final IPortletServant servant = this.getPortletServant(request);
        servant.processAction(null, request, response);
    }

    public Map isCompleted(ActionRequest request, ActionResponse response) throws IOException, PortletException {
        final PortletSession session = request.getPortletSession();
        final IGroupMember[] categoryGroups = (IGroupMember[])session.getAttribute(PortletDefinitionWizard.PARAM_CATEGORY_GROUPS);
        
        if (categoryGroups == null || categoryGroups.length <= 0) {
            return Collections.singletonMap("CATEGORY_GROUPS", "At least one Category must be selected.");
        }
        else {
            groupsManagerServant.reset(request);
            return Collections.EMPTY_MAP;
        }
    }
    
    private IPortletServant getPortletServant(PortletRequest request) {
        PortletSession session = request.getPortletSession();
        groupsManagerServant.setAttribute(request, "groupMembers", session.getAttribute(PortletDefinitionWizard.PARAM_CATEGORY_GROUPS));
        groupsManagerServant.setAttribute(request, "entityTypeFilter", categoriesEntityTypeFilter);
        groupsManagerServant.setAttribute(request,"allowEntitySelect",false);
        groupsManagerServant.setAttribute(request,"allowFinish",false);
        return groupsManagerServant;
    }
    
    public void setCategoriesEntityTypeFilter(Set genericCategoriesEntityTypeFilter) throws ClassNotFoundException {
        this.categoriesEntityTypeFilter = new HashSet<Class>();
        for(Object o : genericCategoriesEntityTypeFilter) {
            if (o instanceof String) {
                this.categoriesEntityTypeFilter.add(Class.forName((String)o));
            } else if (o instanceof Class) {
                this.categoriesEntityTypeFilter.add((Class)o);
            } else {
                throw new IllegalArgumentException("entity type filter elements must be either String or Class: " + o.getClass().getName());
            }
        }
    }
    
    public void setCompositeGroupService(ICompositeGroupService compositeGroupService) {
        this.compositeGroupService = compositeGroupService;
    }
    public IPortletServant getGroupsManagerServant() {
        return groupsManagerServant;
    }
    public void setGroupsManagerServant(IPortletServant groupsManagerServant) {
        this.groupsManagerServant = groupsManagerServant;
    }
}
