/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web.pages.portlet;

import java.io.IOException;
import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.groups.IEntityGroup;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.portal.portlet.servant.IPortletServant;
import org.jasig.portal.portlets.utils.ConfigurationException;


/**
 * Displayes the Groups information wizard page for portlet
 * definition mangement. 
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @author Dan McCallum <a href="mailto:dmccallum@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class PermissionsWizardPage extends BasePortletWizardPage {
	
	private Set<Class> permissionsEntityTypeFilter = null;
    private ICompositeGroupService compositeGroupService;
    private IPortletServant groupsManagerServant;
    
    public String getTitle() {
        return "Permissions";
    }
    
    public void renderPage(RenderRequest request, RenderResponse response) throws IOException, PortletException {
        final IPortletServant servant = this.getPortletServant(request);
        servant.render(request, response);
        final IGroupMember[] permissedGroups = (IGroupMember[]) servant.getResult(request);
        final PortletSession session = request.getPortletSession();
        session.setAttribute(PortletDefinitionWizard.PARAM_PERMISSION_GROUPS, permissedGroups);
            
        // TODO: signalling for the next page is too late at this point.
        // Portlet event model (a la jsr286) will allow for an easy solution, but
        // for now it's probably best to remove the "Done with selection" button in the
        // group portlet, and enable next button on the wizard as soon as any group is
        // selected. 
    }
    
    public void renderSummary(RenderRequest request, RenderResponse response) throws IOException, PortletException {
        final PortletSession session = request.getPortletSession();
        final IGroupMember[] permissedGroups = (IGroupMember[])session.getAttribute(PortletDefinitionWizard.PARAM_PERMISSION_GROUPS);

        if (permissedGroups != null) {
            final IEntityGroup[] entityGroups = new IEntityGroup[permissedGroups.length];
            
            for (int index = 0; index < permissedGroups.length; index++) {
                entityGroups[index] = compositeGroupService.findGroup(permissedGroups[index].getEntityIdentifier().getKey());
            }
            
            request.setAttribute(PortletDefinitionWizard.PARAM_PERMISSION_GROUPS, entityGroups);
        }
        
        super.renderSummary(request, response);
    }
    
    public void processResults(ActionRequest request, ActionResponse response) throws IOException, PortletException, ConfigurationException {
        final IPortletServant servant = this.getPortletServant(request);
        servant.processAction(null, request, response);
    }

    /**
     * Always returns an empty map. A portlet need not be subscribable at all times.
     */
    public Map isCompleted(ActionRequest request, ActionResponse response) throws IOException, PortletException {
        groupsManagerServant.reset(request);
        return Collections.EMPTY_MAP;
    }
    
    private IPortletServant getPortletServant(PortletRequest request) {
        groupsManagerServant.setAttribute(request, "groupMembers", request.getPortletSession().getAttribute(PortletDefinitionWizard.PARAM_PERMISSION_GROUPS));
        groupsManagerServant.setAttribute(request, "entityTypeFilter", permissionsEntityTypeFilter);
        groupsManagerServant.setAttribute(request,"allowEntitySelect",false);
        groupsManagerServant.setAttribute(request,"allowFinish",false);
        return groupsManagerServant;
    }
    
    public void setPermissionsEntityTypeFilter(Set genericCategoriesEntityTypeFilter) throws ClassNotFoundException {
        this.permissionsEntityTypeFilter = new HashSet<Class>();
        for(Object o : genericCategoriesEntityTypeFilter) {
            if (o instanceof String) {
                this.permissionsEntityTypeFilter.add(Class.forName((String)o));
            } else if (o instanceof Class) {
                this.permissionsEntityTypeFilter.add((Class)o);
            } else {
                throw new IllegalArgumentException("entity type filter elements must be either String or Class: " + o.getClass().getName());
            }
        }
    }
    
    public void setCompositeGroupService(ICompositeGroupService compositeGroupService) {
        this.compositeGroupService = compositeGroupService;
    }
    
    public IPortletServant getGroupsManagerServant() {
        return groupsManagerServant;
    }
    
    public void setGroupsManagerServant(IPortletServant groupsManagerServant) {
        this.groupsManagerServant = groupsManagerServant;
    }
}
