/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.manager.web.pages.portlet;

import java.io.IOException;
import java.util.Iterator;
import java.util.Set;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.collections.IteratorUtils;
import org.apache.commons.lang.StringUtils;
import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.groups.IEntity;
import org.jasig.gap.groups.IEntityGroup;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.gap.permissions.IPermission;
import org.jasig.gap.permissions.IPermissionManager;
import org.jasig.gap.permissions.IUpdatingPermissionManager;
import org.jasig.portal.i18n.ILocaleManager;
import org.jasig.portal.portlet.definition.IPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.IPortletDefinitionControl;
import org.jasig.portal.portlet.definition.LocalizedPortletDefinitionControlImpl;
import org.jasig.portal.portlet.definition.PortletApplicationDefinitionId;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.deployment.IPortletDeployment;
import org.jasig.portal.portlet.deployment.PortletDeploymentId;
import org.jasig.portal.portlet.registry.general.IPortletApplicationDefinitionRegistry;
import org.jasig.portal.portlet.registry.general.IPortletDefinitionRegistry;
import org.jasig.portal.portlet.registry.general.IPortletDeploymentRegistry;
import org.jasig.portal.portlet.registry.transitory.ITransientPortletDefinitionRegistry;
import org.jasig.portal.portlet.transfer.IPortletDefinitionTransfer;
import org.jasig.portal.portlet.transfer.impl.PortletApplicationDefinitionTransferImpl;
import org.jasig.portal.portlet.transfer.impl.TransferUtils;
import org.jasig.portal.portlets.utils.BaseWizard;


/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class PortletDefinitionWizard extends BaseWizard {
    
    static final String PARAM_PORT_DEF          = "PORT_DEF";
    static final String PARAM_PORT_DEF_ID       = "PORT_DEF_ID";
    static final String PARAM_PORT_DEP_ID       = "PORT_DEP_ID";
    static final String PARAM_PORT_APP_DEF_ID   = "PORT_APP_DEF_ID";
    static final String PARAM_CATEGORY_GROUPS   = "CATEGORY_GROUPS";
    static final String PARAM_PERMISSION_GROUPS = "PERMISSION_GROUPS";
    
    private static final String PARAM_WIZ_MODE = "WIZ_MODE";
    private static final String MODE_NEW    = "MODE_NEW";
    private static final String MODE_EDIT   = "MODE_EDIT";
    
    // TODO: these should probably be defined elsewhere
    public static final String DEFAULT_FRAMEWORK_OWNER_TOKEN = "UP_FRAMEWORK";
    public static final String DEFAULT_SUBSCRIPTION_ACTIVITY_TOKEN = "SUBSCRIBE";
    public static final String DEFAULT_PORTLET_DEF_TARGET_TOKEN_PREFIX = "CHAN_ID";
    
    private String frameworkOwnerToken = DEFAULT_FRAMEWORK_OWNER_TOKEN;
    private String subscriptionActivityToken = DEFAULT_SUBSCRIPTION_ACTIVITY_TOKEN;
    private String portletDefTargetTokenPrefix = DEFAULT_PORTLET_DEF_TARGET_TOKEN_PREFIX;
    private ITransientPortletDefinitionRegistry transientPortletDefinitionRegistry;
    private IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry;
    private IPortletDefinitionRegistry portletDefinitionRegistry;
    private IPortletDeploymentRegistry portletDeploymentRegistry;
    private ILocaleManager localeManager;
    private ICompositeGroupService compositeGroupService;
    private IAuthorizationService authzService;
    
    
    /**
     * @return Returns the portletApplicationDefinitionRegistry.
     */
    public IPortletApplicationDefinitionRegistry getPortletApplicationDefinitionRegistry() {
        return this.portletApplicationDefinitionRegistry;
    }
    /**
     * @param portletApplicationDefinitionRegistry The portletApplicationDefinitionRegistry to set.
     */
    public void setPortletApplicationDefinitionRegistry(IPortletApplicationDefinitionRegistry portletApplicationDefinitionRegistry) {
        this.portletApplicationDefinitionRegistry = portletApplicationDefinitionRegistry;
    }
    
    /**
     * @return Returns the portletDefinitionRegistry.
     */
    public IPortletDefinitionRegistry getPortletDefinitionRegistry() {
        return this.portletDefinitionRegistry;
    }
    /**
     * @param portletDefinitionRegistry The portletDefinitionRegistry to set.
     */
    public void setPortletDefinitionRegistry(IPortletDefinitionRegistry portletDefinitionRegistry) {
        this.portletDefinitionRegistry = portletDefinitionRegistry;
    }
    
    /**
     * @return Returns the portletDeploymentRegistry.
     */
    public IPortletDeploymentRegistry getPortletDeploymentRegistry() {
        return this.portletDeploymentRegistry;
    }
    /**
     * @param portletDeploymentRegistry The portletDeploymentRegistry to set.
     */
    public void setPortletDeploymentRegistry(IPortletDeploymentRegistry portletDeploymentRegistry) {
        this.portletDeploymentRegistry = portletDeploymentRegistry;
    }
    
    /**
     * @return Returns the transientPortletDefinitionRegistry.
     */
    public ITransientPortletDefinitionRegistry getTransientPortletDefinitionRegistry() {
        return this.transientPortletDefinitionRegistry;
    }
    /**
     * @param transientPortletDefinitionRegistry The transientPortletDefinitionRegistry to set.
     */
    public void setTransientPortletDefinitionRegistry(ITransientPortletDefinitionRegistry transientPortletDefinitionRegistry) {
        this.transientPortletDefinitionRegistry = transientPortletDefinitionRegistry;
    }
    
    /**
     * @return Returns the localeManager.
     */
    public ILocaleManager getLocaleManager() {
        return this.localeManager;
    }
    /**
     * @param localeManager The localeManager to set.
     */
    public void setLocaleManager(ILocaleManager localeManager) {
        this.localeManager = localeManager;
    }
    
    public void setCompositeGroupService(
        ICompositeGroupService compositeGroupService) {
        this.compositeGroupService = compositeGroupService;
    }
    	
    public IAuthorizationService getAuthzService() {
        return authzService;
    }
    	
    public void setAuthzService(IAuthorizationService authzService) {
        this.authzService = authzService;
    }
    
    public String getFrameworkOwnerToken() {
		return frameworkOwnerToken;
	}
	public void setFrameworkOwnerToken(String frameworkOwnerToken) {
		this.frameworkOwnerToken = frameworkOwnerToken;
	}
	
	public String getPortletDefTargetTokenPrefix() {
		return portletDefTargetTokenPrefix;
	}
	public void setPortletDefTargetTokenPrefix(String portletDefTargetTokenPrefix) {
		this.portletDefTargetTokenPrefix = portletDefTargetTokenPrefix;
	}
    
    protected String resolveCurrentPageName(PortletRequest request) {
        final PortletSession session = request.getPortletSession();
        final String paramCurrentPage = super.resolveCurrentPageName(request);
        
        if (paramCurrentPage != null) {
            session.setAttribute(this.getWizardPageParam(), paramCurrentPage);
            return paramCurrentPage;
        }
        else {
            final String sessionCurrentPage = (String)session.getAttribute(this.getWizardPageParam());

            return sessionCurrentPage;
        }
    }

    public void renderPage(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        final PortletSession session = request.getPortletSession();
        final String mode = (String)session.getAttribute(PARAM_WIZ_MODE);
        if (mode != null) {
            request.setAttribute(PARAM_WIZ_MODE, mode);
        }
        
        super.renderPage(request, response);
    }
    
    public void newPortlet(ActionRequest request, ActionResponse response) {
        this.clearSession(request);
        
        final String portAppDefIdStr = StringUtils.trimToNull(request.getParameter(PARAM_PORT_APP_DEF_ID));
        final String portDepIdStr = StringUtils.trimToNull(request.getParameter(PARAM_PORT_DEP_ID));
        
        if (portAppDefIdStr == null) {
            throw new IllegalArgumentException("'" + PARAM_PORT_APP_DEF_ID + "' may not be null");
        }
        if (portDepIdStr == null) {
            throw new IllegalArgumentException("'" + PARAM_PORT_DEP_ID + "' may not be null");
        }

        
        //Get the portlet application definition from the registry
        final PortletApplicationDefinitionId portAppDefId = new PortletApplicationDefinitionId(portAppDefIdStr);
        //TODO: should publishing of a portet from non-published application be allowed ? If not, this should be switched to regular registry
        final IPortletApplicationDefinition portAppDef = this.portletApplicationDefinitionRegistry.getPortletApplicationDefinition(portAppDefId);
        if (portAppDef == null) {
            throw new IllegalStateException("No PortletApplicationDefinition found for ID '" + portAppDefId + "' specified by parameter '" + PARAM_PORT_APP_DEF_ID + "'");
        }
        
        
        //Get the portlet deployment from the registry
        final PortletDeploymentId portDepId = new PortletDeploymentId(portAppDef.getPortletApplicationDeploymentId(), portDepIdStr);
        final IPortletDeployment portDepControl = this.portletDeploymentRegistry.getPortletDeployment(portDepId);
        if (portAppDef == null) {
            throw new IllegalStateException("No PortletDeployment found for ID '" + portDepId + "' specified by parameter '" + PARAM_PORT_DEP_ID + "'");
        }
        
        //Use the temp registry to create a def control for the deployment 
        final IPortletDefinitionControl portDefControl = this.transientPortletDefinitionRegistry.createTransientPortletDefinitionControl(portDepControl, portAppDef);

        final PortletSession session = request.getPortletSession();
        //Set the portlet app def in the session
        session.setAttribute(PARAM_PORT_DEF, new LocalizedPortletDefinitionControlImpl(portDefControl, this.localeManager));
        //Clear the port def ID from the session to ensure a new def is created upon wizard completion
        session.removeAttribute(PARAM_PORT_DEF_ID);
        
        session.setAttribute(PARAM_WIZ_MODE, MODE_NEW);
    }
    
    public void editPortlet(ActionRequest request, ActionResponse response) {
        this.clearSession(request);
        
        final String portDefIdStr = StringUtils.trimToNull(request.getParameter(PARAM_PORT_DEF_ID));
        
        if (portDefIdStr == null) {
            throw new IllegalArgumentException("'" + PARAM_PORT_DEF_ID + "' may not be null");
        }

        final PortletDefinitionId portDefId = new PortletDefinitionId(portDefIdStr);

        this.copyPortletDefinitionIntoSession(request, portDefId);
        this.copyCategoryGroupsIntoSession(request, portDefId);
        this.copyPermissedGroupsIntoSession(request, portDefId);
        
        final PortletSession session = request.getPortletSession();
        //Store the original ID in the session
        session.setAttribute(PARAM_PORT_DEF_ID, portDefId);
        //Change to edit mode
        session.setAttribute(PARAM_WIZ_MODE, MODE_EDIT);
    }
    
    public void copyPortlet(ActionRequest request, ActionResponse response) {
        this.clearSession(request);
        
        final String portDefIdStr = StringUtils.trimToNull(request.getParameter(PARAM_PORT_DEF_ID));
        
        if (portDefIdStr == null) {
            throw new IllegalArgumentException("'" + PARAM_PORT_DEF_ID + "' may not be null");
        }

        final PortletDefinitionId portDefId = new PortletDefinitionId(portDefIdStr);
        
        this.copyPortletDefinitionIntoSession(request, portDefId);
        this.copyCategoryGroupsIntoSession(request, portDefId);
        this.copyCategoryGroupsIntoSession(request, portDefId);
        
        final PortletSession session = request.getPortletSession();
        //Clear the Id from the session so this will be stored as a new definition
        session.removeAttribute(PARAM_PORT_DEF_ID);
        
        session.setAttribute(PARAM_WIZ_MODE, MODE_EDIT);
    }
    
    public void deletePortlet(ActionRequest request, ActionResponse response) {
        final String portDefIdStr = StringUtils.trimToNull(request.getParameter(PARAM_PORT_DEF_ID));
        
        if (portDefIdStr != null) {
            final PortletDefinitionId portDefId = new PortletDefinitionId(portDefIdStr);
            this.portletDefinitionRegistry.removePortletDefinition(portDefId);

            final IEntity portDefIdEnt = compositeGroupService.getEntity(portDefId.toString(), PortletDefinitionId.class);
            for (final Iterator groupItr = portDefIdEnt.getAllContainingGroups(); groupItr.hasNext();) {
                final IEntityGroup group = (IEntityGroup)groupItr.next();
                group.removeMember(portDefIdEnt);
                group.update();
            }
        }
        else {
            throw new IllegalArgumentException("'" + PARAM_PORT_DEF_ID + "' may not be null");
        }
    }
    
    public void submitPortlet(ActionRequest request, ActionResponse response) {
        final PortletSession session = request.getPortletSession();
        PortletDefinitionId portDefId = (PortletDefinitionId)session.getAttribute(PARAM_PORT_DEF_ID);
        final IPortletDefinitionControl portDef = (IPortletDefinitionControl)session.getAttribute(PARAM_PORT_DEF);
        
        if (portDef == null) {
            throw new IllegalStateException("No IPortletDefinitionControl found in session for key '" + PARAM_PORT_DEF + "'");
        }

        this.transientPortletDefinitionRegistry.removeTransientPortletDefinition(portDef.getPortletDefinitionId());

        final IEntity portDefIdEnt;
        if (portDefId == null) {
            // determine portlet definition application id (transient registry could've created a transient one,
            // so we can't rely on what's currently set in portDef. This shouldn't normally happen, since one shouldn't be
            // able to "create new portet" for an application that hasn't been published, but this does happen when
            // publishing with guest user (see TODO comments in newPortlet() method)
            IPortletDeployment portDep = this.portletDeploymentRegistry.getPortletDeployment(portDef.getPortletDeploymentId());
            if(portDep==null) {
                throw new IllegalStateException("unable to obtain portDep for id="+portDef.getPortletDeploymentId()+" (this could happen if the definition was deleted while publishing was taking place)");
            }
            Set applicationDefinitions=this.portletApplicationDefinitionRegistry.getPortletApplicationDefinitions(portDep.getPortletApplicationDeploymentId());
            final PortletApplicationDefinitionId appDefId;
            if(applicationDefinitions==null || applicationDefinitions.isEmpty()) {
                PortletApplicationDefinitionTransferImpl appDefTransfer=new PortletApplicationDefinitionTransferImpl();
                appDefTransfer.setPortletApplicationDeploymentId(portDep.getPortletApplicationDeploymentId());
                IPortletApplicationDefinition newAppDef=this.portletApplicationDefinitionRegistry.createPortletApplicationDefinition(appDefTransfer);
                appDefId=newAppDef.getPortletApplicationDefinitionId();
            } else {
                IPortletApplicationDefinition firstAppDef=(IPortletApplicationDefinition) applicationDefinitions.iterator().next();
                appDefId=firstAppDef.getPortletApplicationDefinitionId();
            }
            portDef.setPortletApplicationDefinitionId(appDefId);
            IPortletDefinition realPortDef=this.portletDefinitionRegistry.createPortletDefinition(portDef);
            portDefIdEnt = compositeGroupService.getEntity(realPortDef.getPortletDefinitionId().toString(), PortletDefinitionId.class);
            portDefId = realPortDef.getPortletDefinitionId();
        }
        else {
            portDef.setPortletDefinitionId(portDefId);
            this.portletDefinitionRegistry.updatePortletDefinition(portDef);
            
            portDefIdEnt = compositeGroupService.getEntity(portDef.getPortletDefinitionId().toString(), PortletDefinitionId.class);
            for (final Iterator groupItr = portDefIdEnt.getAllContainingGroups(); groupItr.hasNext();) {
                final IEntityGroup group = (IEntityGroup)groupItr.next();
                group.removeMember(portDefIdEnt);
                group.update();
            }
            
            clearFrameworkPermissions(request, portDefId);
            
        }
        
        final IGroupMember[] categoryGroups = (IGroupMember[])session.getAttribute(PARAM_CATEGORY_GROUPS);
        for (int index = 0; index < categoryGroups.length; index++) {
            final IEntityGroup categoryGroup = compositeGroupService.findGroup(categoryGroups[index].getEntityIdentifier().getKey());
            categoryGroup.addMember(portDefIdEnt);
            categoryGroup.updateMembers();
        }
        
        saveFrameworkPermissions(request, portDefId);
        
        this.clearSession(request);
        session.removeAttribute(this.getWizardPageParam());
    }
    
    public void cancelPortlet(ActionRequest request, ActionResponse response) {
        this.clearSession(request);
        
        final PortletSession session = request.getPortletSession();
        session.removeAttribute(this.getWizardPageParam());
    }
    
    private void clearFrameworkPermissions(ActionRequest request, final PortletDefinitionId portDefId) {
    	// TODO can we avoid repeatedly creating these perm mgrs?
        IUpdatingPermissionManager permMgr = authzService.newUpdatingPermissionManager(frameworkOwnerToken);
        IPermission[] permissions = 
        	permMgr.getPermissions(subscriptionActivityToken, createTargetId(portDefId));
        permMgr.removePermissions(permissions);
    }
    
    private void saveFrameworkPermissions(ActionRequest request, final PortletDefinitionId portDefId) {
    	
    	final PortletSession session = request.getPortletSession();
        // TODO can we avoid repeatedly creating these perm mgrs?
        
    	final IGroupMember[] permissedGroups = (IGroupMember[])session.getAttribute(PARAM_PERMISSION_GROUPS);
    	
    	if ( permissedGroups == null || permissedGroups.length == 0 ) {
    		return;
    	}
    	
    	IUpdatingPermissionManager permMgr = authzService.newUpdatingPermissionManager(frameworkOwnerToken);
    	
    	final IPermission[] permissions = new IPermission[permissedGroups.length];
        for (int index = 0; index < permissedGroups.length; index++) {
        	IAuthorizationPrincipal authzPrincipal = 
        		authzService.newPrincipal(permissedGroups[index]); 
        	IPermission permission = permMgr.newPermission(authzPrincipal);
        	permission.setActivity(subscriptionActivityToken);
        	permission.setTarget(createTargetId(portDefId));
        	permission.setType(IPermission.PERMISSION_TYPE_GRANT);
        	permissions[index] = permission;
        }
        
        permMgr.addPermissions(permissions);
        
    }
    
    private void copyCategoryGroupsIntoSession(ActionRequest request, final PortletDefinitionId portDefId) {
        final PortletSession session = request.getPortletSession();
        
        final IEntity portDefIdEnt = compositeGroupService.getEntity(portDefId.toString(), PortletDefinitionId.class);
        final Iterator groupItr = portDefIdEnt.getAllContainingGroups();
        final IGroupMember[] categoryGroups = (IGroupMember[])IteratorUtils.toArray(groupItr, IGroupMember.class);
        session.setAttribute(PARAM_CATEGORY_GROUPS, categoryGroups);
    }
    
    private void copyPermissedGroupsIntoSession(ActionRequest request, final PortletDefinitionId portDefId) {
    	final PortletSession session = request.getPortletSession();
        IPermissionManager permMgr = authzService.newPermissionManager(frameworkOwnerToken);
        IAuthorizationPrincipal[] authorizedPrincipals = 
        	permMgr.getAuthorizedPrincipals(subscriptionActivityToken, createTargetId(portDefId));
        IGroupMember[] permissedGroups = new IGroupMember[authorizedPrincipals.length];
        for (int k = 0; k < authorizedPrincipals.length; k++) {
        	permissedGroups[k] = authzService.getGroupMember(authorizedPrincipals[k]);
        }
        session.setAttribute(PARAM_PERMISSION_GROUPS, permissedGroups);
    }

    private void copyPortletDefinitionIntoSession(ActionRequest request, PortletDefinitionId portDefId) {
        final IPortletDefinition portDef = this.portletDefinitionRegistry.getPortletDefinition(portDefId);
        
        if (portDef == null) {
            throw new IllegalStateException("No PortletDefinition found for ID '" + portDefId + "' specified by parameter '" + PARAM_PORT_DEF_ID + "'");
        }
        
        //Get the underlying transfer to avoid the layered view of the portlet def.
        final IPortletDefinitionTransfer portDefTransf = portDef.getPortletDefinitionTransfer();
        
        //Get the IPortletDeployment the app def is based on
        final IPortletDeployment portDepControl = this.portletDeploymentRegistry.getPortletDeployment(portDef.getPortletDeploymentId());
        final IPortletApplicationDefinition portAppDef = this.portletApplicationDefinitionRegistry.getPortletApplicationDefinition(portDef.getPortletApplicationDefinitionId());
        
        //Use the temp registry to create a def control for the deployment 
        final IPortletDefinitionControl portDefControl = this.transientPortletDefinitionRegistry.createTransientPortletDefinitionControl(portDepControl, portAppDef);
        
        //Clone the data from the real app def to the control for in-memory manipulation
        TransferUtils.deepCopyPortletDefinitionTransferProperties(portDefTransf, portDefControl.getPortletDefinitionTransfer());
        
        this.transientPortletDefinitionRegistry.updatePortletDefinition(portDefControl);
        
        final PortletSession session = request.getPortletSession();
        //Set the portlet app def in the session
        session.setAttribute(PARAM_PORT_DEF, new LocalizedPortletDefinitionControlImpl(portDefControl, this.localeManager));
    }
    
    private void clearSession(PortletRequest request) {
        final PortletSession session = request.getPortletSession();

        //Ensure the session is clean and ready to go
        final IPortletDefinitionControl portDef = (IPortletDefinitionControl)session.getAttribute(PARAM_PORT_DEF);
        if (portDef != null) {
            this.transientPortletDefinitionRegistry.removeTransientPortletDefinition(portDef.getPortletDefinitionId());
            session.removeAttribute(PARAM_PORT_DEF);
        }
        
        //Make sure there is no base ID in the session, this signifies a new app
        session.removeAttribute(PARAM_PORT_DEF_ID);
        
        //Clear selected category groups
        session.removeAttribute(PARAM_CATEGORY_GROUPS);
        
        //Clear the mode from the session
        session.removeAttribute(PARAM_WIZ_MODE);
    }
    
    private String createTargetId(PortletDefinitionId portDefId) {
    	return portletDefTargetTokenPrefix + "." + portDefId.toString();
    }
	
	
    
}
