/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.portlets.permissions;

import java.util.HashMap;
import java.util.Map;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.groups.IEntityGroup;
import org.jasig.gap.groups.IEntityNameFinder;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.gap.permissions.IPermission;

public abstract class AuthorizingAction extends DefaultAction {

    private IAuthorizationService authorizationService;
    private ICompositeGroupService compositeGroupService;
    private IEntityNameFinder entityNameFinder;
    
    public void setAuthorizationService(IAuthorizationService  authorizationService) {
	    this.authorizationService = authorizationService;
	}

    public IAuthorizationService getAuthorizationService() {
        return authorizationService;
    }
    
    public IEntityNameFinder getEntityNameFinder() {
        return entityNameFinder;
    }

    public void setEntityNameFinder(IEntityNameFinder entityNameFinder) {
        this.entityNameFinder = entityNameFinder;
    }

    public ICompositeGroupService getCompositeGroupService() {
        return compositeGroupService;
    }

    public void setCompositeGroupService(ICompositeGroupService compositeGroupService) {
        this.compositeGroupService = compositeGroupService;
    }

    protected Map<String, PermissionBean> getPermissions(IAuthorizationPrincipal ap, String owner, String target, String[] activities) {
        Map<String, PermissionBean> m = new HashMap<String, PermissionBean>();
        
        for (int i=0; i<activities.length; i++) {
	        IPermission[] permissions = ap.getPermissions(owner, activities[i], target);
            String permissionValue = deferKey;
            
            // there should be at most one permission object
            if (permissions.length > 1) {
                // TODO log warning
            } else if (permissions.length == 1) {
                permissionValue = permissions[0].getType();
            }
            
            PermissionBean permissionBean = new PermissionBean();
            permissionBean.setValue(permissionValue);
            m.put(activities[i], permissionBean);
        }
        return m;
    }
    
    protected String getPrincipalName(IAuthorizationPrincipal ap) {
        String name;
        
        IGroupMember gm = compositeGroupService.getGroupMember(ap.getKey(), ap.getType());
        
        if (gm.isGroup()) {
            name = ((IEntityGroup)gm).getName();                              
        }
        else {
            name = entityNameFinder.getName(gm.getKey());
        }
        
        return name;
    }
}
