/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.portlets.permissions;

import java.util.Map;

import org.springframework.webflow.core.collection.AttributeMap;
import org.springframework.webflow.core.collection.ParameterMap;
import org.springframework.webflow.execution.Event;
import org.springframework.webflow.execution.RequestContext;

public class CachePermissionSelectionsAction extends DefaultAction {
    
    private String principalActivitySeparator;

    @Override
    protected Event doExecute(RequestContext context) throws Exception {
        
        final ParameterMap parameterMap = context.getExternalContext().getRequestParameterMap();
        
        String[] principals = parameterMap.getArray(submittedPrincipalsKey);
        final AttributeMap attributeMap = parameterMap.asAttributeMap();
        Map<String, Object> params = attributeMap.asMap(); //TODO is this the same?
        
        //Map<String, PrincipalBean> cacheMap =
        //    (Map<String, PrincipalBean>)context.getFlowScope().get(savedPermissionSelectionsKey);
        Map<String, PrincipalBean> cacheMap =
            (Map<String, PrincipalBean>)context.getExternalContext().getSessionMap().get(savedPermissionSelectionsKey);
        
        if (cacheMap != null) {
	        for (int i=0; principals != null && i<principals.length; i++) {
	            for (String name : params.keySet()) {
	                if (name.startsWith(buildNamePrefix(principals[i]))) {
	                    String value = parameterMap.get(name);
	                    String[] activitySplit = name.split(principalActivitySeparator);
	                    if (activitySplit.length != 2) {
	                        // TODO log error
	                        return error();
	                    }
	                    String key = activitySplit[0];
	                    String activity = activitySplit[1];
	                    PrincipalBean principalBean = cacheMap.get(key);
                        PermissionBean permissionBean = principalBean.getPermission(activity);
                        permissionBean.setValue(value);
	                }
	            }
	        }
        }
        
        return success();
    }

    private String buildNamePrefix(String s) {
        return new StringBuffer(s).append(principalActivitySeparator).toString();
    }

    public void setPrincipalActivitySeparator(String principalActivitySeparator) {
        this.principalActivitySeparator = principalActivitySeparator;
    }

}
