/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.portlets.permissions;

import java.util.ArrayList;
import java.util.List;

import org.jasig.gap.groups.IGroupMember;
import org.jasig.portal.groups.security.IPermissible;
import org.springframework.webflow.core.collection.SharedAttributeMap;
import org.springframework.webflow.execution.Event;
import org.springframework.webflow.execution.RequestContext;

public class FlowSetupAction extends DefaultAction {
    
    @Override
    protected Event doExecute(RequestContext context) throws Exception {
        
        final SharedAttributeMap portletScopedSession = context.getExternalContext().getSessionMap();
        //AttributeMap flowScope = context.getFlowScope();
        String event = defaultEvent;
        
        // Attributes that the groups selection interceptor will set
        // as side effects of the group selection servant.
        // We need to recognize them and forward to the proper event.
        IGroupMember[] selectedPrincipals = (IGroupMember[])portletScopedSession.get(
            PermissionsManagerConstants.groupSelectionResultsKey);
        Boolean isGroupSelectionCancelled = portletScopedSession.getBoolean(
            PermissionsManagerConstants.groupSelectionCancelledKey);
        
        // Check for owner
        String ownerId = (String)portletScopedSession.get(ownerIdKey);
        
        if (ownerId != null && !(emptyString.equals(ownerId.trim()))) {
            IPermissible permissible = getPermissibleRegistry().getPermissible(ownerId);
            GenericEntityBean ownerBean = new GenericEntityBean();
            ownerBean.setId(ownerId);
            ownerBean.setLabel(permissible.getOwnerName());
            ownerBean.setDescription(permissible.getOwnerDescription());
            //flowScope.put(ownerBeanKey, ownerBean);
            portletScopedSession.put(ownerBeanKey, ownerBean);
        
	        // Check for target
	        String targetId = (String)portletScopedSession.get(targetIdKey);
	        
	        if (targetId != null && !(emptyString.equals(targetId.trim()))) {
                GenericEntityBean targetBean = new GenericEntityBean();
                targetBean.setId(targetId);
                targetBean.setLabel(permissible.getTargetName(targetId));
                targetBean.setDescription(permissible.getTargetDescription(targetId));
                portletScopedSession.put(targetBeanKey, targetBean);
                
                String[] activityTokens = (String[])portletScopedSession.get(activityTokensKey);
                if (activityTokens != null) {
                    List<ActivityBean> activities = new ArrayList<ActivityBean>();
                    
                    for (int x = 0; x < activityTokens.length; x++) {
                        ActivityBean activityBean = new ActivityBean();
                        activityBean.setId(activityTokens[x]);
                        activityBean.setIndex(x);
                        activityBean.setLabel(permissible.getActivityName(activityTokens[x]));
                        activityBean.setDescription(permissible.getActivityDescription(activityTokens[x]));
                        activities.add(activityBean);
                    }
                    
                    portletScopedSession.put(activityTokensKey, activityTokens);
                    portletScopedSession.put(activityListKey, activities);
                }
                
                if (selectedPrincipals != null) {
                    event = updatePrincipalsEvent;
                } else {
		            event = assignPermissionsEvent;
                }
	        } else {
	            event = noTargetEvent;
            }
        }
        
        return result(event);
    }

}
