package org.jasig.portal.portlets.permissions;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.portal.portlet.servant.IPortletServant;
import org.springframework.core.Ordered;
import org.springframework.web.portlet.HandlerInterceptor;
import org.springframework.web.portlet.ModelAndView;

public class GroupSelectionInterceptor
implements PermissionsManagerConstants, HandlerInterceptor, Ordered {
    
    private ICompositeGroupService compositeGroupService;
    private Set<Class> entityTypeFilter = null;
    private int order;
    private IPortletServant groupsManagerServant;

    public void afterActionCompletion(ActionRequest request, ActionResponse response,
        Object arg2, Exception arg3) throws Exception {
        
        Boolean actionProcessed = (Boolean)
	        request.getAttribute(PermissionsManagerConstants.processedActionKey);
        
        if (isGroupSelectionActive(request, true)) {
            if (!actionProcessed) {
                groupsManagerServant.processAction(null, request, response);
            }
        }
    }

    public void afterRenderCompletion(RenderRequest request, RenderResponse response,
        Object arg2, Exception arg3) throws Exception {
    }

    public void postHandleRender(RenderRequest request, RenderResponse response,
        Object arg2, ModelAndView arg3) throws Exception {
    }
    
    private void fetchAndStoreResults(PortletRequest request, IPortletServant servant) {
        IGroupMember[] selections = (IGroupMember[])servant.getResult(request);
        if (selections != null) {
            request.getPortletSession().setAttribute(PermissionsManagerConstants.groupSelectionResultsKey, selections);
        } 
    }

    public boolean preHandleAction(ActionRequest request, ActionResponse response,
        Object arg2) throws Exception {

        request.setAttribute(PermissionsManagerConstants.processedActionKey, Boolean.FALSE);
        
        if (isGroupSelectionActive(request, false)) {
            groupsManagerServant.processAction(null, request, response);
            boolean isComplete = groupsManagerServant.isComplete(request);
            boolean isCancelled = groupsManagerServant.isCancelled(request);
	        request.setAttribute(PermissionsManagerConstants.processedActionKey, Boolean.TRUE);
            if (isCancelled) {
                request.getPortletSession().setAttribute(PermissionsManagerConstants.groupSelectionCancelledKey, Boolean.TRUE);
                cleanUp(request);
                return true;
            } else if (isComplete) {
                fetchAndStoreResults(request, groupsManagerServant);
                cleanUp(request);
                return true;
            }
            return false;
        }
        return true;
    }
    
    private void setServantAttributes(PortletRequest request) {
        groupsManagerServant.setAttribute(request, "entityTypeFilter", entityTypeFilter);
        groupsManagerServant.setAttribute(request,"allowEntitySelect",true);
        groupsManagerServant.setAttribute(request,"allowFinish",true);
        groupsManagerServant.setAttribute(request, "callerCancelCommandName", "_eventId");
        groupsManagerServant.setAttribute(request, "callerCancelCommandValue", "cancelSelection");
        groupsManagerServant.setAttribute(request, "callerFinishedCommandName", "_eventId");
        groupsManagerServant.setAttribute(request, "callerFinishedCommandValue", "finishSelection");
        groupsManagerServant.setAttribute(request, "callerSessionName", "_flowExecutionKey");
        groupsManagerServant.setAttribute(request, "callerSessionValue", request.getPortletSession().getAttribute("flowExecutionKey"));
        List<PrincipalBean> previouslySelectedPrincipals = (List<PrincipalBean>)request.getPortletSession().getAttribute(principalsKey);
        if (previouslySelectedPrincipals != null && previouslySelectedPrincipals.size() > 0) {
	        IGroupMember[] members = new IGroupMember[previouslySelectedPrincipals.size()];
            for (int i=0; i<members.length; i++) {
                PrincipalBean principalBean = previouslySelectedPrincipals.get(i);
                members[i] = compositeGroupService.getGroupMember(principalBean.getId(), principalBean.getType());
            }
            groupsManagerServant.setAttribute(request, "groupMembers", members);
        }
    }
    
    private void cleanUp(PortletRequest request) {
        PortletSession session = request.getPortletSession();
        session.removeAttribute(PermissionsManagerConstants.groupSelectionServantKey);
        session.removeAttribute(PermissionsManagerConstants.groupSelectionActiveKey);
        session.removeAttribute(PermissionsManagerConstants.invokeGroupsSelectionKey);
        groupsManagerServant.reset(request);
    }
    
    public boolean preHandleRender(RenderRequest request, RenderResponse response,
        Object arg2) throws Exception {
        
        if (isGroupSelectionActive(request, false)) {
            groupsManagerServant.render(request, response);
            return false;
        }
        return true;
    }
    
    private boolean getGroupSelectionActiveAttribute(PortletSession session) {
        Boolean groupSelectionActive = (Boolean)session.getAttribute(
            PermissionsManagerConstants.groupSelectionActiveKey);
        if (groupSelectionActive == null) {
            groupSelectionActive = Boolean.FALSE;
        }
        return groupSelectionActive;
    }
    
    private boolean isGroupSelectionActive(PortletRequest request, boolean check) {
        PortletSession session = request.getPortletSession();
        boolean groupSelectionActive = getGroupSelectionActiveAttribute(session);
        if (!check) return groupSelectionActive;
        
        if (!groupSelectionActive) {
            checkIfGroupSelectionRequested(request);
            groupSelectionActive = getGroupSelectionActiveAttribute(session);
        }
        
        return groupSelectionActive;
    }
    
    private void checkIfGroupSelectionRequested(PortletRequest request) {
        
        PortletSession session = request.getPortletSession();
        
        Boolean invokeGroupSelection = (Boolean)session.getAttribute(
            PermissionsManagerConstants.invokeGroupsSelectionKey);
        if (invokeGroupSelection == null) {
            invokeGroupSelection = Boolean.FALSE;
        }
        
        if (invokeGroupSelection) {
            session.setAttribute(PermissionsManagerConstants.groupSelectionActiveKey, Boolean.TRUE);
            session.removeAttribute(PermissionsManagerConstants.invokeGroupsSelectionKey);
            setServantAttributes(request);
        }
    }

    public void setOrder(int order) {
        this.order = order;
    }

    public int getOrder() {
        return order;
    }
    
    public void setEntityTypeFilter(Set genericCategoriesEntityTypeFilter) throws ClassNotFoundException {
        this.entityTypeFilter = new HashSet<Class>();
        for(Object o : genericCategoriesEntityTypeFilter) {
            if (o instanceof String) {
                this.entityTypeFilter.add(Class.forName((String)o));
            } else if (o instanceof Class) {
                this.entityTypeFilter.add((Class)o);
            } else {
                throw new IllegalArgumentException("entity type filter elements must be either String or Class: " + o.getClass().getName());
            }
        }
    }
    
    public void setCompositeGroupService(
        ICompositeGroupService compositeGroupService) {
        this.compositeGroupService = compositeGroupService;
    }

    public IPortletServant getGroupsManagerServant() {
        return groupsManagerServant;
    }

    public void setGroupsManagerServant(IPortletServant groupsManagerServant) {
        this.groupsManagerServant = groupsManagerServant;
    }

}
