/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.portlets.permissions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.portal.groups.security.IPermissible;
import org.springframework.webflow.core.collection.MutableAttributeMap;
import org.springframework.webflow.core.collection.ParameterMap;
import org.springframework.webflow.core.collection.SharedAttributeMap;
import org.springframework.webflow.execution.Event;
import org.springframework.webflow.execution.RequestContext;

public class ListPermissibleActivitiesAction extends AuthorizingAction {
    
    private Comparator<PrincipalBean> principalBeanComparator;
    private String typeKeySeparator;

    @Override
    protected Event doExecute(RequestContext context) throws Exception {
        
        ParameterMap parameterMap = 
            context.getExternalContext().getRequestParameterMap();
    
        //GenericEntityBean ownerBean =  (GenericEntityBean)context.getFlowScope().get(ownerBeanKey);
        GenericEntityBean ownerBean =  (GenericEntityBean)context.getExternalContext().getSessionMap().get(ownerBeanKey);
        IPermissible permissible = getPermissibleRegistry().getPermissible(ownerBean.getId());
        
        if (permissible == null) {
            // TODO -- How to commmunicate this properly?
            // Logging? PermissibleRegistry will have unRegistered the ownerId,
            // so if we return to owners list, they will not have that option anymore...
            return error();
        }
        
        //GenericEntityBean targetBean = (GenericEntityBean)context.getFlowScope().get(targetBeanKey);
        GenericEntityBean targetBean = (GenericEntityBean)context.getExternalContext().getSessionMap().get(targetBeanKey);
        if (targetBean == null) {
	        String targetId = parameterMap.get(targetIdKey);
	        targetBean = new GenericEntityBean();
	        targetBean.setId(targetId);
	        targetBean.setLabel(permissible.getTargetName(targetId));
	        targetBean.setDescription(permissible.getTargetDescription(targetId));
	        //context.getFlowScope().put(targetBeanKey, targetBean);
	        context.getExternalContext().getSessionMap().put(targetBeanKey, targetBean);
        }
        
		// make sure the owner and target ids are exposed outside so the flow
        // setup action can make use of them, if the flow gets reset by group selection
        context.getExternalContext().getSessionMap().put(ownerIdKey, ownerBean.getId());
        context.getExternalContext().getSessionMap().put(targetIdKey, targetBean.getId());
        
        final MutableAttributeMap requestAttributes = context.getRequestScope();
        requestAttributes.put(activitiesKey, getActivities(context, permissible, targetBean));
        requestAttributes.put(ownerKey, ownerBean);
        requestAttributes.put(targetKey, targetBean);
        requestAttributes.put(principalsKey,
            getSortedPrincipals(context, permissible, targetBean.getId(),
                getActivityTokens(context, permissible, targetBean)));
        
        return success();
    }
    
    private List<ActivityBean> getActivities(RequestContext context, IPermissible permissible, GenericEntityBean targetBean) {
        //List<ActivityBean> activities = (List<ActivityBean>)context.getFlowScope().get(activityListKey);
        List<ActivityBean> activities = (List<ActivityBean>)context.getExternalContext().getSessionMap().get(activityListKey);
        if (activities == null) {
	        activities = new ArrayList<ActivityBean>();
	        String[] activityTokens = getActivityTokens(context, permissible, targetBean);
	        
	        for (int x = 0; x < activityTokens.length; x++) {
	            ActivityBean activityBean = new ActivityBean();
	            activityBean.setId(activityTokens[x]);
	            activityBean.setIndex(x);
	            activityBean.setLabel(permissible.getActivityName(activityTokens[x]));
	            activityBean.setDescription(permissible.getActivityDescription(activityTokens[x]));
	            activities.add(activityBean);
	        }
            //context.getFlowScope().put(activityListKey, activities);
            context.getExternalContext().getSessionMap().put(activityListKey, activities);
        }
        return activities;
    }
    
    private String[] getActivityTokens(RequestContext context, IPermissible permissible, GenericEntityBean targetBean) {
        //String[] activityTokens = (String[])context.getFlowScope().get(activityTokensKey);
        String[] activityTokens = (String[])context.getExternalContext().getSessionMap().get(activityTokensKey);
        if (activityTokens == null) {
            activityTokens = permissible.getActivityTokens(targetBean.getId());
            //context.getFlowScope().put(activityTokensKey, activityTokens);
            context.getExternalContext().getSessionMap().put(activityTokensKey, activityTokens);
        }
        return activityTokens;
    }
    
    private List<PrincipalBean> getSortedPrincipals(RequestContext context, IPermissible permissible, String targetToken, String[] activityTokens) {
        List<PrincipalBean> principals = getPrincipals(context, permissible, targetToken, activityTokens);
        if (principalBeanComparator != null) {
            Collections.sort(principals, principalBeanComparator);
        }
        return principals;
    }
    
    private List<PrincipalBean> getPrincipals(RequestContext context, IPermissible permissible, String targetToken, String[] activityTokens) {
        
        final SharedAttributeMap portletScopedSession = context.getExternalContext().getSessionMap();
        //AttributeMap flowScope = context.getFlowScope();
        List<PrincipalBean> principals = (List<PrincipalBean>)portletScopedSession.get(principalsKey);
        if (principals != null) {
            return principals;
        }
        
        // create a map so CachePermissionSelectionsAction can easily look them up
        //Map<String, PrincipalBean> cacheMap = (Map<String, PrincipalBean>)flowScope.get(savedPermissionSelectionsKey);
        Map<String, PrincipalBean> cacheMap = (Map<String, PrincipalBean>)portletScopedSession.get(savedPermissionSelectionsKey);
        if (cacheMap == null) {
	        cacheMap = new HashMap<String, PrincipalBean>();
            //flowScope.put(savedPermissionSelectionsKey, cacheMap);
            portletScopedSession.put(savedPermissionSelectionsKey, cacheMap);
        }
        
        principals = new LinkedList<PrincipalBean>();
        Set<IAuthorizationPrincipal> authorizationPrincipals =
            new HashSet<IAuthorizationPrincipal>();
        
        for (int i=0; i<activityTokens.length; i++) {
            IAuthorizationPrincipal[] tmpArr = getAuthorizationService().getAuthorizedPrincipals(permissible.getOwnerToken(), activityTokens[i], targetToken);
            for (int j=0; j<tmpArr.length; j++) {
                authorizationPrincipals.add(tmpArr[j]);
            }
        }
        
        for (IAuthorizationPrincipal ap : authorizationPrincipals) {
            PrincipalBean principalBean = new PrincipalBean();
            principalBean.setId(ap.getKey());
            principalBean.setLabel(getPrincipalName(ap));
            principalBean.setType(ap.getType());
            principalBean.setPermissions(getPermissions(ap, permissible.getOwnerToken(), targetToken, activityTokens));
            if (!principals.contains(principalBean)) {
	            principals.add(principalBean);
                cacheMap.put(constructPrincipalKey(principalBean), principalBean);
            }
        }
        
        portletScopedSession.put(principalsKey, principals);
        return principals;
    }
    
    private String constructPrincipalKey(PrincipalBean principalBean) {
        StringBuffer sb = new StringBuffer(principalBean.getType().getName());
        sb.append(typeKeySeparator);
        sb.append(principalBean.getId());
        return sb.toString();
    }

    public void setPrincipalBeanComparator(
        Comparator<PrincipalBean> principalBeanComparator) {
        this.principalBeanComparator = principalBeanComparator;
    }

    public void setTypeKeySeparator(String typeKeySeparator) {
        this.typeKeySeparator = typeKeySeparator;
    }
    
}
