/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.portlets.permissions;

import java.util.ArrayList;
import java.util.List;

import org.jasig.portal.groups.security.IPermissible;
import org.springframework.webflow.core.collection.MutableAttributeMap;
import org.springframework.webflow.core.collection.ParameterMap;
import org.springframework.webflow.execution.Event;
import org.springframework.webflow.execution.RequestContext;

public class ListPermissibleTargetsAction extends DefaultAction {

    @Override
    protected Event doExecute(RequestContext context) throws Exception {
        
        ParameterMap parameterMap = 
                context.getExternalContext().getRequestParameterMap();
        
        String ownerId = parameterMap.get(ownerIdKey);
        IPermissible permissible = getPermissibleRegistry().getPermissible(ownerId);        
        GenericEntityBean ownerBean = new GenericEntityBean();
        ownerBean.setId(ownerId);
        ownerBean.setLabel(permissible.getOwnerName());
        ownerBean.setDescription(permissible.getOwnerDescription());
        //context.getFlowScope().put(ownerBeanKey, ownerBean);
        context.getExternalContext().getSessionMap().put(ownerBeanKey, ownerBean);
        
        if (permissible == null) {
            // TODO -- How to commmunicate this properly?
            // Logging? PermissibleRegistry will have unRegistered the ownerId,
            // so if we return to owners list, they will not have that option anymore...
            return error();
        }
        
        List<GenericEntityBean> targets = new ArrayList<GenericEntityBean>();
        String[] targetTokens = permissible.getTargetTokens();
        
        for (int x = 0; x < targetTokens.length; x++) {
            GenericEntityBean targetBean = new GenericEntityBean();
            targetBean.setId(targetTokens[x]);
            targetBean.setLabel(permissible.getTargetName(targetTokens[x]));
            targetBean.setDescription(permissible.getTargetDescription(targetTokens[x]));
            targets.add(targetBean);
        }
        
        // Default the target to owner if no optional targets exist
        if (targetTokens.length == 0) {
            GenericEntityBean targetBean = new GenericEntityBean();
            targetBean.setId(permissible.getOwnerToken());
            targetBean.setLabel(permissible.getOwnerName());
            targetBean.setDescription("Default target"); //TODO I18N This
            targets.add(targetBean);
        }
      
        final MutableAttributeMap requestAttributes = context.getRequestScope();
        requestAttributes.put(targetsKey, targets);
        requestAttributes.put(ownerKey, ownerBean);
        return success();
    }

}
