/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.portlets.permissions;

import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.groups.IEntityGroup;
import org.jasig.gap.groups.IEntityNameFinder;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.portal.security.UserId;
import org.springframework.webflow.core.collection.MutableAttributeMap;
import org.springframework.webflow.execution.Event;
import org.springframework.webflow.execution.RequestContext;

public class ListPrincipalsAction extends DefaultAction {

    private ICompositeGroupService compositeGroupService;
    private IEntityNameFinder entityNameFinder;
    private Comparator<PrincipalBean> principalBeanComparator;
    
    @Override
    protected Event doExecute(RequestContext context) throws Exception {
        
        List<PrincipalBean> availablePrincipals = new LinkedList<PrincipalBean>();
        
        List<PrincipalBean> previouslySelectedPrincipals = (List<PrincipalBean>)context.getExternalContext().getSessionMap().get(principalsKey);
        //List<PrincipalBean> previouslySelectedPrincipals = (List<PrincipalBean>)context.getFlowScope().get(principalsKey);
        
        IEntityGroup rootGroup = compositeGroupService.getDistinguishedGroup(
                UserId.class.getName()); 
        
        IGroupMember groupMember = null;
        PrincipalBean principalBean = null;
        String name = null;
        String description = null;
        Class type = null;
        
        // Add root group itself to the list
        principalBean = new PrincipalBean();
        principalBean.setId(rootGroup.getKey());
        principalBean.setType(rootGroup.getType());
        principalBean.setLabel(rootGroup.getName());
        principalBean.setDescription(rootGroup.getDescription());
        if (!previouslySelectedPrincipals.contains(principalBean)) {
	        availablePrincipals.add(principalBean);
        }
        
        // Add all members
        Iterator grpIterator = rootGroup.getAllMembers();
        while(grpIterator.hasNext()) {
	        principalBean = new PrincipalBean();
            groupMember = (IGroupMember)grpIterator.next();
            if (groupMember.isGroup()) {
                name = ((IEntityGroup)groupMember).getName();                              
                description = ((IEntityGroup)groupMember).getDescription();                              
                type = groupMember.getType();
            }
            else {
                name = entityNameFinder.getName(groupMember.getKey());
                description = emptyString;
                type = groupMember.getEntityType();
            }
            // Required data: label, description, id, type, isSelected
	        principalBean.setId(groupMember.getKey());
	        principalBean.setType(type);
	        principalBean.setLabel(name);
	        principalBean.setDescription(description);
            if (!previouslySelectedPrincipals.contains(principalBean)) {
                availablePrincipals.add(principalBean);
            }
        }
        
        if (principalBeanComparator != null) {
	        Collections.sort(availablePrincipals, principalBeanComparator);
        }
        final MutableAttributeMap requestAttributes = context.getRequestScope();
        requestAttributes.put(availablePrincipalsKey, availablePrincipals);
        return success();
    }

    /**
     * Sets the <code>ICompositeGroupService</code> which is used to obtain 
     * groups for Principal selection.
     * 
     * @param groupService
     */
    public void setCompositeGroupService(ICompositeGroupService compositeGroupService) {
        this.compositeGroupService = compositeGroupService;
    }

    public void setEntityNameFinder(IEntityNameFinder entityNameFinder) {
        this.entityNameFinder = entityNameFinder;
    }
    
    public static class PrincipalBeanComparator implements Comparator<PrincipalBean> {
		//  Sort it alphabetically based on group/user names
        public int compare(PrincipalBean p1, PrincipalBean p2) {
			// TODO Check for nulls
            return ((String)p1.getLabel()).compareToIgnoreCase((String)p2.getLabel());
        }
        
    }

    public void setPrincipalBeanComparator(Comparator<PrincipalBean> principalBeanComparator) {
        this.principalBeanComparator = principalBeanComparator;
    }

}
