package org.jasig.portal.portlets.permissions;

import java.io.IOException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.ResourceBundle;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletConfig;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.portal.core.exception.PortalRuntimeException;
import org.jasig.portal.portlet.registry.transitory.ITransientPortletWindowRegistry;
import org.jasig.portal.portlet.rendering.IPortletRenderContent;
import org.jasig.portal.portlet.servant.IPortletServant;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.springframework.web.portlet.ModelAndView;
import org.springframework.web.portlet.mvc.PortletWrappingController;

public class PermissionsManagerServantController extends
    PortletWrappingController implements IPortletServant {
    
    
    private boolean afterPropertiesInvoked = false;
    private ITransientPortletWindowRegistry transientPortletWindowRegistry;
    private String permissionsManagerFunctionName;
    private Properties initParameters = new Properties();
    private String portletName;
    
    public PermissionsManagerServantController() {
    }
    
    public Object getAttribute(PortletRequest request, String key) {
        return request.getPortletSession().getAttribute(key);
    }

    public void setAttribute(PortletRequest request, String key, Object value) {
        request.getPortletSession().setAttribute(key, value);
    }

    public String getId() {
        return null;
    }

    public IPortletRenderContent getRenderContent(RenderRequest request,
        RenderResponse response) {
        return null;
    }

    public Object getResult(PortletRequest request) {
        return null;
    }

    public boolean isCancelled(PortletRequest request) {
        Boolean b = (Boolean)getAttribute(request, PermissionsManagerConstants.servantIsCancelledKey);
        if (b == null) {
            b = Boolean.FALSE;
        }
        return b;
    }

    public boolean isComplete(PortletRequest request) {
        Boolean b = (Boolean)getAttribute(request, PermissionsManagerConstants.servantIsCompleteKey);
        if (b == null) {
            b = Boolean.FALSE;
        }
        return b;
    }

    public void processAction(Map actionRequestParameterMap,
        ActionRequest request, ActionResponse response)
        throws PortletException, IOException {
        
        try {
	        super.handleActionRequest(request, response);
        } catch (Exception e) {
            throw new PortletException(e);
        }

    }

    public void render(RenderRequest request, RenderResponse response)
        throws PortletException, IOException {

        try {
	        super.handleRenderRequest(request, response);
        } catch (Exception e) {
            throw new PortletException(e);
        }
    }

    public void reset(PortletRequest request) {
        request.getPortletSession().removeAttribute(PermissionsManagerConstants.servantIsCancelledKey);
        request.getPortletSession().removeAttribute(PermissionsManagerConstants.servantIsCompleteKey);
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // intercept and invoke after we're given a PortletConfig object
    }
    
    private void initializePortletConfig(PortletRequest req) {
        IPortletWindow window = transientPortletWindowRegistry.createTransientPortletWindowControl(permissionsManagerFunctionName);
        if (window == null) {
            throw new PortalRuntimeException("Could not find the permissions manager portlet definition with the functional name="+permissionsManagerFunctionName);
        }
        
        //TODO fix this once the pluto resource bundle is available
//        Locale locale = window.getPortletInfo().getLocale();
//        ResourceBundle resourceBundle = window.getPortletInfo().getResourceBundle(locale);
        Map<Locale, ResourceBundle> resourceBundles = new HashMap<Locale, ResourceBundle>(1);
//        resourceBundles.put(locale, resourceBundle);
        WrappedPortletConfig config = new WrappedPortletConfig(portletName, req.getPortletSession().getPortletContext(), resourceBundles, initParameters);
        super.setPortletConfig(config);
        super.setPortletContext(req.getPortletSession().getPortletContext());
    }

    @Override
    protected void handleActionRequestInternal(ActionRequest req, ActionResponse res) throws Exception {
        if (!afterPropertiesInvoked) {
            initializePortletConfig(req);
            super.afterPropertiesSet();
            afterPropertiesInvoked = true;
        }
        super.handleActionRequestInternal(req, res);
    }

    @Override
    protected ModelAndView handleRenderRequestInternal(RenderRequest req, RenderResponse res) throws Exception {
        if (!afterPropertiesInvoked) {
            initializePortletConfig(req);
            super.afterPropertiesSet();
            afterPropertiesInvoked = true;
        }
        return super.handleRenderRequestInternal(req, res);
    }

    private static class WrappedPortletConfig implements PortletConfig {
        
        private String portletName;
        private PortletContext portletContext;
        private Map<Locale, ResourceBundle> resourceBundles;
        private Properties properties;
        
        public WrappedPortletConfig(String portletName, PortletContext portletContext,
            Map<Locale, ResourceBundle> resourceBundles, Properties properties) {
            this.portletName = portletName;
            this.portletContext = portletContext;
            this.resourceBundles = resourceBundles;
            this.properties = properties;
        }
        
        public String getInitParameter(String name) {
            return properties.getProperty(name);
        }

        public Enumeration getInitParameterNames() {
            return properties.keys();
        }

        public PortletContext getPortletContext() {
            return portletContext;
        }

        public String getPortletName() {
            return portletName;
        }

        public ResourceBundle getResourceBundle(Locale locale) {
            return resourceBundles.get(locale);
        }
    }

    public ITransientPortletWindowRegistry getTransientPortletWindowRegistry() {
        return transientPortletWindowRegistry;
    }

    public void setTransientPortletWindowRegistry(
        ITransientPortletWindowRegistry transientPortletWindowRegistry) {
        this.transientPortletWindowRegistry = transientPortletWindowRegistry;
    }

    public String getPermissionsManagerFunctionName() {
        return permissionsManagerFunctionName;
    }

    public void setPermissionsManagerFunctionName(
        String permissionsManagerFunctionName) {
        this.permissionsManagerFunctionName = permissionsManagerFunctionName;
    }

    public Properties getInitParameters() {
        return initParameters;
    }

    public void setInitParameters(Properties initParameters) {
        this.initParameters = initParameters;
    }

    public String getPortletName() {
        return portletName;
    }

    public void setPortletName(String portletName) {
        this.portletName = portletName;
    }
}
