/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.portlets.permissions;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.portal.groups.security.IPermissible;
import org.springframework.webflow.core.collection.SharedAttributeMap;
import org.springframework.webflow.execution.Event;
import org.springframework.webflow.execution.RequestContext;

public class UpdatePrincipalsAction extends AbstractPrincipalsAction {
    
    private Log log = LogFactory.getLog(UpdatePrincipalsAction.class);
    
    @Override
    protected Event doExecute(RequestContext context) throws Exception {
        
        final SharedAttributeMap portletScopeMap = context.getExternalContext().getSessionMap();
        IGroupMember[] selectedPrincipals = (IGroupMember[])portletScopeMap.get(
                PermissionsManagerConstants.groupSelectionResultsKey);
        
        //AttributeMap flowScope = context.getFlowScope();
        
        //GenericEntityBean ownerBean =  (GenericEntityBean)flowScope.get(ownerBeanKey);
        //GenericEntityBean targetBean = (GenericEntityBean)flowScope.get(targetBeanKey);
        GenericEntityBean ownerBean =  (GenericEntityBean)portletScopeMap.get(ownerBeanKey);
        GenericEntityBean targetBean = (GenericEntityBean)portletScopeMap.get(targetBeanKey);
        IPermissible permissible = getPermissibleRegistry().getPermissible(ownerBean.getId());
        String[] activityTokens = permissible.getActivityTokens(targetBean.getId());
        
        IAuthorizationService authorizationService = getAuthorizationService();
        
		// create a map so CachePermissionSelectionsAction can easily look them up
        //Map<String, PrincipalBean> cacheMap = (Map<String, PrincipalBean>)flowScope.get(savedPermissionSelectionsKey);
        Map<String, PrincipalBean> cacheMap = (Map<String, PrincipalBean>)portletScopeMap.get(savedPermissionSelectionsKey);
        if (cacheMap == null) {
            cacheMap = new HashMap<String, PrincipalBean>();
            //flowScope.put(savedPermissionSelectionsKey, cacheMap);
            portletScopeMap.put(savedPermissionSelectionsKey, cacheMap);
        }
        
        List<PrincipalBean> principals = getCachedPrincipals(context, principalsKey);
        
        for (int i=0; selectedPrincipals != null && i<selectedPrincipals.length; i++) {
            IAuthorizationPrincipal ap = authorizationService.newPrincipal(selectedPrincipals[i].getKey(), selectedPrincipals[i].getType());
            PrincipalBean principalBean = new PrincipalBean();
            principalBean.setId(selectedPrincipals[i].getKey());
            principalBean.setLabel(getPrincipalName(ap));
            principalBean.setType(ap.getType());
            principalBean.setPermissions(getPermissions(ap, permissible.getOwnerToken(), targetBean.getId(), activityTokens));
            if (!principals.contains(principalBean)) {
	            principals.add(principalBean);
	            cacheMap.put(constructPrincipalKey(principalBean), principalBean);
            }
	    }
        
        return success();
    }
    
    private String constructPrincipalKey(PrincipalBean principalBean) {
        StringBuffer sb = new StringBuffer(principalBean.getType().getName());
        sb.append(getTypeKeySeparator());
        sb.append(principalBean.getId());
        return sb.toString();
    }

}
