/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.portlets.subscription;

import org.jasig.gap.groups.IGroupMember;
import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.portal.portlets.manager.web.pages.portlet.PortletDefinitionWizard;
import org.jasig.portal.servants.groups.permissions.GroupsManagerDefaultPermissions;


/**
 * GroupsManagerDefaultPermissions answers if the Authorization Principal is able to
 * perform specific actions on the target Group Member.
 * @author Don Fracapane
 * @version $Revision: 11911 $
 */
public class GroupsManagerPortletSelectionPermissions extends GroupsManagerDefaultPermissions {
    
   private String owner;
   private String activity;
   private String entityTargetPrefix;
   
   public GroupsManagerPortletSelectionPermissions() {
       this.owner = PortletDefinitionWizard.DEFAULT_FRAMEWORK_OWNER_TOKEN;
       this.activity = PortletDefinitionWizard.DEFAULT_SUBSCRIPTION_ACTIVITY_TOKEN;
       this.entityTargetPrefix = PortletDefinitionWizard.DEFAULT_PORTLET_DEF_TARGET_TOKEN_PREFIX;
   }

   /**
    * Answers if principal can select the target group member.
    * @param ap AuthorizationPrincipal
    * @param gm IGroupMember
    * @return boolean
    */
   public boolean canSelect (IAuthorizationPrincipal ap, IGroupMember gm) {
       return  (gm.isEntity() && isAuthorized(ap, "SELECT", gm) && isAuthorized(ap, activity, owner, entityTargetPrefix+gm.getKey()));
   }

}
