/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.subscription;

import java.io.IOException;
import java.util.HashSet;
import java.util.Set;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.layout.uportal2.ILegacyLayoutSourceCommands;
import org.jasig.portal.layout.uportal2.INodeAdditionManager;
import org.jasig.portal.portlet.IActionResponse;
import org.jasig.portal.portlet.definition.IPortletDefinition;
import org.jasig.portal.portlet.definition.PortletDefinitionId;
import org.jasig.portal.portlet.entity.IPortletApplicationEntity;
import org.jasig.portal.portlet.entity.IPortletEntity;
import org.jasig.portal.portlet.factory.IPortletApplicationEntityRegistryFactory;
import org.jasig.portal.portlet.factory.IPortletEntityRegistryFactory;
import org.jasig.portal.portlet.factory.IPortletWindowRegistryFactory;
import org.jasig.portal.portlet.registry.general.IPortletApplicationEntityRegistry;
import org.jasig.portal.portlet.registry.general.IPortletDefinitionRegistry;
import org.jasig.portal.portlet.registry.general.IPortletEntityRegistry;
import org.jasig.portal.portlet.registry.general.IPortletWindowRegistry;
import org.jasig.portal.portlet.servant.IPortletServant;
import org.jasig.portal.portlet.transfer.impl.PortletApplicationEntityTransferImpl;
import org.jasig.portal.portlet.transfer.impl.PortletEntityTransferImpl;
import org.jasig.portal.portlet.transfer.impl.PortletWindowTransferImpl;
import org.jasig.portal.portlet.window.IPortletWindow;
import org.jasig.portal.portlets.utils.BaseActionPage;
import org.jasig.portal.security.UserId;
import org.jasig.portal.servants.groups.GroupManagerMode;
import org.jasig.portal.servants.groups.IEntityActionUrlParts;
import org.jasig.portal.servants.groups.IGroupsManagerPermissions;
import org.jasig.portal.servants.groups.SelectionModel;


/**
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11955 $
 */
public class SubscribePortletDefinitions extends BaseActionPage {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(SubscribePortletDefinitions.class);
    
    
    private IPortletDefinitionRegistry portletDefinitionRegistry;
    private IPortletApplicationEntityRegistryFactory portletApplicationEntityRegistryFactory;
    private IPortletEntityRegistryFactory portletEntityRegistryFactory;
    private IPortletWindowRegistryFactory portletWindowRegistryFactory;
    private INodeAdditionManager nodeAdditionManager;
    private IUserController userController;
    private IPortletServant groupsManagerServant;
    private Set<Class> categoriesEntityTypeFilter = null;
    private IEntityActionUrlParts entityActionUrlParts;
    
    private IGroupsManagerPermissions groupsManagerPermissions;
    
    /**
     * @param nodeAdditionManager The nodeAdditionManager to set.
     */
    public void setNodeAdditionManager(INodeAdditionManager nodeAdditionManager) {
        this.nodeAdditionManager = nodeAdditionManager;
    }
    
    /**
     * @param portletDefinitionRegistry The portletDefinitionRegistry to set.
     */
    public void setPortletDefinitionRegistry(IPortletDefinitionRegistry portletDefinitionRegistry) {
        this.portletDefinitionRegistry = portletDefinitionRegistry;
    }
    
    /**
     * @param portletApplicationEntityRegistryFactory The portletApplicationEntityRegistryFactory to set.
     */
    public void setPortletApplicationEntityRegistryFactory(
                                                           IPortletApplicationEntityRegistryFactory portletApplicationEntityRegistryFactory) {
        this.portletApplicationEntityRegistryFactory = portletApplicationEntityRegistryFactory;
    }

    /**
     * @param portletEntityRegistryFactory The portletEntityRegistryFactory to set.
     */
    public void setPortletEntityRegistryFactory(IPortletEntityRegistryFactory portletEntityRegistryFactory) {
        this.portletEntityRegistryFactory = portletEntityRegistryFactory;
    }
    
    /**
     * @param portletWindowRegistryFactory The portletWindowRegistryFactory to set.
     */
    public void setPortletWindowRegistryFactory(IPortletWindowRegistryFactory portletWindowRegistryFactory) {
        this.portletWindowRegistryFactory = portletWindowRegistryFactory;
    }
    
    /**
     * @param userController The userController to set.
     */
    public void setUserController(IUserController userController) {
        this.userController = userController;
    }
    
    public void renderPage(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        setServantAttributes(request);
        groupsManagerServant.render(request, response);
    }
    
    private void setServantAttributes(PortletRequest request) {
        groupsManagerServant.setAttribute(request, "displayMessage", "Select Portlet for Subscription");
        groupsManagerServant.setAttribute(request, "entityTypeFilter", categoriesEntityTypeFilter);
        groupsManagerServant.setAttribute(request, "selectionModel", SelectionModel.Single);
        groupsManagerServant.setAttribute(request, "overridingPermissions", groupsManagerPermissions);
        groupsManagerServant.setAttribute(request, "allowFinish", true);
        groupsManagerServant.setAttribute(request, "groupManagerMode", GroupManagerMode.entityAction);
        groupsManagerServant.setAttribute(request, "entityActionUrlParts", entityActionUrlParts);
    }

    public void processResults(ActionRequest request, ActionResponse response) throws IOException, PortletException {
        setServantAttributes(request);
        groupsManagerServant.processAction(null, request, response);
    }
    
    public void subscribe(ActionRequest request, ActionResponse response) throws IOException, PortletException {
        final String portDefIdStr = StringUtils.trimToNull(request.getParameter("PORT_DEF_ID"));
        //TODO check for null string
        
        final PortletDefinitionId portDefId = new PortletDefinitionId(portDefIdStr);
        final IPortletDefinition portDef = this.portletDefinitionRegistry.getPortletDefinition(portDefId);
        //TODO check null portDef
        
        final UserId userId = this.userController.getUserId();
        //TODO check null userId
        
        final IPortletApplicationEntityRegistry portletApplicationEntityRegistry = this.portletApplicationEntityRegistryFactory.getPortletApplicationEntityRegistry(userId);
        
        //Get the PortletApplicationEntity to base the window on
        final Set portAppEnts = portletApplicationEntityRegistry.getPortletApplicationEntities(portDef.getPortletApplicationDefinitionId());
        final IPortletApplicationEntity portAppEnt;
        if (portAppEnts==null || portAppEnts.isEmpty()) {
            //Create the PortletApplicationEntity if one does not exist for this app and user
            PortletApplicationEntityTransferImpl transfer=new PortletApplicationEntityTransferImpl();
            transfer.setPortletApplicationDefinitionId(portDef.getPortletApplicationDefinitionId());
            transfer.setUserId(userId);
            portAppEnt=portletApplicationEntityRegistry.createPortletApplicationEntity(transfer);
        }
        else {
            //Just use the first one if multiple found.
            portAppEnt = (IPortletApplicationEntity)portAppEnts.iterator().next();
        }
        
        PortletEntityTransferImpl entityTransfer=new PortletEntityTransferImpl();
        entityTransfer.setPortletApplicationEntityId(portAppEnt.getPortletApplicationEntityId());
        entityTransfer.setPortletDefinitionId(portDef.getPortletDefinitionId());
        final IPortletEntityRegistry portletEntityRegistry = this.portletEntityRegistryFactory.getPortletEntityRegistry(userId);
        final IPortletEntity portEnt = portletEntityRegistry.createPortletEntity(entityTransfer);
        
        PortletWindowTransferImpl windowTransfer=new PortletWindowTransferImpl();
        windowTransfer.setPortletEntityId(portEnt.getPortletEntityId());
        final IPortletWindowRegistry portletWindowRegistry = this.portletWindowRegistryFactory.getPortletWindowRegistry(userId);
        final IPortletWindow realPortWin = portletWindowRegistry.createPortletWindow(windowTransfer);
        
        this.nodeAdditionManager.addPortlet(realPortWin.getPortletWindowId(), realPortWin.getPortletName(), portDef.getFunctionalName());
        
        // request to show addition targets in the redirect url
        if(response instanceof IActionResponse) {
            IActionResponse iar=(IActionResponse) response;
            ILegacyLayoutSourceCommands llsCommands=(ILegacyLayoutSourceCommands) iar.getRedirectUrlConstructor(ILegacyLayoutSourceCommands.CONSTRUCTOR_NAME);
            llsCommands.addMarkings();
        } else {
            logger.error("subscribe(ActionRequest, ActionResponse) - ActionResponse does not implement custom IActionResponse. Unable to signal layout source to show add targets");
        }
        
    }

    public void setGroupsManagerServant(IPortletServant groupsManagerServant) {
        this.groupsManagerServant = groupsManagerServant;
    }
    
    public void setCategoriesEntityTypeFilter(Set genericCategoriesEntityTypeFilter) throws ClassNotFoundException {
        this.categoriesEntityTypeFilter = new HashSet<Class>();
        for(Object o : genericCategoriesEntityTypeFilter) {
            if (o instanceof String) {
                this.categoriesEntityTypeFilter.add(Class.forName((String)o));
            } else if (o instanceof Class) {
                this.categoriesEntityTypeFilter.add((Class)o);
            } else {
                throw new IllegalArgumentException("entity type filter elements must be either String or Class: " + o.getClass().getName());
            }
        }
    }

    public void setEntityActionUrlParts(
        IEntityActionUrlParts entityActionUrlParts) {
        this.entityActionUrlParts = entityActionUrlParts;
    }

    public IGroupsManagerPermissions getGroupsManagerPermissions() {
        return groupsManagerPermissions;
    }

    public void setGroupsManagerPermissions(
        IGroupsManagerPermissions groupsManagerPermissions) {
        this.groupsManagerPermissions = groupsManagerPermissions;
    }
}
