/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlets.tests.servant;

import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.portal.portlet.servant.ServantUtil;

/**
 * A test servant portlet 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class ServantTestPortlet extends GenericPortlet {
    private static final String DEFAULT_VIEW = "/WEB-INF/jsp/tests/servant/servant.jsp";

    static final String VALUE_ATTR       = "servantRequestAttribute";
    static final String COMPLETE_ATTR    = "completeServant";
    static final String CANCEL_ATTR      = "cancelServant";

    @Override
    protected void doView(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        final PortletSession session = request.getPortletSession();
        
        final boolean inServantMode = ServantUtil.isInServantMode(request);
        request.setAttribute("inServantMode", inServantMode);
        
        request.setAttribute(VALUE_ATTR, session.getAttribute(VALUE_ATTR));
        
        final PortletContext context = this.getPortletContext();
        final PortletRequestDispatcher prd = context.getRequestDispatcher(DEFAULT_VIEW);
        prd.include(request, response);
    }

    @Override
    public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {
        final PortletSession session = request.getPortletSession();
        
        final String value = request.getParameter(VALUE_ATTR);
        if (value != null) {
            request.setAttribute(VALUE_ATTR, value);
            session.setAttribute(VALUE_ATTR, value);
        }

        final boolean inServantMode = ServantUtil.isInServantMode(request);
        if (inServantMode) {
            if (request.getParameter(COMPLETE_ATTR) != null) {
                session.setAttribute(COMPLETE_ATTR, true);
                // signal done and pass the value
                ServantUtil.setServantDone(request, value);
            }
            else if (request.getParameter(CANCEL_ATTR) != null) {
                session.setAttribute(CANCEL_ATTR, true);
                // signal canceled
                ServantUtil.setServantCanceled(request);
            }
        }
    }
}
