/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.portlets.themeselector;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.portal.context.common.profile.IProfile;
import org.jasig.portal.context.common.profile.IProfileController;
import org.jasig.portal.context.common.profile.ProfileId;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.web.portlet.ModelAndView;
import org.springframework.web.portlet.mvc.AbstractController;

/**
 * A dfault controller for selecting a theme 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class SelectThemeController extends AbstractController {
    IProfileController profileController;
    
    static final String SELECT_PROFILE_COMMAND="selectProfile";
    static final String CURRENT_PROFILE_ID_ATTRIBUTE="currentProfileId";
    
    
    public static final String MODEL_NAME = "profileConfig";
    public static final String VIEW_NAME = "themes";

    @Required
    public void setProfileController(IProfileController profileController) {
        this.profileController = profileController;
    }
    

    @Override
    protected void handleActionRequestInternal(ActionRequest req, ActionResponse res) throws Exception {
        String newProfileId=req.getParameter(SELECT_PROFILE_COMMAND);
        if(newProfileId!=null) {
            ProfileId id = new ProfileId(newProfileId);
            this.profileController.setCurrentProfile(id);
        }
    }

    @Override
    protected ModelAndView handleRenderRequestInternal(RenderRequest req, RenderResponse res) throws Exception {
        // specify current profile id as a request attribute
        IProfile profile=this.profileController.getCurrentProfile();
        if(profile!=null){
            ProfileId profileId=profile.getId();
            if(profileId!=null) {
                req.setAttribute(CURRENT_PROFILE_ID_ATTRIBUTE,profileId.toString());
            }
        }
        // return entire profile config as a model
        return new ModelAndView(VIEW_NAME, MODEL_NAME, this.profileController.getProfileConfig());
    }
    
}
