/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.utils;

import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;


/**
 * Defines the base rendering unit in the page rendering framework.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public interface IPage {
    /**
     * Renders the UI for the page.
     * 
     * @param request the render request
     * @param response the render response
     * @throws IOException if the streaming causes an I/O problem
     * @throws PortletException if the portlet has problems fulfilling the rendering request
     */
    public abstract void renderPage(RenderRequest request, RenderResponse response) throws IOException, PortletException;
    
    /**
     * Processes the results of an action on the page.
     * 
     * @param request the render request
     * @param response the render response
     * @throws IOException if the streaming causes an I/O problem
     * @throws PortletException if the portlet has problems fulfilling the rendering request
     */
    public abstract void processResults(ActionRequest request, ActionResponse response) throws IOException, PortletException;
}
