/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.utils;

import java.io.IOException;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;




/**
 * Defines a page that may be used in a wizard UI. Provides a title and
 * can render a summary.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public interface IWizardPage extends IPage {
    /**
     * Returns the title to display for the page.
     * 
     * @return The title to display for the page.
     */
    public abstract String getTitle();
    
    
    /**
     * @throws ConfigurationException If there is an issue in the parameter processing for this wizard page.
     */
    public abstract void processResults(ActionRequest request, ActionResponse response) throws IOException, PortletException, ConfigurationException;
    
    /**
     * Informs the caller if the wizard page has a complete configuration. This
     * method should be called between the last call to {@link IPage#renderPage(RenderRequest, RenderResponse)}
     * and the next call to {@link #renderSummary(RenderRequest, RenderResponse)}.
     * 
     * @param request The request to get the state of the page from.
     * @param response The response to get the state of the page from.
     * @return A Map of field names to error messages for fields this page prompts for that are not filled in
     *      (similar to what a {@link ConfigurationException} provides). A zero length Map implies the
     *      configuration is complete. May not be null.
     */
    public Map isCompleted(ActionRequest request, ActionResponse response) throws IOException, PortletException;
    
    /**
     * Renders a summary UI for the data entered on this page. Assumes that the portlet manager
     * is rendering the UI around the summary.
     * 
     * @param request the render request
     * @param response the render response
     * @throws IOException if the streaming causes an I/O problem
     * @throws PortletException if the portlet has problems fulfilling the rendering request
     */
    public abstract void renderSummary(RenderRequest request, RenderResponse response) throws IOException, PortletException;
}
