/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.utils;

import java.io.IOException;

import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;


/**
 * Provides a basis for wizard pages that will use JSPs to render. The
 * subclass merely needs to ensure the appropriate property is set and
 * call super.renderPage or super.renderSummary
 * <br>
 * <br>
 * Configuration:
 * <table border="1">
 *     <tr>
 *         <th align="left">Property</th>
 *         <th align="left">Description</th>
 *         <th align="left">Required</th>
 *         <th align="left">Default</th>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">renderJspPage</td>
 *         <td>
 *             The name of the JSP page to render in the {@link #renderPage(RenderRequest, RenderResponse)}
 *             method. If null this method is a no-op.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">null</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">summaryJspPage</td>
 *         <td>
 *             The name of the JSP page to render in the {@link #renderSummary(RenderRequest, RenderResponse)}
 *             method. If null this method is a no-op.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">null</td>
 *     </tr>
 * </table>
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public abstract class JspWizardPage implements IWizardPage {
    private String renderJspPage = null;
    private String summaryJspPage = null;
    
    /**
     * @return Returns the renderJspPage.
     */
    public String getRenderJspPage() {
        return this.renderJspPage;
    }
    /**
     * @param renderJspPage The renderJspPage to set.
     */
    public void setRenderJspPage(String renderJspPage) {
        this.renderJspPage = renderJspPage;
    }

    /**
     * @return Returns the summaryJspPage.
     */
    public String getSummaryJspPage() {
        return this.summaryJspPage;
    }
    /**
     * @param summaryJspPage The summaryJspPage to set.
     */
    public void setSummaryJspPage(String summaryJspPage) {
        this.summaryJspPage = summaryJspPage;
    }

    
    public void renderPage(RenderRequest request, RenderResponse response) throws IOException, PortletException {
        if (this.renderJspPage != null) {
            RenderUtils.dispatch(request, response, this.renderJspPage);
        }
    }
    
    public void renderSummary(RenderRequest request, RenderResponse response) throws IOException, PortletException {
        if (this.summaryJspPage != null) {
            RenderUtils.dispatch(request, response, this.summaryJspPage);
        }
    }
}
