/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.utils;

import org.springframework.beans.factory.BeanInitializationException;
import org.springframework.beans.factory.InitializingBean;

/**
 * Defines an action on an IActionPage.
 * <br>
 * <br>
 * Configuration:
 * <table border="1">
 *     <tr>
 *         <th align="left">Property</th>
 *         <th align="left">Description</th>
 *         <th align="left">Required</th>
 *         <th align="left">Default</th>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">nextPageName</td>
 *         <td>
 *             The name of the next page to display.
 *         </td>
 *         <td valign="top">Yes</td>
 *         <td valign="top"></td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">actionTargetPageName</td>
 *         <td>
 *             The name of the IActionPage to perform the action processing on. If
 *             not specified the action processing will be done on the IActionPage
 *             specified by nextPageName.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">null</td>
 *     </tr>
 *     <tr>
 *         <td align="right" valign="top">actionMethodName</td>
 *         <td>
 *             The name of the method to call for the action. The method must have
 *             the signature (ActionRequest, ActionResponse). If the actionTargetPageName
 *             property is specified the method is called on the named IActionPage, if not
 *             it is called on the IActionPage named by the nextPageName property.
 *         </td>
 *         <td valign="top">No</td>
 *         <td valign="top">null</td>
 *     </tr>
 * </table>
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class PageAction implements InitializingBean {
    private String actionMethodName;
    private String nextPageName;
    private String actionTargetPageName;
    
    /**
     * @return Returns the actionMethodName.
     */
    public String getActionMethodName() {
        return this.actionMethodName;
    }
    /**
     * @param actionMethodName The actionMethodName to set.
     */
    public void setActionMethodName(String actionMethodName) {
        this.actionMethodName = actionMethodName;
    }
    
    /**
     * @return Returns the nextPageName.
     */
    public String getNextPageName() {
        return this.nextPageName;
    }
    /**
     * @param nextPageName The targetPageName to set.
     */
    public void setNextPageName(String nextPageName) {
        if (nextPageName == null) {
            throw new IllegalArgumentException("nextPageName may not be null");
        }

        this.nextPageName = nextPageName;
    }
    
    /**
     * @return Returns the actionTargetPageName.
     */
    public String getActionTargetPageName() {
        return this.actionTargetPageName;
    }
    /**
     * @param actionTargetPageName The actionTargetPageName to set.
     */
    public void setActionTargetPageName(String actionTargetPageName) {
        this.actionTargetPageName = actionTargetPageName;
    }

    /**
     * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
     */
    public void afterPropertiesSet() throws Exception {
        if (this.nextPageName == null) {
            throw new BeanInitializationException("targetPageName may not be null");
        }
    }
}