/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.utils;

import java.io.IOException;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.GenericPortlet;
import javax.portlet.PortletException;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.portal.spring.PortletWebApplicationContextUtils;
import org.springframework.web.context.WebApplicationContext;


/**
 * Portlet implementation that uses an init-parameter named 'rootPage'
 * to resolve an IPage to delegate to.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class PageRenderingPorlet extends GenericPortlet {
    private static String PAGE_BEAN_KEY = "rootPage";

    /**
     * @see javax.portlet.GenericPortlet#doDispatch(javax.portlet.RenderRequest, javax.portlet.RenderResponse)
     */
    protected void doDispatch(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        final IPage page = this.getRootPage();
        page.renderPage(request, response);
    }

    /**
     * @see javax.portlet.GenericPortlet#processAction(javax.portlet.ActionRequest, javax.portlet.ActionResponse)
     */
    public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {
        final IPage page = this.getRootPage();
        page.processResults(request, response);
    }
    
    /**
     * Gets the configured IPage this portlet should delegate to.
     * 
     * @return The IPage to delegate to.
     * @throws PortletException If the init-parameter is not specified or the bean isn't found in the application context.
     */
    private IPage getRootPage() throws PortletException {
        final String pageBeanName = this.getPortletConfig().getInitParameter(PAGE_BEAN_KEY);
        
        if (pageBeanName == null) {
            throw new PortletException("No '" + PAGE_BEAN_KEY + "' initParameter specified. Cannot find root IPage.");
        }
        
        final WebApplicationContext context = PortletWebApplicationContextUtils.getRequiredWebApplicationContext(this.getPortletContext());
        final IPage managerPage = (IPage)context.getBean(pageBeanName, IPage.class);
        
        if (pageBeanName == null) {
            throw new PortletException("No IPage found for initParameter '" + PAGE_BEAN_KEY + "' = '" + pageBeanName + "'.");
        }
        
        return managerPage;
    }

}
