/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.portlets.utils;

import java.io.IOException;

import javax.portlet.PortletContext;
import javax.portlet.PortletException;
import javax.portlet.PortletRequestDispatcher;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;


/**
 * Utilities for when a portlet is rendering.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public final class RenderUtils {
    private RenderUtils() { }
    
    /**
     * Gets the PortletContext from the PortletSession (Creating one if nessesary)
     * and uses it to create a request dispatcher for the specified resource which
     * the request is included in the response.
     * 
     * @param request The request to get the session from and include with.
     * @param response The response to include with.
     * @param resource The path to the resource for the portlet request dispatcher.
     * @throws PortletException See: {@link PortletRequestDispatcher#include(javax.portlet.RenderRequest, javax.portlet.RenderResponse)}
     * @throws IOException See: {@link PortletRequestDispatcher#include(javax.portlet.RenderRequest, javax.portlet.RenderResponse)}
     */
    public static void dispatch(RenderRequest request, RenderResponse response, String resource) throws PortletException, IOException {
        final PortletSession session = request.getPortletSession(true);
        final PortletContext context = session.getPortletContext();
        final PortletRequestDispatcher dispatcher = context.getRequestDispatcher(resource);
        dispatcher.include(request, response);
    }
}
