/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.problem;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.core.exception.PortalRuntimeException;
import org.springframework.context.ApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

/**
 * Listens to servlet context events and configures the core portal exceptions
 * In particular, injects a reference to an
 * implementation of IProblemsManager and a reference
 * to a message source for localized parameterized error messages.  The implementation is obtained from the application context.
 * @author Ken Weiner, kweiner@unicon.net
 * @version $Revision: 11918 $
 */
public class ExceptionConfigurationListener implements ServletContextListener {
    public static final String PROBLEMSMANAGER_BEAN_NAME_PARAM="problemsManagerBeanName";
    public static final String DEFAULT_PROBLEMSMANAGER_BEAN_NAME="problemsManager";
    
    public void contextInitialized(ServletContextEvent servletContextEvent) {
        ApplicationContext applicationContext = WebApplicationContextUtils.getWebApplicationContext(servletContextEvent.getServletContext());
        // determine problems manager location
        String problemsManagerBeanName=DEFAULT_PROBLEMSMANAGER_BEAN_NAME;
        ServletContext sc=servletContextEvent.getServletContext();
        String paramValue=sc.getInitParameter(PROBLEMSMANAGER_BEAN_NAME_PARAM);
        if(paramValue!=null) {
            problemsManagerBeanName=paramValue;
        }
        // obtain problemsManager
        IProblemsManager problemsManager = (IProblemsManager)applicationContext.getBean(problemsManagerBeanName, IProblemsManager.class);
        PortalException.setProblemsManager(problemsManager);
        PortalRuntimeException.setProblemsManager(problemsManager);
    }

    public void contextDestroyed(ServletContextEvent servletContextEvent) {
        // Do nothing
    }

}