/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.problem;

/**
 * Interface for ProblemsManager implementations.
 * IProblemsManager describes the callback method whereby
 * IPortalExceptions are reported to the ProblemsManager.
 * @author andrew.petro@yale.edu
 * $Revision: 11911 $ $Date: 2007-03-23 10:16:18 -0500 (Fri, 23 Mar 2007) $ 
 */
public interface IProblemsManager {
    
    /**
     * Report to the ProblemsManager a particular throwable.
     * The Throwable provided to this method will often not be 
     * fully constructed, since this method is called from the 
     * (Undeclared)PortalException constructor.  This method
     * provides a reference to a Throwable which may later be
     * interrogated to obtain the message, cause, stack trace, etc.
     * Implementations wishing to log errors should not interrogate
     * the Throwable immediately when this method is called.
     * @param throwable
     */
    public void reportThrowable(Throwable throwable);
    
    /**
     * Get then number of times reportThrowable has been
     * called on the 
     * ProblemsManager implementation.
     * @return the count of Throwables reported
     */
    public long getCount();

}
