/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering;

import java.io.Serializable;
import java.util.List;

import org.jasig.portal.caching.ICacheable;



/**
 * A class describing possible pipeline cache entry point
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public abstract class AbstractCacheEntryPoint implements ICacheEntryPoint {
    int cacheFilterPosition;
    Serializable[] cacheValidity;
    public AbstractCacheEntryPoint(int position, Serializable[] validity) {
        this.cacheFilterPosition=position;
        this.cacheValidity=validity;
    }
    /**
     * Obtain position of the cache filter in the rendering pipeline
     * @return Returns the cacheFilterPosition.
     */
    protected int getCacheFilterPosition() {
        return cacheFilterPosition;
    }

    /**
     * Obtain object validity for the pipeline
     * @return Returns the cacheValidity.
     */
    protected Serializable[] getCacheValidity() {
        return cacheValidity;
    }
    
    /**
     * Cecks if this cache entry is valid for the current
     * rendering cycle
     * @param pipelineElements
     * @return validity
     */
    public boolean isValid(List pipelineElements) {
        if(cacheValidity==null || cacheValidity.length!=cacheFilterPosition) {
            return false;
        }
        
        boolean cacheValid=true;
        for(int pipelinePosition=0; pipelinePosition<cacheFilterPosition; pipelinePosition++) {
            // validate cache
            ICacheable pipelineElement=(ICacheable)pipelineElements.get(pipelinePosition);
            if(!pipelineElement.isCacheValid(cacheValidity[pipelinePosition])) {
                cacheValid=false;
                break;
            }
        }
        return cacheValid;
    }
    
    /**
     * Replay cache
     * @return <code>true</code> if the cache replay was successful
     */
    public abstract boolean replayCache() throws RenderingException;
}