/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering;
import java.util.List;

/**
 * An interface defining pipeline cache managers - helper classes that
 * assist rendering pipelines in determining optimal cache entry points.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public interface IPipelineCacheManager {
    /**
     * Determine optimal cache entry point for the current rendering cycle.
     * The method will start from the beginning of a pipeline, finding all
     * {@link ICacheFilter}s, setting current cache keys and validating cache
     * entries these filters may report. The method will pick an optimal cache
     * entry point.
     * @param pipelineElements
     * @return a valid {@link AbstractCacheEntryPoint} or <code>null</code>.
     */
    public AbstractCacheEntryPoint determineOptimalCacheEntryPoint(List pipelineElements);
}