/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering;

import java.util.EventObject;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * RenderingPipelineEvent description.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class RenderingPipelineEvent extends EventObject {
	final EventType eventType;
	HttpServletRequest httpServletRequest;
	HttpServletResponse httpServletResponse;
	
	/**
	 * An event fired before the rendering cycle starts
	 */
	public static final EventType startRenderingCycleEventType=new EventType("startRenderingCycle");

	/**
	 * An event fired at the end of a rendering cycle
	 */
	public static final EventType endRenderingCycleEventType=new EventType("endRenderingCycle");
	
    /**
     * Create a pipeline event
     * @param source - event source
     * @param eventType - event type
     */
	public RenderingPipelineEvent(Object source,EventType eventType) {
		super(source);
		this.eventType=eventType;
	}

	/**
	 * Create a pipeline event
	 * @param source - event source
	 * @param eventType - event type
	 * @param req - http request associated with the current cycle
	 * @param res - http response associated with the current cycle
	 */
	public RenderingPipelineEvent(Object source,EventType eventType, HttpServletRequest req, HttpServletResponse res) {
		this(source,eventType);		
		this.httpServletRequest=req;
		this.httpServletResponse=res;
	}
	
	/**
	 * @return Returns the http request associated with the current cycle
	 */
	public HttpServletRequest getHttpServletRequest() {
		return httpServletRequest;
	}

	/**
	 * @param httpServletRequest the http request associated with the current cycle.
	 */
	public void setHttpServletRequest(HttpServletRequest httpServletRequest) {
		this.httpServletRequest = httpServletRequest;
	}

	/**
	 * @return Returns the http response associated with the current cycle.
	 */
	public HttpServletResponse getHttpServletResponse() {
		return httpServletResponse;
	}

	/**
	 * @param httpServletResponse the http response associated with the current cycle.
	 */
	public void setHttpServletResponse(HttpServletResponse httpServletResponse) {
		this.httpServletResponse = httpServletResponse;
	}

	/**
	 * @return Returns the eventType.
	 */
	public EventType getEventType() {
		return eventType;
	}

	public static class EventType {
		final String name;
		
		public EventType(String name) {
			this.name=name;
		}
		
		/**
		 * @return Returns the name.
		 */
		public String getName() {
			return name;
		}
		
	}
}

