/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.attributes;

/**
 * A class describing possible XSLT param/attribute persistance scopes
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class AttributePersistenceScope {
    /**
     * <code>oneTimeScope</code> defines a one-time scope where the parameter
     * value will be valid only for the duration of the rendering cycle
     */
    public static final AttributePersistenceScope oneTimeScope=new AttributePersistenceScope("onetime");
    /**
     * <code>sessionScope</code> defines a scope where the parameter
     * value will be persisted for the duration of the current HTTP servlet session.
     */
    public static final AttributePersistenceScope sessionScope=new AttributePersistenceScope("session");
    
    /**
     * <code>localeScope</code> defines a scope where the parameter
     * value will be persisted for a specific locale.
     */
    /*
     * I've taken this out for now, since stylesheet description and related classes would have to
     * be refactored to accomodate mutli-locale defaults, etc.
     public static final AttributePersistenceScope localeScope=new AttributePersistenceScope("locale");
    */
    
    /**
     * <code>permanentScope</code> defines a scope where the parameter
     * value will be persisted across multiple sessions, independent of Locale.
     */
    public static final AttributePersistenceScope permanentScope=new AttributePersistenceScope("permanent");
    
    
    
    String scopeName;
    AttributePersistenceScope(String scopeName) {
        this.scopeName=scopeName;
    } 
    
    public String toString() {
        return this.scopeName;
    }
}
