/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Map;

import javax.xml.transform.Transformer;

import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.rendering.sax.SaxFilterImpl;
import org.jasig.portal.rendering.sax.xslt.IRenderingAttributeInjector;
import org.jasig.portal.rendering.sax.xslt.ITransformerPreferences;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;

/**
 * A uP2-style rendering attribute injector that adds XML attributes
 * corresponding to the rendering attribute names and values. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class AttributeRenderingAttributeInjector extends SaxFilterImpl implements IRenderingAttributeInjector {
    ITransformerPreferences transformerPreferences;
    
    
    public void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        Map attributes=getRenderingAttributeProvider().getAttributeValues(uri,localName,qName,atts);
        if(attributes!=null && !attributes.isEmpty()) {
            AttributesImpl ai=new AttributesImpl();
            // add non-conflicting existing attributes
            for(int i=0;i<atts.getLength();i++) {
                String attName=atts.getLocalName(i);
                if(attName==null) {
                    attName=atts.getQName(i);
                }
                if(!attributes.containsKey(attName)) {
                    ai.addAttribute(atts.getURI(i),atts.getLocalName(i),atts.getQName(i),atts.getType(i),atts.getValue(i));
                }
            }
            
            for (Iterator iter = attributes.entrySet().iterator(); iter.hasNext();) {
                Map.Entry e = (Map.Entry) iter.next();
                String key=(String)e.getKey();
                ai.addAttribute(null,key,key,"CDATA",(String)e.getValue());
            }
            super.startElement(uri,localName,qName,ai);
        } else {
            super.startElement(uri, localName, qName, atts);
        }
    }
    
    public void prepareTransformer(Transformer transformer) {
        Map params=getRenderingAttributeProvider().getParameterValues();
        if(params!=null && !params.isEmpty()) {
            for (Iterator iter = params.entrySet().iterator(); iter.hasNext();) {
                Map.Entry e = (Map.Entry) iter.next();
                transformer.setParameter((String)e.getKey(),(String)e.getValue());
            }
        }
    }

    public ICacheKey generateKey() {
        return getRenderingAttributeProvider().generateKey();
    }
    public boolean isCacheValid(Serializable validityObject) {
        return getRenderingAttributeProvider().isCacheValid(validityObject);
    }

    public ITransformerPreferences getTransformerPreferences() {
        return transformerPreferences;
    }

    public void setTransformerPreferences(ITransformerPreferences transformerPreferences) {
        this.transformerPreferences = transformerPreferences;
    }

    /**
     * @return Returns the renderingAttributeProvider.
     */
    IRenderingAttributeProvider getRenderingAttributeProvider() {
        return transformerPreferences.getRenderingAttributeProvider();
    }
}
