/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.springframework.beans.factory.annotation.Required;
import org.xml.sax.Attributes;

/**
 * A rendering attribute provider that outputs default values 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class DefaultValueRenderingAttributeProvider implements IRenderingAttributeProvider {
    protected IRenderingAttributesDescription renderingAttributeDescription;
    
    private Map defaultParamValues=null;
    private Map defaultAttributeValues=null;
    private String providerName;
    
    public void clearAttributes() {
    }

    public void clearAttributes(String namespaceUri, String localName, String qName, Attributes atts) {
    }

    public void clearParameters() {
    }

    public Map getAttributeValues(String namespaceUri, String localName, String qName, Attributes atts) {
        if(defaultAttributeValues==null) {
            Map values=new HashMap();
            for (Iterator iter = renderingAttributeDescription.getElementAttributeNames().iterator(); iter.hasNext();) {
                String attributeName = (String) iter.next();
                IAttributeDescription attributeDesc=renderingAttributeDescription.getElementAttributeDescription(attributeName);
                String attributeValue=attributeDesc.getDefaultValue();
                if(attributeValue!=null) {
                    values.put(attributeName,attributeValue);
                }
            }
            defaultAttributeValues=values;
        }
        return defaultAttributeValues;
    }

    public Map getParameterValues() {
        if(defaultParamValues==null) {
            Map values=new HashMap();
            for (Iterator iter = renderingAttributeDescription.getParamNames().iterator(); iter.hasNext();) {
                String paramName = (String) iter.next();
                IAttributeDescription paramDesc=renderingAttributeDescription.getParamDescription(paramName);
                String paramValue=paramDesc.getDefaultValue();
                if(paramValue!=null) {
                    values.put(paramName,paramValue);
                }
            }
            defaultParamValues=values;
        }
        return defaultParamValues;
    }

    public String getProviderName() {
        return this.providerName;
    }

    public boolean removeAttribute(String attributeName, String namespaceUri, String localName, String qName, Attributes atts) {
        return false;
    }

    public boolean removeParameter(String parameterName) {
        return false;
    }

    public void setAttributeValue(String attributeName, String attributeValue, String namespaceUri, String localName, String qName, Attributes atts) {
    }

    public void setParameterValue(String parameterName, String parameterValue) {
    }

    public ICacheKey generateKey() {
        return CacheKeyImpl.EMPTY_KEY;
    }

    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }

    @Required
    public void setRenderingAttributeDescription(IRenderingAttributesDescription renderingAttributeDescription) {
        this.renderingAttributeDescription = renderingAttributeDescription;
    }

    public void setProviderName(String providerName) {
        this.providerName = providerName;
    }

}
