/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Map;

import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;
import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * A dummy class to contain attributeValues Map, so that
 * {@link org.jasig.portal.rendering.attributes.RenderingAttributeValuesImpl} does not
 * contain nested collections, which are so much disliked by Hibernate.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class ElementAttributeValues implements ICacheable {
    Map attributeValues;
    Long id;

    public Map getAttributeValues() {
        return attributeValues;
    }

    public void setAttributeValues(Map attributeValues) {
        this.attributeValues = attributeValues;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public ICacheKey generateKey() {
        StringBuffer sb=new StringBuffer();
        for (Iterator iter = attributeValues.entrySet().iterator(); iter.hasNext();) {
            Map.Entry element = (Map.Entry) iter.next();
            sb.append(element.getKey()).append(element.getValue());
        }
        return new CacheKeyImpl(sb.toString());
    }

    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof ElementAttributeValues)) {
            return false;
        }
        ElementAttributeValues rhs = (ElementAttributeValues)object;
        return new EqualsBuilder()
            .append(this.attributeValues, rhs.attributeValues)
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(988329175, 666691879)
            .append(this.attributeValues)
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
            .append("id", this.id)
            .append("attributeValues", this.attributeValues)
            .toString();
    }
    
    
}
