/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.xml.sax.Attributes;

/**
 *  A simple element resolver implementation that matches on a specified list
 *  of element names (resolved as qName or localName), and determines the key
 *  based on the element name and a list of values of specified atttributes.
 *  
 *  The resolver will not resolve an element if the name can not be matched or
 *  any of the attributes are missing.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class ElementResolverImpl implements IElementResolver {

    
    Set elementNames;
    List attributeNames;
    IElementResolverKeyFactory elementResolverKeyFactory;

    /**
     * Empty constructor
     */
    public ElementResolverImpl() {
        super();
    }
    
    public ElementResolverImpl(Set elementNames, List attributeNames, IElementResolverKeyFactory elementResolverKeyFactory) {
        super();
        this.attributeNames = attributeNames;
        this.elementNames = elementNames;
        this.elementResolverKeyFactory = elementResolverKeyFactory;
    }
    
    /**
     * Specify an ordered list of attribute names that should be used
     * to construct element key. The names will appear in the key in
     * the same order, separated by the field separator.
     * @param attributeNames
     */
    public void setAttributeNames(List attributeNames) {
        this.attributeNames = attributeNames;
    }
    
    /**
     * Specify a set of element names that should be resolved. The String
     * value of each name will be matched against elements' qName or localName,
     * whichever is not null.
     * @param elementNames
     */
    public void setElementNames(Set elementNames) {
        this.elementNames = elementNames;
    }


    public ElementResolverKey resolveElementKey(String namespaceUri, String localName, String qName, Attributes atts) {
        String elementName=localName;
        // recognize either qName or localName - whichever is not null
        if(elementName==null) { elementName=qName; }
        
        if(elementName!=null) {
            if(elementNames.contains(elementName)) {
                // matched element
                Map attributeValues=new HashMap();
                for (Iterator iter = attributeNames.iterator(); iter.hasNext();) {
                    String attributeName = (String) iter.next();
                    String attributeValue=atts.getValue(attributeName);
                    if(attributeValue==null) {
                        return null;
                    }
                    attributeValues.put(attributeName,attributeValue);
                }
                return elementResolverKeyFactory.createKey(elementName,attributeValues);
            }
        }
        return null;
    }

    public void setElementResolverKeyFactory(IElementResolverKeyFactory elementResolverKeyFactory) {
        this.elementResolverKeyFactory = elementResolverKeyFactory;
    }
}
