/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.util.Iterator;
import java.util.Map;

/**
 * Default implementation of the resolver key factory, that creates
 * assembled string keys.  
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class ElementResolverKeyFactoryImpl implements IElementResolverKeyFactory {
    /**
     * Default separator character to use when constructing the key.
     */
    public final static String DEFAULT_KEY_FIELD_SEPARATOR = ".";

    private String keySeparator = DEFAULT_KEY_FIELD_SEPARATOR;

    public ElementResolverKey createKey(String qName, Map attributes) {
        final StringBuilder ksb = new StringBuilder(qName);
        for (Iterator iter = attributes.values().iterator(); iter.hasNext();) {
            String value = (String)iter.next();
            ksb.append(keySeparator).append(value);
        }
        
        return new ElementResolverKey(ksb.toString());
    }

    public ElementResolverKey createKey(String namespaceUri, String localName, Map attributes) {
        final StringBuilder ksb = new StringBuilder(namespaceUri);
        ksb.append(keySeparator).append(localName);
        for (Iterator iter = attributes.values().iterator(); iter.hasNext();) {
            String value = (String)iter.next();
            ksb.append(keySeparator).append(value);
        }
        
        return new ElementResolverKey(ksb.toString());
    }

    public void setKeySeparator(String keySeparator) {
        this.keySeparator = keySeparator;
    }

}
