/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

/**
 * An object that provides information on whether a particular
 * attribute should be maintained at a current preference level.
 * For instance, one implementation could discriminate runtime
 * attributes from persistant ones. Another could discriminate
 * locale-persistent attributes from general persistent attributes, etc.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IAttributeLevelResolver {
    /**
     * Determine if parameter should be maintained at the current level
     * @param parameterName
     * @return <code>true</code> if the value of the parameter should be maintained at the current level, <code>false</code> otherwise.
     */
    boolean isParameterInLevel(String parameterName);
    
    /**
     * Determine if an attribute for a particular element
     * should be maintained at the current level
     * @param resolverKey element resolver key
     * @param attributeName
     * @return <code>true</code> if the value of the attribute should be maintained at the current level, <code>false</code> otherwise.
     */
    boolean isElementAttributeInLevel(ElementResolverKey resolverKey, String attributeName);
}
