/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import org.xml.sax.Attributes;

/**
 *  Responsible for recognizing certain elements and returning
 *  a unified key which can be used to map attribute values for
 *  a particular element.
 *  
 *  Element resolvers are typically configured to recognize specific
 *  elements (i.e. "channel" or "folder"), and determine the key by
 *  looking at attribute values (i.e. "ID" attribute).
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IElementResolver {
    /**
     * Resolve attribute mapping key for a given element
     * 
     * @param namespaceUri elements' namespaceUri
     * @param localName elements' localName
     * @param qName elements' qName
     * @param atts elements' attributes
     * @return <code>null</code> if the current provider does not establish attribute mapping for this element, or
     * a key value from which attribute values can be mapped.
     */
    ElementResolverKey resolveElementKey(String namespaceUri, String localName, String qName, Attributes atts);
}
