/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.util.Map;

/**
 *  A factory for creating element resolver keys in a
 *  semi-structured manner (that is providing separation
 *  between element name and attributes)
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IElementResolverKeyFactory {
    /**
     * Create a resolver key based on namespace-aware element description
     * 
     * @param namespaceUri
     * @param localName
     * @param attributes a <code>Map</code> of <code>String</code> attribute names to <code>String</code> values
     * @return resolver key instance
     */
    ElementResolverKey createKey(String namespaceUri, String localName, Map attributes);
    
    
    /**
     * Create a resolver key based on qName
     * 
     * @param qName
     * @param attributes  a <code>Map</code> of <code>String</code> attribute names to <code>String</code> values
     * @return resolver key instance
     */
    ElementResolverKey createKey(String qName, Map attributes);
}
