/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.util.Map;

import org.jasig.portal.caching.ICacheable;
import org.xml.sax.Attributes;

/**
 * Provides values of the underlying attributes.
 * 
 * A typical provider implementation is configured with a set of attribute descriptions,
 * a DAO implementation and an element resolver. Providers may be nested or otherwise
 * composed internally.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public interface IRenderingAttributeProvider extends ICacheable {
    
    /**
     * Obtain provider name
     * 
     * @return provider name
     */
    String getProviderName();
    
    /**
     * Obtain attribute values for a specific node (SAX version)
     * 
     * @param namespaceUri
     * @param localName
     * @param qName
     * @param atts
     * @return a <code>Map</code> from <code>String</code> attribute name to <code>String</code> value.
     */
    Map getAttributeValues(String namespaceUri, String localName, String qName, Attributes atts);
    
    /**
     * Obtain parameter values
     * 
     * @return a <code>Map</code> from <code>String</code> param name to <code>String</code> value
     */
    Map getParameterValues();
    
    /**
     * Set parameter value
     * 
     * @param parameterName
     * @param parameterValue
     */
    void setParameterValue(String parameterName, String parameterValue);
    
    /**
     * Set attribute value for a specific element
     * 
     * @param attributeName
     * @param attributeValue
     * @param namespaceUri
     * @param localName
     * @param qName
     * @param atts
     */
    void setAttributeValue(String attributeName, String attributeValue, String namespaceUri, String localName, String qName, Attributes atts);
    
    /**
     * Remove parameter value
     * @param parameterName
     * @return <code>true</code> if parameter was removed, <code>false</code> if it didn't exist
     */
    boolean removeParameter(String parameterName);
    
    /**
     * Remove attribute value for a given element
     * 
     * @param attributeName
     * @param namespaceUri
     * @param localName
     * @param qName
     * @param atts
     * @return <code>true</code> if attribute was removed, <code>false</code> if it didn't exist
     */
    boolean removeAttribute(String attributeName, String namespaceUri, String localName, String qName, Attributes atts);
    
    /**
     * Clear all parameters
     */
    void clearParameters();
    
    /**
     * Clear all attributes for a particular element
     * 
     * @param namespaceUri
     * @param localName
     * @param qName
     * @param atts
     */
    void clearAttributes(String namespaceUri, String localName, String qName, Attributes atts);
    
    /**
     * Clear all attributes for all elements
     */
    void clearAttributes();

}
