/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import org.xml.sax.Attributes;

/**
 * Rendering attribute provider commands (requires to identify providerName in each call)
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public interface IRenderingAttributeProviderCommands {
    /**
     * Set parameter value
     * 
     * @param parameterName
     * @param parameterValue
     */
    void setParameterValue(String providerName, String parameterName, String parameterValue);
    
    /**
     * Set attribute value for a specific element
     * 
     * @param attributeName
     * @param attributeValue
     * @param namespaceUri
     * @param localName
     * @param qName
     * @param atts
     */
    void setAttributeValue(String providerName, String attributeName, String attributeValue, String namespaceUri, String localName, String qName, Attributes atts);
    
    /**
     * Remove parameter value
     * @param parameterName
     * @return <code>true</code> if parameter was removed, <code>false</code> if it didn't exist
     */
    boolean removeParameter(String providerName, String parameterName);
    
    /**
     * Remove attribute value for a given element
     * 
     * @param attributeName
     * @param namespaceUri
     * @param localName
     * @param qName
     * @param atts
     * @return <code>true</code> if attribute was removed, <code>false</code> if it didn't exist
     */
    boolean removeAttribute(String providerName, String attributeName, String namespaceUri, String localName, String qName, Attributes atts);
    
    /**
     * Clear all parameters
     */
    void clearParameters(String providerName);
    
    /**
     * Clear all attributes for a particular element
     * 
     * @param namespaceUri
     * @param localName
     * @param qName
     * @param atts
     */
    void clearAttributes(String providerName, String namespaceUri, String localName, String qName, Attributes atts);
    
    /**
     * Clear all attributes for all elements
     */
    void clearAttributes(String providerName);
}
