/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.util.Map;
import java.util.Set;

import org.jasig.portal.caching.ICacheable;

/**
 * An interface for an object containing parameter and
 * element attribute vlaues.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public interface IRenderingAttributeValues extends ICacheable {
    
    /**
     * Obtain a map of parameter values
     * @return a <code>Map</code> from <code>String</code> parameter name to <code>String</code> parameter value
     */
    Map getParameterValues();
    
    /**
     * Specify a parameter value
     * 
     * @param parameterName
     * @param parameterValue
     */
    void setParameterValue(String parameterName, String parameterValue);
    
    /**
     * Remove parameter
     * 
     * @param parameterName
     * @return <code>true</code> if the parameter was removed, <code>false</code> if it did not exist
     */
    boolean removeParameter(String parameterName);
    
    /**
     * Obtain a collection of element resolver keys for which attributes are mapped
     * @return a <code>Collection</code> of element resolver keys
     */
    Set getElementResolverKeys();
    
    /**
     * Obtain a map from resolver element keys to value maps.
     * @param resolverKey element resolver key
     * @return a <code>Map</code> from <code>String</code> element attribute names to <code>String</code> values
     */
    Map getElementAttributeValues(ElementResolverKey resolverKey);
    
    /**
     * Specify element attribute value
     * 
     * @param resolverKey
     * @param attributeName
     * @param attributeValue
     */
    void setElementAttributeValue(ElementResolverKey resolverKey, String attributeName, String attributeValue);
    
    /**
     * Remove element attribute
     * 
     * @param resolverKey
     * @param attributeName
     * @return <code>true</code> if the attribute was removed, <code>false</code> if it did not exist
     */
    boolean removeElementAttributeValue(ElementResolverKey resolverKey, String attributeName);
    
    /**
     * Remove all attributes for a given element
     * @param resolverKey
     * @return <code>true</code> if the attributes were removed, <code>false</code> if the element key did not exist
     */
    boolean removeElementAttributes(ElementResolverKey resolverKey);
    
    /**
     * Remove all element attribute values
     */
    void clearElementAttributes();
    
    /**
     * Remove all parameter values
     */
    void clearParameters();
}
