/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.util.Set;

import org.jasig.portal.caching.ICacheable;

/**
 *  Provides definition information on params and element
 *  attributes.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public interface IRenderingAttributesDescription extends ICacheable {
    /**
     * Provides a list of param names
     * @return <code>Set</code> of <code>String</code> param names
     */
    Set getParamNames();
    
    /**
     * Get description of a declared params
     * 
     * @param name of the param
     * @return param description
     */
    IAttributeDescription getParamDescription(String name);
    
    /**
     * Get a list of the element attributes declared for a given qName
     * 
     * @return a <code>Set</code> of attribute name <code>String</code>s.
     */
    Set getElementAttributeNames();
    
    /**
     * Get description of a declared element attribute
     * 
     * @param attributeName name of the attribute
     * @return attribute description
     */
    IAttributeDescription getElementAttributeDescription(String attributeName);
}
