/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.io.Serializable;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.context.common.persistence.IPersistenceController;
import org.jasig.portal.context.common.persistence.IPersistenceControllerEventListener;
import org.jasig.portal.context.common.persistence.PersistenceControllerEvent;
import org.jasig.portal.rendering.attributes.dao.IRenderingAttributesDao;

/**
 * A version of {@link org.jasig.portal.rendering.attributes.IRenderingAttributeValues}
 * that maintains its own persistance.  
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PersistentRenderingAttributeValues implements IRenderingAttributeValues, IPersistenceControllerEventListener {
    IRenderingAttributeValues values;
    IPersistenceController persistenceController;
    IRenderingAttributesDao renderingAttributesDao;
    
    public void receivePersistenceControllerEvent(PersistenceControllerEvent event) {
        if(event.getEventType()==PersistenceControllerEvent.persistPreferencesEventType) {
            if(values!=null) {
                renderingAttributesDao.setRenderingAttributes(values);
            }
        } else if(event.getEventType()==PersistenceControllerEvent.resetPreferencesEventType) {
            values=null;
        }
    }

    public void setPersistenceController(IPersistenceController persistenceController) {
        this.persistenceController = persistenceController;
        persistenceController.addPersistenceControllerListener(this);
    }

    public void setRenderingAttributesDao(IRenderingAttributesDao renderingAttributesDao) {
        this.renderingAttributesDao = renderingAttributesDao;
    }

    IRenderingAttributeValues getValues() {
        if(values==null) {
            // load values from the DAO
            values=renderingAttributesDao.getRenderingAttributes();
        }
        return values;
    }
    
    // delegate methods
    public void clearElementAttributes() {
        this.getValues().clearElementAttributes();
    }

    public void clearParameters() {
        this.getValues().clearParameters();
    }

    public Map getElementAttributeValues(ElementResolverKey resolverKey) {
        return this.getValues().getElementAttributeValues(resolverKey);
    }

    public Set getElementResolverKeys() {
        return this.getValues().getElementResolverKeys();
    }

    public Map getParameterValues() {
        return this.getValues().getParameterValues();
    }

    public boolean removeElementAttributes(ElementResolverKey resolverKey) {
        return this.getValues().removeElementAttributes(resolverKey);
    }

    public boolean removeElementAttributeValue(ElementResolverKey resolverKey, String attributeName) {
        return this.getValues().removeElementAttributeValue(resolverKey, attributeName);
    }

    public boolean removeParameter(String parameterName) {
        return this.getValues().removeParameter(parameterName);
    }

    public void setElementAttributeValue(ElementResolverKey resolverKey, String attributeName, String attributeValue) {
        this.getValues().setElementAttributeValue(resolverKey, attributeName, attributeValue);
    }

    public void setParameterValue(String parameterName, String parameterValue) {
        this.getValues().setParameterValue(parameterName, parameterValue);
    }

    public ICacheKey generateKey() {
        return this.getValues().generateKey();
    }

    public boolean isCacheValid(Serializable validityObject) {
        return this.getValues().isCacheValid(validityObject);
    }
}
