/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.io.Serializable;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;

/**
 * Describes parameters and attributes for a single provider.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class RenderingAttributesDescriptionImpl implements IRenderingAttributesDescription {

    Map attributeDescriptions=Collections.EMPTY_MAP;
    Map parameterDescriptions=Collections.EMPTY_MAP;
    
    private ICacheKey currentCacheKey=null;
    
    public RenderingAttributesDescriptionImpl() {
    }
    
    /**
     * Create a combination from a set of descriptions
     * @param descriptions attribute descriptions that should be merged
     */
    public RenderingAttributesDescriptionImpl(Set<IRenderingAttributesDescription> descriptions) {
        Set parameterDescriptions=new HashSet();
        Set attributeDescriptions=new HashSet();
        for (IRenderingAttributesDescription description: descriptions) {
            Set paramNames=description.getParamNames();
            for (Iterator iterator = paramNames.iterator(); iterator.hasNext();) {
                String paramName = (String) iterator.next();
                parameterDescriptions.add(description.getParamDescription(paramName));
            }
            
            Set attributeNames=description.getElementAttributeNames();
            for (Iterator iterator = attributeNames.iterator(); iterator.hasNext();) {
                String attributeName = (String) iterator.next();
                attributeDescriptions.add(description.getElementAttributeDescription(attributeName));
            }
        }
        this.setParameterDescriptions(parameterDescriptions);
        this.setAttributeDescriptions(attributeDescriptions);
    }
    
    
    public void setAttributeDescriptions(Set<IAttributeDescription> attributeDescriptions) {
        Map<String,IAttributeDescription> m=new HashMap<String,IAttributeDescription>();
        for (IAttributeDescription pd: attributeDescriptions) {
            m.put(pd.getName(),pd);
        }
        this.attributeDescriptions = m;
    }

    public void setParameterDescriptions(Set<IAttributeDescription> parameterDescriptions) {
        Map<String,IAttributeDescription> m=new HashMap<String,IAttributeDescription>();
        for (IAttributeDescription pd: parameterDescriptions) {
            m.put(pd.getName(),pd);
        }
        this.parameterDescriptions = m;
    }

    public IAttributeDescription getElementAttributeDescription(String attributeName) {
        return (IAttributeDescription) attributeDescriptions.get(attributeName);
    }

    public Set getElementAttributeNames() {
        return attributeDescriptions.keySet();
    }

    public IAttributeDescription getParamDescription(String name) {
        return (IAttributeDescription) parameterDescriptions.get(name);
    }

    public Set getParamNames() {
        return parameterDescriptions.keySet();
    }

    public ICacheKey generateKey() {
        if(currentCacheKey==null) {
            StringBuffer sb=new StringBuffer();
            for (Iterator iter = parameterDescriptions.values().iterator(); iter.hasNext();) {
                IAttributeDescription d = (IAttributeDescription) iter.next();
                appendKey(sb,d);
            }
            for (Iterator iter = attributeDescriptions.values().iterator(); iter.hasNext();) {
                IAttributeDescription d = (IAttributeDescription) iter.next();
                appendKey(sb,d);
            }
            currentCacheKey=new CacheKeyImpl(sb.toString());
        }
        return currentCacheKey;
    }
    
    /**
     * Appends serialized key info of a description to a supplied string buffer
     * @param sb
     * @param d
     */
    private void appendKey(StringBuffer sb, IAttributeDescription d) {
        // I don't see how description or persistance scope could 
        // influence rendering, so they are not included in the key.
        sb.append(d.getName()).append(d.getDefaultValue());
    }

    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }

}
