/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.springframework.beans.factory.annotation.Required;


/**
 * Selects session-scoped attributes 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class SessionAttributeLevelResolver implements IAttributeLevelResolver {
    IRenderingAttributesDescription renderingAttributeDescription;

    public boolean isElementAttributeInLevel(ElementResolverKey resolverKey, String attributeName) {
        IAttributeDescription desc=renderingAttributeDescription.getElementAttributeDescription(attributeName);
        if(desc==null) {
            // attributes that are not present in description should not be persisted
            return true;
        } else {
            AttributePersistenceScope scope=desc.getPersistenceScope();
            return AttributePersistenceScope.sessionScope.equals(scope);
        }
    }

    public boolean isParameterInLevel(String parameterName) {
        IAttributeDescription desc=renderingAttributeDescription.getParamDescription(parameterName);
        if(desc==null) {
            // attributes that are not present in description should not be persisted
            return true;
        } else {
            AttributePersistenceScope scope=desc.getPersistenceScope();
            return AttributePersistenceScope.sessionScope.equals(scope);
        }
    }

    public void setRenderingAttributesDescription(IRenderingAttributesDescription renderingAttributeDescription) {
        this.renderingAttributeDescription = renderingAttributeDescription;
    }

    /**
     * Specify a set of rendering attribute descriptions
     * @param renderingAttributeDescriptions a Set of {@link IRenderingAttributesDescription}s
     */
    public void setRenderingAttributeDescriptions(Set<IRenderingAttributesDescription> renderingAttributeDescriptions) {
        // create a combined description
        this.renderingAttributeDescription=new RenderingAttributesDescriptionImpl(renderingAttributeDescriptions);
    }
}
