/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.util.Map;

/**
 * An element resolver key factory that creates an {@link org.jasig.portal.rendering.attributes.InformativeElementResolverKey}
 * based on a single attribute (defined by attributeName property);
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class SingleAttributeElementResolverKeyFactory implements IElementResolverKeyFactory {
    String attributeName;

    protected final static String ELEMENT_NAMESPACE_SEPARATOR=":";
    
    public ElementResolverKey createKey(String qName, Map attributes) {
        String attributeValue=(String) attributes.get(attributeName);
        if(attributeValue!=null) {
            return new InformativeElementResolverKey(attributeValue,qName,attributes);
        } else {
            throw new IllegalArgumentException("attributes does not contain '"+attributeName+"' attribute");
        }
    }

    public ElementResolverKey createKey(String namespaceUri, String localName, Map attributes) {
        return createKey(namespaceUri+ELEMENT_NAMESPACE_SEPARATOR+localName,attributes);
    }

    /**
     * Specify the name of the attribute that will be recorded in the resulting key instance
     * @param attributeName
     */
    public void setAttributeName(String attributeName) {
        this.attributeName = attributeName;
    }
    
    
}
