/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.util.Map;

/**
 * A resolver key factory that returns a {@link org.jasig.portal.rendering.attributes.ElementResolverKey}
 * containing the value of a specified attribute.
 * 
 * Note: While {@link org.jasig.portal.rendering.attributes.SingleAttributeElementResolverKeyFactory} can be used
 * to construct informative keys that contain the value of a specified attribute in addition to the regular key, 
 * the current class is used to construct key that contains just the value of the attribute. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class SingleAttributeResolverKeyFactory implements IElementResolverKeyFactory {
    String attributeName;
    
    public ElementResolverKey createKey(String qName, Map attributes) {
        String attributeValue=(String) attributes.get(attributeName);
        if(attributeValue!=null) {
            return new ElementResolverKey(attributeValue);
        } else {
            throw new IllegalArgumentException("attributes does not contain '"+attributeName+"' attribute");
        }
    }

    public ElementResolverKey createKey(String namespaceUri, String localName, Map attributes) {
        return createKey(null,attributes);
    }

    /**
     * Specify the name of the attribute that will be used as a key
     * @param attributeName
     */
    public void setAttributeName(String attributeName) {
        this.attributeName = attributeName;
    }
}
