/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes;

import java.util.HashMap;
import java.util.Map;

import org.xml.sax.Attributes;

/**
 * Resolves a single element name with a single id
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class SingleElementResolver implements IElementResolver {
    String elementName;
    String attributeName;
    IElementResolverKeyFactory elementResolverKeyFactory;

    /**
     * Empty constructor
     */
    public SingleElementResolver() {
        super();
    }
    
    public SingleElementResolver(String elementName, String attributeName, IElementResolverKeyFactory elementResolverKeyFactory) {
        super();
        this.attributeName = attributeName;
        this.elementName = elementName;
        this.elementResolverKeyFactory = elementResolverKeyFactory;
    }
    

    public ElementResolverKey resolveElementKey(String namespaceUri, String localName, String qName, Attributes atts) {
        if(this.elementName.equals(localName) || this.elementName.equals(qName)) {
            // matched element
            String attributeValue=atts.getValue(attributeName);
            if(attributeValue!=null) {
                Map attributeValues=new HashMap();
                attributeValues.put(attributeName,attributeValue);
                return elementResolverKeyFactory.createKey(elementName,attributeValues);
            } 
        }
        return null;
    }

    public void setElementResolverKeyFactory(IElementResolverKeyFactory elementResolverKeyFactory) {
        this.elementResolverKeyFactory = elementResolverKeyFactory;
    }
}
