/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.rendering.attributes.dao;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.rendering.attributes.IRenderingAttributeValues;
import org.jasig.portal.rendering.attributes.RenderingAttributeValuesImpl;
import org.jasig.portal.security.UserId;

/**
 * An implementation of {@link org.jasig.portal.rendering.attributes.dao.IRenderingAttributesDao}
 * that persists attributes using userId and explicitly supplied stylesheet name as the primary key.
 * UserId is retrieved from the {@link org.jasig.portal.context.common.user.IUserController}
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class UserRenderingAttributeValuesDao implements IRenderingAttributesDao {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(UserRenderingAttributeValuesDao.class);

    IUserController userController;
    String stylesheetName;
    IUserRenderingAttributesDao userRenderingAttributesDao;
    
    public IRenderingAttributeValues getRenderingAttributes() {
        UserId userId=userController.getUserId();
        if (logger.isDebugEnabled()) {
            logger.debug("getRenderingAttributes() - retrieving rendering attributes : stylesheetName=" + stylesheetName+"; userId="+userId);
        }
        IRenderingAttributeValues values=userRenderingAttributesDao.getRenderingAttributes(userId,stylesheetName);
        if(values==null) {
            if (logger.isInfoEnabled()) {
                logger.info("getRenderingAttributes() - creating new rendering attribute values : stylesheetName=" + stylesheetName+"; userId="+userId);
            }
            return new RenderingAttributeValuesImpl();
        } else {
            return values;
        }
    }
    
    public void setRenderingAttributes(IRenderingAttributeValues values) {
        UserId userId=userController.getUserId();
        if (logger.isDebugEnabled()) {
            logger.debug("setRenderingAttributes() - saving rendering attributes : stylesheetName=" + stylesheetName+"; userId="+userId);
        }
        userRenderingAttributesDao.setRenderingAttributes(values, userId, stylesheetName);
    }
    
    public void setStylesheetName(String stylesheetName) {
        this.stylesheetName = stylesheetName;
    }
    public void setUserController(IUserController userController) {
        this.userController = userController;
    }

    public void setUserRenderingAttributesDao(IUserRenderingAttributesDao userRenderingAttributesTransferDao) {
        this.userRenderingAttributesDao = userRenderingAttributesTransferDao;
    }
    
    
}
