/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes.dao;

import java.io.Serializable;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.jasig.portal.rendering.attributes.IRenderingAttributeValues;
import org.jasig.portal.security.UserId;

/**
 * A transfer object for rendering attribute values that
 * adds user id and stylesheet name that form primary key. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class UserRenderingAttributeValuesTransfer implements Serializable {
    final static long serialVersionUID = 1L;
    
    UserId userId;
    String stylesheetName;
    IRenderingAttributeValues values;
    
    public String getStylesheetName() {
        return stylesheetName;
    }
    /**
     * Specify stylesheet name to be used in the primary key.
     * Note: the stylesheet name is simply used to determine primary key
     * (in combination with the userId). If, for instance, there are two
     * stylesheets that should be using the same parameters, one can and should
     * provide the same stylesheet name in both cases.
     */
    public void setStylesheetName(String providerName) {
        this.stylesheetName = providerName;
    }
    public UserId getUserId() {
        return userId;
    }
    public void setUserId(UserId userId) {
        this.userId = userId;
    }
    public IRenderingAttributeValues getValues() {
        return values;
    }
    public void setValues(IRenderingAttributeValues values) {
        this.values = values;
    }
    public boolean equals(Object obj) {
        if (!(obj instanceof UserRenderingAttributeValuesTransfer)) {
            return false;
        }
        if (this == obj) {
            return true;
        }
        
        UserRenderingAttributeValuesTransfer other = (UserRenderingAttributeValuesTransfer)obj;
        return new EqualsBuilder()
            .append(this.userId, other.userId)
            .append(this.stylesheetName, other.stylesheetName)
            .isEquals();
    }
    public int hashCode() {
        //Binary or on each to ensure we get an odd number
        final int classCode = this.getClass().hashCode() | 1;
        final int classNameCode = this.getClass().getName().hashCode() | 1;
        
        return new HashCodeBuilder(classCode, classNameCode)
            .append(this.userId)
            .append(this.stylesheetName)
            .toHashCode();
    }
    
    
}
