/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.attributes.url;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.rendering.attributes.IRenderingAttributeProvider;
import org.jasig.portal.url.IRequestParameterProcessor;
import org.jasig.portal.url.IWritableHttpServletRequest;
import org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider;
import org.jasig.portal.url.syntax.UrlBasicSyntaxProviderImpl;
import org.jasig.portal.utils.IStringConcatenator;
import org.jasig.portal.utils.UrlEncodingStringConcatenator;
import org.xml.sax.helpers.AttributesImpl;

/**
 *  Default implementation of the rendering attribute request parameter processor
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11917 $
 */
public class RenderingAttributeRequestParameterProcessor implements IRequestParameterProcessor {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(RenderingAttributeRequestParameterProcessor.class);

    public static final String DEFAULT_PROCESSOR_PREFIX="rap";
    
    // injected properties
    IUrlBasicSyntaxProvider basicSyntaxProvider=new UrlBasicSyntaxProviderImpl();
    String processorPrefix=DEFAULT_PROCESSOR_PREFIX;
    IStringConcatenator stringConcatenator=new UrlEncodingStringConcatenator();
    
    
    // internal properties
    HashMap renderingAttributeProviderMap=null;

    public void setProcessorPrefix(String processorPrefix) {
        this.processorPrefix = processorPrefix;
    }
    
   

    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res) {
        // process setParameterValue commands
        String[] setParamValues=req.getParameterValues(basicSyntaxProvider.incorporateProcessorPrefix(processorPrefix,RenderingAttributeUrlConstructorImpl.SET_PARAMETER_COMMAND));
        if(setParamValues!=null && setParamValues.length>0) {
            for (int i = 0; i < setParamValues.length; i++) {
                // extract full information
                List l=stringConcatenator.split(setParamValues[i]);
                if(l.size()==3) {
                    Iterator it=l.iterator();
                    String providerName=(String) it.next();
                    String parameterName=(String) it.next();
                    String parameterValue=(String) it.next();
                    IRenderingAttributeProvider provider=(IRenderingAttributeProvider) renderingAttributeProviderMap.get(providerName);
                    if(provider==null) {
                        logger.error("Ivalid rendering attribute provider specified : providerName=" + providerName +" setParamValues="+setParamValues[i], null);
                    } else {
                        if (logger.isDebugEnabled()) {
                            logger.debug("setting parameter value : providerName=" + providerName + ", parameterName=" + parameterName + ", parameterValue=" + parameterValue);
                        }

                        provider.setParameterValue(parameterName,parameterValue);
                    }
                } else {
                    logger.error("wrong number of items included in the setParameterValue url. setParamValues="+setParamValues[i]);
                    throw new IllegalStateException("setParameterValues param list.size()!=3");
                }
                
            }
        }
        
        // process setParameterValue commands
        String[] setAttributeValues=req.getParameterValues(basicSyntaxProvider.incorporateProcessorPrefix(processorPrefix,RenderingAttributeUrlConstructorImpl.SET_ATTRIBUTE_COMMAND));
        if(setAttributeValues!=null && setAttributeValues.length>0) {
            for (int i = 0; i < setAttributeValues.length; i++) {
                // extract full information
                List l=stringConcatenator.split(setAttributeValues[i]);
                if(l.size()>=4) {
                    Iterator it=l.iterator();
                    String providerName=(String) it.next();
                    String attName=(String) it.next();
                    String attValue=(String) it.next();
                    String qName=(String) it.next();
                    // parse out element attributes
                    AttributesImpl attsImpl=new AttributesImpl();
                    while(it.hasNext()) {
                        String aN=(String) it.next();
                        if(it.hasNext()) {
                            String aV=(String) it.next();
                            attsImpl.addAttribute("",aN,aN,"CDATA",aV);
                        } else {
                            logger.error("wrong number of items included in the setAttributeValue url - unable to find value for element attribute name "+aN +" setAttributeValues="+setAttributeValues[i]);
                        }
                    }
                    IRenderingAttributeProvider provider=(IRenderingAttributeProvider) renderingAttributeProviderMap.get(providerName);
                    if(provider==null) {
                        logger.error("Ivalid rendering attribute provider specified : providerName=" + providerName+" setAttributeValues="+setAttributeValues[i], null);
                    } else {
                        if (logger.isDebugEnabled()) {
                            String element="<"+qName;
                            for(int j=0;j<attsImpl.getLength();j++) {
                                element+=" "+attsImpl.getQName(j)+"="+attsImpl.getValue(j);
                            }
                            element+="/>";
                            logger.debug("setting attribute value : providerName=" + providerName + ", attName=" + attName + ", attValue=" + attValue + ", element="+element);
                        }
                        provider.setAttributeValue(attName,attValue,"",qName,qName,attsImpl);
                    }
                } else {
                    logger.error("too few itesm items included in the setAttributeValue url. setAttributeValues="+setAttributeValues[i]);
                    throw new IllegalStateException("setParameterValues param list.size() < 4");
                }
                
            }
        }
        
        return true;
    }

    /**
     * Specify a list of rendering attribute providers to which the events should be delivered
     * @param renderingAttributeProviders
     */
    public void setRenderingAttributeProviders(Set renderingAttributeProviders) {
        // establish name->instance map
        renderingAttributeProviderMap=new HashMap();
        for (Iterator iter = renderingAttributeProviders.iterator(); iter.hasNext();) {
            IRenderingAttributeProvider provider = (IRenderingAttributeProvider) iter.next();
            renderingAttributeProviderMap.put(provider.getProviderName(),provider);
        }
    }
    
    public boolean canEvaluateVariables() {
        return false;
    }


    public void setBasicSyntaxProvider(IUrlBasicSyntaxProvider basicSyntaxProvider) {
        this.basicSyntaxProvider = basicSyntaxProvider;
    }



    public void setStringConcatenator(IStringConcatenator stringConcatenator) {
        this.stringConcatenator = stringConcatenator;
    }
}
