/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.hybrid;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jasig.portal.rendering.AbstractCacheEntryPoint;
import org.jasig.portal.rendering.PipelineCacheManagerImpl;

/**
 * A pipeline cache manager that combines multiple cache types by keeping
 * a set of type-specific cache managers internally.
 * For example, by adding string- and sax- pipeline cache managers to the composite
 * results in a cache manager supporting both string and sax cache types.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class CompositePipelineCacheManager extends PipelineCacheManagerImpl {
    PipelineCacheManagerImpl[] cacheManagers;
    
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.PipelineCacheManagerImpl#checkAndCreateCacheEntryPoints(java.util.List, int)
     */
    public AbstractCacheEntryPoint[] checkAndCreateCacheEntryPoints(List pipelineElements, int pipelinePosition) {
        // go through all cache managers, starting with the last one (cache type with smallest priority)
        List cacheEntryPoints=new ArrayList();
        for (int i = cacheManagers.length-1; i >=0; i--) {
            AbstractCacheEntryPoint[] p=cacheManagers[i].checkAndCreateCacheEntryPoints(pipelineElements,pipelinePosition);
            if(p!=null) {
                for (int j = 0; j < p.length; j++) {
                    cacheEntryPoints.add(p[j]);
                }
            }
        }
        if(cacheEntryPoints.size()>0) {
            return (AbstractCacheEntryPoint[]) cacheEntryPoints.toArray(new AbstractCacheEntryPoint[cacheEntryPoints.size()]);
        } else {
            return null;
        }
    }

    /**
     * Set a list of cache managers. The managers should be listed in a order
     * of cache type priority, with first manager producing cache entries that
     * should be given the highest priority, and last manager producing entries
     * that should be given the lowest priority.
     * 
     * @param cacheManagers list of {@link PipelineCacheManagerImpl} instances.
     */
    public void setCacheManagers(List cacheManagers) {
        this.cacheManagers=new PipelineCacheManagerImpl[cacheManagers.size()];
        int i=0;
        for (Iterator iter = cacheManagers.iterator(); iter.hasNext();i++) {
            PipelineCacheManagerImpl manager = (PipelineCacheManagerImpl) iter.next();
            this.cacheManagers[i]=manager;
        }
    }
}
