/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.hybrid;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jasig.portal.rendering.IPortalRenderer;
import org.jasig.portal.rendering.PipelineComponentManager;
import org.jasig.portal.rendering.PipelineMisconfigurationException;

/**
 * A pipeline component manager that can be configured with arbitrary types of components.
 * The implementation relies on a set of {@link org.jasig.portal.rendering.hybrid.IComponentConnector}s
 * to wire pipeline components together.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11918 $
 */
public class CompositePipelineComponentManager extends PipelineComponentManager {
    List pipelineElements;
    IComponentConnector[] componentConnectors;

    /**
     * Set component connectors.
     * @param componentConnectors the componentConnectors to set.
     */
    public void setComponentConnectors(List componentConnectors) {
        this.componentConnectors=new IComponentConnector[componentConnectors.size()];
        int i=0;
        for (Iterator iter = componentConnectors.iterator(); iter.hasNext();i++) {
            this.componentConnectors[i]=(IComponentConnector) iter.next();
        }
    }
    
    /**
     * Set pipeline elements. The first element must implement {@link org.jasig.portal.rendering.IPortalRenderer}
     * @param pipelineElements The pipelineElements to set.
     */
    public void setPipelineElements(List pipelineElements) throws PipelineMisconfigurationException {
        List downstreamElements=new ArrayList();
        
        Iterator i=pipelineElements.iterator();
        IPortalRenderer firstElement=(IPortalRenderer)i.next();
        Object upstream=firstElement;
        while(i.hasNext()) {
            Object element=i.next();
            if(!connectNeighbours(upstream,element)) {
                throw new PipelineMisconfigurationException("Could not connect neighbors '" + upstream + "' and '" + element + "'");
            }
            downstreamElements.add(element);
            upstream=element;
        }
        
        super.setFirstPipelineElement(firstElement);
        super.setDownstreamElements(downstreamElements);
    }
    
    
    protected boolean connectNeighbours(Object upstream, Object downstream) {
        boolean connected=false;
        for (int i = 0; i < componentConnectors.length; i++) {
            connected=connected || componentConnectors[i].connectNeighbors(upstream,downstream);
        }
        return connected;
    }
}
