/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.rendering.sax;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * ContentHandlerConsumer is a wrapper to make plain content handler objects look like valid sax consumers.
 * Note: this is especially useful for terminal components of the rendering pipelines.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class ContentHandlerConsumer implements ISaxConsumer {
    ContentHandler contentHandler;
    
    public ContentHandlerConsumer() {};
    public ContentHandlerConsumer(ContentHandler ch) {
        this.contentHandler=ch;
    }
    
    /**
     * @param ch
     * @param start
     * @param length
     * @throws org.xml.sax.SAXException
     */
    public void characters(char[] ch, int start, int length) throws SAXException {
        contentHandler.characters(ch, start, length);
    }

    /**
     * @throws org.xml.sax.SAXException
     */
    public void endDocument() throws SAXException {
        contentHandler.endDocument();
    }

    /**
     * @param namespaceURI
     * @param localName
     * @param qName
     * @throws org.xml.sax.SAXException
     */
    public void endElement(String namespaceURI, String localName, String qName) throws SAXException {
        contentHandler.endElement(namespaceURI, localName, qName);
    }

    /**
     * @param prefix
     * @throws org.xml.sax.SAXException
     */
    public void endPrefixMapping(String prefix) throws SAXException {
        contentHandler.endPrefixMapping(prefix);
    }

    /**
     * @param ch
     * @param start
     * @param length
     * @throws org.xml.sax.SAXException
     */
    public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException {
        contentHandler.ignorableWhitespace(ch, start, length);
    }

    /**
     * @param target
     * @param data
     * @throws org.xml.sax.SAXException
     */
    public void processingInstruction(String target, String data) throws SAXException {
        contentHandler.processingInstruction(target, data);
    }

    /**
     * @param locator
     */
    public void setDocumentLocator(Locator locator) {
        contentHandler.setDocumentLocator(locator);
    }

    /**
     * @param name
     * @throws org.xml.sax.SAXException
     */
    public void skippedEntity(String name) throws SAXException {
        contentHandler.skippedEntity(name);
    }

    /**
     * @throws org.xml.sax.SAXException
     */
    public void startDocument() throws SAXException {
        contentHandler.startDocument();
    }

    /**
     * @param namespaceURI
     * @param localName
     * @param qName
     * @param atts
     * @throws org.xml.sax.SAXException
     */
    public void startElement(String namespaceURI, String localName, String qName, Attributes atts) throws SAXException {
        contentHandler.startElement(namespaceURI, localName, qName, atts);
    }

    /**
     * @param prefix
     * @param uri
     * @throws org.xml.sax.SAXException
     */
    public void startPrefixMapping(String prefix, String uri) throws SAXException {
        contentHandler.startPrefixMapping(prefix, uri);
    }

    public boolean canRenderSax() {
        return true;
    }
}
