/*
 * Copyright 2004 The JA-SIG Collaborative. All rights reserved. See license distributed with this
 * file and available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.sax;

import java.io.Serializable;
import java.util.List;

import org.jasig.portal.rendering.AbstractCacheEntryPoint;
import org.jasig.portal.rendering.RenderingException;
import org.xml.sax.SAXException;

/**
 * A cache entry point, specific for {@link org.jasig.portal.rendering.sax.ISaxCacheFilter}.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11918 $
 */
public class SaxCacheEntryPoint extends AbstractCacheEntryPoint {
    ISaxCacheFilter filter;


    /**
     * Create new SAX cache entry point
     * 
     * @param filter cache filter
     * @param position position of the cache filter in the pipeline
     * @param validity validity objects
     */
    public SaxCacheEntryPoint(ISaxCacheFilter filter, int position, Serializable[] validity) {
        super(position, validity);
        this.filter = filter;
    }

    /**
     * In addition to normal validity checks that filter is connected downstream
     */
    public boolean isValid(List pipelineElements) {
        // make sure pipeline hasn't changed
        if (this.getFilter() != pipelineElements.get(this.getCacheFilterPosition())) {
            throw new IllegalStateException("filter instance is no longer found at the specified pipeline position");
        }
        return (filter.canRenderSax() && super.isValid(pipelineElements));
    }

    protected ISaxCacheFilter getFilter() {
        return this.filter;
    }

    public boolean replayCache() throws RenderingException {
        try {
            return this.getFilter().replaySaxCache();
        }
        catch (SAXException se) {
            throw new RenderingException("Failed to replay SAX event cache", se);
        }
    }
}
