/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.sax.xslt;

import java.net.URI;

import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.sax.TransformerHandler;

import org.jasig.portal.core.exception.PortalException;

/**
 * An interface describing an XSLT transformation helper
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public interface ITransformerHelper {
    /**
     * Obtain a <code>Templates</code> instance for a given stylesheet URI
     * @param stylesheetURI
     * @return stylesheet templates
     * @throws PortalException if the problem occurs
     */
    Templates getTemplates(URI stylesheetURI) throws PortalException;
    
    /**
     * Obtain a <code>Transformer</code> instance for a given stylesheet URI
     * @param stylesheetURI
     * @return transformer corresponding to the requested stylehseet
     * @throws PortalException if the problem occurs
     */
    Transformer getTransformer(URI stylesheetURI) throws PortalException;
    
    /**
     * Obtain a <code>TransformerHandler</code> instance for a given stylesheet URI
     * @param stylesheetURI
     * @return transformer corresponding to the requested stylehseet
     * @throws PortalException if the problem occurs
     */
    TransformerHandler getTransformerHandler(URI stylesheetURI) throws PortalException;
}
