/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.sax.xslt;

import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;


/**
 * Default implementation of XSLT stylesheet description
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class StylesheetDescriptionImpl implements IStylesheetDescription {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(StylesheetDescriptionImpl.class);
    
    ICacheKey currentCacheKey=null;

    URI stylesheetLocation;
    
    String mimeType=null;
    
    
    /**
     * Specify stylesheet location as a string, which to be parsed as a URL, and if that
     * doesn't work, resolved as a resource with the local classloader.
     * @param stylesheetLocation The stylesheetLocation to set.
     * @throws URISyntaxException 
     */
    public void setStylesheetLocation(String stylesheetLocation) throws URISyntaxException  {
        try {
            new URL(stylesheetLocation);
            this.stylesheetLocation=new URI(stylesheetLocation);
        } catch (MalformedURLException e){
            URL url=this.getClass().getResource(stylesheetLocation);
            if(url==null) {
                final IllegalArgumentException iae = new IllegalArgumentException("unable to find stylesheet using class resource loader : stylesheetLocation=" + stylesheetLocation, e);
                logger.error(iae.getMessage(), iae);
                throw iae;
            }
            this.stylesheetLocation = new URI(this.getClass().getResource(stylesheetLocation).toString());
        }
        resetKey();
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.xslt.IStylesheetDescription#getStylesheetLocation()
     */
    public URI getStylesheetLocationUri() {
        return stylesheetLocation;
    }
   
    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#generateKey()
     */
    public ICacheKey generateKey() {
        if(currentCacheKey==null) {
            StringBuffer key=new StringBuffer();
            
            if(stylesheetLocation!=null) {
                key.append(stylesheetLocation.toString());
            }
            
            if(mimeType!=null) {
                key.append(mimeType);
            }
            
            currentCacheKey=new CacheKeyImpl(key.toString());
        }
        return currentCacheKey;   
    }
    
    protected void resetKey() {
        this.currentCacheKey=null;
    }
    
    /* (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#isCacheValid(java.io.Serializable)
     */
    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.xslt.IStylesheetDescription#getMimeType()
     */
    public String getMimeType() {
        return this.mimeType;
    }
    /**
     * @param mimeType The mimeType to set.
     */
    public void setMimeType(String mimeType) {
        this.mimeType = mimeType;
    }

}
