/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.sax.xslt;

import java.io.IOException;
import java.net.URI;

import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXTransformerFactory;
import javax.xml.transform.sax.TemplatesHandler;
import javax.xml.transform.sax.TransformerHandler;

import org.jasig.portal.caching.ISimpleCacheProviderFacade;
import org.jasig.portal.core.exception.PortalException;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * Default implementation of the transformer helper.
 * TODO implement caching, once the interface is finalzied
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class TransformerHelperImpl implements ITransformerHelper {
    
    private static SAXTransformerFactory transformerFactory=(SAXTransformerFactory)TransformerFactory.newInstance();
    private ISimpleCacheProviderFacade templatesCache;
    
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.xslt.ITransformerHelper#getTemplates(java.net.URI)
     */
    public Templates getTemplates(URI stylesheetURI) throws PortalException {
        ISimpleCacheProviderFacade cache=getTemplatesCache();
    	Templates templates=null;
    	if(cache!=null) { 
    	    templates=(Templates) cache.getFromCache(stylesheetURI.toString());
    	}
    	if (templates == null) {
            try {
                TemplatesHandler thand = getTransformerFactory().newTemplatesHandler();
                XMLReader reader = XMLReaderFactory.createXMLReader();
                reader.setContentHandler(thand);
                reader.parse(stylesheetURI.toString());
                templates=thand.getTemplates();
                // cache templates
                if(cache!=null) {
                    cache.putInCache(stylesheetURI.toString(),templates);
                }
            } catch (IOException ioe) {
                throw new PortalException("Unable to read stylesheet from URI " + stylesheetURI.toString());
            } catch (TransformerConfigurationException tce) {
                throw new PortalException("Current TRAX configuration does not allow for TemplateHandlers. Please reconfigure/reinstall your TRAX implementation.");
            } catch (SAXParseException px) {
                throw new PortalException("Unable to parse stylesheet with URI " + stylesheetURI.toString() + " SAXParseExeption: " + px.getMessage() + " line:" + px.getLineNumber() + " col:" + px.getColumnNumber(), px);
            } catch (SAXException sx) {
                // Catch the sax exception so we can report line number info
                if (null != sx.getException() && (sx.getException() instanceof TransformerException)) {
                    TransformerException trx = (TransformerException) sx.getException();
                    throw new PortalException("Exception raised while parsing the sylesheet with URI " + stylesheetURI.toString() + "  :" + trx.getMessageAndLocation(), sx);
                }
                throw new PortalException("Exception raised while parsing the sylesheet with URI " + stylesheetURI.toString(), sx);
            }
        }
    	return templates;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.xslt.ITransformerHelper#getTransformer(java.net.URI)
     */
    public Transformer getTransformer(URI stylesheetURI) throws PortalException {
        try {
            return getTemplates(stylesheetURI).newTransformer();
        } catch (TransformerConfigurationException tce) {
            throw new PortalException("Current TRAX configuration does not allow for Transformer to be obtained from the teamplates for the stylesheet with URI"+stylesheetURI.toString(),tce);
        }
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.sax.xslt.ITransformerHelper#getTransformerHandler(java.net.URI)
     */
    public TransformerHandler getTransformerHandler(URI stylesheetURI) throws PortalException {
        try {
            return getTransformerFactory().newTransformerHandler(getTemplates(stylesheetURI));
        } catch (TransformerConfigurationException tce) {
            throw new PortalException("Current TRAX configuration does not allow for TransformerHandler to be obtained from the teamplates for the stylesheet with URI"+stylesheetURI.toString(),tce);
        }
    }
    

    /**
     * @return Returns the transformerFactory.
     */
    static SAXTransformerFactory getTransformerFactory() {
        return transformerFactory;
    }
	/**
	 * @return Returns the templatesCache.
	 */
	public ISimpleCacheProviderFacade getTemplatesCache() {
		return templatesCache;
	}
	/**
	 * Set a cache implementation in which stylesheet <code>Template</code> objects will be cached
	 * @param templatesCache The cache to put <code>Template</code> objects in, or null to turn off template caching, in which case the stylesheet will
	 * be re-parsed every time.
	 */
	public void setTemplatesCache(ISimpleCacheProviderFacade templatesCache) {
		this.templatesCache = templatesCache;
	}
}
