/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.sax.xslt;

import java.io.Serializable;

import org.jasig.portal.caching.CacheKeyListImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.rendering.attributes.IRenderingAttributeProvider;

/**
 * A default implementation of the XSLT transformer preferences
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class TransformerPreferencesImpl implements ITransformerPreferences {
    IStylesheetDescription stylesheetDescription;
    IRenderingAttributeProvider renderingAttributeProvider;
    
    public IStylesheetDescription getStylesheetDescription() {
        return this.stylesheetDescription;
    }
    
    public ICacheKey generateKey() {
        if(renderingAttributeProvider!=null) {
            ICacheKey[] keys={stylesheetDescription.generateKey(),renderingAttributeProvider.generateKey()};
            return new CacheKeyListImpl(keys);
        } else {
            return stylesheetDescription.generateKey();
        }
    }
    
    public boolean isCacheValid(Serializable validityObject) {
        if(renderingAttributeProvider!=null) {
            Serializable[] vo=(Serializable[]) validityObject;
            return renderingAttributeProvider.isCacheValid(vo[1]) && stylesheetDescription.isCacheValid(vo[0]);
        } else {
            return stylesheetDescription.isCacheValid(validityObject);
        }
    }

    public IRenderingAttributeProvider getRenderingAttributeProvider() {
        return renderingAttributeProvider;
    }

    public void setRenderingAttributeProvider(IRenderingAttributeProvider renderingAttributeProvider) {
        this.renderingAttributeProvider = renderingAttributeProvider;
    }

    public void setStylesheetDescription(IStylesheetDescription stylesheetDescription) {
        this.stylesheetDescription = stylesheetDescription;
    }
}
