/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.sax.xslt;

import java.io.Serializable;
import java.util.Map;

import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.rendering.attributes.IRenderingAttributeProvider;
import org.springframework.beans.factory.annotation.Required;
import org.xml.sax.Attributes;

/**
 * A proxy class that delegates methods to rendering attribute provider of the specified {@link org.jasig.portal.rendering.sax.xslt.ITransformerPreferences} instance.
 * 
 *  Note: because of the delegate implementaiton, this should be used only when object is not frequently accessed.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class TransformerPreferencesRenderingAttributeProvider implements IRenderingAttributeProvider {
    ITransformerPreferences transformerPreferences;
    
    public void clearAttributes() {
        getRenderingAttributeProvider().clearAttributes();
    }

    public void clearAttributes(String namespaceUri, String localName, String qName, Attributes atts) {
        getRenderingAttributeProvider().clearAttributes(namespaceUri, localName, qName, atts);
    }

    public void clearParameters() {
        getRenderingAttributeProvider().clearParameters();
    }

    public ICacheKey generateKey() {
        return getRenderingAttributeProvider().generateKey();
    }

    public Map getAttributeValues(String namespaceUri, String localName, String qName, Attributes atts) {
        return getRenderingAttributeProvider().getAttributeValues(namespaceUri, localName, qName, atts);
    }

    public Map getParameterValues() {
        return getRenderingAttributeProvider().getParameterValues();
    }

    public String getProviderName() {
        return getRenderingAttributeProvider().getProviderName();
    }

    public boolean isCacheValid(Serializable validityObject) {
        return getRenderingAttributeProvider().isCacheValid(validityObject);
    }

    public boolean removeAttribute(String attributeName, String namespaceUri, String localName, String qName, Attributes atts) {
        return getRenderingAttributeProvider().removeAttribute(attributeName, namespaceUri, localName, qName, atts);
    }

    public boolean removeParameter(String parameterName) {
        return getRenderingAttributeProvider().removeParameter(parameterName);
    }

    public void setAttributeValue(String attributeName, String attributeValue, String namespaceUri, String localName, String qName, Attributes atts) {
        getRenderingAttributeProvider().setAttributeValue(attributeName, attributeValue, namespaceUri, localName, qName, atts);
    }

    public void setParameterValue(String parameterName, String parameterValue) {
        getRenderingAttributeProvider().setParameterValue(parameterName, parameterValue);
    }

    /**
     * @return Returns the renderingAttributeProvider.
     */
    IRenderingAttributeProvider getRenderingAttributeProvider() {
        return this.transformerPreferences.getRenderingAttributeProvider();
    }

    
    /**
     * Specify transformer preferences from which the attribute provider should be acquired
     * @param transformerPreferences
     */
    @Required
    public void setTransformerPreferences(ITransformerPreferences transformerPreferences) {
        this.transformerPreferences = transformerPreferences;
    }


}
