/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.rendering.sax.xslt.legacy;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.jasig.portal.rendering.attributes.IRenderingAttributeProviderCommands;
import org.jasig.portal.url.IUrlConstructor;
import org.jasig.portal.url.UrlInfo;
import org.xml.sax.Attributes;

/**
 * TransformationFilterLegacyCommandsUrlConstructor is a URL constructor implementing {@link IRenderingAttributeProviderCommands}.
 * Handles hard-coded "structure" and "theme" processor names.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class TransformationFilterLegacyCommandsUrlConstructor implements IRenderingAttributeProviderCommands, IUrlConstructor {
    
    // temp list for maintaining parameter name/values
    List parameterValues=new ArrayList();

    public void setAttributeValue(String providerName, String attributeName, String attributeValue, String namespaceUri, String localName, String qName, Attributes atts) {
        String attribute_postfix=TransformationFilterLegacyRequestParameterProcessor.FOLDER_ATTRIBUTE_POSTFIX;
        String id_postfix=TransformationFilterLegacyRequestParameterProcessor.FOLDERID_POSTFIX;
        // check if it's a channel
        String elementName=localName; if(elementName==null) { elementName=qName; }
        if(TransformationFilterLegacyRequestParameterProcessor.CHANNEL_QNAME.equals(elementName)) {
            attribute_postfix=TransformationFilterLegacyRequestParameterProcessor.CHANNEL_ATTRIBUTE_POSTFIX;
            id_postfix=TransformationFilterLegacyRequestParameterProcessor.CHANNELID_POSTFIX;
        }
        String id=atts.getValue(TransformationFilterLegacyRequestParameterProcessor.ID_ATTRIBUTE_NAME);
        
        parameterValues.add(new StringTuple(TransformationFilterLegacyRequestParameterProcessor.UP_PREFIX+getProviderCode(providerName)+attribute_postfix,attributeName));
        parameterValues.add(new StringTuple(attributeName+id_postfix,id));
        parameterValues.add(new StringTuple(attributeName+TransformationFilterLegacyRequestParameterProcessor.NAME_ID_SEPARATOR+TransformationFilterLegacyRequestParameterProcessor.VALUE_POSTFIX,attributeValue));
    }

    public void setParameterValue(String providerName, String paramName, String paramValue) {
        parameterValues.add(new StringTuple(TransformationFilterLegacyRequestParameterProcessor.UP_PREFIX+getProviderCode(providerName)+TransformationFilterLegacyRequestParameterProcessor.PARAM_POSTFIX,paramName));
        parameterValues.add(new StringTuple(paramName,paramValue));
    }
    
    protected String getProviderCode(String providerName) {
        if("structure".equals(providerName)) {
            return "s";
        } else if("theme".equals(providerName)) {
            return "t";
        } else {
            throw new IllegalArgumentException("processorName has to be 'structure' or 'theme'");
        }
    }

    public void writeToUrl(UrlInfo urlInfo) {
        for (Iterator iter = parameterValues.iterator(); iter.hasNext();) {
            StringTuple tuple = (StringTuple) iter.next();
            urlInfo.addParameterValue(tuple.parameter,tuple.value);
        }
    }
    
    static class StringTuple {
        public String parameter;
        public String value;
        public StringTuple(String parameter, String value) {
            this.parameter = parameter;
            this.value = value;
        }
    }
    
    public void clearAttributes(String providerName, String namespaceUri, String localName, String qName, Attributes atts) {
        throw new UnsupportedOperationException("method not implemented - no legacy syntax exisits for this command");
    }

    public void clearAttributes(String providerName) {
        throw new UnsupportedOperationException("method not implemented - no legacy syntax exisits for this command");
    }

    public void clearParameters(String providerName) {
        throw new UnsupportedOperationException("method not implemented - no legacy syntax exisits for this command");
    }

    public boolean removeAttribute(String providerName, String attributeName, String namespaceUri, String localName, String qName, Attributes atts) {
        throw new UnsupportedOperationException("method not implemented - no legacy syntax exisits for this command");
    }

    public boolean removeParameter(String providerName, String parameterName) {
        throw new UnsupportedOperationException("method not implemented - no legacy syntax exisits for this command");
    }

}
