/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.sax.xslt.legacy.persistance;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.rendering.attributes.IAttributeDescription;
import org.jasig.portal.rendering.attributes.IRenderingAttributesDescription;
import org.jasig.portal.rendering.attributes.RenderingAttributesDescriptionImpl;
import org.jasig.portal.rendering.sax.xslt.legacy.ILegacyStylesheetDescription;
import org.springframework.beans.factory.annotation.Required;

/**
 * Exposes rendering attribute descriptions (combined params, channel and folder) given {@link org.jasig.portal.rendering.sax.xslt.legacy.ILegacyStylesheetDescription} 
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class LegacyRenderingAttributeDescriptionAdapter implements IRenderingAttributesDescription {
    IRenderingAttributesDescription rap;
    
    @Required
    public void setLegacyStylesheetDescription(ILegacyStylesheetDescription legacyStylesheetDescription) {
        Set<IRenderingAttributesDescription> allDescriptions=new HashSet<IRenderingAttributesDescription>();
        allDescriptions.add(legacyStylesheetDescription.getParamsDescription());
        allDescriptions.add(legacyStylesheetDescription.getFolderAttributesDescription());
        allDescriptions.add(legacyStylesheetDescription.getChannelAttributesDescription());
        this.rap=new RenderingAttributesDescriptionImpl(allDescriptions);
    }
    
    public ICacheKey generateKey() {
        return rap.generateKey();
    }
    public IAttributeDescription getElementAttributeDescription(String attributeName) {
        return rap.getElementAttributeDescription(attributeName);
    }
    public Set getElementAttributeNames() {
        return rap.getElementAttributeNames();
    }
    public IAttributeDescription getParamDescription(String name) {
        return rap.getParamDescription(name);
    }
    public Set getParamNames() {
        return rap.getParamNames();
    }
    public boolean isCacheValid(Serializable validityObject) {
        return rap.isCacheValid(validityObject);
    }
    
}
