/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.sax.xslt.legacy.persistance;

import java.io.Serializable;
import java.net.URI;

import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.rendering.attributes.IRenderingAttributesDescription;
import org.jasig.portal.rendering.sax.xslt.legacy.ILegacyStylesheetDescription;

/**
 * A self-persistent legacy stylesheet description.
 * 
 * Note: a simple class that delegates all {@link org.jasig.portal.rendering.sax.xslt.legacy.ILegacyStylesheetDescription}
 * calls to an instance obtained from the specified {@link org.jasig.portal.rendering.sax.xslt.legacy.persistance.ILegacyStylesheetDescriptionDao}.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PersistentLegacyStylesheetDescription implements ILegacyStylesheetDescription {
    // injected properties
    ILegacyStylesheetDescriptionDao legacyStylesheetDescriptionDao;
    String stylesheetDescriptionLocation;
    String stylesheetLocation;

    protected ILegacyStylesheetDescription legacyStylesheetDescription;


    public void setStylesheetDescriptionLocation(String stylesheetDescriptionLocation) {
        this.stylesheetDescriptionLocation = stylesheetDescriptionLocation;
    }

    public void setStylesheetLocation(String stylesheetLocation) {
        this.stylesheetLocation = stylesheetLocation;
    }
    
    public void setLegacyStylesheetDescriptionDao(ILegacyStylesheetDescriptionDao legacyStylesheetDescriptionDao) {
        this.legacyStylesheetDescriptionDao = legacyStylesheetDescriptionDao;
    }

    protected ILegacyStylesheetDescription getLegacyStylesheetDescription() {
        if(legacyStylesheetDescription==null) {
            // read in stylesheet description from DAO
            legacyStylesheetDescription=legacyStylesheetDescriptionDao.getLegacyStylesheetDescription(stylesheetDescriptionLocation,stylesheetLocation);
        }
        return legacyStylesheetDescription;
    }

    
    // delegate methods
    
    public ICacheKey generateKey() {
        return getLegacyStylesheetDescription().generateKey();
    }

    public String getDescription() {
        return getLegacyStylesheetDescription().getDescription();
    }

    public String getDeviceType() {
        return getLegacyStylesheetDescription().getDeviceType();
    }

    public String getMimeType() {
        return getLegacyStylesheetDescription().getMimeType();
    }

    public String getName() {
        return getLegacyStylesheetDescription().getName();
    }

    public String getParentStructureStylesheet() {
        return getLegacyStylesheetDescription().getParentStructureStylesheet();
    }

    public String getSampleIconUri() {
        return getLegacyStylesheetDescription().getSampleIconUri();
    }

    public String getSamplePictureUri() {
        return getLegacyStylesheetDescription().getSamplePictureUri();
    }

    public String getSerializer() {
        return getLegacyStylesheetDescription().getSerializer();
    }

    public URI getStylesheetLocationUri() {
        return getLegacyStylesheetDescription().getStylesheetLocationUri();
    }

    public String getUserPreferencesModuleClass() {
        return getLegacyStylesheetDescription().getUserPreferencesModuleClass();
    }

    public boolean isCacheValid(Serializable validityObject) {
        return getLegacyStylesheetDescription().isCacheValid(validityObject);
    }

    public IRenderingAttributesDescription getChannelAttributesDescription() {
        return getLegacyStylesheetDescription().getChannelAttributesDescription();
    }

    public IRenderingAttributesDescription getFolderAttributesDescription() {
        return getLegacyStylesheetDescription().getFolderAttributesDescription();
    }

    public IRenderingAttributesDescription getParamsDescription() {
        return getLegacyStylesheetDescription().getParamsDescription();
    }
}
