/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.rendering.string;

import java.io.UnsupportedEncodingException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.oro.text.regex.MalformedPatternException;
import org.apache.oro.text.regex.MatchResult;
import org.jasig.portal.portlet.rendering.IPortletRenderContent;
import org.jasig.portal.portlet.rendering.IPortletWindowManager;
import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * PortletContentIncorporationFilter description
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletContentIncorporationFilter extends BasePortletFilter {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PortletContentIncorporationFilter.class);
    
    /**
     * Default regular expression matching portlet element and portlet window id as a first group
     */
    //public final static String DEFAULT_PORTLET_ELEMENT_REGEX="<portlet[^>]+portletWindowId=\"(\\w+)\"[^>]*>";
    //  TODO: move to the portlet default above
    public final static String DEFAULT_PORTLET_ELEMENT_REGEX="<channel[^>]+portletWindowId=\"([\\w\\.]+)\"[^>]*>";
    
    public PortletContentIncorporationFilter() {
        super();
        try {
            setRegex(DEFAULT_PORTLET_ELEMENT_REGEX);
        } catch (MalformedPatternException e) {
            logger.error("PortletContentIncorporationFilter() - failed to set default portlet id regex pattern: \""+DEFAULT_PORTLET_ELEMENT_REGEX+"\"", e);
        }
    }
    
    public void processMatch(MatchResult match) {
        if (logger.isDebugEnabled()) {
            logger.debug("processMatch() - matched portletWindowId \""+match.group(1)+"\"");
        }
        // obtain id in an object form
        // TODO: define an interface for id factory, otherwise what's the point of having an interface id, if one can only create a specific implementation?
        PortletWindowId windowId=new PortletWindowId(match.group(1));
        // obtain portlet content
        IPortletWindowManager windowManager=getPortletWindowManager();
        IPortletRenderContent content=windowManager.getPortletRenderContent(windowId);
        
        String contentString="";
        try {
            contentString = content.getCharacterOutput();
        } catch (UnsupportedEncodingException e) {
            //TODO: replace by error portlet
            logger.error("processMatch() - can not translate portlet content to string : windowId = " + windowId, e);
        }
        // perform substitution
        StringBuffer b=this.getContentBuffer();
        b.replace(match.beginOffset(0),match.endOffset(0),contentString);
    }
    
    /**
     * Set a custom portlet id pattern. Pattern must be such that the first group in the
     * match will correspond to a portlet id.
     * @param portletIdPattern The portletIdPattern to set.
     */
    public void setPortletIdPattern(String portletIdPattern) throws MalformedPatternException {
        setRegex(portletIdPattern);
    }

}
