/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.rendering.string;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.oro.text.regex.MalformedPatternException;
import org.apache.oro.text.regex.MatchResult;
import org.jasig.portal.portlet.rendering.IPortletWindowManager;
import org.jasig.portal.portlet.window.PortletWindowId;

/**
 * PortletRenderingInitiationFilter is a filter that will pick out portlet window ids from the content string
 * and will make calls to the portlet window manager to initiate rendering.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11955 $
 */
public class PortletRenderingInitiationFilter extends BasePortletFilter {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PortletRenderingInitiationFilter.class);
    
    
    /**
     * Default regular expression matching portlet element and portlet window id as a first group
     */
    //public final static String DEFAULT_PORTLET_WINDOW_ID_REGEX="<portlet[^>]+portletWindowId=\"(\\w+)\"";
    // TODO: move to the portlet default above
    public final static String DEFAULT_PORTLET_WINDOW_ID_REGEX="<channel[^>]+portletWindowId=\"([\\w\\.]+)\"";
    
    public PortletRenderingInitiationFilter() {
        super();
        try {
            setRegex(DEFAULT_PORTLET_WINDOW_ID_REGEX);
        } catch (MalformedPatternException e) {
            logger.error("PortletRenderingInitiationFilter() - failed to set default portlet id regex pattern: \""+DEFAULT_PORTLET_WINDOW_ID_REGEX+"\"", e);
        }
    }
    
    public void processMatch(MatchResult match) {
        if (logger.isDebugEnabled()) {
            logger.debug("processMatch() - matched portletWindowId \""+match.group(1)+"\"");
        }
        // obtain id in an object form
        // TODO: define an interface for id factory, otherwise what's the point of having an interface id, if one can only create a specific implementation?
        PortletWindowId windowId=new PortletWindowId(match.group(1));
        // initiate portlet window rendering
        IPortletWindowManager windowManager=getPortletWindowManager();
        windowManager.startPortletRendering(windowId);
    }
    
    /**
     * Set a custom portlet id pattern. Pattern must be such that the first group in the
     * match will correspond to a portlet id.
     * @param portletIdPattern The portletIdPattern to set.
     */
    public void setPortletIdPattern(String portletIdPattern) throws MalformedPatternException {
        setRegex(portletIdPattern);
    }
 
}
