/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.rendering.string;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.oro.text.regex.MalformedPatternException;
import org.apache.oro.text.regex.MatchResult;
import org.jasig.portal.portlet.rendering.IPortletRenderContent;
import org.jasig.portal.portlet.rendering.IPortletWindowManager;
import org.jasig.portal.portlet.window.PortletWindowId;
import org.jasig.portal.rendering.sax.MockPortletTitleInjector;

/**
 * Replaces mock portlet title with the title from the portets' rendered content  
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 1.0$
 */
public class PortletTitleIncorporationFilter extends BasePortletFilter {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(PortletTitleIncorporationFilter.class);

    private static final String DEFAULT_TITLE_PATTERN="";
    private String titlePattern=MockPortletTitleInjector.DEFAULT_TITLE_PATTERN;

    public PortletTitleIncorporationFilter() {
        super();
        try {
            setRegex(titlePattern);
        } catch (MalformedPatternException e) {
            logger.error("PortletTitleIncorporationFilter() - failed to set default title regex pattern: \""+this.titlePattern+"\"", e);
        }
    }
    
    public void processMatch(MatchResult match) {

        PortletWindowId windowId=new PortletWindowId(match.group(1));
        // obtain portlet content
        IPortletWindowManager windowManager=getPortletWindowManager();
        IPortletRenderContent content=windowManager.getPortletRenderContent(windowId);
        
        String titleString = content.getTitle();
        if (logger.isDebugEnabled()) {
            logger.debug("processMatch() - matched portletWindowId \""+match.group(1)+"\", title=\""+titleString+"\"");
        }
        // perform substitution
        StringBuffer b=this.getContentBuffer();
        if(titleString==null) { titleString=DEFAULT_TITLE_PATTERN; }
        if(b!=null) {
            b.replace(match.beginOffset(0),match.endOffset(0),titleString);
        }
    }

    /**
     * Specify title pattern. 
     * Defaults to {@link org.jasig.portal.rendering.sax.MockPortletTitleInjector}.DEFAULT_TITLE_PATTERN
     * @param titlePattern
     * @throws MalformedPatternException 
     */
    public void setTitlePattern(String titlePattern) throws MalformedPatternException {
        this.titlePattern = titlePattern;
        super.setRegex(titlePattern);
    }
    
}
