/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.string;

import java.io.Serializable;
import java.util.List;

import org.jasig.portal.rendering.AbstractCacheEntryPoint;

/**
 * A cache entry point describing {@link IStringCacheFilter}.
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11917 $
 */
public class StringCacheEntryPoint extends AbstractCacheEntryPoint {
    IStringCacheFilter filter;
    
    public StringCacheEntryPoint(IStringCacheFilter filter,int position, Serializable[] validity) {
        super(position,validity);
        this.filter=filter;
    }

    /**
     * In addition to normal validity checks performed by the {@link AbstractCacheEntryPoint},
     * this implementation makes sure that subsequent pipeline element implements
     * {@link IStringFilter}. 
     */
    public boolean isValid(List pipelineElements) {
        // make sure pipeline hasn't changed
        if(this.getFilter()!=pipelineElements.get(this.getCacheFilterPosition())) {
            throw new IllegalStateException("filter instance is no longer found at the specified pipeline position");
        }
        return (filter.canRenderString() && super.isValid(pipelineElements)); 
    }
    
    public boolean replayCache() {
        return this.getFilter().replayStringCache();
    }
    
    protected IStringCacheFilter getFilter() {
        return filter;
    }
}