/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.string;

import java.io.Serializable;
import java.util.List;

import org.jasig.portal.rendering.AbstractCacheEntryPoint;
import org.jasig.portal.rendering.PipelineCacheManagerImpl;

/**
 * A cache manager designed to work exclusively with {@link IStringCacheFilter}s.
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class StringPipelineCacheManager extends PipelineCacheManagerImpl {
    
    
    /* (non-Javadoc)
     * @see org.jasig.portal.rendering.PipelineCacheManagerImpl#checkAndCreateCacheEntryPoint(java.util.List, int)
     */
    public AbstractCacheEntryPoint[] checkAndCreateCacheEntryPoints(List pipelineElements, int pipelinePosition) {
        Object element=pipelineElements.get(pipelinePosition);
        if(element instanceof IStringCacheFilter) {
            IStringCacheFilter filter=(IStringCacheFilter)element;
            if(filter.canRenderString()) {
                Serializable[] cachedValidityObjects=filter.getStringCacheValidityObjects();
                if(cachedValidityObjects!=null) { // cache entry exists for the current key
                    AbstractCacheEntryPoint[] entryPoints={new StringCacheEntryPoint(filter,pipelinePosition,cachedValidityObjects)};
                    return entryPoints;
                }
            }
        }
        return null;
    }
}
