/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.rendering.string;

import java.io.IOException;

import javax.servlet.http.HttpServletResponse;

import org.jasig.portal.core.exception.PortalException;
import org.jasig.portal.core.exception.PortalRuntimeException;
import org.jasig.portal.rendering.IRenderingPipelineEventListener;
import org.jasig.portal.rendering.RenderingPipelineEvent;

/**
 * A component that can be used to output string content into the http response (i.e. designed to be last component in string rendering pipelines)
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11918 $
 */
public class StringResponseWriter implements IStringPipelineDownstreamComponent, IRenderingPipelineEventListener {
    HttpServletResponse currentResponse;
    
    public void filterRendering(String input) {
        if(currentResponse!=null) {
            try {
                currentResponse.getWriter().write(input);
            } catch (IOException ioe) {
                throw new PortalRuntimeException("error encountered writing content to servlet response output stream",ioe);
            }
        }
    }

    public void receiveSAXPipelineEvent(RenderingPipelineEvent ev) throws PortalException {
        if(ev.getEventType()==RenderingPipelineEvent.startRenderingCycleEventType) {
            this.currentResponse=ev.getHttpServletResponse();
        } else if(ev.getEventType()==RenderingPipelineEvent.endRenderingCycleEventType) {
            this.currentResponse=null;
        }
    }

    public boolean canRenderString() {
        return currentResponse!=null;
    }

}
