/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.security;

import java.security.Principal;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * The user implementation.
 * @author Michael Ivanov, mvi@immagic.com
 * @version $Revision: 11955 $
 */
public class UserImpl implements IUser {
    private final Principal principal = new UserPrincipal(this);
    private UserId userId;
    private String userName;

    /**
     * @see org.jasig.portal.security.IUser#setId(org.jasig.portal.security.UserId)
     */
    public void setId(UserId userId) {
        this.userId = userId;
    }

    /**
     * @see org.jasig.portal.security.IUser#getId()
     */
    public UserId getId() {
        return userId;
    }

    /**
     * @see org.jasig.portal.security.IUser#getUserName()
     */
    public String getUserName() {
        return this.userName;
    }

    /**
     * @see org.jasig.portal.security.IUser#setUserName(java.lang.String)
     */
    public void setUserName(String userName) {
        this.userName = userName;
    }

    /**
     * @see org.jasig.portal.security.IUser#getPrincipal()
     */
    public Principal getPrincipal() {
        return this.principal;
    }
    
    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(188054681, -442757043)
            .append(this.userId)
            .append(this.userName)
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return new ToStringBuilder(this).appendSuper(super.toString())
            .append("id", this.getId())
            .append("userName", this.userName)
            .toString();
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(final Object object) {
        if (object == this) {
            return true;
        }
        if (!(object instanceof UserImpl)) {
            return false;
        }
        
        UserImpl rhs = (UserImpl)object;
        return new EqualsBuilder()
            .append(this.userId, rhs.userId)
            .append(this.userName, rhs.userName)
            .isEquals();
    }
}