/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.security;

import java.security.Principal;

/**
 * An implementation of Principal that wraps an IUser object to provide the Principal
 * information.
 * 
 * @author Eric Dalquist <a href="mailto:eric.dalquist@doit.wisc.edu">eric.dalquist@doit.wisc.edu</a>
 * @version $Revision$
 */
public class UserPrincipal implements Principal {
    private final IUser user;
    
    public UserPrincipal(final IUser user) {
        if (user == null) {
            throw new IllegalArgumentException("IUser may not be null");
        }

        this.user = user;
    }

    /**
     * @see java.security.Principal#getName()
     */
    public String getName() {
        return this.user.getUserName();
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }
        if (!(obj instanceof Principal)) {
            return false;
        }
        
        final String thisName = this.getName();
        
        final Principal p = (Principal)obj;
        final String otherName = p.getName();

        return thisName == otherName || (thisName != null && thisName.equals(otherName));
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    @Override
    public int hashCode() {
        final String name = this.getName();
        if (name == null) {
            return 0;
        }
        else {
            return name.hashCode();
        }
    }

    /**
     * @see java.lang.Object#toString()
     */
    @Override
    public String toString() {
        final String name = this.getName();
        if (name == null) {
            return "null";
        }
        else {
            return name;
        }
    }
}
