/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.security.acegi;

import javax.sql.DataSource;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.support.JdbcDaoSupport;

/**
 * An implementation of password setter that's compatible with
 * {@link org.acegisecurity.userdetails.jdbc.JdbcDaoImpl}.
 * 
 * Note that it does not allow to add users.
 * 
 * Default implementation uses {@link org.acegisecurity.providers.encoding.PlaintextPasswordEncoder}
 * with no salt.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class AcegiJdbcDaoPasswordSetter extends JdbcDaoSupport implements IPasswordSetter {
    public static final String DEF_SET_PASSWORD_QUERY = "SELECT username,password,enabled FROM users WHERE username = ?";
    
    String usersTableName="users";
    String updatePasswordQuery="UPDATE "+usersTableName+" SET password=? WHERE username=?";
    
    public void setPassword(String principal, String encodedPassword) {
        DataSource ds=getDataSource();
        JdbcTemplate jt = new JdbcTemplate(ds);
        jt.update(updatePasswordQuery, new Object[] {encodedPassword, principal});

        if (logger.isInfoEnabled()) {
            logger.info("setPassword(String, String) - updated password for principal=" + principal);
        }
    }
    /**
     * Set users table name
     * @param usersTableName
     */
    public void setUsersTableName(String usersTableName) {
        this.usersTableName = usersTableName;
    }
    /**
     * Specify the query by which update should be made (with two variabes - encodedPassword,principal)
     * @param updatePasswordQuery
     */
    public void setUpdatePasswordQuery(String updatePasswordQuery) {
        this.updatePasswordQuery = updatePasswordQuery;
    }
}
