/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.security.acegi;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.acegisecurity.context.SecurityContext;
import org.acegisecurity.context.SecurityContextHolder;

/**
 *  Takes ACEGI context from the ContextHolder and places it
 *  into a request attribute. It also initializes {@link org.jasig.portal.security.acegi.TrackingContextHolder}
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class HttpRequestContextIntegrationFilter implements Filter {
    public final static String ACEGI_CONTEXT_ATTRIBUTE_NAME="ACEGI.context";
   

    public void doFilter(ServletRequest req, ServletResponse res, FilterChain chain) throws IOException, ServletException {
        setContext(SecurityContextHolder.getContext(),req);
        chain.doFilter(req,res);
    }

    public static void setContext(SecurityContext context, ServletRequest req) {
        TrackingContextHolder.setContext(context);
        req.setAttribute(ACEGI_CONTEXT_ATTRIBUTE_NAME,context);
    }
    
    public static SecurityContext getContext(ServletRequest req) {
        return (SecurityContext) req.getAttribute(ACEGI_CONTEXT_ATTRIBUTE_NAME);
    }
    
    public void init(FilterConfig arg0) throws ServletException {}
    public void destroy() {}
}
