/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.security.acegi;

import org.acegisecurity.ConfigAttributeDefinition;
import org.aopalliance.intercept.MethodInvocation;

/**
 * An interface for a helper class that picks out a target on which
 * the authorization check should be performed from a method invocation.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public interface IPermissionTargetResolver {
    
    /**
     * Determine target (usually id of a domain object) from configuration parameters or method
     * invocation.
     * @param methodInvocation intercepted method description
     * @param config configuration attributes for the current intercept
     * @return a valid target id, or <code>null</code> if valid target can not be found
     * among the method invocation arguments.
     */
    String getTarget(MethodInvocation methodInvocation,ConfigAttributeDefinition config) throws TargetResolutionException;

}
