/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.security.dao.springjdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.security.IUser;
import org.jasig.portal.security.UserId;
import org.jasig.portal.security.UserImpl;
import org.jasig.portal.security.dao.IUserDao;
import org.springframework.jdbc.core.SqlParameter;
import org.springframework.jdbc.core.support.JdbcDaoSupport;
import org.springframework.jdbc.object.MappingSqlQuery;

/**
 * Spring JDBC implementation of the user DAO
 * @author Michael Ivanov, mivanov@unicon.net
 * @version $Revision: 11955 $
 */
public class UserDaoImpl extends JdbcDaoSupport implements IUserDao {

    protected final static Log log = LogFactory.getLog(UserDaoImpl.class);

    private final static String UP_USER = "UP_USER";
    private final static String USER_ID = "ID";
    private final static String USER_NM = "USER_NM";
    private final static String USER_QUERY_BY_ID = "SELECT " + USER_ID + "," + USER_NM + " FROM " + UP_USER + " WHERE " + USER_ID + "=?";
    private final static String USER_QUERY_BY_NAME = "SELECT " + USER_ID + "," + USER_NM + " FROM " + UP_USER + " WHERE " + USER_NM + "=?";

    private UserQuery userQueryById;
    private UserQuery userQueryByName;

    /**
     * Subclasses can override this for custom initialization behavior.
     * Gets called after population of this instance's bean properties.
     *
     * @throws Exception if initialization fails
     */
    protected void initDao() throws Exception {
        super.initDao();
        DataSource ds = getDataSource();
        userQueryById = new UserQuery(ds, USER_QUERY_BY_ID);
        userQueryByName = new UserQuery(ds, USER_QUERY_BY_NAME);
    }

    /**
     * Persists the user.
     * @param user
     */
    public void setUser(IUser user) {
        throw new UnsupportedOperationException("This method call has no effect!");
    }

    /**
     * Gets the user object.
     * @param user ID
     * @return IUser intance
     */
    public IUser getUser(UserId userId) {
        return (IUser)userQueryById.findObject(userId.toString());
    }

    /**
     * Gets the user object.
     * @param user login
     * @return IUser intance
     */
    public IUser getUser(String userLogin) {
        return (IUser)userQueryByName.findObject(new Object[] { userLogin });
    }

    protected class UserQuery extends MappingSqlQuery {

        public UserQuery(DataSource ds, String sql) {
            super(ds, sql);
            if (sql.equals(USER_QUERY_BY_ID)) {
                super.declareParameter(new SqlParameter("user id", Types.VARCHAR));
            }
            else if (sql.equals(USER_QUERY_BY_NAME)) {
                super.declareParameter(new SqlParameter("user name", Types.VARCHAR));
            }
            compile();
        }

        protected Object mapRow(ResultSet rs, int rowNum) throws SQLException {
            final IUser user = new UserImpl();
            user.setUserName(rs.getString(USER_NM));
            
            final UserId userId = new UserId(rs.getString(USER_ID));
            user.setId(userId);
            
            return user;
        }
    }
}
