/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.groups;

import java.util.Map;

import javax.portlet.WindowState;

import org.jasig.gap.groups.IEntity;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Returns an xml element for a given IEntity or IEntity key.
 * @author Don Fracapane
 * @version $Revision: 11911 $
 */
public class EntityWrapper extends GroupMemberWrapper {
    
   // IoC injection
   private GroupsManagerXml groupsManagerXml;

   /** Creates new EntityWrapper */
   public EntityWrapper () {
      ELEMENT_TAGNAME = ENTITY_TAGNAME;
   }

   /**
    * Returns an xml element for a given IEntity.
    * @param gm IGroupMember
    * @param anElem Element
    * @param sessionData CGroupsManagerUnrestrictedSessionData
    * @return Element
    */
   public Element getXml (IGroupMember gm, Element anElem, GroupsSessionData sessionData) {
      Document aDoc = sessionData.getModel();
      Element rootElem = (anElem != null ? anElem : GroupsManagerXml.createElement(ELEMENT_TAGNAME,
            aDoc, false));
      Utility.logMessage("DEBUG", "EntityWrapper.getXml(): START, Element: " + rootElem);
      try {
         IEntity ent = (IEntity) gm;
         rootElem.setAttribute("id", GroupsManagerXml.getNextUid());
         rootElem.setAttribute("key", gm.getKey());
         rootElem.setAttribute("type", gm.getLeafType().getName());
         rootElem.setAttribute("displayName", groupsManagerXml.getEntityName(ent.getLeafType(),
               ent.getKey()));
         rootElem.setAttribute("selected", "false");
         
         IEntityActionUrlParts entityActionUrlParts = sessionData.getEntityActionUrlParts();
         if (entityActionUrlParts != null) {
             Element entityActionUrlElem = aDoc.createElement("entityActionUrl");
             rootElem.appendChild(entityActionUrlElem);
             
	         Map<String, String> parameters = entityActionUrlParts.getPortletParameters(gm);
             if (parameters != null) {
                 Element parametersElem = aDoc.createElement("parameters");
                 entityActionUrlElem.appendChild(parametersElem);
                 for (String name : parameters.keySet()) {
                     Element paramElem = aDoc.createElement("param");
                     parametersElem.appendChild(paramElem);
			         paramElem.setAttribute("name", name);
                     paramElem.setAttribute("value", parameters.get(name));
                 }
             }
             WindowState windowState = entityActionUrlParts.getWindowState(gm);
             if (windowState != null) {
                 Element windowStateElem = aDoc.createElement("windowState");
                 windowStateElem.setAttribute("state", windowState.toString());
                 entityActionUrlElem.appendChild(windowStateElem);
             }
             ActionUrlType actionUrlType = entityActionUrlParts.getActionUrlType(gm);
             if (actionUrlType == null) {
                 actionUrlType = ActionUrlType.normal;
             }
             Element actionUrlTypeElem = aDoc.createElement("actionUrlType");
             actionUrlTypeElem.setAttribute("type", actionUrlType.toString());
             entityActionUrlElem.appendChild(actionUrlTypeElem);
         }
         
         // set user permissions for entity
         IGroupsManagerPermissions gmp = sessionData.getPermissions();
         IAuthorizationPrincipal ap = sessionData.getAuthorizationPrincipal();
         applyPermissions (rootElem, gm, gmp, ap);
      } catch (Exception e) {
         Utility.logMessage("ERROR", "EntityWrapper.getXml(): ERROR retrieving entity "
               + e, e);
      }
      return  rootElem;
   }

    /**
    * Returns a GroupMember for a key.
    * @param aKey String
    * @param aType String
    * @return IGroupMember
    */
   protected IGroupMember retrieveGroupMember (String aKey, String aType) {
      return (IGroupMember) Utility.retrieveEntity(aKey, aType, groupsManagerXml.getGroupsManager().getCompositeGroupService());
   }
   
/**
 * @param groupsManagerXml The groupsManagerXml to set.
 */
public void setGroupsManagerXml(GroupsManagerXml groupsManagerXml) {
    this.groupsManagerXml = groupsManagerXml;
}

}

