/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.groups;

import org.jasig.gap.groups.IEntityGroup;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * Returns an xml element for a given IEntityGroup or IEntityGroup key.
 * @author Don Fracapane
 * @version $Revision: 11911 $
 */
public class GroupWrapper extends GroupMemberWrapper {
    
   private boolean limitRetrievals = false;
   private int retrievalLimit = -1;
   
   // IoC injection
   private GroupsManagerXml groupsManagerXml;
   
   /** Creates new GroupWrapper */
   public GroupWrapper () {
      super();
      ELEMENT_TAGNAME = GROUP_TAGNAME;
   }
   

    /**
     * @return Returns the limitRetrievals.
     */
    public boolean isLimitRetrievals() {
        return this.limitRetrievals;
    }
    /**
     * @param limitRetrievals The limitRetrievals to set.
     */
    public void setLimitRetrievals(boolean limitRetrievals) {
        this.limitRetrievals = limitRetrievals;
    }

    /**
     * @return Returns the retrievalLimit.
     */
    public int getRetrievalLimit() {
        return this.retrievalLimit;
    }
    /**
     * @param retrievalLimit The retrievalLimit to set.
     */
    public void setRetrievalLimit(int retrievalLimit) {
        this.retrievalLimit = retrievalLimit;
    }


   /**
    * Returns an xml element for a given IEntityGroup.
    * @param gm IGroupMember
    * @param anElem Element
    * @param sessionData CGroupsManagerUnrestrictedSessionData
    * @return Element
    */
   public Element getXml (IGroupMember gm, Element anElem, GroupsSessionData sessionData) {
      Document aDoc = sessionData.getModel();
      String nextID;
      IEntityGroup entGrp = (IEntityGroup)gm;
      Element rootElem = (anElem != null ? anElem : GroupsManagerXml.createElement(ELEMENT_TAGNAME, aDoc, false));
      Utility.logMessage("DEBUG", "GroupWrapper::getXml(): START, Element Expanded: " + rootElem.getAttribute("expanded"));
      try {
         String uid = rootElem.getAttribute("id");
         if (Utility.areEqual(uid, "")) {
            nextID = GroupsManagerXml.getNextUid();
            rootElem.setAttribute("id", nextID);
         }
         rootElem.setAttribute("key", gm.getKey());
         rootElem.setAttribute("entityType",gm.getLeafType().getName());
         rootElem.setAttribute("type", gm.getType().getName());
         rootElem.setAttribute("editable", String.valueOf(entGrp.isEditable()));
         boolean hasMems = gm.hasMembers();
         if (!hasMems) {
            rootElem.setAttribute("expanded", "false");
         }
         boolean isGroupExpanded = (Boolean.valueOf(rootElem.getAttribute("expanded")).booleanValue());
         if (!Utility.areEqual(rootElem.getAttribute("selected"), "true")) {
            rootElem.setAttribute("selected", "false");
         }
         rootElem.setAttribute("hasMembers", String.valueOf(hasMems));

         // set user permissions for group
         IGroupsManagerPermissions gmp = sessionData.getPermissions();
         IAuthorizationPrincipal ap = sessionData.getAuthorizationPrincipal();
         applyPermissions (rootElem, gm, gmp, ap);

         // If no rdf element, create it, otherwise refresh the element
         NodeList nList = rootElem.getElementsByTagName("rdf:RDF");
         if (nList.getLength() == 0) {
            Element rdf = groupsManagerXml.createRdfElement(entGrp, aDoc);
            rootElem.appendChild(rdf);
         }
         else{
             groupsManagerXml.refreshAllNodesIfRequired(sessionData, rootElem);
         }
         if (isGroupExpanded) {
            expandElement(gm, rootElem, sessionData);
         }
         Utility.logMessage("DEBUG", "GroupWrapper::getXml(): FINISHED");
      } catch (Exception e) {
         Utility.logMessage("ERROR", "GroupWrapper::getXml(): ERROR retrieving entity " + e, e);
      }
      return  rootElem;
   }

    /**
    * Returns a GroupMember for a key.
    * @param aKey String
    * @param aType String
    * @return IGroupMember
    */
   protected IGroupMember retrieveGroupMember (String aKey, String aType) {
      return (IGroupMember)Utility.retrieveGroup(aKey,groupsManagerXml.getGroupsManager().getCompositeGroupService());
   }

    /**
    * Returns the xml element for a given IEntityGroup, populated with child elements.
    * @param gm IGroupMember
    * @param anElem Element
    * @param sessionData CGroupsManagerUnrestrictedSessionData
    * @return Element
    */
   private Element expandElement (IGroupMember gm, Element anElem, GroupsSessionData sessionData) {
      Utility.logMessage("DEBUG", "GroupWrapper::expandElement(): START");
      Utility.logMessage("DEBUG", "GroupWrapper::expandElement(): Group Member: " + gm);
      Utility.logMessage("DEBUG", "GroupWrapper::expandElement(): Element: " + anElem);
      //We only want to expand the element if the attribute is set to "true"
      if (!Utility.areEqual(anElem.getAttribute("expanded"), "true")) {return anElem;}
      java.util.Iterator gmItr = null;
      IGroupMember aChildGm = null;
      Element tempElem = null;
      try {
         Utility.logMessage("DEBUG", "GroupWrapper::expandElement():  HERE COME THE KIDS");
         gmItr = gm.getMembers();
         // add new elements for new group members
         int gmCount = 0;
         while (gmItr.hasNext()) {
            aChildGm = (IGroupMember)gmItr.next();
            // if the limit has been exceeded, and this is not a group, skip it
            if (!limitRetrievals || aChildGm.isGroup() || gmCount < retrievalLimit) {
               String childKey = aChildGm.getKey();
               Utility.logMessage("DEBUG", "GroupWrapper::expandElement():  " + aChildGm);
               boolean memberElementFound = false;
               String tagname = (aChildGm.isGroup() ? GROUP_TAGNAME : ENTITY_TAGNAME);
               memberElementFound = GroupsManagerXml.getNodesByTagNameAndKey(anElem, tagname,
                     childKey).hasNext();
               if (!memberElementFound) {
                  tempElem = groupsManagerXml.getGroupMemberXml(aChildGm,false, null, sessionData);
                  Utility.logMessage("DEBUG", "GroupWrapper::expandElement():  APPENDING "
                        + tempElem.getNodeName());
                  anElem.appendChild(tempElem);
                  Utility.logMessage("DEBUG", "GroupWrapper::expandElement():  APPENDING ACCOMPLISHED");
               }
               gmCount++;
            }
         }

         // Remove elements for groups that are no longer members
         // Remember that getChildNodes does not only return IGroupMember elements
         NodeList nList = anElem.getChildNodes();
         for (int i = 0; i < nList.getLength(); i++) {
            tempElem = (Element)nList.item(i);
            if (Utility.notEmpty(tempElem.getAttribute("entityType"))){
               Utility.logMessage("DEBUG", "GroupWrapper::expandElement():  Checking if child element (id = " + tempElem.getAttribute("id") + ") still a member");
               Utility.logMessage("DEBUG", "GroupWrapper::expandElement():  child element (key = " + tempElem.getAttribute("key") + ")");
               Utility.logMessage("DEBUG", "GroupWrapper::expandElement():  child element (entityType = " + tempElem.getAttribute("entityType") + ")");
               aChildGm = GroupsManagerXml.retrieveGroupMemberForElement (tempElem,groupsManagerXml.getGroupsManager().getCompositeGroupService());
               if (aChildGm == null || !gm.contains(aChildGm)){
                  Utility.logMessage("DEBUG", "GroupWrapper::expandElement():  About to remove child element");
                  anElem.removeChild((Node)tempElem);
               }
            }
         }

      } catch (Exception e) {
         Utility.logMessage("ERROR", "GroupWrapper::expandElement(): ERROR expanding \nElement: "
            + anElem + "\nFor group member: " + gm + "\n" + e, e);
      }
      return  anElem;
   }
   
/**
 * @param groupsManagerXml The groupsManagerXml to set.
 */
public void setGroupsManagerXml(GroupsManagerXml groupsManagerXml) {
    this.groupsManagerXml = groupsManagerXml;
}

}
