/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.groups;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.xml.transform.dom.DOMSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.groups.IEntityGroup;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.gap.services.entitytypes.IEntityTypes;
import org.jasig.portal.ChannelRuntimeData;
import org.jasig.portal.context.common.user.IUserController;
import org.jasig.portal.event.PortalEvent;
import org.jasig.portal.groups.security.IPermissible;
import org.jasig.portal.groups.services.EntityNameFinderService;
import org.jasig.portal.groups.services.EntityPropertyRegistry;
import org.jasig.portal.portlet.servant.IPortletServant;
import org.jasig.portal.portlets.xslt.GenericXsltBean;
import org.jasig.portal.security.UserId;
import org.jasig.portal.servants.groups.commands.AssignPermissions;
import org.jasig.portal.servants.groups.commands.DoneWithSelection;
import org.jasig.portal.servants.groups.permissions.GroupsManagerDefaultPermissions;
import org.jasig.portal.servants.permissions.IPermissibleRegistry;
import org.w3c.dom.Document;

/**
 * CGroupsManager allows users to graphically administer all groups for which
 * user has administrtaive permissions.
 * @author Don Fracapane
 * @version $Revision: 11955 $
 */
public class GroupsManagerBean extends GenericXsltBean implements GroupsManagerConstants {
    
   private Log log = LogFactory.getLog(GroupsManagerBean.class);
   
   private static final String SESSION_DATA_KEY = "groups.session.data.key";
   private static final String DEFAULT_PERMISSIONS_SERVANT_INSTANCE_KEY =
       AssignPermissions.class.getName();

    
   private GroupsManagerXml groupsManagerXml;
   
   // IoC injections
   private EntityNameFinderService entityNameFinderService;
   private IAuthorizationService authorizationService;
   private EntityPropertyRegistry entityPropertyRegistry;
   private IUserController userController;
   private IEntityTypes entityTypes;
   private ICompositeGroupService compositeGroupService;
   private IPermissible permissible;
   private IPermissibleRegistry permissibleRegistry;
   private String xslUriStr;
   private URI xslUri = null;
   private IPortletServant permissionsManagerServant;
   
   private static final String ACTION_PROCESSED = "portlet action processed";

   /** Creates new GroupsManagerChannel */
   public GroupsManagerBean () {
   }
   
   public void processAction(ActionRequest req, ActionResponse res) throws PortletException, IOException {
       GroupsSessionData sessionData = getGroupsSessionData(req);
       if (sessionData.portletServant != null) {
           sessionData.portletServant.processAction(null, req, res);
           // the request object isn't set yet in sessionData, pass it along also
           checkServantComplete(req, sessionData);
       } else {
	       genericPortletRequestPreProcess(req);
	       // mark that the action has been processed, so that we don't have to re-run pre-processing during render
	       res.setRenderParameter(ACTION_PROCESSED,"true");
       }
   }
   
   private void checkServantComplete(PortletRequest req, GroupsSessionData sessionData) {
       if (sessionData.portletServant != null) {
           if (sessionData.portletServant.isComplete(req)) {
               sessionData.portletServant.reset(req);
               sessionData.portletServant = null;
           }
       }
   }
   
   protected void genericPortletRequestPreProcess(PortletRequest req) {
        // Init processing
        initSessionData(req); 
         
        portletRequestPreProcessSetXslParams(req, new HashMap());
   }
   
   protected void portletRequestPreProcessSetXslParams(PortletRequest req, Map xslParams) {
       GroupsSessionData sessionData = getGroupsSessionData(req);
             
         if (log.isDebugEnabled()) {
             log.debug("GroupsManagerBean::genericPortletRequestPreProcess : sessionData: " + sessionData);
             log.debug("GroupsManagerBean::genericPortletRequestPreProcess : ignorePermissions: " + sessionData.isAdminUser());
             log.debug("GroupsManagerBean::genericPortletRequestPreProcess : customMessage: " + sessionData.customMessage);
             log.debug("GroupsManagerBean::genericPortletRequestPreProcess : mode: " + sessionData.mode);
             log.debug("GroupsManagerBean::genericPortletRequestPreProcess : page: " + sessionData.currentPage);
             log.debug("GroupsManagerBean::genericPortletRequestPreProcess : highlightedGroupID: " + sessionData.highlightedGroupID);
             log.debug("GroupsManagerBean::genericPortletRequestPreProcess : rootViewGroupID: " + sessionData.rootViewGroupID);
             log.debug("GroupsManagerBean::genericPortletRequestPreProcess : defaultRootViewGroupID: " + sessionData.defaultRootViewGroupID);
             log.debug("GroupsManagerBean::genericPortletRequestPreProcess : feedback: " + sessionData.feedback);
             log.debug("GroupsManagerBean::genericPortletRequestPreProcess : blockFinishActions: " + !sessionData.allowFinish);
             log.debug("GroupsManagerBean::genericPortletRequestPreProcess : xslParams: " + xslParams);
         }
             
         if (sessionData.isAdminUser()) {
            xslParams.put("ignorePermissions", "true");
         }
         if (sessionData.customMessage !=null) {
             xslParams.put("customMessage", sessionData.customMessage);
         }
         if (SelectionModel.Single.equals(sessionData.selectionModel)) {
	         xslParams.put("selectionType", "radio");
         } else {
	         xslParams.put("selectionType", "checkbox");
         }
         xslParams.put("mode", sessionData.mode.toString());
         xslParams.put("page", String.valueOf(sessionData.currentPage));
         if (sessionData.highlightedGroupID != null) {
            xslParams.put("highlightedGroupID", sessionData.highlightedGroupID);
         }
         if (sessionData.rootViewGroupID != null) {
            xslParams.put("rootViewGroupID", sessionData.rootViewGroupID);
         }
         else if (sessionData.defaultRootViewGroupID != null){
            xslParams.put("rootViewGroupID", sessionData.defaultRootViewGroupID);
         }
         if (sessionData.feedback != null) {
            xslParams.put("feedback", sessionData.feedback);
            sessionData.feedback = null;
         }
         if (!sessionData.allowFinish) {
            xslParams.put("blockFinishActions", "true");
         }
         
         sessionData.setAttribute("xslParams",xslParams);
   }
   
   public void render(RenderRequest req, RenderResponse res) throws PortletException, IOException {
       
       GroupsSessionData sessionData = getGroupsSessionData(req);

       if (sessionData.portletServant != null) {
           // the request object isn't set yet in sessionData, pass it along also
           checkServantComplete(req, sessionData);
           if (sessionData.portletServant != null) {
               try {
                   sessionData.portletServant.render(req, res);
               } catch (Exception e) {
                   throw new PortletException(e);
               }
           } else {
               internalRender(req, res);
           }
       } else {
           internalRender(req, res);
       }
   }
   
   protected boolean okToRender(RenderRequest req) {
       return true;
   }
   
   protected boolean needsToPreProcess(RenderRequest req) {
       return req.getParameter(ACTION_PROCESSED)==null;
   }

   protected void internalRender(RenderRequest req, RenderResponse res) throws PortletException, IOException {
       genericPortletRequestPreProcess(req);
       
       GroupsSessionData sessionData = getGroupsSessionData(req);
       
       if (!okToRender(req)) {
           res.setContentType("text/html");
           return;
       }
       
       
       Map xslParams = (Map) sessionData.getAttribute("xslParams");
       Document xmlDoc = sessionData.getModel();
       if ( xmlDoc != null ) {
          //System.out.println("XML_UTIL: " + XmlUtils.serializeNode(xmlDoc));
          processXslt(getXslUri(req),new DOMSource(xmlDoc),xslParams,req,res);
       } else {
          throw new PortletException("The groups manager XML document is NULL!");
       }
   }

   /**
    * Passes an outside event to a channel.
    * Events should normally come from the LayoutBean.
    * @param ev PortalEvent object
    * @see PortalEvent
    */
   protected void finishSession (PortletRequest request) {
      GroupsSessionData sd = getGroupsSessionData(request);
      if (sd.lockedGroup != null) {
         sd.lockedGroup.getLock().release();
         sd.lockedGroup = null;
      }
         
      releaseGroupsSessionData(request);
      request.getPortletSession().removeAttribute("groupManagerFinished");
   }


   /**
    * This function is called prior to the render() call.
    * @param rd channel runtime data
    * @see ChannelRuntimeData
    * @param uid
    */
   public void initSessionData (PortletRequest request) {
      GroupsSessionData sessionData = getGroupsSessionData(request);
      sessionData.setPortletRequest(request);
      sessionData.startRD = Calendar.getInstance().getTime().getTime();
      
      String grpCommand = request.getParameter(GROUP_COMMAND);
      
      if (log.isDebugEnabled()) {
          log.debug("GroupsManagerBean::initSessionData : grpCommand: " + grpCommand);
          for (Enumeration en = request.getParameterNames(); en.hasMoreElements();) {
              String name = (String)en.nextElement();
	          log.debug("GroupsManagerBean::initSessionData : portlet param: " + name + '=' + request.getParameter(name));
          }
          for (Enumeration en = request.getAttributeNames(); en.hasMoreElements();) {
              String name = (String)en.nextElement();
	          log.debug("GroupsManagerBean::initSessionData : portlet attribute: " + name + '=' + request.getAttribute(name));
          }
      }
      
      //if ( !ServerPortletUtil.isServantMode(request) ) {
        if (Utility.hasValue(grpCommand)) {
            IGroupsManagerCommand c = GroupsManagerCommandFactory.get(grpCommand, compositeGroupService);
            if (c != null) {
               try{
                c.execute(sessionData,getGroupsManagerXml(),authorizationService);
               }
               catch(Exception e){
                  Utility.logMessage("ERROR", e.toString(), e);
                  sessionData.feedback = "Error executing command "+grpCommand+": "+e.getMessage();
               }
            }
         }
         if (Utility.hasValue(request.getParameter("grpPageForward"))){
            sessionData.currentPage += Integer.parseInt(request.getParameter("grpPageForward"));
         }
         if (Utility.hasValue(request.getParameter("grpPageBack"))){
            sessionData.currentPage -= Integer.parseInt((String)request.getParameter("grpPageBack"));
         }
      //}
 
   }

/**
 * @param entityNameFinderService The entityNameFinderService to set.
 */
public void setEntityNameFinderService(EntityNameFinderService entityNameFinderService) {
    this.entityNameFinderService = entityNameFinderService;
}

/**
 * @param authorizationService The authorizationService to set.
 */
public void setAuthorizationService(IAuthorizationService authorizationService) {
    this.authorizationService = authorizationService;
}

protected String getGroupsSessionDataKey() {
    return SESSION_DATA_KEY;
}

public GroupsSessionData getGroupsSessionData(PortletRequest request) {
    
    String key = getGroupsSessionDataKey();
    GroupsSessionData sessionData = (GroupsSessionData) request.getPortletSession().getAttribute(key);

    if ( sessionData == null ) {
        sessionData = new GroupsSessionData(this);
        initializeGroupsSessionData(request, sessionData);
	    request.getPortletSession().setAttribute(getGroupsSessionDataKey(), sessionData);
    }
    return sessionData;
}

protected void initializeGroupsSessionData(PortletRequest request, GroupsSessionData sessionData) {
    String username = userController.getUser().getUserName();
    IAuthorizationPrincipal authPrinc = authorizationService.newPrincipal(username, UserId.class);
    IEntityGroup adminGroup = compositeGroupService.getDistinguishedGroup(IDistinguishedGroupConstants.PORTAL_ADMINISTRATORS);
    IGroupMember currUser = authorizationService.getGroupMember(authPrinc);
    IGroupsManagerPermissions permissions = GroupsManagerDefaultPermissions.getInstance();
    boolean adminUser = adminGroup != null && adminGroup.deepContains(currUser);
    
    sessionData.permissible = permissible;
    sessionData.setAdminUser(adminUser);
    sessionData.setUserId(userController.getUserId());
    sessionData.setAuthorizationPrincipal(authPrinc);
    sessionData.setPermissionsManagerServant(permissionsManagerServant);
          
    if (sessionData.getPermissions() == null) {
        sessionData.setPermissions(permissions);
    }

    if (sessionData.getModel() == null) {
        sessionData.setModel(getGroupsManagerXml().getGroupsManagerXml(sessionData));
    }
          
}

private void releaseGroupsSessionData(PortletRequest request) {
    request.getPortletSession().removeAttribute(getGroupsSessionDataKey());
}


protected GroupsManagerXml getGroupsManagerXml() {
    if (groupsManagerXml == null) {
        groupsManagerXml = new GroupsManagerXml(this);
    }
    return groupsManagerXml;
}

/**
 * Returns an array of objects representing the result set.
 * Note that these are IGroupMembers !!
 * @return Object[]
 */
public Object[] getResults (PortletRequest request) {
    
   PortletSession session = request.getPortletSession(); 
   if ( session == null ) {
      return new IGroupMember[0];
   }
    
   GroupsSessionData sessionData = getGroupsSessionData(request);
   IGroupsManagerCommand cmd = GroupsManagerCommandFactory.get(DoneWithSelection.class.getSimpleName(), compositeGroupService);
   try {
      cmd.execute(sessionData,getGroupsManagerXml(),authorizationService);
   }
   catch (Exception e) {
      sessionData.feedback = "Error executing command Done: "+e.getMessage();
      throw new RuntimeException ("getResults: "+e.toString(), e);
   }
   
   Object[] results = (Object[]) request.getAttribute("princResults");
   if ( results == null ) {
     results = new IGroupMember[0];
   }
   return results;
}

/**
 * @return Returns the authorizationService.
 */
public IAuthorizationService getAuthorizationService() {
    return authorizationService;
}

/**
 * @return Returns the entityNameFinderService.
 */
public EntityNameFinderService getEntityNameFinderService() {
    return entityNameFinderService;
}

/**
 * @return Returns the entityPropertyRegistry.
 */
public EntityPropertyRegistry getEntityPropertyRegistry() {
    return entityPropertyRegistry;
}

/**
 * @param entityPropertyRegistry The entityPropertyRegistry to set.
 */
public void setEntityPropertyRegistry(EntityPropertyRegistry entityPropertyRegistry) {
    this.entityPropertyRegistry = entityPropertyRegistry;
}

/**
 * @param userController The userController to set.
 */
public void setUserController(IUserController userController) {
    this.userController = userController;
}

public void setEntityTypes(IEntityTypes entityTypes) {
    this.entityTypes = entityTypes;
}

public void setCompositeGroupService(
    ICompositeGroupService compositeGroupService) {
    this.compositeGroupService = compositeGroupService;
}

public IEntityTypes getEntityTypes() {
    return entityTypes;
}

public ICompositeGroupService getCompositeGroupService() {
    return compositeGroupService;
}

public void setPermissible(IPermissible permissible) {
    this.permissible = permissible;
}

public IPermissibleRegistry getPermissibleRegistry() {
    return permissibleRegistry;
}

public void setPermissibleRegistry(IPermissibleRegistry permissibleRegistry) {
    this.permissibleRegistry = permissibleRegistry;
}

public void setXslUri(String xslUri) {
    this.xslUriStr = xslUri;
}

protected URI getXslUri(RenderRequest req) {
    if (xslUri == null) {
        // see if this is a well-formed URL
        xslUri = URI.create(xslUriStr);
        if ( !xslUri.isAbsolute() ) {
            try {
                xslUri = URI.create(req.getPortletSession().getPortletContext().getResource(xslUriStr).toString());
            } catch (MalformedURLException e) {
                throw new IllegalArgumentException(
                    "Misconfigured xslUri property: " + xslUriStr, e);
            }
        }
    }
    return xslUri;
}
public IPortletServant getPermissionsManagerServant() {
    return permissionsManagerServant;
}
public void setPermissionsManagerServant(IPortletServant permissionsManagerServant) {
    this.permissionsManagerServant = permissionsManagerServant;
}

public IUserController getUserController() {
    return userController;
}

}



