/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.groups;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Map;

import org.jasig.gap.groups.ICompositeGroupService;

/**
 * A class designed to decouple a request for an action to be performed from the
 * class responsible for the action.  Also centralizes authorization for commands.
 * @author Don Fracapane
 * @version $Revision: 11911 $
 */
public class GroupsManagerCommandFactory implements GroupsManagerConstants {
    
   protected static final Map<String, IGroupsManagerCommand> bindings = new HashMap<String, IGroupsManagerCommand>();
   protected static boolean INITIALIZED = false;
   private static Class[] commandArgs = { ICompositeGroupService.class };

   /**
    * Returns the instance of a command object bound to a name.
    * @param name
    * @return IGroupsManagerCommand
    */
   public static IGroupsManagerCommand get (String name, ICompositeGroupService compositeGroupService) {
      Utility.logMessage("DEBUG", "GroupsManagerCommandFactory::get(): about to get command: "
            + name);
      IGroupsManagerCommand command = bindings.get(name);
      if (command == null) {
          command = instantiateCommand(name, compositeGroupService);
          bindings.put(name, command);
      }
      return command;
   }
   
   private static IGroupsManagerCommand instantiateCommand(String name, ICompositeGroupService compositeGroupService) {
       try {
	       Constructor c = Class.forName(COMMANDS_PACKAGE + "." + name).getConstructor(commandArgs);
           Object[] args = {compositeGroupService};
           return (IGroupsManagerCommand)c.newInstance(args);
       } catch (Exception e) {
           throw new RuntimeException("No command exists for name: " + name);
       }
   }
}
