/* Copyright 2006 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */
package org.jasig.portal.servants.groups;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.groups.IEntityGroup;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.portal.groups.security.IPermissible;
import org.jasig.portal.groups.services.EntityNameFinderService;

public class GroupsManagerPermissible implements IPermissible,
    GroupsManagerConstants {
    
    private static final String PORTLETS = "Portlets";

    private ICompositeGroupService compositeGroupService;
    private EntityNameFinderService entityNameFinderService;

    private Map<String, MetadataWrapper> activities;

    private Map<String, MetadataWrapper> targets;

    public GroupsManagerPermissible(ICompositeGroupService compositeGroupService,
        EntityNameFinderService entityNameFinderService) {
        this.compositeGroupService = compositeGroupService;
        this.entityNameFinderService = entityNameFinderService;
        initActivities();
        initTargets();
    }

    private void initActivities() {
        activities = new HashMap<String, MetadataWrapper>();
        activities.put(CREATE_PERMISSION, new MetadataWrapper("Create",
            "Create a group in this context"));
        activities.put(VIEW_PERMISSION, new MetadataWrapper("View",
            "View this group"));
        activities.put(UPDATE_PERMISSION, new MetadataWrapper("Rename",
            "Rename this group"));
        activities.put(DELETE_PERMISSION, new MetadataWrapper("Delete",
            "Delete this group"));
        activities.put(SELECT_PERMISSION, new MetadataWrapper("Select",
            "Select this group"));
        activities.put(ADD_REMOVE_PERMISSION, new MetadataWrapper("Manage",
            "Manage this group's members"));
        activities.put(ASSIGN_PERMISSION, new MetadataWrapper("Permiss",
            "Assign Permissions for this group"));
    }

    private void initTargets() {
        targets = new HashMap<String, MetadataWrapper>();
        IEntityGroup everyone = compositeGroupService.getDistinguishedGroup(IDistinguishedGroupConstants.EVERYONE);
        if ( everyone != null ) { 
         targets.put(everyone.getKey(), new MetadataWrapper(everyone.getName(), everyone.getDescription()));
         Iterator allgroups = everyone.getAllMembers();
         while (allgroups.hasNext()) {
            IGroupMember gm = (IGroupMember)allgroups.next();
            if (gm.isGroup()) {
               if (targets.get(gm.getKey()) == null) {
                  try {
                     IEntityGroup g = (IEntityGroup)gm;
                     targets.put(g.getKey(), new MetadataWrapper(g.getName(), g.getDescription()));
                  } catch (Exception e) {
                     throw new RuntimeException("unable to add target" + e, e);
                  }
               }
            }
         }
        } 

        IEntityGroup allChans = compositeGroupService.getDistinguishedGroup(PORTLETS);
        if ( allChans != null ) {
         targets.put(allChans.getKey(), new MetadataWrapper(allChans.getName(), allChans.getDescription()));
         Iterator allcgroups = allChans.getAllMembers();
         while (allcgroups.hasNext()) {
            IGroupMember gm = (IGroupMember)allcgroups.next();
            if (gm.isGroup()) {
               if (targets.get(gm.getKey()) == null) {
                  try {
                     IEntityGroup g = (IEntityGroup)gm;
                     targets.put(g.getKey(), new MetadataWrapper(g.getName(), g.getDescription()));
                  } catch (Exception e) {
                      throw new RuntimeException("GroupsManagerBean.initGroups():: unable to add target" + e, e);
                  }
               }
            }
         }
        } 
    }
    
    public String getActivityDescription(String token) {
        String description = null;
        MetadataWrapper metadata = activities.get(token);
        if (metadata != null) {
            description = metadata.getDescription();
        }
        return description;
    }

    public String getActivityName(String token) {
        String name = null;
        MetadataWrapper metadata = activities.get(token);
        if (metadata != null) {
            name = metadata.getName();
        }
        return name;
    }

    public String[] getActivityTokens(String targetToken) {
        // all groups have the same activity set
        return (String[]) activities.keySet().toArray(new String[0]);
    }

    public String getOwnerDescription() {
        return "Groups Manager Portlet Permissible";
    }

    public String getOwnerName() {
        return "Groups Manager";
    }

    public String getOwnerToken() {
        return OWNER;
    }

    public String getTargetDescription(String token) {
        String description = null;
        MetadataWrapper metadata = targets.get(token);
        if (metadata != null) {
            description = metadata.getDescription();
        }
        return description;
    }

    public String getTargetName(String token) {
        String name = null;
        MetadataWrapper metadata = targets.get(token);
        if (metadata != null) {
            name = metadata.getName();
        } else {
            try {
                name = entityNameFinderService.getEntityNameFinder(
                    IEntityGroup.class).getName(token);
            } catch (Exception e) {
                Utility.logMessage("Error finding name by token: " + token, e.toString(), e);
            }
        }
        return name;
    }

    public String[] getTargetTokens() {
        return (String[]) targets.keySet().toArray(new String[0]);
    }

    private static class MetadataWrapper {
        private String name;

        private String description;

        public MetadataWrapper(String name, String description) {
            this.name = name;
            this.description = description;
        }

        public String getName() {
            return name;
        }

        public String getDescription() {
            return description;
        }
    }

}
