package org.jasig.portal.servants.groups;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.gap.groups.IEntityGroup;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.portal.portlet.rendering.IPortletRenderContent;
import org.jasig.portal.portlet.servant.IPortletServant;
import org.jasig.portal.security.UserId;
import org.jasig.portal.servants.groups.commands.DoneWithSelection;
import org.jasig.portal.servants.groups.permissions.GroupsManagerBlockEntitySelectPermissions;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class GroupsManagerServantBean extends GroupsManagerBean implements
    IPortletServant {
    
    private static long nextInstanceId = 0L;
    
    // servant instance id, each caller should instantiate their own
    // instance.
    private long instanceId;
    
    public GroupsManagerServantBean() {
        this.instanceId = getNextInstanceId();
    }
    
    private static synchronized long getNextInstanceId() {
        return nextInstanceId++;
    }
    
    @Override
    protected boolean okToRender(RenderRequest req) {
        boolean isComplete = isComplete(req);
        boolean isCancelled = isCancelled(req);
        boolean allowFinish = getGroupsSessionData(req).allowFinish;
        return !allowFinish || (!isComplete && !isCancelled);
    }
    
    protected String getGroupsSessionDataKey() {
        return Long.toString(instanceId) + '.' + super.getGroupsSessionDataKey();
    }
    
    @Override
    protected boolean needsToPreProcess(RenderRequest req) {
        return true;
    }

    @Override
    protected void initializeGroupsSessionData(PortletRequest request, GroupsSessionData sessionData) {
        String username = getUserController().getUser().getUserName();
        IAuthorizationPrincipal authPrinc = getAuthorizationService().newPrincipal(username, UserId.class);
        IEntityGroup adminGroup = getCompositeGroupService().getDistinguishedGroup(IDistinguishedGroupConstants.PORTAL_ADMINISTRATORS);
        IGroupMember currUser = getAuthorizationService().getGroupMember(authPrinc);
        boolean adminUser = adminGroup != null && adminGroup.deepContains(currUser);
        
        IGroupMember[] groupMembers = (IGroupMember[]) getAttribute(request,"groupMembers");
        Boolean allowEntitySelect = getBooleanAttribute(request,"allowEntitySelect");
        Boolean allowFinish = getBooleanAttribute(request, "allowFinish");
        SelectionModel selectionModel = (SelectionModel)getAttribute(request, "selectionModel");
        String displayMessage = (String)getAttribute(request, "displayMessage");
        if (displayMessage == null) {
            displayMessage = "Groups for selection";
        }
        
        IGroupsManagerPermissions overridingPermissions = (IGroupsManagerPermissions)getAttribute(request, "overridingPermissions");
        
        GroupManagerMode groupManagerMode = (GroupManagerMode)getAttribute(request, "groupManagerMode");
        
        IEntityActionUrlParts entityActionUrlParts = (IEntityActionUrlParts)getAttribute(request, "entityActionUrlParts");

        String type=(String) getAttribute(request,"type");
        Set<Class> entityTypeFilter = (Set<Class>)getAttribute(request,"entityTypeFilter");
        
        GroupsManagerXml groupsManagerXml = getGroupsManagerXml();
        
        sessionData.setEntityActionUrlParts(entityActionUrlParts);
        sessionData.entityTypeFilter = entityTypeFilter;
        sessionData.allowFinish = allowFinish;
        sessionData.setAuthorizationPrincipal(authPrinc);
        sessionData.setAdminUser(adminUser);
        
        if (groupManagerMode != null) {
            sessionData.mode = groupManagerMode;
        } else {
            sessionData.mode = GroupManagerMode.select;
        }
        
        if (selectionModel != null) {
            sessionData.selectionModel = selectionModel;
        }
        
        if (overridingPermissions != null) {
          sessionData.setPermissions(overridingPermissions);
        } else if(!allowEntitySelect) {
          sessionData.setPermissions(GroupsManagerBlockEntitySelectPermissions.getInstance());
        }

        if (displayMessage != null){
          sessionData.customMessage = displayMessage;
        }
        String typeKey = null;
        if (type!=null && !type.equals("")){
          try{
            typeKey = groupsManagerXml.getGroupsManager().getCompositeGroupService().getDistinguishedGroup(type).getKey();
          } catch(Exception e){
              Utility.logMessage("getDistinguishedGroup: ",e.toString());
          }
        }
        
        sessionData.setModel(groupsManagerXml.getGroupsManagerXml(sessionData));
        
        if (typeKey!=null){
          sessionData.rootViewGroupID = Utility.translateKeytoID(groupsManagerXml,typeKey,sessionData,groupsManagerXml.getGroupsManager().getCompositeGroupService());
        }
        sessionData.highlightedGroupID = sessionData.rootViewGroupID;
       
        if (groupMembers!=null && groupMembers.length>0){
          Document viewDoc = sessionData.getModel();

            Element rootElem = viewDoc.getDocumentElement();
            try{
                for (int mm = 0; mm< groupMembers.length;mm++){
                  IGroupMember mem = groupMembers[mm];
                  Element memelem = groupsManagerXml.getGroupMemberXml(mem,false,null,sessionData);
                  memelem.setAttribute("selected","true");
                  rootElem.appendChild(memelem);
                }
            }
            catch (Exception e){
              Utility.logMessage("ERROR", e.toString(), e);
            }
        }
        sessionData.setNamespace(Long.toString(instanceId));
        
        super.initializeGroupsSessionData(request, sessionData);
    }
    
    @Override
    protected void genericPortletRequestPreProcess(PortletRequest req) {
		// Init processing
        initSessionData(req); 
        
        Map xslParams = new HashMap();
        
        GroupsSessionData sessionData = getGroupsSessionData(req);
        
        boolean isComplete=isComplete(req);
        sessionData.warnOnEmptySelection=isComplete;
        
        String callerCancelCommandName = (String)sessionData.getAttribute("callerCancelCommandName");
        if (callerCancelCommandName != null) {
            xslParams.put("callerCancelCommandName", callerCancelCommandName);
        }
        String callerCancelCommandValue = (String)sessionData.getAttribute("callerCancelCommandValue");
        if (callerCancelCommandValue != null) {
            xslParams.put("callerCancelCommandValue", callerCancelCommandValue);
        }
        String callerFinishedCommandName = (String)sessionData.getAttribute("callerFinishedCommandName");
        if (callerFinishedCommandName != null) {
            xslParams.put("callerFinishedCommandName", callerFinishedCommandName);
        }
        String callerFinishedCommandValue = (String)sessionData.getAttribute("callerFinishedCommandValue");
        if (callerFinishedCommandValue != null) {
            xslParams.put("callerFinishedCommandValue", callerFinishedCommandValue);
        }
        String callerSessionName = (String)sessionData.getAttribute("callerSessionName");
        if (callerSessionName != null) {
            xslParams.put("callerSessionName", callerSessionName);
        }
        String callerSessionValue = (String)sessionData.getAttribute("callerSessionValue");
        if (callerSessionValue != null) {
            xslParams.put("callerSessionValue", callerSessionValue);
        }
        xslParams.put("grpServantMode", "true");
        
        super.portletRequestPreProcessSetXslParams(req, xslParams);
    }
    
    private Boolean getBooleanAttribute(PortletRequest request, String key) {
        Boolean b = (Boolean)getAttribute(request, key);
        if (b == null) {
            b = Boolean.FALSE;
        }
        return b;
    }

    public Object getAttribute(PortletRequest request, String key) {
        return request.getPortletSession().getAttribute(buildAttributeKey(key));
    }

    public void setAttribute(PortletRequest request, String key, Object value) {
        request.getPortletSession().setAttribute(buildAttributeKey(key), value);
    }

    public String getId() {
        return Long.toString(instanceId);
    }

    public IPortletRenderContent getRenderContent(RenderRequest request,
        RenderResponse response) {
        return null;
    }

    public void processAction(Map actionRequestParameterMap,
        ActionRequest request, ActionResponse response) throws PortletException, IOException {
        super.processAction(request, response);
    }
    
    @Override
    protected void finishSession(PortletRequest request) {
        initSessionData(request);
        GroupsSessionData sessionData = getGroupsSessionData(request);
        sessionData.removeAttribute("groupMembers");
        sessionData.removeAttribute("allowEntitySelect");
        sessionData.removeAttribute("allowFinish");
        sessionData.removeAttribute("entityTypeFilter");
        sessionData.removeAttribute("type");
        sessionData.removeAttribute("groupManagerFinished");
        super.finishSession(request);
    }
    
    private String buildAttributeKey(String key) {
        String newKey = Long.toString(instanceId) + '.' + key;
        return newKey;
    }

    public void reset(PortletRequest request) {
        finishSession(request);
    }
    
    public Object getResult(PortletRequest request) {
        return getResults(request);
    }

    public boolean isComplete (PortletRequest request) {
        return DoneWithSelection.class.getSimpleName().equalsIgnoreCase(request.getParameter(GROUP_COMMAND))?true:false;
    }
    
    public boolean isCancelled(PortletRequest request) {
        return "true".equals(getAttribute(request, "groupManagerFinished"));
    }

}
