/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.groups;

import java.util.Set;

import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.portlet.PortletURL;
import javax.portlet.RenderResponse;

import org.jasig.gap.groups.ILockableEntityGroup;
import org.jasig.portal.groups.security.IPermissible;
import org.jasig.portal.portlet.servant.IPortletServant;


/**
 * Session data for a cached thread stored in a Map in CGroupsManager
 * @author Don Fracapane
 * @version $Revision: 11911 $
 */
public class GroupsSessionData extends UnrestrictedGroupsSessionData implements GroupsManagerConstants {
    
   private PortletRequest portletRequest;
   private String namespace;
   private IPortletServant permissionsManagerServant;
   private IEntityActionUrlParts entityActionUrlParts;
   
   public boolean allowFinish = true;
   public boolean warnOnEmptySelection=true;
   public long startRD;
   public ILockableEntityGroup lockedGroup;
   public String highlightedGroupID;
   public int currentPage = 1;
   public String rootViewGroupID;
   public String defaultRootViewGroupID = "0";
   public GroupManagerMode mode = GroupManagerMode.browse;
   public GroupManagerMode returnToMode;
   public String feedback;             // use to display info to user (eg. "Unable to lock...")
   public String customMessage;
   public IPermissible permissible;
   public Set<Class> entityTypeFilter;
   public IPortletServant portletServant;
   public GroupsManagerBean groupsManagerBean;
   public boolean actionProcessed = false;
   public SelectionModel selectionModel = SelectionModel.Multiple;
   
   public GroupsSessionData(GroupsManagerBean groupsManagerBean) {
       this.groupsManagerBean = groupsManagerBean;
   }

   /**
    * Returns a subset of unrestricted variables to be used for Document manipulation.
    * This is accomplished by casting this into the unrestricted partent class.
    * @return CGroupsManagerUnrestrictedSessionData
    */
   public UnrestrictedGroupsSessionData getUnrestrictedData () {
      return  (UnrestrictedGroupsSessionData) this;
   }
   
/**
 * @return Returns the portletRequest.
 */
public PortletRequest getPortletRequest() {
    return portletRequest;
}
/**
 * @param portletRequest The portletRequest to set.
 */
public void setPortletRequest(PortletRequest portletRequest) {
    this.portletRequest = portletRequest;
}

/**
 * @return Returns the portletSession.
 */
public PortletSession getPortletSession() {
    return portletRequest.getPortletSession();
}

public GroupsManagerBean getGroupsManagerBean() {
    return groupsManagerBean;
}

public void setAttribute(String key, Object value) {
    getPortletSession().setAttribute(getAttributeKey(key), value);
}

public Object getAttribute(String key) {
    PortletSession session = getPortletSession();
    return session.getAttribute(getAttributeKey(key));
}

public void removeAttribute(String key) {
    getPortletSession().removeAttribute(getAttributeKey(key));
}

private String getAttributeKey(String key) {
    if (namespace != null) {
        return namespace + '.' + key;
    }
    return key;
}

public String getNamespace() {
    return namespace;
}

public void setNamespace(String namespace) {
    this.namespace = namespace;
}

public IPortletServant getPermissionsManagerServant() {
    return permissionsManagerServant;
}

public void setPermissionsManagerServant(
    IPortletServant permissionsManagerServant) {
    this.permissionsManagerServant = permissionsManagerServant;
}

public IEntityActionUrlParts getEntityActionUrlParts() {
    return entityActionUrlParts;
}

public void setEntityActionUrlParts(
    IEntityActionUrlParts entityActionUrlParts) {
    this.entityActionUrlParts = entityActionUrlParts;
}

}
