/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.groups;

import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.portal.security.UserId;
import org.w3c.dom.Document;


/**
 * Session data that is a subset of CGroupsManagerSessionData.
 * 
 * @author Don Fracapane
 * @version $Revision: 11955 $
 */
public class UnrestrictedGroupsSessionData implements GroupsManagerConstants {
    
   private Document model;
   private UserId userId;
   private boolean isAdminUser = false;
   private IGroupsManagerPermissions permissions;
   private IAuthorizationPrincipal authorizationPrincipal;
  
   /** 
    * Creates new CGroupsManagerUnrestrictedSessionData
    */
   public UnrestrictedGroupsSessionData () {
   }

   /** 
    * Creates new CGroupsManagerUnrestrictedSessionData
    * @param model Document
    * @param user IPerson
    * @param isAdminUser boolean
    * @param gmPermissions IGroupsManagerPermissions
    * @param authPrincipal IAuthorizationPrincipal
    */
   public UnrestrictedGroupsSessionData (UserId userId, boolean isAdminUser,
           IGroupsManagerPermissions permissions, IAuthorizationPrincipal authorizationPrincipal) {
      this();
      this.userId = userId;
      this.isAdminUser = isAdminUser;
      this.permissions = permissions;
      this.authorizationPrincipal = authorizationPrincipal;
   }
   
   public String toString() {
        StringBuffer sb =  new StringBuffer();
        sb.append("[");
        sb.append(UnrestrictedGroupsSessionData.class.getName());
        if (this.userId != null){
            sb.append(" user=");
            sb.append(this.userId);
        }
        sb.append(" isAdminUser=");
        sb.append(this.isAdminUser);
        if (getPermissions() != null){
            sb.append(" gmPermissions=");
            sb.append(getPermissions());
        }
        if (getAuthorizationPrincipal() != null){
            sb.append(" authPrincipal=");
            sb.append(getAuthorizationPrincipal());
        }
        if (this.model != null){
            sb.append(" model=");
            sb.append(this.model);
        }
        sb.append("]");
        return sb.toString();
      }
/**
 * @return Returns the model.
 */
public Document getModel() {
    return model;
}
/**
 * @param model The model to set.
 */
public void setModel(Document model) {
    this.model = model;
}

/**
 * @return Returns the user.
 */
public UserId getUserId() {
    return userId;
}
/**
 * @param user The user to set.
 */
public void setUserId(UserId userId) {
    this.userId = userId;
}
/**
 * @return Returns the isAdminUser.
 */
public boolean isAdminUser() {
    return isAdminUser;
}
/**
 * @param isAdminUser The isAdminUser to set.
 */
public void setAdminUser(boolean isAdminUser) {
    this.isAdminUser = isAdminUser;
}
/**
 * @return Returns the permissions.
 */
public IGroupsManagerPermissions getPermissions() {
    return permissions;
}
/**
 * @param permissions The permissions to set.
 */
public void setPermissions(IGroupsManagerPermissions permissions) {
    this.permissions = permissions;
}
/**
 * @return Returns the authorizationPrincipal.
 */
public IAuthorizationPrincipal getAuthorizationPrincipal() {
    return authorizationPrincipal;
}
/**
 * @param authorizationPrincipal The authorizationPrincipal to set.
 */
public void setAuthorizationPrincipal(IAuthorizationPrincipal authorizationPrincipal) {
    this.authorizationPrincipal = authorizationPrincipal;
}

}

