/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.groups.commands;


import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.portal.servants.groups.GroupManagerMode;
import org.jasig.portal.servants.groups.GroupsManagerXml;
import org.jasig.portal.servants.groups.GroupsSessionData;
import org.jasig.portal.servants.groups.Utility;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/** 
 *  AddMembers is a command which sets the id of the parent group.
 *  The parent group is the group to which child members will be added.
 *  Control is then passed to a selection view where the child group members will
 *  be selected for addition. When the selection has been completed by the user,
 *  the DoneWithSelection command will be invoked where the selected children
 *  group members are actually processed.  Alternatively, the CancelSelection
 *  command is invoked to cancel the selection process and reset the mode and
 *  view control parameters.
 * @author Don Fracapane
 * @version $Revision: 11911 $
 */
public class AddMembers extends GroupsManagerCommand {

   public AddMembers(ICompositeGroupService compositeGroupService) {
      super(compositeGroupService);
   }

   /**
    * This is the public method
    * @param sessionData
    * @throws Exception
    */
   public void execute (GroupsSessionData sessionData,GroupsManagerXml groupsManagerXml,IAuthorizationService authorizationService) throws Exception {
      
      PortletRequest request = sessionData.getPortletRequest();
      PortletSession session = request.getPortletSession();
      
      Document model = getXmlDoc(sessionData);
      Utility.logMessage("DEBUG", "AddMembers::execute(): Start");
      String parentAddElemId = getCommandArg(request);

      IGroupMember pg = GroupsManagerXml.retrieveGroupMemberForElementId(model, parentAddElemId, getCompositeGroupService());
      sessionData.rootViewGroupID = Utility.translateKeytoID(groupsManagerXml,getCompositeGroupService().getDistinguishedGroup(pg.getLeafType().getName()).getKey(),sessionData, getCompositeGroupService());
      // Parent was locked so no other thread or process could have changed it, but
      // child members could have changed.
      Element parentElem = GroupsManagerXml.getElementById(model, parentAddElemId);
      groupsManagerXml.refreshAllNodesRecursivelyIfRequired(sessionData, parentElem);
      sessionData.returnToMode = sessionData.mode;
      sessionData.mode=GroupManagerMode.select;
      sessionData.highlightedGroupID = sessionData.rootViewGroupID;
      Utility.logMessage("DEBUG", "AddMembers::execute(): Uid of parent element = " +
            parentAddElemId);
      sessionData.setAttribute("groupParentId", parentAddElemId);
   }
}



