/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.groups.commands;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.portal.servants.groups.GroupsSessionData;
import org.jasig.portal.servants.groups.GroupsManagerXml;
import org.jasig.portal.servants.groups.Utility;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * If the children xml elements have not already been created, this command will
 * retrieve the group members and created the elements. This command then sets
 * the expanded attribute for an element to "true" and lets the transformation
 * handle the tree expansion display.
 * @author Don Fracapane
 * @version $Revision: 11911 $
 */

public class ExpandGroup extends GroupsManagerCommand {

   public ExpandGroup(ICompositeGroupService compositeGroupService) {
      super(compositeGroupService);
   }

   /**
    * This is the public method
    * @param sessionData
    * @throws Exception
    */
   public void execute (GroupsSessionData sessionData,GroupsManagerXml groupsManagerXml,IAuthorizationService authorizationService) throws Exception {
       
      Utility.logMessage("DEBUG", "ExpandGroup::execute(): Start");
      Document model = getXmlDoc(sessionData);
      // Due to the networked relationship of groups, the next method has to return a list of elements.
      String elemUid = getCommandArg(sessionData.getPortletRequest());
      Utility.logMessage("DEBUG", "ExpandGroup::execute(): Uid of expanded element = "
            + elemUid);
      Element expandedElem = GroupsManagerXml.getElementByTagNameAndId(model, GROUP_TAGNAME, elemUid);
      if (expandedElem != null) {
        groupsManagerXml.refreshAllNodesRecursivelyIfRequired(sessionData, expandedElem);

        if (expandedElem.getAttribute("searchResults").equals("true")){
          expandedElem.setAttribute("expanded","true");
        }
        else{
          groupsManagerXml.expandGroupElementXML(expandedElem,sessionData);
        }
      }
   }
}
