/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.servants.groups.commands;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.portal.servants.groups.GroupManagerMode;
import org.jasig.portal.servants.groups.GroupsManagerCommandFactory;
import org.jasig.portal.servants.groups.GroupsManagerXml;
import org.jasig.portal.servants.groups.GroupsSessionData;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * A Groups Manager command to highlight a particular element.  Also
 * releases any held locks on other groups, moves to BROWSE mode from EDIT
 *
 * @author Alex Vigdor
 * @version $Revision: 11911 $
 */
public class HighlightGroup extends GroupsManagerCommand{
    
   public HighlightGroup(ICompositeGroupService compositeGroupService) {
      super(compositeGroupService);
   }

   /**
    * put your documentation comment here
    * @param sessionData
    * @throws Exception
    */
   public void execute(GroupsSessionData sessionData, GroupsManagerXml groupsManagerXml,IAuthorizationService authorizationService) throws Exception{
      Document model = getXmlDoc(sessionData);
      sessionData.highlightedGroupID = getCommandArg(sessionData.getPortletRequest());
      sessionData.currentPage = 1;
      GroupsManagerCommandFactory.get(ExpandGroup.class.getSimpleName(), getCompositeGroupService()).execute(sessionData,groupsManagerXml,authorizationService);
      // expand parent
      Element expandedElem = GroupsManagerXml.getElementByTagNameAndId(model, GROUP_TAGNAME, getCommandArg(sessionData.getPortletRequest()));
      if (expandedElem != null) {
        groupsManagerXml.expandGroupElementXML((Element) expandedElem.getParentNode(), sessionData);
      }
      // unlock and discard any other group that may be held in a locked state
      if((sessionData.lockedGroup!=null) && (!sessionData.lockedGroup.getEntityIdentifier().getKey().equals(sessionData.highlightedGroupID)) && (!GroupManagerMode.select.equals(sessionData.mode))){
         sessionData.lockedGroup.getLock().release();
         sessionData.lockedGroup = null;
         sessionData.mode = GroupManagerMode.browse;
      }
   }
}