/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.groups.commands;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.groups.ILockableEntityGroup;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.portal.servants.groups.GroupManagerMode;
import org.jasig.portal.servants.groups.GroupsManagerXml;
import org.jasig.portal.servants.groups.GroupsSessionData;
import org.jasig.portal.servants.groups.Utility;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/** This command delegates to the GroupsService to find entities requested
 *  by the user.
 * @author Don Fracapane
 * @version $Revision: 11911 $
 */
public class LockGroup extends GroupsManagerCommand {

   public LockGroup(ICompositeGroupService compositeGroupService) {
      super(compositeGroupService);
   }

   /**
    * This is the public method
    * @param sessionData
    * @throws Exception
    */
   public void execute (GroupsSessionData sessionData, GroupsManagerXml groupsManagerXml,IAuthorizationService authorizationService) throws Exception {

      Document model = getXmlDoc(sessionData);
      Utility.logMessage("DEBUG", "LockGroup::execute(): Start");
      String parentElemId = getCommandArg(sessionData.getPortletRequest());
      // if not IPerson group, then set view root to root for requested type

      String userID = getUserID(sessionData);
      //String userName = GroupsManagerXML.getEntityName(ENTITY_CLASSNAME, userID);
      // TODO: get a user key
      //String userKey = sessionData.user.getEntityIdentifier().getKey();
      String userKey = "";
      String lockKey = userID + "::" + userKey;
      Utility.logMessage("DEBUG", "LockGroup::execute(): lockKey = " + lockKey);
      Element parentElem = GroupsManagerXml.getElementById(model, parentElemId);
      String parentKey = parentElem.getAttribute("key");
      ILockableEntityGroup lockedGroup = getCompositeGroupService().findGroupWithLock(parentKey, lockKey);
      // if lockedGroup is null, an exception has probably already been thrown
      if (lockedGroup != null){
         groupsManagerXml.refreshAllNodesRecursivelyIfRequired(sessionData, parentElem);
         // store in sessionData
         sessionData.lockedGroup=lockedGroup;
         sessionData.mode = GroupManagerMode.edit;
         sessionData.setAttribute("groupParentId", parentElemId);
      }

      Utility.logMessage("DEBUG", "LockGroup::execute(): Uid of parent element = " +
            parentElemId);
   }
}
