/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.groups.commands;

import javax.portlet.PortletRequest;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.groups.IEntityGroup;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.gap.services.entitytypes.IEntityIdentifier;
import org.jasig.portal.servants.groups.GroupsManagerCommandFactory;
import org.jasig.portal.servants.groups.GroupsManagerXml;
import org.jasig.portal.servants.groups.GroupsSessionData;
import org.jasig.portal.servants.groups.SearchResultsGroupImpl;
import org.jasig.portal.servants.groups.Utility;
import org.w3c.dom.Document;
import org.w3c.dom.Element;


/** This command delegates to the GroupsService to find entities requested
 *  by the user.
 * @author Don Fracapane
 * @version $Revision: 11911 $
 */
public class Search extends GroupsManagerCommand {
    
   private static String grpPrefix = "IEntityGroup::";
   private static String[] methods;
   static {
      methods = new String[5];
      methods[1] = " is ";
      methods[2] = " starts with ";
      methods[3] = " ends with ";
      methods[4] = " contains ";
   }

   public Search(ICompositeGroupService compositeGroupService) {
       super(compositeGroupService);
   }

   /**
    * This is the public method
    * @param sessionData
    * @throws Exception
    */
   public void execute (GroupsSessionData sessionData,GroupsManagerXml groupsManagerXml,IAuthorizationService authorizationService) throws Exception {
       
      PortletRequest request = sessionData.getPortletRequest();
       
      Utility.logMessage("DEBUG", "SearchForEntities::execute(): Start");
      Class type;
      String grpTypeName = null;
      IEntityIdentifier[] results;
      String label = null;
      boolean isGroupSearch;
      // if present, the command arg will be the ancestor
      String ancestorKey = getCommandArg(request);
      IEntityGroup entGrp = Utility.retrieveGroup(ancestorKey,getCompositeGroupService());
      String query = request.getParameter("grpQuery");
      String method = request.getParameter("grpMethod");
      int methodInt = Integer.parseInt(method);
      // For an EntityGroup search, the grpType will have the  form of "IEntityGroup::classname"
      // For an Entity search, the grpType will have the  form of "classname"
      String grpType = request.getParameter("grpType");
      if (grpType.startsWith(grpPrefix)) {
         isGroupSearch = true;
         grpTypeName = grpType.substring(grpPrefix.length());
      }
      else {
         isGroupSearch = false;
         grpTypeName = grpType;
      }
      type = Class.forName(grpTypeName);
      if (isGroupSearch) {
         label = "Group of " + groupsManagerXml.getEntityTypes().getDescriptiveNameForType(type) + "s";
         if (entGrp != null) {
            results = getCompositeGroupService().searchForGroups(query, methodInt, type, entGrp);
         }
         else {
            results = getCompositeGroupService().searchForGroups(query, methodInt, type);
         }
      }
      else {
         label = groupsManagerXml.getEntityTypes().getDescriptiveNameForType(type);
         if (entGrp != null) {
            results = getCompositeGroupService().searchForEntities(query, methodInt, type, entGrp);
         }
         else {
            results = getCompositeGroupService().searchForEntities(query, methodInt, type);
         }
      }
      Document model = getXmlDoc(sessionData);
      IEntityGroup sr = new SearchResultsGroupImpl(type);
      sr.setName("Search Results");
      sr.setDescription("Search for a " + label + " whose name" + methods[methodInt] + "'" + query + "'");
      sr.setCreatorID(sessionData.getUserId().toString());
      for (int sub = 0; sub < results.length; sub++) {
         IEntityIdentifier entID = results[sub];
         IGroupMember resultGroup = getCompositeGroupService().getGroupMember(entID);
         sr.addMember(resultGroup);
      }
      Element searchElem = groupsManagerXml.getGroupMemberXml(sr, true, null, sessionData);
      searchElem.setAttribute("searchResults", "true");
      model.getDocumentElement().appendChild(searchElem);
      this.setCommandArg(request, searchElem.getAttribute("id"));
      GroupsManagerCommandFactory.get(HighlightGroup.class.getSimpleName(), getCompositeGroupService()).execute(sessionData,groupsManagerXml,authorizationService);
   }
  
}
