/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.servants.groups.commands;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.gap.services.entitytypes.IEntityIdentifier;
import org.jasig.gap.services.entitytypes.impl.EntityIdentifier;
import org.jasig.portal.servants.groups.GroupsSessionData;
import org.jasig.portal.servants.groups.GroupsManagerXml;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * A Groups Manager command to expose properties from the EntityPropertyRegistry
 * for any entity or group
 *
 * @author Alex Vigdor
 * @version $Revision: 11911 $
 */

public class ShowProperties extends GroupsManagerCommand {

   public ShowProperties(ICompositeGroupService compositeGroupService) {
      super(compositeGroupService);
   }

   /**
    * put your documentation comment here
    * @param sessionData
    * @throws Exception
    */
  public void execute (GroupsSessionData sessionData,GroupsManagerXml groupsManagerXml,IAuthorizationService authorizationService) throws Exception{
    Document model = getXmlDoc(sessionData);
    String id = this.getCommandArg(sessionData.getPortletRequest());
    Element e = GroupsManagerXml.getElementById(model,id);
    if (e != null){
      Element props = model.createElement(PROPERTIES_TAGNAME);
      IEntityIdentifier ei = null;
      try{
        ei = new EntityIdentifier(e.getAttribute("key"),Class.forName(e.getAttribute("type")));
      }
      catch (Exception ce){
        throw new RuntimeException("Unable to instantiate class:  type "+e.getAttribute("type")+" unknown",ce);
      }
      String[] names = groupsManagerXml.getEntityPropertyRegistry().getPropertyNames(ei);
      //System.out.println("Found "+names.length+" properties");
      for(int i=0; i<names.length;i++){
          Element prop = model.createElement("property");
          prop.setAttribute("name",names[i]);
          prop.setAttribute("value",groupsManagerXml.getEntityPropertyRegistry().getProperty(ei,names[i]));
          props.appendChild(prop);
      }
      e.appendChild(props);
    }
  }
}