/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.servants.groups.commands;

import org.jasig.gap.groups.ICompositeGroupService;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.portal.servants.groups.GroupManagerMode;
import org.jasig.portal.servants.groups.GroupsSessionData;
import org.jasig.portal.servants.groups.GroupsManagerXml;
import org.jasig.portal.servants.groups.Utility;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * A Groups Manager command to release a lock on a group, return to browse mode
 *
 * @author Alex Vigdor
 * @version $Revision: 11911 $
 */

public class UnlockGroup extends GroupsManagerCommand{


   public UnlockGroup(ICompositeGroupService compositeGroupService) {
      super(compositeGroupService);
   }

   public void execute (GroupsSessionData sessionData,GroupsManagerXml groupsManagerXml,IAuthorizationService authorizationService) throws Exception {
      Element parentElem = null;
      sessionData.mode = GroupManagerMode.browse;
      Document model = sessionData.getModel();
      String key = sessionData.lockedGroup.getLock().getEntityKey();
      Utility.logMessage("DEBUG", "UnlockGroup::execute(): Locked group key = " + key);
      sessionData.lockedGroup.getLock().release();
      sessionData.lockedGroup = null;
      String parentID = getParentId(sessionData);

      // Parent was locked so no other thread or process could have changed it, but
      // child members could have changed.
      // Parent element id is not always set.
      if (!Utility.areEqual(parentID, "")){
         parentElem = GroupsManagerXml.getElementById(model, parentID);
         sessionData.getPortletSession().removeAttribute("groupParentId");
      }
      Utility.logMessage("DEBUG", "UnlockGroup::execute(): parentElem = " + parentElem);
      if (parentElem != null){
         groupsManagerXml.refreshAllNodesRecursivelyIfRequired(sessionData, parentElem);
      }
   }
}