/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.permissions;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;

import javax.portlet.PortletRequest;
import javax.portlet.PortletResponse;
import javax.portlet.PortletSession;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.gap.groups.IEntityGroup;
import org.jasig.gap.groups.IGroupMember;
import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.gap.permissions.IAuthorizationService;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;
import org.jasig.portal.event.PortalEvent;
import org.jasig.portal.servants.groups.IDistinguishedGroupConstants;

/**
 * CPermissionsManager allows graphical administration of permissions for all owners
 * that have a representative implementation of IPermissible recorded in the table
 * UPC_PERM_MGR
 *
 * @author Alex Vigdor
 * @version $Revision: 11911 $
 */
public class PermissionsManager implements ICacheable {
    
    private static final Log log = LogFactory.getLog(PermissionsManager.class);
    
    protected PermissionsSessionData session;
    protected final String sslLocation = "CPermissionsManager.ssl";
    
    // IoC injection
    private PermissionsManagerXml permissionsManagerXml;
    private IAuthorizationService authorizationService;


    /**
     * put your documentation comment here
     * @param rD
     */
    public void initRequestProcessing (PortletRequest request) {
        session.startRD = Calendar.getInstance().getTime().getTime();
        session.setPortletRequest(request);
        log.debug("PermissionsManager - setting runtime data");
        // test if servant exists and has finished
        if (session.servant != null){
          try {
            if (session.servant.isComplete(request)) {
              getGroupServantResults(session,request);
            }
          }
          catch (Exception e){
            log.error(e, e);
          }
        }

        String prmCommand = request.getParameter("prmCommand");
        if (prmCommand != null) {
            IPermissionCommand pc = CommandFactory.get(prmCommand);
            if (pc != null) {
              try{
                pc.execute(session,authorizationService);
              }
              catch(Exception e){
                log.error("Error executing command [" + prmCommand + "]", e);
                // TODO: set attribute to the request??
                session.getPortletRequest().setAttribute("commandResponse","Error executing command "+ prmCommand +": "+e.getMessage());
              }
            }
        }


        if ((!session.gotOwners)) {
            session.view="Select Owners";
        }
        if ((session.gotOwners) && (!session.gotActivities)) {
            PermissionsManagerXml.autoSelectSingleChoice(session,"activity");
            session.view="Select Activities";
        }
        if (session.gotActivities && !session.gotTargets) {
            PermissionsManagerXml.autoSelectSingleChoice(session,"target");
            session.view="Select Targets";
        }
        if (session.gotTargets && (session.principals == null)) {
            log.debug("PermissionsManager - Checking Servant");
            prepServant(session);
        }
        if (session.principals != null){
            if (session.getPortletSession().getAttribute("prmView") != null) {
              session.view = (String) session.getPortletSession().getAttribute("prmView");
              session.getPortletSession().removeAttribute("prmView");
            }
            else if (request.getParameter("prmView") != null) {
              session.view=request.getParameter("prmView");
            }
            else if (!session.view.startsWith("Assign")){
              session.view="Assign By Principal";
            }
        }
    }

    protected void prepServant(PermissionsSessionData session) {
      // TODO: implement initialization of the groups session data and the rest
      /*try {
          if (session.servant == null) {

              // get a different version of the servant depnding on whether Owners have been retrieved
              if (!session.gotOwners){
                log.debug("PermissionsManager - creating new basic Servant");
                session.servant =
                  GroupsSessionDataFactory.getGroupsServantforSelection(
                    "Select principals you would like to assign permissions to",
                    IGroupConstants.EVERYONE);
              }
              else {
                  // build an array of groupmembers for pre-selection
                  log.debug("PermissionsManager - creating new pre-selecting Servant");
                  ArrayList gmembers = new ArrayList();
                  Element[] owners = PermissionsManagerXml.getSelectedOwners(session);
                  for (int j= 0; j < owners.length ; j++){
                    if (log.isDebugEnabled())
                      log.debug("analyzing owner "+owners[j].getAttribute("name"));
                    String ownerKey = owners[j].getAttribute("token");
                    IPermissionManager pm = authorizationService.newPermissionManager(ownerKey);
                    String[] acts = null;
                    if (session.gotActivities){
                      acts = PermissionsManagerXml.getSelectedActivities(session,owners[j]);
                    }
                    String[] tgts = null;
                    if (session.gotTargets){
                       tgts = PermissionsManagerXml.getSelectedTargets(session,owners[j]);
                    }

                    if (acts != null){
                      for (int a = 0; a< acts.length ; a++){
                        if (tgts != null){
                           for (int t=0; t< tgts.length; t++){
                              populateMembers(gmembers,pm.getAuthorizedPrincipals(acts[a],tgts[t]));
                           }
                        }
                        else {
                            populateMembers(gmembers,pm.getAuthorizedPrincipals(acts[a], null));
                        }
                      }
                    }
                    else {
                      if (tgts !=null){
                         for (int t=0; t< tgts.length; t++){
                            populateMembers(gmembers,pm.getAuthorizedPrincipals(null,tgts[t]));
                         }
                      }
                      else {
                          populateMembers(gmembers,pm.getAuthorizedPrincipals(null, null));
                      }
                    }

                    // use pre-populated list to get servant
                    session.servant = GroupsSessionDataFactory.getGroupsServantforSelection(
                      "You may view principals with existing permissions on the items you have selected by clicking \"Done\", or use the select and deselect buttons to add or remove principals for whom you would like to view/assign permissions",
                      IGroupConstants.EVERYONE,true,true,
                      (IGroupMember[])gmembers.toArray(new IGroupMember[0]));
                  }
              }
              session.view="Select Principals";
          }
          else {
              log.debug("PermissionsManager - using existing Servant");
          }



      } catch (Exception e) {
          log.error(e, e);
      }*/
    }

    protected void populateMembers(ArrayList gmembers, IAuthorizationPrincipal[] aps){
      if (log.isDebugEnabled())
          log.debug("PermissionsManager.PopulateMembers(): checking principal set of size"+aps.length);
      for (int a = 0; a< aps.length ; a++){
        try {
          IGroupMember agm = authorizationService.getGroupMember(aps[a]);
          if (log.isDebugEnabled())
              log.debug("PermissionsManager.PopulateMembers(): checking whether "+
                      agm.getType()+"."+agm.getKey()+" needs to be added");
          if (!gmembers.contains(agm)){
            gmembers.add(agm);
          }
        }
        catch(Exception e){
          log.error(e, e);
        }
      }
    }

    protected void getGroupServantResults(PermissionsSessionData session, PortletRequest request){
      /*try {
        log.debug("PermissionsManager - Getting servant results");
        Object[] results = session.servant.getResults(request);
        if (results != null && results.length > 0) {
            IAuthorizationPrincipal[] iap = new IAuthorizationPrincipal[results.length];
            for (int i = 0; i< results.length ; i++){
              IGroupMember gm = (IGroupMember) results[i];
              iap[i] = authorizationService.newPrincipal(gm);
            }
            session.principals=iap;
            log.debug("PermissionsManager - Getting rid of Servant");
            session.servant=null;
            permissionsManagerXml.populatePrincipals(session);
        }
        else {
            log.debug("PermissionsManager - Group Servant yielded no results, assuming abort and running Cancel");
            IPermissionCommand cmd = CommandFactory.get("Cancel");
            cmd.execute(session,authorizationService);
        }
      }
      catch (Exception e) {
        log.error(e, e);
      }*/
    }

    /**
     * put your documentation comment here
     * @param portalEvent
     */
    public void receiveEvent (PortalEvent portalEvent) {}

    /**
     * put your documentation comment here
     * @param out
     */
    public void process (PortletRequest request, PortletResponse response) {
        try {
            long time1 = Calendar.getInstance().getTime().getTime();
            if (session.view.equals("Select Principals") &&
                    session.isAuthorized) {
                try {
                    log.debug("PermissionsManager - Calling servant renderXML");
                    //IChannel servant = (IChannel)staticData.get("prmServant");
                    session.servant.render((RenderRequest)request,(RenderResponse)response);
                } catch (Exception e) {
                    log.error( "CPermissionsManager: failed to use servant"
                            + e);
                }
            }
            if (!session.view.equals("Select Principals")
                    || !session.isAuthorized) {
                long time2 = Calendar.getInstance().getTime().getTime();
                // TODO
                /*XSLT xslt = XSLT.getTransformer(this, session.runtimeData.getLocales());
                xslt.setXML(PermissionsXml.getViewDoc(session));
                xslt.setTarget(out);
                xslt.setStylesheetParameter("baseActionURL", session.runtimeData.getBaseActionURL());
                xslt.setStylesheetParameter("prmView", session.view);
                if (session.runtimeData.get("commandResponse") != null) {
                    xslt.setStylesheetParameter("commandResponse", session.runtimeData.getParameter("commandResponse"));
                }
                xslt.setXSL(sslLocation, "CPermissions", session.runtimeData.getBrowserInfo());
                transform(xslt);
                if (log.isDebugEnabled()) {
                    long time3 = Calendar.getInstance().getTime().getTime();
                    log.debug("CPermissionsManager timer: "
                            + String.valueOf((time3 - time1)) + " ms total, xsl took "
                            + String.valueOf((time3 - time2)) + " ms for view " + session.view);
                    log.debug("CPermissionsManager timer: "
                            + String.valueOf((time3 - session.startRD)) + " since start RD");
                }
               */
            }
        } catch (Exception e) {
            log.error(e, e);
        }
    }

    /**
     * put your documentation comment here
     * @param xslt
     */
    // TODO
    /*protected void transform (XSLT xslt) {
        try {
            if (session.isAuthorized) {
                xslt.setStylesheetParameter("isAdminUser", "true");
            }
            xslt.transform();
        } catch (Exception e) {
            log.error(e, e);
        }
    }*/

    /**
     * put your documentation comment here
     * @param sD
     */
    protected void setPortletSession (PortletSession portletSession) {
        this.session = new PermissionsSessionData();
        try {
            //IEntityGroup admin = GroupServiceFacade.getDistinguishedGroup(IDistinguishedGroupConstants.PORTAL_ADMINISTRATORS);
            // TODO: get authorizationPrincipal from the portlet session
            /*IGroupMember me = AuthorizationService.instance().getGroupMember(this.session.staticData.getAuthorizationPrincipal());
            if (admin.deepContains(me)) {
                session.isAuthorized = true;
            }*/
        } catch (Exception e) {
            log.error(e, e);
        }
        session.isFinished=false;

    }
    
    /**
     * put your documentation comment here
     * 
     * @param validity
     * @return <code>true</code> if cache is valid; <code>false</code> otherwise
     */
    public boolean isCacheValid (Serializable validity) {
        boolean valid = false;
        if (session.servant == null) {
            if (validity != null) {
                if (validity.equals(session.getPortletRequest().getParameter("prmView")) && session.getPortletRequest().getAttribute("commandResponse")
                        == null) {
                    valid = true;
                }
            }
            if (log.isDebugEnabled()) {
                long time3 = Calendar.getInstance().getTime().getTime();
                log.debug("CPermissionsManager.isCacheValid() time since setRD: "
                        + String.valueOf((time3 - session.startRD)) + ", valid=" + valid);
            }

        }
        else {
            valid = ((ICacheable)session.servant).isCacheValid(validity);
        }
        return  valid;
    }
    
    /** (non-Javadoc)
     * @see org.jasig.portal.caching.ICacheable#generateKey()
     */
    public ICacheKey generateKey() {
       // TODO
       return null;
    }
    /**
     * @param permissionsManagerXml The permissionsManagerXml to set.
     */
    public void setPermissionsManagerXml(PermissionsManagerXml permissionsManagerXml) {
        this.permissionsManagerXml = permissionsManagerXml;
    }
    /**
     * @param authorizationService The authorizationService to set.
     */
    public void setAuthorizationService(IAuthorizationService authorizationService) {
        this.authorizationService = authorizationService;
    }
}



