/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.permissions;

import java.io.IOException;
import java.util.HashMap;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletException;
import javax.portlet.PortletRequest;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.jasig.portal.groups.security.IPermissible;
import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.gap.permissions.IPermission;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
//import org.jasig.portal.utils.XSLT;


/**
 * CPermissionsManagerServant is an IServant subclass of CPermissionsManager
 * This will allow other channels to delegate to CPermissionsManager at runtime
 *
 * Master channels should instantiate this channel with the following
 * staticData parameter preset:
 *
 * prmOwners = IPermissible[] owners
 *
 * where owners is an array with a single element being an instance of the
 * master's representative IPermissible class.
 *
 * see org.jasig.portal.IPermissible for more information
 *
 * @author Alex Vigdor
 * @version $Revision: 11911 $
 */
public class PermissionsManagerServant extends PermissionsManager {
    
    private static final Log log = LogFactory.getLog(PermissionsManagerServant.class);
    
    private IPermission[] results;
    
    // IoC injection
    private PermissionsManagerXml permissionsManagerXml;

    /** Creates new CPermissionsManagerServant */
    public PermissionsManagerServant () {
      super();
    }
    
    public String getId() {
      // TODO  
      return null;
    }

    /**
     * put your documentation comment here
     * @param xslt
     */
    // TODO
    /*protected void transform (XSLT xslt) {
        if (!isFinished()) {
            try {
                xslt.setStylesheetParameter("isAdminUser", "true");
                xslt.transform();
            } catch (Exception e) {
                log.debug(e, e);
            }
        }
    }*/
    
    public void render(RenderRequest request, RenderResponse response) throws PortletException, IOException {
        initRequestProcessing (request);
       // TODO
    }

    public void processAction(ActionRequest request, ActionResponse response) throws PortletException, IOException {
        initRequestProcessing (request);
       // TODO 
    }

    /**
     * put your documentation comment here
     * 
     * @return <code>true</code> if finished; <code>false</code> otherwise
     */
    public boolean isComplete (PortletRequest request) {
        return  session.isFinished;
    }

    /**
     * put your documentation comment here
     * Currently returns and empy array
     * @return the results
     */
    public Object[] getResults (PortletRequest request) {
        return  results;
    }

    public void initRequestProcessing (PortletRequest request) {

        super.setPortletSession(request.getPortletSession(false));
        
        session.isAuthorized = true;

        // handle pre-selection for servant mode
        if (session.getPortletSession().getAttribute("prmOwners") != null) {
            log.debug("PermissionServant.setStaticData(): processing pre-selection");

            // use specified set of owners
            IPermissible[] selOwners = (IPermissible[])session.getPortletSession().getAttribute("prmOwners");
            for (int j =0; j < selOwners.length ; j++){
              PermissionsManagerXml.setSelected(session,selOwners[j].getClass().getName(),"owner", selOwners[j].getOwnerToken(),true);

              if (session.getPortletSession().getAttribute("prmTargets") != null) {
                  if (((HashMap)session.getPortletSession().getAttribute("prmTargets")).get(selOwners[j].getOwnerToken())
                          != null) {
                    String[] selTargets = (String[])((HashMap)session.getPortletSession().getAttribute("prmTargets")).get(selOwners[j].getOwnerToken());
                    if (log.isDebugEnabled())
                        log.debug("PermissionServant.setStaticData(): got "+selTargets.length+" pre-selected targets");
                    for (int s=0;s<selTargets.length;s++){
                      PermissionsManagerXml.setSelected(session,selOwners[j].getClass().getName(),"target",selTargets[s],true);
                    }
                    session.gotTargets = true;
                  }
                  else {
                    log.debug("PermissionServant.setStaticData(): error retrieving pre-selected targets");
                  }
              }

              if (session.getPortletSession().getAttribute("prmActivities") != null) {
                  if (((HashMap)session.getPortletSession().getAttribute("prmActivities")).get(selOwners[j].getOwnerToken())
                          != null) {
                      String[] selActivities = (String[])((HashMap)session.getPortletSession().getAttribute("prmActivities")).get(selOwners[j].getOwnerToken());
                      if (log.isDebugEnabled())
                          log.debug("PermissionServant.setStaticData(): got "+selActivities.length+" pre-selected activities");
                      for (int s=0;s<selActivities.length;s++){
                        PermissionsManagerXml.setSelected(session,selOwners[j].getClass().getName(),"activity",selActivities[s],true);
                      }
                      session.gotActivities = true;
                  }
                  else {
                    log.debug("PermissionServant.setStaticData(): error retrieving pre-selected activities");
                  }
              }

            }
            session.gotOwners = true;
        }
        if (session.getPortletSession().getAttribute("prmPrincipals")!=null){
          session.principals = (IAuthorizationPrincipal[]) session.getPortletSession().getAttribute("prmPrincipals");
          permissionsManagerXml.populatePrincipals(session);
        }
    }
    
    /**
     * @param permissionsManagerXml The permissionsManagerXml to set.
     */
    public void setPermissionsManagerXml(PermissionsManagerXml permissionsManagerXml) {
        this.permissionsManagerXml = permissionsManagerXml;
    }
}



