/* Copyright 2001 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package  org.jasig.portal.servants.permissions;

import java.util.Calendar;
import java.util.HashMap;

import javax.portlet.PortletSession;
import javax.portlet.PortletRequest;

//import org.jasig.portal.ChannelStaticData;
import org.jasig.portal.groups.security.IPermissible;
import org.jasig.gap.permissions.IAuthorizationPrincipal;
import org.jasig.portal.portlet.servant.IPortletServant;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;


/**
 * CPermissionsManagerServantFactory
 *
 * calling getPermissionsServant will return an instance of the default
 * CPermissionsManagerServant implementation
 *
 * @author Alex Vigdor
 * @version $Revision: 11911 $
 */
public class PermissionsManagerServantFactory {
    
    private static final Log log = LogFactory.getLog(PermissionsManagerServantFactory.class);
    private static PermissionsManagerServantFactory _instance;
    private HashMap servantClasses = new HashMap();

    /** Creates new CPermissionsManagerServantFactory */
    protected PermissionsManagerServantFactory () {
    }

    /**
     * put your documentation comment here
     * @return
     */
    static IPortletServant getPermissionsServant () {
        return  getPermissionsServant("CPermissionsManagerServant");
    }

    /**
     * put your documentation comment here
     * @param name
     * @return
     */
    protected static IPortletServant getPermissionsServant (String name) {
        return  instance().getServant(name);
    }

    /**
     * put your documentation comment here
     * @param owner
     * @param principals
     * @param activities
     * @param targets
     * @param staticData
     * @return the permissions servant
     * @exception org.jasig.portal.PortalException
     */
    public static IPortletServant getPermissionsServant (IPermissible owner, PortletRequest request, IAuthorizationPrincipal[] principals,
            String[] activities, String[] targets) {
        boolean isOK = true;
        long time1 = Calendar.getInstance().getTime().getTime();
        RDBMPermissibleRegistry.registerPermissible(owner.getClass().getName());

        IPortletServant servant = getPermissionsServant();
        IPermissible[] owners = new IPermissible[1];
        owners[0] = owner;
        if (activities == null) {
            activities = owner.getActivityTokens(null);
        }
        if (targets == null) {
            targets = owner.getTargetTokens();
        }
        PortletSession session = request.getPortletSession(false);
        if (session != null) {
            try {
                HashMap tHash = new HashMap(1);
                tHash.put(owner.getOwnerToken(), targets);
                HashMap aHash = new HashMap(1);
                aHash.put(owner.getOwnerToken(), activities);
                if (principals != null) {
                    session.setAttribute("prmPrincipals", principals);
                }
                session.setAttribute("prmOwners", owners);
                session.setAttribute("prmActivities", aHash);
                session.setAttribute("prmTargets", tHash);
                session.setAttribute("prmView", "Assign By Owner");
            } catch (Exception e) {
                isOK = false;
            }
        }
        else {
            isOK = false;
        }
        if (!isOK) {
            throw  (new RuntimeException("CPermissionsServantFactory.getPermissionsServant():: unable to properly initialize servant, check that mast staticData is being properly passed to this method"));
        }
        if (log.isInfoEnabled()) {
            long time2 = Calendar.getInstance().getTime().getTime();
            log.info( "CPermissionsManagerFactory took  "
                    + String.valueOf((time2 - time1)) + " ms to instantiate");
        }

        return  servant;
    }

    /**
     * put your documentation comment here
     * @param name
     * @return
     */
    protected IPortletServant getServant (String name) {
        IPortletServant rs = null;
        if (servantClasses.get(name) == null) {
            try {
                Class cserv = Class.forName(PermissionsManager.class.getPackage().getName()+"."+name);
                servantClasses.put(name, cserv);
            } catch (Exception e) {
                log.error(e, e);
            }
        }
        if (servantClasses.get(name) != null) {
            try {
                rs = (IPortletServant)((Class)servantClasses.get(name)).newInstance();
            } catch (Exception e) {
                log.error(e, e);
            }
        }
        return  rs;
    }

    /**
     * put your documentation comment here
     * @return
     */
    protected static synchronized PermissionsManagerServantFactory instance () {
        if (_instance == null) {
            _instance = new PermissionsManagerServantFactory();
        }
        return  _instance;
    }
}



