/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.spring;

import org.springframework.beans.factory.BeanFactory;

/**
 * Tracks the resolved context and leaf bean name. If the name of the bean equals
 * {@link org.jasig.portal.utils.lang.StringUtils#CURRENT_ELEMENT_NAME} the resolved
 * bean is the ownerFactory from the answer. 
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class ContextResolverAnswer {
    private final String leafBeanName;
    private final BeanFactory ownerFactory;
    
    /**
     * Creates a new answer.
     * 
     * @param leafBeanName
     * @param ownerFactory
     */
    public ContextResolverAnswer(String leafBeanName, BeanFactory ownerFactory) {
        this.leafBeanName = leafBeanName;
        this.ownerFactory = ownerFactory;
    }

    /**
     * @return Returns the leafBeanName.
     */
    public String getLeafBeanName() {
        return this.leafBeanName;
    }

    /**
     * @return Returns the ownerFactory.
     */
    public BeanFactory getOwnerFactory() {
        return this.ownerFactory;
    }
}
