/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.spring;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

/**
 * ContextResolvingClasspathXmlApplicationContext that supports hierarchical bean references.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class ContextResolvingClasspathXmlApplicationContext extends ClassPathXmlApplicationContext implements IContextResolvingBeanFactory {
    private IContextResolver contextResolver;
    
    /**
     * @see ClassPathXmlApplicationContext#ClassPathXmlApplicationContext(java.lang.String)
     */
    public ContextResolvingClasspathXmlApplicationContext(String configLocation) throws BeansException {
        super(configLocation);
    }

    /**
     * @see ClassPathXmlApplicationContext#ClassPathXmlApplicationContext(java.lang.String[])
     */
    public ContextResolvingClasspathXmlApplicationContext(String[] configLocations) throws BeansException {
        super(configLocations);
    }

    /**
     * @see ClassPathXmlApplicationContext#ClassPathXmlApplicationContext(java.lang.String[], boolean)
     */
    public ContextResolvingClasspathXmlApplicationContext(String[] configLocations, boolean refresh) throws BeansException {
        super(configLocations, refresh);
    }
    
    /**
     * @see ClassPathXmlApplicationContext#ClassPathXmlApplicationContext(java.lang.String[], boolean)
     */
    public ContextResolvingClasspathXmlApplicationContext(String configLocation, boolean refresh) throws BeansException {
        super(new String[] {configLocation}, refresh);
    }

    /**
     * @see ClassPathXmlApplicationContext#ClassPathXmlApplicationContext(java.lang.String[], org.springframework.context.ApplicationContext)
     */
    public ContextResolvingClasspathXmlApplicationContext(String[] configLocations, ApplicationContext parent) throws BeansException {
        super(configLocations, parent);
    }
    
    /**
     * @see ClassPathXmlApplicationContext#ClassPathXmlApplicationContext(java.lang.String[], org.springframework.context.ApplicationContext)
     */
    public ContextResolvingClasspathXmlApplicationContext(String configLocation, ApplicationContext parent) throws BeansException {
        super(new String[] {configLocation}, parent);
    }

    /**
     * @see ClassPathXmlApplicationContext#ClassPathXmlApplicationContext(java.lang.String[], boolean, org.springframework.context.ApplicationContext)
     */
    public ContextResolvingClasspathXmlApplicationContext(String[] configLocations, boolean refresh, ApplicationContext parent) throws BeansException {
        super(configLocations, refresh, parent);
    }

    /**
     * @see org.springframework.context.support.AbstractRefreshableApplicationContext#createBeanFactory()
     */
    protected DefaultListableBeanFactory createBeanFactory() {
        final ContextResolvingDefaultListableBeanFactory resolvingFactory = new ContextResolvingDefaultListableBeanFactory(this, getInternalParentBeanFactory());
        resolvingFactory.setContextResolver(this.getContextResolver());
        return resolvingFactory;
    }
    
    public IContextResolvingBeanFactory getItself() {
        return this;
    }

    public synchronized IContextResolver getContextResolver() {
        if (this.contextResolver == null) {
            this.contextResolver = new ContextResolverImpl();
        }
        
        return this.contextResolver;
    }

    /**
     * @param contextResolver The contextResolver to set.
     */
    public void setContextResolver(IContextResolver contextResolver) {
        this.contextResolver = contextResolver;
    }
}
