/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.spring;

import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.web.portlet.context.XmlPortletApplicationContext;

/**
 * An extension of the XmlWebApplicationContext that resolves hierarchical reference syntax.
 *
 * @author Peter Kharchenko, <pkharchenko at unicon.com>
 * @version $Revision: 11911 $
 */
public class ContextResolvingXmlPortletApplicationContext extends XmlPortletApplicationContext implements IContextResolvingBeanFactory {
    private IContextResolver contextResolver;
    
    /**
     * @see org.springframework.context.support.AbstractRefreshableApplicationContext#createBeanFactory()
     */
    protected DefaultListableBeanFactory createBeanFactory() {
        final ContextResolvingDefaultListableBeanFactory resolvingFactory = new ContextResolvingDefaultListableBeanFactory(this, getInternalParentBeanFactory());
        resolvingFactory.setContextResolver(this.getContextResolver());
        return resolvingFactory;
    }
    
    public IContextResolvingBeanFactory getItself() {
        return this;
    }

    public synchronized IContextResolver getContextResolver() {
        if (this.contextResolver == null) {
            this.contextResolver = new ContextResolverImpl();
        }
        
        return this.contextResolver;
    }

    /**
     * @param contextResolver The contextResolver to set.
     */
    public void setContextResolver(IContextResolver contextResolver) {
        this.contextResolver = contextResolver;
    }
}
