/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.spring;

import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.context.support.AbstractApplicationContext;
import org.springframework.web.context.support.AbstractRefreshableWebApplicationContext;
import org.springframework.web.context.support.XmlWebApplicationContext;

/**
 * An extension of the XmlWebApplicationContext that resolves hierarchical reference syntax.
 *
 * @author Peter Kharchenko, <pkharchenko at unicon.com>
 * @version $Revision: 11911 $
 */
public class ContextResolvingXmlWebApplicationContext extends XmlWebApplicationContext implements IContextResolvingBeanFactory {
    private IContextResolver contextResolver;
    
    /**
     * @see XmlWebApplicationContext#XmlWebApplicationContext()
     */
    public ContextResolvingXmlWebApplicationContext() {
        super();
    }
    
    /**
     * Calls {@link AbstractApplicationContext#setParent(org.springframework.context.ApplicationContext)},
     * {@link AbstractRefreshableWebApplicationContext#setServletContext(javax.servlet.ServletContext)},
     * {@link AbstractRefreshableWebApplicationContext#setNamespace(java.lang.String)} then
     * {@link AbstractRefreshableWebApplicationContext#refresh()}.
     * 
     * @param namespace
     * @param parentContext
     * @see XmlWebApplicationContext#XmlWebApplicationContext()
     */
    public ContextResolvingXmlWebApplicationContext(String namespace, XmlWebApplicationContext parentContext) {
        super();
        this.setParent(parentContext);
        this.setServletContext(parentContext.getServletContext());
        this.setNamespace(namespace);
        super.refresh();
    }
    
    
    /**
     * @see org.springframework.context.support.AbstractRefreshableApplicationContext#createBeanFactory()
     */
    protected DefaultListableBeanFactory createBeanFactory() {
        final ContextResolvingDefaultListableBeanFactory resolvingFactory = new ContextResolvingDefaultListableBeanFactory(this, getInternalParentBeanFactory());
        resolvingFactory.setContextResolver(new ContextResolverImpl());
        return resolvingFactory;
    }
    
    public IContextResolvingBeanFactory getItself() {
        return this;
    }

    public synchronized IContextResolver getContextResolver() {
        if (this.contextResolver == null) {
            this.contextResolver = new ContextResolverImpl();
        }
        
        return this.contextResolver;
    }

    /**
     * @param contextResolver The contextResolver to set.
     */
    public void setContextResolver(IContextResolver contextResolver) {
        this.contextResolver = contextResolver;
    }
}
