/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.spring;

import org.springframework.beans.factory.NoSuchBeanDefinitionException;

/**
 * IContextResolver defines an interfaces for the BeanFactory context resolver helper classes
 *
 * @author Peter Kharchenko, <pkharchenko at unicon.com>
 * @version $Revision: 11911 $
 */
public interface IContextResolver {
    
    /**
     * Takes a base factory and absolute or relative hierarchical bean name and resolves the
     * leaf bean name and parent factory.
     * 
     * @param factory The factory to start resolution with.
     * @param beanName The full name of the bean to resolve
     * @return The answer with the factory and leaf bean name.
     * @throws NoSuchBeanDefinitionException If an invalid {@link org.springframework.beans.factory.HierarchicalBeanFactory} is specified in the beanName, no check for a valid leaf bean is done. 
     */
    public ContextResolverAnswer resolveFactoryAndName(IContextResolvingBeanFactory factory, String beanName) throws NoSuchBeanDefinitionException;

}