package org.jasig.portal.spring;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.springframework.context.i18n.LocaleContext;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.portlet.DispatcherPortlet;

/**
 * Works around thread-local LocaleContext and RequestAttributes handling in
 * doActionService() and doRenderService(). Standard implementations of those
 * methods replace but do not restore thread-local LocaleContext and RequestAttributes 
 * references such that framework processing following DispatcherPortlet invokation
 * may fail if it depends on session-scoped beans. 
 * 
 * @author dmccallum
 *
 */
public class PoliteDispatcherPortlet extends DispatcherPortlet {
	
	@Override
	protected void doActionService(ActionRequest request, ActionResponse response) throws Exception {
		
		LocaleContext currentLocaleContext = LocaleContextHolder.getLocaleContext();
		RequestAttributes currentRequestAttributes = RequestContextHolder.getRequestAttributes();
		
		try {
		    super.doActionService(request, response);
		} finally {
			LocaleContextHolder.setLocaleContext(currentLocaleContext);
			RequestContextHolder.setRequestAttributes(currentRequestAttributes);
		}
		
	}
	
	@Override
	protected void doRenderService(RenderRequest request, RenderResponse response) throws Exception {
		
		LocaleContext currentLocaleContext = LocaleContextHolder.getLocaleContext();
		
		try {
		    super.doRenderService(request, response);
		} finally {
			LocaleContextHolder.setLocaleContext(currentLocaleContext);
		}
		
	}

}
