package org.jasig.portal.spring;

import org.jasig.portal.utils.ThreadLocalCopyRunnable;
import org.springframework.context.i18n.LocaleContext;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;

/**
 * A Spring-specific mechanism for transfering request-specific thread local 
 * objects from a parent thread to a Runnable assumed to execute in a child 
 * thread.
 * 
 * @author dmccallum
 *
 */
public class RequestContextAwareThreadLocalCopyRunnable extends ThreadLocalCopyRunnable {
	
	private RequestAttributes requestAttributes;
	private LocaleContext localeContext;
	
	public RequestContextAwareThreadLocalCopyRunnable(Runnable wrappedRunnable) {
		this(wrappedRunnable, false);
	}
	
	public RequestContextAwareThreadLocalCopyRunnable(Runnable wrappedRunnable, boolean runOnce) {
		super(wrappedRunnable, runOnce);
		requestAttributes = RequestContextHolder.getRequestAttributes();
		localeContext = LocaleContextHolder.getLocaleContext();
	}
	
	public void run() {
		
		RequestContextHolder.setRequestAttributes(requestAttributes);
		LocaleContextHolder.setLocaleContext(localeContext);
		try {
		    super.run();
		} finally {
			RequestContextHolder.setRequestAttributes(null);
			LocaleContextHolder.setLocaleContext(null);
			
			if ( isRunOnce() ) {
				requestAttributes = null;
				localeContext = null;
			}
			
		}
		
	}

}
