/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.spring;

import javax.servlet.ServletConfig;


/**
 * Simple proxy bean which provides access to the ServletConfig object. Other beans
 * can reference this bean in the bean def XML and use it to get the ServletConfig
 * after the associated servlet has been initialized.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class ServletConfigProviderBean implements IServletConfigProviderBean {
    private ServletConfig servletConfig = null;
    
    /**
     * @see org.jasig.portal.spring.IServletConfigProvider#getServletConfig()
     * @throws IllegalStateException If setServletConfig has not been called first.
     */
    public ServletConfig getServletConfig() {
        if (this.servletConfig == null)
            throw new IllegalStateException("setServletConfig must be called first.");

        return this.servletConfig;
    }
    
    /**
     * @see org.jasig.portal.spring.IServletConfigProviderBean#setServletConfig(javax.servlet.ServletConfig)
     * @throws IllegalArgumentException If config is null.
     */
    public void setServletConfig(final ServletConfig config) {
        if (config == null)
            throw new IllegalArgumentException("config cannot be null.");

        this.servletConfig = config;
    }
}
