/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.tools.dbloader;

import java.net.MalformedURLException;


/**
 * Loads the dbLoaderContext, parses the command line
 * parameters, configures the objects and calls the process
 * method on DbLoader
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class DbLoaderRunner {
    public static void main(String[] args) {
        DbLoader loader = null;
        
        try {
            loader = DbLoader.newConfiguredInstance();
            
            //read command line arguements to override properties in context
            readOverrides(loader.getConfig(), args);

            //Run loader
            loader.process();
        }
        catch (Exception e) {
            if (loader != null && loader.getConfig() != null)
                loader.logger.error("Error running dbLoader", e);
            else
                e.printStackTrace(System.out);
        }
    }

    /**
     * Reads an array of command line parameters and updates the configuration
     * object.
     * 
     * @param config
     * @param args
     * @throws MalformedURLException
     */
    public static void readOverrides(Configuration config, String[] args) throws MalformedURLException {
        boolean useTableXml = false;
        boolean useDataXml = false;
        boolean upgrade = false;
//TODO update the possible command line args
        for (int i = 0; i < args.length; i++) {
            //Process argument value
            if (!args[i].startsWith("-")) {
                if (useTableXml) {
                    config.setTablesXml(args[i]);
                    useTableXml = false;
                }
                else if (useDataXml) {
                    config.setDataXml(args[i]);
                    useDataXml = false;
                }
                else if (upgrade) {
                    String upgradeVersion = args[i];
                    config.setUpgradeVersion(upgradeVersion);
                    upgrade = false;
                }
            }
            //Process argument switch
            else if (args[i].equals("-u")) {
                upgrade = true;
            }
            else if (args[i].equals("-t")) {
                useTableXml = true;
            }
            else if (args[i].equals("-d")) {
                useDataXml = true;
            }
            else if (args[i].equals("-c")) {
                config.setCreateScript(true);
            }
            else if (args[i].equals("-nc")) {
                config.setCreateScript(false);
            }
            else if (args[i].equals("-D")) {
                config.setDropTables(true);
            }
            else if (args[i].equals("-nD")) {
                config.setDropTables(false);
            }
            else if (args[i].equals("-C")) {
                config.setCreateTables(true);
            }
            else if (args[i].equals("-nC")) {
                config.setCreateTables(false);
            }
            else if (args[i].equals("-P")) {
                config.setPopulateTables(true);
            }
            else if (args[i].equals("-nP")) {
                config.setPopulateTables(false);
            }
        }
    }
}
