/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 *  See license distributed with this file and
 *  available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.tools.dbloader;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Holds mapping information from generic data types for table columns
 * to specific types for different databases.
 *
 * @author Eric Dalquist, edalquist@unicon.net
 * @version $Revision: 11911 $
 */
public class DbTypeMappings {
    private Map dbToTypeMapping = Collections.EMPTY_MAP;

    /**
     * @param typeMappings Sets the DbTypeMapping objects
     */
    public void setTypeMappings(final List typeMappings) {
        final Map mappingBuilder = new HashMap();
        
        for (final Iterator mappingItr = typeMappings.iterator(); mappingItr.hasNext(); ) {
            final DbTypeMapping mapping = (DbTypeMapping)mappingItr.next();
            
            for (final Iterator dbItr = mapping.getDatabases().iterator(); dbItr.hasNext(); ) {
                final DbInfo db = (DbInfo)dbItr.next();
                
                mappingBuilder.put(db, mapping);
            }
        }
        
        this.dbToTypeMapping = mappingBuilder;
    }
    
    /**
     * Gets a DbTypeMapping object for a specified DbInfo object.
     * 
     * @param db The DbInfo object to find a type mapping for.
     * @return The associated DbTypeMapping or null if there is no mapping.
     */
    public DbTypeMapping getMappingForDb(final DbInfo db) {
        return (DbTypeMapping)this.dbToTypeMapping.get(db);
    }
}