package org.jasig.portal.tools.dbloader;

import java.util.Map;

/**
 * Compound implementation of {@link IColumnTargetType}, where
 * contained IColumnTargetType are indexed by name.
 * 
 * @author Dan McCallum, dmccallum@unicon.net
 *
 */
public class NameMappedColumnTargetTypeImpl implements IColumnTargetType {

	private Map mappings;
	
	/**
	 * Initialize this NameMappedColumnTargetTypeImpl with a <code>Map</code>
	 * of {@link IColumnTargetType}s, keyed by Strings.
	 * 
	 * @param mappings a <code>String</code>-to-{@link IColumnTargetType} map.
	 *   May be null.
	 */
	public NameMappedColumnTargetTypeImpl(Map mappings) {
		this.mappings = mappings; 
	}

	/**
	 * {@inheritDoc}
	 */
	public Integer getTargetType(String typeName) {
		
		if ( mappings == null ) {
			return null;
		}
		
		IColumnTargetType mapping = (IColumnTargetType) mappings.get(typeName);
		if ( mapping == null ) {
			return null;
		}
		
		return mapping.getTargetType(typeName);
		
	}
	
}
