package org.jasig.portal.tools.dbloader;

import java.sql.Types;

import org.jasig.portal.tools.dbloader.utils.DbUtils;

/**
 * An {@link IColumnTargetType} with one possible target value.
 * 
 * @author Dan McCallum, dmccallum@unicon.net
 */
public class SimpleColumnTargetTypeImpl implements IColumnTargetType {

	private Integer targetJdbcType;
	
	/**
	 * Identical to calling 
	 * <code>new SimpleColumnTargetTypeImpl(DbUtils.getJavaSqlType(String)</code>
	 * 
	 * @param targetJdbcTypeName
	 */
	public SimpleColumnTargetTypeImpl(String targetJdbcTypeName) {
		this(DbUtils.getJavaSqlType(targetJdbcTypeName));
	}
	
	/**
	 * Initialize this <code>SimpleColumnTargetTypeImpl</code> with
	 * an Integer representing a JDBC type.
	 * 
	 * @param targetJdbcType must not be null. Must not correspond to
	 *   <code>java.sql.Types.NULL</code>
	 */
	public SimpleColumnTargetTypeImpl(Integer targetJdbcType) {
		if ( targetJdbcType == null ) {
			throw new IllegalArgumentException("Must specify a non-null target JDBC type");
		}
		
		if ( targetJdbcType.equals(new Integer(Types.NULL))) {
			throw new IllegalArgumentException("Must specify a sensible JDBC column type");
		}
		
		this.targetJdbcType = targetJdbcType;
	}
	
	/**
	 * Implemented to simply return the Integer assigned or
	 * calculated at construction.
	 * 
	 * @param typeName ignored.
	 */
	public Integer getTargetType(String typeName) {
		return targetJdbcType;
	}
	
}
