/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.url;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;


/**
 *  A utility base class for implementing URL constructors.
 *  Constructor accumulates request parameters in a requestParameters list
 *  variable, which is then written out to the URL.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class BaseUrlConstructorImpl implements IUrlConstructor {
    protected List requestParameters=new LinkedList();
    
    public void writeToUrl(UrlInfo urlInfo) {
        for (Iterator iter = requestParameters.iterator(); iter.hasNext();) {
            StringTuple tuple = (StringTuple) iter.next();
            urlInfo.addParameterValue(tuple.parameter,tuple.value);
        }    
    }
    
    public static class StringTuple {
        public String parameter;
        public String value;
        public StringTuple(String parameter, String value) {
            this.parameter = parameter;
            this.value = value;
        }
    }
    
    /**
     * Obtain explicit list of request parameters for unit testing
     * @return List of BaseUrlConstructorImpl.StringTuple
     */
    public List getRequestParameters() {
        return requestParameters;
    }
    
}
