/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.url;

import java.io.Serializable;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;
import org.jasig.portal.context.ServletPathContextResolver;
import org.jasig.portal.context.common.IHttpRequestResponseAccessor;
import org.springframework.beans.factory.annotation.Required;

/**
 * Decorates URL with the servlet context path, and uPortal context path if such was specified.
 * The implementation calls {@link org.jasig.portal.context.ServletPathContextResolver}.getContextPathElement() method
 * to determine uPortal context path. 
 * 
 * Note: alternatively, if the context name is always specified (or never specified) one can use {@link org.jasig.portal.url.AbsoluteUrlDecorator}
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class ContextPathDecorator implements IUrlDecorator, ICacheable {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(ContextPathDecorator.class);
    
    private IHttpRequestResponseAccessor requestAccessor;

    public void decorateUrl(IPortalUrl url) {
            IServletPathExtender constructor=(IServletPathExtender) url.getUrlConstructor(IServletPathExtender.class.getName());
            String contextPath=getContextPath();
            if (logger.isDebugEnabled()) {
                logger.debug("decorateUrl(IPortalUrl) - setting servlet path="+contextPath);
            }
            constructor.addPathElement(contextPath);
    }

    
    private String getContextPath() {
        HttpServletRequest req=requestAccessor.getCurrentHttpServletRequest();
        String contextPath=req.getContextPath();
        String portalContextPath=ServletPathContextResolver.getContextPathElement(req);
        if(portalContextPath!=null) {
            return contextPath+"/"+portalContextPath;
        } else {
            return contextPath;
        }
    }

    @Required
    public void setRequestAccessor(IHttpRequestResponseAccessor requestAccessor) {
        this.requestAccessor = requestAccessor;
    }

    public ICacheKey generateKey() {
        String contextPath=this.getContextPath();
        return new CacheKeyImpl(contextPath);
    }

    public boolean isCacheValid(Serializable validityObject) {
        return true;
    }

    public boolean canEvaluateVariables() {
        return false;
    }
}
