/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.url;

import javax.servlet.http.HttpServletResponse;

/**
 * Http request parameter processor interface.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public interface IRequestParameterProcessor {
	
    /**
     * (Overrides)
     * Analyze current request, process necessary URL parameters,
     * delivering information to the appropriate components.
     * 
     * @param req - incoming request
     * @param res - outgoing response
     * @return <code>true</code> if the processor completed request processing, <code>false</code> if the processor requires other processors to complete some of
     * the values before processing can be complete. In that case the framework will invoke the method call again, after invoking the method
     * on all other processors that have not yet completed the request parameter processing. Even when returning <code>false</code>, the processor
     * can and should process as many parameters as it can during each invokation of this method. Note that the return value is checked only
     * for processors that return <code>true</code> on <code>canEvaluateVariables()</code> call.
     */
    public boolean processParameters(IWritableHttpServletRequest req, HttpServletResponse res);
    
    /**
     * Specifies whether given processor has an ability to evaluate request parameter variables.
     * Note that a processor returning <code>false</code> value can make use of parameters containing variables, iff
     * these variables are defined and evaluated by some other processor.
     *
     * @return <code>true</code> if the processor can potentially replace a variable with a value, <code>false</code> if the processor
     * does not, by itself, define and evaluate variables.
     */
    public boolean canEvaluateVariables();
}
