/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.url;

import javax.servlet.http.HttpServletRequest;

/**
 * IWritableHttpServletRequest is an extension of the {@link javax.servlet.http.HttpServletRequest} interface,
 * which allows to manage (add, remove and modify) request parameters.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public interface IWritableHttpServletRequest extends HttpServletRequest {
    /**
     * Adds a value of a request parameter
     * @param parameterName
     * @param parameterValue
     * @return <code>true</code> if the parameter already existsted 
     */
    public boolean addParameterValue(String parameterName, String parameterValue);
    
    /**
     * Sets a value of request parameter (overwriting any previous values)
     * @param parameterName
     * @param parameterValue
     * @return <code>true</code> if the parameter already existsted
     */
    public boolean setParameterValue(String parameterName, String parameterValue);
    /**
     * Sets a set of values of request parameter (overwriting any previous values)
     * @param parameterName
     * @param parameterValues
     * @return <code>true</code> if the parameter already existsted
     */
    public boolean setParameterValues(String parameterName, String[] parameterValues);
    
    /**
     * Removes any values of an existing parameter
     * @param parameterName
     * @return <code>true</code> if the parameter existed
     */
    public boolean deleteParameter(String parameterName);
    
    /**
     * Removes a specific value of a request parameter
     * @param parameterName
     * @param parameterValue
     * @return <code>true</code> if the parameter value existed
     */
    public boolean deleteParameterValue(String parameterName, String parameterValue);
    
}
