/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.url;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 *  Default implementation of portal Url.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class PortalUrlImpl implements IPortalUrl {
    //  Map containing other uPortal-specific URL constructors (thread-safe)
    private Map urlConstructors = Collections.synchronizedMap(new HashMap());  // constructor name -> IUrlConstructor instance
    protected IUrlConstructorProviderFactoryService urlConstructorProviderService; // factory to obtain new url constructors from
    protected List urlDecorators;
    
    // UrlInfo instance
    protected final UrlInfo urlInfo=new UrlInfo();
    
    /**
     * Construct a new portal url given a url constructor provider factory service and a list of decorators.
     * @param service
     * @param decorators
     */
    public PortalUrlImpl(IUrlConstructorProviderFactoryService service, List decorators) {
        urlConstructorProviderService = service;
        urlDecorators = decorators;
    }

    public String toString() {
       // process url decorators
       if(urlDecorators!=null && !urlDecorators.isEmpty()) {
           for (Iterator iter = urlDecorators.iterator(); iter.hasNext();) {
               IUrlDecorator decorator = (IUrlDecorator) iter.next();
               decorator.decorateUrl(this);
           }
       }
       
       // process all instantiated constructors
       for ( Iterator i = urlConstructors.values().iterator(); i.hasNext(); ) {
         IUrlConstructor constructor = (IUrlConstructor) i.next();
         constructor.writeToUrl(urlInfo);
       }
       
       return urlInfo.toString();   
    }
    
    /**
     * Returns an implementation of the specified URL constructor
     * 
     * @param name - name of the appropriate URL constructor. The name must be unique within current context.
     * @return an implementation of the specified interface
     */
    public Object getUrlConstructor(String name) {
        Object urlConstructor=urlConstructors.get(name);
        if(urlConstructor==null) {
            IUrlConstructorProvider urlProvider = (IUrlConstructorProvider) urlConstructorProviderService.getUrlConstructorProvider(name);
            if(urlProvider!=null) {
                urlConstructor=urlProvider.getUrlConstructor();
                if(urlConstructor!=null) { // record instance in a map
                    urlConstructors.put(name,urlConstructor);
                }
            }
        }
        return urlConstructor;
    }
}
