/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.url;

import java.util.List;
import java.util.Stack;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;

import org.apache.xalan.extensions.XSLProcessorContext;
import org.apache.xalan.templates.ElemExtensionCall;
import org.apache.xalan.transformer.TransformerImpl;

/**
 *  Xalan extension elements for constructing portal URLs.
 *  This class encodes outer <url/> element
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class PortalUrlXalanElements {
    /**
     * TransformerImpl parameter name for portal url stack.
     */
    public final static String PORTAL_URL_STACK_PARAMETER_NAME="portalUrlStack";
    
    /**
     * TransformerImpl parameter name for {@link org.jasig.portal.url.IUrlConstructorProviderFactoryService} instance  
     */
    public final static String URL_CONSTRUCTOR_CONFIGURATION_PARAMETER_NAME="urlConstructorConfiguration";
    
    /**
     * TransformerImpl parameter name for a List of {@link org.jasig.portal.url.IUrlDecorator} instances
     */
    public final static String URL_DECORATOR_CONFIGURATION_PARAMETER_NAME="urlDecoratorConfiguration";
        
    
    /**
     * Outer url element. Recursively evaluates all templates for internal elements,
     * returns a string value corresponding to the url constructed by the internal
     * element commands.
     * 
     * @param context
     * @param elem
     * @return serialized portal url corresponding to commands in the child elements
     * @throws TransformerException
     */
    public String url(XSLProcessorContext context, ElemExtensionCall elem) throws TransformerException {
        TransformerImpl transformerImpl = context.getTransformer();

        // retrieve configuration
        IUrlConstructorProviderFactoryService constructors=(IUrlConstructorProviderFactoryService) transformerImpl.getParameter(URL_CONSTRUCTOR_CONFIGURATION_PARAMETER_NAME);
        List decorators=(List) transformerImpl.getParameter(URL_DECORATOR_CONFIGURATION_PARAMETER_NAME);
        
        // create new IPortletUrl instance
        IPortalUrl portalUrl=new PortalUrlImpl(constructors,decorators);
        // put it on the stack
        Stack urlStack=getCreateUrlStack(transformerImpl);
        urlStack.push(portalUrl);
        transformerImpl.executeChildTemplates(elem,true);
        // remove url fromt the stack
        urlStack.pop();
        return portalUrl.toString();
    }
    
    /**
     * A helper method to obtain portal url stack from a transformer impl,
     * creating a new one if neccessary. This can be done manually using get/set
     * transformer parameter methods.
     * @param transformer
     * @return
     * @see PORTAL_URL_STACK_PARAMETER_NAME
     */
    public static Stack getCreateUrlStack(Transformer transformer) {
        Stack urlStack=(Stack) transformer.getParameter(PORTAL_URL_STACK_PARAMETER_NAME);
        if(urlStack==null) {
            urlStack=new Stack();
            transformer.setParameter(PORTAL_URL_STACK_PARAMETER_NAME,urlStack);
        }
        return urlStack;
    }
    
    /**
     * A helper method to obtain portal url stack from transformer impl.
     * 
     * @param transformer
     * @return portal url stack or <code>null</code> if it hasn't been created.
     */
    public static Stack getUrlStack(Transformer transformer) {
        return (Stack) transformer.getParameter(PORTAL_URL_STACK_PARAMETER_NAME);
    }
}
