/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.url;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.jasig.portal.caching.CacheKeyImpl;
import org.jasig.portal.caching.CacheKeyListImpl;
import org.jasig.portal.caching.ICacheKey;
import org.jasig.portal.caching.ICacheable;

/**
 * Util methods for generating cache keys of URL constructors/decorators.
 * 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class UrlCacheUtils {
    
    public static ICacheKey getCacheKey(IUrlConstructorProviderFactoryService cpfs, List<IUrlDecorator> decorators) {
        List<ICacheKey> cacheKeys=new LinkedList<ICacheKey>();
        if(decorators!=null) {
            for (Iterator<IUrlDecorator> iter = decorators.iterator(); iter.hasNext();) {
                IUrlDecorator decorator = (IUrlDecorator) iter.next();
                if(decorator instanceof ICacheable) {
                    ICacheable cd=(ICacheable)decorator;
                    ICacheKey cdKey=cd.generateKey();
                    if(cdKey==null) {
                        return null;
                    }
                    cacheKeys.add(cdKey);
                }
            }
        }
        if(cpfs!=null) {
            Collection cps=cpfs.getConstructorProviders();
            if(cps!=null) {
                for (Iterator iter = cps.iterator(); iter.hasNext();) {
                    IUrlConstructorProvider provider = (IUrlConstructorProvider) iter.next();
                    if(provider instanceof ICacheable) {
                        ICacheable cd=(ICacheable)provider;
                        ICacheKey cdKey=cd.generateKey();
                        if(cdKey==null) {
                            return null;
                        }
                        cacheKeys.add(cdKey);
                    }
                }
            }
        }
        if(cacheKeys.isEmpty()) {
            return CacheKeyImpl.EMPTY_KEY;
        } else {
            return new CacheKeyListImpl(cacheKeys);
        }
    }
    
    public static ICacheKey getCacheKey(IUrlConstructorProviderFactoryServiceAccessor cpfsa, IUrlDecoratorListAccessor dla, HttpServletRequest req) {
        return getCacheKey(cpfsa.getUrlConstructorProviderFactoryService(req),dla.getUrlDecorators(req));
    }

}
