/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.url;

import java.util.Collection;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;


/**
 * UrlConstructorProviderFactoryServiceImpl is a default iplementation of the url factory service
 * 
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @author Michael Ivanov, mivanov at unicon.net
 * @version $Revision: 11911 $
 */
public class UrlConstructorProviderFactoryServiceImpl implements IUrlConstructorProviderFactoryService {
    
	private Map providers = new Hashtable(); // url constructor name -> provider instance
	
	
	public UrlConstructorProviderFactoryServiceImpl() { }
	
	public UrlConstructorProviderFactoryServiceImpl(Set providerSet) { 
		setUrlConstructorProviders(providerSet);
	}
    
    // TODO: return an ordered list, so that extracted cache key sequence would match from one call to another
    public Collection getConstructorProviders() {
        return providers.values();
    }
	
	private boolean providerExists ( IUrlConstructorProvider constProvider ) {
        return providers.containsKey(constProvider.getName());
	}
	
    public void setUrlConstructorProviders(Set providerSet) {
        for (Iterator iter = providerSet.iterator(); iter.hasNext();) {
            IUrlConstructorProvider provider = (IUrlConstructorProvider) iter.next();
            if ( providerExists(provider) )
              throw new IllegalArgumentException("URL constructor provider already exists under name '"+provider.getName()+"'");
            providers.put(provider.getName(),provider);
        }
    }

    public IUrlConstructorProvider getUrlConstructorProvider(String name) {
        return (IUrlConstructorProvider) providers.get(name);
    }

}
