/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.url;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.security.Principal;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Map;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletInputStream;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

/**
 * WritableHttpServletRequestImpl description.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class WritableHttpServletRequestImpl implements IWritableHttpServletRequest {
    protected HttpServletRequest request;
    protected Map parameterMap;
    
    /**
     * Construct a writable request wrapping a real request
     * @param request
     */
    public WritableHttpServletRequestImpl(HttpServletRequest request) {
        this.request=request;
        this.parameterMap=new Hashtable();
        // place all parameters into the map
        parameterMap.putAll(request.getParameterMap());
    }

    public boolean addParameterValue(String parameterName, String parameterValue) {
        String[] currentValues=(String[]) parameterMap.get(parameterName);
        if(currentValues==null) {
            String[] values= {parameterValue};
            parameterMap.put(parameterName,values);
            return false;
        } else {
            String[] values=new String[currentValues.length+1];
            for (int i = 0; i < currentValues.length; i++) {
                values[i]=currentValues[i];
            }
            values[values.length-1]=parameterValue;
            parameterMap.put(parameterName,values);
            return true;
        }
    }

    public boolean deleteParameter(String parameterName) {
        return parameterMap.remove(parameterName)!=null;
    }

    public boolean deleteParameterValue(String parameterName, String parameterValue) {
        String[] currentValues=(String[]) parameterMap.get(parameterName);
        if(currentValues==null) {
            return false;
        } else {
            String[] values=new String[currentValues.length+1];
            boolean found=false;
            for (int i = 0; i < currentValues.length; i++) {
                if(!currentValues[i].equals(parameterValue)) {
                    values[i]=currentValues[i];
                } else {
                    found=true;
                }
            }
            parameterMap.put(parameterName,values);
            return found;
        }
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.url.IWritableHttpServletRequest#setParameterValue(java.lang.String, java.lang.String)
     */
    public boolean setParameterValue(String parameterName, String parameterValue) {
        String[] values={parameterValue};
        return parameterMap.put(parameterName,values)!=null;
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.url.IWritableHttpServletRequest#setParameterValues(java.lang.String, java.lang.String[])
     */
    public boolean setParameterValues(String parameterName, String[] parameterValues) {
        return parameterMap.put(parameterName,parameterValues)!=null;
    }

    public Object getAttribute(String arg0) {
        return request.getAttribute(arg0);
    }

    public Enumeration getAttributeNames() {
        return request.getAttributeNames();
    }

    public String getAuthType() {
        return request.getAuthType();
    }

    public String getCharacterEncoding() {
        return request.getCharacterEncoding();
    }

    public int getContentLength() {
        return request.getContentLength();
    }

    public String getContentType() {
        return request.getContentType();
    }

    public String getContextPath() {
        return request.getContextPath();
    }

    public Cookie[] getCookies() {
        return request.getCookies();
    }

    public long getDateHeader(String arg0) {
        return request.getDateHeader(arg0);
    }

    public String getHeader(String arg0) {
        return request.getHeader(arg0);
    }

    public Enumeration getHeaderNames() {
        return request.getHeaderNames();
    }

    public Enumeration getHeaders(String arg0) {
        return request.getHeaders(arg0);
    }

    public ServletInputStream getInputStream() throws IOException {
        return request.getInputStream();
    }

    public int getIntHeader(String arg0) {
        return request.getIntHeader(arg0);
    }

    public Locale getLocale() {
        return request.getLocale();
    }

    public Enumeration getLocales() {
        return request.getLocales();
    }

    public String getMethod() {
        return request.getMethod();
    }

    public String getParameter(String parameterName) {
        String[] pValues=(String[])parameterMap.get(parameterName);
        if(pValues!=null && pValues.length>0) {
            return pValues[0];
        } else {
            return null;
        }
    }

    public Map getParameterMap() {
        return parameterMap;
    }

    public Enumeration getParameterNames() {
        return Collections.enumeration(parameterMap.keySet());
    }

    public String[] getParameterValues(String parameterName) {
        return (String[]) parameterMap.get(parameterName);
    }

    public String getPathInfo() {
        return request.getPathInfo();
    }

    public String getPathTranslated() {
        return request.getPathTranslated();
    }

    public String getProtocol() {
        return request.getProtocol();
    }

    public String getQueryString() {
        return request.getQueryString();
    }

    public BufferedReader getReader() throws IOException {
        return request.getReader();
    }

    /**
     * @deprecated
     * @see javax.servlet.ServletRequest#getRealPath(java.lang.String)
     */
    public String getRealPath(String arg0) {
        return request.getRealPath(arg0);
    }

    public String getRemoteAddr() {
        return request.getRemoteAddr();
    }

    public String getRemoteHost() {
        return request.getRemoteHost();
    }

    public String getRemoteUser() {
        return request.getRemoteUser();
    }

    public RequestDispatcher getRequestDispatcher(String arg0) {
        return request.getRequestDispatcher(arg0);
    }

    public String getRequestedSessionId() {
        return request.getRequestedSessionId();
    }

    public String getRequestURI() {
        return request.getRequestURI();
    }

    public StringBuffer getRequestURL() {
        return request.getRequestURL();
    }

    public String getScheme() {
        return request.getScheme();
    }

    public String getServerName() {
        return request.getServerName();
    }

    public int getServerPort() {
        return request.getServerPort();
    }

    public String getServletPath() {
        return request.getServletPath();
    }

    public HttpSession getSession() {
        return request.getSession();
    }

    public HttpSession getSession(boolean arg0) {
        return request.getSession(arg0);
    }

    public Principal getUserPrincipal() {
        return request.getUserPrincipal();
    }

    public boolean isRequestedSessionIdFromCookie() {
        return request.isRequestedSessionIdFromCookie();
    }

    /**
     * @deprecated
     * @see javax.servlet.http.HttpServletRequest#isRequestedSessionIdFromUrl()
     */
    public boolean isRequestedSessionIdFromUrl() {
        return request.isRequestedSessionIdFromUrl();
    }

    public boolean isRequestedSessionIdFromURL() {
        return request.isRequestedSessionIdFromURL();
    }

    public boolean isRequestedSessionIdValid() {
        return request.isRequestedSessionIdValid();
    }

    public boolean isSecure() {
        return request.isSecure();
    }

    public boolean isUserInRole(String arg0) {
        return request.isUserInRole(arg0);
    }

    public void removeAttribute(String arg0) {
        request.removeAttribute(arg0);
    }

    public void setAttribute(String arg0, Object arg1) {
        request.setAttribute(arg0, arg1);
    }

    public void setCharacterEncoding(String arg0) throws UnsupportedEncodingException {
        request.setCharacterEncoding(arg0);
    }

    public String getLocalAddr() {
        return request.getLocalAddr();
    }

    public String getLocalName() {
        return request.getLocalName();
    }

    public int getLocalPort() {
        return request.getLocalPort();
    }

    public int getRemotePort() {
        return request.getRemotePort();
    }
}
