/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */


package org.jasig.portal.url.syntax;

import java.util.ArrayList;
import java.util.List;

import org.apache.oro.text.perl.Perl5Util;
import org.apache.oro.text.regex.MatchResult;

/**
 * Default implementation of IUrlBasicSyntaxProvider.
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class UrlBasicSyntaxProviderImpl implements IUrlBasicSyntaxProvider {
    protected static final String PROCESSOR_NAME_SERPARATOR=".";
    
    /* (non-Javadoc)
     * @see org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider#containsVariable(java.lang.String)
     */
    public boolean containsVariable(String s) {
        Perl5Util p5u=new Perl5Util();
        if(p5u.match("/{.+}/", s)) {
            return true;
        } else {
            return false;
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider#getVaribleNames(java.lang.String)
     */
    public String[] getVaribleNames(String s) {
        Perl5Util p5u=new Perl5Util();
        List variableNames=new ArrayList();
        String text=s;
        while(p5u.match("/{(.+?)}/", text)) {
            MatchResult result=p5u.getMatch();
            String variableName=result.group(1);
            variableNames.add(variableName);
            text=p5u.postMatch();
        }
        return (String[]) variableNames.toArray(new String[variableNames.size()]);
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider#replaceVariableWithValue(java.lang.String, java.lang.String)
     */
    public String replaceVariableWithValue(String variableName, String value, String text) {
        Perl5Util p5u=new Perl5Util();
        return p5u.substitute("s/{"+variableName+"}/"+value+"/g",text);
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider#getProcessorName(java.lang.String)
     */
    public String[] separateProcessorPrefix(String parameterName) {
        int separatorIndex=parameterName.indexOf(PROCESSOR_NAME_SERPARATOR);
        if(separatorIndex<0) {
            return null;
        } else {
            String[] r=new String[2];
            r[0]=parameterName.substring(0,separatorIndex);
            r[1]=parameterName.substring(separatorIndex+1);
            return r;
        }
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider#incorporateProcessorName(java.lang.String, java.lang.String)
     */
    public String incorporateProcessorPrefix(String processorName, String parameterName) {
        return processorName+PROCESSOR_NAME_SERPARATOR+parameterName;
    }

    /* (non-Javadoc)
     * @see org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider#containsVariable(java.lang.String, java.lang.String)
     */
    public boolean containsVariable(String s, String variableName) {
        Perl5Util p5u=new Perl5Util();
        if(p5u.match("/{"+variableName+"}/", s)) {
            return true;
        } else {
            return false;
        }
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider#containsVariableForComponent(java.lang.String, java.lang.String)
     */
    public boolean containsVariableForProcessor(String s, String processorName) {
        Perl5Util p5u=new Perl5Util();
        if(p5u.match("/{"+processorName+PROCESSOR_NAME_SERPARATOR+".+}/", s)) {
            return true;
        } else {
            return false;
        }
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider#getVariableNamesForComponent(java.lang.String, java.lang.String)
     */
    public String[] getVariableNamesForProcessor(String s, String processorName) {
        Perl5Util p5u=new Perl5Util();
        List variableNames=new ArrayList();
        String text=s;
        while(p5u.match("/{"+processorName+PROCESSOR_NAME_SERPARATOR+"(.+?)}/", text)) {
            MatchResult result=p5u.getMatch();
            String variableName=result.group(1);
            variableNames.add(variableName);
            text=p5u.postMatch();
        }
        return (String[]) variableNames.toArray(new String[variableNames.size()]);
    }
    /* (non-Javadoc)
     * @see org.jasig.portal.url.syntax.IUrlBasicSyntaxProvider#getProcessorBareParameterName(java.lang.String, java.lang.String)
     */
    public String getProcessorBareParameterName(String processorName, String fullParameterName) {
        if(fullParameterName.startsWith(processorName+PROCESSOR_NAME_SERPARATOR)) {
            return fullParameterName.substring(processorName.length()+PROCESSOR_NAME_SERPARATOR.length());
        } else {
            return null;
        }
    }
}
