/* Copyright 2005 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.utils;

import java.util.Comparator;

import org.jasig.portal.portlet.definition.ILocalizedPortletApplicationDefinition;
import org.jasig.portal.portlet.definition.ILocalizedPortletDefinition;
import org.jasig.portal.portlet.deployment.IPortletApplicationDeployment;
import org.jasig.portal.portlet.deployment.IPortletDeployment;


/**
 * Contains static final fields that implement the Comparator interface. Each
 * implementation is for use with a specific uPortal class type.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public final class Comparators {
    /**
     * Compares two ILocalizedPortletDefinition instances using the local display name field.
     */
    public static final Comparator LOCALIZED_PORTLET_DEFINITION_NAME_COMPARATOR = new Comparator() {
        public int compare(Object o1, Object o2) {
            final ILocalizedPortletDefinition def1 = (ILocalizedPortletDefinition)o1;
            final ILocalizedPortletDefinition def2 = (ILocalizedPortletDefinition)o2;
            

            final String name1 = def1.getLocalDisplayName();
            final String name2 = def2.getLocalDisplayName();
            
            if (name1 == name2 || (name1 != null && name1.equals(name2))) {
                return def1.getPortletDefinitionId().compareTo(def2.getPortletDefinitionId());
            }
            else if (name1 == null) {
                return -1;
            }
            else if (name2 == null) {
                return 1;
            }
            else {
                return name1.compareToIgnoreCase(name2);
            }
        }
    };
    
    /**
     * Compares two ILocalizedPortletApplicationDefinition instances using the local display name field.
     */
    public static final Comparator LOCALIZED_PORTLET_APPLICATION_DEFINITION_NAME_COMPARATOR = new Comparator() {
        public int compare(Object o1, Object o2) {
            final ILocalizedPortletApplicationDefinition def1 = (ILocalizedPortletApplicationDefinition)o1;
            final ILocalizedPortletApplicationDefinition def2 = (ILocalizedPortletApplicationDefinition)o2;
            
            final String name1 = def1.getLocalDisplayName();
            final String name2 = def2.getLocalDisplayName();
            
            if (name1 == name2 || (name1 != null && name1.equals(name2))) {
                return def1.getPortletApplicationDefinitionId().compareTo(def2.getPortletApplicationDefinitionId());
            }
            else if (name1 == null) {
                return -1;
            }
            else if (name2 == null) {
                return 1;
            }
            else {
                return name1.compareToIgnoreCase(name2);
            }
        }
    };
    
    /**
     * Compares two IPortletDeployment instances using the portletName field.
     */
    public static final Comparator PORTLET_DEPLOYMENT_NAME_COMPARATOR = new Comparator() {
        public int compare(Object o1, Object o2) {
            final IPortletDeployment dep1 = (IPortletDeployment)o1;
            final IPortletDeployment dep2 = (IPortletDeployment)o2;
            
            return dep1.getPortletName().compareToIgnoreCase(dep2.getPortletName());
        }
    };
    
    
    /**
     * Compares two IPortletApplicationDeployment instances using the name field.
     */
    public static final Comparator PORTLET_APPLICATION_DEPLOYMENT_NAME_COMPARATOR = new Comparator() {
        public int compare(Object o1, Object o2) {
            final IPortletApplicationDeployment dep1 = (IPortletApplicationDeployment)o1;
            final IPortletApplicationDeployment dep2 = (IPortletApplicationDeployment)o2;
            
            return dep1.getPortletApplicationDeploymentId().compareTo(dep2.getPortletApplicationDeploymentId());
        }
    };
    
    private Comparators() { }
}
