/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
*  See license distributed with this file and
*  available online at http://www.uportal.org/license.html
*/

package org.jasig.portal.utils;

import javax.sql.DataSource;

/**
 * An interface that keeps track of multiple unique ID counters.
 *
 * @author Mike DeSimone, mike@rsmart.com
 * @version $Revision: 11955 $
 */
public interface IIncrementer {

    /**
     * get the next key to use
     * @return the next key value
     */
    public long getNextKey();

    /**
     * Create a new sequence in the database
     * @param startVal starting value of the sequence
     * @throws RuntimeException if counter already exists  (todo: set to appropriate exception)
     */
    public void createIncrementer(long startVal);

    /**
     * set counter to a new value in the database
     * WARNING: use with care; decreasing the current value will probably cause database integrity issues
     * @param newValue
     * @throws RuntimeException if counter doesn't exist (todo: set to appropriate exception)
     */
    public void setIncrementerValue(long newValue) throws RuntimeException;

    /**
     * set the name for this incrementer (non bean-factory objects need to set this)
     * @param name
     */
    public void setIncrementerName(String name);

    /**
     * set the datasource for this incrementer (non bean-factory objects need to set this)
     * @param dataSource
     */
    public void setDataSource(DataSource dataSource);

    /**
     * see if this incrementer exists in the db
     * @return true if incrementer exists in the db
     */
    public boolean existsInDb();
}

