package org.jasig.portal.utils;

import java.io.Serializable;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;

/**
 * A serializable object which may be used to wrap an Object intended
 * to be used as a key in a Map or cache. The wrapper uses the namespace
 * String and key for equality, hashcode and toString functions.
 * 
 * @author Eric Dalquist <a href="mailto:edalquist@unicon.net">edalquist@unicon.net</a>
 * @version $Revision: 11911 $
 */
public class NamespacingKeyWrapper implements Serializable {
    private final String namespace;
    private final Object key;
    
    public NamespacingKeyWrapper(Object key) {
        this("KEY_WRAPPER__", key);
    }
    
    public NamespacingKeyWrapper(String namespace, Object key) {
        this.namespace = namespace;
        this.key = key;
    }
    
    /**
     * @return Returns the key.
     */
    public Object getKey() {
        return this.key;
    }

    /**
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (!(obj instanceof NamespacingKeyWrapper)) {
            return false;
        }
        
        final NamespacingKeyWrapper other = (NamespacingKeyWrapper)obj;
        return new EqualsBuilder()
            .append(this.namespace, other.namespace)
            .append(this.key, other.key)
            .isEquals();
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode() {
        return new HashCodeBuilder(-1486736713, -967903727)
            .append(this.namespace)
            .append(this.key)
            .toHashCode();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return this.namespace + this.key;
    }
}