/*
 * Copyright 2004 The JA-SIG Collaborative. All rights reserved. See license distributed with this
 * file and available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.utils;

import org.jasig.portal.core.exception.PortalRuntimeException;

/**
 * Exception for when a requested resource could not be found.
 * 
 * @author andrew.petro@yale.edu
 * @version $Revision: 11918 $ $Date: 2007-04-02 12:48:03 -0500 (Mon, 02 Apr 2007) $
 */
public class ResourceNotFoundException extends PortalRuntimeException {

    /**
     * The name of the resource that could not be found.
     */
    private final String resourceName;

    /**
     * The classpath on which the resource could not be found.
     */
    private final String classPath;

    /**
     * A resource could not be found.
     * 
     * @param resourceName - the name of the requested resource.
     */
    public ResourceNotFoundException(String resourceName) {
        this(resourceName, (Throwable)null);
    }

    public ResourceNotFoundException(String resourceName, Throwable cause) {
        super("The resource [" + resourceName + "] could not be found.", cause);
        this.resourceName = resourceName;
        this.classPath = null;
    }

    /**
     * A resource could not be found on a particular classpath.
     * 
     * @param resourceName - the name of the desired resource
     * @param classPath - the classpath on which it could not be found.
     */
    public ResourceNotFoundException(String resourceName, String classPath) {
        this(resourceName, classPath, null);
    }

    public ResourceNotFoundException(String resourceName, String classPath, Throwable cause) {
        super("The resource [" + resourceName + "] could not be found on the classpath [" + classPath + "].", cause);
        this.resourceName = resourceName;
        this.classPath = null;
    }

    public String getClassPath() {
        return this.classPath;
    }

    public String getResourceName() {
        return this.resourceName;
    }
}
