/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.utils;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Concatenates the strings using space character, using URL escaping
 * on individual pieces and on the resulting concatenation. 
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 * @version $Revision: 11911 $
 */
public class UrlEncodingStringConcatenator implements IStringConcatenator {
    /**
     * Logger for this class
     */
    private static final Log logger = LogFactory.getLog(UrlEncodingStringConcatenator.class);

    protected String encoding="UTF-8";
    protected String delimeter=" ";
    
    protected static final String emptyString="";
    
    public String concatenate(Collection pieces) {
        StringBuffer sb=new StringBuffer(5*pieces.size());
        try {
            for (Iterator iter = pieces.iterator(); iter.hasNext();) {
                String element = (String) iter.next();
                sb.append(URLEncoder.encode(element,encoding));
                if(iter.hasNext()) {
                    sb.append(" ");
                }
            }
            return URLEncoder.encode(sb.toString(),encoding);
        } catch (UnsupportedEncodingException uee) {
            logger.error("concatenate(Collection) - unsupported encoding specified for string concatenator utility : encoding=" + encoding, uee);
            return null;
        }
    }

    public List split(String concatenation) {
        List pieces=new LinkedList();
        String unencodedConcatination;
        try {
            unencodedConcatination = URLDecoder.decode(concatenation,encoding);
            boolean prevDelimeter=false;
            for(StringTokenizer tokenizer=new StringTokenizer(unencodedConcatination,delimeter,true);tokenizer.hasMoreTokens();) {
                String piece=tokenizer.nextToken();
                if(delimeter.equals(piece)) {
                    if(prevDelimeter) {
                        // an empty string was concatenated
                        pieces.add(emptyString);
                    } else {
                        prevDelimeter=true;
                    }
                } else {
                    prevDelimeter=false;
                    pieces.add(URLDecoder.decode(piece,encoding));
                }
            }
            return pieces;
        } catch (UnsupportedEncodingException e) {
            logger.error("spint(String) - unsupported encoding specified for string concatenator utility : encoding=" + encoding, e);
            return null;
        }
    }

    public void setDelimeter(String delimeter) {
        this.delimeter = delimeter;
    }

    public void setEncoding(String encoding) {
        this.encoding = encoding;
    }

}
