/* Copyright 2004 The JA-SIG Collaborative.  All rights reserved.
 * See license distributed with this file and
 * available online at http://www.uportal.org/license.html
 */

package org.jasig.portal.utils;

import java.io.NotSerializableException;
import java.util.AbstractSet;
import java.util.Iterator;
import java.util.WeakHashMap;

/**
 * WeakHashSet is a has set that maintains weak references to its contained objects
 *
 * @author Peter Kharchenko: pkharchenko at unicon.net
 */
public class WeakHashSet<E> extends AbstractSet<E> {
    // Dummy value to associate with an Object in the backing Map
    private static final Object PRESENT = new Object();

    private WeakHashMap<E, Object> map = new WeakHashMap<E, Object>();

    /**
     * @see java.util.Set#add(java.lang.Object)
     */
    public boolean add(E o) {
        return this.map.put(o, PRESENT) == null;
    }

    /**
     * @see java.util.AbstractCollection#iterator()
     */
    @Override
    public Iterator<E> iterator() {
        return this.map.keySet().iterator();
    }

    /**
     * @see java.util.AbstractCollection#size()
     */
    @Override
    public int size() {
        return this.map.size();
    }

    /**
     * @see java.util.AbstractCollection#isEmpty()
     */
    @Override
    public boolean isEmpty() {
        return this.map.isEmpty();
    }
    
    /**
     * @see java.util.AbstractCollection#contains(java.lang.Object)
     */
    @Override
    public boolean contains(Object o) {
        return this.map.containsKey(o);
    }

    /**
     * @see java.util.AbstractCollection#remove(java.lang.Object)
     */
    @Override
    public boolean remove(Object o) {
        return this.map.remove(o) == PRESENT;
    }
    
    /**
     * @see java.util.AbstractCollection#clear()
     */
    @Override
    public void clear() {
        this.map.clear();
    }

    private void writeObject(java.io.ObjectOutputStream s) throws java.io.IOException {
        throw new NotSerializableException("WeakHashSet is not a Serializable class");
    }
}
